#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Base
#
# This is the base class of the RBT project.
#
# All classes that desire to have a common functionality should subclass
# from this class here.
#
# Not every class in the RBT project will need to subclass from this class,
# so you should first check whether the RBT::Base class is needed.
#
# For example - the class Prefix does not really need to subclass from
# this class here, as it barely needs the variables defined herein.
#
# The most important part of this class is probably the @debug variable.
# Every subclass should have the @debug variable, which can be useful
# for finding out where a problem occurs within the RBT project.
#
# Usage example:
#   RBT::Base.new
# =========================================================================== #
# require 'rbt/base/base.rb'
# =========================================================================== #
# === How to subclass from this class:
#
#   require 'rbt/base/base.rb'; class Foo < ::RBT::Base
#
# =========================================================================== #
require 'English'
require 'rbt/base/prototype.rb'
require 'rbt/base/email.rb'
# =========================================================================== #
# Add exceptions:
# =========================================================================== #
require 'rbt/errors/exceptions.rb'
# =========================================================================== #
# Add some rbt/toplevel_methods/:
# =========================================================================== #
require 'rbt/toplevel_methods/copy_recursively.rb'
require 'rbt/toplevel_methods/does_this_file_exist.rb'
require 'rbt/toplevel_methods/fast_return_file_size_of_this_program.rb'
require 'rbt/toplevel_methods/is_included.rb'
require 'rbt/toplevel_methods/is_on_windows.rb'
require 'rbt/toplevel_methods/md5sum.rb'
require 'rbt/toplevel_methods/split_at.rb'
require 'rbt/toplevel_methods/to_bool.rb'
require 'rbt/toplevel_methods/determine_appdir_prefix_from_this_input.rb'

begin
  require 'program_information'
rescue LoadError; end

module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === remove_unnecessary_data_from_url
  #
  # This method removes some meaningless information that can be found
  # in some URLs.
  # ========================================================================= #
  def remove_unnecessary_data_from_url(i)
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    i = File.basename(i)
    i.gsub(/download\?file=/,'').
      sub(/\/download$/,'').
      sub(/\?download$/,'').
      gsub(/-fullsrc/,'').
      gsub(/-source/,'').
      gsub(/\?use_mirror=dfn/,'').
      gsub(/\.src/,'').
      gsub(/-src/,'')
      # .gsub(/_/,'-') # This here may be controversial, hence it  
                       # was disabled as of June 2010.
      # i = remove_file_extension(i)
      # ^^^ We can not do the above, because program_full_name
      # must include the archive.
  end; alias return_program_full_name_from_url remove_unnecessary_data_from_url # === return_program_full_name

  # ========================================================================= #
  # === does_the_cookbook_include_this_program?
  # ========================================================================= #
  def does_the_cookbook_include_this_program?(i)
    RBT.does_include?(i)
  end

  # ========================================================================= #
  # === to_bool
  # ========================================================================= #
  def to_bool(i)
    RBT.to_bool(i)
  end

  # ========================================================================= #
  # === is_on_windows?
  # ========================================================================= #
  def is_on_windows?
    RBT.is_on_windows?
  end

  # ========================================================================= #
  # === copy_recursively
  # ========================================================================= #
  def copy_recursively(
      what,
      where_to   = '/usr/',
      be_verbose = false
    )
    ::RBT.copy_recursively(what, where_to, be_verbose)
  end

  # ========================================================================= #
  # === display_md5sum?
  # ========================================================================= #
  def display_md5sum?
    RBT.display_md5sum?
  end; alias show_md5sum? display_md5sum? # === show_md5sum?

  # ========================================================================= #
  # === do_not_show_names
  # ========================================================================= #
  def do_not_show_names
    RBT.do_not_show_names
  end

  # ========================================================================= #
  # === determine_archive_type
  #
  # We determine the archive type with this method, which really is
  # a simplification.
  # ========================================================================= #
  def determine_archive_type(i)
    RBT.return_archive_type(i)
  end

  # ========================================================================= #
  # === cookbook_files?
  #
  # This method will return all available cookbook files in an Array.
  #
  # Usage example:
  #   cookbook_files?(:show_full_path)
  # ========================================================================= #
  def cookbook_files?(show_how = :show_only_name)
    result = Dir["#{individual_cookbooks_directory?}*yml"].sort
    case show_how
    when :show_only_name # In this case, show only the name.
      result.map! {|entry|
        File.basename(entry).gsub( File.extname(entry), '' )
      }
    when :show_full_path # In this case make no modifications.
    end
    return result
  end; alias available_cookbook_files? cookbook_files? # === available_cookbook_files?

  # ========================================================================= #
  # === RBT::Base.registered_binaries?
  #
  # Tell us where our binaries are registered.
  # ========================================================================= #
  def self.registered_binaries?
    RBT.all_binaries?
  end

  # ========================================================================= #
  # === show_overview_over_available_utility_scripts
  # ========================================================================= #
  def show_overview_over_available_utility_scripts
    RBT.show_overview_over_available_utility_scripts
  end

  # ========================================================================= #
  # === report_n_programs
  #
  # Usage example for the report functionality of this method:
  #   report_n_programs
  # ========================================================================= #
  def report_n_programs
    consider_reporting_how_many_programs_are_registered
  end

  # ========================================================================= #
  # === use_yaml_database_for_tags?
  # ========================================================================= #
  def use_yaml_database_for_tags?
    RBT.use_yaml_database_for_tags?
  end

  # ========================================================================= #
  # === program_exists?
  #
  # Use this method to find out if a program exists.
  #
  # Usage Example:
  #   if program_exists? :htop
  # ========================================================================= #
  def program_exists?(this_program)
    does_program_exist = false # Defaults to false - the program is not installed.
    this_program = this_program.to_s.dup #. downcase # sanitize. downcase is BAD, it breaks stuff like "Esetroot"
    if this_program.include? ' ' # split it, and take the first word in this case.
      this_program = this_program.split(' ').first 
    end
    path_variable_file = "#{LINUX}YAML/path_variable.yml"
    if File.exist? path_variable_file
      array_available_paths = YAML.load_file(path_variable_file)['path'].strip.split(' ')
      array_available_paths.each {|path|
        _ = "#{path}/#{this_program}"
        does_program_exist = true if File.exist?(_)
      }
    end
    # Hack - an exception for configure:
    does_program_exist = true if this_program.include? '/configure'
    return does_program_exist
  end

  # ========================================================================= #
  # === recipes_dir?
  # ========================================================================= #
  def recipes_dir?
    RBT.configuration?.recipes_dir
  end

  # ========================================================================= #
  # === exit_on_unregistered_cookbook_entries?
  # ========================================================================= #
  def exit_on_unregistered_cookbook_entries?
    RBT.configuration?.exit_on_unregistered_cookbook_entries
  end

  # ========================================================================= #
  # === run_ldconfig?
  # ========================================================================= #
  def run_ldconfig?
    RBT.configuration?.run_ldconfig
  end; alias use_ldconfig? run_ldconfig? # === use_ldconfig?

  # ========================================================================= #
  # === capitalize_program_names?
  # ========================================================================= #
  def capitalize_program_names?
    RBT.configuration?.capitalize_program_names
  end; alias do_capitalize? capitalize_program_names? # === do_capitalize?

  # ========================================================================= #
  # === warn_and_exit
  #
  # This method will exit from the scripts.
  #
  # You should be careful when using this, because in some situations we
  # do not actually want to exit completely - for example when we
  # chain-compile something.
  # ========================================================================= #  
  def warn_and_exit(input, exit_mode = true)
    warn(input, exit_mode)
  end; alias warn_then_exit warn_and_exit # === warn_then_exit

  # ========================================================================= #
  # === RBT::Base.make_command?
  # ========================================================================= #
  def self.make_command?
    RBT.configuration?.make_command # The Configuration object has our make command.
  end

  # ========================================================================= #
  # === make_command?
  # ========================================================================= #
  def make_command?
    Base.make_command?
  end

  # ========================================================================= #
  # === use_stow?
  # ========================================================================= #
  def use_stow?
    RBT.configuration?.use_stow
  end

  # ========================================================================= #
  # === config?
  # ========================================================================= #
  def config?
    RBT.configuration?
  end

  # ========================================================================= #
  # === cat                                                         (cat tag)
  # ========================================================================= #
  def cat(i)
    if File.exist? i
      return File.readlines(i).join
    end
  end

  # ========================================================================= #
  # === make_install_command?
  #
  # This is the default "make install" command for now.
  # ========================================================================= #
  def make_install_command?
    'make install'
  end

  # ========================================================================= #
  # === use_ccache?
  #
  # Whether we will make use of ccache or whether we don't, by default.
  # ========================================================================= #
  def use_ccache?
    RBT.configuration?.use_ccache
  end

  # ========================================================================= #
  # === rbt_logs?
  #
  # This method will return the path of where we store RBT-related
  # log files.
  # ========================================================================= #
  def rbt_logs?
    RBT.log_directory?
  end

  # ========================================================================= #
  # === wrap_at
  # ========================================================================= #
  def wrap_at(
      i,
      n_characters = :default
    )
    RBT.wrap_at(i, n_characters)
  end


  # ========================================================================= #
  # === fast_return_file_size_of_this_program
  # ========================================================================= #
  def fast_return_file_size_of_this_program(i)
    RBT.fast_return_file_size_of_this_program(i)
  end

  # ========================================================================= #
  # === host_system?
  # ========================================================================= #
  def host_system?
    RBT.determine_host_architecture
  end

  # ========================================================================= #
  # === store_into_this_directory?
  # ========================================================================= #
  def store_into_this_directory?
    RBT.store_into_this_directory?
  end

  # ========================================================================= #
  # === use_mrxvt?
  # ========================================================================= #
  def use_mrxvt?
    RBT.configuration?.use_mrxvt
  end

  # ========================================================================= #
  # === use_tags?
  # ========================================================================= #
  def use_tags?
    RBT.configuration?.use_tags
  end

  # ========================================================================= #
  # === always_show_dependencies?
  #
  # If this setting is true then we will always show the dependencies
  # on the commandline.
  # ========================================================================= #
  def always_show_dependencies?
    RBT.configuration?.always_show_dependencies
  end

  # ========================================================================= #
  # === play_song?
  #
  # Query over as to whether we will play a song or not, after compilation
  # has finished.
  # ========================================================================= #
  def play_song?
    RBT.configuration?.play_song
  end

  # ========================================================================= #
  # === new_cookbook_instance_for
  #
  # Use a slightly shorter wrapper to access class Cookbook.
  # ========================================================================= #
  def new_cookbook_instance_for(name_of_the_program)
    RBT::Cookbooks::Cookbook.new(name_of_the_program) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === report_to_developers
  #
  # Feedback information to the user so that he can report useful
  # information to the project lead.
  # ========================================================================= #
  def report_to_developers(
      this_email = RBT.configuration?.email
    )
    opnn; ewarn 'To report to the developer(s) of RBT, use this '\
                'email address:'
    opnn; e
    opnn; efancy '  '+this_email
    opnn; e
    opnn; ewarn 'You can also use this URL:'
    opnn; e
    opnn; efancy "  #{RBT_HOMEPAGE_URL}"
    opnn; e
  end

  # ========================================================================= #
  # === esystem_gold
  # ========================================================================= #
  def esystem_gold(i)
    e gold(i)
    system i
  end

  # ========================================================================= #
  # === set_source_directory
  #
  # Set the source dir to ANY new location. Defaults to Dir.pwd.
  #
  # Example to test it this method:
  #   ry mantis --source_dir=/Depot/j
  # ========================================================================= #
  def set_source_directory(
      i          = return_pwd,
      be_verbose = false
    )
    i << '/' unless i.end_with? '/'
    RBT.set_source_directory(i)
    if be_verbose?
      e "Setting source directory to `#{sdir(i)}` next." 
    end
  end; alias set_source_dir set_source_directory # === set_source_dir

  # ========================================================================= #
  # === is_this_program_included?
  #
  # Use this method to query whether a program is included or whether it
  # is not.
  #
  # The second argument is, by default, nil. It can be a symbol such as
  # :include_abbreviations, in which case we will include abbreviations.
  # ========================================================================= #
  def is_this_program_included?(
      i, optional_arguments = nil
    )
    RBT.is_this_program_included?(i, optional_arguments) # bl $RBT/toplevel_methods/available_programs.rb
  end; alias cookbooks_include_this_program? is_this_program_included? # === cookbooks_include_this_program?

  # ========================================================================= #
  # === chop_off_archive
  #
  # Remove the trailing part of an archive, for the most part. This isn't
  # very sophisticated but it "just works" for most cases (TM).
  # ========================================================================= #
  def chop_off_archive(i)
    if i.include? '#' # Chop off at '#' characters.
      i = i[0..(i.index('#')-1)]
    end
    i.sub!(/\.source$/,'') if i.include? 'source'
    i.sub!(/-stable$/,'')  if i.end_with? '-stable'
    i = i.sub(/&type=zip$/,'').
          sub(/\.lz$/,'').
          sub(/\.gz$/,'').
          sub(/\.xz$/,'').
          sub(/\.bz2$/,'').
          sub(/\.tar$/,'').
          sub(/\.zip$/,'').
          sub(/\.js$/,'')
    return i
  end

  # ========================================================================= #
  # === set_xorg_buffer
  # ========================================================================= #
  def set_xorg_buffer(i)
    begin
      require 'xorg_buffer'
    rescue LoadError; end
    XorgBuffer.set_xorg_buffer(i)
  end

  # ========================================================================= #
  # === does_this_file_exist?
  # ========================================================================= #
  def does_this_file_exist?(
      i, prepend_this = nil
    )
    ::RBT.does_this_file_exist?(i, prepend_this)
  end

  # ========================================================================= #
  # === does_this_file_exist_and_is_it_a_file?
  # ========================================================================= #
  def does_this_file_exist_and_is_it_a_file?(i)
    ::RBT.does_this_file_exist_and_is_it_a_file?(i)
  end

  # ========================================================================= #
  # === appdir_location_of?
  #
  # Give an input of something like 'foo', this method will return a
  # path such as '/Programs/Foo/Current'.
  # ========================================================================= #
  def appdir_location_of?(i)
    "#{programs_dir?}#{i.capitalize}/Current"
  end

  # ========================================================================= #
  # === file_specification_of_registered_cookbook_entries
  # ========================================================================= #
  def file_specification_of_registered_cookbook_entries
    ::RBT.file_specification_of_registered_cookbook_entries
  end

  # ========================================================================= #
  # === repackage
  #
  # This method will repackage an archive format such as .tar.gz
  # into .tar.xz.
  # ========================================================================= #
  def repackage(
      i, use_this_for_the_opnn_namespace = 'RBT::UpdateEntry'
    )
    begin
      require 'repackage'
      if File.exist? i
        opnn(
          namespace_to_use: use_this_for_the_opnn_namespace
        )
        e "Trying to repackage `#{sfile(i)}` next:"
        Repackage.new(i) { :run_already }
      else
        # Not sure whether we should report to the user or not.
      end
    rescue LoadError
      e 'The gem called "repackage" is not available - please install it'
      e "in order to repackage the archive at hand here (`#{sfile(i)}`)."
    end
  end; alias repackage_this repackage # === repackage_this

  # ========================================================================= #
  # === directory_validation?
  # ========================================================================= #
  def directory_validation?
    RBT.directory_validation?
  end

  # ========================================================================= #
  # === cd_to_the_log_directory
  # ========================================================================= #
  def cd_to_the_log_directory
    cd log_directory?
  end

  # ========================================================================= #
  # === opnef                                                     (opnef tag)
  # ========================================================================= #
  def opnef(i)
    opn(namespace: @namespace) if use_opn?; ee i
  end

  # ========================================================================= #
  # === opnerror                                               (opnerror tag)
  # ========================================================================= #
  def opnerror(
      i, use_this_as_namespace = @namespace
    )
    opn(namespace: use_this_as_namespace) if use_opn?
    stderr i
  end

  # ========================================================================= #
  # === system_tags?
  # ========================================================================= #
  def system_tags?
    RBT.system_tags?
  end

  # ========================================================================= #
  # === determine_appdir_prefix_from_this_input
  #
  # This method has to return a String, which constitutes the AppDir
  # prefix of the target program at hand.
  # ========================================================================= #
  def determine_appdir_prefix_from_this_input(
      i               = nil,
      program_version = nil
    )
    return RBT.determine_appdir_prefix_from_this_input(
      i, program_version
    )
  end; alias static_appdir_prefix_of? determine_appdir_prefix_from_this_input # === static_appdir_prefix_of?

  # ========================================================================= #
  # === open_in_browser
  # ========================================================================= #
  def open_in_browser(i)
    require 'open_in_browser'
    OpenInBrowser[i]
  end

  # ========================================================================= #
  # === is_ccache_available?
  #
  # This method queries as to whether ccache is available or whether
  # it is not.
  # ========================================================================= #
  def is_ccache_available?
    result = true
    begin
      check_for_ccache = `ccache 2>&1`
      if check_for_ccache.include? 'command not found'
        result = false
      end
    rescue Errno::ENOENT
      # ===================================================================== #
      # This clause can happen when there is no
      # /bin/sh symlink.
      # ===================================================================== #
      check_for_ccache = false
    end
    result
  end; alias ccache_is_available? is_ccache_available? # === ccache_is_available?

  # ========================================================================= #
  # === is_porg_available?
  #
  # This method is required to determine whether porg is available or not.
  # ========================================================================= #
  def is_porg_available?
    result = `porg 2>&1`.include? 'porg: command not found'
    return !result
  end; alias porg_is_available? is_porg_available? # === porg_is_available?

  # ========================================================================= #
  # === use_porg?
  #
  # Whether we will use porg for installation or whether we will not.
  #
  # Most users probably do not need/want this, so this is mostly for
  # my home system, but also those users who may wish to use porg.
  # ========================================================================= #
  def use_porg?
    RBT.configuration?.use_porg
  end

  # ========================================================================= #
  # === opne                                                       (opne tag)
  # ========================================================================= #
  def opne(i)
    opn(namespace: @namespace) if use_opn?; e i
  end

  # ========================================================================= #
  # === opnewarn
  # ========================================================================= #
  def opnewarn(i)
    opne swarn(i)
  end

  # ========================================================================= #
  # === return_program_information
  # ========================================================================= #
  def return_program_information(i)
    if i.include? '-'
      i = ProgramInformation.return_real_short_name(i)
    end
    i
  end

  # ========================================================================= #
  # === file_last_symlinked_program?
  # ========================================================================= #
  def file_last_symlinked_program?
    RBT.file_last_symlinked_program?
  end

end; end

if __FILE__ == $PROGRAM_NAME
  base = RBT::Base.new(ARGV)
  pp base
end # rbt_base_file