#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file contains code for a "prototype" RBT::Base class.
#
# The distinction towards file rbt/base/base.rb is somewhat arbitrary but
# the core idea is to keep ile prototype.rb semi-minimal and only put in
# code that is really important for ALL the classes that subclass from it;
# whereas for larger classes, they are encouraged to require base.rb
# instead.
#
# If you need to subclass, you should first try to require base.rb because
# this one also pulls in the file here as well. But if you need only less
# code in your subclass, and even more importantly so, you need to avoid
# "circular dependency" warnings, then you can (safely) require and subclass
# from the file here instead.
#
# In fact, this is why the file here was actually created:
#
#   I needed a file with some functionality but I could not use base.rb.
# =========================================================================== #
# require 'rbt/base/prototype.rb'
# =========================================================================== #
begin
  require 'class_docu_shower'
rescue LoadError; end
require 'rbt/requires/require_base.rb'
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/encoding/encoding.rb'
# =========================================================================== #
# Pull in rbt/base/ specific .rb files next:
# =========================================================================== #
require 'rbt/base/available_programs.rb'
require 'rbt/base/be_verbose.rb'
require 'rbt/base/change_directory.rb'
require 'rbt/base/cliner.rb'
require 'rbt/base/colours.rb'
require 'rbt/base/constants.rb'
require 'rbt/base/copy_directory.rb'
require 'rbt/base/copy_files.rb'
require 'rbt/base/create_directory.rb'
require 'rbt/base/debug.rb'
require 'rbt/base/editor.rb'
require 'rbt/base/ee.rb'
require 'rbt/base/exit_program.rb'
require 'rbt/base/expanded_cookbooks.rb'
require 'rbt/base/ftp.rb'
require 'rbt/base/initialize.rb'
require 'rbt/base/opn.rb'
require 'rbt/base/programs_dir.rb'
require 'rbt/base/remove_directory.rb'
require 'rbt/base/remove_file.rb'
require 'rbt/base/reset.rb'
require 'rbt/base/return_pwd.rb'
require 'rbt/base/save_file.rb'
require 'rbt/base/stub.rb'
require 'rbt/base/symlink.rb'
require 'rbt/base/time.rb'
# =========================================================================== #
# Next add code from some rbt/toplevel_methods/ .rb files:
# =========================================================================== #
require 'rbt/toplevel_methods/archive_type.rb'
require 'rbt/toplevel_methods/chmod.rb'
require 'rbt/toplevel_methods/convert_global_env.rb'
require 'rbt/toplevel_methods/create_directory.rb'
require 'rbt/toplevel_methods/determine_host_architecture.rb'
require 'rbt/toplevel_methods/esystem.rb'
require 'rbt/toplevel_methods/get_all_directories_from.rb'
require 'rbt/toplevel_methods/get_all_files_and_directories_from.rb'
require 'rbt/toplevel_methods/gobolinux.rb'
require 'rbt/toplevel_methods/n_programs_available.rb'
require 'rbt/toplevel_methods/remove_archive_from_the_end.rb'
require 'rbt/toplevel_methods/return_all_archives_from_this_directory.rb'
require 'rbt/toplevel_methods/return_location_to_this_programs_yaml_file.rb'
require 'rbt/toplevel_methods/roebe.rb'
require 'rbt/toplevel_methods/save_file.rb'
require 'rbt/toplevel_methods/source_base_directory.rb'
require 'rbt/toplevel_methods/stderr.rb'
require 'rbt/toplevel_methods/sysbin_directory.rb'
require 'rbt/toplevel_methods/sysetc_directory.rb'
require 'rbt/toplevel_methods/syslib_directory.rb'
require 'rbt/toplevel_methods/sysinclude_directory.rb'
require 'rbt/toplevel_methods/system_directory.rb'
require 'rbt/toplevel_methods/system_share_directory.rb'
require 'rbt/toplevel_methods/verbose_truth.rb'

module RBT

class Base # === RBT::Base

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    puts i
  end

  # ========================================================================= #
  # === readlines
  #
  # This is added in the event that we may wish to use a special encoding,
  # by default, one day.
  # ========================================================================= #
  def readlines(i)
    File.readlines(i)
  end

  # ========================================================================= #
  # === main_encoding?
  # ========================================================================= #
  def main_encoding?
    USE_THIS_ENCODING
  end

  # ========================================================================= #
  # === readlines_with_proper_encoding
  #
  # File.readlines() variant with proper encoding.
  # ========================================================================= #
  def readlines_with_proper_encoding(i)
    File.readlines(
      i, encoding: main_encoding?
    )
  end

  # ========================================================================= #
  # === capitalize_first_alphabetical_character
  #
  # This method will capitalize on the given input, but it will act
  # on the first alphabetical character. So for example, if we
  # input a string such as "/programs" then this method will return
  # "/Programs".
  # ========================================================================= #
  def capitalize_first_alphabetical_character(i = "/programs")
    new_string = ''.dup
    capitalized_already = false
    i.chars.each {|char|
      if capitalized_already
      else
        if char =~ /[a-zA-Z]/
          char.upcase!
          capitalized_already = true
        end
      end
      new_string << char
    }
    new_string
  end

  # ========================================================================= #
  # === remove_newlines
  #
  # Get rid of the newlines.
  # ========================================================================= #
  def remove_newlines(i)
    i.delete("\n")
  end

  # ========================================================================= #
  # === remove_double_slashes                                       (rds tag)
  #
  # Replace // with / in a given string.
  # ========================================================================= #
  def remove_double_slashes(i)
    i.squeeze '/'
  end; alias rds remove_double_slashes # === rds

  # ========================================================================= #
  # === touch                                                     (touch tag)
  #
  # Use this unified method whenever you wish to create a new file,
  # like the UNIX "touch" command.
  # ========================================================================= #
  def touch(
      i,
      be_verbose = false
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    if be_verbose
      e "Next creating the file `#{sfile(i)}`."
    end
    FileUtils.touch(i)
  end; alias create_file touch # === create_file
       alias touch_file  touch # === touch_file

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint(
      optional_message      = nil,
      use_this_as_exit_code = 0
    )
    Signal.trap('SIGINT') {
      if optional_message
        e optional_message
      end
      exit(use_this_as_exit_code)
    }
  end

  # ========================================================================= #
  # === to_camelcase
  # ========================================================================= #
  def to_camelcase(i)
    i.split('_').map { |_| _.capitalize }.join
  end

  # ======================================================================== #
  # === sanitize_for_environment_variable
  # ======================================================================== #
  def sanitize_for_environment_variable(i)
    RBT.convert_global_env(i)
  end; alias convert_global_env sanitize_for_environment_variable # === convert_global_env

  # ========================================================================= #
  # === rename
  # ========================================================================= #
  def rename(old_name, new_name)
    if File.exist? old_name
      File.rename(old_name, new_name)
    end
  end

  # ========================================================================= #
  # === gem_version
  #
  # This method is a wrapper over Gem::Version.new()
  #
  # The reason as to why this resides in a method is so that we can do
  # some input-sanitizing, and easier rescue, if this is necessary one
  # day.
  # ========================================================================= #
  def gem_version(i)
    i = i.to_s.delete('vr')
    begin
      if i =~ /\d+/ # Must have at the least one number.
        return Gem::Version.new(i)
      end
    rescue ArgumentError => error
      opnn; e 'An error occurred in gem_version()'
      pp error
    end
    nil # Indicate "failure" aka unable to parse this version.
  end

  # ========================================================================= #
  # === esystem
  #
  # Combine system() with output of the command.at hand.
  # ========================================================================= #
  def esystem(i)
    RBT.esystem(i)
  end

  # ========================================================================= #
  # === silently_create_this_directory_if_it_does_not_yet_exist
  # ========================================================================= #
  def silently_create_this_directory_if_it_does_not_yet_exist(i)
    create_directory(i, :be_quiet)
  end

  # ========================================================================= #
  # === n_programs_available?
  # ========================================================================= #
  def n_programs_available?
    RBT.n_programs_available?
  end; alias n_programs_available n_programs_available? # === n_programs_available

  # ========================================================================= #
  # === get_files_and_directories_from
  # ========================================================================= #
  def get_files_and_directories_from(
      i = Dir.pwd
    )
    RBT.get_files_and_directories_from(i)
  end

  # ========================================================================= #
  # === all_binaries?
  # ========================================================================= #
  def all_binaries?
    require 'rbt/toplevel_methods/all_binaries.rb'
    RBT.all_binaries?
  end

  # ========================================================================= #
  # === all_libraries?
  # ========================================================================= #
  def all_libraries?
    require 'rbt/toplevel_methods/all_libraries.rb'
    RBT.all_libraries?
  end
     
  # ========================================================================= #
  # === verbose_truth
  #
  # This will give us back "yes" or "no", in String form.
  # ========================================================================= #
  def verbose_truth(i)
    RBT.verbose_truth(i)
  end; alias vt             verbose_truth # === vt
       alias verbose_truth? verbose_truth # === verbose_truth?

  # ========================================================================= #
  # === load_yaml
  # ========================================================================= #
  def load_yaml(i)
    YAML.load_file(i)
  end

  # ========================================================================= #
  # === store_into_this_directory?
  # ========================================================================= #
  def store_into_this_directory?
    _ = RBT.store_into_this_directory?
    ensure_that_this_directory_exists(_) # Make sure that the directory exists.
    return _
  end; alias store_where? store_into_this_directory? # === store_where?

  # ========================================================================= #
  # === is_roebe?
  #
  # If we are on our local computer, or on another computer.
  # ========================================================================= #
  def is_roebe?
    RBT.is_roebe?
  end; alias is_on_roebe? is_roebe? # === is_on_roebe?
       alias on_roebe?    is_roebe? # === on_roebe?

  # ========================================================================= #
  # === is_on_gobolinux?
  #
  # Method to query whether the host system is using gobolinux or
  # whether it is not.
  # ========================================================================= #
  def is_on_gobolinux?
    RBT.is_on_gobolinux?
  end

  # ========================================================================= #
  # === move_file                                          (move tag, mv tag)
  #
  # Move a file via this method here. Use this consistently whenever you
  # move a file. Could also be done via File.mv().
  # ========================================================================= #
  def move_file(
      this_file, where_to, be_verbose = true
    )
    if be_verbose
      e 'Moving from '+sfile(this_file)+' to '+sfile(where_to)+'.'
    end
    FileUtils.mv(this_file, where_to)
  end; alias mv move_file # === mv

  # ========================================================================= #
  # === get_all_files_from
  # ========================================================================= #
  def get_all_files_from(i)
    RBT.get_all_files_from(i)
  end

  # ========================================================================= #
  # === chmod
  # ========================================================================= #
  def chmod(i)
    RBT.chmod(i)
  end

  # ========================================================================= #
  # === remove                                                   (remove tag)
  #
  # General remove entry-method. If you wish to delete a file or a directory
  # or a symlink, use this method.
  # ========================================================================= #
  def remove(
      this_target,
      be_verbose = false
    )
    if this_target.is_a? Array
      this_target.each {|entry| remove(entry, be_verbose) }
    else
      # ===================================================================== #
      # The next check requires a rescue clause, because the file may no
      # longer exist.
      # ===================================================================== #
      begin
        type = File.ftype(this_target)
      rescue Errno::ENOENT
        type = 'missing_entry'
      end
      if File.exist? this_target
        case type # case tag
        # =================================================================== #
        # === remove_symlink
        # =================================================================== #
        when 'link','missing_entry' # This entry is for symlinks.
          remove_symlink(this_target)
        # =================================================================== #
        # === remove_file
        # =================================================================== #
        when 'file'
          remove_file(this_target, be_verbose)
        # =================================================================== #
        # === remove_directory
        # =================================================================== #
        when 'directory'
          remove_directory(this_target)
        else # This else clause will not be entered, I think.
          ewarn "Not removing `#{simp(this)}` as \"#{simp(type)}\" is "\
                "not registered."
        end
      else
        if be_verbose
          opnn; e swarn('WARNING: file ')+this_target.to_s+swarn(' does not exist.')
          opnn; e swarn('Thus, it can not be removed.')
        end
      end
    end
  end; alias delete remove # === delete

  # ========================================================================= #
  # === go_to_base_dir
  #
  # Simply go to the temp-directory.
  # ========================================================================= #
  def go_to_base_dir(be_verbose = true)
    _ = temp_dir?
    if be_verbose
      opnn; e 'Changing to '+sdir(_)
    end
    cd(_)
  end

  # ========================================================================= #
  # === cd_to_the_temp_directory
  #
  # This will cd to the temp directory, while being quiet.
  # ========================================================================= #
  def cd_to_the_temp_directory(
      be_verbose = :be_quiet
    )
    cd temp_directory?, be_verbose
  end

  # ========================================================================= #
  # === find_cookbook_alias_for
  #
  # This method depends on the Cookbooks gem.
  #
  # It allows us to find the registered alias for a given program, and
  # thus acts as an "input-sanitizer".
  # ========================================================================= #
  def find_cookbook_alias_for(i)
    RBT.find_cookbook_alias_for(i)
  end

  # ========================================================================= #
  # === debug
  #
  # Debug functionality here.
  # ========================================================================= #
  def debug(i)
    cliner { ewarn(i) }
  end

  # ========================================================================= #
  # === run_simulation?
  #
  #
  # This method will tell us whether we shall we run in simulation mode or
  # whether we shall not. The default is false, as in, we will not run in
  # simulation mode.
  #
  # The simulation mode is required to tell the user what we would do,
  # without actually doing any of these changes. It is a "dry run", a
  # test run.
  # ========================================================================= #
  def run_simulation?
    @run_simulation
  end; alias run_simulation run_simulation? # === run_simulation
       alias in_simulation? run_simulation? # === in_simulation?

  # ========================================================================= #
  # === run_simulation
  #
  # Setter method for the ivar @run_simulation.
  #
  # This way we can run in simulation mode. That way, we won't do any
  # modifications, we will only assume that certain things be done.
  # ========================================================================= #
  def run_simulation=(i = false)
    @run_simulation = i
  end

  # ========================================================================= #
  # === cookbook_directory
  #
  # This method will return a String such as:
  #
  #   /Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/individual_cookbooks/
  #
  # ========================================================================= #
  def cookbook_directory?
    RBT.cookbook_directory?
  end; alias cookbooks_directory             cookbook_directory? # === cookbooks_directory
       alias cookbooks_directory?            cookbook_directory? # === cookbooks_directory?
       alias cookbook_directory?             cookbook_directory? # === cookbook_directory?
       alias individual_cookbooks_dir?       cookbook_directory? # === individual_cookbooks_dir?
       alias individual_cookbooks_directory? cookbook_directory? # === individual_cookbooks_directory?
  
  # ========================================================================= #
  # === convert_env_variable
  #
  # This method can be used to convert a ENV variable, such as $BLA,
  # into its corresponding "real" value.
  # ========================================================================= #
  def convert_env_variable(i)
    ConvertGlobalEnv[i]
  end; alias convert_global_env convert_env_variable # === convert_global_env

  # ========================================================================= #
  # === is_file?
  # ========================================================================= #
  def is_file?(i)
    File.file? i
  end

  # ========================================================================= #
  # === in_simulation?
  # ========================================================================= #
  def in_simulation?
    @run_simulation
  end

  # ========================================================================= #
  # === abbreviations?
  #
  # Easier getter-method over the available abbreviations.
  # ========================================================================= #
  def abbreviations?
    RBT.hash_aliases_to_the_available_programs?
  end

  # ========================================================================= #
  # === return_program_name
  #
  # This will always return the proper program name. If you need to modify
  # that name, you must do so on your own.
  # ========================================================================= #
  def return_program_name(i, optional_padding = ' ')
    _ = ''.dup
    if REPORT_SPECIFIC_PROGRAM_NAME
      _ << "#{File.basename(i)}:#{optional_padding}"
    end
    return _
  end; alias rpn return_program_name # === rpn

  # ========================================================================= #
  # === get_all_programs                                   (all programs tag)
  #
  # Simply returns all programs from the $PROGRAMS directory.
  # ========================================================================= #
  def get_all_programs(
      from = "#{RBT.programs_directory?}*"
    )
    Dir[from] # The constant PROGRAMS is set in config.rb. It has a trailing / 
  end

  # ========================================================================= #
  # === return_program_name_for_gobolinux_systems
  #
  # This method should ideally return the program name for GoboLinux
  # systems.
  # ========================================================================= #
  def return_program_name_for_gobolinux_systems(i)
    return i
  end

  # ========================================================================= #
  # === word_wrap
  #
  # The first argument is the text that will be reformatted.
  #
  # The second argument is at which position we will wrap it.
  # ========================================================================= #
  def word_wrap(
      this_text, n_characters_limit = 78
    )
    if this_text.respond_to? :encoding
      unless this_text.encoding.to_s.include? main_encoding?
        this_text = ensure_main_encoding_for(this_text)
      end
      this_text.gsub(/(.{1,#{n_characters_limit}})(\s+|$)/, "\\1\n")
    end
  end

  # ========================================================================= #
  # === archive_type_of?
  #
  # Determine the archive type of the given input, such as '.tar.xz' and
  # so forth.
  # ========================================================================= #
  def archive_type_of?(i)
    RBT.return_archive_type(i)
  end; alias return_archive_type archive_type_of? # === return_archive_type
       alias archive_type?       archive_type_of? # === archive_type
       alias archive_type_of     archive_type_of? # === archive_type_of

  # ========================================================================= #
  # === find_this_yaml_file
  #
  # This method can be used to find a proper yaml file.
  #
  # We will delegate to Cookbooks for this though.
  #
  # We will return the absolute path to the .yml file in question.
  #
  # Usage example:
  #
  #   find_this_yaml_file(:poppler) # => "/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/yaml/individual_cookbooks/poppler.yml"
  #
  # ========================================================================= #
  def find_this_yaml_file(i = :poppler)
    i = i.to_s
    if File.exist? i
      i
    else
      RBT.return_location_of_this_yaml_file(i)
    end
  end

  # ========================================================================= #
  # === read_file                                                  (read tag)
  #
  # Use this method whenever you want to read in a file.
  # ========================================================================= #
  def read_file(i)
    File.read(i) if File.exist? i
  end

  # ========================================================================= #
  # === read_file_in_default_encoding
  # ========================================================================= #
  def read_file_in_default_encoding(
      i, use_this_encoding = ::RBT.encoding?
    )
    if File.exist? i
      File.read(
        i, encoding: use_this_encoding
      )
    else
      e "Can not read file `#{i}` as it does not exist."
    end
  end

  # ========================================================================= #
  # === return_location_to_this_programs_yaml_file
  #
  # Easier access-method to determine where the yaml file may be.
  # ========================================================================= #
  def return_location_to_this_programs_yaml_file(i)
    RBT.return_location_to_this_programs_yaml_file(i)
  end

  # ========================================================================= #
  # === ensure_main_encoding_for
  #
  # The input to this method should be a String object.
  # ========================================================================= #
  def ensure_main_encoding_for(
      i, use_this_encoding = USE_MAIN_ENCODING
    )
    unless i.encoding.to_s.include? use_this_encoding
      i = i.force_encoding(use_this_encoding)
    end
    return i
  end

  # ========================================================================= #
  # === file_predefined_installation_instructions?
  # ========================================================================= #
  def file_predefined_installation_instructions?
    RBT.file_predefined_installation_instructions
  end; alias file_predefined_installation_instructions file_predefined_installation_instructions? # === file_predefined_installation_instructions?

  # ========================================================================= #
  # === predefined_installation_instructions?
  # ========================================================================= #
  def predefined_installation_instructions? 
    RBT.predefined_installation_instructions?
  end

  # ========================================================================= #
  # === cheering_person?
  # ========================================================================= #
  def cheering_person?
    '\o/'
  end; alias return_cheering_person cheering_person? # === return_cheering_person

  # ========================================================================= #
  # === return_all_archives_from_this_directory
  # ========================================================================= #
  def return_all_archives_from_this_directory(i)
    RBT.return_all_archives_from_this_directory(i)
  end

  # ========================================================================= #
  # === no_such_file_exists
  # ========================================================================= #
  def no_such_file_exists(i)
    e "No file called `#{sfile(i)}` appears to exist."
  end; alias no_file_exists_at no_such_file_exists # === no_file_exists_at

  # ========================================================================= #
  # === no_directory_exists_at
  # ========================================================================= #
  def no_directory_exists_at(i)
    e "No directory exists at `#{sdir(i)}`."
  end

  # ========================================================================= #
  # === project_base_dir?
  # ========================================================================= #
  def project_base_dir?
    RBT::PROJECT_BASE_DIRECTORY
  end

  # ========================================================================= #
  # === home_dir?
  # ========================================================================= #
  def home_dir?
    ENV['HOME'].to_s.dup
  end

  # ========================================================================= #
  # === stderr
  # ========================================================================= #
  def stderr(
      i = '', use_puts_or_print = :puts
    )
    RBT.stderr(i, use_puts_or_print)
  end

  # ========================================================================= #
  # === get_all_directories_from
  #
  # This method can be used to obtain all directories from the given
  # (first) input argument.
  # ========================================================================= #
  def get_all_directories_from(
      here             = return_pwd,
      return_full_path = false
    )
    RBT.get_all_directories_from(here, return_full_path)
  end; alias get_dir_listing       get_all_directories_from # === get_dir_listing
       alias get_directory_listing get_all_directories_from # === get_directory_listing
       alias get_directories_from  get_all_directories_from # === get_directories_from
       alias get_dir_listing       get_all_directories_from # === get_dir_listing

  # ========================================================================= #
  # === is_an_archive?
  # ========================================================================= #
  def is_an_archive?(i)
    RBT.is_an_archive?(i)
  end; alias is_archive? is_an_archive? # === is_archive?

  # ========================================================================= #
  # === change_permission                                         (chown tag)
  #
  # This simple method can be used to change permissions, by
  # making use of FileUtils.chown().
  # ========================================================================= #
  def change_permission(
      file                = 'test',
      use_this_uid_number = '1000'
    )
    FileUtils.chown(use_this_uid_number, use_this_uid_number, file) # Hardcoded right now.
  end; alias chown change_permission # === chown

  # ========================================================================= #
  # === remove_file_extension
  #
  # This method will try to remove anything we declare to be an improper
  # file extension. We do this by delegating towards class RemoveFileSuffix,
  # an external dependency.
  # ========================================================================= #
  def remove_file_extension(
      i, also_remove_the_basename = true
    )
    case also_remove_the_basename
    when :do_not_strip_anything
      also_remove_the_basename = false
    end
    if also_remove_the_basename
      i = File.basename(i)
    end
    if    i.end_with? '.src'
      i[-4,4] = ''
    elsif i.end_with? '-src'
      i[-4,4] = ''
    end
    i = remove_unnecessary_data_from_url(i.to_s.dup)
    remove_archive_from_the_end(i)
  end; alias remove_extension                remove_file_extension # === remove_extensions
       alias remove_file_suffix              remove_file_extension # === remove_file_suffix
       alias remove_archive_at_the_end       remove_file_extension # === remove_archive_at_the_end
       alias remove_archive_stuff_at_the_end remove_file_extension # === remove_archive_stuff_at_the_end

  # ========================================================================= #
  # === unicode_cliner
  # ========================================================================= #
  def unicode_cliner(a = :default, b = :default, &block)
    ::RBT.unicode_cliner(a, b, &block)
  end

  # ========================================================================= #
  # === unicode_middle_cliner
  # ========================================================================= #
  def unicode_middle_cliner(a = :default, b = :default)
    ::RBT.unicode_cliner(a, b) { :unicode_middle_horizontal_bar }
  end

  # ========================================================================= #
  # === remove_archive_from_the_end
  #  
  # This method will remove the "archive part" of the given input, such
  # as by removing ".tar.xz" from the given input argument (which ought
  # to be a String).
  # ========================================================================= #
  def remove_archive_from_the_end(i)
    RBT.remove_archive_from_the_end(i)
  end; alias remove_archive_at_the_end      remove_archive_from_the_end # === remove_archive_at_the_end
       alias remove_file_archive_at_the_end remove_archive_from_the_end # === remove_file_archive_at_the_end

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(i)
    available_programs?.include? i
  end; alias included?                   is_this_program_included? # === included?
       alias includes?                   is_this_program_included? # === includes?
       alias program_was_found?          is_this_program_included? # === program_was_found?
       alias include?                    is_this_program_included? # === include?
       alias is_included?                is_this_program_included? # === is_included?
       alias includes_this_program?      is_this_program_included? # === includes_this_program?
       alias does_include?               is_this_program_included? # === does_include?
       alias is_this_program_registered? is_this_program_included? # === is_this_program_registered?

  # ========================================================================= #
  # === log_directory?
  #
  # This will return a String such as "/Depot/Temp/rbt/".
  # ========================================================================= #
  def log_directory?
    RBT.log_directory?
  end; alias rbt_temp_directory? log_directory? # === rbt_temp_directory?
       alias log_dir?            log_directory? # === log_dir?
       alias rbt_log_dir?        log_directory? # === rbt_log_dir?
       alias rbt_log_directory?  log_directory? # === rbt_log_directory?

  # ========================================================================= #
  # === project_yaml_directory?
  #
  # This will return the path to the yaml directory.
  # ========================================================================= #
  def project_yaml_directory?
    RBT.yaml_directory?
  end; alias rbt_path? project_yaml_directory? # === rbt_path?

  # ========================================================================= #
  # === source_base_directory?
  # ========================================================================= #
  def source_base_directory?
    RBT.source_base_directory?
  end; alias source_directory? source_base_directory? # === source_directory?
       alias source_directory  source_base_directory? # === source_directory
       alias source_dir?       source_base_directory? # === source_dir?
       alias archive_dir?      source_base_directory? # === archive_dir?
       alias src_dir?          source_base_directory? # === src_dir?

  # ========================================================================= #
  # === temp_directory?
  # ========================================================================= #
  def temp_directory?
    RBT.temp_directory?
  end; alias temp_dir? temp_directory? # === temp_dir?
       alias log_dir?  temp_directory? # === log_dir?
       alias base_dir? temp_directory? # === base_dir?

  # ========================================================================= #
  # === system_directory?
  # ========================================================================= #
  def system_directory?
    RBT.system_directory?
  end

  # ========================================================================= #
  # === programs_dir?
  # ========================================================================= #
  def programs_dir?
    RBT.programs_directory?
  end; alias programs_directory? programs_dir? # === programs_directory?
       alias program_dir?        programs_dir? # === program_dir?

  # ========================================================================= #
  # === sysbin_directory?
  # ========================================================================= #
  def sysbin_directory?
    RBT.sysbin_directory?
  end

  # ========================================================================= #
  # === sysinclude_directory?
  # ========================================================================= #
  def sysinclude_directory?
    RBT.sysinclude_directory?
  end

  # ========================================================================= #
  # === syslib_directory?
  # ========================================================================= #
  def syslib_directory?
    RBT.syslib_directory?
  end

  # ========================================================================= #
  # === sysshare_directory?
  # ========================================================================= #
  def sysshare_directory?
    RBT.sysshare_directory?
  end

  # ========================================================================= #
  # === sysetc_directory?
  # ========================================================================= #
  def sysetc_directory?
    RBT.sysetc_directory?
  end

end; end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  x = RBT::Base.new
  e x.project_yaml_directory?
  e x.return_weekday_based_on_utc
  e x.return_utc_time_in_a_format_similar_to_slackware
  e x.verbose_truth('true')
  e x.verbose_truth('t')
end # rbt_prototype