#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Time and date-related content should be stored in this file here.
# =========================================================================== #
# require 'rbt/base/time.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/time.rb'

module RBT

class Base

  # ========================================================================= #
  # === return_utc
  #
  # This method will return a Time object that is in UTC format, such
  # as "2018-12-28 14:09:26 UTC".
  # ========================================================================= #
  def return_utc
    Time.now.getutc
  end

  # ========================================================================= #
  # === get_date
  #
  # This method will return a String such as "21 Sep 2017".
  # ========================================================================= #
  def get_date
    ::RBT.get_date
  end

  # ========================================================================= #
  # === return_full_time
  #
  # This method will return a String such as '21.09.2017, 03:03:09'.
  # ========================================================================= #
  def return_full_time
    "#{return_date}, #{return_current_hour_minutes_second}"
  end

  # ========================================================================= #
  # === return_weekday_based_on_utc
  #
  # This will return e. g. 'Mon' or 'Fri' or something like that.
  # ========================================================================= #
  def return_weekday_based_on_utc
    _ = return_utc
    wday = _.wday
    Time::RFC2822_DAY_NAME[wday]
  end

  # ========================================================================= #
  # === return_day_of_the_month_based_on_utc
  # ========================================================================= #
  def return_day_of_the_month_based_on_utc
    return_utc.day.to_s
  end

  # ========================================================================= #
  # === return_year_based_on_utc
  # ========================================================================= #
  def return_year_based_on_utc
    return_utc.year.to_s
  end

  # ========================================================================= #
  # === return_hours_minutes_seconds_based_on_utc
  # ========================================================================= #
  def return_hours_minutes_seconds_based_on_utc
    _ = return_utc
    _.strftime('%H:%M:%S') 
  end

  # ========================================================================= #
  # === return_utc_time_in_a_format_similar_to_slackware
  #
  # Slackware changelog uses a format such as this one here, in a UTC
  # format:
  #   Thu Sep 21 01:23:24 UTC 2017
  # ========================================================================= #
  def return_utc_time_in_a_format_similar_to_slackware
    return_weekday_based_on_utc.to_s+' '+
    return_month_based_on_utc.to_s+' '+
    return_day_of_the_month_based_on_utc+' '+
    return_hours_minutes_seconds_based_on_utc+
    ' UTC '+
    return_year_based_on_utc
  end
  
  # ========================================================================= #
  # === return_month_based_on_utc
  # ========================================================================= #
  def return_month_based_on_utc
    Date::MONTHNAMES[return_utc.month]
  end

  # ========================================================================= #
  # === return_month_based_on_this_number
  #
  # The input to this method should be an Integer, for the month, such
  # as 1,2,3 and so forth.
  # ========================================================================= #
  def return_month_based_on_this_number(i)
    Date::MONTHNAMES[i.to_i]
  end

  # ========================================================================= #
  # === current_hour?
  #
  # Consistently use this method whenever you wish to return the
  # current hour.
  # ========================================================================= #
  def current_hour?
    Time.now.strftime('%H:%M:%S') # '04:21:14'
  end; alias return_time current_hour? # === current_hour?

  # ========================================================================= #
  # === today?
  #
  # This method will return a String such as "21 September 2017".
  # ========================================================================= #
  def today?(
      display_in_long_format = true
    )
    if display_in_long_format # This is the default.
      Time.now.strftime('%d %B %Y') # => "28 December 2018"
    else
      Time.now.strftime('%d.%m.%Y') # => "28.12.2018"
    end
  end

  # ========================================================================= #
  # === return_current_hour_minutes_second
  #
  # This method will return the current time, in HH::MM::SS format.
  # ========================================================================= #
  def return_current_hour_minutes_second
    Time.now.strftime '%H:%M:%S' # => "16:08:40"
  end

  # ========================================================================= #
  # === return_date
  #
  # This method wil return a date (a day), such as "21.09.2017" or
  # "03.06.2018" - in other words, the dd.mm.yyyy format.
  # ========================================================================= #
  def return_date
    ::RBT.return_date
  end; alias return_current_date return_date # === return_current_date
       alias dd_mm_yyyy          return_date # === dd_mm_yyyy

  # ========================================================================= #
  # === dd_mmm_yyy
  #
  # This method-format is specifically the default for cookbook-files.
  # ========================================================================= #
  def dd_mmm_yyy
    current_day = Time.now.strftime('%d') # This will return a string such as "30".
    current_day+
    ' '+
    Date::MONTHNAMES[Date.today.month][0,3].capitalize+
    ' '+
    Time.now.strftime('%Y')

  end

end; end