#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForRemoteWebpages
#
# This is the base class from which all other .rb files in the same
# directory should subclass from. These classes have a common
# functionality in that they deal with updating programs from a
# remote source.
# =========================================================================== #
# require 'rbt/check_for_updates/base.rb'
# =========================================================================== #
require 'open-uri'
require 'optparse'
require 'rbt/base/base.rb'
require 'rbt/constants/remote_urls.rb'
require 'rbt/cookbooks/class/class.rb'
require 'rbt/utility_scripts/update_entry.rb'
begin
  require 'program_information'
rescue LoadError; end

module RBT

module Cookbooks

class CheckForRemoteWebpages < Base # === RBT::Cookbooks::CheckForRemoteWebpages

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @dataset = nil # <- Will store the downloaded-dataset (from open-uri).
    @downloaded_n_programs = 0 # <- Keeps track of how many programs were batch-downloaded.
    # ======================================================================= #
    # The next Array can keep track of which programs were updated.
    # ======================================================================= #
    @array_these_programs_were_updated = []
    @commandline_arguments = []
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === help
      # ===================================================================== #
      when /^-?-?help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input?       commandline_arguments? # === input?
       alias commandline? commandline_arguments? # === commandline?

  # ========================================================================= #
  # === commandline_size_query_was_issued?
  # ========================================================================= #
  def commandline_size_query_was_issued?
    @commandline_arguments.any? {|entry|
      entry.include? 'size'
    }
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = ARGV)
    i = [i].flatten.compact
    @commandline_arguments = i
  end; alias set_commandline set_commandline_arguments # === set_commandline
       alias set_input       set_commandline_arguments # === set_input

  # ========================================================================= #
  # === return_local_version_of_this_program
  # ========================================================================= #
  def return_local_version_of_this_program(i)
    RBT::Cookbooks::Cookbook.new(i) { :bypass_menu }.program_version?
  end

  # ========================================================================= #
  # === correct_for_common_mistakes_in_program_names
  #
  # The BLFS folks, but also the slackware folks, use some wrong names.
  #
  # The following ad-hoc case/when menu will correct some of these
  # wrong names.
  #
  # The correct name is the one that comes from the official tarball
  # release. So for example, for "gtk+-3.93.0.tar.xz", the official
  # name is exactly "gtk+" and NOT "gtk+3", despite the BLFS team
  # thinking otherwise.
  # ========================================================================= #
  def correct_for_common_mistakes_in_program_names(i)
    case i # case tag, sanitize tag
    when 'node.js'
      i = 'node'
    when 'openjpeg2'
      i = 'openjpeg'
    when 'gtk+3'
      i = 'gtk+'
    end
    return i
  end
    
  # ========================================================================= #
  # === report_the_remote_webpage
  #
  # Standard method to report the remote webpage in use.
  # ========================================================================= #
  def report_the_remote_webpage(
      i = remote_webpage?
    )
    opnn; e 'Attempting to obtain data from the remote URL'
    opnn; e "  `#{sfancy(i)}`."
  end

  # ========================================================================= #
  # === report_that_these_programs_were_updated
  # ========================================================================= #
  def report_that_these_programs_were_updated(array)
    unless array.empty?
      opnn; e "The following #{simp(array.size.to_s)} programs were updated:"
      array.each {|this_program|
        e "  #{sfancy(this_program)}"
      }
    end
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === notify_the_user_that_no_downloads_have_occurred
  # ========================================================================= #
  def notify_the_user_that_no_downloads_have_occurred
    opnn; e 'No downloads occurred. This usually means that all '\
            'local programs are up to date! '+yellow('\\o/')
  end

  # ========================================================================= #
  # === obtain_dataset_from_the_remote_webpage
  # ========================================================================= #
  def obtain_dataset_from_the_remote_webpage(
      url_to_the_remote_webpage = remote_webpage?
    )
    begin
      @dataset = open(url_to_the_remote_webpage).read
    rescue OpenURI::HTTPError => error
      opnn; e 'OpenURI::HTTPError was encountered. This usually'
      opnn; e "happens when the URL is invalid (#{sfancy(url_to_the_remote_webpage)}"
      opnn; e 'Consider fixing the URL and trying again.'
      pp error
      exit
    end
  end; alias obtain_remote_dataset obtain_dataset_from_the_remote_webpage # === obtain_remote_dataset

end; end; end