#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForNewReleaseOnRubygems
#
# This class accepts a single argument, the remote URL to a .gem project.
# It will then check for an update, and if there is one, automatically
# update this entry by tapping into class UpdateEntry.
#
# This file also holds code that allows us to batch-update all
# registered rubygems.
# =========================================================================== #
# require 'rbt/check_for_updates/check_for_new_release_on_rubygems.rb'
# =========================================================================== #
require 'rbt/check_for_updates/base.rb'
require 'rbt/utility_scripts/update_entry.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/toplevel_methods/update_all_ruby_gems.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class CheckForNewReleaseOnRubygems < CheckForRemoteWebpages # === RBT::Cookbooks::CheckForNewReleaseOnRubygems

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === RUBYGEMS_ORG_DOWNLOADS_URL
  # ========================================================================= #
  RUBYGEMS_ORG_DOWNLOADS_URL =
    'https://rubygems.org/downloads/'

  # ========================================================================= #
  # === USE_THIS_REGEX
  #
  # This is the main regex that will be used.
  #
  # See: http://rubular.com/r/QcooOoIXVU
  # ========================================================================= #
  USE_THIS_REGEX =
    /    <a rel="nofollow" class="gem__link t-list__item" id="download" href="\/downloads\/(.+).gem">Download<\/a>/

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      remote_url,
      run_already = true
    )
    reset
    set_remote_url(
      remote_url
    )
    case remote_url
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @array_we_did_update_these_programs = []
  end

  # ========================================================================= #
  # === set_remote_url
  # ========================================================================= #
  def set_remote_url(i)
    i = i.first if i.is_a? Array
    @remote_url = i
  end

  # ========================================================================= #
  # === remote_url?
  # ========================================================================= #
  def remote_url?
    @remote_url
  end

  # ========================================================================= #
  # === identify_homepage
  #
  # This method will determine the homepage of the given .gem file at
  # hand. Keep in mind that a homepage on rubygems.org may have '-'
  # characters.
  # ========================================================================= #
  def identify_homepage
    _ = remote_url?
    if _.end_with? '.gem'
      basename = File.basename(_)
      basename = ProgramInformation.new(basename).program_name?
      basename << '/' unless basename.end_with? '/'
      @homepage = 'https://rubygems.org/gems/'+basename
      opnn; e "Next checking for any #{crimson('updates')}"\
              " on the remote URL #{sfancy(@homepage)}"
      obtain_dataset
      apply_the_main_regex_onto_the_dataset
      compare_the_remote_program_version_with_the_local_program_version
    end
  end

  # ========================================================================= #
  # === reset_main_array
  # ========================================================================= #
  def reset_main_array
    @array_we_did_update_these_programs = []
  end

  # ========================================================================= #
  # === colourize_this_local_version
  # ========================================================================= #
  def colourize_this_local_version(i)
    springgreen(i)
  end

  # ========================================================================= #
  # === compare_the_remote_program_version_with_the_local_program_version
  # ========================================================================= #
  def compare_the_remote_program_version_with_the_local_program_version
    reset_main_array
    program_name = File.basename(@homepage)
    local_version = RBT::Cookbooks::Cookbook.new(program_name) { :bypass_menu_check }.program_version?
    remote_program_version = ProgramInformation.new(@remote_program_version).version?
    if remote_program_version == local_version
      opnn; e "The local program #{sfancy(program_name)} is "\
              "#{turquoise('up to date')}"\
              "! (At version: #{colourize_this_local_version(local_version)})"
    else
      # ===================================================================== #
      # We must build up the remote download-URL next.
      # ===================================================================== #
      remote_download_url =
        "#{RUBYGEMS_ORG_DOWNLOADS_URL}#{@remote_program_version}.gem"
      opnn; e "Now updating to a new program version "\
              "at `#{sfancy(remote_download_url)}`."
      # ===================================================================== #
      # Delegate towards class UpdateEntry next.
      # ===================================================================== #
      RBT::UpdateEntry.new(remote_download_url)
      @array_we_did_update_these_programs << remote_download_url
    end
    unless @array_we_did_update_these_programs.empty?
      opnn; e "The following "\
              "#{simp(@array_we_did_update_these_programs.size.to_s)}"\
              " programs were updated:"
      @array_we_did_update_these_programs.each {|remote_url|
        e sfancy("  #{remote_url}")
      }
      # ===================================================================== #
      # Also store this result in a module-level instance variable next.
      # That way we can query the state of programs that were compiled.
      # ===================================================================== #
      RBT.set_array_these_ruby_gems_were_updated(
        @array_we_did_update_these_programs
      )
    end
  end

  # ========================================================================= #
  # === apply_the_main_regex_onto_the_dataset
  # ========================================================================= #
  def apply_the_main_regex_onto_the_dataset
    @dataset =~ USE_THIS_REGEX
    @remote_program_version = $1.to_s.dup
  end

  # ========================================================================= #
  # === obtain_dataset
  #
  # The first URL to this method should be the remote homepage of the
  # given .gem at hand.
  # ========================================================================= #
  def obtain_dataset(
      use_this_url = @homepage
    )
    begin
      @dataset = open(use_this_url).read
    rescue OpenURI::HTTPError
      opnn; e 'OpenURI::HTTPError error for `'+sfancy(use_this_url)+'`.'
    end
  end

  # ========================================================================= #
  # === check_this_remote_url
  # ========================================================================= #
  def check_this_remote_url(remote_url)
    set_remote_url(remote_url)
    run
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    identify_homepage
  end

end; end

  # ========================================================================= #
  # === RBT.update_all_ruby_gems
  #
  # This method can be used to update all local ruby gems (.gem files).
  # ========================================================================= #
  def self.update_all_ruby_gems
    # ======================================================================= #
    # Instantiate the main class defined in this .rb file here:
    # ======================================================================= #
    _ = RBT::Cookbooks::CheckForNewReleaseOnRubygems.new(:dont_run_yet)
    base_dir = RBT.individual_cookbooks_directory?
    # ======================================================================= #
    # First, we must obtain all entries thave have the strings "url1"
    # and ".gem". We will ask ruby to perform a grep-operation. We will
    # store these in an array.
    # ======================================================================= #
    array_ruby_gems = []
    available_programs = available_programs?
    available_programs.each {|this_program|
      target_yaml_file = "#{base_dir}#{this_program}.yml"
      if File.exist? target_yaml_file
        dataset = File.read(target_yaml_file)
        if dataset.include?('url1')
          line_holding_url1 = dataset.scan(/^ url1: (.+)$/).flatten.first
          if line_holding_url1.nil?
            e 'No result has been found on the dataset at hand, from the file'
            e sfile(target_yaml_file)
          end
          if line_holding_url1
            if line_holding_url1.end_with? '.gem' # Found ruby gems.
              array_ruby_gems << [this_program, line_holding_url1] 
            end
          end
        end
      end
    }
    # ======================================================================= #
    # Ok - now that we have obtained the files that are .gem files,
    # we can proceed them and check for an updated. This taps into
    # a specialized class that will do so for us.
    # ======================================================================= #
    array_ruby_gems.each {|this_ruby_gem, remote_url|
      _.check_this_remote_url(remote_url)
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForNewReleaseOnRubygems.new(ARGV)
end # check_for_new_release_on_rubygems https://rubygems.org/downloads/actionmailer-5.1.4.gem