#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Commandline
# =========================================================================== #
# require 'rbt/commandline/menu.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/archive_type.rb'
require 'rbt/toplevel_methods/download.rb'
require 'rbt/toplevel_methods/handle_this_extract_related_user_input.rb'
require 'rbt/toplevel_methods/return_base_directory_of_this_program.rb'

module RBT

class Commandline < RBT::Base # === RBT::Commandline

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      full_line = i.join(' ').strip # Get the full line here.
      # ===================================================================== #
      # We parse a few entries differently, namely --download and --extract.
      # Note that the following clause checks at the least for two different
      # conditions.
      # ===================================================================== #
      if (i.size > 1 and i.any? {|line| line.start_with?('--download') }) or
                         i.any? {|line| line.start_with?('--download') }
        # =================================================================== #
        # === rbt --download htop
        # or
        # === rbt --download=htop
        # or
        # === rbt --download-into-proper-directory htop
        # or
        # === rbt --download-proper htop
        # =================================================================== #
        if full_line.include?('-into') or full_line.include?('-download')
          case full_line
          when /^-?-?download(-|_)?into(-|_)?proper(-|_)?directory (.+)/, # $4
               # ============================================================ #
               # === rbt --download-into-proper-dir htop
               # ============================================================ #
               /^-?-?download(-|_)?into(-|_)?proper(-|_)?dir (.+)/, # $4
               # ============================================================ #
               # === rbt --download-into-dir htop
               # ============================================================ #
               /^-?-?download(-|_)?into(-|_)?dir (.+)/, # $3
               # ============================================================ #
               # === rbt --download-proper htop
               # ============================================================ #
               /^-?-?download(-|_)?proper (.+)/ # $2
            match = $2.to_s.dup
            match = $3.to_s.dup if $3
            match = $4.to_s.dup if $4
            target_dir = RBT.return_base_directory_of_this_program(match)
            unless File.directory? target_dir
              opnn; mkdir(target_dir) { :be_verbose }
            end
            opnn; cd(target_dir) { :be_verbose } 
            RBT.download(File.basename(target_dir).to_sym)
            opnn; e "Finished downloading into the local "\
                    "directory `#{sdir(target_dir)}`."
          # ================================================================= #
          # === rbt --download=m4
          # ================================================================= #
          when /^-?-?download=(.+)/ # $1
            RBT.download($1.to_s.dup.to_sym)
          end
        else
          # ================================================================= #
          # === rbt --download glibc-2.28
          # ================================================================= #
          full_line =~ /^-?-?download(=| )?(.+)/ # $2
          _ = $2.to_s.dup
          # ================================================================= #
          # If the input contains a '-' and at the least one number, then
          # we will assume that a specific program version is to be used.
          # The next code block deals with this situation.
          # ================================================================= #
          if _.include?('-') and (_ =~ /\d+/)
            data = ProgramInformation.new(_)
            remote_url = RBT.return_url1_of_this_program(data.program_name?)
            RBT.download(remote_url, data.program_version?)
          else
            RBT.download(_.to_sym) # In this case we will pass in a Symbol.
          end
        end
        set_user_input_was_handled_already
      # ===================================================================== #
      # === --extract
      #
      # We have to exclude --extract-to here since this will be handled by
      # class RBT::Compile instead, through the menu() method.
      # ===================================================================== #
      elsif i.size > 1 and i.any? {|line| line.start_with?('--extract') and !line.include?('-to') }
        # =================================================================== #
        # === rbt --extract htop
        # =================================================================== #
        full_line =~ /--extract (.+)/
        user_input = $1.to_s.dup
        RBT.handle_this_extract_related_user_input(user_input)
        set_user_input_was_handled_already
      elsif i.size > 1 and i.any? {|line| line.start_with?('--set') }
        # =================================================================== #
        # Using the general setter-clause.
        # =================================================================== #
        # === rbt --set-source-dir /Files/Archive/
        # =================================================================== #
        if full_line =~ /^-?-?set(-|_)?source(-|_)?dir (.+)/
          opnn; RBT.permanently_set_source_directory($3.to_s.dup) { :be_verbose }
          set_user_input_was_handled_already
        end
      else
        # =================================================================== #
        # else do recursive processing past this point
        # =================================================================== #
        i.each {|entry| menu(entry) }
      end
    else
      case i # case tag
      # ===================================================================== #
      # === rbt --version
      # ===================================================================== #
      when *ARRAY_HELP_VERSION_OPTIONS
        require 'rbt/toplevel_methods/report_rbt_version.rb'
        RBT.report_rbt_version
        exit
      # ===================================================================== #
      # === rbt --colour-make
      # ===================================================================== #
      when /^-?-?colour(_|-)?make$/i
        require 'rbt/make/make.rb'
        RBT::Make.new
        set_user_input_was_handled_already
      # ===================================================================== #
      # === rbt --colour-make-install
      # ===================================================================== #
      when /^-?-?colour(_|-)?make(_|-)?install$/i
        require 'rbt/make/colour_make_install.rb'
        RBT::ColourMakeInstall.new
        set_user_input_was_handled_already
      # ===================================================================== #
      # === rbt --be_verbose
      # ===================================================================== #
      when /^-?-?be(_|-)?verbose$/
        pp RBT.configuration?.be_verbose
      # ===================================================================== #
      # === rbt programs-dir?
      # ===================================================================== #
      when /^programs(_|-)?dir\??$/i,
           /^programs(_|-)?directory\??$/i
        e RBT.programs_directory?
        set_user_input_was_handled_already
      # ===================================================================== #
      # === rbt is_the_gobolinux_configuration_enabled?
      # ===================================================================== #
      when /^-?-?is(_|-)?the(_|-)?gobolinux(_|-)?configuration(_|-)?enabled\??$/i
        e 'Do we use the GoboLinux configuration: '+
          RBT.configuration?.enable_gobolinux_config.to_s
        set_user_input_was_handled_already
      # ===================================================================== #
      # === Feedback all available aliases
      #
      # Invocation example:
      #   rbt --available_aliases?
      # ===================================================================== #
      when /^-?-?available(_|-)?aliases\??$/i
        pp RBT.hash_aliases_to_the_available_programs?
        set_user_input_was_handled_already
      else
        if File.exist? i
          # ================================================================= #
          # Then we can compile it at once, as it exists as-is.
          # ================================================================= #
          do_compile_these_programs(i)
        end
      end
    end
  end

end; end