#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileBase
#
# This class has been created so that we can use a unified subclass
# for class Compile and class PostinstallationStep, but also for
# other classes within RBT that may require a more specialized
# Base class.
#
# Usage example:
#   RBT::CompileBase.new
# =========================================================================== #
# require 'rbt/compile/base/compile_base.rb'
# RBT::CompileBase.new
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/fetch_all_m4_macros.rb'
require 'rbt/utility_scripts/symlink_headers.rb'
require 'rbt/utility_scripts/cleanup_system_tags.rb'
require 'rbt/toplevel_methods/error_message.rb'
require 'rbt/symlink_this_program/initialize.rb'
begin
  require 'program_information'
rescue LoadError; end

module RBT

class CompileBase < RBT::Base # === RBT::CompileBase.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # The following instance-variable can be used to determine whether we
    # will silently redirect the output-stream or whether we will not.
    # ======================================================================= #
    @silent_redirection = false
  end

  # ========================================================================= #
  # === silent_redirection?
  # ========================================================================= #
  def silent_redirection?
    @silent_redirection
  end; alias original_stdout  silent_redirection? # === original_stdout
       alias original_stdout? silent_redirection? # === original_stdout?

  # ========================================================================= #
  # === play_song
  #
  # Play a song when the compilation phase has finished. This can be disabled
  # via the configuration file, in the entry called play_song. Please call
  # CONFIG.play_song before calling this method. We will play the audio file
  # only if it exists, otherwise we will be silent.
  # ========================================================================= #
  def do_play_song(
      song_location = '/Depot/Audio/YoonDohyun_Dolgo.mp3'
    )
    _ = "mplayer #{song_location} -endpos 4"
    esystem(_) if File.exist? song_location
  end; alias play_song do_play_song # === play_song

  # ========================================================================= #
  # === remove_compiling_is_in_progress
  #
  # We must get rid of special files. Right now only of a file called
  # "COMPILING_IS_IN_PROGRESS" - this file will be created when we
  # start compiling.
  # ========================================================================= #
  def remove_compiling_is_in_progress
    _ = 'COMPILING_IS_IN_PROGRESS'
    remove_file(_) if File.exist? _
  end

  # ========================================================================= #
  # === symlink_into_tags
  #
  # Use this method if you want to symlink into /System/Tags/*
  #
  # The first argument to this method is the name of the program
  # at hand.
  #
  # The second argument to this method is target_directory, which may
  # either be an AppDir, or another (non-AppDir) prefix.
  #
  # If there are no entries under the bin/ subdirectory, or not registered
  # as binary for that program, then this method will not symlink anything
  # at all.
  # ========================================================================= #
  def symlink_into_tags(
      compile_this_program = compile_which_program?,
      target_directory     = real_prefix?.to_s,
      dataset_to_use       = cookbook_dataset?,
      use_tags             = use_tags?
    )
    # ======================================================================= #
    # Handle tags only when we have specified so via the configuration
    # file "use_tags.yml". The following check ensures exactly that.
    # ======================================================================= #
    if use_tags
      # ===================================================================== #
      # So the next job is to obtain the tags.
      # ===================================================================== #
      tags = [dataset_to_use.tags?].flatten.compact
      if tags and !tags.empty? # Ok, we found tags here - and they are not empty.
        tags.each {|this_tag|
          # case this_tag # A bit sanitizing.
          # when 'game' # From game to games.
          #   this_tag = 'games'
          # end # <- No longer allowed since as of Feb 2017. Will be
          # removed past the middle of 2019 or so; perhaps a bit later
          # than that, but eventually it will be gone.
          # ================================================================= #
          # If you wish to CamelCase directory names, then the
          # following line will achieve precisely this. We will
          # CamelCase these tags.
          # ================================================================= #
          if this_tag.include? '_'
            this_tag = this_tag.split('_').map {|e| e.capitalize }.join
          end
          this_tag[0,1] = this_tag[0,1].upcase # Nicer to have capitalized directories.
          # ================================================================= #
          # We have to ensure that the relevant /System/Tags/* subdirectory does
          # actually exist. And if not, then we must create it.
          # ================================================================= #
          target_directory_for_the_system_tags = "#{system_tags?}#{this_tag}/"
          unless File.directory? target_directory_for_the_system_tags
            create_directory(target_directory_for_the_system_tags) # <- Create the subdirectory here.
          end
          # ================================================================= #
          # Next, obtain all binaries of the given program. This will include
          # both bin/ and sbin/ subdirectories. However had, the code here
          # will only work if we have compiled something as an AppDir. For
          # non-appdir programs we have to fetch the binaries that are
          # registered for that particular program.
          # ================================================================= #
          if uses_an_appdir_prefix?(target_directory)
            all_binaries = Dir["#{target_directory}bin/*"]
            if File.directory? "#{target_directory}sbin/"
              all_binaries << Dir["#{target_directory}sbin/*"] # <- Add sbin/ entries here.
              all_binaries.flatten!
            end
          else
            all_binaries = dataset_to_use.binaries?
            # =============================================================== #
            # And cd to the target directory, aka, the prefix.
            # =============================================================== #
            cd target_directory
          end
          unless all_binaries.empty?
            opnn; e "We will next symlink the various tags into the `"\
                    "#{sdir(target_directory_for_the_system_tags)}` hierarchy."
            all_binaries.each { |binary|
              # ============================================================= #
              # We must find the target next:
              # ============================================================= #
              _ = target_directory_for_the_system_tags+File.basename(binary)
              if File.exist? 'bin/'+binary
                binary = return_pwd+'bin/'+binary
              elsif File.exist? 'sbin/'+binary
                binary = return_pwd+'sbin/'+binary
              end
              # ============================================================= #
              # We have to check if the target already exists. The current
              # behaviour (Oct 2018) is to delete the old target, if it is
              # a symlink. This will also get rid of symlinks that are
              # invalid - e. g. no longer point at a correct target.
              # ============================================================= #
              if File.symlink?(_)
                if is_on_roebe?
                  opnn; e "Deleting the old symlink at `#{royalblue(_)}`."
                end
                delete_symlink(_) # <- Get rid of the old symlink finally.
              end
              symlink(
                from: binary,
                to:   _
              )
            }
          end
        }
      end
    end
  end

  # ========================================================================= #
  # === consider_removing_libtool_la_files
  #
  # To test this, try:
  #   ry apr ntrad
  # ========================================================================= #
  def consider_removing_libtool_la_files
    # ======================================================================= #
    # Must check for two conditions:
    #
    #   (a) the user specifically wants to delete libtool files, due
    #       to setting a configuration value
    #   (b) the program at hand uses an appdir-like prefix
    # ======================================================================= #
    if delete_libtool_files?
      require 'rbt/libtool/remove_libtool_files.rb'
      prefix_to_the_libtool_files = rds("#{prefix?.to_s}/lib/")
      RBT::Libtool::RemoveLibtoolFiles.new(
        prefix_to_the_libtool_files
      )
    end
  end

  # ========================================================================= #
  # === create_a_new_current_symlink_here
  # ========================================================================= #
  def create_a_new_current_symlink_here(i)
    target = File.dirname(i)+'/Current'
    if File.symlink? target
      remove_symlink(target)
    end
    opne "Creating a symlink at #{slateblue(i)}."
    symlink(i, target)
  end

  # ========================================================================= #
  # === consider_symlinking_lib64_into_lib
  #
  # This method will symlink lib64/ into lib/. Only invoke this method
  # if you have an AppDir versioned directory though.
  # ========================================================================= #
  def consider_symlinking_lib64_into_lib(
      shall_we_symlink = cookbook_dataset_autosymlink_lib64?
    )
    if shall_we_symlink
      old_dir = Dir.pwd # Keep a reference to the old directory.
      cd prefix?.to_s # cd into the AppDir prefix.
      opne 'Next trying to symlink '+sdir('lib64/')+' into '+
           sdir('lib/')+' from the directory '+sdir(return_pwd)+':'
      symlink('lib64','lib')
      # ===================================================================== #
      # Next check whether the operation was a success, so that we can
      # inform the user about this.
      # ===================================================================== #
      if File.symlink?('lib') and
         File.directory?('lib')
        opne 'Success!'
      else
        opne swarn('Failed to symlink `')+sdir('lib64/')+
             swarn('` towards `')+sdir('lib/')+swarn('`.')
      end
      cd old_dir # And go back again.
    end
  end

  # ========================================================================= #
  # === cookbook_dataset_autosymlink_lib64?
  # ========================================================================= #
  def cookbook_dataset_autosymlink_lib64?
    cookbook_dataset?.autosymlink_lib64?
  end

  # ========================================================================= #
  # === symlink_m4_macros
  #
  # The various .m4 macros usually reside within the share/ subdirectory.
  #
  # So in order to symlink them, we need to enter that directory.
  #
  # This method is called from do_postinstall_actions().
  # ========================================================================= #
  def symlink_m4_macros(
      i, do_run_fetch_all_m4_macros = do_run_fetch_all_m4_macros?
    )
    if do_run_fetch_all_m4_macros
      # ===================================================================== #
      # Delegate the job to class RBT::FetchAllM4Macros
      # ===================================================================== #
      RBT::FetchAllM4Macros.new(i) { :do_not_report_anything_if_there_were_no_matches }
    end
  end

  # ========================================================================= #
  # === symlink_program?
  # ========================================================================= #
  def symlink_program?
    @internal_hash[:symlink_program]
  end

  # ========================================================================= #
  # === use_gobolinux_symlinkprogram?
  # ========================================================================= #
  def use_gobolinux_symlinkprogram?
    @internal_hash[:use_gobolinux_symlinkprogram]
  end

  # ========================================================================= #
  # === create_architecture_file
  #
  # This method here will create a file called "Architecture".
  #
  # The first argument, `create_where`, should be the AppDir path to
  # the program in question, such as /Programs/Htop/Current.
  # ========================================================================= #
  def create_architecture_file(
      create_where                        = prefix?.to_s,
      can_we_create_the_architecture_file = is_on_gobolinux?
    )
    if can_we_create_the_architecture_file
      this_architecture = `uname -m`
      resources_directory = rds("#{create_where}/Resources/")
      # ===================================================================== #
      # Point to the new target that will hold the file Architecture:
      # ===================================================================== #
      file_location = rds(resources_directory+'Architecture')
      mkdir(resources_directory) unless File.directory? resources_directory
      # ===================================================================== #
      # Remove the old file it if already exists.
      # ===================================================================== #
      remove_file(file_location) if File.exist? file_location
      write_what_into(this_architecture, file_location)
    end
  end

  # ========================================================================= #
  # === install_only_binaries?
  # ========================================================================= #
  def install_only_binaries?
    @internal_hash[:install_only_binaries]
  end

  # ========================================================================= #
  # === shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
  # ========================================================================= #
  def shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
    @internal_hash[:shall_we_symlink_the_header_files_into_the_usr_include_hierarchy]
  end

  # ========================================================================= #
  # === check_whether_we_will_install_only_programs_from_the_bin_subdirectory
  # ========================================================================= #
  def check_whether_we_will_install_only_programs_from_the_bin_subdirectory
    if install_only_binaries?
      opne 'We will only retain the '+sdir('bin/')+' subdirectory.'
      delete_these_directories = Dir[return_proper_prefix?.to_s+'*'].select {|entry|
        File.directory? entry
      }
      delete_these_directories.reject! {|entry| entry.end_with? 'bin' }
      unless delete_these_directories.empty?
        opne 'We will now delete several directories that we did find.'
        remove(delete_these_directories)
      end
    end
  end

  # ========================================================================= #
  # === consider_symlinking_header_files_into_the_usr_include_hierarchy
  #
  # This method can be called if you wish to symlink any .h files or
  # directories within the AppDir at hand, into the /usr/include/
  # hierarchy.
  # ========================================================================= #
  def consider_symlinking_header_files_into_the_usr_include_hierarchy(
      i                                                                = compile_which_program?,
      shall_we_symlink_the_header_files_into_the_usr_include_hierarchy =
        shall_we_symlink_the_header_files_into_the_usr_include_hierarchy?
    )
    if shall_we_symlink_the_header_files_into_the_usr_include_hierarchy
      # ===================================================================== #
      # Delegate into class RBT::SymlinkHeaders next.
      # ===================================================================== #
      opne "Next attempting to symlink all .h files into the "\
           "#{sdir('/usr/include/')} hierarchy."
      opne "In order for this to work, #{royalblue('.h ')}"\
           " files have to exist in the AppDir directory."
      RBT::SymlinkHeaders.new(i) { :do_not_notify_if_there_were_no_header_files }
    end
  end

  # ========================================================================= #
  # === cleanup_system_tag_directory
  # ========================================================================= #
  def cleanup_system_tag_directory
    # ======================================================================= #
    # Be quiet - and only report when a stray symlink is removed.
    # ======================================================================= #
    RBT::CleanupSystemTags.new { :be_quiet }
  end

  # ========================================================================= #
  # === symlink_into_usr_lib_dir?
  # ========================================================================= #
  def symlink_into_usr_lib_dir?
    @internal_hash[:symlink_into_usr_lib_dir]
  end

  # ========================================================================= #
  # === consider_symlinking_cmake_modules
  #
  # Prior to entering this method, there is already a check whether we
  # are using an AppDir-like prefix. Only if this is the case, will
  # this method here be entered at all.
  #
  # This method will normally be called from the method
  # consider_performing_postinstall_actions().
  # ========================================================================= #
  def consider_symlinking_cmake_modules
    # ======================================================================= #
    # We must first check whether there are any cmake modules in the
    # first place.
    # ======================================================================= #
    target_cmake_dir = "#{prefix?}lib64/cmake/"
    if File.directory? target_cmake_dir
      # ===================================================================== #
      # But we must also check whether a cmake directory exists at a
      # /usr hierarchy.
      # ===================================================================== #
      if File.directory? '/usr/lib/cmake/'
        opne "Symlinking from #{sdir(target_cmake_dir)} into "\
             "#{sdir('/usr/lib/cmake/')}"
        # =================================================================== #
        # Symlink only directories.
        # =================================================================== #
        all_dirs = Dir["#{target_cmake_dir}*"].select {|entry| File.directory? entry }
        all_dirs.each {|this_directory|
          basename = File.basename(this_directory)
          target_in_usr_lib_cmake = "/usr/lib/cmake/#{basename}"
          # ================================================================= #
          # Only on roebe-like systems will we also remove the target
          # directory.
          # ================================================================= #
          if is_on_roebe?
            if File.exist?(target_in_usr_lib_cmake) and
               File.directory?(target_in_usr_lib_cmake) and
              !File.symlink?(target_in_usr_lib_cmake)
              remove_directory(target_in_usr_lib_cmake)
            end
          end
          symlink(this_directory, target_in_usr_lib_cmake)
        }
      end
    end
  end

  # ========================================================================= #
  # === cookbook_dataset_program_version?
  #
  # Obtain the version string of the program at hand through this
  # method. This method is quite reliable and can ideally be used
  # whenever we need to determine the version of a program.
  # ========================================================================= #
  def cookbook_dataset_program_version?
    if cookbook_dataset?
      return cookbook_dataset?.program_version?
    end
    nil
  end

  # ========================================================================= #
  # === register_program
  #
  # This method will register the program files into a yaml "database".
  #
  # The second argument can be used to determine whether we will really
  # register the program at hand.
  # ========================================================================= #
  def register_program(
      this_program                = compile_which_program?,
      can_we_register_the_program = RBT.register_program_files_into_yaml_database?
    )
    if can_we_register_the_program
      require 'rbt/utility_scripts/register_program_files_into_yaml_database.rb'
      appdir_prefix_of_this_program = static_appdir_prefix_of?(this_program)
      # ===================================================================== #
      # Next we will delegate towards class
      # RBT::RegisterProgramFilesIntoYamlDatabase, but this can be
      # controlled via the above flag.
      # ===================================================================== #
      RBT::RegisterProgramFilesIntoYamlDatabase.new(
        appdir_prefix_of_this_program
      ) # bl RegisterProgramFilesIntoYamlDatabase
    end
  end

  # ========================================================================= #
  # === do_invoke_symlink_program
  #
  # This will either use the RBT-internal class called RBT::SymlinkProgram;
  # or it will, on GoboLinux, make use of SymlinkProgram instead.
  # ========================================================================= #
  def do_invoke_symlink_program(
      i                        = compile_which_program?,
      use_this_program_version = program_version?
    )
    if use_gobolinux_symlinkprogram?
      # ================================================================= #
      # === Delegate onto GoboLinux next
      #
      # In this case, use the GoboLinux program called 'SymlinkProgram.'
      #
      # This program then requires two arguments:
      #
      #   (1) the program name, and
      #   (2) the program_version
      #
      # ================================================================= #
      # We should also check via 'NamingConventions'.
      # ================================================================= #
      _ = "SymlinkProgram #{compile_which_program?} #{use_this_program_version}"
      esystem _
    else
      symlink_program?.run
    end
  end

  # ========================================================================= #
  # === uses_appdir_prefix?
  #
  # This method will return true if the program at hand is compiled in
  # an AppDir-like fashion (e. g. having a prefix including the String
  # "/Programs/" directory), and false otherwise.
  # ========================================================================= # 
  def uses_appdir_prefix?(
      use_this_as_prefix           = prefix?.to_s,
      use_this_as_cookbook_dataset = cookbook_dataset?
    )
    if use_this_as_cookbook_dataset.nil?
      ensure_dataset_for_this_program(compile_which_program?)
    end
    use_this_as_prefix.to_s.include? programs_dir?
  end; alias is_installed_in_an_appdir_like_fashion? uses_appdir_prefix? # === is_installed_in_an_appdir_like_fashion?
       alias is_an_appdir_program?                   uses_appdir_prefix? # === is_an_appdir_program?
       alias is_appdir_compilation_style?            uses_appdir_prefix? # === is_appdir_compilation_style?
       alias is_an_appdir_installation?              uses_appdir_prefix? # === is_an_appdir_installation?
       alias uses_an_appdir_prefix?                  uses_appdir_prefix? # === uses_an_appdir_prefix?
       alias use_appdir_prefix?                      uses_appdir_prefix? # === use_appdir_prefix?
       alias appdir_prefix?                          uses_appdir_prefix? # === use_appdir_prefix?
       alias uses_appdir_approach?                   uses_appdir_prefix? # === uses_appdir_approach?

  # ========================================================================= #
  # === has_an_error_occurred?
  # ========================================================================= #
  def has_an_error_occurred?
    RBT.has_an_error_occurred?
  end

  # ========================================================================= #
  # === no_error_has_occurred?
  # ========================================================================= #
  def no_error_has_occurred?
    !has_an_error_occurred?
  end

  # ========================================================================= #
  # === store_full_configure_command_into_resources_directory
  #
  # We will store the full configure command into:
  #
  #   /Programs/Name/Version/Resources/full_configure_command
  #
  # ========================================================================= #
  def store_full_configure_command_into_resources_directory
    what = full_configure_command?
    unless what.strip.empty?
      # ===================================================================== #
      # Designate the file into where to store this:
      # ===================================================================== #
      into = real_prefix?.to_s.dup
      into << '/Resources/full_configure_command.md'
      unless File.directory? File.dirname(into)
        mkdir(File.dirname(into))
      end
      # ===================================================================== #
      # Remove old files at that position first.
      # ===================================================================== #
      delete_file(into) if File.exist? into
      append_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileBase.new(ARGV)
end # compile_base