#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Everything pertaining to a build directory will be stored here in
# this file. Since as of July 2018, class RBT::BuildDirectory is
# used for most of the logic behind a build directory.
# =========================================================================== #
# require 'rbt/compile/build_directory.rb'
# =========================================================================== #
require 'rbt/utility_scripts/build_directory.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === cd_to_the_build_directory
  # ========================================================================= #
  def cd_to_the_build_directory
    cd @internal_hash[:build_directory].path?
  end

  # ========================================================================= #
  # === consider_handling_build_directory
  #
  # This method can create a build directory, that is, a directory which
  # will keep other generated files and directories inside of it.
  # ========================================================================= #
  def consider_handling_build_directory
    if use_build_directory?
      # ===================================================================== #
      # First determine the path we ought to use for the build directory.
      #
      # This has to be a correct path since we depend on it for the
      # change_directory action lateron. Note that if the name for the
      # build directory begins with a '/' then it will be used as the
      # full build directory instead - in this case we assume that the
      # user knew what he wanted to do.
      # ===================================================================== #
      if name_for_the_build_directory?.start_with? '/'
        use_this_as_path_to_the_build_directory =
          name_for_the_build_directory?.dup
      else
        use_this_as_path_to_the_build_directory = 
          @internal_hash[:configure_base_directory]+
          name_for_the_build_directory?
      end
      # ===================================================================== #
      # Delegate towards class BuildDirectory next:
      # ===================================================================== #
      build_directory = BuildDirectory.new(
        path_to_build_directory: use_this_as_path_to_the_build_directory.dup
      )
      # ===================================================================== #
      # Next assign it onto the @internal_hash variable:
      # ===================================================================== #
      @internal_hash[:build_directory] = build_directory
      build_directory.silently_make_sure_that_the_build_directory_exists
      opnn; build_directory.cd :be_verbose # cd into that directory.
    end
  end; alias consider_creating_the_build_directory consider_handling_build_directory # === consider_creating_the_build_directory

  # ========================================================================= #
  # === DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY
  #
  # What name will our default build directory have? Useful when we
  # wish to use other build dirs.
  #
  # The reason the name is so long is because the shorter name "BUILD"
  # sometimes may already be an existing file. The program "brotli"
  # has had this problem - it carried a file called "BUILD" by default,
  # which obviously means that a "cd BUILD/" event will fail lateron.
  # ========================================================================= #
  DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY = 'BUILD_DIRECTORY/' # Upcase it since as of Mar 2011.

  # ========================================================================= #
  # === set_name_for_the_build_directory
  # ========================================================================= #
  def set_name_for_the_build_directory(
      i = DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY,
      be_verbose = false
    )
    case be_verbose
    when :report_back
      be_verbose = true
    end
    if be_verbose
     opne "The name of the build directory will be `#{royalblue(i)}`."
    end
    @internal_hash[:name_for_the_build_directory] = i
  end; alias use_this_build_directory=    set_name_for_the_build_directory # === use_this_build_directory=
       alias use_this_build_directory     set_name_for_the_build_directory # === use_this_build_directory
       alias set_use_this_build_directory set_name_for_the_build_directory # === set_use_this_build_directory

  # ========================================================================= #
  # === do_use_build_directory
  #
  # Use this method to specifcally "tell" a program to do use a build
  # directory. For cmake-based projects it is recommended to use a
  # build directory.
  #
  # Usage example:
  #   ry htop build
  # ========================================================================= #
  def do_use_build_directory(
      be_verbose = true
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      opne 'A build directory will be used.'
    end
    @internal_hash[:use_build_directory] = true
  end; alias use_build_directory      do_use_build_directory # === use_build_directory
       alias use_a_build_directory    do_use_build_directory # === use_a_build_directory
       alias do_use_a_build_directory do_use_build_directory # === do_use_a_build_directory

  # ========================================================================= #
  # === use_build_directory?
  #
  # Query-method over as to whether we will use a build directory or not.
  #
  # This method will return a Boolean value (true or false).
  # ========================================================================= #
  def use_build_directory?
    _ = cookbooks_dataset?.use_build_directory
    if @internal_hash[:use_build_directory]
      _ = @internal_hash[:use_build_directory]
    end
    _
  end; alias use_a_build_directory?       use_build_directory? # === use_a_build_directory?
       alias use_build_dir?               use_build_directory? # === use_build_dir?
       alias do_we_use_a_build_directory? use_build_directory? # === do_we_use_a_build_directory?

  # ========================================================================= #
  # === do_use_random_build_directory
  #
  # This method can be used to use an ad-hoc random build directory.
  # ========================================================================= #
  def do_use_random_build_directory(
      be_verbose = true
    )
    use_this_as_the_new_name = ''.dup
    n_times = (8..16).to_a.sample
    n_times.times {
      use_this_as_the_new_name << ALPHABET.sample
    }
    if be_verbose
      opne "We will use a #{lightgreen('random directory')} as "\
           "build directory (#{sfancy(use_this_as_the_new_name)})."
    end
    do_use_build_directory :be_quiet
    set_name_for_the_build_directory(use_this_as_the_new_name)
  end

  # ========================================================================= #
  # === name_for_the_build_directory?
  # ========================================================================= #
  def name_for_the_build_directory?
    @internal_hash[:name_for_the_build_directory]
  end; alias name_of_the_build_directory? name_for_the_build_directory? # === name_of_the_build_directory?

  # ========================================================================= #
  # === use_this_build_directory?
  #
  # Ask the dataset for the proper build directory to use.
  #
  # This may not always be true though.
  # ========================================================================= #
  def use_this_build_directory?
    build_directory?.path?.to_s
  end

end; end