#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Compile
#
# This class can be used in order to compile or otherwise install a program.
#
# By default, the GNU "configure", "make" and "make install" steps are
# used.
#
# The first argument to this class should be the name of the program or
# the name of the programs that ought to be compiled. This will always
# be converted into an Array, internally, so that this class can operate
# in a batch-oriented matter.
#
# Specific usage examples from within ruby code:
#
#   htop = RBT::Compile.new 'htop'
#   php  = RBT::Compile.new 'php'
#
# Commandline invocation examples:
#
#   ry wxwid
#   ry PYTHON gobo # <-- input will be lowercased anyway
#   ry PYTHON trad
#   ry PYTHON --ntrad
#   ry bmp oldschool
#
# =========================================================================== #
# require 'rbt/compile/compile.rb'
# =========================================================================== #
require 'program_information'
require 'rbt/compile/base/compile_base.rb'
# =========================================================================== #
# Next include other .rb files that belong to class RBT::Compile. These
# .rb files will reside under the rbt/compile/ subdirectory.
# =========================================================================== #
require 'rbt/compile/autoconf.rb'
require 'rbt/compile/autogen.rb'
require 'rbt/compile/blfs.rb'
require 'rbt/compile/build_directory.rb'
require 'rbt/compile/ccache.rb'
require 'rbt/compile/cflags.rb'
require 'rbt/compile/clang.rb'
require 'rbt/compile/cmake.rb'
require 'rbt/compile/colours.rb'
require 'rbt/compile/commandline.rb'
require 'rbt/compile/configure.rb'
require 'rbt/compile/constants.rb'
require 'rbt/compile/cookbook_dataset.rb'
require 'rbt/compile/create_program.rb'
require 'rbt/compile/debug.rb'
require 'rbt/compile/do_compile_this_program.rb'
require 'rbt/compile/ensure.rb'
require 'rbt/compile/env.rb'
require 'rbt/compile/exit.rb'
require 'rbt/compile/extract.rb'
require 'rbt/compile/feedback.rb'
require 'rbt/compile/ftp.rb'
require 'rbt/compile/games.rb'
require 'rbt/compile/gnome.rb'
require 'rbt/compile/gobolinux.rb'
require 'rbt/compile/headers.rb'
require 'rbt/compile/help.rb'
require 'rbt/compile/initialize.rb'
require 'rbt/compile/infobox.rb'
require 'rbt/compile/kde.rb'
require 'rbt/compile/libtool.rb'
require 'rbt/compile/ldconfig.rb'
require 'rbt/compile/lxqt.rb'
require 'rbt/compile/make.rb'
require 'rbt/compile/make_install.rb'
require 'rbt/compile/mate_desktop.rb'
require 'rbt/compile/menu.rb'
require 'rbt/compile/meson_and_ninja.rb'
require 'rbt/compile/misc.rb'
require 'rbt/compile/obtain.rb'
require 'rbt/compile/opn.rb'
require 'rbt/compile/path.rb'
require 'rbt/compile/permanent.rb'
require 'rbt/compile/porg.rb'
require 'rbt/compile/postinstallation_step.rb'
require 'rbt/compile/prefix.rb'
require 'rbt/compile/preinstall.rb'
require 'rbt/compile/problems_and_errors.rb'
require 'rbt/compile/python.rb'
require 'rbt/compile/register.rb'
require 'rbt/compile/reset.rb'
require 'rbt/compile/ruby.rb'
require 'rbt/compile/run.rb'
require 'rbt/compile/sed.rb'
require 'rbt/compile/shell.rb'
require 'rbt/compile/show_and_report.rb'
require 'rbt/compile/sigint.rb'
require 'rbt/compile/static.rb'
require 'rbt/compile/strip.rb'
require 'rbt/compile/system.rb'
require 'rbt/compile/symlink.rb'
require 'rbt/compile/tab.rb'
require 'rbt/compile/time.rb'
require 'rbt/compile/upload_and_download.rb'
require 'rbt/compile/warnings.rb'
require 'rbt/compile/xfce.rb'
require 'rbt/compile/xorg.rb'
require 'rbt/compile/yaml.rb'
require 'rbt/compile/linux/gentoo.rb'
# =========================================================================== #
# Require the files under rbt/make/ next:
# =========================================================================== #
require 'rbt/make/make.rb'
require 'rbt/make/colour_make_install.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === show_which_files_will_be_installed
  #
  # This method will show to the user which files will be installed.
  #
  # Invocation examples:
  #
  #   ry htop ifiles
  #   ry htop --installed-files
  #   ry ruby --installed-files
  #
  # ========================================================================= #
  def show_which_files_will_be_installed(
      i = compile_which_program?
    )
    file = static_appdir_prefix?+'yaml/registered_files.yml'
    if File.exist? file
      opne 'The following files will be installed by '\
           'the program `'+simp(i)+'`:'
      dataset = YAML.load_file(file).sort
      e; dataset.each {|entry| e '  '+entry }; e
    else
      opne 'The file '+sfile('registered_files.yml')+
           'could not be found,'
      opne ' thus we can not show which files are installed.'
    end
  end

  # ========================================================================= #
  # === set_on_ihash
  #
  # The second argument is the dataset that you wish to use.
  # ========================================================================= #
  def set_on_ihash(
      which_key_to_use = :yaml_dataset,
      which_dataset_to_use
    )
    @internal_hash[which_key_to_use] = which_dataset_to_use
  end

  # ========================================================================= #
  # === ihash
  # ========================================================================= #
  def ihash?
    @internal_hash
  end

  # ========================================================================= #
  # === is_superuser?
  # ========================================================================= #
  def is_superuser?
    Process.uid == 0
  end

  # ========================================================================= #
  # === do_install_only_binaries
  # ========================================================================= #
  def do_install_only_binaries
    @internal_hash[:install_only_binaries] = true
  end

  # ========================================================================= #
  # === do_compile_all_of_mate
  # ========================================================================= #
  def do_compile_all_of_mate
    i = RBT.returned_chained_programs_of_the_mate_desktop
    opne "We will compile all #{sfancy(i.size.to_s)} programs "\
         "of the mate-desktop."
    set_compile_these_programs(i)
  end

  # ========================================================================= #
  # === array_sys_command_string?
  #
  # The array is fed in line by line.
  # ========================================================================= #
  def array_sys_command_string?
    @internal_hash[:array_sys_command_string]
  end; alias sys_command_string? array_sys_command_string? # === sys_command_string?

  # ========================================================================= #
  # === compile_this_program
  #
  # Query which program we are about to (want to) compile next.
  # ========================================================================= #
  def compile_this_program?
    @internal_hash[:compile_this_program]
  end; alias compile_which_program? compile_this_program? # === compile_which_program?
       alias which_program?         compile_this_program? # === which_program?
       alias program?               compile_this_program? # === program?
       alias internal_program_name? compile_this_program? # === internal_program_name?
       alias compile_this_program?  compile_this_program? # === compile_this_program?
       alias this_program?          compile_this_program? # === this_program?
       alias first_arg              compile_this_program? # === first_arg
       alias name_of_program?       compile_this_program? # === name_of_program?
       alias real_program_name?     compile_this_program? # === real_program_name?
       alias program_name?          compile_this_program? # === program_name?
       alias name?                  compile_this_program? # === name?

  # ========================================================================= #
  # === set_compile_this_program
  #
  # This method has to be used to register the name of the current
  # program that is to be compiled or installed.
  # ========================================================================= #
  def set_compile_this_program(i)
    i = i.to_s.dup.downcase.delete('/_-') # <- Get rid of '/', '_' and other characters.
    i.chop! if i.end_with? ':','#' # Get rid of trailing ':' and '#'.
    i = return_possible_abbreviation_to_this_input(i)
    # ======================================================================= #
    # Keep track of the program we are compiling in the Hash called @internal_hash.
    # ======================================================================= #
    @internal_hash[:compile_this_program] = i # Must be a String.
  end; alias set_program_name set_compile_this_program # === set_program_name

  # ========================================================================= #
  # === assign_to_the_main_dataset
  #
  # This method should be used when we wish to assign to the main
  # dataset at hand. The expected input should be class
  # ::RBT::Cookbooks::Cookbook. If it is not then this will cause
  # an argument error.
  # ========================================================================= #
  def assign_to_the_main_dataset(i)
    unless i.is_a? ::RBT::Cookbooks::Cookbook
      raise ArgumentError,
            'The method assign_to_the_main_dataset() expects input as '\
            'class RBT::Cookbooks::Cookbook'
    end
    set_on_ihash(:yaml_dataset, i)
  end; alias use_this_as_cookbook  assign_to_the_main_dataset # === use_this_as_cookbook
       alias use_this_as_cookbook= assign_to_the_main_dataset # === use_this_as_cookbook=

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Compile.new(ARGV)
end # rbt_compile ccache
    # ry htop ntrad