#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Constants relevant or class RBT::Compile will be bundled together here.
# =========================================================================== #
# require 'rbt/compile/constants.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
require 'rbt/toplevel_methods/log_directory.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  #
  # Set the default program here.
  # ========================================================================= #
  DEFAULT_PROGRAM = 'htop'

  # ========================================================================= #
  # === ALSO_AUTOMATICALLY_INSTALL_THE_UPDATED_GEM
  #
  # If the following constant is set to true, then the RBT project will
  # also attempt to automatically install this given ruby-gem.
  #
  # That way you can easily batch-update all registered rubygems; they
  # must have a corresponding .yml file that is registered in the
  # respective .yml file at hand, though.
  # ========================================================================= #
  ALSO_AUTOMATICALLY_INSTALL_THE_UPDATED_GEM = true

  # ========================================================================= #
  # === SILENT_REDIRECTION
  #
  # This can simply be appended.
  # ========================================================================= #
  SILENT_REDIRECTION = ' > /dev/null'

  # ========================================================================= #
  # === THIS_FILE
  # ========================================================================= #
  THIS_FILE = "#{RBT.project_dir?}compile/compile.rb"

  # ========================================================================= #
  # === ARRAY_HELP_REMOVE_ALL_SYMLINKS_OPTIONS
  #
  # This Array collects all options used for when we wish to remove
  # all symlinks in the /Programs hierarchy.
  # ========================================================================= #
  ARRAY_HELP_REMOVE_ALL_SYMLINKS_OPTIONS = %w(
    --remove-all-symlinks
    --removeallsymlinks
    clear_symlinks
    rsymlinks
    remove_all_symlinks
    removeallsymlinks
    CLEAR_SYMLINKS
    REMOVE_ALL_SYMLINKS
  )

  # ========================================================================= #
  # === ARRAY_HELP_GENERATE_SHELL_COMPLETION_OPTIONS
  #
  # There are the options for shell-completion.
  # ========================================================================= #
  ARRAY_HELP_GENERATE_SHELL_COMPLETION_OPTIONS = %w(
    --generate-shell-completions
    --generate-shell-completion
    --generate--shellcompletion
    --generate-shell
      generate-shell
    --generate-tab-completion
    --generate-tab
    --generatecompletion
    --generate-completions
    --generate-completion
    --generate_shell_completion
    --generateshellcompletion
    --generate
    --shellcompletion
    --gsc
    --tab-completion
    --shell-completion
    --completion
    generate-tab-completion
    generatecompletion
    bash_completion
    bashcompletion
    autocompletion
    autogenerate_completion
    generateshellcompletion
    generateshellcompletion
    generate_shell_completion
    generate_shell_scripts
    generate_all_completions
    generate_tab_completion
    gshellcompletion
    generate_completions
    generate_completion
    generate-tab
    generate_tab
    generatebash
    generatetabcompletion
    generatecompletion
    generatetab
    generate
    gencompletion
    gsc
    tabcomplete
    tabcompletion
    tab_completion
    tab
    TAB
    shellcomp
    shellcompletion
    shell_completion
    update_shell_scripts
    completion
    gtab
    GENERATE_COMPLETION
  )

  # ========================================================================= #
  # === ARRAY_CHECK_FOR_BINARY_DUPLICATES_OPTIONS
  #
  # This constant shall include regexes that check for the following
  # commandline options:
  #
  #     rbt --check-for-binary-duplicates
  #     rbt --report-binary-duplicates
  #     rbt --report-duplicates
  #     rbt --binary-duplicates
  #     rbt --duplicate-binaries
  #
  # ========================================================================= #
  ARRAY_CHECK_FOR_BINARY_DUPLICATES_OPTIONS = [
    /^-?-?check(-|_)?for(-|_)?binary(-|_)?duplicates$/,
    /^-?-?report(-|_)?binary(-|_)?duplicates$/,
    /^-?-?report(-|_)?duplicates\??$/,
    /^-?-?binary(-|_)?duplicates\??$/,
    /^-?-?duplicate(-|_)?binaries\??$/,
    /^-?-?duplicates\??$/
  ]

  # ========================================================================= #
  # === ARRAY_HELP_SIMULATION_MODE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_SIMULATION_MODE_OPTIONS = %w(
    --simulation
    --simulate
    --pretend
    --no-harm
    --noharm
    --noh
    --no-build
    --nobuild
    no-harm
    noharm
    noh
    pretend
    runsimulation
    run_simulation
    simulation
    simulate
    simu
    sim
  )

  # ========================================================================= #
  # === ARRAY_HELP_ENABLE_STATIC                                 (static tag)
  #
  # These are the options for static compilation mode.
  # ========================================================================= #
  ARRAY_HELP_ENABLE_STATIC = %w(
     static
     enable_static
     enable-static
     enablestatic
     make_static
     be_static
     statically
     build_static
     do_static
     --static
     --enable-static
     --enablestatic
   )

  # ========================================================================= #
  # === ARRAY_HELP_CREATE_SHELLSCRIPT_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_CREATE_SHELLSCRIPT_OPTIONS = %w(
    --generate_shellscript
    create_shellscript
    create_shell_script
    generate_shellscript
    shellscript
    to_shell
    toshell
    cshell
    to_shellscript
    shell
    generateshell
    shellshell
    generate_shell
    gscript
    gshell
    generate_only_one_shell_script
    one_shell
    SHELLSCRIPT
    generate_a_shellscript
  )

  # ========================================================================= #
  # === ARRAY_HELP_LAST_COMPILED_PROGRAM_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_LAST_COMPILED_PROGRAM_OPTIONS = %w(
    LAST
    last
    --last
  )

  # ========================================================================= #
  # === ARRAY_HELP_CHECK_ALL_TAGS_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_CHECK_ALL_TAGS_OPTIONS = %w(
    --setup-tags
    setup-tags
    setuptags
    stags
    check_all_tags_under_programs_directory
    all-tags
    --check-all-tags
    check-all-tags
  )

  # ========================================================================= #
  # === ARRAY_HELP_PURGE_ALL_MAN_ENTRIES
  # ========================================================================= #
  ARRAY_HELP_PURGE_ALL_MAN_ENTRIES = %w(
    purge_all_man_entries
    --remove_man_pages
    --no-man-pages
    --nomanpage
    remove_man_pages
    remove_pages
    no_man_page
    no_man_pages
    nomanpage
    purgeman
    no-man-pages
  )

  # ========================================================================= #
  # === ARRAY_HELP_GENERATE_SLACK_DESC_FILE
  #
  # Help entries to generate a slack-desc file.
  # ========================================================================= #
  ARRAY_HELP_GENERATE_SLACK_DESC_FILE = %w(
    --generate-slack-desc
    --desc-file
    --slack
    --generate-slack-desc-file
    --slack-desc
    --slack_desc
    -slack
    slack-desc-file
    slackdesc
    slackware
    slack
  )

  # ========================================================================= #
  # === ARRAY_HELP_DONT_KEEP_EXTRACTED_OPTIONS
  #
  # Allowed options for when we wish to keep the extracted archive.
  # ========================================================================= #
  ARRAY_HELP_DONT_KEEP_EXTRACTED_OPTIONS = %w(
    dontkeep do_not_keep set_dont_keep_archive
    dont_keep_archive dontkeeparchive remove 2
    remove_extracted
    removeextracted
    -s --remove-sources
    --remove-archive -a
  )

  # ========================================================================= #
  # === ARRAY_HELP_DISABLE_AUTOGEN
  # ========================================================================= #
  ARRAY_HELP_DISABLE_AUTOGEN = %w(
    --no-autogen
    --disable-autogen
    disable-autogen
    disableautogen
    dautogen
    no-use-autogen
    --do-not-use-autogen
    --donotuseautogen
    donotuseautogen
    dontrunautogen
    skip-autogen
  )

  # ========================================================================= #
  # === ARRAY_HELP_SHOW_LAST_UPDATE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_SHOW_LAST_UPDATE_OPTIONS = %w(
    --last_update?
    --last_update
    --lastupdate?
    --lastupdate
    last_update?
    last_update
    show_last_updated
    slu
    show_last_update
    showlastupdated
    lastupdate
  )

  # ========================================================================= #
  # === ARRAY_HELP_USE_AUTOGEN
  #
  # All options when we wish to use autogen.
  # ========================================================================= #
  ARRAY_HELP_USE_AUTOGEN = %w(
    --use-autogen
    --useautogen
    --useautogen
    useautogen
    AUTOGEN
    autogen
    enable_autogen
    enableautogen
    use-autogen
  )

  # ========================================================================= #
  # === ARRAY_HELP_SHOW_REGISTERED_KEYS
  # ========================================================================= #
  ARRAY_HELP_SHOW_REGISTERED_KEYS = %w(
    --show_registered_keys
    --allregistered?
    show_registered_keys
    register_all?
    registerall?
    rall?
    allregistered?
  )

  # ========================================================================= #
  # === ARRAY_HELP_FIND_ALL_ARCHIVES
  # ========================================================================= #
  ARRAY_HELP_FIND_ALL_ARCHIVES = %w(
    --archive-types
    --archives?
    --archives
  )

  # ========================================================================= #
  # === RBT::ARRAY_HELP_TEMP_DIR
  #
  # This constant will keep track of the RBT temp directory, e. g.
  # where we will extract archives to.
  # ========================================================================= #
  ARRAY_HELP_TEMP_DIR = %w(
    --tempdir?
    --temp-dir?
    --temp_dir?
    --temp_dir
    --extract_to?
    --basedir?
    basedir?
  )

  # ========================================================================= #
  # === ARRAY_HELP_PREFIX_IN_USE
  # ========================================================================= #
  ARRAY_HELP_PREFIX_IN_USE = %w(
    --prefix?
    prefix?
    prefix
    feedback_prefix_in_use
    --which-prefix?
    --whichprefix?
  )

  # ========================================================================= #
  # === ARRAY_HELP_DO_COMPILE_IN_A_NON_TRADITIONAL_WAY_OPTIONS    (ntrad tag)
  #
  # These are the options for when you wish to install in an app-dir like
  # manner.
  # ========================================================================= #
  ARRAY_HELP_DO_COMPILE_IN_A_NON_TRADITIONAL_WAY_OPTIONS = %w(
    non-traditional
    --app_dir
    --ntrad
    --ntad
    ntrad
    ntrd
    nrad
    notrad
    tradntrad
    overrule
    gobo
    gobstyle
    gstyle
    gobolinux
    glinux
    gobostyle
    non_traditional
    nontrad
    ntraditional
    ntrda
    ntra
    ntr
    nta
    ng
    nt
    tradn
    leicht
    tnrad
    ntrax
    mntrad
    nrtad
    ntad
    trag
    nnnn
  )

  # ========================================================================= #
  # === ARRAY_HELP_ALL_AVAILABLE_PROGRAMS
  # ========================================================================= #
  ARRAY_HELP_ALL_AVAILABLE_PROGRAMS = %w(
    --available_programs?
    --all_available_programs?
    all_available_programs?
    available_programs?
    available_programs
    availableprograms?
    availableprograms
    programs?
    all_programs?
    all_programs
    allprograms
    all?
    allprograms?
    p?
    a?
    allp?
  )

  # ========================================================================= #
  # === ARRAY_HELP_AVAILABLE_PROGRAM_VERSIONS
  # ========================================================================= #
  ARRAY_HELP_AVAILABLE_PROGRAM_VERSIONS = %w(
    --available-versions?
    --available_versions
    available_versions
    available_version
    availableversion
    available_versions?
    versions
    versions?
    --versions?
    v?
  )

  # ========================================================================= #
  # === ARRAY_HELP_VIEW_AVAILABLE_LICENSES_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_VIEW_AVAILABLE_LICENSES_OPTIONS = %w(
    --all_licenses
    --alllicenses
    --view-available-licenses
    --licenses?
    --licenses
    feedback_licenses
    license
    all_license
    l
    lisence
    licenses
    all_licenses
    flicenses
    feedbacklicenses
    licenses?
  )

  # ========================================================================= #
  # === ARRAY_HELP_FEEDBACK_LAST_COMPILED_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_FEEDBACK_LAST_COMPILED_OPTIONS = %w(
    --last-compiled?
    --last-5-compiled?
    --last-five-compiled?
    --last-compiled?=5
  )

  # ========================================================================= #
  # === ARRAY_HELP_INCLUDED_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_INCLUDED_OPTIONS = %w(
    --included?
    --included
    isincluded?
    isincluded
    is_included?
    included?
    included
    include?
  )

  # ========================================================================= #
  # === ARRAY_HELP_DO_USE_STOW_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_DO_USE_STOW_OPTIONS = %w(
    --use-stow?
    --stow?
    stow?
    stow
  )

  # ========================================================================= #
  # === ARRAY_HELP_TOGGLE_DEBUG_VALUE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_TOGGLE_DEBUG_VALUE_OPTIONS = %w(
    debug
    toggle_debug_value
    toggle
  )

  # ========================================================================= #
  # === ARRAY_HELP_FEEDBACK_EVERY_CASE_OPTION_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_FEEDBACK_EVERY_CASE_OPTION_OPTIONS = %w(
    --caseoptions
    --case_options?
    case_options?
    case_options
    caseoptions
    feedback_all_case_options_of_the_main_menu
    top
  )

  # ========================================================================= #
  # === OPTIONS_NO_COLOURS
  #
  # Synonyms for colour usage. All these instructions can be used to
  # disable colours.
  # ========================================================================= #
  OPTIONS_NO_COLOURS = %w(
    --disable-colours
    --disable_colours
    --nocolor
    --nocol
    --nocolour
    --nocolours
    --ncolours
    --nocolours
    --no-colours 
    nocolor
    nocol
    nocolour
    nocolours
    disablecolours
    disable_colours
    --dont-use-colours
    dont-use-colours
  )

  # ========================================================================= #
  # === ARRAY_HELP_FEEDBACK_PROGRAM_VERSION_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_FEEDBACK_PROGRAM_VERSION_OPTIONS = %w(
    --program-version?
    --program-version
    --program_version
    version
    versio
    versi
    vers
    ver
    version?
    ve
    v
    ver?
    VERSION
  )

  # ========================================================================= #
  # === ARRAY_HELP_OUTPUT_ALL_URLS_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_OUTPUT_ALL_URLS_OPTIONS = %w(
    --all-urls?
    --all-urls
    --all_urls
    --allurls?
    --all-URLs
    --show-all-URLs
    --output_all_urls
    --showallURLs
    --showallurls
    output_all_urls
    outputallurls
    programurls
    all_programs_with_urls
    all_urls?
    all_urls
    all_url
    allurls
    allurl
    aurls
    all-urls
    allprograms
  )

  # ========================================================================= #
  # === ARRAY_HELP_SHOW_HIGHEST_PROGRAMS_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_SHOW_HIGHEST_PROGRAMS_OPTIONS = %w(
    --highest
    highest
    high
    highest?
    --show_highest
    show_highest
    showhighest
    show_n_largest_local_programs
    size
  )

  # ========================================================================= #
  # === ARRAY_HELP_GENERATE_PDF_TUTORIAL_OPTIONS
  #
  # All options that can be used to generate the .pdf tutorial.
  # ========================================================================= #
  ARRAY_HELP_GENERATE_PDF_TUTORIAL_OPTIONS = %w(
    --pdfgeneration
    --pdf-generation
    --pdf
    --generate-pdf-tutorial
    --generate-pdftutorial
    --generatepdftutorial
    pdftutorial
    generate_pdf_tutorial
    generatepdftutorial
    generate
    pdf?
    pdf
  )

  # ========================================================================= #
  # === ARRAY_HELP_FEEDBACK_VERSIONS_OPTIONS
  #
  # This Array is used to feedback different versions in use.
  # ========================================================================= #
  ARRAY_HELP_FEEDBACK_VERSIONS_OPTIONS = %w(
    --versions-in-use?
    --feedback-version
    --feedbackversion
    feedback_version
    feedbackversion
    fversion
    no-opn
  )

  # ========================================================================= #
  # === ARRAY_HELP_KEEP_EXTRACTED_OPTIONS
  #
  # Note that an alias to this is --dont-remove-extracted-archive
  # ========================================================================= #
  ARRAY_HELP_KEEP_EXTRACTED_OPTIONS = %w(
    --keep_extracted
    --keep
    -k
    keep_extracted
    keepextracted
    kextracted
    ka
    kee
    ke
    kep
    ket
    --dont-remove-extracted-archive
    --dontremoveextractedarchive
    dontremoveextractedarchive
  )

  # ========================================================================= #
  # === ARRAY_HELP_GENERATE_HOMEPAGE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_GENERATE_HOMEPAGE_OPTIONS = %w(
    --generate-homepage
    --generatehomepage
    generate_homepage
    generatehomepage
    genhomepage
  )

  # ========================================================================= #
  # === ARRAY_HELP_SKIP_EXTRACTING_OPTIONS
  #
  # Options in order to skip extracting the tarball.
  # ========================================================================= #
  ARRAY_HELP_SKIP_EXTRACTING_OPTIONS = %w(
    --no-extract
    --noextract
    --skip-extracting
    --skipextract
    --dontextract
    -n
    noextract
    noext
    nextract
    set_skip_extracting
    skip_extracting
    skipextracting
    skipextract
    skipe
    sextract
    dontextract
    se
    sa
    ske
    kese
    sext
    speed
    skep
    sk
  )

  # ========================================================================= #
  # === ARRAY_HELP_DOWNLOAD_THE_SOURCE_OPTIONS
  #
  # All download options are stored here.
  # ========================================================================= #
  ARRAY_HELP_DOWNLOAD_THE_SOURCE_OPTIONS = %w(
    --download-source
    --downloadsource
    --download
    --dload
    download
    downloa
    downoad
    downlo
    downl
    down
    dow
    do
    runterladen
    get
    take
    --wget
    wge
    wg
    dload
    w
    nobuild
    no-build
    downloadonly
    download_only
    --download_only
    --downloadonly
  )

  # ========================================================================= #
  # === ARRAY_HELP_DO_NOT_SYMLINK_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_DO_NOT_SYMLINK_OPTIONS = %w(
    -S
    --symlink=no
    --no-symlink
    --no-sym
    --nosym
    nosym
    --dont-symlink
    --dontsymlink
    --nosymlink
    set_do_not_symlink
    --set_do_not_symlink
    --do-not-symlink
    donotsymlink
    dont_symlink
    dontsymlink
    dontsymlin
    dsymlink
    dodnotsymlink
    no_symlink
    dontsym
    nosymlink
    ds
  )

  # ========================================================================= #
  # === ARRAY_HELP_QUERY_THE_EXISTANCE_OF_A_BINARY_NAME
  # ========================================================================= #
  ARRAY_HELP_QUERY_THE_EXISTANCE_OF_A_BINARY_NAME = %w(
    --binary?
    binary?
    package?
    as_binary?
    as_binaryname
    as_binaryname?
  )

  # ========================================================================= #
  # === ARRAY_HELP_UPDATE_BINARY_NAMES_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_UPDATE_BINARY_NAMES_OPTIONS = %w(
    --update-registered-binaries
    --updateregisteredbinaries
    --update-binary-names
    updatebinarynames
    update_binary_names
    ubn
    registered_binaries
    registered_bin
    regbin
    makebinaries
    generate_binaries
  )

  # ========================================================================= #
  # === ARRAY_HELP_PACKAGE_RBT_OPTIONS
  #
  # All options if one wishes to package up the RBT scripts.
  # ========================================================================= #
  ARRAY_HELP_PACKAGE_RBT_OPTIONS = %w(
    --package-cookbooks
    --package_up
    package_cookbooks package_cookbooks packagecookbooks
    package_rbt_scripts
    prs generate_archive release make_release makerelease
    packageup packagescripts rbt_scripts package_the_rbt_scripts
  )

  # ========================================================================= #
  # === ARRAY_SYMLINK_ALL_PROGRAMS_OPTIONS
  #
  # All "symlink all programs" options are stored here.
  # ========================================================================= #
  ARRAY_SYMLINK_ALL_PROGRAMS_OPTIONS = %w(
    symlink_all_programs symlink_all symlinkall 
    traverse traverse_all traversinstall
    symlink_everything symlink_every_program
    --resymlink symlinkeveryprogram
    --symlink_every_program
    symlinkeverything
  )

  # ========================================================================= #
  # === ARRAY_HELP_SHOW_ALL_OPTIONS
  #
  # This covers all options that can be obtained via e. g "ry gcc show_all".
  # ========================================================================= #
  ARRAY_HELP_SHOW_ALL_OPTIONS = %w(
    --show-all
    --show_all
    show-all
    show_all
    showall
    showal
    SHOW_ALL
    sall
    s_all
    show_yaml_file
    show-content
    show_content
    showcontent  
  )

  # ========================================================================= #
  # === ARRAY_HELP_REPLACE_SYMLINKS_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_REPLACE_SYMLINKS_OPTIONS = %w(
    --replace-symlinks
    --replace_symlinks
    --replacesymlinks
    replace_symlinks
    replacesymlinks
    repsymlinks
    replacesym
    replace2
  )

  # ========================================================================= #
  # === ARRAY_HELP_SCAN_SOURCE_ARCHIVE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_SCAN_SOURCE_ARCHIVE_OPTIONS = %w(
    --scan-source-archive
    --scan_source_archive
    --scana
    -scana
    scan_source_archive
    scansourcearchive
    scanarchive
    scanning
    scana
    scan
    ssa
  )

  # ========================================================================= #
  # === ARRAY_HELP_DO_NOT_COMPILE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_DO_NOT_COMPILE_OPTIONS = %w(
    -I
    --do-not-compile
    --no-install
    skipcompile
    skipcompiling
    dont_compile
    dontcompile
    do_not_compile
    dont-compile
    dcompile
    dc
    dcom
  )
           
  # ========================================================================= #
  # === ARRAY_HELP_LAST_DOWNLOADED_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_LAST_DOWNLOADED_OPTIONS = %w(
    LAST_DOWNLOADED lastdownload lastdownloaded last
    last_downloaded --last-downloaded last-downloaded
    --lastdl lastdl
  )

  # ========================================================================= #
  # === ARRAY_FEEDBACK_MAKE_OPTIONS
  # ========================================================================= #
  ARRAY_FEEDBACK_MAKE_OPTIONS = %w(
    --make
    --make_options
    --make_opts
    make_opts
    makeopts
    makeopt
    feedback_make_options
  )+['make options']

  # ========================================================================= #
  # === ARRAY_HELP_CONFIG_OPTIONS
  #
  # These options are valid for when the user wishes to feedback the
  # main configuration options.
  # ========================================================================= #
  ARRAY_HELP_CONFIG_OPTIONS = %w(
    --main-config?
    --main-config
    --mainconfig?
    mainconfig?
    mainconfig
    mconfig
    config?
    config_dir?
  )

  # ========================================================================= #
  # === ARRAY_HELP_LAST_CONFIGURE_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_LAST_CONFIGURE_OPTIONS = %w(
    --last-options
    --lconfigure?
    last_configure_options?
    lconfigure?
    last_options?
    lastoptions?
    last-options
    lastoptions
    last_conf_options
  )

  # ========================================================================= #
  # === ARRAY_HELP_LOG_DIR_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_LOG_DIR_OPTIONS = %w(
    rbtbase?
    rbase?
    store_where?
    btbase?
    rbt_log_dir?
    log_dir?
    log-dir?
    --log_dir?
  )

  # ========================================================================= #
  # === ARRAY_FEEDBACK_CONFIGURE_OPTIONS
  # ========================================================================= #
  ARRAY_FEEDBACK_CONFIGURE_OPTIONS = %w(
    co
    con
    conf
    confi
    configure
    options
    optionen
    opts
    opt
    optio
    config
    configs
    configure?
    conf?
    configoptions?
    config_options?
    configureoptions?
    configure_options?
    configureoptions
    config_options
    configoptions
    configure_options
    configuration?
    coptions?
    sco
    co?
    xxx
    c?
  )

  # ========================================================================= #
  # === ARRAY_HELP_FEEDBACK_ALL_BINARIES_OPTIONS
  #
  # Keep track of which binaries a specific program should install.
  # ========================================================================= #
  ARRAY_HELP_FEEDBACK_ALL_BINARIES_OPTIONS = %w(
    --feedback-all-binaries
    --all_binaries
    --all_binaries?
    --regbin?
    --binaries?
    show-binaries
    installs_these_binaries?
    feedback_all_binaries
    allbinaries?
    all_binaries?
    its_binaries?
    bin?
    abin?
    which_binaries?
    binaries?
    binaries
    showbinaries
    show_binaries
    allbinaries
    bins?
    bin1
    bin2
  )

  # ========================================================================= #
  # === ARRAY_HELP_USE_BUILD_DIRECTORY_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_USE_BUILD_DIRECTORY_OPTIONS = %w(
    build
    --use-build-directory
    --use-builddir
    --use-build-dir
    use_this_build_directory
    use_specific_build_directory
    use_build_directory
    use_build_dir
    usebuilddir
    usebuild
    use_build
    -b
  )

  # ========================================================================= #
  # === ARRAY_HELP_GENERATE_ALL_SLACK_DESC_FILES_OPTIONS
  #
  # All the registered options for creating slackware .desc files.
  # ========================================================================= #
  ARRAY_HELP_GENERATE_ALL_SLACK_DESC_FILES_OPTIONS = %w(
    --generate-all-slack-desc-files
    --all-slack-files
    all-slack-files
    all-slacks
    allslacks
    aslacks
    allslack
  )

  # ========================================================================= #
  # === ARRAY_HELP_SHOW_WHICH_COLOURS_WILL_BE_USED
  #
  # The constants stored in the Array here can be used to display
  # which colours will be used by the RBT project.
  # ========================================================================= #
  ARRAY_HELP_SHOW_WHICH_COLOURS_WILL_BE_USED = %w(
    colours?
    use_which_colours?
    --show-which-colours-will-be-used
  )

  # ========================================================================= #
  # === ARRAY_HELP_DO_COMPILE_IN_A_TRADITIONAL_WAY_OPTIONS         (trad tag)
  #
  # These options are for compilation into the "/usr/" hierarchy, also
  # called the traditional way to compile a program.
  # ========================================================================= #
  ARRAY_HELP_DO_COMPILE_IN_A_TRADITIONAL_WAY_OPTIONS = %w(
    --traditional
    --trad
    trad 
    traditional
    traditiona
    tradition
    traditio
    traditi
    tradit
    tradry
    tradi
    rtad
    trad~
    rad
    tra
    tr
    oldschool
    t
    trade
    ntradt
    tradt
    na
    ttrad
    trda
    ntradd
    traed
    ntradtrad
    trd
    tad
    tras
  )

  # ========================================================================= #
  # === ARRAY_HELP_REPORT_N_PROGRAMS_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_REPORT_N_PROGRAMS_OPTIONS = %w(
    --n_programs?
    --nprograms
    --report-n-registered-programs
    registered_programs
    registered_programs?
    nprograms
    n_programs
    n_registered?
    registered?
    nprograms?
    nregistered?
    stats n_programs?
    entries?
    stats?
    n_registered?
    registered? 
  )+[
    /^-?-?report(_|-)?registered(_|-)?programs$/
  ]

  # ========================================================================= #
  # === ARRAY_SHOW_LAST_UPDATED_OPTIONS
  # ========================================================================= #
  ARRAY_SHOW_LAST_UPDATED_OPTIONS = %w(
    --last_update
    --last-update
    --lastupdate
    --slu
    --show-last-updated-programs
    --show_last_updated_programs
    --showlastupdatedprograms
    show_last_updated
    showlastupdated
    slu
    last-update?
    lastupdate
  )

  # ========================================================================= #
  # === ARRAY_HELP_LAST_COMPILED_OPTIONS
  # ========================================================================= #
  ARRAY_HELP_LAST_COMPILED_OPTIONS = %w(
    --last-compiled-program?
    --last_compile?
    --last_compiled_program?
    --lastprogram?
    lastcompile?
    last_compile?
    last_compiled_program?
    lastcompiledprogram?
    lastcompiled?
    lastprogram?
    last_program?
  )

end; end