#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/do_compile_this_program.rb'
# =========================================================================== #
require 'rbt/compile/batch_compile_the_programs.rb'
require 'rbt/toplevel_methods/infer_build_system.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === do_compile_this_program
  #
  # This method can be used to compile a given program.
  # ========================================================================= #
  def do_compile_this_program(
      this_program
    )
    if this_program.respond_to?(:end_with?) and
       this_program.end_with?('.la') and
      !File.exist?(this_program)
      do_not_try_to_autofix
      opnn; e "Trying to autocorrect the libtool entry at "\
              "#{royalblue(this_program)}"
      try_to_autocorrect_this_erroneous_libtool_entry(
        this_program
      )
      return # And "exit" here.
    end
    if block_given?
      yielded = yield
    else
      yielded = nil
    end
    if yielded and yielded == :do_not_initialize_the_dataset
    else
      initialize_default_values # <- Reset relevant variables here.
    end
    target_file = log_dir?+'is_on_gobolinux.md'
    if File.exist?(target_file) and
      !File.empty?(target_file)
      # ===================================================================== #
      # The above file must have been set on a GoboLinux system, when the
      # user did a rbt --auto before (or created the file manually).
      # ===================================================================== #
      enable_is_on_gobolinux
    end
    # ======================================================================= #
    # Handle the * argument. We will show to the user what programs
    # exist that start with this name.
    #
    # To trigger this method, do:
    #   ry a*
    #   ry e*
    # ======================================================================= #
    if this_program.respond_to?(:include) and
       this_program.include?('*')
      verbose_find_all_keys_starting_with(this_program)
      exit_program
    end
    # ======================================================================= #
    # Next, ensure that if the cookbooks file at hand is using any special
    # environment variable, that this will also be used. For example,
    # the program called "htop" will use these LDFLAGS: -ltinfow,
    # so this has to be enabled.
    # ======================================================================= #
    considering_setting_any_special_environment_variable
    # ======================================================================= #
    # Query for the use and availability of ccache.
    # ======================================================================= #
    if use_ccache? and !is_ccache_available?
      notify_the_user_that_ccache_is_not_available
    end
    # ======================================================================= #
    # The next variable will keep track of the tab-title that is to be used
    # for the KDE konsole tab title.
    # ======================================================================= #
    use_this_as_tab_title = :default
    # ======================================================================= #
    # Set a default program path to be used.
    #
    # Note that by default we will check whether the given input is a
    # file and does exist - but sometimes this is unwanted, so a flag
    # exists to control the behaviour. The following code does ensure
    # precisely that.
    # ======================================================================= #
    if check_whether_the_given_input_is_a_file_and_does_exist? and
       does_this_file_exist_and_is_it_a_file?(this_program)# This may be the case when the user supplied a path.
      # ===================================================================== #
      # In this case, we do not have to do as much checking, since we
      # will just try to compile the given program at hand, using
      # automatic inference.
      # However had, it may still be beneficial to determine whether
      # the program is registered. If it is then we can try to make
      # use of the dataset for the program, and then just change certain
      # parts of that program.
      # ===================================================================== #
      inferred_program_name = ProgramInformation.return_short_name(this_program)
      if RBT.is_this_program_included?(inferred_program_name)
        set_compile_this_program(
          File.basename(inferred_program_name)
        )
        if may_we_initialize_a_new_cookbook_dataset?
          initialize_cookbook_dataset_for(inferred_program_name)
        end
        do_show_the_infobox
        this_program = File.absolute_path(this_program)
        cookbook_dataset?.set_program_path(this_program)
      else
        do_not_show_the_infobox
      end
      set_program_path(this_program)
      set_program_name(program_path?)
      # ===================================================================== #
      # We must also re-write the program version:
      # ===================================================================== #
      set_program_version(
        ProgramInformation.return_version(
          File.basename(program_path?)
        )
      )
      use_this_as_tab_title = remove_archive_at_the_end(
        File.basename(this_program)
      )
    else # Else we must check for inclusion in the dataset.
      this_program = this_program.to_s unless this_program.is_a? String
      # ===================================================================== #
      # === Register the program for compilation
      #
      # Next, we have to register the name of the program somewhere,
      # which is to be done through a method-call. But first we will
      # try to sanitize it a little bit, if it includes at the least
      # one '/' character.
      # ===================================================================== #
      this_program = File.basename(this_program) if this_program.include? '/'
      this_program = set_compile_this_program(this_program)
      # ===================================================================== #
      # First, we make a re-write action if the cookbooks does not
      # include the given input at hand.
      # ===================================================================== #
      is_the_program_included = cookbooks_include_this_program? this_program
      unless is_the_program_included
        # =================================================================== #
        # We have to do another check, this time including the abbreviations.
        # =================================================================== #
        is_the_program_included = cookbooks_include_this_program?(
          this_program, :with_abbreviations
        )
        if is_the_program_included
          abbreviation_points_to = abbreviations?[this_program]
          unless abbreviation_points_to == this_program
            this_program = return_possible_abbreviation_to_this_input(this_program)
            set_program_name(this_program)
            set_compile_this_program(this_program)
          end
        # =================================================================== #
        # Query whether this is a registered binary name.
        # =================================================================== #
        elsif is_this_a_registered_binary_name?(this_program)
          belongs_to_this_package = RBT.all_binaries?[this_program]
          opnn; e 'This ('+orange(this_program)+') appears to be a registered '\
                  'binary name, belonging to the package `'+
                  sfancy(belongs_to_this_package)+'`.'
          set_program_name(belongs_to_this_package)
          set_compile_this_program(belongs_to_this_package)
          ensure_that_the_program_exists(belongs_to_this_package)
          this_program = belongs_to_this_package
        else
          ensure_that_the_program_exists(this_program)
        end
      end
      # ===================================================================== #
      # cd into the main log directory:
      # ===================================================================== #
      cd rbt_log_directory?, :be_quiet
      # ===================================================================== #
      # Determine whether the program is included or not:
      # ===================================================================== #
      if cookbooks_include_this_program?(this_program) and can_we_continue?
        # =================================================================== #
        # Ok, the program is included.
        # =================================================================== #
        yaml_dataset = load_yaml_dataset_for_this_program(
          this_program, :return_flat_datastructure
        )
        # =================================================================== #
        # Now that we have a dataset, we need to set (assign) it at once,
        # so that the subsequent method-calls work on a sane dataset.
        # =================================================================== #
        assign_to_the_main_dataset(
          yaml_dataset
        )
        # =================================================================== #
        # === Download the program if it does not yet exist locally
        #
        # Download the program if it does not exist here. This functionality
        # must come before we try to extract the program.
        #
        # Obtain the program path first, though.
        # =================================================================== #
        program_path = program_path?
        if program_path.nil?
          # ================================================================= #
          # This is a safeguard - but it is also the default, actually. The
          # cookbook-dataset is much more likely to contain the local "URL"
          # to the program at hand.
          # ================================================================= #
          program_path = cookbook_dataset_program_path?
        end
        unless File.exist? program_path
          download_program
          unicode_middle_cliner # Looks better with cliner, I think.
        end
        # =================================================================== #
        # Sometimes we must re-write the program path, e. g. if we take
        # another program version. The following method does that for us.
        # =================================================================== #
        consider_sanitizing_the_program_path
        # =================================================================== #
        # === Honour the "do_not_download" settings if it exists.
        # =================================================================== #
        do_not_download if cookbook_dataset?.do_not_download?
        # =================================================================== #
        # Also reset the compiled n-times counter next.
        # =================================================================== #
        reset_compiled_n_times_counter
        # =================================================================== #
        # Additionally, we must re-evaluate the user_prefix? variable.
        # For example, "ntrad scumm" should contain the new AppDir prefix.
        # =================================================================== #
        if user_prefix? and is_an_abbreviation?
          # ================================================================= #
          # If it was defined, we will assume that the user wanted to
          # use another prefix. This presently (Oct 2018) will not work
          # for all prefixes, so we may have to fix this completely at
          # a later time - for now it shall suffice.
          # ================================================================= #
          if user_prefix?.to_s.delete('/') == programs_directory?.delete('/')
            do_run_via_nontraditional_way # <- This should correctly re-set the prefix.
          end
        end
        determine_the_configure_base_directory_to_use
        # =================================================================== #
        # We must sync some settings onto the dataset-object next. These
        # entries will be listed next:
        #
        #   (1) user prefix if it exists
        #
        # =================================================================== #
        if user_prefix?
          # ================================================================= #
          # We must check for the user_prefix early.
          # ================================================================= #
          use_this_prefix = user_prefix?
          case use_this_prefix
          when :use_ntrad_prefix_at_a_later_time
            _ = yaml_dataset.program_name? # <- This combines short_name+'-'+program_version
            use_this_prefix = Prefix.new(_) { :ntrad }
            set_user_prefix(use_this_prefix)
          end
          yaml_dataset.set_prefix(use_this_prefix.to_s)
          # ================================================================= #
          # And sync it onto regular prefix setting too:
          # ================================================================= #
          @internal_hash[:prefix].set_prefix(user_prefix?.to_s)
        else
          # ================================================================= #
          # And sync from the yaml_dataset back to the common prefix.
          # ================================================================= #
          @internal_hash[:prefix].set_prefix(yaml_dataset.prefix?)
        end
        # =================================================================== #
        # Determine the program_path variable next.
        # =================================================================== #
        determine_the_program_path
        # =================================================================== #
        # Append the archive type next - but consider using a glob first.
        # =================================================================== #
        glob = Dir["#{program_path?}*"]
        unless glob.empty?
          first_result = glob.first
          set_program_path first_result
        end
        if File.exist? program_path?
          this_program = program_path?
        end
      end
    end
    # ======================================================================= #
    # Next we will have to check again whether there is a proper archive
    # at the specified source-dir location. If there is not a proper
    # archive AND if the program_path? does NOT point to a valid archive,
    # then we will attempt to guess an archive type, if there are locally
    # available files such as foobar-1.0.tar.xz.
    # ======================================================================= #
    if archive_type? and
       archive_type?.empty? and
       program_path? and
      !program_path?.empty? and
      !is_an_archive?(program_path?)
      all_possible_archives = return_all_archives_from_this_directory(
        File.dirname(program_path?)
      )
      unless all_possible_archives.empty?
        first_entry = all_possible_archives.first # Not ideal, but this should suffice for now.
        set_archive_type(first_entry)
        update_program_full_name
        # =================================================================== #
        # And we must re-build the full path too.
        # =================================================================== #
        assemble_full_path
        set_program_path(cookbook_dataset_program_path?)
      end
    end
    we_can_not_continue if an_error_has_occurred?
    return unless can_we_continue?
    # ======================================================================= #
    # Notify the user which program will be compiled/installed next:
    # ======================================================================= #
    opnn; e "The program `#{sfancy(compile_which_program?)}` "\
            "will be compiled/installed next."
    # ======================================================================= #
    # First, check whether we have to extract the archive at hand. We do
    # not have to extract .gem files, since "gem" does this for us.
    # ======================================================================= #
    skip_extracting_if_it_is_a_gem_file
    # ======================================================================= #
    # === (1) Consider changing the tab title:
    #
    # Changing the name of the tab title should happen only after the
    # program name was set. The method will check whether we will actually
    # do any tab-change action or not. Note that when we refer to "tab",
    # we really mean KDE konsole tabs primarily.
    # ======================================================================= #
    consider_changing_the_tab_title(use_this_as_tab_title)
    # ======================================================================= #
    # === (2) Set the CFLAGS next. This should come before we extract
    # the program. We will also honour the build_static variable.
    #
    # We will do this only for programs that have to be compiled, though.
    # ======================================================================= #
    if has_to_be_compiled?
      if build_static?
        # =================================================================== #
        # Honour static compilation settings in the ymal file.
        # =================================================================== #
        opnn; e 'The yaml file specifies that this program is to '\
                'be compiled statically.'
        opnn; e "Thus, we will use these CFLAGS: "\
                "#{lightblue(default_cflags?)}"
        # =================================================================== #
        # We also have to scan the configure command string. If that string
        # contains an instruction such as "--disable-static", then we will
        # get rid of it, and notify the user. The reason is that, quite
        # obviously, we can not have conflicting options passed.
        # Note that this may be changed at a later time in the future, if
        # we decide that only user-supplied options to RBT may change this
        # behaviour. For now, we simply add a way to check the logic on
        # it, which is an improvement over the old code which did not
        # do any logic-checking at all whatsoever.
        # =================================================================== #
        _ = configure_options?
        if _.include? '--disable-static'
          opnn; e 'The configure option to '+sfancy(compile_which_program?)+
                  ' is using the option called '+simp('--disable-static')+'.'
          opnn; e 'As this '+steelblue('would contradict')+
                  ' with our goal to enable static '\
                  'compilation,'
          opnn; e 'this option was replaced with --enable-static, from '\
                  'the configure command string.'
          cookbook_dataset?.configure_options = configure_options?.sub(
            /-?-?disable-?static/, '--enable-static'
          )
        end
        set_cflags(:build_static)
      end
    end
    # ======================================================================= #
    # In the past, we did set the default extract-directory via
    # extract_to_default_directory, before invoking
    # set_extract_to_this_directory() - but we no longer do so since
    # as of December 2018.
    # ======================================================================= #
    # (3) Extract the program if it exists: 
    # ======================================================================= #
    if extract_the_archive?
      if program_path?.nil?
        opnn; ewarn 'The program path is nil. Some error must have happened.'
        opnn; ewarn "We did try to compile the program: "\
                    "#{orange(compile_which_program?)}"
        opnn; ewarn 'We thus exit now.'
        exit
      else
        # =================================================================== #
        # === Extract the archive if it exists                  (extract tag)
        #
        # Extract the program next.
        # =================================================================== #
        extract_this_program_if_it_exists(program_path?)
        unless File.exist? program_path?
          opnn; e "No archive exists at `#{sfile(program_path?)}`."
          opnn; e 'As it was specified to extract the archive in this '\
                  'case, yet this'
          opnn; e 'archive could not be found at the above reported path, '\
                  'we will exit'
          opnn; e 'now. You may consider downloading the source archive, '\
                  'should'
          opnn; e 'it exist.' 
          exit_program
        end
        cd_to_the_configure_base_directory
      end
    end
    cd_to_the_configure_base_directory
    # ======================================================================= #
    # === Optional start messages
    #
    # Display optional start message here before we continue to the
    # preinstall? stage and other stages.
    # ======================================================================= #
    consider_displaying_the_start_message
    # ======================================================================= #
    # Consider applying a patch next - this should come before the
    # preinstall action, but after the optional start message.
    # ======================================================================= #
    consider_applying_a_patch
    # ======================================================================= #
    # === Check for sed-action
    #
    # Next, we check whether we must apply sed. It must come before any
    # preinstall-action, and after we have cd-ed to the correct directory.
    # This also means after we have extracted the archive at hand.
    # ======================================================================= #
    check_for_sed_action
    # ======================================================================= #
    # === Pre-install actions
    #
    # Next, handle pre-install actions first, should they exist.
    # ======================================================================= #
    run_preinstall_actions
    # ======================================================================= #
    # Next register that we are compiling a program.
    # ======================================================================= #
    register_that_this_program_is_currently_being_compiled
    # ======================================================================= #
    # We could set some environment variables if we use ccache.
    # However had, programs such as "cmake" may then no longer work,
    # so the following lines of code were disabled.
    # ======================================================================= #
    # if use_ccache?
    #   ENV['CC']  = 'ccache gcc'
    #   ENV['CXX'] = 'ccache g++'
    # end
    exit_program if extract_only?
    # ======================================================================= #
    # (4) Reset any possible error message at this point.
    # ======================================================================= #
    RBT.reset_error_message
    # ======================================================================= #
    # (5) Change directory to the proper path:
    # ======================================================================= #
    unless is_a_gem_file?
      # ===================================================================== #
      # We have to be careful here when we build up this target.
      # ===================================================================== #
      change_into_this_target = extract_to_which_directory?.dup+
                                dataset_program_short_name?.to_s.dup+
                                '-'+
                                program_version?.dup.to_s+'/'
      set_extracted_base_directory(
        change_into_this_target.dup
      )
      if cookbook_dataset_configure_base_directory?
        change_into_this_target << cookbook_dataset_configure_base_directory?.dup
      end
      set_configure_base_directory(
        change_into_this_target
      )
      unless return_pwd == configure_base_directory?
        opnn; e "Entering the directory "\
                "#{sdir(configure_base_directory?)}"
      end
      cd_to_the_configure_base_directory
    end
    # ======================================================================= #
    # Next, we have to consider whether we will use a build directory.
    # Some programs insist on using a separate build directory.
    # ======================================================================= #
    consider_handling_build_directory
    # ======================================================================= #
    # Next, we have to diverge - for files that exist as-is, we have
    # to guess the build type. The other ones can be compiled based
    # on the information stored in the cookbook dataset itself. We
    # determine whether the file exists by querying whether there
    # is a yaml_dataset.
    # ======================================================================= #
    if yaml_dataset?
      consider_running_the_installation_steps_if_they_have_been_defined
      # ===================================================================== #
      # === The (optional) autogen-step
      #
      # Consider running "autogen" next.
      #
      # This should come after the archive was already extracted, and
      # pre-installation steps were performed - but it should come
      # BEFORE we invoke "autoconf" (the latter one is an optional step
      # as well, though).
      # ===================================================================== #
      decide_whether_to_invoke_autogen # (autogen tag)
      # ===================================================================== #
      # === Infer the default build system in use
      #
      # Next, we will find out which build system is to be used. By default
      # no assumptions are made, so we have to remain flexible and try to
      # guess which build system makes sense. The user may overrule this
      # behaviour, though, by specifically instructing RBT to use a specific
      # build system. This may sometimes be necessary when there are
      # projects that make use of more than one build system.
      # ===================================================================== #
      use_this_build_system = 
        RBT.infer_build_system(
          use_this_build_system,
          program_path?,
          required_deps_on?,
          { use_a_build_directory: use_build_directory? }
        )
      determine_which_compiler_will_be_used
      # ===================================================================== #
      # The user can overrule the ^^^ above guess-work with the
      # following flag:
      # ===================================================================== #
      if @internal_hash[:use_this_build_system].nil?
        unless cookbook_dataset?.build_system?.is_a?(Symbol) or # Symbols will be ignored.
               cookbook_dataset?.build_system?.empty?
          use_this_build_system = cookbook_dataset?.build_system?
        end
      else
        use_this_build_system = @internal_hash[:use_this_build_system]
      end
      cookbook_dataset?.set_use_this_build_system(use_this_build_system)
      # ===================================================================== #
      # Consider showing the infobox next. The infobox may also show
      # the required dependencies of the given program at hand, which
      # should always come before the archive at hand is extracted.
      #
      # A configuration showing the dependencies, within the file
      # called "always_show_dependencies.yml".
      #
      # Note that since as of Nov 2017, the default is to not always
      # show the dependencies, so the user has to specifically enable
      # this option if he/she wants to have that.
      #
      # In the old code, we used to call the method
      # feedback_dependencies_of_the_program() rather than
      # show the infobox at hand.
      # ===================================================================== #
      consider_showing_the_infobox
      # ===================================================================== #
      # In the following case-when menu, we must invoke the proper
      # build-system.
      # ===================================================================== #
      case use_this_build_system # case tag
      # ===================================================================== #
      # === meson
      # ===================================================================== #
      when 'meson'
        set_configure_command(:meson)
        _ = "meson --prefix=#{prefix?.to_s}".dup
        if meson_configure_options? and
          !meson_configure_options?.empty?
          meson_configure_options = meson_configure_options?.strip.sub(/PLACEHOLDER/,'').strip
          unless meson_configure_options.to_s.empty?
            # =============================================================== #
            # Append the meson-configure options in this case.
            # =============================================================== #
            opne "Making use of the #{royalblue('meson_configure_options')}, "\
                 "as specified by the #{slateblue('.yml')} file."
            _ << " #{meson_configure_options?.to_s}"
          end
        else
          if use_configure_options?
            _ << " #{configure_options?.to_s}"
          end
        end
        _.squeeze!(' ')
        e teal(_)
        run_this_sys_command(_)
        cd name_of_the_build_directory?
        ninja_install
      # ===================================================================== #
      # === rubygem
      # ===================================================================== #
      when 'rubygem'
        install_this_ruby_gem(program_path?)
      # ===================================================================== #
      # === ninja
      # ===================================================================== #
      when 'ninja'
        # =================================================================== #
        # Since as of 02.10.2018, we try to use ninja here automatically,
        # if the cookbook_dataset has a dependency on ninja.
        # =================================================================== #
        set_use_this_make_command('ninja')
      # ===================================================================== #
      # === waf
      # ===================================================================== #
      when 'waf'
        run_waf_based_installation
      # ===================================================================== #
      # === cmake
      # ===================================================================== #
      when 'cmake'
        run_cmake_then_make_then_make_install
      # ===================================================================== #
      # === scons
      # ===================================================================== #
      when 'scons'
        run_scons_based_installation
      # ===================================================================== #
      # === python
      # ===================================================================== #
      when 'python'
        run_python_installation
      # ===================================================================== #
      # === default
      #
      # This is the default. We will default to 'configure', 'make'
      # and 'make install'. The else clause is also used for 'configure',
      # so the default is essentially 'configure' as well.
      # ===================================================================== #
      else
        # =================================================================== #
        # === GNU configure step
        #
        # This entry point is used to run the traditional "configure",
        # "make" and "make install" steps, right after considering
        # creating the AppDir skeleton.
        #
        # It is the assumed default entry point, as most programs will still
        # use the GNU configure chain (at the least in the year 2019).
        # =================================================================== #
        consider_creating_appdir_skeleton_then_consider_running_autoconf_then_run_configure_then_make_then_make_install
      end
    else
      # ===================================================================== #
      # In this case we have to guess the build type, before we proceed,
      # since we have given the path to a file that exists.
      # ===================================================================== #
      guessed_build_type = guess_build_type
      opnn; e "Guessing the build-type next ... the build type is: "\
              "#{slateblue(':'+guessed_build_type.to_s)}"
# =========================================================================== #
# === FIX THE NEXT THING:
# TODO: FIXME
# =========================================================================== #
cliner
e swarn("DEBUG: THIS HERE OUGHT TO BE FIXED")
ecrimson extract_to_this_directory?
ecrimson extract_to_this_directory?
ecrimson extract_to_this_directory?
cliner
      # ===================================================================== #
      # Next handle the different build-types appropriately:
      # ===================================================================== #
      case guessed_build_type
      when :configure
        run_configure_then_make_then_make_install
      end
    end
    we_can_not_continue if an_error_has_occurred?
    # ======================================================================= #
    # Consider symlink glib-schema files into the /usr/ hierarchy.
    # ======================================================================= #
    consider_symlinking_glib_schema_files_into_the_usr_share_hierarchy
    # ======================================================================= #
    # Make our instance of SymlinkProgram behave properly:
    # ======================================================================= #
    prepare_symlink_program_instance
    if successfully_compiled?
      # ===================================================================== #
      # Next, consider performing postinstallation steps:
      # ===================================================================== #
      consider_performing_postinstallation_actions # <- Postinstallation step
      # ===================================================================== #
      # Consider stripping the compiled binaries.
      # ===================================================================== #
      if had_to_be_compiled?
        do_strip_the_compiled_binaries
        # =================================================================== #
        # Next, we will consider creating and populating the subdirectory
        # called "lib/pkgconfig/". This may be necessary for programs that
        # install into "share/pkgconfig/" rather than "lib/pkgconfig".
        #
        # This step has to be done before we call the method
        # consider_symlinking_the_pkgconfig_files().
        # =================================================================== #
        consider_creating_and_populating_the_pkgconfig_directory
        # =================================================================== #
        # If the cookbook file at hand has symlinking_pkgconfig_files
        # enabled, and if the program is compiled, as AppDir, then we
        # will symlink .pc iles into /usr/lib/pkgconfig/, if the latter
        # one exists.
        #
        # This should actually be part of the post-installation actions,
        # but before running ldconfig.
        # =================================================================== #
        consider_symlinking_the_pkgconfig_files
        # =================================================================== #
        # Next register the configure-line into the configure "database",
        # unless a problem or error has occurred.
        # =================================================================== #
        if no_problem_was_encountered? and
           no_error_was_encountered?
          register_configure_line_into_configure_database
          # ================================================================= #
          # Also register it to the file that keeps track of which
          # programs were compiled successfully.
          # ================================================================= #
          register_last_compiled_program
        end
      end
      # ===================================================================== #
      # Consider running "ldconfig" next.
      # ===================================================================== #
      consider_running_ldconfig
    else
      # ===================================================================== #
      # Else a problem or an error may have occurred. We will thus check
      # for that. The error will be re-set more on the beginning of
      # this method rather than its end.
      # ===================================================================== #
      check_for_common_problems_and_common_errors
    end
    # ======================================================================= #
    # === Remove the extracted archive / directory
    #
    # We have to clean-up again, by removing the extracted archive. This
    # behaviour can be changed by the user, though. Additionally, the
    # individual .yml file may decide for us whether we will remove the
    # extracted directory or whether we will keep that directory. User
    # instructions overrule the settings in the .yml file, though.
    # ======================================================================= #
    consider_removing_the_extracted_archive
    # ======================================================================= #
    # Next we will consider removing empty subdirectories. This has to
    # happen here again, after the postinstall step, because the installation
    # may have failed. The method will also do a more general clean-up
    # step in the respective AppDir prefix.
    # ======================================================================= #
    consider_removing_empty_subdirectories
    consider_comparing_the_installed_libraries_versus_the_assumed_libraries
    store_time_snapshot
    consider_reporting_the_time_it_took_to_compile_this_program
    # ======================================================================= #
    # Also store the prefix that we used on the toplevel namespace.
    # ======================================================================= #
    RBT.set_last_prefix_used(use_which_prefix?)
    show_success_or_failure_status
    # ======================================================================= #
    # Ensure that the expanded cookbook .yml file exists:
    # ======================================================================= #
    if is_on_roebe? and !does_the_expanded_cookbook_file_exist_for_this_program?(this_program)
      expand_this_cookbook(this_program)
    end
    # ======================================================================= #
    # Return to the temp-directory as start again. This has to happen because
    # the underlying directory may have been removed, so we need to go to
    # a "stable" directory instead.
    # The second argument exists so that we are silent.
    # ======================================================================= #
    change_directory(temp_directory?, :do_not_report_anything)
    consider_changing_the_tab_title('.')
  end; alias compile_this_program do_compile_this_program # === compile_this_program

end; end
