#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/env.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === clear_env_variables_but_ensure_the_path_variable_works
  # ========================================================================= #
  def clear_env_variables_but_ensure_the_path_variable_works
    old_path = query_path?
    clear_env
    unless old_path.empty?
      # ===================================================================== #
      # Set a new PATH here, as otherwise compilation would fail.
      # ===================================================================== #
      opnn; e "Note that this will lead to compile-time failures because "\
              "#{orange('$PATH')}"
      opnn; e 'would not be set. We will use the old path entry '\
              'there as a consequence:'
      opnn; e sfancy("  #{old_path}")
      set_path(old_path)
    end
  end

  # ========================================================================= #
  # === verbose_do_not_use_special_flags
  # ========================================================================= #
  def verbose_do_not_use_special_flags
    opnn; e 'We will not use any special ENV flags, save for PATH.'
    clear_env_variables_but_ensure_the_path_variable_works
  end

  # ========================================================================= #
  # === clear_env
  #
  # This method will clear ENV. This may sometimes be necessary to start
  # from a clean state.
  # ========================================================================= #
  def clear_env(
      be_verbose = true
    )
    if be_verbose
      opnn; e "Clearing (resetting) the environment variable of "\
              "ruby via #{slateblue('ENV.clear()')}."
    end
    ENV.clear
  end

  # ========================================================================= #
  # === use_this_environment_variable
  #
  # This method will use a specific environment variable as the new
  # prefix. An example follows next.
  #
  # To use this, try a command invocation such as:
  #
  #   --use-this-environment-variable=XFCE_PREFIX
  #   --use-this-environment-variable=MATE_DESKTOP_PREFIX
  #
  # ========================================================================= #
  def use_this_environment_variable(i)
    i = i.to_s.dup # We only want to work on unfrozen Strings.
    if i.include? '--'
      i.gsub!(/--/,'') # Only get rid of '--', not '-'.
    end
    if i.include? '$'
      i.delete!('$')
    end
    if i.include? '='
      i = i.split('=').last
      # ======================================================================= #
      # Now we assume this to be an environment variable.
      # ======================================================================= #
      if ENV.has_key? i
        i = ENV[i]
      end
    else
      i = ENV[i] if ENV.has_key? i
    end
    # ======================================================================= #
    # Finally, set the user-prefix next.
    # ======================================================================= #
    set_user_prefix(i)
  end

  # ========================================================================= #
  # === verbose_do_not_use_special_environment_flags
  #
  # If we do not want to use any ENV variable, this entry point will
  # help us.
  # ========================================================================= #
  def verbose_do_not_use_special_environment_flags
    opnn; e 'We will not use any '+teal('special environmental flag')+' (ENV).'
    path = query_path?
    ENV.clear
    set_path(path) # Must re-enable PATH environment as it was.
    @internal_hash[do_not_use_special_environment_flags] = true
  end

  # ========================================================================= #
  # === set_env_variables?
  #
  # Determine whether we have to set any environment variable or not.
  # ========================================================================= #
  def set_env_variables?
    _ = cookbook_dataset?
    if _ and _.respond_to?(:set_env_variables?)
      _.set_env_variables?
    else
      false
    end
  end; alias set_env? set_env_variables? # === set_env?

  # ========================================================================= #
  # === considering_setting_any_special_environment_variable
  #
  # This is the method that will be used to determine whether we
  # have to set any special environment variable.
  # ========================================================================= #
  def considering_setting_any_special_environment_variable(
      be_verbose = true
    )
    _ = set_env_variables?
    if _ and !extract_only? # If we only extract something, then we don't need to do the rest.
      if be_verbose
        opnn; e "Making use of these #{aliceblue('special environment variables')}:"
      end
      set_these_env_variables(
        _, :do_set_these_variables, :be_verbose
      )
    end
  end

  # ========================================================================= #
  # === set_these_env_variables
  #
  # This method can be used to modify ENV variables.
  #
  # The first input argument to this method should be the Hash, consisting
  # of the key-value pairs in the form of key being the NAME of the shell
  # variable that is to be modified; and the corresponding value being
  # the new VALUE that you wish to set this variable to.
  #
  # The third argument to this method determines whether we will output
  # what we do to the user. By default this is true, but sometimes we
  # may want to be able to quietly set these environment variables,
  # such as when we attempt to use clang.
  #
  # Legitimate usage examples for this method, in ruby, would be:
  #
  #   set_this_env_variable 'CFLAGS = -gcc'
  #   set_this_env_variable 'LIBS: -lpthread'
  #
  # ========================================================================= #
  def set_these_env_variables(
      hash_or_string, # We allow Hash and String as input. But String must be in a special format.
      do_which_action = :report_only,
      be_verbose      = true
    )
    if block_given?
      yielded = yield
      if yielded.is_a? Symbol
        do_which_action = yielded
      end
    end
    unless @internal_hash[:do_not_use_special_flags]
      case do_which_action
      when :be_quiet, :be_silent
        be_verbose = false
        # =================================================================== #
        # Also change the do_which_action variable.
        # =================================================================== #
        do_which_action = :do_set_these_variables
      end
      case be_verbose
      when :be_verbose
        be_verbose = true
      when :be_quiet, :be_silent
        be_verbose = false
      end
      # ===================================================================== #
      # === Next, handle Strings
      # ===================================================================== #
      if hash_or_string.is_a? String
        if hash_or_string.include?('=') or
           hash_or_string.include?(':')
          if hash_or_string.include?('=')
            use_this_as_split_token = '='
          elsif hash_or_string.include?(':')
            use_this_as_split_token = ':'
          end
          splitted = hash_or_string.sub(/^export /,'').split(use_this_as_split_token)
          this_key   = splitted.first
          this_value = splitted.last
          hash_or_string = { this_key => this_value } 
        end
      end
      # ===================================================================== #
      # === Next, handle Hashes
      # ===================================================================== #
      hash_or_string.each_pair {|key, value|
        value.strip! if value.is_a? String
        # =================================================================== #
        # Make some exceptions, e. g. for gems.
        # =================================================================== #
        return if is_a_gem?
        if be_verbose and is_compiled? and !extract_only? and not value.to_s.empty?
          # ================================================================= #
          # Notify the user as to the env-value modification. Since as of
          # November 2018 we will no longer use opnn() here.
          # ================================================================= #
          e "  #{sfancy(key)} will be set to the "\
            "value(s): #{lightblue(value.to_s)}"
        end
        if value.is_a? FalseClass or
           value.is_a? TrueClass or
           value.is_a? Integer
          value = value.to_s
        end
        case do_which_action
        when :do_set_these_variables
          # ================================================================= #
          # Next set the value to the environment variable.
          # ================================================================= #
          ENV[key] = value
        end 
      }
    end
  end; alias set_this_env_variable set_these_env_variables # === set_this_env_variable

end; end