#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/feedback.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/feedback_description_of.rb'
require 'rbt/cookbooks/display_cookbook_dataset/feedback/feedback.rb'
require 'rbt/cookbooks/display_cookbook_dataset/feedback/feedback_url.rb'

module RBT

class Compile < RBT::CompileBase

  begin
    require 'case_parser'
  rescue LoadError; end

  # ========================================================================= #
  # === feedback_whether_we_will_strip_or_whether_we_will_not
  #
  # Feedback whether we will run the "strip" binary or whether we
  # will not.
  #
  # Invocation example:
  #
  #   ry --strip?
  #
  # ========================================================================= #
  def feedback_whether_we_will_strip_or_whether_we_will_not
    _ = strip_binaries?
    case _
    when true
      opne 'Yes, we will run "'+slateblue('strip')+'" on the resulting '\
           'binaries of'
      opne 'the given program at hand.'
    when false
      opne 'No, we will '+swarn('NOT')+' run "strip" on the '\
           'resulting binaries of'
      opne 'the given program at hand.'
    end
  end

  # ========================================================================= #
  # === feedback_make_options
  #
  # To trigger this method, do e. g.:
  #
  #   ry ruby make_opts
  #   ry binutils make_opts
  #
  # ========================================================================= #
  def feedback_make_options(
      i = compile_which_program?
    )
    _ = cookbook_dataset_make_options?
    _ = _.first if _.is_a? Array
    e
    e royalblue(_)
    e
  end

  # ========================================================================= #
  # === show_all_binaries_of_this_program
  #
  # Use this method to show the registered binaries of the input-argument,
  # the given program at hand.
  # ========================================================================= #
  def show_all_binaries_of_this_program(
      this_program = program?
    )
    if dataset?.nil?
      ensure_that_a_dataset_exists_for_this_program(this_program)
    end
    _ = binaries?
    if _.empty?
      opne "No binaries/executables appear to be registered for "\
           "the program `#{sfancy(this_program)}`."
    else
      e; _.each_with_index {|this_binary, index| index += 1
        index = "  #{slateblue((index.to_s+') ').rjust(5))}"
        e "#{index}#{sfancy(this_binary)}"
      }; e
    end
  end; alias feedback_all_binaries show_all_binaries_of_this_program # === feedback_all_binaries

  # ========================================================================= #
  # === feedback_licenses
  #
  # This will feedback all licenses, or apply a filter.
  #
  # Usage example:
  #   ry htop flicenses
  # ========================================================================= #
  def feedback_licenses(
      filter_for_licenses = '*'
    )
    RBT::Cookbooks::FeedbackLicenses.new(filter_for_licenses) # bl $RUBY_COOKBOOKS/lib/cookbooks/feedback_licenses.rb
  end

  # ========================================================================= #
  # === feedback_program_version
  # ========================================================================= #
  def feedback_program_version
    RBT::Cookbooks::DisplayCookbookDataset::Feedback.feedback_program_version(program?)
  end

  # ========================================================================= #
  # === feedback_programs_dir
  # ========================================================================= #
  def feedback_programs_dir
    opne 'The programs directory is at '+sfile(programs_dir?)
  end

  # ========================================================================= #
  # === feedback_all_case_options_of_the_main_menu
  # ========================================================================= #
  def feedback_all_case_options_of_the_main_menu
    _ = THIS_FILE
    if File.exist? _
      if Object.const_defined? :CaseParser
        _ = CaseParser.parse(_)
        _.each {|entry|
          e entry
        }
      else
        opne 'The project case_parser is not available. Please install it.'
        e
        opne sfancy('  gem install case_parser')
        e
      end
    else # Else the file does not exist.
      opne "No file at #{sfile(_)} could be found. We "\
           "thus can not use CaseParser."
    end
  end

  # ========================================================================= #
  # === feedback_all
  # ========================================================================= #
  def feedback_all(i = program?)
    RBT::Cookbooks::Cookbook.feedback_all(i)
  end

  # ========================================================================= #
  # === feedback_various_versions
  #
  # This actually will only feedback the RBT version in use, since we
  # have merged the different projects together in September 2018.
  #
  # To invoke this method, do:
  #   ry htop feedback_version
  # ========================================================================= #
  def feedback_various_versions(
      optional_use_opn = true,
      ljust            = 45
    )
    case optional_use_opn
    when :do_not_use_opn
      optional_use_opn = false
    end
    feedback_rbt_version(ljust, optional_use_opn)
  end

  # ========================================================================= #
  # === feedback_sanitize_dataset_version
  # ========================================================================= #
  def feedback_sanitize_dataset_version(
      ljust = 35, use_opn = true
    )
    opnn if use_opn
    e sfancy('RBT::Cookbooks::Cookbook version: '.ljust(ljust))+
      simp(RBT.version?)
  end

  # ========================================================================= #
  # === feedback_rbt_version
  # ========================================================================= #
  def feedback_rbt_version(
      ljust = 35, use_opn = true
    )
    opnn if use_opn
    e sfancy('RBT Version: '.ljust(ljust))+
      simp(RBT.version?)
  end

  # ========================================================================= #
  # === feedback_main_config
  #
  # In order to invoke this method, do:
  #
  #   rbt config?
  #
  # ========================================================================= #
  def feedback_main_config
    opne 'We will show the configuration options set for the '\
         'RBT project next.'
    cliner {
      hash = RBT.configuration?._ # Delegate to the config.
      hash.each_pair {|key, value|
        case key.to_s.strip
        when 'email'
          value = crimson(value.to_s.ljust(10))
        else
          value = slateblue(value.to_s.ljust(10))
        end
        key = lightgoldenrodyellow( ((key.to_s+':').ljust(46)) )
        e '  '+key+' '+value
      }
    }
  end

  # ========================================================================= #
  # === feedback_main_config_then_exit
  # ========================================================================= #
  def feedback_main_config_then_exit
    feedback_main_config
    exit_program
  end

  # ========================================================================= #
  # === feedback_what_we_are_currently_compiling?
  #
  # This method will feedback what we are currently compiling.
  #
  # To invoke this method, do:
  #   ry --what-are-we-currently-compiling?
  # ========================================================================= #
  def feedback_what_we_are_currently_compiling?
    status_file = FILE_STATUS
    if File.exist? status_file
      dataset = YAML.load_file(status_file)
    end
    this_key = 'currently_compiling_these_programs'
    if dataset.has_key?(this_key) and
       !dataset[this_key].empty? # This should work because it is an Array.
      these_programs_are_currently_compiled = dataset[this_key]
      opne 'The following programs are currently compiled:'
      e
      these_programs_are_currently_compiled.each {|this_program|
        e "  #{sfancy(this_program)}"
      }; e
    else
      opne 'No program is being currently compiled.'
    end
  end

  # ========================================================================= #
  # === show_all_locally_available_programs
  #
  # Invocation example:
  #    rbt --locally-available-programs?
  # ========================================================================= #
  def show_all_locally_available_programs(
      slice_n_chunks = 3
    )
    programs_dir = programs_dir?
    all_dirs = Dir[programs_dir+'*'].select {|entry| File.directory? entry }
    all_dirs.map! {|entry|
      use_this_regex = /^#{programs_dir}/
      entry = entry.sub(use_this_regex,'')
      entry # .downcase # To downcase or not to downcase.
    }
    n_programs = all_dirs.size.to_s
    opne "The locally available `#{orange(n_programs)}` programs are:"
    sliced = all_dirs.each_slice(slice_n_chunks)
    result = ''.dup
    sliced.each {|inner_group_array|
      joined = inner_group_array.map {|entry|
        (entry+',').ljust(25)
      }.join
      result << '  '+sfancy(joined.strip).delete(',')+N
    }
    result = result.strip.chop.chomp.strip
    result.sub!(/,\e\[0;37$/, '.') # Reformat it.
    e
    e result
    e
  end

  # ========================================================================= #
  # === feedback_whether_the_cookbook_will_modify_env_variables
  # ========================================================================= #
  def feedback_whether_the_cookbook_will_modify_env_variables
    _ = set_env_variables?
    if _.nil?
      opne 'The program '+sfancy(program_name?)+' does not '\
           'set any ENV variable.'
    else
      opne 'The program '+sfancy(program_name?)+' will modify '\
           'at the least one ENV variable.'
      opne 'Displaying these next:'
      set_these_env_variables(_, :report_only)
    end
  end

  # ========================================================================= #
  # === feedback_configure_options
  #
  # To test this method, try something like this:
  #   ry htop configure_options?
  #   ry gcc configure_options?
  # ========================================================================= #
  def feedback_configure_options(i = compile_this_program?)
    opne 'The configure options for the program '+sfancy(i)+' are:'
    if dataset?.nil?
      ensure_that_we_have_a_dataset(i)
    end
    _ = configure_options?
    e
    e '  '+sfancy(_)
    e
  end

  # ========================================================================= #
  # === feedback_program_description
  #
  # We feedback the description here, by tapping into the method
  # feedback_description() from class Cookbooks.
  # ========================================================================= #
  def feedback_program_description(i = program?)
    e
    RBT.feedback_description_of(i)
    e
  end; alias feedback_description feedback_program_description # === feedback_description

  # ========================================================================= #
  # === feedback_extra_description
  #
  # To trigger this method, do:
  #    ry php edesc
  # ========================================================================= #
  def feedback_extra_description(
      of_this_program = program?
    )
    RBT::Cookbooks::DisplayCookbookDataset::Feedback.feedback_extra_description(of_this_program)
  end

  # ========================================================================= #
  # === feedback_all_libraries_and_all_binaries
  #
  # Invoke this by issuing:
  #   ry htop installs_what?
  # ========================================================================= #
  def feedback_all_libraries_and_all_binaries(
      of_this_program = compile_which_program?
    )
    feedback_all_binaries(of_this_program)
    feedback_all_libraries(of_this_program)
  end

  # ========================================================================= #
  # === feedback_all_libraries
  #
  # Feedback all libraries from a given program.
  # ========================================================================= #
  def feedback_all_libraries(
      from_this_program = program?
    )
    RBT::Cookbooks::DisplayCookbookDataset::Feedback.feedback_all_libraries(from_this_program)
  end

  # ========================================================================= #
  # === feedback_dependencies_of_this_program
  #
  # Feedback the dependencies of a given program through this method.
  #
  # Invocation example:
  #   ry pango --feedback_dependencies_of_this_program
  # ========================================================================= #
  def feedback_dependencies_of_this_program(
      i = compile_which_program?
    )
    if dataset?.nil?
      ensure_proper_dataset(i)
    end
    dependencies = dataset?.deps_on?
    unless dependencies.empty?
      cliner {
        opne 'We did find these dependencies for `'+sfancy(i)+'`:'
        # =================================================================== #
        # We will next assume that dependencies is an Array.
        # =================================================================== #
        dependencies.each {|dep|
          opne '  A dependency on: `'+olivedrab(dep)+'`.'
        }
      }
    else
      opne 'No dependency was found for `'+sfancy(i)+'`.'
    end
  end; alias feedback_dependencies                feedback_dependencies_of_this_program # === feedback_dependencies
       alias feedback_dependencies_of_the_program feedback_dependencies_of_this_program # === feedback_dependencies_of_the_program

  # ========================================================================= #
  # === feedback_default_values_of_the_internal_hash
  #
  # Invocation example from the commandline:
  #   ry --feedback-the-default-values-of-the-internal-hash
  # ========================================================================= #
  def feedback_default_values_of_the_internal_hash
    dataset = return_default_internal_hash
    e
    e "The following #{lightblue(dataset.keys.size.to_s)} entries are known:"
    e
    dataset.each_pair {|key, value|
      if value.nil?
        value = 'nil'
      end
      e "  #{slateblue(key)}: #{lightgreen(value)}"
    }; e
  end

  # ========================================================================= #
  # === feedback_prefix_in_use?
  #
  # Use this method to feedback which prefix is in use.
  #
  # Usage examples:
  #   ry allegro --prefix?
  #   ry openmpi --prefix?
  #   ry htop    prefix?
  # ========================================================================= #
  def feedback_prefix_in_use?(
      shall_we_exit = true
    )
    program_name = compile_which_program?.dup
    if cookbook_dataset?.nil?
      ensure_proper_dataset_for(program_name)
    end
    e "The prefix in use for #{lightgreen(program_name)} will be:"
    e "  #{sfancy(prefix?)}"
    exit_program if shall_we_exit
  end; alias feedback_prefix_in_use feedback_prefix_in_use? # === feedback_prefix_in_use

  # ========================================================================= #
  # === feedback_whether_we_will_use_stow
  #
  # To invoke this, do:
  #
  #   ry --use-stow?
  #
  # ========================================================================= #
  def feedback_whether_we_will_use_stow
    opne "Will we use stow? #{royalblue(verbose_truth(use_stow?))}"
  end

  # ========================================================================= #
  # === feedback_url
  #
  # Wrapper around Cookbooks functionality.
  #
  # Usage example:
  #   rbt htop url?
  # ========================================================================= #
  def feedback_url(i = compile_which_program?)
    RBT::Cookbooks::Url.new(i)
    # ======================================================================= #
    # The next line of code has been commented out at 21.09.2018:
    # ======================================================================= #
    # ::RBT::Cookbooks::DisplayCookbookDataset.feedback_url.feedback_url(compile_which_program?)
    # ======================================================================= #
    # The next part may be controversial, as it sets the xorg buffer.
    # As of May 2014 I have disabled it.
    # ======================================================================= #
    #   result = @cookbooks.feedback_url
    #   set_xorg_buffer(result)
  end

  # ========================================================================= #
  # === feedback_local_location
  #
  # Invoke this like:
  #   ry htop feedback_local_location
  # ========================================================================= #
  def feedback_local_location(
      i = compile_which_program?
    )
    RBT::Cookbooks::DisplayCookbookDataset::Feedback.feedback_local_location(i)
  end

  # ========================================================================= #
  # === feedback_local_base
  #
  # Feedback which local base directory is used for a particular
  # program.
  # ========================================================================= #
  def feedback_local_base(
      i = compile_which_program?
    )
    RBT::Cookbooks::DisplayCookbookDataset.feedback_local_base(i)
  end

  # ========================================================================= #
  # === feedback_all_registered_binaries_of_all_programs
  #
  # This method will show, on the commandline, all registered binaries
  # of all the programs together.
  #
  # Invocation example:
  #   rbt --all-binaries
  # ========================================================================= #
  def feedback_all_registered_binaries_of_all_programs
    _ = RBT.registered_binaries?
    opne 'These '+royalblue(_.size.to_s)+' binaries are registered:'
    e
    _.each_with_index {|this_binary, index| index += 1
      padded_index = (index.to_s+') ').rjust(5)
      e '  '+lightgreen(padded_index)+
             slateblue(this_binary)
    }
    e
  end

  # ========================================================================= #
  # === feedback_last_compiled
  #
  # There are several ways how to invoke this method.
  # We thus need to be rather flexible about feedbacking this to the
  # user.
  #
  # One of the first distinctions will be whether we have a '=' token
  # in the input or whether we do not.
  #
  # Examples:
  #
  #   rbt --last-compiled?
  #   rbt --last-5-compiled?
  #   rbt --last-five-compiled?
  #   rbt --last-compiled?=5
  #   rbt --last-compiled?=6
  #
  # ========================================================================= #
  def feedback_last_compiled(i)
    i = i.first if i.is_a? Array # Grab the first entry.
    # ======================================================================= #
    # Next, we determine the dataset of which programs have been compiled.
    # ======================================================================= #
    dataset = compiled_programs?
    show_n_programs = 1 # This is the default.
    if i.to_s =~ /^\d+$/
      show_n_programs = i.to_i
    end
    if i and i.to_s.include?('=')
      splitted = i.split('=')
      show_n_programs = splitted.last.to_i
    elsif (show_n_programs > 1)
      # pass through in this case
    else # Else it does not include a '='.
      regex_to_use = /--last(-\d+-)compiled\?/
      i =~ regex_to_use
      show_n_programs = $1.to_s.dup.delete('-').to_i if $1
      array = HASH_ENGLISH_NUMBERS.keys
      if array.any? {|entry| i.include? entry }
        result = array.select {|entry| i.include? entry }
        result = result.first unless result.empty?
        # =================================================================== #
        # Now we must convert this:
        # =================================================================== #
        result = HASH_ENGLISH_NUMBERS[result]
        show_n_programs = result.to_i
      end
    end
    # ======================================================================= #
    # Narrow our dataset down.
    # ======================================================================= #
    if show_n_programs > dataset.size
      opne 'We have not kept track of that many ('+
            sfancy(show_n_programs.to_s)+') registered programs, so we will'
      opne "only show #{dataset.keys.size.to_s} programs instead."
      show_n_programs = dataset.size
    end
    dataset = dataset.reverse[0 .. (show_n_programs - 1)].reverse
    e
    opne "The last #{simp(show_n_programs.to_s)} compiled "\
         "programs (most recent one comes first) were:"
    # ======================================================================= #
    # Now we can report this dataset, which contains our programs.
    # ======================================================================= #
    e; dataset.reverse.each {|key|
      e "  #{sfancy(key)}"
    }; e
  end

end; end # end RBT::Compile