#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/gobolinux.rb'
# =========================================================================== #
require 'rbt/linux/gobolinux/create_recipe.rb'
require 'rbt/linux/gobolinux/create_dependencies_file.rb'
require 'rbt/toplevel_methods/gobolinux.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile.new

  # ========================================================================= #
  # === enable_is_on_gobolinux
  #
  # This method can be used to default to a GoboLinux behaviour.
  #
  # We should closely follow the behaviour of GoboLinux via the RBT
  # scripts, 1:1 - no need to create incompatibilities at this point.
  # ========================================================================= #
  def enable_is_on_gobolinux
    # ENV['IS_ON_GOBOLINUX'] = '1' # <- Disabled as of 25.09.2018; we probably no longer need to do this.
    @internal_hash[:is_on_gobolinux] = true
    use_non_traditional_prefix
    do_not_symlink_headers
    do_not_symlink_pkgconfig_files
    do_use_gobolinux_symlinkprogram # <-- Trying this as of Oct 2017.
    do_not_register_into_the_yaml_database
  end; alias is_on_gobolinux enable_is_on_gobolinux # === is_on_gobolinux

  # ========================================================================= #
  # === do_not_register_into_the_yaml_database
  # ========================================================================= #
  def do_not_register_into_the_yaml_database
    RBT.do_not_register_into_the_yaml_database
  end

  # ========================================================================= #
  # === show_instructions_for_gobolinux
  # ========================================================================= #
  def show_instructions_for_gobolinux
    _ = RUBY_SRC_DIR+'rbt/doc/INSTRUCTIONS_FOR_GOBOLINUX.md'
    if File.exist? _
      e File.read(_)
    end
  end

  # ========================================================================= #
  # === is_on_gobolinux?
  #
  # Query whether we are on GoboLinux or whether we are not.
  #
  # In the past this was only a check for an environment variable.
  #
  # Since we will allow commandline options to change this, we also had
  # to use a proper variable to keep track of this.
  # ========================================================================= #
  def is_on_gobolinux?
    @internal_hash[:is_on_gobolinux]
  end

  # ========================================================================= #
  # === report_whether_we_are_on_gobolinux
  #
  # To invoke this method, do:
  #
  #   rbt --is-on-gobolinux?
  #
  # ========================================================================= #
  def report_whether_we_are_on_gobolinux
    opnn; e 'Are we on GoboLinux: '+
             royalblue(
               verbose_truth(is_on_gobolinux?.to_s)
             )
  end; alias verbose_query_whether_we_are_on_a_gobolinux_like_system report_whether_we_are_on_gobolinux # === verbose_query_whether_we_are_on_a_gobolinux_like_system

  # ========================================================================= #
  # === permanently_enable_is_on_gobolinux
  #
  # Use this method to "enable" the GoboLinux file layout.
  #
  # To call this method from the commandline, do:
  #   rbt --permanently-enable-is-on-gobolinux
  # ========================================================================= #
  def permanently_enable_is_on_gobolinux(
      be_verbose = :be_verbose
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    _ = RBT.file_is_on_gobolinux
    if File.exist? _
      opnn; e "The file at `#{sfile(_)}` already exists."
    else
      if be_verbose
        opnn; e "Enabling GoboLinux layout now via the file `#{sfile(_)}`."
      end
      opnn; e "Now creating the file #{sfile(_)}."
      create_file(_)
      enable_is_on_gobolinux # And also enable the GoboLinux behaviour.
    end
  end

  # ========================================================================= #
  # === do_use_gobolinux_symlinkprogram
  # ========================================================================= #
  def do_use_gobolinux_symlinkprogram(be_verbose = true)
    if be_verbose
      opnn; e 'Using the GoboLinux SymlinkProgram rather than '\
              'the internal RBT::SymlinkProgram script.'
    end
    @internal_hash[:use_gobolinux_symlinkprogram] = true
  end

  # ========================================================================= #
  # === generate_all_gobolinux_recipes
  #
  # This method can be used to auto-generate all GoboLinux recipes.
  #
  # Invocation example:
  #   ry --generate-all-gobolinux-recipes
  # ========================================================================= #
  def generate_all_gobolinux_recipes
    # ======================================================================= #
    # Must first change into an appropriate directory.
    # ======================================================================= #
    target_dir = "#{rbt_log_directory?}gobolinux/"
    if File.directory? target_dir
      # ===================================================================== #
      # Always remove the old directory should it exist.
      # ===================================================================== #
      remove_directory(target_dir)
    end
    mkdir(target_dir)
    if File.directory? target_dir
      opnn; cd target_dir, :be_verbose
      # ===================================================================== #
      # Next, create the subdirectories but be silent about it.
      # ===================================================================== #
      array = ('a'..'z').to_a
      # ===================================================================== #
      # Add two exceptions.
      # ===================================================================== #
      array[0,0] = '3'
      array[0,0] = '0'
      array.each {|entry| mkdir(entry, :be_quiet) }
    else
      opnn; e "The target directory `#{sdir(target_dir)}` does not exist."
      opnn; e 'Please create it before we can create GoboLinux recipes.'
      return
    end
    available_programs?.each {|this_program|
      opnn; e 'Now generating a GoboLinux recipe for '\
              'the program `'+sfancy(this_program)+'`.'
      create_gobolinux_recipe_for(this_program)
    }
    # ======================================================================= #
    # Now that we have generated all these recipes, let's create a tarball
    # of the directory.
    # ======================================================================= #
    cd rbt_log_directory?, :be_quiet
    begin
      require 'totarxz'
      gobolinux_recipes_archive = "gobolinux-recipes-#{return_current_date}"
      mv 'gobolinux', gobolinux_recipes_archive
      ToTarXz.new(gobolinux_recipes_archive)
      target_archive = "#{gobolinux_recipes_archive.dup}.tar.xz"
      if File.exist? target_archive
        opnn; e "Everything has been packaged up into `"\
                "#{sfancy(target_archive)}` (cwd: #{sdir_return_pwd})."
        # =================================================================== #
        # Rename the directory again.
        # =================================================================== #
        opnn; mv gobolinux_recipes_archive, 'gobolinux'
        # =================================================================== #
        # FTP upload if we are on Roebe.
        # =================================================================== #
        if is_on_roebe?
          require 'ftp_paradise'
          if File.exist? target_archive
            FtpParadise.upload_this_binary_file(target_archive)
          else
            opnn; e 'Can not upload the file '+sfile(target_archive)
            opnn; e 'as it does not exist.' 
          end
          # ================================================================= #
          # Next, connect and upload all the individual directories to
          # my remote FTP host.
          # ================================================================= #
        end
        # =================================================================== #
        # Next, upload the individual directories to the remote FTP host.
        # Unfortunately this has a problem so we have disabled this for now.
        # =================================================================== #

# =========================================================================== #
# DISABLED FOR NOW. MUST BE FIXED ONE DAY.
# =========================================================================== #
if false
        _ = FtpParadise.new(:shevy)
        _.remote_mkdir 'gobolinux'
        _.remote_cd    'gobolinux'
        opnn; cd 'gobolinux', :be_verbose
        array.each {|this_directory|
          _.remote_mkdir(this_directory)
          _.rcd(this_directory)
          # ================================================================= #
          # Ok, we can obtain the local directories and then upload them
          # all.
          # ================================================================= #
          local_target = this_directory+'/*'
          all_local_directories = Dir[local_target].select {|entry|
            File.directory? entry
          }
          if all_local_directories.empty?
            opnn; e 'Nothing found under '+sfancy(local_target)+'.'
          else
            all_local_directories.each {|this_entry|
              opnn; e 'Now uploading the directory `'+sdir(this_entry)+'`.'
              _.upload_this_directory(this_entry)
            }
          end
          _.rcd '..' # Return to the base directory again.
        }
end # end false clause (DEBUG - FIXME!!!)
      end
    rescue LoadError
      opnn; e 'Please install the gem totarxz, in order to create '\
              'a '+sorange('.tar.xz')+' archive.'
    end
  end

  # ========================================================================= #
  # === create_dependencies_file
  #
  # This method will create a Gobolinux dependencies file.
  #
  # To invoke it, do something like:
  #   ry htop create_dependencies_file
  # ========================================================================= #
  def create_dependencies_file(
      create_where = rbt_log_directory?
    )
    _ = RBT::Linux::Gobolinux::CreateDependenciesFile.new(
          create_where, 
          required_deps_on?,
          :dont_run_yet
        )
    _.allow_creation_of_resources
    _.run # Finally, run.
  end

  # ========================================================================= #
  # === load_data_from_this_gobolinux_recipe
  #
  # This method will allow us to load the dataset from a local
  # GoboLinux recipe, as a drop-in replacement.
  #
  # The most important setting there will be to use:
  #
  #   url="http://ftp.gnome.org/pub/gnome/sources/GConf/3.2/GConf-3.2.6.tar.xz"
  #
  # which is part of the file called "Recipe".
  #
  # The other settings are also useful but not as important.
  # ========================================================================= #
  def load_data_from_this_gobolinux_recipe(i)
    i = i.to_s.dup # Always work on a new String-copy anyway.
    case i
    when 'Dir.pwd'
      i = return_pwd
    end
    # ======================================================================= #
    # === Handle GoboLinux Recipes past that point
    # ======================================================================= #
    if i.include? '/'
      the_program_is_included # Enable the fact that the program is "included":
      # ===================================================================== #
      # Assume the input to be something like '/Depot/jjjjj/GConf/3.2.6-r3/'.
      # ===================================================================== #
      if i.count('/') > 1 # At the least two '/'.
        splitted = i.split('/')
        program_name = splitted[-2]
        program_version = splitted[-1]
        set_program_name(program_name)
        initialize_yaml_dataset(program_name, :bypass_check)
        set_program_version(program_version)
        chop_off_revision_number
        set_base_dir(i)
        # =================================================================== #
        # The position of the Description file.
        # =================================================================== #
        description_file = 'Resources/Description'
        dataset_from_the_description_file = nil
        assume_default_archive_type
        # =================================================================== #
        # Next, read in the information from the file 'Recipe'.
        # =================================================================== #
        this_file = 'Recipe'
        if File.exist? this_file
          recipe_dataset = File.read(this_file)
          use_this_regex = /^url="(.+)"/
          use_this_url = recipe_dataset.scan(use_this_regex).flatten.first
          set_url(use_this_url)
          set_this_archive(File.basename(use_this_url))
        else
          opnn; e 'Something appears to be incorrect - no file '\
                  'called '+sfile(this_file)+' could be found.'
          exit_program
        end
        if File.exist? description_file
          dataset_from_the_description_file = File.read(description_file)
          # ================================================================= #
          # Next, use some regexes to properly interprete the dataset.
          # ================================================================= #
          regex_for_url1              = /\[Homepage\] (.+)/
          regex_for_license           = /\[License\] (.+)/
          regex_for_description       = /\[Description\] (.+)/
          regex_for_short_description = /\[Summary\] (.+)/
          _regex_for_name             = /\[Name\] (.+)/
          # ================================================================= #
          # Next determine some values here.
          # ================================================================= #
          dataset_from_the_description_file =~ regex_for_url1
          set_url2($1.to_s.dup) # Here we change the "homepage" URL.
          dataset_from_the_description_file =~ regex_for_license
          set_license($1.to_s.dup)
          dataset_from_the_description_file =~ regex_for_description
          set_description($1.to_s.dup)
          dataset_from_the_description_file =~ regex_for_short_description
          set_short_desc($1.to_s.dup)
          set_short_name(program_name)
          set_real_short_name(program_name)
          # ================================================================= #
          # The following code is probably no longer needed, but we can
          # keep it for now - maybe one day we have to use it.
          #   dataset_from_the_description_file =~ regex_for_name
          #   this_program = $1.to_s.dup.downcase
          #   set_this_program(this_program)
          # ================================================================= #
          attach_program_name_and_program_version
          determine_program_full_name # This is like: 'gnome-mahjonng-5.08.tar.xz'
          assemble_program_path
          unless File.exist?(File.basename(program_full_name?))
            e 'As the program does not exist locally, we will download it from:'
            e
            e '  '+sfancy(url?.to_s)
            e
            download_this_remote_program(url1)
          end
          # ================================================================= #
          # Now we assume that the program must exist locally.
          # ================================================================= #
          opnn; e 'The following information is known about this program:'
          cliner
          e main_eight? # Show some more information.
          cliner
          target = File.basename(program_full_name?)
          extract_this_archive(target, return_pwd)
          new_target = dataset_program_name?
          opnn; cd new_target, :be_verbose
          do_use_appdir_prefix(:be_quiet)
          use_default_configure_command
          # ================================================================= #
          # Determine the configure options.
          # ================================================================= #
          use_this_regex = /^configure_options=\(([\s\S]+)\)$/ # See: http://rubular.com/r/Zy29SMPvLm
          set_configure_options(
            recipe_dataset.scan(use_this_regex).flatten.first
          )
          set_configure_base_dir('')
          set_use_this_build_directory('')
          do_not_remove_the_extracted_archive
          # ================================================================= #
          # Run "configure" next.
          # ================================================================= #
          run_configure_via_system(nil, :do_not_change_the_directory)
          run_premake_make_then_make_install
          run_all_actions_after_the_make_install_step
        end
      end
    end
  end

  # ========================================================================= #
  # === generate_gobolinux_recipe
  #
  # This will delegate towards Cookbooks and generate  gobolinux recipe
  # for the given program.
  #
  # Invocation example:
  #   ry zenity --gobolinux-recipe
  # ========================================================================= #
  def generate_gobolinux_recipe(i = compile_which_program?)
    RBT.create_gobolinux_recipe(i) { :store_in_alphanumeric_subdirectory }
  end; alias create_gobolinux_recipe_for generate_gobolinux_recipe # === create_gobolinux_recipe_for

end; end