#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/ldconfig.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === run_ldconfig?
  # ========================================================================= #
  def run_ldconfig?
    RBT.configuration?.run_ldconfig
  end

  # ========================================================================= #
  # === do_not_use_ldconfig
  #
  # Use only this method when you wish to disable running ldconfig.
  #
  # To set this, do something like:
  #   ry htop do_not_use_ldconfig
  # ========================================================================= #
  def do_not_use_ldconfig(
      be_verbose = be_verbose?
    )
    case be_verbose
    when :do_not_be_verbose, :be_silent
      be_verbose = false
    end
    if be_verbose
      opne "We will not run `#{teal('ldconfig')}`."
    end
    RBT.configuration?.run_ldconfig = false
  end

  # ========================================================================= #
  # === do_not_run_ldconfig
  # ========================================================================= #
  def do_not_run_ldconfig
    do_not_use_ldconfig :be_silent
  end

  # ========================================================================= #
  # === consider_to_run_ldconfig                               (ldconfig tag)
  #
  # Run 'ldconfig' here in this method.
  #
  # Check whether we are allowed to invoke this method before calling it
  # however.
  # ========================================================================= #
  def consider_to_run_ldconfig
    if continue_after_make_step? # Makes only sense if we are past the make-step.
      # ===================================================================== #
      # ruby and python add-ons normally do not require ldconfig,
      # so we don't have to invoke it.
      # ===================================================================== #
      if tags?.include?('ruby') or
         tags?.include?('python')
        do_not_run_ldconfig
      end
      # ===================================================================== #
      # Only run it if the config-file tells us to do so and only for
      # those programs that have to be compiled.
      #
      # class RBT::Compile will, however had, NOT run ldconfig when the
      # program has failed to install - that is specifically if an
      # error has occurred.
      # ===================================================================== #
      if run_ldconfig? and
         has_to_be_compiled? and
         can_we_continue? and
        !a_problem_has_occurred? and # Also check for no problem.
        !an_error_has_occurred?
        # =================================================================== #
        # Build up the command to run next.
        # =================================================================== #
        cmd_to_run = 'ldconfig'.dup
        # =================================================================== #
        # We could be verbose, but it is not that useful, so uncomment it.
        # cmd_to_run << ' -v'
        # =================================================================== #
        # Since as of July 2011, we will be quieter and ignore whether
        # ldconfig exists or not.
        # =================================================================== #
        cmd_to_run << ' 2>&1'
        # =================================================================== #
        # Check for superuser privilege next. ldconfig requires the
        # superuser.
        # =================================================================== #
        if is_superuser?
          opne "Now running `#{sfancy(cmd_to_run)}`." # We output the ldconfig command variant here.
          system cmd_to_run
        else
          opne "You are not the superuser so we will not "\
               "try to run `#{sfancy(cmd_to_run)}`."
        end
      end
    end
  end; alias consider_running_ldconfig consider_to_run_ldconfig # === consider_running_ldconfig

end; end