#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/libtool.rb'
# =========================================================================== #
require 'rbt/libtool/remove_libtool_files.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === delete_libtool_files?
  #
  # Query whether we will delete .la files or whether we will not. If we
  # will not, then any installed .la files will be removed.
  #
  # Note that this will ONLY be honoured if compilation was a success
  # AND we use an AppDir path as prefix - so .la files that are installed
  # under /usr/lib/ will not be removed, ever.
  # ========================================================================= #
  def delete_libtool_files?
    if @internal_hash[:keep_la_files]
      !@internal_hash[:keep_la_files]
    else
      RBT.configuration?.delete_libtool_files
    end
  end; alias remove_la_files_after_compilation? delete_libtool_files? # === remove_la_files_after_compilation?

  # ========================================================================= #
  # === try_to_autocorrect_this_erroneous_libtool_entry
  #
  # This method will delegate towards class RBT::Libtool in order to
  # fix a faulty .la file.
  # ========================================================================= #
  def try_to_autocorrect_this_erroneous_libtool_entry(i)
    require 'rbt/libtool/class_methods/autocorrect_this_file.rb'
    opnn; e 'Now delegating towards class '+slateblue('RBT::Libtool')+
            ' to attempt to autocorrect the problem:'
    opnn; e "  #{sfancy(i)}"
    RBT::Libtool.autocorrect_this_file(i)
    # ======================================================================= #
    # The next subsection can re-compile again. We do so at maximum
    # 5 times.
    # ======================================================================= #
    if is_on_roebe? and try_to_autofix? and compiled_n_times? < 5
      increment_compiled_counter # Add +1 to the times we have tried to compile.
      clear_problems
      compile_this_program_again(program?)
    end
  end

  # ========================================================================= #
  # === fix_libtool_la_file_from_saved_file
  # ========================================================================= #
  def fix_libtool_la_file_from_saved_file
    RBT.fix_libtool_la_file_from_saved_file
  end

  # ========================================================================= #
  # === try_to_autocorrect_this_erroneous_libtool_entry_later
  #
  # Use this method to register an erroneous libtool entry. It will
  # be fixed at a later time during the regular compile-cycle.
  # ========================================================================= #
  def try_to_autocorrect_this_erroneous_libtool_entry_later(i)
    # ======================================================================= #
    # Store this faulty .la entry into a local file as well, next. This
    # was added in September 2018.
    # ======================================================================= #
    store_this_as_last_libtool_file(i)
    @internal_hash[:invalid_libtool_archive] = i # This is the libtool-entry that should go away.
    opnn; e 'Note that you can try to correct this error via:'
    e
    e orange('  rbt --fix-libtool-from-file')
    e
  end

  # ========================================================================= #
  # === run_through_the_erroneous_libtool_entries
  #
  # The first argument should be an Array; or an object that is able to
  # respond to :each.
  # ========================================================================= #
  def run_through_the_erroneous_libtool_entries(
      i = erroneous_libtool_entries?
    ) 
    i.each {|do_this_action|
      if is_on_roebe? and autofix?
        # =================================================================== #
        # Register this erroneous line for now - the fixing part will
        # happen at a later time. Only on roebe-systems for now, though.
        # =================================================================== #
        try_to_autocorrect_this_erroneous_libtool_entry_later(do_this_action)
      end
    }
  end

  # ========================================================================= #
  # === registered_erroneous_libtool_entries?
  # ========================================================================= #
  def registered_erroneous_libtool_entries?
    RBT::Errors::MapLineToASpecificError.erroneous_libtool_entries?
  end; alias erroneous_libtool_entries? registered_erroneous_libtool_entries? # === erroneous_libtool_entries?

  # ========================================================================= #
  # === do_keep_la_files
  #
  # Use this method if you wish to keep the libtool .la files. This may
  # sometimes be necessary, even though libtool .la files are a fairly
  # bad solution altogether.
  # ========================================================================= #
  def do_keep_la_files
    @internal_hash[:keep_la_files] = true
  end

  # ========================================================================= #
  # === do_remove_la_files_after_compilation
  # ========================================================================= #
  def do_remove_la_files_after_compilation
    @internal_hash[:keep_la_files] = false
  end

  # ========================================================================= #
  # === store_this_as_last_libtool_file
  #
  # We will remove "' or unhandled argument" if it is part of the
  # given input at hand.
  # ========================================================================= #
  def store_this_as_last_libtool_file(what)
    # ======================================================================= #
    # First sanitize the input a little bit:
    # ======================================================================= #
    if what.include? "' or unhandled argument"
      what = what.split(/' or unhandled argument/).first
    end
    unless what.empty?
      base_dir = "#{rbt_log_directory?}libtool/"
      mkdir(base_dir) unless File.directory? base_dir
      into = "#{base_dir}last_faulty_libtool_file.md"
      if is_on_roebe?
        if File.exist?(into) and File.read(into).strip.end_with? what
          # =================================================================== #
          # In this case, we will not store the same result again.
          # =================================================================== #
          return
        end
        opnn; e 'Also storing `'+royalblue(what)+'` into `'+sfile(into)+'`.'
      end 
      write_what_into(what, into)
    end
  end

end; end