#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/make.rb'
# =========================================================================== #
require 'rbt/utility_scripts/change_prefix.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === run_make?
  # ========================================================================= #
  def run_make?
    _ = cookbook_dataset_run_make?
    unless @internal_hash[:run_make_command].nil?
      _ = @internal_hash[:run_make_command]
    end
    _
  end

  # ========================================================================= #
  # === run_make_command                             (make tag, Makefile tag)
  #
  # This method will, in the end, delegate towards RBT.colour_make,
  # which in turn runs the command/binary called "make".
  #
  # The specific command, in this case "make", can also be a different
  # one, as defined by the respective .yml file for the given program
  # at hand.
  #
  # The method will also have to ensure that a Makefile-file exists, but
  # only for installation types that do happen to require a Makefile.
  #
  # Some build systems, such as meson/ninja, may use 'make', but may
  # also skip it and do "ninja install" instead.
  # ========================================================================= #
  def run_make_command
    if exit_before_make?
      opne 'Exiting now as instructed before reaching '\
           'the "'+orange('make')+'" stage.'
      exit_program
    end
    if modify_the_makefile?
      opnn; e 'The Makefile will be modified next (PREFIX value will be changed).'
      do_use_this_makefile_here = return_pwd+'Makefile'
      RBT::ChangePrefix.new(
        use_this_makefile:          do_use_this_makefile_here,
        use_this_as_the_new_prefix: prefix?.to_s.dup
      )
    end
    if run_make?
      # ===================================================================== #
      # Clear the old buffer first:
      # ===================================================================== #
      reset_error_map_line_to_a_specific_error
      # ===================================================================== #
      # "make" is usually used, but we may also make use of "ninja" instead,
      # so we have to pass in this option towards RBT.colour_make.
      # ===================================================================== #
      if run_simulation?
        opne 'We are running a simulation, hence the following command'
        opne 'will not be run:'
        opn; efancy "  #{make_command?}"
      else
        if has_to_be_compiled?
          consider_using_maximum_speed_for_compilation
        end
        # =================================================================== #
        # We will have to make a cd-operation to the configure base directory
        # if run_configure is no/false and if we use a build directory.
        # This, however had, breaks cmake builds, so it was disabled in
        # February 2019. It may have to be changed at a later time
        # anyway.
        # =================================================================== #
        # if !run_configure? and use_build_directory?
        #   cd_to_the_configure_base_directory
        # end
        # =================================================================== #
        # Next consider running pre-make commands - but this must come
        # AFTER running GNU configure and similar build-style runs.
        # =================================================================== #
        consider_running_premake_instructions
        unless installation_steps?.empty?
          installation_steps?.each {|this_step|
            opnn; e "Next doing this step: #{sfancy(this_step)}"
            esystem this_step
          }
        end
        # =================================================================== #
        # Delegate towards RBT.colour_make next:
        # =================================================================== #
        RBT.colour_make { return_default_hash_to_be_passed_into_colour_make }
      end
      run_through_the_registered_problems_registered_errors_and_behaviour_changes
      reset_error_map_line_to_a_specific_error
      if automatically_create_package?
        if is_on_slackware?
          # ================================================================= #
          # Create a slackware-package in this case.
          # ================================================================= #
          opne 'Next trying to create a package for `'+royalblue('slackware')+'`.'
          # The following subsection is incomplete. The path could be used
          # to find out whether things are put in their right position.
          # path = create_slackware_description_file
          RBT.run_makepkg_command(compile_which_program?, program_version?)
        end
      end
    end
  end; alias run_make                                    run_make_command # === run_make
       alias considering_running_make_command_via_system run_make_command # === considering_running_make_command_via_system

  # ========================================================================= #
  # === do_not_run_make
  # ========================================================================= #
  def do_not_run_make
    @internal_hash[:run_make_command] = false
  end

  # ========================================================================= #
  # === set_parameters_to_make
  #
  # If we have to pass specials options to the program called "make", then
  # we can do so via this setter-method.
  #
  # These are additional options that can be given to the program called
  # "make".
  # ========================================================================= #
  def set_parameters_to_make(i)
    # ======================================================================= #
    # The parameters must always be kept as an Array.
    # ======================================================================= #
    i = [i].flatten.compact
    @internal_hash[:parameters_to_make] = i # Do not change them to a String via .to_s
  end; alias set_make_options set_parameters_to_make # === set_make_options

  # ========================================================================= #
  # === parameters_to_make?
  #
  # The aliased method to this method, called make_options?, is mildly
  # discouraged after 12.04.2019. The primary reason for this is that
  # the name parameters_to_make? is more explicit, and thus better in
  # the long run (less confusing).
  # ========================================================================= #
  def parameters_to_make?
    _ = @internal_hash[:parameters_to_make]
    if _.nil?
      _ = cookbook_dataset_parameters_to_make?
    end
    _
  end; alias make_options? parameters_to_make? # === make_options?

  # ========================================================================= #
  # === append_make_options
  #
  # This will append parameters issued to "make". It will also start
  # with a ' ', always.
  # ========================================================================= #
  def append_make_options(i)
    @make.append_make_options(i)
    append_this = i.to_s.dup
    if make_options?
      joined_make_options = make_options?
      if joined_make_options.respond_to? :join
        joined_make_options = joined_make_options.join(' ').strip
      end
      unless append_this.include? joined_make_options
        append_this.prepend(joined_make_options.dup)
      end
    end
    # ======================================================================= #
    # Prepend a ' ' for some spacing.
    # ======================================================================= #
    unless append_this.empty?
      append_this.prepend(' ') unless append_this.start_with? ' '
      set_make_options(append_this)
    end
  end; alias append_make_option        append_make_options # === append_make_option
       alias append_parameters_to_make append_make_options # === append_parameters_to_make

  # ========================================================================= #
  # === use_this_make_command?
  #
  # This is most commonly just "make" - but it may also be "ninja" or
  # another build system.
  # ========================================================================= #
  def use_this_make_command?
    _ = 'make' # Just some default.
    _ = cookbook_dataset_make_command? if cookbook_dataset_make_command?
    unless @internal_hash[:use_this_make_command].nil?
      _ = @internal_hash[:use_this_make_command]
    end
    _ # And return it here.
  end; alias use_which_make_command? use_this_make_command? # === use_which_make_command?

  # ========================================================================= #
  # === run_premake_commands
  #
  # Here we try to run the premake commands, in case they exist.
  #
  # If there are no pre_make_commands, then we will not do anything,
  # as our array will be empty.
  #
  # This method is normally run from do_compile_this_program(), via
  # a call to its alias name consider_running_premake_instruction().
  # ========================================================================= #
  def run_premake_commands(
      array = cookbook_dataset_pre_make_commands?
    )
    unless array.empty?
      opne "#{sfancy('Doing pre_make_commands next from ')}"\
           "#{sdir_return_pwd}#{sfancy('.')}"
      array.each { |cmd|
        opnn
        e "In the directory #{sdir_return_pwd}, doing the following command next:"
        e
        e "  #{slateblue(cmd)}"
        e
        e 'next.'
        custom_system(cmd)
      }
    end
  end; alias consider_running_premake_instructions run_premake_commands # === consider_running_premake_instructions

  # ========================================================================= #
  # === make_file_exists?
  #
  # Determine whether a "Makefile" exists or whether it does not exist.
  # ========================================================================= #
  def make_file_exists?
    File.exist? 'Makefile'
  end; alias make_file? make_file_exists? # === make_file?

  # ========================================================================= #
  # === continue_after_make_step?
  # ========================================================================= #
  def continue_after_make_step?
    @internal_hash[:continue_after_make_step]
  end

  # ========================================================================= #
  # === has_makefile?
  #
  # Query whether we have a file called Makefile in the current working
  # directory.
  # ========================================================================= #
  def has_makefile?(i = 'Makefile') 
    File.exist?(i)
  end

  # ========================================================================= #
  # === has_linux_makefile?
  #
  # Return true if we find a linux-specific Makefile.
  # ========================================================================= #
  def has_linux_makefile?(i = 'Makefile.linux')
    File.exist?(i)
  end

  # ========================================================================= #
  # === is_make_available?
  #
  # This method will attempt to find out whether make is available. It will
  # do so by checking on the path.
  # ========================================================================= #
  def is_make_available?
    is_make_available = false
    path = query_path?
    if path.include? ':'
      check_these_directories = path.split(':')
      check_these_directories.each {|dir|
        if File.exist?(dir+'/make')
          is_make_available = true # Yup, make is available.
        end
      }
    end
    return is_make_available
  end

  # ========================================================================= #
  # === make_command?
  #
  # Tell us which make command we will be using.
  # ========================================================================= #
  def make_command?
    @internal_hash[:make_command]
  end

  # ========================================================================= #
  # === set_make_command
  #
  # This method can be used to set which make command is to be used.
  # ========================================================================= #
  def set_make_command(
      i = 'make'
    )
    i = 'make' if i.nil? # Set to the default in this case.
    @internal_hash[:make_command] = i
  end

  # ========================================================================= #
  # === run_premake_make_then_make_install
  #
  # Issue "premake commands", then "make" and then "make install" commands.
  # ========================================================================= #
  def run_premake_make_then_make_install
    if run_make?
      run_premake_commands
      run_make_then_make_install
    end
  end; private :run_premake_make_then_make_install

  # ========================================================================= #
  # === run_make_then_make_install
  #
  # Run "make" and "make install" command here.
  # ========================================================================= #
  def run_make_then_make_install
    run_make_command
    run_make_install_command
  end; alias run_make_and_make_install_command run_make_then_make_install # === run_make_and_make_install_command

  # ========================================================================= #
  # === set_use_this_make_command
  #
  # This can be used to use another make command, such as "ninja".
  # ========================================================================= #
  def set_use_this_make_command(
      i = :default
    )
    case i
    when :default
      i = :make
    end
    i = i.to_s
    @internal_hash[:use_this_make_command] = i
  end

end; end