#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/make_install.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === run_make_install?
  #
  # Query whether we will do the "make install" step or whether we
  # will not. Will return a boolean value (true/false).
  # ========================================================================= #
  def run_make_install?
    @internal_hash[:run_make_install]
  end; alias may_we_run_make_install_command? run_make_install? # == may_we_run_make_install_command?

  # ========================================================================= #
  # === run_make_install_command
  #
  # This method will ultimately default to run something such
  # as "make install". Note that some programs do not use
  # "make install" and instead use something such as "ninja install".
  #
  # The method here thus has to remain somewhat flexible, in order to
  # install such programs as well.
  # ========================================================================= #
  def run_make_install_command
    # ======================================================================= #
    # Do not try to compile via make, if it does not have to be compiled.
    # ======================================================================= #
    return unless has_to_be_compiled?
    unless pre_make_install_sed.empty?
      pre_make_install_sed.each {|sed_instruction|
        run_this_sed_instruction(sed_instruction)
      }
    end
    if exit_before_make_install?
      opne 'Exiting now as instructed before reaching '\
           'the "'+orange('make install')+'" stage.'
      exit_program
    end
    if run_make_install?
      # ===================================================================== #
      # Clear the old buffer keeping track of our errors first:
      # ===================================================================== #
      reset_error_map_line_to_a_specific_error
      use_this_make_command = use_this_make_command?
      if use_this_make_install_command?
        # =================================================================== #
        # In this case we will use that make-install command instead.
        # =================================================================== #
        use_this_make_command = use_this_make_install_command?
      end
      use_porg = false
      if use_porg?
        if porg_is_available?
          use_porg = true
        else
          # ================================================================= #
          # Notify the user that porg is NOT available. We will continue
          # anyway, though.
          # ================================================================= #
          notify_the_user_that_porg_is_not_available
        end
      end
      if use_stow?
        RBT.colour_make_install {
          return_default_hash_to_be_passed_into_colour_make(
            append: " DESTDIR=/stow/#{compile_which_program?} install",
            porg:                  use_porg,
            use_this_make_command: use_this_make_command
          )
        }
      else
        RBT.colour_make_install {
          return_default_hash_to_be_passed_into_colour_make(
            porg:                  use_porg,
            use_this_make_command: use_this_make_command
          )
        }
      end
      run_through_the_registered_problems_registered_errors_and_behaviour_changes
      reset_error_map_line_to_a_specific_error
      if an_error_has_occurred?
        # =================================================================== #
        # Register that this step led to an error in this case.
        # =================================================================== #
        @internal_hash[:make_install_stage_was_a_success] = false
      end
      if !run_configure? and use_build_directory?
        cd_to_the_build_directory
      end
    end
  end; alias run_make_install                                    run_make_install_command # === run_make_install
       alias considering_running_make_install_command_via_system run_make_install_command # === considering_running_make_install_command_via_system

  # ========================================================================= #
  # === do_not_run_make_install
  #
  # Use this method if you wish to specifically skipping the "make install"
  # step.
  # ========================================================================= #
  def do_not_run_make_install(
      be_verbose = false
    )
    @internal_hash[:run_make_install] = false
    if be_verbose
      opnn; e 'We will not run the "'+slateblue('make install')+'" command.'
    end
    # ======================================================================= #
    # Running ldconfig seems pointless, so it is also disabled (silently).
    # ======================================================================= #
    do_not_use_ldconfig(:do_not_be_verbose)
  end; alias dont_make_install do_not_run_make_install # === dont_make_install

end; end