#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Bundle together meson and ninja-related code in this file here.
# =========================================================================== #
# require 'rbt/compile/meson_and_ninja.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === do_use_meson
  #
  # This method will allow us to use meson. It will, on roebe systems,
  # also try to use Python 3 if it is not the main one in use.
  # ========================================================================= #
  def do_use_meson
    use_utf8_encoding_in_env
    set_use_this_configure_command(:meson)
    set_use_this_build_directory('build')
    consider_enabling_python3
  end

  # ========================================================================= #
  # === do_use_ninja_as_the_build_system
  # ========================================================================= #
  def do_use_ninja_as_the_build_system
    set_use_this_make_command(:ninja)
  end

  # ========================================================================= #
  # === do_use_meson_and_ninja
  # ========================================================================= #
  def do_use_meson_and_ninja
    do_use_meson
    do_use_ninja_as_the_build_system
  end

  # ========================================================================= #
  # === meson_build_file_exists?
  #
  # When we check for the file called "meson.build", we must also
  # keep in mind that we may use a build directory. This is the reason
  # why the following code uses a variable.
  # ========================================================================= #
  def meson_build_file_exists?
    check_for_this_file = 'meson.build'
    # ======================================================================= #
    # Next check whether we do use a build directory or whether we do not.
    # ======================================================================= #
    if use_build_directory?
      check_for_this_file = check_for_this_file.dup if check_for_this_file.frozen?
      check_for_this_file.prepend('../')
    end
    does_this_file_exist?(check_for_this_file)
  end

  # ========================================================================= #
  # === depends_on_ninja?
  # ========================================================================= #
  def depends_on_ninja?
    ::RBT.depends_on_ninja?(required_deps_on?)
  end

  # ========================================================================= #
  # === is_meson_build_type?
  #
  # This method will return true if we can use meson in the current
  # directory; and false otherwise.
  # ========================================================================= #
  def is_meson_build_type?
    RBT.build_type? == :meson
  end; alias uses_meson? is_meson_build_type? # === uses_meson?

  # ========================================================================= #
  # === uses_the_meson_build_system?
  # ========================================================================= #
  def uses_the_meson_build_system?
    use_this_build_system? == 'meson'
  end

  # ========================================================================= #
  # === do_not_run_ninja
  # ========================================================================= #
  def do_not_run_ninja
    @internal_hash[:may_we_run_ninja] = false 
  end

  # ========================================================================= #
  # === may_we_run_ninja?
  # ========================================================================= #
  def may_we_run_ninja?
    @internal_hash[:may_we_run_ninja]
  end

  # ========================================================================= #
  # === ninja_install
  # ========================================================================= #
  def ninja_install
    this_file = 'build.ninja'
    if may_we_run_ninja?
      if File.exist?(this_file)
        # =================================================================== #
        # === ninja
        # =================================================================== #
        e teal('ninja')
        reset_error_map_line_to_a_specific_error
        run_this_system_command 'ninja' # <- Run the sys-command here.
        run_through_the_registered_problems_registered_errors_and_behaviour_changes
        reset_error_map_line_to_a_specific_error
        # =================================================================== #
        # === ninja install
        # =================================================================== #
        e teal('ninja install')
        reset_error_map_line_to_a_specific_error
        run_this_system_command 'ninja install' # <- Run the sys-command here.
        run_through_the_registered_problems_registered_errors_and_behaviour_changes
        reset_error_map_line_to_a_specific_error
      else
        opne "No file called #{sfile(this_file)} could be found - thus,"
        opne "the build tool called #{orange('ninja')} can not be run."
        # =================================================================== #
        # Register this error as well:
        # =================================================================== #
        error_is :could_not_find_file_called_build_ninja
      end
    end
  end; alias ninjait   ninja_install # === ninjait
       alias run_ninja ninja_install # === run_ninja

end; end