#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/misc.rb'
# =========================================================================== #
require 'rbt/compile/set_compile_these_programs.rb'
require 'rbt/compile/set.rb'
require 'rbt/compile/tags.rb'
require 'rbt/compile/version.rb'
require 'rbt/misc/create_and_populate_pkgconfig_directory.rb'
require 'rbt/toplevel_methods/return_blfs_page_of_this_program.rb'
require 'rbt/toplevel_methods/check_for_binary_duplicates.rb'
# =========================================================================== #
# Next load up some important rbt/utility_scripts/ programs:
# =========================================================================== #
require 'rbt/utility_scripts/all_urls.rb'
require 'rbt/utility_scripts/build_detector.rb'
require 'rbt/utility_scripts/expand_cookbooks.rb'
require 'rbt/utility_scripts/purge_libraries_of_this_program.rb'
require 'rbt/utility_scripts/suggest_cookbook_for.rb'
require 'rbt/utility_scripts/lfs/lfs.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === return_default_hash_to_be_passed_into_colour_make
  # ========================================================================= #
  def return_default_hash_to_be_passed_into_colour_make(
      optional_input = nil
    )
    hash = {
      use_this_make_command:            use_which_make_command?,
      append_these_options:             parameters_to_make?,
      run_make:                         run_make?,
      run_make_install:                 run_make_install?,
      use_porg:                         use_porg?,
      program_name_and_program_version: program_name_and_program_version?
    }
    if optional_input
      hash.update(optional_input)
    end
    hash 
  end

  # ========================================================================= #
  # === consider_running_autoconf_then_run_configure_then_make_then_make_install
  # ========================================================================= #
  def consider_running_autoconf_then_run_configure_then_make_then_make_install
    consider_running_the_pre_configure_steps
    decide_whether_to_invoke_the_autoconf_command
    run_configure_then_make_then_make_install
  end

  # ========================================================================= #
  # === consider_creating_appdir_skeleton_then_consider_running_autoconf_then_run_configure_then_make_then_make_install
  # ========================================================================= #
  def consider_creating_appdir_skeleton_then_consider_running_autoconf_then_run_configure_then_make_then_make_install
    consider_creating_appdir_skeleton
    consider_running_autoconf_then_run_configure_then_make_then_make_install
  end

  # ========================================================================= #
  # === make_object?
  #
  # Query-method towards the @make object (aka RBT::Make).
  # ========================================================================= #
  def make_object?
    @make
  end

  # ========================================================================= #
  # === try_to_use_this_profile
  #
  # The .tr() operation in the method body is valid, because we
  # will only store the filenames in a lowercased variant, such
  # as "gcc_minimal.md" rather than "gcc-minimal.md".
  #
  # Usage example:
  #
  #   ry gcc --profile=gcc_lfs_pass1
  #   ry gcc --profile=gcc_minimal
  #
  # ========================================================================= #
  def try_to_use_this_profile(i)
    i = i.to_s.dup.tr('-','_')
    if i.start_with? '-' # This is always a mistake, for the time being, so remove it.
      i[/^-+/] = ''
    elsif i.start_with? '_' # This is always a mistake, for the time being, so remove it.
      i[/^_+/] = ''
    end
    case i
    # ======================================================================= #
    # === gcc-minimal-but-lots-of-languages
    #
    # To use this, try:
    #
    #   ry gcc trad --profile=gccp1
    #
    # ======================================================================= #
    when /^gcc(-|_)?minimal(-|_)?but(-|_)?lots(-|_)?of(-|_)?languages$/i,
         /^gcc(-|_)?minimal(-|_)?but(-|_)?lots(-|_)?of(-|_)?languages(-|_)?enabled$/i,
         /^language(-|_)?rich$/i,
         'gccp1',
         'gcc1'
      i = 'gcc_minimal_but_lots_of_languages_enabled'
      if no_cookbook_dataset?
        set_really_compile_this_program(:gcc)
      end
    # ======================================================================= #
    # === qt
    # ======================================================================= #
    when 'qt'
      if no_cookbook_dataset?
        set_really_compile_this_program(:qt)
      end
    # ======================================================================= #
    # === gcc_lfs_pass_1
    # ======================================================================= #
    when /^gcc(_|-)?lfs(_|-)?pass(_|-)?1$/
      i = 'gcc_lfs_pass1.md'
      if no_cookbook_dataset?
        set_really_compile_this_program(:gcc)
      end
    end
    i = i.dup if i.frozen?
    # ======================================================================= #
    # First we will have to try to find the proper profile:
    # ======================================================================= #
    i << '.md' unless i.end_with? '.md'
    target = DIRECTORY_RBT_PROFILES+i
    if File.exist? target
      opnn; e 'Making use of the profile found at this location:'
      opnn; e '  '+sfile(target)
      # ===================================================================== #
      # The file exists, so we can load it.
      # ===================================================================== #
      dataset = File.readlines(target).reject {|line|
        line.strip.start_with? '#' # Get rid of comments.
      }.join(' ').delete("\n").strip.squeeze(' ')
      clear_old_configure_options
      # ===================================================================== #
      # Store it in the internal hash next:
      # ===================================================================== #
      set_configure_options_on_internal_hash(dataset)
    else
      opnn; no_file_exists_at(target)
      opnn; e 'The current behaviour is that RBT will exit upon such an error -'
      opnn; e 'only existing profiles may be used.'
      e
      e 'The available profiles are:'
      e
      show_the_available_profiles_for_configure_options
      e
      exit
    end
  end

  # ========================================================================= #
  # === show_available_profiles
  #
  # This method will show the available (local) profiles.
  #
  # To invoke this method try:
  #
  #   rbt --show-available-profiles
  #
  # ========================================================================= #
  def show_available_profiles(
      target_directory = DIRECTORY_RBT_PROFILES
    )
    available_profiles = Dir[target_directory+'*.md'].map {|entry|
      File.basename(entry).sub(/.md$/,'')
    }
    if available_profiles.empty?
      opne 'No profiles could be found in the directory '+sdir(target_directory)+'.'
    else
      e 'The following profiles are available:'
      e
      available_profiles.each {|entry|
        e slateblue('  '+entry)
      }
      e
    end
  end

  # ========================================================================= #
  # === install_palemoon
  #
  # To invoke this ad-hoc method, try:
  #
  #   rbt --install-palemoon
  #
  # ========================================================================= #
  def install_palemoon(
      path = '/Users/x/SRC/palemoon/palemoon-'\
             '28.5.2-64bit.tar.xz'
    )
    if File.exist? path
      _ = remove_archive_from_the_end(
        File.basename(path)
      ).sub(/-64bit$/, '')
      program_name    = ProgramInformation.return_name(_)
      program_version = ProgramInformation.return_version(_)
      target = program_dir?+program_name.capitalize+'/'+program_version+'/'
      create_app_skeleton(target)
      cd target
      extract_to = target
      extract_what_to(path, extract_to)
      e 'Done!'
    else
      opnn; no_file_at(path)
    end
  end

  # ========================================================================= #
  # === set_really_compile_this_program
  # ========================================================================= #
  def set_really_compile_this_program(i = :qt)
    set_compile_this_program(i)
    set_compile_these_programs(i) if compile_these_programs?.empty?
    initialize_cookbook_dataset
  end

  # ========================================================================= #
  # === do_check_for_binary_duplicates
  #
  # If you wish to check for binary-duplicates under /usr/bin/, then
  # this method may be helpful to you.
  # ========================================================================= #
  def do_check_for_binary_duplicates
    target_dir = :default
    # ======================================================================= #
    # The next line of code handles the situation when we are in the
    # directory /usr/sbin/, and when we are on a roebe-system (aka at home).
    # ======================================================================= #
    if is_on_roebe? and (return_pwd == '/usr/sbin/')
      target_dir = '/usr/sbin/'
    end
    array = RBT.array_binary_duplicates(:default, target_dir)
    if array.empty?
      target_dir = '/usr/bin/' if target_dir == :default
      opne "No duplicates appear to exist under the "\
           "#{sdir(target_dir)} hierarchy."
    else
      opne 'The following duplicates were found:'
      e
      array.each {|entry|
        e "  #{sfile(entry.ljust(40))}"
      }
    end
  end

  # ========================================================================= #
  # === linux_from_scratch                                          (lfs tag)
  #
  # We simply delegate this task to the class LFS.
  # ========================================================================= #
  def linux_from_scratch
    RBT::LFS.new
  end

  # ========================================================================= #
  # === try_to_use_configure_options?
  # ========================================================================= #
  def try_to_use_configure_options?
    _ = true # True by default.
    unless @internal_hash[:try_to_use_configure_options].nil?
      _ = @internal_hash[:try_to_use_configure_options]
    end
    _
  end; alias use_configure_options? try_to_use_configure_options? # === use_configure_options?

  # ========================================================================= #
  # === show_the_available_profiles_for_configure_options
  # ========================================================================= #
  def show_the_available_profiles_for_configure_options
    Dir[DIRECTORY_RBT_PROFILES+'*.md'].each {|this_profile|
      e '  - '+royalblue(File.basename(this_profile)).sub(/\.md$/,'')
    }
  end

  # ========================================================================= #
  # === set_configure_options_on_internal_hash
  # ========================================================================= #
  def set_configure_options_on_internal_hash(i)
    @internal_hash[:configure_options] = i
  end

  # ========================================================================= #
  # === create_yaml_file_for_this_program
  # ========================================================================= #
  def create_yaml_file_for_this_program(
      i = compile_which_program?
    )
    require 'rbt/utility_scripts/expand_cookbooks.rb'
    e
    opnn; RBT.expand_this_cookbook(i)
    e
  end; alias expand_this_cookbook create_yaml_file_for_this_program # === expand_this_cookbook

  # ========================================================================= #
  # === do_expand_the_cookbooks
  # ========================================================================= #
  def do_expand_the_cookbooks
    ::RBT.expand_cookbooks(:all_of_them)
  end

  # ========================================================================= #
  # === register_all_protos_for_compilation
  #
  # Invoke this method like so:
  #
  #   ry --protos
  #
  # ========================================================================= #
  def register_all_protos_for_compilation
    proto_programs = available_programs?.select {|entry|
      entry.include?('proto')
    }
    proto_programs.reject! {|line|
      line.include? 'protobuf'
    }
    # ======================================================================= #
    # Reject those that are .gem files next - but this will only work if
    # the directory with expanded cookbooks is available:
    # ======================================================================= #
    if directory_expanded_cookbooks_exists?
      proto_programs.reject! {|name_of_the_program|
        this_file = directory_expanded_cookbooks+name_of_the_program+'.yml'
        if File.exist? this_file
          dataset = YAML.load_file(this_file)
          dataset['program_path'].end_with? '.gem'
        else
          e 'No file called '+sfile(this_file)+' was found.'
          nil
        end
      }
    end
    opne "We will next compile all "\
         "#{sfancy(proto_programs.size.to_s)}"\
         " proto-programs."
    set_compile_these_programs(proto_programs)
  end

  # ========================================================================= #
  # === report_whether_porg_is_available
  # ========================================================================= #
  def report_whether_porg_is_available
    e 'Is porg available? '+
      sfancy(verbose_truth(is_porg_available?))
  end

  # ========================================================================= #
  # === do_not_initialize_a_new_cookbook_dataset
  # ========================================================================= #
  def do_not_initialize_a_new_cookbook_dataset
    @internal_hash[:may_we_initialize_a_new_cookbook_dataset] = false
  end

  # ========================================================================= #
  # === may_we_initialize_a_new_cookbook_dataset?
  # ========================================================================= #
  def may_we_initialize_a_new_cookbook_dataset?
    @internal_hash[:may_we_initialize_a_new_cookbook_dataset]
  end

  # ========================================================================= #
  # === instantiate_a_new_cookbook_instance_for_this_program
  #
  # Since as of January 2018, whenever you need an instance from
  # class RBT::Cookbooks::Cookbook, this method here should be
  # used.
  #
  # The reason is simple:
  # - This method will do all the required invocations needed
  #   to properly instantiate a new Cookbook instance.
  # ========================================================================= #
  def instantiate_a_new_cookbook_instance_for_this_program(i)
    # ======================================================================= #
    # Next, instantiate a new Cookbooks::Cookbook instance.
    # ======================================================================= #
    RBT::Cookbooks::Cookbook.new(i) {{
      use_menu_interface: :bypass_menu_check,
      make_use_of_aliases_kept_in_the_file_cookbook_aliases: make_use_of_cookbook_aliases?
    }}
  end; alias instantiate_a_new_cookbook_instance instantiate_a_new_cookbook_instance_for_this_program # === instantiate_a_new_cookbook_instance
       alias instantiate_a_new_cookbook          instantiate_a_new_cookbook_instance_for_this_program # === instantiate_a_new_cookbook
       alias instantiate_a_new_cookbook_for      instantiate_a_new_cookbook_instance_for_this_program # === instantiate_a_new_cookbook_for
       alias create_a_new_cookbook_instance      instantiate_a_new_cookbook_instance_for_this_program # === create_a_new_cookbook_instance

  # ========================================================================= #
  # === reuse_old_configure_options
  #
  # This method currently works only with the target program being GCC.
  #
  # Perhaps it may be extended for other programs one day.
  #
  # Invocation examples:
  #
  #   ry gcc --copy-old-version
  #   ry gcc --use-this-version=7.1.0 --use-old-configure-options
  #
  # ========================================================================= #
  def reuse_old_configure_options(
      use_this_program = compile_which_program?
    )
    _ = `#{use_this_program} -v 2>&1`
    options = _.scan(/^Configured .+/).first.
                sub(/^Configured with:/,'').
                sub(/\.\.\/configure /,''). 
                strip
    opne "Re-using the old configure option, which "\
         "was `#{sfancy(options)}`."
    if options.include? programs_directory?
      # ===================================================================== #
      # Next, replace the oldprefix.
      # ===================================================================== #
      programs_dir = Regexp.quote(programs_directory?)
      if dataset?.nil?
        ensure_that_the_default_dataset_exists_for_this_program(use_this_program)
      end
      if @internal_hash[:use_this_specific_program_version]
        dataset?.set_program_version(@internal_hash[:use_this_specific_program_version])
        dataset?.reassemble_program_path
      end
      regex_to_use = /(--prefix=#{programs_dir}Gcc\/\d{1,2}\.\d{1,2}\.\d{1,2})/
      options.sub!(regex_to_use, '--prefix='+return_appdir_prefix)
      options.squeeze!('/')
      # ===================================================================== #
      # Now, simply use that as configure version.
      # ===================================================================== #
      extract_to_default_directory
      set_configure_base_directory
      cd_to_the_configure_base_directory
      run_preinstall_sed_and_currently_being_compiled
      consider_creating_the_build_directory
      e "These are the #{orange('options')} that will be used:"
      e
      e "  #{simp(options)}"
      e
      set_configure_options(options)
      run_configure_make_and_make_install_with_this_configure_line(options)
      run_all_actions_after_the_make_install_step
      exit_program
    end
  end; alias use_old_configure_options reuse_old_configure_options # === use_old_configure_options

  # ========================================================================= #
  # === compiler_to_use?
  # ========================================================================= #
  def compiler_to_use?
    @internal_hash[:compiler_to_use]
  end

  # ========================================================================= #
  # === automatically_infer_program_name_and_program_version_from_this_input
  #
  # This method will try to automatically set the program_name and
  # @program_version variables.
  # ========================================================================= #
  def automatically_infer_program_name_and_program_version_from_this_input(
      i,
      optional_arguments = nil
    )
    if compile_which_program?.nil?
      set_program_name(
        ProgramInformation.return_program_real_name(i)
      )
    end
    if program_version?.nil?
      set_program_version(
        ProgramInformation.return_program_version(i)
      )
    end
    case optional_arguments
    when :overwrite_old_values_should_they_exist
      set_program_name ProgramInformation.return_program_real_name(i)
      set_program_version ProgramInformation.return_program_version(i)
    end
    @program_name_and_program_version = compile_which_program?+'-'+program_version?
  end

  # ========================================================================= #
  # === program_name_and_program_version?
  # ========================================================================= #
  def program_name_and_program_version?
    @program_name_and_program_version
  end

  # ========================================================================= #
  # === return_alias_name_for
  # ========================================================================= #
  def return_alias_name_for(i)
    ::RBT::Cookbooks::Aliases.new(i).result?
  end

  # ========================================================================= #
  # === configure_script_exists?
  # ========================================================================= #
  def configure_script_exists?
    File.exist?('configure')
  end; alias configure_script? configure_script_exists? # === configure_script?

  # ========================================================================= #
  # === return_program_name_then_hyphen_then_the_program_version
  # ========================================================================= #
  def return_program_name_then_hyphen_then_the_program_version
    "#{compile_which_program?.to_s}-#{program_version?}"
  end

  # ========================================================================= #
  # === set_original_input
  #
  # This method can be used to keep track of the original input given
  # to RBT::Compile.
  # ========================================================================= #
  def set_original_input(i = nil)
    @internal_hash[:original_input] = i
  end

  # ========================================================================= #
  # === original_input?
  # ========================================================================= #
  def original_input?
    @internal_hash[:original_input]
  end; alias input? original_input? # === input?

  # ========================================================================= #
  # === full_configure_command?
  #
  # The @full_configure_command contains the full configure command
  # that we will use. The advantage in using this is that we can
  # easily copy/paste this at a later time, should we need to do so.
  # ========================================================================= #
  def full_configure_command?
    @internal_hash[:full_configure_command]
  end; alias return_full_configure_command full_configure_command? # === return_full_configure_command

  # ========================================================================= #
  # === program_information_return_name
  #
  # This method can be used to specifically return the short name of
  # a given input-String. That input-String usually is a remote URL
  # to some program.
  # ========================================================================= #
  def program_information_return_name(i)
    ProgramInformation.return_short_name(File.basename(i))
  end

  # ========================================================================= #
  # === consider_sanitizing_the_program_path
  #
  # This method will only be used when 
  # @internal_hash[:use_this_specific_program_version] has been set. 
  # ========================================================================= #
  def consider_sanitizing_the_program_path
    if @internal_hash[:use_this_specific_program_version]
      # ===================================================================== #
      # The above variable is nil by default, so if it has another value
      # we can use this.
      # ===================================================================== #
      splitted    = @internal_hash[:use_this_specific_program_version].split('/')
      last        = splitted.last # <- This will hold the new program version.
      old_value   = cookbook_dataset_program_path?.dup
      old_version = ProgramInformation.return_version(old_value)
      set_cookbook_dataset_program_version(
        last
      )
      old_value.sub!(/#{old_version}/, last)
      set_cookbook_dataset_program(
        remove_extension(
          File.basename(old_value)
        )
      )
      set_cookbook_dataset_program_path(old_value)
    end
  end

  # ========================================================================= #
  # === program_path?
  # ========================================================================= #
  def program_path?
    @internal_hash[:program_path]
  end; alias path? program_path? # === path?

  # ========================================================================= #
  # === stop_on_error
  #
  # Method call to stop if the user wanted to.
  # ========================================================================= #
  def stop_on_error(
      be_verbose = true
    )
    if be_verbose
      stderr return_opnn, :use_print
      stderr 'We will exit now because this behaviour is enabled'
      stderr return_opnn, :use_print
      stderr "in the configuration file (the entry: #{teal('stop_on_error')})."
    end
    exit_program if stop_on_error?
  end

  # ========================================================================= #
  # === try_to_automatically_setup_the_rbt_project
  #
  # This method could be used to automatically try and setup the
  # project on a default GoboLinux system.
  # ========================================================================= #
  def try_to_automatically_setup_the_rbt_project
    if RBT.is_this_a_real_gobolinux_system?
      opnn; e 'Found a local GoboLinux system.'
      opnn; e 'Setting up some settings to make use of this as our host system.'
      target_file = log_dir?+'is_on_gobolinux.md'
      write_what_into('1', target_file)
      opnn; e 'This will be stored into the file `'+sfile(target_file)+'`.'
    end
  end

  # ========================================================================= #
  # === set_use_this_build_system
  #
  # Consistently use this method when you modify the value at
  # @internal_hash[:use_this_build_system].
  # ========================================================================= #
  def set_use_this_build_system(i)
    i = i.to_sym unless i.is_a? Symbol
    @internal_hash[:use_this_build_system] = i
  end

  # ========================================================================= #
  # === use_configure_as_build_system
  # ========================================================================= #
  def use_configure_as_build_system
    set_use_this_build_system(:configure)
  end

  # ========================================================================= #
  # === consider_displaying_the_start_message
  # ========================================================================= #  
  def consider_displaying_the_start_message
    _ = cookbook_dataset?.optional_start_message
    if _
      unless _.empty? 
        opne sfancy(_)
      end
    end
  end

  # ========================================================================= #
  # === return_possible_abbreviation_to_this_input
  #
  # This method will return an abbreviation to a given input.
  # ========================================================================= #
  def return_possible_abbreviation_to_this_input(i)
    # ======================================================================= #
    # If the cookbooks-project already includes the given input at hand
    # then we do not have to do anything else.
    # ======================================================================= #
    is_the_program_included = cookbooks_include_this_program? i
    unless is_the_program_included
      # =================================================================== #
      # We have to do another check, this time including the abbreviations.
      # =================================================================== #
      is_the_program_included = cookbooks_include_this_program?(
        i, :with_abbreviations
      )
      if is_the_program_included
        abbreviation_points_to = abbreviations?[i]
        unless abbreviation_points_to == i
          # =============================================================== #
          # Notify the user that we will "re-write" the original input
          # in this case.
          # =============================================================== #
          opnn; e "The given input #{sfancy(i)} could not be "\
                  "found, but an abbreviation"
          opnn; e "was found, which is now used as the new name: "\
                  "#{mediumspringgreen(abbreviation_points_to)}"
          i = abbreviation_points_to.to_s.delete('_').dup
          set_is_an_abbreviation
        end
      end
    end
    return i
  end

  # ========================================================================= #
  # === set_program_path
  # ========================================================================= #
  def set_program_path(i)
    @internal_hash[:program_path] = i
  end

  # ========================================================================= #
  # === determine_the_program_path
  # ========================================================================= #
  def determine_the_program_path
    value = source_directory?+
            yaml_dataset?.program_short_name?.downcase.delete('-')+'/'+
            yaml_dataset?.program_name_and_program_version?+
            yaml_dataset?.archive_type?
    set_program_path(value)
  end

  # ========================================================================= #
  # === create_a_slackdesc_file_for_this_program
  # ========================================================================= #
  def create_a_slackdesc_file_for_this_program(i)
    require 'rbt/linux/slackware/generate_slack_desc_file.rb'
    RBT::Linux::Slackware::GenerateSlackDescFile.new(i)
  end

  # ========================================================================= #
  # === use_which_build_system?
  # ========================================================================= #
  def use_which_build_system?
    _ = cookbookset_dataset_use_which_build_system?
    unless @internal_hash[:use_this_build_system].nil?
      _ = @internal_hash[:use_this_build_system]
    end
    _
  end; alias build_system? use_which_build_system? # === build_system?

  # ========================================================================= #
  # === consider_running_the_installation_steps_if_they_have_been_defined
  #
  # This method is especially necessary for rubygem related custom
  # installation steps (aka the file "rubygems.yml").
  # ========================================================================= #
  def consider_running_the_installation_steps_if_they_have_been_defined
    unless cookbook_installation_steps?.empty?
      _ = cookbook_installation_steps?
      _.each {|this_step|
        case this_step
        # =================================================================== #
        # On top of this case-when menu we should put entries that only
        # require a simple esystem() run.
        # =================================================================== #
        # =================================================================== #
        # === Python-based installations
        #
        # We can do /^python setup.py/ here and then pass it into
        # esystem() - but the problem is that a user may not have
        # a "python" binary installed. So this simplistic assumption
        # is not great. This is why we will use run_python_installation()
        # since as of 17.02.2019.
        # =================================================================== #
        when /^python setup.py/
          run_python_installation(this_step)
        # =================================================================== #
        # === ./Build install
        # =================================================================== #
        when './Build install',
             'perl Build.PL' # === perl Build.PL
          esystem this_step
        # =================================================================== #
        # === perl Makefile.PL
        # =================================================================== #
        when /^perl(_|-| )?Makefile.PL$/
          esystem this_step
        # =================================================================== #
        # === make
        # =================================================================== #
        when /^make(_|-| )?install$/
          run_make_install_command
        # =================================================================== #
        # === make
        # =================================================================== #
        when 'make'
          run_make_command
        # =================================================================== #
        # === copy_ruby
        # =================================================================== #
        when /copy(_|-)?ruby/
          copy_ruby
        # =================================================================== #
        # === ruby setup.rb setup
        #
        # We handle config, setup and install next:
        # =================================================================== #
        when /^ruby setup.rb (.+)/
          case $1.to_s.dup
          when 'config'
            ruby_setup_config
          when 'setup'
            ruby_setup_setup
          when 'install'
            ruby_setup_install
          end
        else # else notify the user
          opne 'The installation step '+sfancy(this_step)+' is not yet registered.'
          opne 'This should be changed.'
        end
      }
      delete_file 'setup.rb', :be_quiet
      delete_file 'InstalledFiles', :be_quiet
      delete_file '.config', :be_quiet
      exit # <- This may change at a later time, but for now we keep it this way.
    end
  end

  # ========================================================================= #
  # === show_build_systems_priorities
  #
  # Invocation example:
  #
  #   rbt --prioritize?
  #
  # ========================================================================= #
  def show_build_systems_priorities
    array = YAML.load_file(FILE_BUILD_SYSTEMS_PRIORITIES)
    e
    e lightgreen(
        'The build system priority is the following '\
        '(first one has higher priority):'
      )
    e
    array.each_with_index {|content, index| index += 1
      e slateblue(('('+index.to_s+') ').rjust(6))+
        orange(content)
    }
    e
  end

  # ========================================================================= #
  # === configure_has_a_higher_priority_than_meson?
  #
  # This method will return true if "configure" has a higher priority
  # than "meson". A yaml file keeps track of these priorities.
  # ========================================================================= #
  def configure_has_a_higher_priority_than_meson?
    ::RBT.configure_has_a_higher_priority_than_meson?
  end

  # ========================================================================= #
  # === return_hash_for_the_postinstallation_step
  #
  # This method will return a Hash that can be used as the main
  # initializer for class RBT::PostinstallationStep.
  #
  # The method accepts an argument (optional). That argument should be
  # an instance of RBT::Compile, if it is provided. If provided we
  # will fetch our information from that instance, rather than from
  # the default internal one.
  # ========================================================================= #
  def return_hash_for_the_postinstallation_step(
      optional_rbt_compile_object = nil
    )
    # ======================================================================= #
    # First we will setup some local variables that will be returned
    # via the Hash.
    # ======================================================================= #
    register_the_program_or_do_not_register_the_program = false
    shall_we_run_fetch_all_m4_macros = true
    shall_we_invoke_symlink_program = false
    shall_we_symlink_the_header_files_into_the_usr_include_hierarchy = false
    if optional_rbt_compile_object
      if optional_rbt_compile_object.continue_after_make_step? and
         optional_rbt_compile_object.make_install_stage_was_a_success? and
         optional_rbt_compile_object.no_error_has_occurred?
        register_the_program_or_do_not_register_the_program = true
      end
      if optional_rbt_compile_object.do_not_symlink?
        shall_we_run_fetch_all_m4_macros = false
      elsif !optional_rbt_compile_object.continue_after_make_step?
        shall_we_run_fetch_all_m4_macros = false
      end
      if optional_rbt_compile_object.continue_after_make_step? and
         optional_rbt_compile_object.make_install_stage_was_a_success?
        # ===================================================================== #
        # if i.include? '-' # Added this as of Jul 2014.
        #   i = i.to_s.delete('-')
        # end # ^^^ Disabled as of Sep 2017. We would have to go about
        # this via some central configuration option in RBT, rather
        # than an ad-hoc hack such as the above.
        # ===================================================================== #
        shall_we_invoke_symlink_program = true
      end unless optional_rbt_compile_object.do_not_symlink?
      if optional_rbt_compile_object.symlink_headers_into_usr_include_hierarchy? and
         optional_rbt_compile_object.uses_an_appdir_prefix? and
         File.directory?(optional_rbt_compile_object.real_prefix?.to_s+'include/') and
         !Dir["#{optional_rbt_compile_object.real_prefix?}include/**.h"].empty?
        shall_we_symlink_the_header_files_into_the_usr_include_hierarchy = true
      end
    else
      if continue_after_make_step? and
         make_install_stage_was_a_success? and
         no_error_has_occurred?
        register_the_program_or_do_not_register_the_program = true
      end
      if do_not_symlink?
        shall_we_run_fetch_all_m4_macros = false
      elsif !continue_after_make_step?
        shall_we_run_fetch_all_m4_macros = false
      end
      if continue_after_make_step? and
         make_install_stage_was_a_success?
        # ===================================================================== #
        # if i.include? '-' # Added this as of Jul 2014.
        #   i = i.to_s.delete('-')
        # end # ^^^ Disabled as of Sep 2017. We would have to go about
        # this via some central configuration option in RBT, rather
        # than an ad-hoc hack such as the above.
        # ===================================================================== #
        shall_we_invoke_symlink_program = true
      end unless do_not_symlink?
      if symlink_headers_into_usr_include_hierarchy? and
         uses_an_appdir_prefix? and
         File.directory?(real_prefix?.to_s+'include/') and
         !Dir["#{real_prefix?}include/**.h"].empty?
        shall_we_symlink_the_header_files_into_the_usr_include_hierarchy = true
      end
    end
    hash = {
      compile_which_program:            compile_which_program?,
      use_this_prefix:                  real_prefix?.to_s,
      dataset_to_use:                   cookbook_dataset?,
      use_tags:                         use_tags?,
      full_configure_command:           full_configure_command?,
      do_register_the_program:          register_the_program_or_do_not_register_the_program,
      do_run_fetch_all_m4_macros:       shall_we_run_fetch_all_m4_macros,
      shall_we_invoke_symlink_program:  shall_we_invoke_symlink_program,
      install_only_binaries:            install_only_binaries?,
      use_gobolinux_symlinkprogram:     use_gobolinux_symlinkprogram?,
      delete_libtool_files:             delete_libtool_files?,
      symlink_into_usr_lib_dir:         symlink_into_usr_lib_dir?,
      postinstall:                      postinstall?,
      shall_we_symlink_the_header_files_into_the_usr_include_hierarchy: shall_we_symlink_the_header_files_into_the_usr_include_hierarchy
    }
    return hash
  end

  # ========================================================================= #
  # === show_downloadable_programs
  # ========================================================================= #
  def show_downloadable_programs
    _ = available_programs?.sort
    chunked = _.each_slice(5)
    opnn; e 'The following programs can be downloaded:'
    e
    chunked.each {|this_chunk|
      e '  '+slateblue(this_chunk.join(', ').strip.chop)
    }
    e
  end

  # ========================================================================= #
  # === report_whether_this_program_is_included_or_whether_it_is_not
  # ========================================================================= #
  def report_whether_this_program_is_included_or_whether_it_is_not(
      i = program?
    )
    e teal(
      verbose_truth(
        is_this_program_included?(i, :be_quiet)
      )
    )
  end

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(
      i, be_verbose = :default
    )
    RBT::Compile.is_this_program_included?(i, be_verbose)
  end

  # ========================================================================= #
  # === consider_applying_a_patch
  #
  # Check whether we should apply a patch_set or not.
  #
  # This should happen before we make use of preinstall, and after
  # displaying an optional start message.
  # ========================================================================= #
  def consider_applying_a_patch
    apply_patch = apply_patch?
    if apply_patch
      patchset = apply_patch
      patchset = [patchset] unless patchset.is_a? Array
      patchset.flatten.each {|this_patchset|
        cliner {
          opne simp('Trying to ')+
               orange('apply a patch')+
               simp(' next:')+
               simp(' (in ')+
               sdir(configure_base_directory?)+
               simp(')')
          opne "  #{this_patchset}"
        }
        # =================================================================== #
        # (1) Download the patch unless it exists locally.
        # =================================================================== #
        unless File.exist? File.basename(this_patchset)
          do_download_this_program(this_patchset)
        end
        this_command = "patch -Np1 -i #{File.basename(this_patchset)}"
        opne "Running this command next (from directory "\
             "#{sdir(return_pwd)}): #{sfancy(this_command)}"
        # =================================================================== #
        # (2) Run the patchset.
        # =================================================================== #
        custom_system(this_command) # Delegate to Kernel#system.
      }
    end
  end

  # ========================================================================= #
  # === report_status_information
  #
  # Invocation example:
  #   rbt --report-status
  # ========================================================================= #
  def report_status_information
    e
    e 'The source directory is at:                      '+sdir(RBT.source_dir?)
    e 'The temp directory is at:                        '+sdir(RBT.temp_dir?)
    report_where_archives_will_be_extracted_into
    e
  end

  # ========================================================================= #
  # === return_chained_programs
  # ========================================================================= #
  def return_chained_programs 
    RBT.return_chained_programs
  end

  # ========================================================================= #
  # === update_binaries_of_this_program
  #
  # Invocation examples:
  #
  #   rbt --update-binaries-of=poppler
  #   rbt --update-binaries-of=htop
  #   rbt --update-binaries-of=xinelib
  #
  # ========================================================================= #
  def update_binaries_of_this_program(i)
    these_binaries = RBT.infer_these_binaries_will_be_installed_by(i)
    # ======================================================================= #
    # Now that we have obtained the binaries we can add it to our
    # .yml file. Note that this will always overwrite any other existing
    # entry, so the first step is to get rid of any old entries, before
    # adding the new entries.
    # ======================================================================= #
    path   = find_this_yaml_file(i)
    string = read_file_in_default_encoding(path)
    if is_on_roebe?
      opnn; e 'Creating a backup of the file first.'
      backup_directory_at = log_dir?+'backup/'
      mkdir(backup_directory_at) unless File.directory?(backup_directory_at)
      # ===================================================================== #
      # We will copy the old file, so that we could re-use it if we need to.
      # ===================================================================== #
      opnn; copy_file(
              path,
              backup_directory_at+File.basename(path),
              :be_verbose
            )
    end
    if string.include? ' binaries:'
      if is_on_roebe?
        # =================================================================== #
        # Notify only me on my home system in this case, as most users
        # won't need this information here really.
        # =================================================================== #
        opnn; e "Removing the old #{slateblue('binaries:')} entries first."
        new_string = ''.dup
        state_is_open = false
        string.split(N).each {|line|
          if line.include? ' binaries:'
            state_is_open = true
          else
            # =============================================================== #
            # Toggle it unless the next line starts with a ' - '.
            # =============================================================== #
            unless line.start_with?(' - ')
              state_is_open = false
            end
          end
          unless state_is_open
            new_string << line << N
          end
        }
        string = new_string # And re-assign it here.
      end
    end
    # ======================================================================= #
    # Ok, now we can find the proper position and simply add the binaries.
    # We will first check for a "homepage:" String; then "url2:" and
    # then "url1:". These are our "anchors".
    # ======================================================================= #
    if these_binaries.empty?
      opnn; e 'Can not store anything because no binaries could be found.'
    else
      append_this_string = ' binaries:'+N+' - '+these_binaries.join(N+' - ')+N
      splitted = string.split(N)
      array = []
      if string.include? ' blfs: '
        splitted.each {|line|
          if line.include? ' blfs: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      elsif string.include? 'configure_options: '
        splitted.each {|line|
          if line.include? ' configure_options: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      elsif string.include? 'url2: '
        splitted.each {|line|
          if line.include? ' url2: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      else
        opnn; e 'Can not anchor it as no string '\
                'called '+slateblue('blfs:')+
                ':, '+slateblue('configure_options:')
        opnn; e 'or '+slateblue('url2:')+' could be found.'
      end
      what = array.join.chomp
      # ===================================================================== #
      # Next update the old content.
      # ===================================================================== #
      into = path
      opnn; e 'Next overwriting the file `'+sfile(into)+'`.'
      write_what_into(what, into)
      if is_on_roebe?
        # =================================================================== #
        # Also auto-updating the rbt project.
        # =================================================================== #
        opnn; e 'Since we are on roebe we will also auto-update the '\
                'rbt project next.'
        cd RUBY_SRC_DIR_AT_HOME+'rbt/'
        require 'roebe/toplevel_methods/rinstall2.rb'
        Roebe.rinstall2
      end
    end
  end

  # ========================================================================= #
  # === update_libraries_of_this_program
  #
  # Invocation examples:
  #
  #   rbt --update-libraries-of=poppler
  #   rbt --update-libraries-of=htop
  #   rbt --update-libraries-of=xinelib
  #
  # ========================================================================= #
  def update_libraries_of_this_program(i)
    these_libraries = RBT.infer_these_libraries_will_be_installed_by(i)
    # ======================================================================= #
    # Now that we have obtained the libraries we can add it to our
    # .yml file. Note that this will always overwrite any other existing
    # entry, so the first step is to get rid of any old entries, before
    # adding the new entries.
    # ======================================================================= #
    path   = find_this_yaml_file(i)
    string = read_file_in_default_encoding(path)
    if is_on_roebe?
      opnn; e 'Creating a backup of the file first.'
      backup_directory_at = log_dir?+'backup/'
      mkdir(backup_directory_at) unless File.directory?(backup_directory_at)
      # ===================================================================== #
      # We will copy the old file, so that we could re-use it if we need to.
      # ===================================================================== #
      opnn; copy_file(
              path,
              backup_directory_at+File.basename(path),
              :be_verbose
            )
    end
    if string.include? ' libraries:'
      if is_on_roebe?
        # =================================================================== #
        # Notify only me on my home system in this case, as most users
        # won't need this information here really.
        # =================================================================== #
        opnn; e "Removing the old #{slateblue('libraries:')} entries first."
        new_string = ''.dup
        state_is_open = false
        string.split(N).each {|line|
          if line.include? ' libraries:'
            state_is_open = true
          else
            # =============================================================== #
            # Toggle it unless the next line starts with a ' - '.
            # =============================================================== #
            unless line.start_with?(' - ')
              state_is_open = false
            end
          end
          unless state_is_open
            new_string << line << N
          end
        }
        string = new_string # And re-assign it here.
      end
    end
    # ======================================================================= #
    # Ok, now we can find the proper position and simply add the libraries.
    # We will first check for a "homepage:" String; then "url2:" and
    # then "url1:". These are our "anchors".
    # ======================================================================= #
    if these_libraries.empty?
      opnn; e 'Can not store anything because no libraries could be found.'
    else
      append_this_string = ' libraries:'+N+' - '+these_libraries.join(N+' - ')+N
      splitted = string.split(N)
      array = []
      if string.include? ' homepage: '
        splitted.each {|line|
          if line.include? ' homepage: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      elsif string.include? 'url2: '
        splitted.each {|line|
          if line.include? ' url2: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      elsif string.include? 'url1: '
        splitted.each {|line|
          if line.include? ' url1: '
            array << line+N+append_this_string
          else
            array << "#{line}#{N}"
          end
        }
      else
        opnn; e 'Can not anchor it as no string '\
                'called '+slateblue('homepage:')+
                ':, '+slateblue('url2:')
        opnn; e 'or '+slateblue('url1:')+' could be found.'
      end
      what = array.join.chomp
      # ===================================================================== #
      # Next update the old content.
      # ===================================================================== #
      into = path
      opnn; e 'Next overwriting the file `'+sfile(into)+'`.'
      write_what_into(what, into)
      if is_on_roebe?
        # =================================================================== #
        # Also auto-updating the rbt project.
        # =================================================================== #
        opnn; e 'Since we are on roebe we will also auto-update the '\
                'rbt project next.'
        cd RUBY_SRC_DIR_AT_HOME+'rbt/'
        require 'roebe/toplevel_methods/rinstall2.rb'
        Roebe.rinstall2
      end
    end
  end

  # ========================================================================= #
  # === consider_comparing_the_installed_libraries_versus_the_assumed_libraries
  #
  # This method will try to compare the installed libraries in an AppDir
  # with the ones that are registered in the various cookbook .yml files.
  #
  # This will happen ONLY if we are on roebe; and ONLY if we use an
  # AppDir approach.
  # ========================================================================= #
  def consider_comparing_the_installed_libraries_versus_the_assumed_libraries
    if is_on_roebe? and is_an_appdir_installation? and
      has_been_installed_successfully?
      assumed_libraries = RBT.infer_these_libraries_will_be_installed_by(
        compile_which_program?
      )
      if assumed_libraries
        libraries?.each {|this_already_registered_library|
          # ================================================================= #
          # The variable this_already_registered_library may be something
          # like 'libgmodule-2.0.so'.
          # ================================================================= #
          if assumed_libraries.any? {|entry|
              entry.include? this_already_registered_library
            }
          else
            opnn; e 'The library '+
                    orange(this_already_registered_library)+
                    ' is not registered.'
            opnn; e 'Consider '+crimson('updating')+' it (file: '+
                    sfile(compile_which_program?+'.yml')+
                    ').'
          end
        }
      end
    end
  end

  # ========================================================================= #
  # === cd_to_the_configure_base_directory
  #
  # This method can be used to cd into the configure base directory.
  #
  # It can report what it is doing to, in the verbose variant.
  # ========================================================================= #
  def cd_to_the_configure_base_directory(
      be_verbose_or_be_quiet = :be_quiet
    )
    target = @internal_hash[:configure_base_directory].dup
    case be_verbose_or_be_quiet
    when :be_quiet
      cd target, :be_quiet
    else
      opnn; cd target, :be_verbose
    end
  end; alias cd_to_the_configure_base_directory_to_be_used cd_to_the_configure_base_directory # === cd_to_the_configure_base_directory_to_be_used
  
  # ========================================================================= #
  # === consider_modifying_postinstall_with_this_prefix
  #
  # This method can be used to modify all prefix-targets in the
  # postinstall Array.
  #
  # We will only modify the program_version normally here.
  # ========================================================================= #
  def consider_modifying_postinstall_with_this_prefix(
      i = program_version?
    )
    return unless i
    unless i.start_with? '/'
      i = i.dup if i.frozen?
      i.prepend('/') # A bit hackish as a workaround but this works.
    end
    # ======================================================================= #
    # The regex is also stored here:
    #
    #   https://rubular.com/r/U7J9cbbqUBPI
    # 
    # ======================================================================= #
    regex_to_use = /\/(\d{1,2}\.?\d{1,2}\.?\d{0,2})/
    set_postinstallation(
      cookbook_dataset_postinstallation?.map {|line|
        # =================================================================== #
        # We must use .gsub!() here rather than .sub!() because the
        # full line may be like this:
        #
        #   ln -s /Programs/Gtk+/3.24.1/include/gtk-2.0 /Programs/Gtk+/3.24.1/include/gtk
        #
        # =================================================================== #
        line.gsub!(regex_to_use, i)
        line
      }
    )
  end; alias consider_modifying_postinstall consider_modifying_postinstall_with_this_prefix # === consider_modifying_postinstall

  # ========================================================================= #
  # === determine_which_compiler_will_be_used
  #
  # This method currently will only check if the clang-compiler is used.
  # ========================================================================= #
  def determine_which_compiler_will_be_used
    case use_which_compiler?
    when 'clang'
      do_use_clang_compiler(:be_quiet)
    end
  end

  # ========================================================================= #
  # === will_gcc_be_used_for_compilation?
  # ========================================================================= #
  def will_gcc_be_used_for_compilation?
    use_which_compiler? == 'gcc'
  end

  # ========================================================================= #
  # === use_which_compiler?
  # ========================================================================= #
  def use_which_compiler?
    config?.use_this_compiler
  end

  # ========================================================================= #
  # === show_which_compiler_will_be_used
  #
  # This method will show which compiler will be used.
  #
  # Commandline invocation:
  #
  #   rbt --compiler?
  #
  # ========================================================================= #
  def show_which_compiler_will_be_used
    opne "The compiler that will be used for compiling "\
         "programs is `#{slateblue(use_which_compiler?)}`."
  end

  # ========================================================================= #
  # === compile_all_programs
  #
  # This method is a direct approach to compile every program that we have
  # registered, in our database.
  #
  # This method accepts one argument, which decides whether we will quit
  # afterwards or whether we will not.
  #
  # Invocation example:
  #
  #   ry --compile_all_programs
  #
  # ========================================================================= #
  def compile_all_programs(
      shall_we_exit = true
    )
    case shall_we_exit
    when :do_exit_afterwards
      shall_we_exit = true
    end
    _ = return_available_programs.shuffle # We shuffle for more "randomness".
    n_programs = _.size.to_s
    clear_commandline_options
    opnn; e sfancy('Compiling all available programs next (by a brute'\
            'force approach - we found ')+
            simp(n_programs)+sfancy(' programs):')
    compile_every_program(_)
    exit_program if shall_we_exit
  end

  # ========================================================================= #
  # === opne
  # ========================================================================= #
  def opne(i = '')
    opnn; e i
  end

  # ========================================================================= #
  # === do_not_use_any_configure_option
  #
  # This option allows us to avoid using any configure option.
  #
  # Invocation example from the commandline:
  #
  #   ry htop --dont-use-configure-options
  #
  # ========================================================================= #
  def do_not_use_any_configure_option
    if block_given? and yield == :be_verbose
      # ===================================================================== #
      # Be verbose in this case.
      # ===================================================================== #
      opne 'No configure options will be used '\
           'for this run.'
    end
    @internal_hash[:try_to_use_configure_options] = false
  end

  # ========================================================================= #
  # === report_which_libraries_are_missing
  #
  # Report which libraries are missing.
  #
  # Invocation example:
  #   ry --not-found
  # ========================================================================= #
  def report_which_libraries_are_missing
     require 'rbt/utility_scripts/not_found_libraries.rb'
     RBT::NotFoundLibraries.new
  end

  # ========================================================================= #
  # === generate_slack_desc_file_for_this_program
  #
  # Trigger like so:
  #
  #   rbt htop --gsc
  #
  # ========================================================================= #
  def generate_slack_desc_file_for_this_program(
      i = compile_which_program?
    )
    path_to_this_file = RBT.generate_slackware_description_file(i)
    return path_to_this_file
  end; alias create_slackware_description_file generate_slack_desc_file_for_this_program # ===  create_slackware_description_file

  # ========================================================================= #
  # === automatically_create_package?
  # ========================================================================= #
  def automatically_create_package?
    RBT.automatically_create_package?
  end

  # ========================================================================= #
  # === is_on_slackware?
  #
  # This method will return true if we are using slackware; and false
  # otherwise.
  # ========================================================================= #
  def is_on_slackware?
    RBT.configuration?.operating_system == 'slackware'
  end

  # ========================================================================= #
  # === run_configure_make_and_make_install_with_this_configure_line
  # ========================================================================= #
  def run_configure_make_and_make_install_with_this_configure_line(i)
    run_configure_via_system(i)
    run_make
    run_make_install
  end

  # ========================================================================= #
  # === try_to_use_the_file_windows_cookbook?
  # ========================================================================= #
  def try_to_use_the_file_windows_cookbook?
    config?.try_to_use_the_file_windows_cookbook
  end; alias try_to_use_the_file_windows_cookbook try_to_use_the_file_windows_cookbook? # === try_to_use_the_file_windows_cookbook

  # ========================================================================= #
  # === run_preinstall_sed_and_currently_being_compiled
  # ========================================================================= #
  def run_preinstall_sed_and_currently_being_compiled
    run_preinstall_actions
    check_for_sed_action
    register_that_this_program_is_currently_being_compiled
  end

  # ========================================================================= #
  # === copy_headers_from
  #
  # This method will copy the registered headers from the given program
  # at hand.
  #
  # The first argument to this method is the path to the header files.
  #
  # Invocation examples:
  #   ry glibc --copy-headers-from=/usr/include
  #   ry glibc --copy-headers-to-pwd
  # ========================================================================= #
  def copy_headers_from(i)
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    to = return_pwd 
    headers = headers?
    unless headers.empty?
      headers.each {|this_header|
        from = "#{i}#{this_header}"
        if this_header.include? '/'
          # ================================================================= #
          # In this case we must create such a subdirectory.
          # ================================================================= #
          dirname = File.dirname(this_header)
          unless File.directory? File.dirname(dirname)
            mkdir(dirname)
          end
        end
        e 'Now trying to copy `'+sfile(from)+'` to `'+
          sfile(to+this_header)+'`.'
        copy_file(from, to+this_header)
      }
    end
  end

  # ========================================================================= #
  # === check_for_binary_duplicates_in_this_directory
  #
  # This method can be used to check for binary duplicates, aka binaries
  # that exist more than once on the given system, in the given (passed)
  # directory.
  # ========================================================================= #
  def check_for_binary_duplicates_in_this_directory(
      i = Dir.pwd
    )
    require 'rbt/utility_scripts/check_for_duplicate_binaries.rb'
    RBT::CheckForDuplicateBinaries.new(Dir.pwd)
  end

  # ========================================================================= #
  # === try_to_show_all_binary_duplicates_of_this_program
  #
  # Use this method to show which binary duplicates exist for the given
  # input program.
  # ========================================================================= #
  def try_to_show_all_binary_duplicates_of_this_program(
      i = compile_which_program?
    )
    program_name = i
    if i.nil?
      opnn; e 'A nil value has been found in '+orange(__method__.to_s)+'.'
      pp caller()
    end
    capitalized_program_name = program_name.capitalize
    target = programs_directory?+capitalized_program_name+program_name.capitalize+'/Current/bin/*'
    opne 'Obtaining all files at `'+sdir(target)+'`.'
    all_binaries = Dir[target]
    all_binaries.select! {|entry|
      File.exist? '/usr/bin/'+File.basename(entry)
    }
    if all_binaries.empty?
      opne 'The '+sdir('/usr/bin/')+' hierarchy does not contain '\
           'any of the binaries'
      opne "belonging to #{sfancy(program_name)}."
    else
      n_duplicates = all_binaries.size.to_s
      opne "Found #{sfancy(n_duplicates)} binary duplicates."
      all_binaries.each {|file|
        e "/usr/bin/#{File.basename(file)}"
      }
    end
  end

  # ========================================================================= #
  # === append_this_to_full_configure_command
  # ========================================================================= #
  def append_this_to_full_configure_command(i = ' build')
    unless i.end_with? ' '
      i = i.dup if i.frozen?
      i << ' ' # Properly pad it here.
    end
    i.squeeze!(' ')
    @internal_hash[:full_configure_command] << i
  end

  # ========================================================================= #
  # === verbose_purge_fhs_binaries_of_this_program
  #
  # This method will notify the user as to what will be done, then
  # delegate towards purge_fhs_binaries_of_this_program().
  # ========================================================================= #
  def verbose_purge_fhs_binaries_of_this_program(i)
    i = [i].flatten.compact
    i.each {|entry|
      entry.downcase!
      opne "Now purging all #{mediumspringgreen('binaries')} of "\
           "the program `#{sfancy(entry)}`, at the #{sdir('/usr/bin/')}"\
           " hierarchy."
      purge_fhs_binaries_of_this_program(entry)
    }
  end

  # ========================================================================= #
  # === purge_fhs_binaries_of_this_program
  #
  # This method will purge binaries that may reside at the /usr/bin/
  # hierarchy. Be careful when using this - only use it when you are
  # sure that you want this functionality.
  #
  # Invocation examples:
  #   ry littleutils --purge-fhs-binaries
  #   ry --purge-traditional-binaries-from=xscreensaver
  # ========================================================================= #
  def purge_fhs_binaries_of_this_program(
      i = compile_which_program?
    )
    if i.nil?
      opne 'Please provide a valid program name to this method.'
      return
    end
    PurgeBinariesOfThisProgram.new(i.downcase)
  end

  # ========================================================================= #
  # === verbose_guess_the_build_type
  # ========================================================================= #
  def verbose_guess_the_build_type
    opne 'The (guessed) build type is `'+slateblue(guess_build_type.to_s)+'`.'
  end

  # ========================================================================= #
  # === guess_build_type
  # ========================================================================= #
  def guess_build_type
    RBT.build_type?
  end; alias do_guess_the_build_type guess_build_type # === do_guess_the_build_type

  # ========================================================================= #
  # === try_to_find_an_alias_to_this_input_unless_the_program_exists
  #
  # This method will either return the original input unmodified;
  # or it will modify the input in such a way that it points to
  # a valid, registered program.
  #
  # For example, the String "ph" may become "php". 
  # ========================================================================= #
  def try_to_find_an_alias_to_this_input_unless_the_program_exists(i)
    unless cookbooks_include_this_program? i.downcase
      is_the_program_included = cookbooks_include_this_program?(
        i.downcase, :with_abbreviations
      )
      if is_the_program_included
        i = abbreviations?[i.downcase]
      end
    end
    i
  end

  # ========================================================================= #
  # === copy_these_archives
  #
  # This method will copy the selected applications into the current
  # working directory.
  #
  # To invoke this method from the commandline, try:
  #   rbt --copy-archives=kde5_plasma
  # ========================================================================= #
  def copy_these_archives(
      i = :kde5_plasma
    )
    if i.is_a? String
      i = i.split(',') if i.include? ','
    end
    case i
    when :kde5_plasma, 'kde5_plasma'
      i = return_kde_plasma_components
    end
    i = [i].flatten.compact.reject {|entry| entry.strip.empty? }.map {|entry|
      try_to_find_an_alias_to_this_input_unless_the_program_exists(entry)
    }
    if i.empty?
      opne 'Please provide at the least one argument - the '\
           'directory that is to be copied.'
    else
      opne "Now copying #{i.size.to_s} entries into the local "\
           "working directory:"
      i.each {|this_program|
        target_directory = source_dir?+this_program.downcase+'/'
        if File.directory? target_directory
          copy_directory(target_directory, :be_verbose)
        else
          opne "No directory exists at `#{sdir(target_directory)}`."
        end
      }
    end
  end

  # ========================================================================= #
  # === purge_these_binaries
  #
  # This method can be used to purge binaries.
  # ========================================================================= #
  def purge_these_binaries(
      i = binaries?
    )
    removed_n_binaries = 0
    opne "Now attempting to remove #{sfancy(i.size.to_s)} binaries:"
    e
    i.each {|this_binary|
      target = "/usr/bin/#{this_binary}"
      ftype = nil
      begin
        ftype = File.ftype(target)
      rescue Errno::ENOENT; end
      if File.exist? target
        e "Deleting the file `#{sfile(target)}` next."
        remove_file(target)
        removed_n_binaries += 1
      else
        if ftype == 'link'
          e "Deleting the stray symlink at `#{sfile(target)}` next."
          remove_symlink(target)
        end
      end
    }
    e
    if removed_n_binaries == 0
      opne 'None of these binaries could be '\
           'found at '+sdir('/usr/bin/')+'.'
    end
  end

  # ========================================================================= #
  # === try_to_batch_compile_this_stack
  #
  # This method can be used to batch-compile several applications.
  #
  # Invocation examples:
  #
  #   rbt --batch-compile=autotools
  #   rbt --batch-compile=vid_suite
  #   rbt --batch-compile=mate
  #   rbt --batch-compile=kde5_porting_aids
  # ========================================================================= #
  def try_to_batch_compile_this_stack(i)
    # ======================================================================= #
    # === Do some sanitizing of the input first.
    # ======================================================================= #
    case i
    # ======================================================================= #
    # === rbt --batch-compile=phonon
    # ======================================================================= #
    when 'phonon',
         /^kde(_|-)?plasma/
      i = 'kde5_plasma'
    # ======================================================================= #
    # === rbt --batch-compile=kde_apps
    # ======================================================================= #
    when /^kde(_|-)?apps/
      i = 'kde5_applications'
    # ======================================================================= #
    # === rbt --batch-compile=protos
    # ======================================================================= #
    when 'protos'
      i = 'xorg_protos'
    end
    case i # case tag
    # ======================================================================= #
    # === rbt --batch-compile=kde5_porting_aids
    # ======================================================================= #
    when /^kde5?(_|-)?porting(_|-)?aids?$/i,
         /^porting(_|-)?aids?$/i
      i = return_kde5_porting_aids
    # ======================================================================= #
    # === rbt --batch-compile=mate
    # ======================================================================= #
    when 'mate',
         /^mate(-|_)?desktop$/
      i = RBT.returned_chained_programs_of_the_mate_desktop
      opne "We will compile all #{sfancy(i.size.to_s)} programs "\
           "of the mate-desktop."
      opne 'These include:'
      e; i.each {|entry|
        e '  - '+sfancy(entry)
      }; e
    else
      hash = return_chained_programs
      i = hash[i]
    end
    if i.is_a? String and i.include? ','
      i = i.split(',').map(&:strip).flatten # Now it should be an Array.
    end
    if i.is_a? Array
      i.map! {|entry| entry.delete('-') }
    end
    return i
  end

  # ========================================================================= #
  # === use_maximum_speed_for_compilation?
  # ========================================================================= #
  def use_maximum_speed_for_compilation?
    result = false
    if config?.use_maximum_speed_for_compilation
      result = config?.use_maximum_speed_for_compilation
    end
    return result
  end

  # ========================================================================= #
  # === do_not_try_to_autofix
  #
  # This method will specifically disable any attempts to try to autofix
  # libtool-related problems. It is currently silent, all of the time.
  # ========================================================================= #
  def do_not_try_to_autofix
    @internal_hash[:try_to_autofix] = false
  end

  # ========================================================================= #
  # === we_tried_to_compile_this_program_n_times
  # ========================================================================= #
  def we_tried_to_compile_this_program_n_times?
    @internal_hash[:we_tried_to_compile_this_program_n_times]
  end; alias compiled_n_times? we_tried_to_compile_this_program_n_times? # === compiled_n_times?

  # ========================================================================= #
  # === increment_compiled_this_program_n_times_counter
  #
  # By default this method will increment by +1.
  # ========================================================================= #
  def increment_compiled_this_program_n_times_counter(
      by_plus_n = 1
    )
    @internal_hash[:we_tried_to_compile_this_program_n_times] += by_plus_n
  end; alias increment_compiled_counter increment_compiled_this_program_n_times_counter # === increment_compiled_counter

  # ========================================================================= #
  # === verbose_enable_glib_schema
  # ========================================================================= #
  def verbose_enable_glib_schema
    opne "Enabling #{slateblue('glib-schemata')} next."
    do_use_glib_schema # Enable the glib-schemata here.
  end

  # ========================================================================= #
  # === do_use_glib_schema
  #
  # Use this method to enable the use of a glib-schema.
  # ========================================================================= #
  def do_use_glib_schema
    @internal_hash[:use_glib_schema] = true
  end

  # ========================================================================= #
  # === set_full_configure_command
  # ========================================================================= #
  def set_full_configure_command(i)
    i.strip! # <- This is mandatory; may be reconsidered in the future though.
    @internal_hash[:full_configure_command] = i
  end

  # ========================================================================= #
  # === is_this_a_registered_binary_name?
  # ========================================================================= #
  def is_this_a_registered_binary_name?(i)
    ::RBT.has_this_binary?(i)
  end

  # ========================================================================= #
  # === consider_removing_empty_subdirectories
  #
  # We will remove empty subdirectories but only from AppDir-like prefixes.
  # ========================================================================= #
  def consider_removing_empty_subdirectories
    if uses_an_appdir_prefix?
      prefix = prefix?.to_s # <- This will be like "/Programs/Gcide/0.52/".
      subdirectories = Dir[prefix+'*'].select {|entry|
        File.directory?(entry) and Dir.empty?(entry)
      }
      remove_directories(
        subdirectories
      )
      consider_removing_the_current_symlink_if_the_directory_is_empty
      consider_removing_the_settings_directory_if_it_is_empty
      consider_removing_the_empty_versioned_directory
      consider_removing_the_empty_base_directory_of_the_program
    end
  end; alias consider_running_cleanup_actions consider_removing_empty_subdirectories # === consider_running_cleanup_actions

  # ========================================================================= #
  # === consider_removing_the_empty_base_directory_of_the_program
  # ========================================================================= #
  def consider_removing_the_empty_base_directory_of_the_program
    if uses_an_appdir_prefix? and !is_a_gem_file?
      target = rds(
        programs_dir?+program_name?.capitalize.delete('-_')+'/'
      )
      if File.directory?(target) and
         Dir.empty?(target)
        opne 'Removing the empty directory at `'+sdir(target)+'` next.'
        remove_this_directory(target)
      end
    end
  end

  # ========================================================================= #
  # === consider_removing_the_settings_directory_if_it_is_empty
  #
  # This method will attempt to remove the /Settings subdirectory, if
  # it is empty.
  # ========================================================================= #
  def consider_removing_the_settings_directory_if_it_is_empty
    if uses_an_appdir_prefix?
      prefix = prefix?.to_s
      settings_directory = File.dirname(prefix)+'/Settings'
      if File.directory?(settings_directory) and
         Dir.empty?(settings_directory)
        remove_this_directory(settings_directory)
      end
    end
  end

  # ========================================================================= #
  # === consider_removing_the_empty_versioned_directory
  # ========================================================================= #
  def consider_removing_the_empty_versioned_directory
    if uses_an_appdir_prefix?
      prefix = prefix?.to_s
      if File.directory?(prefix) and
        Dir.empty?(prefix)
        remove_this_directory(prefix)
      end
    end
  end

  # ========================================================================= #
  # === consider_removing_the_current_symlink_if_the_directory_is_empty
  # ========================================================================= #
  def consider_removing_the_current_symlink_if_the_directory_is_empty
    if uses_an_appdir_prefix?
      prefix = prefix?.to_s
      symlink_current = rds(
        File.dirname(prefix)+'/Current'
      )
      if File.symlink?(symlink_current) and 
         File.directory?(File.readlink(symlink_current)) and
         Dir.empty?(File.readlink(symlink_current))
        delete_this_symlink(symlink_current)
      end
    end
  end 

  # ========================================================================= #
  # === check_whether_the_given_input_is_a_file_and_does_exist?
  # ========================================================================= #
  def check_whether_the_given_input_is_a_file_and_does_exist?
    @internal_hash[:check_whether_the_given_input_is_a_file_and_does_exist]
  end

  # ========================================================================= #
  # === use_this_as_programs_dir
  # ========================================================================= #
  def use_this_as_programs_dir(
      i = programs_directory?
    )
    i = convert_end_variable # Sanitize it.
    i << '/' unless i.end_with? '/'
    opne "Assigning a new Programs directory at: #{simp(i)}"
    RBT.configuration?.programs_directory = i
    # ======================================================================= #
    # Next we also must sync towards the main dataset.
    # ======================================================================= #
    cookbook_dataset?.programs_directory = i
    cookbook_dataset?.sanitize_prefix :appdir_path
  end

  # ========================================================================= #
  # === notify_the_user_that_something_failed_at_configure_stage
  # ========================================================================= #
  def notify_the_user_that_something_failed_at_configure_stage
    opne swarn('Something appears to have failed at the configure stage.')
    report_status
  end

  # ========================================================================= #
  # === do_not_continue_after_make_step
  # ========================================================================= #
  def do_not_continue_after_make_step
    @internal_hash[:continue_after_make_step] = false
  end

  # ========================================================================= #
  # === generate_global_yaml_database
  # ========================================================================= #
  def generate_global_yaml_database
    RBT.generate_global_yaml_database
  end

  # ========================================================================= #
  # === do_run_simulation
  # ========================================================================= #
  def do_run_simulation
    @internal_hash[:run_simulation] = true
  end

  # ========================================================================= #
  # === save_into_compiled_programs
  #
  # This method will try to save a lot of useful data into a file
  # called "compiled_programs.yml".
  #
  # That way we can try to figure out what happened at any moment
  # in time (hopefully).
  #
  # The variable full_configure_command will store the full configure
  # command, whereas configure_options only shows the part afterwards,
  # such as which parts to enable or disable and so forth.
  #
  # This should also store ARCH info and such.
  # ========================================================================= #
  def save_into_compiled_programs(
      store_into = FILE_COMPILED_PROGRAMS
    )
    what  = ''.dup # Need a non-frozen String.
    what << '# --- '+program_name?+N
    what << program_name?+':'+N
    what << ' CFLAGS: '+cflags?.to_s+N
    what << ' compiled_at: '+today?.to_s+', '+current_hour?+N
    what << ' prefix: '+real_prefix?.to_s+N
    what << ' full_configure_command: '+
            full_configure_command?.to_s+N
    what << ' configure_options: '+configure_options?.to_s+N
    what << ' successfully_compiled: true/false'+N # This line here is obviously a stub.
    what << N # Extra newline - makes reading this file easier.
    append_what_into(what, store_into)
  end

  # ========================================================================= #
  # === a_program_has_been_initialized?
  # ========================================================================= #
  def a_program_has_been_initialized?
    !main_dataset?.nil?
  end

  # ========================================================================= #
  # === either_report_the_total_amount_of_binaries_or_how_many_binaries_this_program_will_install
  # ========================================================================= #
  def either_report_the_total_amount_of_binaries_or_how_many_binaries_this_program_will_install
    if a_program_has_been_initialized?
      report_the_amount_of_binaries
    else
      RBT.report_how_many_binaries_are_registered
    end
  end

  # ========================================================================= #
  # === compile_the_main_program_including_the_dependencies
  # ========================================================================= #
  def compile_the_main_program_including_the_dependencies
    ensure_that_the_main_dataset_exists
    compile_these_programs?.prepend(
      required_deps_on?
    )
    compile_these_programs?.flatten!
  end

  # ========================================================================= #
  # === compile_which_programs?
  # ========================================================================= #
  def compile_which_programs?
    @internal_hash[:compile_these_programs]
  end; alias compile_these_programs? compile_which_programs? # === compile_these_programs?

  # ========================================================================= #
  # === do_upgrade_from_this_file
  #
  # This method can be used to read input from a file, in order to
  # determine which programs are to be compiled.
  # ========================================================================= #
  def do_upgrade_from_this_file(i)
    case i # case tag
    when :default, :def
      i = 'these_programs_can_be_upgraded.yml'
    end
    if File.exist? i
      if i.end_with? '.yml'
        dataset_to_use = YAML.load_file(i)
      else
        dataset_to_use = File.readlines(i).map {|line| line.strip }
      end
      _ = dataset_to_use
      _ = _.join(', ') if _.is_a? Array
      opne "We will use this dataset: #{slateblue(_)}"
      set_compile_these_programs(dataset_to_use)
    else
      opne "No file called #{sfile(i)} appears to exists."
    end
  end

  # ========================================================================= ##
  # === set_is_an_abbreviation
  # ========================================================================= #
  def set_is_an_abbreviation(i = true)
    @internal_hash[:is_an_abbreviation] = i
  end

  # ========================================================================= #
  # === is_an_abbreviation?
  # ========================================================================= #
  def is_an_abbreviation?
    @internal_hash[:is_an_abbreviation]
  end

  # ========================================================================= #
  # === determine_the_configure_base_directory_to_use
  #
  # This method can be used to determine the full path to the configure
  # base directory to be used.
  # ========================================================================= #
  def determine_the_configure_base_directory_to_use
    target = extract_to_this_dir?.dup+
             yaml_dataset?.program_name_and_program_version?+
             '/'+
             cookbook_dataset_configure_base_directory?.to_s
    set_configure_base_directory(target)
  end

  # ========================================================================= #
  # === extract_program_and_cd_to_the_configure_base_directory
  # ========================================================================= #
  def extract_program_and_cd_to_the_configure_base_directory
    do_extract
    cd_to_the_configure_base_directory
  end

  # ========================================================================= #
  # === cookbook_dataset_short_name?
  # ========================================================================= #
  def cookbook_dataset_short_name?
    cookbook_dataset?.short_name?.to_s
  end; alias short_name? cookbook_dataset_short_name? # === short_name?

  # ========================================================================= #
  # === prepend_dot_to_configure_string
  # ========================================================================= #
  def prepend_dot_to_configure_string
    full_configure_command?.prepend('.')
  end

  # ========================================================================= #
  # === enable_simulation
  #
  # Enable to run in simulation mode. The first argument is an optional 
  # message we give, in which case we will output that message.
  # ========================================================================= #
  def enable_simulation(
      optional_message = ''
    )
    unless optional_message.empty?
      opne optional_message
    end
    do_run_simulation
  end

  # ========================================================================= #
  # === verbose_md5_checking
  #
  # Use this method to do md5-checksum checking.
  # ========================================================================= #
  def verbose_md5_checking
    _ = program_path?
    opne sfancy('The md5sum of `')+simp(program_name?)+
         sfancy('` in `')+sfile(_)+sfancy('` is:')
    opne simp('  '+return_md5(_)) # This method is defined here.
  end

  # ========================================================================= #
  # === disable_md5sum_calculation
  #
  # This method here can be used to disable the md5sum calculation
  # altogether.
  #
  # Invocation example:
  #   ry htop --no-md5sum
  # ========================================================================= #
  def disable_md5sum_calculation
    opne 'We will not calculate the md5sum of an archive. This '\
         'should speed up'
    opne 'things a little bit, in particular for very large files.'
    RBT::Cookbooks::Cookbook.disable_md5sum_calculation # Cookbooks::Cookbook must provide this method.
  end

  # ========================================================================= #
  # === use_url
  #
  # Usage example:
  #   ry ffmpeg use_url
  # ========================================================================= #
  def use_url
    dataset?.program_name = 'USE_URL' # RBT::Cookbooks::Cookbook will handle this call.
  end

  # ========================================================================= #
  # === do_not_compile?
  # ========================================================================= #
  def do_not_compile?
    @internal_hash[:do_not_compile]
  end

  # ========================================================================= #
  # === get_packages
  #
  # Get all packages from the directory /Depot/Packages and compare
  # them to our given request.
  #
  # I am no longer making heavy use of Packages but perhaps one day
  # I may extend it, so this is one reason why it is kept.
  # 
  # Invoke it like so:
  #   ry php pgk?
  # ========================================================================= #
  def get_packages(
      this_program = compile_which_program?
    )
    this_program = this_program.first if this_program.is_a? Array
    packages_dir = packages_directory? # This constant is defined in Configuration.rb
    opnn
    e "Searching packages for `#{simp(this_program)}` matches now inside "+
      sdir(packages_dir)+'.'
    _ = get_files_from(packages_dir, true)
    if _
      result = _.grep(/#{this_program}/i)
      pp result
      return result
    else
      opnn
      e "Unfortunately we did not find any match in `#{sdir(packages_dir)}`."
    end
  end

  # ========================================================================= #
  # === update_this_entry
  # ========================================================================= #
  def update_this_entry(i)
    require 'rbt/utility_scripts/update_entry.rb'
    RBT::UpdateEntry.new(i)
  end

  # ========================================================================= #
  # === consider_making_use_of_cookbook_aliases
  # ========================================================================= #
  def consider_making_use_of_cookbook_aliases
    unless make_use_of_cookbook_aliases? == :user_disabled_this_setting_via_the_commandline
      @internal_hash[:make_use_of_cookbook_aliases] = true
    end
  end

  # ========================================================================= #
  # === verbose_purge_fhs_libraries_of_this_program
  #
  # This method will notify the user as to what will be done, then
  # delegate towards purge_fhs_libraries_of_this_program().
  # ========================================================================= #
  def verbose_purge_fhs_libraries_of_this_program(i)
    i = RBT.return_alias_program_name_for(i)
    opne "Now purging all #{mediumspringgreen('lib/')} programs "\
         "of `#{sfancy(i)}` at the #{sdir('/usr/lib/')}"\
         " hierarchy."
    purge_fhs_libraries_of_this_program(i)
  end

  # ========================================================================= #
  # === show_registered_keys
  #
  # This delegates towards the Register class.
  # ========================================================================= #
  def show_registered_keys
    RBT::RegisterProgramFilesIntoYamlDatabase.show_registered_keys
  end

  # ========================================================================= #
  # === honour_check_to_ensure_that_the_program_exists?
  # ========================================================================= #
  def honour_check_to_ensure_that_the_program_exists?
    @internal_hash[:honour_check_to_ensure_that_the_program_exists] # This is true by default.
  end

  # ========================================================================= #
  # === can_we_continue?
  # ========================================================================= #
  def can_we_continue?
    @internal_hash[:can_we_continue]
  end

  # ========================================================================= #
  # === we_can_not_continue
  # ========================================================================= #
  def we_can_not_continue(be_verbose = false)
    case be_verbose
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      opne 'The RBT scripts can not continue.'
    end
    @internal_hash[:can_we_continue] = false
  end; alias can_not_continue we_can_not_continue # === can_not_continue
       alias do_not_continue  we_can_not_continue # === do_not_continue

  # ========================================================================= #
  # === we_can_not_continue_and_we_will_not_run_make_install
  # ========================================================================= #
  def we_can_not_continue_and_we_will_not_run_make_install
    we_can_not_continue
    do_not_run_make_install
  end

  # ========================================================================= #
  # === verbose_disable_maximum_speed_for_compilation
  # ========================================================================= #
  def verbose_disable_maximum_speed_for_compilation
    opne 'We will not be using maximum speed.'
    configuration?.use_maximum_speed_for_compilation = false
  end

  # ========================================================================= #
  # === package_rbt_scripts                        (package tag, release tag)
  #
  # You can use this method to package the RBT scripts.
  #
  # A specialized class will do this for us, class PackageRbtScripts.
  #
  # Invocation examples:
  #   ry --package_rbt_scripts
  #   ry --release
  # ========================================================================= #
  def package_rbt_scripts(
      also_exit = false
    )
    RBT::PackageRbtScripts.new
    also_exit = true if also_exit == :then_exit
    exit_program if also_exit
  end

  # ========================================================================= #
  # === convert_to_non_traditional
  #
  # Use this method to convert a program to non-traditional (Gobolinux style)
  # again.
  # ========================================================================= #
  def convert_to_non_traditional(i = program?)
    RBT::ConvertToNonTraditional.new(i)
  end

  # ========================================================================= #
  # === backup_program
  #
  # Backup an existingp rogram under the /Programs/ hierarchy.
  # ========================================================================= #
  def backup_program(
      i = compile_which_program?
    )
    require 'rbt/utility_scripts/backup_program.rb'
    RBT::BackupProgram.new(i)
  end

  # ========================================================================= #
  # === do_not_use_any_compile_time_flags
  #
  # If you do not want to use any compile-time flags, aka CFLAGS, then
  # you can use this method here.
  # ========================================================================= #
  def do_not_use_any_compile_time_flags
    opne 'Setting compile-time flags ('+
         royalblue('CFLAGS')+', '+
         royalblue('LDFLAGS')+
         ') to empty.'
    set_cflags('')
  end

  # ========================================================================= #
  # === run_through_the_required_dependencies
  #
  # We have to sync the dependencies back into class RBT::Compile.
  # ========================================================================= #
  def run_through_the_required_dependencies
    RBT::Errors::MapLineToASpecificError.required_dependency?.each {|this_array|
      register_required_dependency(this_array.first, this_array.last)
    }
  end

  # ========================================================================= #
  # === run_through_the_registered_problems
  #
  # Sync the problems towards class RBT::Compile next.
  # ========================================================================= #
  def run_through_the_registered_problems
    RBT::Errors::MapLineToASpecificError.registered_problems?.each {|do_this_action|
      problem_is do_this_action
    }
  end

  # ========================================================================= #
  # === be_completely_quiet
  # ========================================================================= #
  def be_completely_quiet
    @original_stdout = $stdout
    # $stdout = StringIO.new # This is the redirection.
    $stdout = File.open(IO::NULL, 'w') # This one works better, according to drbrain.
  end

  # ========================================================================= #
  # === do_not_compile
  #
  # This method can be invoked to set the :continue_after_configure_step
  # entry in the internal hash to false.
  #
  # If set to false then we will NOT compile a program. Instead, we will
  # run the configure script and quit after having done so.
  # ========================================================================= # 
  def do_not_compile(
      be_verbose = true
    )
    case be_verbose
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      opne 'We will not continue after configure phase.'
    end
    @internal_hash[:continue_after_configure_step] = false
    @internal_hash[:do_not_compile] = true
    set_do_not_symlink(:be_verbose) # So we don't run advanced symlink.
  end

  # ========================================================================= #
  # === scan_source_archive
  #
  # Scan the complete source archive.
  #
  # Invocation example:
  #   ry --scan_source_archive
  # ========================================================================= #
  def scan_source_archive(
      shall_we_exit = true
     )
    opne 'Scanning for source Archives now.'
    RBT::Cookbooks::ScanSourceArchive.new # bl ScanSourceArchive
    exit_program if shall_we_exit
  end

  # ========================================================================= #
  # === run_through_the_registered_errors
  #
  # Sync the errors towards class RBT::Compile next.
  # ========================================================================= #
  def run_through_the_registered_errors
    RBT::Errors::MapLineToASpecificError.registered_errors?.each {|do_this_action|
      error_is do_this_action
    }
  end

  # ========================================================================= #
  # === run_through_the_behaviour_changes
  #
  # We run through the module that keeps track of the behaviour
  # changes that out to be run in class RBT::Compile.
  # ========================================================================= #
  def run_through_the_behaviour_changes
    RBT::Errors::MapLineToASpecificError.behaviour_changes?.each {|do_this_action|
      self.send(do_this_action)
    }
  end

  # ========================================================================= #
  # === purge_all_man_files_in_the_programs_hierarchy
  #
  # Invoke this method by issuing something like this:
  #   ry --nomanpage
  # ========================================================================= #
  def purge_all_man_files_in_the_programs_hierarchy
    target = programs_directory?
    opne "Now purging all man-pages in `#{sdir(target)}`."
    opne '(Note that we will only work on entries pointed at '\
         'with a Current symlink.)'
    Dir[target+'*'].each {|entry|
      entry << '/' unless entry.end_with? '/'
      current = entry + 'Current'
      if File.symlink? current
        # Ok, the Current -> symlink exists, thus we can continue.
        manpage = File.readlink(current)+'/share/man/'
        manpage = rds(manpage)
        if File.exist? manpage and File.directory? manpage
           opne 'Now removing the directory `'+sdir(manpage)+'`.'
           remove_directory(manpage) unless manpage.squeeze('/') == '/' # Trivial "safeguard".
        end
      end
    }
  end

  # ========================================================================= #
  # === find_all_keys_starting_with
  #
  # This method will find all keys starting with a specific character.
  #
  # Currently we will always exist when we use this method, so returning
  # something is not hugely important.
  # ========================================================================= #
  def find_all_keys_starting_with(
      i
    )
    i.delete!('*') # Get rid of '*' since we no longer need it past this point.
    all_matches = all_programs?.select {|key| key =~ /^#{i}/ } 
    e all_matches.sort.join(', ').split(N).map {|entry| '  '+entry }.join(N)
    e
    opne 'Found a total of '+
         royalblue(all_matches.size.to_s)+
         ' matches.'
    return all_matches
  end

  # ========================================================================= #
  # === verbose_find_all_keys_starting_with
  #
  # This method will feedback all programs that start with a given
  # (input) letter.
  # ========================================================================= #
  def verbose_find_all_keys_starting_with(i)
    opne 'Finding all matches starting with the letter `'+
          sfancy(i.delete('*'))+'` now:'
    e
    find_all_keys_starting_with(i)
    e
  end

  # ========================================================================= #
  # === create_standalone_yaml_file
  #
  # This will create standalone yaml files.
  #
  # Usage example:
  #
  #   ry htop --csyf
  #
  # ========================================================================= #
  def create_standalone_yaml_file
    cliner {
      opne 'Dumping all available recipes next.'
    }
    store_here = RBT.configuration?.recipes_directory
    all_programs.each { |my_program|
      e sfancy("This program is: `")+simp(my_program)+sfancy('`.')
      update_dataset(my_program)
      _ = RBT::Cookbooks::CreateRecipe.new(my_program, :do_not_run_yet)
      _.store_where(store_here)
      _.run
    }
    cliner {
      opne sfancy('Finished dumping all Recipes into ')+sdir(store_here)
      opne "#{N}There were: #{sfancy(all_programs.size.to_s)} of them.#{N}"
    }
  end

  # ========================================================================= #
  # === add_package_user
  #
  # This allows you to add a package-user.
  #
  # Usage example:
  #   ry htop add_package_user
  # ========================================================================= #
  def add_package_user(
      name = compile_which_program?
    )
    _ = '/Users/Packages/'
    cmd_to_run = 'useradd -m -d '+_+name.capitalize+' '+name
    create_directory _ unless File.directory? _
    opne "Adding a new user called `#{simp(name)}`:"
    custom_system cmd_to_run
  end

  # ========================================================================= #
  # === disable_logging
  #
  # We no longer log here.
  # ========================================================================= #
  def disable_logging(
      be_verbose = true
    )
    if be_verbose
      opne 'We will not log.'
    end
    @internal_hash[:may_we_log] = false
  end

  # ========================================================================= #
  # === convert_to_traditional
  #
  # Convert to traditional format via this method here.
  #
  # Invoke this by doing something like:
  #
  #   ry php ctotrad
  #
  # ========================================================================= #    
  def convert_to_traditional(
      i = compile_which_program?
    )
    RBT::ConvertToTraditional.new(i) # bl ConvertToTraditional
  end

  # ========================================================================= #
  # === find_all_archive_types
  # ========================================================================= #
  def find_all_archive_types
    RBT::Cookbooks::FindAllArchiveTypes.new.show_results
  end

  # ========================================================================= #
  # === update_library_names
  #
  # To invoke this method, do something like:
  #   ry htop uln
  # ========================================================================= #
  def update_library_names
    RBT::Cookbooks::RegisteredLibraries.new
  end

  # ========================================================================= #
  # === zero_this_global_flag_file
  # ========================================================================= #
  def zero_this_global_flag_file(i)
    opne "Now resetting all global flags to "\
         "zero again. (File: #{sfile(i)})"
    remove_file(i)
    touch_file(i)
  end

  # ========================================================================= #
  # === clear_global_flags
  #
  # Clear the global files in use that may be stored in the file
  # global_flags.yml.
  # ========================================================================= #
  def clear_global_flags
    _ = RBT.file_global_flags
    if File.exist?(_) and !File.empty?(_)
      zero_this_global_flag_file(_)
    end
    if is_on_roebe? # And on my home system as well.
      _ = ROEBE_FILE_GLOBAL_FLAGS
      zero_this_global_flag_file(_)
    end
  end

  # ========================================================================= #
  # === return_md5
  #
  # md5sum support via this method. It will make use of the class Md5sum.
  # ========================================================================= #
  def return_md5(this_file)
    _ = Md5sum.new(this_file, :dont_run_yet) # bl $RUBY_SRC/md5sum/lib/md5sum/md5sum.rb
    return _.return_md5sum(this_file)
  end

  # ========================================================================= #
  # === use_utf8_encoding_in_env
  # ========================================================================= #
  def use_utf8_encoding_in_env
    ENV['LC_ALL'] = 'en_US.utf8'
  end

  # ========================================================================= #
  # === generate_new_cookbook
  #
  # This method can be used to generate a new cookbook file. It is no
  # longer that useful, but retained mostly for legacy reasons.
  #
  # To generate a new cookbook, do:
  #   ry htop ---generate_new_cookbook
  # ========================================================================= #
  def generate_new_cookbook(
      shall_we_exit = true
    )
    opne 'Will generate a new cookbook file, as instructed.'
    RBT::Cookbooks::CreateNewCookbook.new # bl $RUBY_COOKBOOKS/new_cook*
    exit_program if shall_we_exit
  end

  # ========================================================================= #
  # === uninstall_this_appdir
  #
  # Invocation example:
  #   rbt evince --uninstall-appdir
  # ========================================================================= #
  def uninstall_this_appdir(
      i = compile_which_program?
    )
    opne "Now trying to uninstall the program called `#{teal(i)}`."
    ::RBT::RemoveProgram.new(i)
  end

  # ========================================================================= #
  # === is_on_32_bit_system?
  # ========================================================================= #
  def is_on_32_bit_system?
    TARGET_CPU == 'i686'
  end

  # ========================================================================= #
  # === binary?
  #
  # Wrapper towards answering the question of binary pointing.
  # ========================================================================= #
  def binary?(
      i = compile_which_program?
    )
    i = i.first if i.is_a? Array
    i = i.downcase
    opne "Checking for a binary name called `#{simp(i)}` next:"
    RBT::Cookbooks::RegisteredBinaries.search_for_this_program(i)
  end

  # ========================================================================= #
  # === check_for_latest
  #
  # Method-Wrapper over class LocalCheckForLatest.
  #
  # Invocation example:
  #   ry --checkforlatest
  # ========================================================================= #
  def check_for_latest
    RBT::Cookbooks::LocalCheckForLatest.new # bl $RUBY_COOKBOOKS/lib/cookbooks/local_check_for_latest.rb
  end

  # ========================================================================= #
  # === return_blfs_page_of_this_program
  # ========================================================================= #
  def return_blfs_page_of_this_program(i)
    RBT.return_blfs_page_of_this_program(i)
  end

  # ========================================================================= #
  # === do_not_check_whether_the_program_exists
  # ========================================================================= #
  def do_not_check_whether_the_program_exists
    @internal_hash[:check_whether_the_given_input_is_a_file_and_does_exist] = false
  end; alias do_not_honour_check_to_ensure_that_the_program_exists do_not_check_whether_the_program_exists # === do_not_honour_check_to_ensure_that_the_program_exists 

  # ========================================================================= #
  # === purge_fhs_libraries_of_this_program
  #
  # Get rid of all /usr/lib/ libraries of the respective program at
  # hand.
  #
  # Invocation example:
  #   ry --fhs-purge-libraries=libgc
  # ========================================================================= #
  def purge_fhs_libraries_of_this_program(
      i = compile_which_program?
    )
    if i.nil?
      opne 'Please provide a valid program name to this method.'
      return
    end
    PurgeLibrariesOfThisProgram.new(i)
  end

  # ========================================================================= #
  # === verbose_generate_the_pdf_tutorial
  # ========================================================================= #
  def verbose_generate_the_pdf_tutorial
    opne 'We will generate a .pdf file next which will contain the'
    opne 'instructions on how to use the RBT project.'
    RBT.generate_pdf_tutorial
  end

  # ========================================================================= #
  # === update_binary_names
  #
  # To invoke this method, try:
  #   ry htop update_binary_names
  #   ry --update-binary-names
  # ========================================================================= #
  def update_binary_names(
      also_exit = false
    )
    also_exit = true if also_exit == :then_exit
    require 'rbt/utility_scripts/registered_binaries.rb'
    RBT::Cookbooks::RegisteredBinaries.new
    exit_program if also_exit
  end

  # ========================================================================= #
  # === return_random_program
  # ========================================================================= #
  def return_random_program
    return_available_programs.sample
  end

  # ========================================================================= #
  # === pid?
  # ========================================================================= #
  def pid?
    $?.pid
  end

  # ========================================================================= #
  # === create_description_file
  #
  # Creates a Description file, for use in Gobolinux systems.
  # ========================================================================= #
  def create_description_file(create_where)
    if create_where.include? programs_directory?
      create_where = create_where.delete('-')
    end
    file_location = create_where+'Resources/Description'
    remove_file(file_location) if File.exist? file_location # Remove it if it already exists.
    append_what_into(
      dataset?.description?,
      file_location
    )
  end

  # ========================================================================= #
  # === install_the_dependencies_of_this_program
  #
  # Invocation example:
  #   ry rails --install-its-dependencies
  # ========================================================================= #
  def install_the_dependencies_of_this_program(
      this_program = compile_which_program?
    )
    dependencies = required_deps_on?
    e
    opne 'Next, installing the '+orange(dependencies.size.to_s)+
         ' dependencies of the program `'+
          sfancy(this_program)+'`.'
    e
    opne 'These programs are:'
    e
    dependencies.each {|this_entry|
      e slateblue('  '+this_entry)
    }
    e
    dependencies.each {|this_entry|
      install_that_program = true
      # ===================================================================== #
      # Consider skipping a few.
      # ===================================================================== #
      case this_entry
      when 'ruby'
        target = "#{programs_directory?}Ruby/Current/bin/ruby"
        if File.exist? target
          install_that_program = false
          e 'Skipping ruby as it is already installed at '+sfancy(target)+'.'
        end
      end
      if install_that_program
        # =================================================================== #
        # We ought to not quit after a rubygem has been installed.
        # =================================================================== #
        _ = RBT::Compile.new(this_entry, :do_not_run_yet)
        _.do_not_exit_after_a_rubygem_has_been_installed
        _.run
      end
    }
  end

  # ========================================================================= #
  # === generate_homepage
  #
  # Generate a LFS-like homepage, which is part of the Cookbooks project.
  # ========================================================================= #
  def generate_homepage
    require 'rbt/utility_scripts/generate_homepage/generate_homepage.rb'
    RBT::Cookbooks::GenerateHomepage[]
  end

  # ========================================================================= #
  # === check_for_inclusion
  #
  # We will check the source directory for program-inclusion.
  #
  # To invoke this method, try something like this:
  #
  #    ry htop --check_for_inclusion
  #
  # ========================================================================= #
  def check_for_inclusion
    RBT::Cookbooks::CheckForInclusion.new # bl $RUBY_COOKBOOKS/lib/cookbooks/check_for_inclusion.rb
  end

  # ========================================================================= #
  # === verbose_enable_porg
  # ========================================================================= #
  def verbose_enable_porg
    opne 'We will use the program "'+plum('porg')+
         '" for the "make install" step.'
    enable_porg
  end

  # ========================================================================= #
  # === query_configuration_options
  #
  # This method will query the configuration option of a given program.
  #
  # Invocation example:
  #   ry glibc qco
  # ========================================================================= #
  def query_configuration_options(
      i = compile_this_program?
    )
    RBT::Cookbooks::ShowConfigurationOptions.new(i)
    exit_program
  end

  # ========================================================================= #
  # === output_all_urls_then_exit
  # ========================================================================= #
  def output_all_urls_then_exit
    output_all_urls
    exit_program
  end

  # ========================================================================= #
  # === output_all_urls
  #
  # This method will output all URLs of every registered program.
  #
  # Invocation example:
  #
  #   ry --output_all_urls
  #
  # ========================================================================= #
  def output_all_urls
    dataset = RBT.all_urls(:no_opn).result? # Tap into this method here, without outputting the namespace.
    dataset = dataset.map {|entry|
      entry[0] = '  '+(entry[0]+' ').ljust(30)
      entry.join
    }.join("\n")
    store_where = rbt_log_dir?+'all_program_urls.md'
    File.delete(store_where) if File.exist? store_where # Let's get rid of the old file first.
    opne "Storing into #{sfile(store_where)}"
    save_file(dataset, store_where)
  end

  # ========================================================================= #
  # === generate_machomebrew_formula
  # ========================================================================= #
  def generate_machomebrew_formula(
      i = compile_which_program?
    )
    RBT::GenerateMachomebrewFormula.new(i)
  end

  # ========================================================================= #
  # === reset_the_status_file
  #
  # This method will clear the status.yml file, aka re-set it to zero.
  #
  # To invoke this method, do:
  #   rbt --reset-the-status-file
  # ========================================================================= #
  def reset_the_status_file
    this_file = FILE_STATUS
    dataset = {}
    dataset['currently_compiling_these_programs'] = []
    write_what_into(YAML.dump(dataset), this_file)
  end

  # ========================================================================= #
  # === consider_creating_and_populating_the_pkgconfig_directory
  # ========================================================================= #
  def consider_creating_and_populating_the_pkgconfig_directory
    if uses_an_appdir_prefix?
      CreateAndPopulatePkgconfigDirectory.new(appdir_prefix?)
    end
  end

  # ========================================================================= #
  # === stop_on_error?
  # ========================================================================= #
  def stop_on_error?
    STOP_ON_ERROR
  end

  # ========================================================================= #
  # === enable_stow
  #
  # This method will enable stow.
  # ========================================================================= #
  def enable_stow
    configuration?.use_stow = true
    opne 'Enabling stow next.'
    configuration?.save # And store this.
  end

  # ========================================================================= #
  # === disable_stow
  # ========================================================================= #
  def disable_stow
    configuration?.use_stow = false
    opne 'Disabling stow next.'
    configuration?.save
  end

  # ========================================================================= #
  # === set_delay_before_continuing                               (sleep tag)
  #
  # Ensure that the argument passed to us is correct.
  # ========================================================================= #
  def set_delay_before_continuing(n_seconds)
    sleep(n_seconds.to_i)
  end

  # ========================================================================= #
  # === configuration?
  # ========================================================================= #
  def configuration?
    RBT.configuration?
  end; alias config? configuration? # === config?

  # ========================================================================= #
  # === generate_slack_desc_files_for_every_program
  # ========================================================================= #
  def generate_slack_desc_files_for_every_program
    target = "#{rbt_log_directory?}slackware_desc_files/"
    opnn; mkdir target, :be_verbose
    cd target
    all_programs?.each {|entry|
      generate_slack_desc_file_for_this_program(entry)
    }
  end

  # ========================================================================= #
  # === all_available_programs?
  #
  # To invoke and test this:
  #   ry htop all_available_programs?
  # ========================================================================= #
  def all_available_programs?
    e
    opne 'These programs are available:'
    e
    RBT.feedback_available_programs
  end

  # ========================================================================= #
  # === show_last_compiled_program
  #
  # Use this method to show the last compiled program.
  # ========================================================================= #
  def show_last_compiled_program(
      shall_we_exit = false
    )
    shall_we_exit = true if shall_we_exit == :then_exit
    opne 'The last compiled program was `'+
          sfancy(last_compiled_program?)+'` (as registered'
    opne "in the file #{sfile(RBT.file_last_compiled_program)})."
    exit if shall_we_exit
  end

  # ========================================================================= #
  # === load_last_compiled_program
  #
  # This essentially does the following:
  #
  #   x = YAML.load_file('/Depot/Temp/rbt_logs/last_compiled_programs.yml')
  #
  # To obtain this information from the commandline, try:
  #
  #   rbt --last-compiled-program?
  #
  # ========================================================================= #
  def load_last_compiled_program
    _ = RBT.file_compiled_programs
    if File.exist? _
      @last_compiled_program = YAML.load_file(_)
      result = @last_compiled_program
      # ===================================================================== #
      # If we store it as an Array, the last compiled program is the
      # one that comes last in that Array, too.
      # ===================================================================== #
      result = result.last if result.is_a? Array
      # ===================================================================== #
      # It may include a '#' character, in which case we will treat this
      # as a comment:
      # ===================================================================== #
      if result.include? '#'
        result = result[0, result.index('#')].strip
      end
      return result # <- And return the result here.
    else
      opne "No file exists at `#{sfile(_)}`."
      opne 'Thus, we can not load anything.'
      exit_program
    end
  end; alias last_compiled_program? load_last_compiled_program # === last_compiled_program?

  # ========================================================================= #
  # === compiled_programs?
  #
  # This will grab the full dataset of compiled_programs.yml
  # ========================================================================= #
  def compiled_programs?
    _ = RBT.file_compiled_programs
    if File.exist? _
      return YAML.load_file(_)
    else
      opne "The file at #{sfile(_)} does not exist."
    end
  end

  # ========================================================================= #
  # === does_this_file_exist?
  #
  # This method checks whether the given-input argument, an assumed
  # path to a file, exists.
  #
  # The reason why this method exists is because it was necessary to
  # do a "does this file exist" check in other directories rather
  # than the current working directory.
  # ========================================================================= #
  def does_this_file_exist?(
      this_file = FILE_SCONS
    )
    this_file = this_file.dup # <- We must always work on a copy.
    # ======================================================================= #
    # We have to check whether the cookbook_dataset has been initialized.
    # If it has not been initialized yet, we will not query whether we
    # ought to use a build directory or not.
    # ======================================================================= #
    if cookbook_dataset? and use_build_directory?
      this_file.prepend('../')
    end
    super(this_file)
  end

  # ========================================================================= #
  # === last_error_line?
  # ========================================================================= #
  def last_error_line?
    sys_command_string?.last
  end

  # ========================================================================= #
  # === run_through_the_registered_problems_registered_errors_and_behaviour_changes
  # ========================================================================= #
  def run_through_the_registered_problems_registered_errors_and_behaviour_changes
    run_through_the_registered_problems
    run_through_the_registered_errors
    run_through_the_behaviour_changes
    run_through_the_erroneous_libtool_entries
    run_through_the_required_dependencies
  end

  # ========================================================================= #
  # === program_has_to_be_compiled?
  #
  # The query-method program_has_to_be_compiled? determines whether our
  # program has to be compiled or whether it does not have to be compiled.
  #
  # For example, ruby .gem files typically do not have to be compiled,
  # whereas C/C++ projects have to be compiled.
  # ========================================================================= #
  def program_has_to_be_compiled?
    cookbook_dataset? and cookbook_dataset_program_has_to_be_compiled?
  end; alias is_compiled?                    program_has_to_be_compiled? # === is_compiled?
       alias has_to_be_compiled?             program_has_to_be_compiled? # === has_to_be_compiled?
       alias had_to_be_compiled?             program_has_to_be_compiled? # === had_to_be_compiled?
       alias the_program_has_to_be_compiled? program_has_to_be_compiled? # === the_program_has_to_be_compiled?

  # ========================================================================= #
  # === consider_using_this_file_for_the_configure_options_that_are_to_be_used
  # ========================================================================= #
  def consider_using_this_file_for_the_configure_options_that_are_to_be_used(this_file)
    if File.exist? this_file
      opnn; e 'Making use of the file '+sfile(this_file)+' for configure-options.'
      dataset = File.read(this_file).strip
      opnn; e 'The options are:'
      e
      e sfancy("  #{dataset}")
      e
      clear_configure_options
      @internal_hash[:configure_options] = dataset
    else
      opnn; no_file_exists_at(this_file)
    end
  end

  # ========================================================================= #
  # === clear_configure_options
  # ========================================================================= #
  def clear_configure_options
    @internal_hash[:configure_options] = nil
    cookbook_dataset?.clear
  end

  # ========================================================================= #
  # === compile_all_programs_that_belong_to_this_tag
  #
  # Invocation examples:
  #
  #   rbt --compile-this-tag=gnome
  #   rbt --compile-this-tag=kde
  #   rbt --compile-this-tag=kde5
  #   rbt --compile-this-tag=game
  #
  # ========================================================================= #
  def compile_all_programs_that_belong_to_this_tag(i)
    # ======================================================================= #
    # (1) first obtain all programs that belong to a particular tag
    # ======================================================================= #
    these_programs = RBT.return_array_of_this_tag(i)
    # ======================================================================= #
    # (2) then set to the main array
    # ======================================================================= #
    set_compile_these_programs(
      these_programs
    )
  end

  # ======================================================================= #
  # === compile_all_games
  #
  # This method can be used to compile all registered games.
  # ======================================================================= #
  def compile_all_games
    compile_all_programs_that_belong_to_this_tag(:game)
  end

  # ========================================================================= #
  # === register_configure_line_into_configure_database
  #
  # This method will register the complete configure-line used by the
  # program at hand, into a yaml "database". This is just a single
  # .yml file that keeps track of all configure options used when
  # compiling a program.
  # ========================================================================= #
  def register_configure_line_into_configure_database
    compile_this_program = compile_which_program?
    store_this = full_configure_command?.dup
    unless store_this.empty?
      # ===================================================================== #
      # First load up the old "database" if it exists.
      # ===================================================================== #
      into = RBT.file_configure_database?
      hash = {}
      # ===================================================================== #
      # Load the dataset from the file if it already exists.
      # ===================================================================== #
      if File.exist? into
        hash.update(YAML.load_file(into))
      end
      # ===================================================================== #
      # We do not need to delete any old entry, because the following line
      # will overwrite the old content anyway, if it exists.
      # ===================================================================== #
      hash[compile_this_program] = store_this
      what = YAML.dump(hash)
      opne 'Storing the configure command'
      opne "  `#{sfancy(store_this)}`"
      opne "into the file `#{sfile(into)}`."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === try_to_fix_all_shebangs
  #
  # Use this to fix shebangs of the RBT project.
  # ========================================================================= #
  def try_to_fix_all_shebangs
    begin
      require 'roebe/classes/shebang_fixer.rb'
      cliner {
        opne 'Trying to fix all Shebangs now:'
        entries = Dir["#{base_directory?}**/**.rb"]
        entries.each { |file|
          file = ENV['RBT_DIR'].to_s+'/'+file
          Roebe::ShebangFixer.new(file)
        }
      }
    rescue LoadError
      e 'class Roebe::ShebangFixer is not available.'
    end
  end

  # ========================================================================= #
  # === consider_using_maximum_speed_for_compilation
  #
  # This method also has to check whether we will compile the given
  # program at all or whether we will not.
  # ========================================================================= #
  def consider_using_maximum_speed_for_compilation
    if use_maximum_speed_for_compilation? and
       the_program_has_to_be_compiled?
      opne "The configuration file "\
           "#{sfile('use_maximum_speed_for_compilation.yml')}"\
           " specified"
      opne 'that maximum speed should be used for for compilation.'
      opne "This means to pass the #{orange('-j')} option "\
           "to the program #{slateblue('make')},"
      opne 'with the number of processors this computer has.'
      speed_up_the_compilation
    end
  end

  # ========================================================================= #
  # === speed_up_the_compilation
  #
  # We can speed up compilation via the -j option to "make".
  #
  # See:
  #
  #   https://www.math-linux.com/linux/tip-of-the-day/article/speedup-gnu-make-build-and-compilation-process
  #
  # ========================================================================= #
  def speed_up_the_compilation
    append_parameters_to_make(
      " -j#{return_n_processors.to_s}"
    )
  end

  # ========================================================================= #
  # === try_to_output_how_many_processors_this_computer_has
  #
  # To try this method on the commandline, do:
  #   rbt --n-cores?
  # ========================================================================= #
  def try_to_output_how_many_processors_this_computer_has
    _ = FILE_CPUINFO
    if File.exist? FILE_CPUINFO
      opne 'This computer has '+
           lightseagreen(
             return_n_processors_of_this_computer.to_s+' processors'
           )+'.'
    else
      opne 'Can not determine the number of processors as '
      opne "there is no file at #{sfile(FILE_CPUINFO)}."
    end
  end

  # ========================================================================= #
  # === return_n_processors_of_this_computer
  #
  # This method will return how many processors this computer has. This
  # can then be used to automatically determine the maximum option
  # given to the "make" binary.
  # ========================================================================= #
  def return_n_processors_of_this_computer
    _ = FILE_CPUINFO
    if File.exist? _
      selected_dataset = File.readlines(_).select {|line|
        line.start_with? 'processor'
      }
      selected_dataset.size
    else
      0
    end
  end; alias return_n_processors return_n_processors_of_this_computer # === return_n_processors

end; end