#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/postinstallation_step.rb'
# =========================================================================== #
require 'rbt/utility_scripts/post_install.rb'
require 'rbt/utility_scripts/symlink_into_usr_lib_directory.rb'
require 'rbt/utility_scripts/postinstallation_step/postinstallation_step.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === run_postinstall_actions?
  #
  # We will default to the cookbook dataset - but this will be overruled
  # if there is a corresponding entry in the @internal_hash variable.
  # ========================================================================= #
  def run_postinstall_actions?
    _ = cookbook_dataset_postinstall?
    unless @internal_hash[:run_postinstall_actions].nil?
      _ = @internal_hash[:run_postinstall_actions]
    end
    _
  end; alias run_postinstall_steps? run_postinstall_actions? # === run_postinstall_steps?

  # ========================================================================= #
  # === display_postinstall_dataset
  #
  # This method will display the postinstall actions that will be done
  # for the given program at hand.
  #
  # Invocation examples:
  #
  #   ry perl p?
  #   ry docbookxsl --display_postinstall
  #   ry libraw1394 --postinstallation_steps?
  #   ry bash --postinstallation_steps?
  #
  # ========================================================================= #
  def display_postinstall_dataset
    result = cookbook_dataset_postinstall?
    unless result.empty?
      opne "The postinstall instructions for `#{royalblue(program?)}` are:"
      result = [result].flatten.compact # <- We need an Array past this point.
      result.each {|entry|
        e
        e "  #{yellowgreen(entry)}" # Display with a slight padding.
        e
      }
      opne 'Also assigning the above result to the xorg buffer.'
      set_xorg_buffer(result.join(N))
    end
  end

  # ========================================================================= #
  # === skip_postinstall_actions
  #
  # Whether to skip postinstall actions or whether we don't.
  # ========================================================================= #
  def skip_postinstall_actions
    @internal_hash[:run_postinstall_actions] = false
  end; alias do_skip_postinstall_actions    skip_postinstall_actions # === do_skip_postinstall_actions
       alias do_not_run_postinstall_actions skip_postinstall_actions # === do_not_run_postinstall_actions
       alias no_postinstall_actions         skip_postinstall_actions # === no_postinstall_actions
       alias no_postinstall_step            skip_postinstall_actions # === no_postinstall_step

  # ========================================================================= #
  # === set_postinstallation
  #
  # Note that the postinstallation steps will always be stored as
  # an Array.
  # ========================================================================= #
  def set_postinstallation(i)
    @internal_hash[:postinstallation] = [i].flatten.compact
  end; alias set_postinstall set_postinstallation # === set_postinstall

  # ========================================================================= #
  # === postinstallation?
  # ========================================================================= #
  def postinstallation?
    _ = cookbook_dataset_postinstall?
    # ======================================================================= #
    # The following part is, as of January 2019, only added when we have
    # modified this variable through the method set_user_prefix(). The
    # user can (currently) not modify this variable on his/her own.
    # ======================================================================= #
    unless @internal_hash[:postinstallation].nil?
      _ = @internal_hash[:postinstallation]
    end
    _ # Return it here.
  end; alias postinstall? postinstallation? # === postinstall? 

  # ========================================================================= #
  # === consider_performing_postinstallation_actions (postinstall tag, post tag)
  #
  # Postinstall actions are optional - they are done after the "make install"
  # step has finished.
  #
  # This method here handles postinstall-related actions, like starting
  # a service after compilation has finished, or removing a directory 
  # after installation - these sorts of actions.
  #
  # A song may also be played if the configuration option tells us to do
  # so. (This may give us an audio-cue that compilation has finished.
  # I no longer make use of this, but if others want to, here we go)
  #
  # We make use of RBT::SymlinkProgram to symlink the AppDir properly,
  # in the event that we compile into AppDirs.
  #
  # Since as of February 2019, a separate class handles these
  # postinstallation steps.
  #
  # As a summary and an overview, these are valid postinstall actions:
  #
  #   - Running cleanup actions
  #   - Renaming a shell (optional)
  #   - Playing a song (when the configuration option told us to do so)
  #   - Symlinking files after installation
  #   - Copying several files to another location
  #   - Deleting any .la files if the user specified so (this is
  #     now the default; RBT::Compile will always try to remove
  #     .la files under an AppDir prefix, unless the user has
  #     specified otherwise)
  #
  # ========================================================================= #
  def consider_performing_postinstallation_actions
    # ======================================================================= #
    # Only run these actions if we actually do run postinstall actions:
    # ======================================================================= #
    if run_postinstall_actions? and can_we_continue?
      # ===================================================================== #
      # Next delegate towards class RBT::PostinstallationStep.
      # ===================================================================== #
      RBT::PostinstallationStep.new(
        return_hash_for_the_postinstallation_step
      )
    end
  end; alias do_postinstall_actions consider_performing_postinstallation_actions # === do_postinstall_action

end; end