#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/prefix.rb'
# =========================================================================== #
require 'rbt/prefix/prefix.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === use_non_traditional_prefix
  #
  # This method will make use of AppDir-style prefixes, such as
  # "/Programs/Htop/2.0.2".
  # ========================================================================= #
  def use_non_traditional_prefix
    do_compile_in_a_non_traditional_way(
      nil, :be_quiet
    )
  end; alias do_run_via_nontraditional_way use_non_traditional_prefix # === do_run_via_nontraditional_way
       alias do_use_an_appdir_prefix       use_non_traditional_prefix # === do_use_an_appdir_prefix

  # ========================================================================= #
  # === do_compile_in_a_traditional_way
  #
  # Use this method when you want to use '/usr/' as the main prefix of
  # compilation.
  # ========================================================================= #
  def do_compile_in_a_traditional_way(
      be_verbose = true
    )
    if can_we_continue?
      @internal_hash[:we_tried_to_compile_this_program_n_times] += 1
      if be_verbose
        opne 'Using a "'+sfancy(:traditional)+'" strategy for compilation '\
             '(which means prefix '+sdir('/usr/')+').'
      end
      set_user_prefix '/usr/' # Specify a user-prefix here.
    end
  end; alias enable_traditional_compilation do_compile_in_a_traditional_way # === enable_traditional_compilation

  # ========================================================================= #
  # === prefix?
  #
  # This will properly handle the prefix in use. It is the general
  # getter-method which should be used whenever we wish to obtain
  # the prefix in user.
  # ========================================================================= #
  def prefix?
    if @internal_hash[:user_prefix] # aka: if user_prefix?
      use_this_as_prefix = @internal_hash[:user_prefix]
    else
      use_this_as_prefix = @internal_hash[:prefix]
      # ===================================================================== #
      # Or we could try:
      # ===================================================================== #
      #  use_this_prefix = cookbook_prefix?
    end
    unless use_this_as_prefix.end_with? '/'
      use_this_as_prefix = use_this_as_prefix.dup
      use_this_as_prefix << '/'
    end if use_this_as_prefix.is_a? String
    use_this_as_prefix # Return it here.
  end; alias use_this_prefix?      prefix? # === use_this_prefix?
       alias use_which_prefix?     prefix? # === use_which_prefix?
       alias real_prefix?          prefix? # === real_prefix?
       alias prefix                prefix? # === prefix
       alias return_proper_prefix? prefix? # === return_proper_prefix?
       alias return_proper_prefix  prefix? # === return_proper_prefix

  # ========================================================================= #
  # === compile_into_the_home_directory
  # ========================================================================= #
  def compile_into_the_home_directory
    set_user_prefix('$HOME')
  end

  # ========================================================================= #
  # === is_this_input_an_appdir_prefix?
  #
  # This method will return true if the given input is an AppDir-prefix;
  # and false otherwise.
  # ========================================================================= #
  def is_this_input_an_appdir_prefix?(
      i = prefix?.to_s
    )
    if i and !i.is_a?(Symbol)
      i.to_s.include?(programs_directory?) and (i.to_s =~ /\d+/) # And "has a number".
    else
      false
    end
  end; alias is_an_appdir_prefix? is_this_input_an_appdir_prefix? # === is_an_appdir_prefix?

  # ========================================================================= #
  # === set_user_prefix_to_nil
  # ========================================================================= #
  def set_user_prefix_to_nil
    set_user_prefix(nil)
  end

  # ========================================================================= #
  # === user_prefix?
  #
  # The user_prefix? is the one that the user may modify.
  # ========================================================================= #
  def user_prefix?
    @internal_hash[:user_prefix]
  end

  # ========================================================================= #
  # === use_default_chroot_as_prefix
  #
  # Here we will use the default chroot as prefix. On my system this
  # defaults to /Depot/Chroot/.
  # ========================================================================= #
  def use_default_chroot_as_prefix(
      i = '/Depot/Chroot/'
    )
    set_user_prefix(i)
  end

  # ========================================================================= #
  # === use_opt_as_prefix
  # ========================================================================= #
  def use_opt_as_prefix
    set_user_prefix('/opt/')
  end

  # ========================================================================= #
  # === do_use_appdir_prefix
  #
  # This method will call do_compile_in_a_non_traditional_way, but it
  # will always be quiet.
  # ========================================================================= #
  def do_use_appdir_prefix
    do_compile_in_a_non_traditional_way(:be_quiet)
  end

  # ========================================================================= #
  # === appdir_prefix?                                        (appdir prefix)
  #
  # This method will return the (assumed) AppDir prefix for a given
  # program at hand.
  #
  # The program version must be set before calling this method. If
  # the program_version? has not been defined properly then the
  # method here may return an incorrect result.
  #
  # This method MUST be guaranteed to return a trailing '/' character.
  # ========================================================================= #
  def appdir_prefix?(
      i                        = compile_which_program?,
      use_this_program_version = :default
    )
    if cookbook_dataset?.nil?
      # ===================================================================== #
      # We do the next method primarily so that we can find
      # a proper program version.
      # ===================================================================== #
      initialize_cookbook_dataset_for(i)
    end
    case use_this_program_version
    when :default
      use_this_program_version = program_version?
    end
    # ======================================================================= #
    # We need a safeguard next, as the program version may be nil.
    # ======================================================================= #
    if use_this_program_version.nil?
      use_this_program_version = cookbook_dataset_program_version?
    end
    result = rds(
      programs_directory?.to_s+
      i.to_s.capitalize.to_s+'/'+
      use_this_program_version+'/'
    )
    # ======================================================================= #
    # === Handle GoboLinux next
    # ======================================================================= #
    if is_on_gobolinux?
      # ===================================================================== #
      # If we are on GoboLinux, then we have to honour the other
      # variant called program_compile_name? instead.
      # ===================================================================== #
      result = rds(
        programs_directory?.to_s+
        program_compile_name?.to_s+'/'+
        use_this_program_version.to_s+'/'
      )
    end
    result
  end; alias static_appdir_prefix?         appdir_prefix? # === static_appdir_prefix?
       alias return_appdir_prefix          appdir_prefix? # === return_appdir_prefix
       alias return_non_traditional_prefix appdir_prefix? # === return_non_traditional_prefix

  # ========================================================================= #
  # === do_compile_in_both_traditional_and_non_traditional_manner
  #
  # This method combines both main strategies for compilation - first,
  # into the /usr/ prefix, and then into the appdir prefix.
  # ========================================================================= #
  def do_compile_in_both_traditional_and_non_traditional_manner
    opne 'We will first compile in the traditional way, and then after that'
    opne 'we will compile in the non-traditional (appdir) way.'
    do_compile_in_a_traditional_way # First, compile in a traditional way.
    @internal_hash[:extra_compilation] = true
  end

  # ========================================================================= #
  # === compile_into_the_home_directory_as_appdir
  #
  # This method will enable compilation into the user's home directory,
  # via AppDir style.
  #
  # Invocation example:
  #   rbt xwd --homedir
  # ========================================================================= #
  def compile_into_the_home_directory_as_appdir
    set_user_prefix('$HOME'+static_appdir_prefix?)
  end

  # ========================================================================= #
  # === is_appdir_prefix?
  #
  # This method will return true if we use an AppDir prefix, such as
  # one that includes the programs directory (e. g. the string "/Programs/",
  # and false otherwise.
  # ========================================================================= #
  def is_appdir_prefix?(
      use_this_prefix = programs_dir?
    )
    prefix?.include?(use_this_prefix)
  end; alias program_uses_an_appdir_prefix? is_appdir_prefix? # === program_uses_an_appdir_prefix?

  # ========================================================================= #
  # === do_compile_in_a_non_traditional_way
  #
  # We enable Gobolinux-style compilation here, as in AppDir-like
  # compilation.
  # ========================================================================= #
  def do_compile_in_a_non_traditional_way(
      optional_command_given = nil,
      be_verbose             = true
    )
    if can_we_continue?
      @internal_hash[:counter_called_traditional_or_non_traditional_compilation] += 1
      case optional_command_given
      when :be_silent,
           :be_quiet
        be_verbose = false
      end
      case be_verbose
      when :be_silent,
           :be_quiet
        be_verbose = false
      end
      if be_verbose
        opne 'Overruling any potential traditional setting.'
      end
      set_use_this_prefix :use_ntrad_prefix_at_a_later_time
    end
  end; alias enable_non_traditional_compilation do_compile_in_a_non_traditional_way # === enable_non_traditional_compilation
       alias set_standalone_compilation         do_compile_in_a_non_traditional_way # === set_standalone_compilation # This is a legacy alias.
       alias compile_as_appdir                  do_compile_in_a_non_traditional_way # === compile_as_appdir
       alias ntrad                              do_compile_in_a_non_traditional_way # === ntrad

  # ========================================================================= #
  # === set_user_prefix                      (set_prefix tag, set prefix tag)
  #
  # This method may only be modified by the user, or internally if we know
  # what we are doing.
  #
  # If set to non-nil then the input argument will become the new
  # user_prefix - in other words, the prefix into which we will
  # compile into.
  #
  # A specialized class called RBT::Prefix will handle the prefix.
  # ========================================================================= #
  def set_user_prefix(
      i = :use_ntrad_prefix_at_a_later_time
    )
    if i.is_a? Symbol
      # ===================================================================== #
      # This clause will handle Symbols such as :app_dir_like or
      # :use_ntrad_prefix_at_a_later_time.
      # ===================================================================== #
      case i.to_s # case tag
      # ===================================================================== #
      # === non_traditional
      # ===================================================================== #
      when /^app(_|-)?dir$/,
           /^app(_|-)?dir(_|-)?like$/,
           'non_traditional','gobolinux','ntrad','false',
           'default',
           'app_dir_like','',
           'return_appdir_path','f',
           'appdir_prefix',
           /^use(_|-)?appdir(_|-)?prefix$/i
        i = return_non_traditional_prefix(internal_program_name?) # defined in this file here.
        # =================================================================== #
        # Since as of May 2014 we get rid of all '-'
        # =================================================================== #
        i.delete!('-') if i.include? '-'
      end
    end
    if i.is_a? String
      # ===================================================================== #
      # Handle the situation when we input a String that starts with the
      # character ':'. This will be assumed to be a shortcut to an
      # environment variable.
      # ===================================================================== #
      if i.start_with?(':')
        # ===================================================================== #
        # Next, we must add the proper program_version. This is either the
        # program_version at hand - or it is an ENV variable if it exists.
        # We check for the ENV variable first.
        # ===================================================================== #
        this_key = (i.delete(':').upcase+'_PREFIX').dup
        if ENV.has_key? this_key
          # ================================================================= #
          # The environment variables must be UPCASED and have appended
          # the string '_PREFIX'.
          # ================================================================= #
          i = ENV[this_key].dup.to_s
        else
          # ================================================================= #
          # Assume a special prefix variant targetting the /Programs/
          # hierarchy.
          # ================================================================= #
          i = programs_directory?+i.to_s.delete(':').capitalize+'/'
          i << program_version?.dup.to_s
        end
      end
      # ===================================================================== #
      # === Convert environment variables, if given
      # ===================================================================== #
      if i.is_a?(String)
        if i.include?('$')
          i = convert_env_variable(i) # Replace ENV variables here.
          i << '/' unless i.end_with? '/'
        end
        if i.include? '--'
          if i.include? '--prefix=' # Input is assumed to be like '--prefix=/Depot/j'
            i = i.split('=').last # <- Obtain just the last part in this case.
          end
        end
      end
      # ===================================================================== #
      # Next, perform some sanitize operations.
      # ===================================================================== #
      case i # case tag
      # ===================================================================== #
      # === trad
      # ===================================================================== #
      when /^trad$/
        i = '/usr/'
      # ===================================================================== #
      # === --opt_prefix
      # ===================================================================== #
      when /^-?-?opt(_|-)?prefix$/,
           :opt
        i = '/opt/'
      # ===================================================================== #
      # === ULOCAL
      # ===================================================================== #
      when 'ULOCAL'
        i = '/usr/local/'
      end
      if i.empty?
        i = RBT::Prefix.new
      else
        i = RBT::Prefix.new(i)
      end
    elsif i.is_a? Symbol
      case i
      when :use_ntrad_prefix_at_a_later_time
        # ===================================================================== #
        # This is a special case, which we will leave as-is - we will
        # determine the real prefix here at a later time.
        # ===================================================================== #
      else
        i = RBT::Prefix.new(i)
      end
    end
    # ======================================================================= #
    # Unfreeze frozen Strings.
    # ======================================================================= #
    if i.is_a? String and i.frozen?
      i = i.dup # "unfreeze" it.
    end
    # ======================================================================= #
    # === Ensure trailing '/' for directories.
    # ======================================================================= #
    unless i.to_s.end_with? '/' # .to_s to protect against Symbols given.
      i << '/' if File.directory?(i.to_s)
    end
    # ======================================================================= #
    # We must rewrite the program version in the following case. The
    # input may be like this:
    #  "/Programs/Gobjectintrospection/1.53.4/"
    # ======================================================================= #
    if use_this_specific_program_version?
      if i.is_a?(String) and i.include?(programs_directory?)
        use_this_regex = /(\d{1,2}\.?\d{1,2}\.?\d{1,2}\/)$/ # See: http://rubular.com/r/PPb2Y96XfF
        i.sub!(use_this_regex, use_this_specific_program_version?)
        i << '/' unless i.end_with? '/'
      elsif i.is_a?(Symbol)
        i = use_this_specific_program_version?
      end
    end
    # ======================================================================= #
    # Next, we must check for post-install actions and modify these,
    # if they exist. We only do so if we have an AppDir-prefix.
    # ======================================================================= #
    if is_an_appdir_prefix?(i)
      consider_modifying_postinstall
    end
    @internal_hash[:user_prefix] = i
  end; alias set_use_this_prefix set_user_prefix # === set_use_this_prefix
       alias prefix=             set_user_prefix # === prefix=
       alias set_prefix          set_user_prefix # === set_prefix

end; end