#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file here holds code, as part of RBT::Compile, which handles
# specific "problems", in particular during GNU configure invocations,
# and more serious problems, also called "errors" (aka "critical
# problems").
#
# Now - what does this mean?
#
# For instance, consider that the configure script determines out that you
# are missing some .h header file or some linking can not be done. Then
# we should notify the user about this problem in a constructive,
# useful manner as well - or at the least be able to do so.
#
# The configure scripts are often extremely cryptic to a user, so this code
# here also attempts to make it easier to discover what kind of problems
# are the issue. Some errors such as faulty libtool files, can sometimes
# be automatically corrected too.
#
# For each of the "triumvirate" of compiling, that is "configure",
# "make" and "make install" steps, we can set success or failure
# individually.
# =========================================================================== #
# require 'rbt/compile/problems_and_errors.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/error_message.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === errors?
  # ========================================================================= #
  def errors?
    RBT.error_message?
  end; alias error_is?      errors? # === error_is?
       alias error_message? errors? # === error_message?

  # ========================================================================= #
  # === error_is                                               (error_is tag)
  #
  # This registered an error. An error is always a problem, but some
  # problems allow us to continue the installation steps. An error
  # on the other hand will end the installation procedure.
  # ========================================================================= #
  def error_is(
      this_error = :meson_could_not_be_found
    )
    problem_is(this_error)
    an_error_has_occurred
    # ======================================================================= #
    # Register the error on the main RBT namespace as well.
    # ======================================================================= #
    RBT.append_error_message(
      this_error
    )
  end

  # ========================================================================= #
  # === no_error_was_encountered?
  # ========================================================================= #
  def no_error_was_encountered?
    !an_error_has_occurred?
  end; alias has_been_installed_successfully? no_error_was_encountered? # === has_been_installed_successfully?

  # ========================================================================= #
  # === an_error_has_occurred?
  #
  # Query-method over as to whether an error has occurred. 
  # ========================================================================= #
  def an_error_has_occurred?
    @internal_hash[:an_error_has_occurred]
  end; alias has_an_error_occurred?     an_error_has_occurred? # === has_an_error_occurred?
       alias did_we_encounter_an_error? an_error_has_occurred? # === did_we_encounter_an_error?

  # ========================================================================= #
  # === problem_is
  #
  # Keep in mind that not every problem is an error. An error will indicate
  # that copilationg will (or already has) fail(ed).
  #
  # An example for an error may be :missing_headers, which indicates that
  # some .h files may be missing (or could not be found, anyway).
  #
  # An example for a problem may be some add-on failing, without this
  # being a "critical problem" (aka an error). 
  # ========================================================================= #
  def problem_is(
      i                          = nil,
      optional_extra_information = nil
    )
    case i
    when :default,
         :no_problem # This is the default too. The nil value will be used in these cases.
      i = nil
    end
    if i.is_a? Array
      i = i.first # Only use the first Array entry.
    end
    if optional_extra_information
      @internal_hash[:problem] = [i, optional_extra_information]
    else # This here is the default - it is a LOT more common.
      @internal_hash[:problem] = i
    end
  end

  # ========================================================================= #
  # === problem?
  # ========================================================================= #
  def problem?
    @internal_hash[:problem]
  end; alias problems?          problem? # === problems?
       alias problem_is?        problem? # === problem_is?
       alias registered_errors? problem? # === registered_errors?

  # ========================================================================= #
  # === an_error_has_occurred
  #
  # This is a setter-method.
  # ========================================================================= #
  def an_error_has_occurred
    we_can_not_continue_and_we_will_not_run_make_install
    @internal_hash[:an_error_has_occurred] = true
    the_program_has_failed_to_install
  end

  # ========================================================================= #
  # === check_for_common_problems_and_common_errors
  # ========================================================================= #
  def check_for_common_problems_and_common_errors
    check_for_common_problems
    check_for_common_errors
  end

  # ========================================================================= #
  # === check_for_common_problems                 (problems tag, problem tag)
  #
  # This method will check for some problems and offer ways to resolve
  # these issues. For this to work, the particular problem at hand
  # has to be registered in the following case/when menu.
  #
  # This method is for checking problems - but for the time being (Nov 2018)
  # we will also check for common errors within this method. This is perhaps
  # not ideal, but simplifies the code for now. In the future this may
  # have to be changed (or the method may have to be renamed).
  # ========================================================================= #
  def check_for_common_problems
    _ = problems?
    if _
      # ===================================================================== #
      # We will only handle registered Symbols next.
      # ===================================================================== #
      if _.is_a? Symbol
        case _ # case tag
        # =================================================================== #
        # === Libtool could not find the library
        # =================================================================== #
        when :libtool_could_not_find_the_library
          if is_on_roebe?
            unless @internal_hash[:invalid_libtool_archive].nil?
              line = @internal_hash[:invalid_libtool_archive]
              try_to_autocorrect_this_erroneous_libtool_entry(line)
            end
          end
        # =================================================================== #
        # === Encountered an invalid libtool archive
        # =================================================================== #
        when :encountered_an_invalid_libtool_archive
          if is_on_roebe?
            unless @internal_hash[:invalid_libtool_archive].nil?
              line = @internal_hash[:invalid_libtool_archive]
              try_to_autocorrect_this_erroneous_libtool_entry(line)
            end
          end
        # =================================================================== #
        # === Libtool can not install to another prefix
        # =================================================================== #
        when :libtool_can_not_install_to_another_prefix
          opne 'Libtool apparently can not install to another prefix.'
          opne 'The main reason for this is usually that the configure script'
          opne 'was run from a directory before, but with another --prefix.'
          opne 'Try to run "make clean" in the base directory or remove the'
          opne 'extracted archive, in order to try to fix this problem.'
          if is_on_roebe?
            opne 'Removing the extracted archive next.'
            remove_the_extracted_archive
          end
        # =================================================================== #
        # === missing_header_ladspa
        # =================================================================== #
        when :missing_header_ladspa
          opne 'The header ladspa.h is missing. This is part of the '\
               'ladspa-sdk package.'
          opne 'The remote URL should be at: '+
                sfancy(return_url_for(:ladspa))
          e
          e 'This program can probably be installed via:'
          e
          e "  #{sfancy('rbt ladspa')}"
          e
        # =================================================================== #
        # === Incomplete configure error via cmake
        #
        # This entry point is for cmake-related errors that are not further
        # specified - in other words, generic cmake-related errors.
        # =================================================================== #
        when :incomplete_configure_error_via_cmake
          do_skip_postinstall_actions
          opne "Something went wrong via `#{powderblue('cmake')}`."
        # =================================================================== #
        # === The totem playlist was not found
        # =================================================================== #
        when :totem_playlist_was_not_found
          opne 'The totem playlist was not found. You can try '\
               'to compile it:'
          e
          opne sfancy('  rbt totem-playlist')
          e
        # =================================================================== #
        # === Missing C .h (header) file
        # =================================================================== #
        when :missing_c_header_file
          opne swarn('An important C .h file is missing - the compilation can '\
               'not proceed.')
          # ================================================================= #
          # The next line was added on 22.08.2018. I wanted to make sure to
          # easily look at the error for programs that have failed, like
          # this program here.
          # ================================================================= #
          do_not_remove_extracted_archive
          which_h_file = sys_command_string?.select {|inner_line|
            inner_line.include?('fatal error:') and inner_line.include?('No such file or directory')
          }
          use_this_regex = /fatal error. (.+\.h):/
          which_h_file.first =~ use_this_regex
          this_file = $1.to_s.dup
          case this_file
          # ================================================================= #
          # === xf86Resources.h
          # ================================================================= #
          when 'xf86Resources.h'
            opne "The file #{sfile(this_file)} belongs to the very old package"
            opne 'xf86-video-amd-2.7.7.7.tar.xz. This has not been updated since'
            opne '2008 so things will most likely no longer compile.'
          end
        # =================================================================== #
        # === Python import error: an undefined symbol
        #
        # Note that this is a problem, not necessarily an error.
        # =================================================================== #
        when :python_import_error_undefined_symbol
          opne 'A python '+swarn('ImportError')+' has happened, about '\
               'a missing/undefined Symbol.'
          if sys_command_string?.any? {|inner_line|
              inner_line.include? 'lib/gobject-introspection'
            }
            opne 'The problem appears to be somehow related to gobject-introspection.'
            opne 'This error may appear when different python versions or '
            opne 'different library locations are used at the same time.'
          end
        # =================================================================== #
        # === Docbook failed to load a network entity
        #
        # This entry point is related to docbook.
        # =================================================================== #
        when :docbook_failed_to_load_a_network_entity
          opne 'Docbook failed to load a (remote) network entity.'
          opne 'In theory, this could be resolved by installing the docbook'
          opne 'stack, but this is not trivial. Have a look at the LFS'
          opne 'page for now:'
          e '  '+sfancy(::RBT.return_blfs_entry_for('docbook'))
        # =================================================================== #
        # === No configure file
        # =================================================================== #
        when :no_configure_file
          opnn; _ = 'No '+sfancy('configure')+' file was found.'
        # =================================================================== #
        # === Jam is missing
        # =================================================================== #
        when :jam_is_missing
          do_skip_postinstall_actions
          notify_the_user_that_this_program_is_missing(:jam)
        # =================================================================== #
        # === Meson could not be found
        # =================================================================== #
        when :meson_could_not_be_found
          do_skip_postinstall_actions
          notify_the_user_that_this_program_is_missing(:meson) {
            e orange(
                'Note that this will require a recent python 3.x '\
                'version on your host system.'
              )
            e orange(
                'Additionally, you may need the build tool called '\
                'ninja:'
              )
            e
            e '  '+sfancy('rbt ninja')
            e
          }
        end
      elsif _.is_a? Array
        _ = problem?
        first = _.first
        last  = _.last
        last  = last.first if last.is_a? Array
        case first # case tag
        # =================================================================== #
        # === python_import_error_missing_module
        # =================================================================== #
        when :python_import_error_missing_module
          opne "The python module `#{sfancy(last)}` is missing."
          case last
          # ================================================================= #
          # === libxml2
          # ================================================================= #
          when 'libxml2','libxml'
            opne 'You may be able to resolve this problem, by '\
                 'compiling libxml2:'
            e
            opne "  #{sfancy('rbt libxml2')}"
            e
          end
        end
      end
    end
  end; alias try_to_report_the_problem check_for_common_problems # === try_to_report_the_problem
       alias check_for_problems_again  check_for_common_problems # === check_for_problems_again

  # ========================================================================= #
  # === notify_the_user_that_this_program_is_missing
  #
  # The first argument to this method should be the program that is
  # missing, such as 'ruby' or 'htop' or 'glib'.
  #
  # The method will also notify the user how to install the program via
  # the RBT scripts, if this is applicable - aka, if this program is
  # registered.
  # ========================================================================= #
  def notify_the_user_that_this_program_is_missing(
      i = :jam
    )
    i = i.to_s unless i.is_a? String
    e; cliner(use_colour: :konsole_slateblue) {
      e "#{rev}#{sfancy(i)} was not found on your system."
      e
      e 'As you are running via the RBT scripts, you can possibly '\
        'install '+simp(i)
      e 'by issuing the following command:'
      e
      e '  '+sfancy("rbt #{i}")
      e
      # ===================================================================== #
      # Extra options can be passed into the block to this method.
      # ===================================================================== #
      if block_given?
        yield
      end
    }; e
  end

  # ========================================================================= #
  # === clear_problems
  #
  # Reset the problems, so that we don't have any problems.
  # ========================================================================= #
  def clear_problems
    problem_is :no_problem
  end

  # ========================================================================= #
  # === the_program_has_failed_to_install
  # ========================================================================= #
  def the_program_has_failed_to_install
    @internal_hash[:has_the_program_been_installed_successfully] = false
  end

  # ========================================================================= #
  # === no_error_has_occurred?
  # ========================================================================= #
  def no_error_has_occurred?
    !an_error_has_occurred?
  end

  # ========================================================================= #
  # === no_problem_has_occurred?
  #
  # This method will return true if no problem has occurred - otherwise
  # it will return false.
  # ========================================================================= #
  def no_problem_has_occurred?
    !problem?
  end; alias no_problem_was_encountered? no_problem_has_occurred? # === no_problem_was_encountered?

  # ========================================================================= #
  # === has_a_problem_occurred?
  #
  # Query-method over as to whether a problem has occurred or whether
  # it has not.
  # ========================================================================= #
  def has_a_problem_occurred?
    problem?
  end; alias a_problem_has_occurred? has_a_problem_occurred? # === a_problem_has_occurred?

  # ========================================================================= #
  # === a_problem_has_occurred
  #
  # This method simply denotes that a problem has occurred. It is not
  # more specific than that.
  # ========================================================================= #
  def a_problem_has_occurred
    @internal_hash[:problem] = true
  end

  # ========================================================================= #
  # === has_the_program_been_installed_successfully?
  # ========================================================================= #
  def has_the_program_been_installed_successfully?
    @internal_hash[:has_the_program_been_installed_successfully]
  end; alias successfully_compiled?          has_the_program_been_installed_successfully? # === successfully_compiled?
       alias has_been_successfully_compiled? has_the_program_been_installed_successfully? # === has_been_successfully_compiled?
       alias installation_was_successful?    has_the_program_been_installed_successfully? # === installation_was_successful?

  # ========================================================================= #
  # === clear_errors
  # ========================================================================= #
  def clear_errors
    RBT.clear_errors
  end

  # ========================================================================= #
  # === check_for_common_errors
  #
  # This mehod can be used to check (and handle) errors that we have
  # encountered.
  # ========================================================================= #
  def check_for_common_errors
    _ = errors?
    if _.is_a? Array
      _.each {|entry|
        case entry
        # =================================================================== #
        # === Python syntax error: Invalid Syntax
        # =================================================================== #
        when :python_syntax_error_invalid_syntax
          opnerror 'This InvalidSyntax error is often related to the different'
          opnerror 'syntax that python2 and python3 use.'
          # ================================================================= #
          # Consider autoswitching the python-version next:
          # ================================================================= #
          if is_on_roebe?
            if autoswitch_python?
              opne "The configuration file "\
                   "#{slateblue('autoswitch_python.yml')}"\
                   " has been set to true."
              opne "We will consider switching python "\
                   "next (to #{slateblue('python')})."
              result_of_enabling_python2 = enable_python2
              if result_of_enabling_python2 == true
                clear_errors
                do_compile_this_program(
                  compile_which_program?
                )
              end
            else
              # ============================================================= #
              # In this case, notify the user that autoswitching could
              # be enabled. This clause here is entered when autoswitching
              # has been disabled.
              # ============================================================= #
              opne 'Do note that you could set the autoswitch-python '\
                   'variable to true.'
              opne "This could be done by modifying the content "\
                   "of the file #{slateblue('autoswitch_python.yml')}."
              opne 'Having autoswitch-python enabled may, however had, '\
                   'also lead to problems sometimes, so'
              opne 'make sure that you really want to enable this '\
                   'functionality, prior to changing that file.'
            end
          end
        # =================================================================== #
        # === A general make-related error
        # =================================================================== #
        when :make_encountered_some_error
          # ================================================================= #
          # This error type is too generic for now - but perhaps in the
          # future we may also handle it somehow, or give the user
          # more information about it.
          # ================================================================= #
        else
          # ================================================================= #
          # For now this "debugging" part only occurs on roebe-systems.
          # ================================================================= #
          if is_on_roebe?
            stderr "Unregistered error: #{slateblue(entry)}"\
                   " (class: #{lightblue(entry.class.to_s)})"
          end
        end
      }
    end
  end

  # ========================================================================= #
  # === make_install_stage_was_a_success?
  # ========================================================================= #
  def make_install_stage_was_a_success?
    @internal_hash[:make_install_stage_was_a_success]
  end

end; end