#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/python.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === does_a_python_binary_exist?
  #
  # This method will return true if a "python" binary can be found
  # in the PATH variable - and it will return false otherwise.
  # ========================================================================= # 
  def does_a_python_binary_exist?
    _ = false
    check_these_directories = query_path?
    if check_these_directories.is_a?(String) and
       check_these_directories.include?(':')
      splitted = check_these_directories.split(':')
      splitted.each {|this_path|
        target = rds(this_path+'/python')
        if File.exist?(target)
          _ = true
        end
      }
    end
    _
  end

  # ========================================================================= #
  # === notify_the_user_that_no_binary_python_could_be_found
  # ========================================================================= #
  def notify_the_user_that_no_binary_python_could_be_found
    opnn; e 'No binary called "'+gold('python')+'" could be found. This may'
    opnn; e 'be resolved by a symlink to e. g. python3.'
  end

  # ========================================================================= #
  # === run_python_installation
  #
  # You should check whether setup.py exists prior to invoking this
  # method here.
  #
  # You can also pass in another file, such as configure.py. By default,
  # the method will make use of a file called "setup.py", though.
  # ========================================================================= #
  def run_python_installation(
      i = 'setup.py'
    )
    if make_options?.nil?
      _ = cookbook_dataset_make_options?
      if _ and !_.empty?
        i = _
      end
    end
    i = [i].flatten.compact
    # ======================================================================= #
    # We work on an Array here since some make_options entries also
    # specify an Array.
    # ======================================================================= #
    i.each {|do_this_action|
      case do_this_action
      when FILE_SETUP_PY # <- This is equivalent to "setup.py".
        if does_a_python_binary_exist?
          esystem "python #{do_this_action} config"
          esystem "python #{do_this_action} build"
          esystem "python #{do_this_action} install"
        else
          notify_the_user_that_no_binary_python_could_be_found
        end
      else
        run_this_command = "#{do_this_action}".dup
        if run_this_command.start_with?('python') and !run_this_command.include?('/')
          # ================================================================= #
          # This is valid for a command line such as:
          #   "python setup.py config"
          # ================================================================= #
          splitted = run_this_command.split(' ')
          splitted[1].prepend(configure_base_dir?.dup)
          run_this_command = splitted.join(' ')
        end
        unless run_this_command.start_with? 'python'
          run_this_command.prepend('python ')
        end
        if does_a_python_binary_exist?
          esystem run_this_command
        else
          notify_the_user_that_no_binary_python_could_be_found
        end
      end
    }
    consider_renaming_the_kde_konsole_tab :clear, :do_rename
  end; alias run_python_install run_python_installation # === run_python_install

  # ========================================================================= #
  # === scons_file_exists?
  # ========================================================================= #
  def scons_file_exists?
    File.exist? 'SConstruct'
  end; alias scons_file? scons_file_exists? # === scons_file?

  # ========================================================================= #
  # === waf_file_exists?
  #
  # This boolean query method will determine whether a waf-file exists.
  #
  # The input-argument is used primarily so that we can use another
  # target-directory for where the 'waf' file may exist.
  #
  # The method resides in this file here because waf ultimately depends
  # on python.
  # ========================================================================= #
  def waf_file_exists?(
      this_file = 'waf'
    )
    File.exist? this_file
  end; alias waf_file? waf_file_exists? # === waf_file?

  # ========================================================================= #
  # === depends_on_python?
  #
  # Whether python is registered as a dependency.
  # ========================================================================= #
  def depends_on_python?
    RBT.depends_on_python?(required_deps_on?)
  end

  # ========================================================================= #
  # === depends_on_waf?
  # ========================================================================= #
  def depends_on_waf?
    ::RBT.depends_on_waf?
  end

  # ========================================================================= #
  # === consider_enabling_python2
  #
  # This may ensure that we will use python 2.x, primarily due to the
  # meson build system. Right now it only works on roebe-systems though.
  #
  # It can also be enabled from the commandline, such as by issuing
  # the following command:
  #   rbt --enable-python2
  # ========================================================================= #
  def consider_enabling_python2(
      shall_we_enable_python_2 = (on_roebe? and !do_we_use_python3?)
    )
    if shall_we_enable_python_2
      # ======================================================================= #
      # Try to use python version 3 here.
      # ======================================================================= #
      if File.directory?(programs_directory?+'Python/')
        availabe_python_versions = Dir[programs_directory?+'Python/*'].select {|entry|
          File.directory? entry
        }.select {|entries_with_numbers|
          entries_with_numbers =~ /\d+/
        }.select {|entries_that_might_be_python_version_two|
          entries_that_might_be_python_version_two.include? 'Python/2'
        }
        unless availabe_python_versions.empty?
          use_this_python_version = availabe_python_versions.first
          unless RBT.const_defined? :ToCurrent
            require 'rbt/utility_scripts/to_current.rb'
          end
          RBT::ToCurrent.new(use_this_python_version)
          return true # Determine that it was a success.
        end
      end
    end
    false
  end; alias do_use_python2 consider_enabling_python2 # === do_use_python2

  # ========================================================================= #
  # === enable_python3
  # ========================================================================= #
  def enable_python2
    consider_enabling_python2(true)
  end

  # ========================================================================= #
  # === consider_enabling_python3
  #
  # This may ensure that we will use python 3.x, primarily due to the
  # meson build system. Right now it only works on roebe-systems though.
  #
  # It can also be enabled from the commandline, such as by issuing
  # the following command:
  #   rbt --enable-python3
  # ========================================================================= #
  def consider_enabling_python3(
      shall_we_enable_python_3 = (on_roebe? and !do_we_use_python3?)
    )
    if shall_we_enable_python_3
      # ======================================================================= #
      # Try to use python version 3 here.
      # ======================================================================= #
      if File.directory?(programs_directory?+'Python/')
        availabe_python_versions = Dir[programs_directory?+'Python/*'].select {|entry|
          File.directory? entry
        }.select {|entries_with_numbers|
          entries_with_numbers =~ /\d+/
        }.select {|entries_that_might_be_python_version_three|
          entries_that_might_be_python_version_three.include? 'Python/3'
        }
        unless availabe_python_versions.empty?
          use_this_python_version = availabe_python_versions.first
          unless RBT.const_defined? :ToCurrent
            require 'rbt/utility_scripts/to_current.rb'
          end
          RBT::ToCurrent.new(use_this_python_version)
          return true # Determine that it was a success.
        end
      end
    end
    false
  end; alias do_use_python3 consider_enabling_python3 # === do_use_python3

  # ========================================================================= #
  # === enable_python3
  # ========================================================================= #
  def enable_python3
    consider_enabling_python3(true)
  end

  # ========================================================================= #
  # === do_we_use_python3?
  #
  # This method will return true if we use python version 3 or higher,
  # and false otherwise.
  # ========================================================================= #
  def do_we_use_python3?
    version = `python -V 2>&1`.chomp
    if version.include? ' '
      version = version.split(' ').last
    end
    version.start_with? '3' # <- This check has to suffice for now.
  end

  # ========================================================================= #
  # === run_waf
  #
  # This method can be used to install a project that uses "waf" as
  # its build system.
  # ========================================================================= #
  def run_waf
    _ = 'waf'.dup
    if use_build_directory?
      _.prepend(configure_base_directory?)
    end
    # ======================================================================= #
    # Specify which subcommands to use, e. g. 'waf configure' followed by
    # 'waf build' and so forth.
    # ======================================================================= #
    run_these_subcommands = %w(
      configure
      build
      install
    )
    run_these_subcommands.each {|this_subcommand|
      esystem _+' '+this_subcommand
    }
  end; alias run_waf_based_installation run_waf # === run_waf_based_installation

  # ========================================================================= #
  # === run_scons
  #
  # Invoke this method here only when we wish to use scons in order to
  # install a program.
  #
  # Scons is python-related, which is why it sits here in this file.
  # ========================================================================= #
  def run_scons
    _ = 'scons'.dup
    if use_build_directory?
      _.prepend('../')
    end
    esystem _
  end; alias run_scons_based_installation run_scons # === run_scons_based_installation

  # ========================================================================= #
  # === run_python_install1
  # ========================================================================= #
  def run_python_install1
    dataset = YAML.load_file(FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS)['PINSTALL1']
    if dataset.is_a?(String) and dataset.include?(';')
      dataset.split(';').each {|this_command|
        _ = this_command.strip
        e _
        run_this_sys_command(_)
      }
    end
  end

  # ========================================================================= #
  # === run_python_install2
  # ========================================================================= #
  def run_python_install2
    dataset = YAML.load_file(FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS)['PINSTALL2']
    if dataset.is_a?(String) and dataset.include?(';')
      dataset.split(';').each {|this_command|
        _ = this_command.strip
        e _
        run_this_sys_command(_)
      }
    end
  end

end; end