#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/reset.rb'
# =========================================================================== #
require 'rbt/misc/colourize_parser.rb'
require 'rbt/errors/map_line_to_a_specific_error.rb'
require 'rbt/make/make.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    disable_colours unless RBT.use_colours?
    RBT::Errors::MapLineToASpecificError.reset
    # ======================================================================= #
    # Following comes the main Hash that keeps internal data.
    # ======================================================================= #
    @internal_hash = {}
    add_default_dataset_to_the_internal_hash
    # ======================================================================= #
    # Next, prepare the stage for recording-problems. No problems are
    # known initially, on startup. That method call mMust come after
    # the internal Hash has been created.
    # ======================================================================= #
    clear_problems
    initialize_default_values
    set_cflags { :be_silent } # <- Initialize the CLFAGS here.
  end

  # ========================================================================= #
  # === initialize_default_values
  #
  # We need some default values, which are set here.
  # ========================================================================= #
  def initialize_default_values
    @internal_hash[:compile_this_program] = nil
    set_program_path(nil)
    @program_name_and_program_version = nil
    # ======================================================================= #
    # Instantiate a new RBT::Make object next, as our wrapper to 'make'.
    # ======================================================================= #
    @make = RBT::Make.new
    # ======================================================================= #
    # We ought to populate our internal Hash with some default values.
    # ======================================================================= #
    populate_internal_hash_with_default_values
    extract_to_default_directory
  end

  # ========================================================================= #
  # === add_default_dataset_to_the_internal_hash
  #
  # The variables in use in this method here can, for the most part, be
  # modified via the commandline. In other words, the user can change
  # these variables and thus toggle the behaviour of RBT::Compile.
  #
  # That is one reason why many of these variables are set to nil initially,
  # so we can easily find out which values have been modified on the
  # commandline, and which one has not.
  # ========================================================================= #
  def add_default_dataset_to_the_internal_hash
    # ======================================================================= #
    # === Keeping track of the commandline arguments
    #
    # @commandline_arguments are set on startup only once, so they can not
    # be put inside the method initialize_default_values().
    # ======================================================================= #
    @internal_hash[:commandline_arguments] = []
    # ======================================================================= #
    # Whether we will use special configure options or not. By default
    # we will not, hence why we initialize the next setting to nil.
    # ======================================================================= #
    @internal_hash[:configure_options] = nil
    # ======================================================================= #
    # Specify which compiler is to be used. By default we will use GCC,
    # but clang may also be an option. It will be stored as a lowercased
    # Symbol, though, so :gcc would refer to GCC.
    # ======================================================================= #
    @internal_hash[:compiler_to_use] = :gcc
    # ======================================================================= #
    # Default value is nil for the next variable. If true then we will
    # NOT extract.
    # ======================================================================= #
    @internal_hash[:skip_extracting] = nil
    # ======================================================================= #
    # The next entry point allows us to pass options/parameters to the
    # "make" binariy.
    # ======================================================================= #
    @internal_hash[:parameters_to_make] = nil
    # ======================================================================= #
    # Use another program version than the default one:
    # ======================================================================= #
    @internal_hash[:use_this_specific_program_version] = nil
    # ======================================================================= #
    # Determine which build system is to be used. By default it will
    # be nil.
    # ======================================================================= #
    @internal_hash[:use_this_build_system] = nil
    # ======================================================================= #
    # The next variable can be used to allow the user to determine another
    # directory for where we will extract archives into. Initially set
    # to nil in order to allow distinction to the default extractable
    # directory that is to be used.
    # ======================================================================= #
    @internal_hash[:extract_to_this_directory_as_specified_by_the_user] = nil
    @internal_hash[:postinstallation] = nil
    # ======================================================================= #
    # The next variable will keep track of class SymlinkProgram. We
    # need to be able to modify its behaviour at "runtime".
    # ======================================================================= #
    set_symlink_program(nil)
    # ======================================================================= #
    # The next variable has to be kept for if we actually do want to
    # check whether a program is included or not. Normally we should
    # check, but there may be situations where you may build up your
    # own functionality - so we need to provide an option that can be
    # toggled by the user. This is why the next variable exists, and
    # is kept at true.
    # ======================================================================= #
    @internal_hash[:honour_check_to_ensure_that_the_program_exists] = true
    # ======================================================================= #
    # The next variable can be used to determine whether we will compile
    # a particular program or whether we will not.
    # ======================================================================= #
    @internal_hash[:shall_we_compile_the_program] = true
    @internal_hash[:last_compiled_program] = nil  # Define it once at least.
    # ======================================================================= #
    # Must initialize the build directory to a nil value next. It may
    # be set to a non-nil value lateron.
    # ======================================================================= #
    @internal_hash[:use_build_directory] = nil
    # ======================================================================= #
    # The next variable, false on startup, tells RBT::Compile to retain
    # all environment variables. Sometimes we may want to NOT use the
    # environment variables, which is why this flag exists.
    # ======================================================================= #
    @internal_hash[:do_not_use_special_flags] = false
    # ======================================================================= #
    # Use a default build directory. There is another entry that determines
    # whether we will use any build directory at all or not, so it is safe
    # to assign to the variable here - the entry called
    # @internal_hash[:use_build_directory].
    # ======================================================================= #
    @internal_hash[:name_for_the_build_directory] = DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY
    # ======================================================================= #
    # Determine whether we are on GoboLinux or whether we are not.
    # ======================================================================= #
    @internal_hash[:is_on_gobolinux] = RBT.are_we_on_gobolinux?
    # ======================================================================= #
    # Next, how often we invoked traditional or non-traditional compilation.
    # ======================================================================= #
    @internal_hash[:counter_called_traditional_or_non_traditional_compilation] = 0
    # ======================================================================= #
    # Determine whether we will run the configure script or whether we
    # will not.
    # ======================================================================= #
    @internal_hash[:run_configure_script] = nil
    # ======================================================================= #
    # Whether we are allowed to use special ENV flags. By default we are
    # allowed to use special flags, hence why the variable is set to false.
    # ======================================================================= #
    @internal_hash[:do_not_use_special_environment_flags] = false
    # ======================================================================= #
    # Next, determine whether we will make use of GoboLinux's SymlinkProgram.
    # This will be automatically changed if we are on GoboLinux. For other
    # systems we will prefer to use SymlinkProgram from RBT itself.
    # ======================================================================= #
    @internal_hash[:use_gobolinux_symlinkprogram] = false
    # ======================================================================= #
    # Determine whether we will run postinstallation steps.
    # ======================================================================= #
    @internal_hash[:run_postinstall_actions] = true
    # ======================================================================= #
    # The next variable is an empty Array by default. Must be defined
    # outside other methods such as initialize_default_values().
    # ======================================================================= #
    @internal_hash[:compile_these_programs] = []
    @internal_hash[:we_tried_to_compile_this_program_n_times] = 0
    set_user_prefix(nil) # <- Purposely defined here, outside of other methods.
    @internal_hash[:colourize_parser] = RBT::ColourizeParser.new
    @internal_hash[:make_use_of_cookbook_aliases] = true
    # ======================================================================= #
    # Determine whether we shall time the compilation or whether we shall
    # not.
    # ======================================================================= #
    @internal_hash[:time_compilation] = YAML.load_file(FILE_TIME_COMPILATION)
    if @internal_hash[:time_compilation]
      @internal_hash[:array_time_compilation] = []
    end
    # ======================================================================= #
    # Use autoconf or not - by default we will not.
    # ======================================================================= #
    @internal_hash[:use_autoconf] = nil
    # ======================================================================= #
    # Determine whether we will try to use any declared configure options
    # from the cookbook-yaml file. This is true by default. The reason
    # why the following variable is set to nil initially is so that we
    # can distinguish the situation from when the user did specifically
    # want to disable this functionality, from the commandline.
    # ======================================================================= #
    @internal_hash[:try_to_use_configure_options] = nil
    # ======================================================================= #
    # By default we will extract the archive.
    # ======================================================================= #
    @internal_hash[:extract_the_archive] = true
    # ======================================================================= #
    # The keep-extracted setting will lateron be sanitized. The default
    # value will be nil.
    # ======================================================================= #
    @internal_hash[:keep_the_extracted_archive] = nil
    # ======================================================================= #
    # The next variable must be defined here. A value of nil specifies
    # that this has not been set via the commandline yet. A value of
    # nil here essentially means a default to false - aka we will not
    # try to compile a program in a static manner. 
    # ======================================================================= #
    @internal_hash[:build_static] = nil
    # ======================================================================= #
    # By default we will not strip the binaries. The configuration
    # setting can change this value though.
    # ======================================================================= #
    @internal_hash[:strip_binaries] = false
    # ======================================================================= #
    # The configuration setting can change the above default false-value.
    # ======================================================================= #
    if config?
      @internal_hash[:strip_binaries] = config?.use_strip
    end
    # ======================================================================= #
    # The next flag determines whether we will delete .la files or whether
    # we will keep them.
    # ======================================================================= #
    @internal_hash[:keep_la_files] = nil
    # ======================================================================= #
    # Default value for use_autogen will be nil. The user is able to
    # override this from the commandline, which is why we will set
    # this variable to nil.
    # ======================================================================= #
    @internal_hash[:use_autogen] = nil
    # ======================================================================= #
    # The next variable, if true, instructs the class to check whether the
    # given input argument(s) do exist as a regular local file. If false
    # this check is skipped.
    # ======================================================================= #
    @internal_hash[:check_whether_the_given_input_is_a_file_and_does_exist] = true
    # ======================================================================= #
    # Next, determine whether to use the internal sed-wrapper. No
    # longer true by default since as of February 2018, but perhaps
    # we may re-enable it at some later moment in time.
    # ======================================================================= #
    @internal_hash[:use_the_internal_sed_wrapper] = false
    # ======================================================================= #
    # Whether to apply a sed-operation. Will be nil by default. If the
    # user passes in some special option here, then we know that we
    # have to use this value to overrule any other value for sed.
    # ======================================================================= #
    @internal_hash[:sed] = nil
    @internal_hash[:symlink_headers] = nil
    @internal_hash[:symlink_glib_schema] = nil
    # ======================================================================= #
    # Determine whether we will symlink the content of lib/ into 
    # /usr/lib/. This setting is only honoured if we compile in an
    # appdir-like fashion.
    # ======================================================================= #
    @internal_hash[:symlink_into_usr_lib_dir] = false
    # ======================================================================= #
    # Only install the .h header files, if this variable is set to true.
    # ======================================================================= #
    @internal_hash[:install_only_the_headers] = false
    # ======================================================================= #
    # Whether to time the compilation procedure or not:
    # ======================================================================= #
    @internal_hash[:time_compilation] = true
    # ======================================================================= #
    # Whether we will exit when we reach the "make install" stage.
    # ======================================================================= #
    @internal_hash[:exit_before_make_install] = false
    # ======================================================================= #
    # Whether we will exit when we reach the "make" stage.
    # ======================================================================= #
    @internal_hash[:exit_before_make] = false
    # ======================================================================= #
    # === Show the infobox
    #
    # Next determine whether we will show the infobox or not. This has
    # to be in the method here so that we do not re-set it for each
    # program that is to be compiled/installed.
    # ======================================================================= #
    @internal_hash[:show_the_infobox] = true
    # ======================================================================= #
    # If the next variable is set to true, we will exit after we have
    # extracted the archive. We will first initialize it to nil, though,
    # in order to distinguish between the default value and user-supplied
    # values.
    # ======================================================================= #
    @internal_hash[:extract_only] = nil
    # ======================================================================= #
    # The use_ccache entry has to be persistent.
    # ======================================================================= #
    @internal_hash[:use_ccache] = RBT.configuration?.use_ccache
    # ======================================================================= #
    # Determine whether we will make use of porg.
    # ======================================================================= #
    @internal_hash[:use_porg] = configuration?.use_porg
    # ======================================================================= #
    # Whether we will run in simulation-mode or not. Simulation-mode means
    # that we will not make any real lasting changes. This thus allows a
    # user to predict (and show a summary) what would happen if a
    # program would be installed/compiled.
    # ======================================================================= #
    @internal_hash[:run_simulation] = false
    set_name_for_the_build_directory # Set a default name for the build directory.
    # ======================================================================= #
    # The next variable is a bit unclear - it used to exist to allow for
    # easy redirection, but may no longer be that important. We may have to
    # re-evaluate this at a later time (written at 12.04.2019).
    # ======================================================================= #
    @original_stdout = nil
  end

  # ========================================================================= #
  # === populate_internal_hash_with_default_values
  #
  # This method will populate the internal Hash with sane default values.
  #
  # These are values that, for the most part, can NOT be persistently
  # set on the commandline. This may change at a later time, though,
  # or is not true for some variables, such as :prefix. But it is
  # true for entries such as :have_we_successfully_installed_the_program.
  # ========================================================================= #
  def populate_internal_hash_with_default_values
    # ======================================================================= #
    # Whether we have encountered a problem or not. Must be reset for
    # any new per-compile cycle run.
    # ======================================================================= #
    @internal_hash[:problem] = false
    # ======================================================================= #
    # The next variable must be true by default.
    # ======================================================================= #
    @internal_hash[:may_we_initialize_a_new_cookbook_dataset] = true
    # ======================================================================= #
    # The variable at :can_we_continue is reset during every iteration.
    # It will be true by default.
    # ======================================================================= #
    @internal_hash[:may_we_download] = RBT::TRY_TO_DOWNLOAD_IF_NOT_FOUND
    @internal_hash[:can_we_continue] = true
    @internal_hash[:compile_this_program] = nil
    @internal_hash[:may_we_run_ninja] = true # <- By default we can always run ninja again.
    @internal_hash[:prefix] = RBT::Prefix.new(:trad) # Set a default prefix to use.
    # ======================================================================= #
    # The next variable will be nil by default, so that we can grab the
    # value that the cookbook-dataset is using here.
    # ======================================================================= #
    @internal_hash[:run_make_command] = nil
    # ======================================================================= #
    # By default, we will run "make install". This covers most programs
    # that are to be compiled from source.
    # ======================================================================= #
    @internal_hash[:run_make_install] = true
    @internal_hash[:use_opn] = true # <- Whether we use opn or whether we don't.
    # ======================================================================= #
    # The next variable can keep track of sys-commands that were run.
    # ======================================================================= #
    @internal_hash[:array_sys_command_string] = []
    @internal_hash[:exit_after_a_rubygem_has_been_installed] = false
    @internal_hash[:install_only_binaries] = false
    # ======================================================================= #
    # Whether the "make install" stage was a success. This is true by
    # default and will only become false if an error is encountered.
    # ======================================================================= #
    @internal_hash[:make_install_stage_was_a_success] = true
    # ======================================================================= #
    # The next variable keeps track over as to whether we have
    # successfully installed the program or whether we have not.
    # ======================================================================= #
    @internal_hash[:have_we_successfully_installed_the_program] = false
    @internal_hash[:rename_tab_title] = RBT.configuration?.rename_tab_title
    # ======================================================================= #
    # Whether the given input is an abbreviation or not. By default we
    # assume that the user has NOT used an abbreviated term for a real
    # program.
    # ======================================================================= #
    @internal_hash[:is_an_abbreviation] = false
    # ======================================================================= #
    # By default, all programs will have been installed successfully. The
    # next variable keeps track of this.
    # ======================================================================= #
    @internal_hash[:has_the_program_been_installed_successfully] = true
    # ======================================================================= #
    # We have to store the full configure command as well. It is not just
    # for configure but also for cmake, meson and so forth.
    # ======================================================================= #
    @internal_hash[:full_configure_command] = ''.dup
    # ======================================================================= #
    # Register any reported dependencies.
    # ======================================================================= #
    @internal_hash[:array_required_dependencies] = []
    @internal_hash[:an_error_has_occurred] = false
    @internal_hash[:try_to_autofix] = true
    @internal_hash[:try_to_recompile_upon_error] = true
    @internal_hash[:continue_after_make_step] = true
  end; alias reset_internal_hash populate_internal_hash_with_default_values # === reset_internal_hash

  # ========================================================================= #
  # === reset_compiled_n_times_counter
  # ========================================================================= #
  def reset_compiled_n_times_counter
    @internal_hash[:we_tried_to_compile_this_program_n_times] = 0
  end

  # ========================================================================= #
  # === reset_error_map_line_to_a_specific_error
  # ========================================================================= #
  def reset_error_map_line_to_a_specific_error
    RBT::Errors::MapLineToASpecificError.clear
  end

end; end