#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Ruby-related code, e. g. for the file ruby.yml and similar, may reside
# in this file called "ruby.rb".
# =========================================================================== #
# require 'rbt/compile/ruby.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/array_all_rubygems.rb'
require 'rbt/check_for_updates/check_for_new_release_on_rubygems.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === skip_extracting_if_it_is_a_gem_file
  # ========================================================================= #
  def skip_extracting_if_it_is_a_gem_file
    if is_a_gem_file?
      skip_extracting
    end
  end

  # ========================================================================= #
  # === show_registered_gems
  #
  # To invoke this method from the commandline, try:
  #
  #   rbt --show-registered-gems
  #
  # ========================================================================= #
  def show_registered_gems
    e 'These gems are available:'
    e
    RBT.array_all_rubygems.each_with_index {|this_gem, index| index += 1
      e "  #{lightgreen(index.to_s.rjust(4))} #{slateblue(this_gem)}"
    }
    e
  end

  # ========================================================================= #
  # === is_a_gem_file?
  #
  # Query whether we have a .gem file at hand here.
  # ========================================================================= #
  def is_a_gem_file?(
      i = program_path?
    )
    i.to_s.end_with? '.gem'
  end; alias is_a_gem? is_a_gem_file? # === is_a_gem?

  # ========================================================================= #
  # === ruby_setup_config
  # ========================================================================= #
  def ruby_setup_config
    system 'ruby setup.rb --quiet config'
  end

  # ========================================================================= #
  # === ruby_setup_setup
  # ========================================================================= #
  def ruby_setup_setup
    system 'ruby setup.rb --quiet setup'
  end

  # ========================================================================= #
  # === ruby_setup_install
  # ========================================================================= #
  def ruby_setup_install
    system 'ruby setup.rb --quiet install'
  end

  # ========================================================================= #
  # === copy_ruby
  # ========================================================================= #
  def copy_ruby
    begin
      require 'roebe/toplevel_methods/setup.rb'
      Roebe.copy_setup_file_to_pwd
    rescue LoadError
      opnn; e 'The method Roebe.copy_setup_file_to_pwd() is not available.'
      opnn; e "Please install the #{slateblue('roebe')} project:"
      e
      e '  gem install roebe'
      e
    end
  end

  # ========================================================================= #
  # === do_not_exit_after_a_rubygem_has_been_installed
  # ========================================================================= #
  def do_not_exit_after_a_rubygem_has_been_installed
    @internal_hash[:exit_after_a_rubygem_has_been_installed] = false
  end

  # ========================================================================= #
  # === exit_after_a_rubygem_has_been_installed?
  # ========================================================================= #
  def exit_after_a_rubygem_has_been_installed?
    @internal_hash[:exit_after_a_rubygem_has_been_installed]
  end

  # ========================================================================= #
  # === return_gem_install_options
  # ========================================================================= #
  def return_gem_install_options
    _ = "#{RBT_YAML_DIRECTORY}ruby_gem_install_options.yml"
    if File.exist? _
      YAML.load_file(_).strip
    else
      ''
    end
  end

  # ========================================================================= #
  # === install_ruby_gem                  (install gem, gem install, gem tag)
  #
  # Use this method to install a ruby gem, then exit.
  #
  # The input argument should be the path towards the .gem file in
  # question.
  #
  # Right now we will always exit when we use this method, by default.
  # This behaviour can be changed through the variable 'shall_we_exit'.
  #
  # Invocation example:
  #
  #   ry /Users/x/SRC/ruby/gems/wirble-0.1.3.gem
  #   ry $MY_SRC/ruby/gems/unf-0.0.5.gem
  #   ry thor
  #
  # ========================================================================= #  
  def install_ruby_gem(
      this_gem      = program_path?,
      shall_we_exit = exit_after_a_rubygem_has_been_installed?
    )
    do_not_display_the_infobox
    do_not_symlink :be_quiet
    case this_gem
    when :default
      this_gem = program_path?
    end
    case shall_we_exit
    when :then_exit, :do_exit
      shall_we_exit = true
    when :do_not_exit
      shall_we_exit = false
    end
    # ======================================================================= #
    # === Download the gem
    #
    # If the path to a gem includes a "http" string, then we will assume
    # that we will first have to download it.
    # ======================================================================= #
    if this_gem and this_gem.include? 'http'
      go_to_base_dir
      do_download(this_gem)
    end
    # ======================================================================= #
    # Next, install the gem via a system() call. There are several ways
    # how to install a gem through, most importantly, with or without
    # dependency management by gem.
    # ======================================================================= #
    _ = ("gem install #{return_gem_install_options} #{this_gem}").squeeze(' ')
    opnn; e _
    system _
    exit_program if shall_we_exit
  end; alias install_this_gem      install_ruby_gem # === install_this_gem
       alias install_this_ruby_gem install_ruby_gem # === install_this_ruby_gem

  # ========================================================================= #
  # === update_all_gems
  #
  # This method allows us to update all gems.
  #
  # To invoke this method from within RBT::Compile, try this:
  #
  #   rbt --update-all-gems
  #
  # Note that the constant ALSO_AUTOMATICALLY_INSTALL_THE_UPDATED_GEM is
  # used to automatically update a thus-downloaded gem.
  # ========================================================================= #
  def update_all_gems
    require 'rbt/check_for_updates/check_for_new_release_on_rubygems.rb'
    opnn; e 'Next '+
            royalblue('updating all local gems')+'.'
    RBT.update_all_ruby_gems
    if ALSO_AUTOMATICALLY_INSTALL_THE_UPDATED_GEM
      array_to_use = RBT.array_these_ruby_gems_were_updated?
      update_these_rubygems(array_to_use)
    end
  end

  # ========================================================================= #
  # === update_these_rubygems
  #
  # This method can be used to specifically update the input-array
  # rubygems.
  #
  # By default we will feed in all registered rubygems, that is -
  # all .gem files that are registered within the RBT project.
  #
  # If you want to use another array of programs, pass in the Array
  # of .gem files that you wish to use, into this method.
  #
  # To invoke this method from the commandline, issue something like
  # this:
  #
  #   rbt --install-all-rubygems
  #
  # ========================================================================= #
  def update_these_rubygems(
      array_to_use = RBT.array_all_rubygems
    )
    unless array_to_use.empty?
      initialize_default_values
      array_to_use.each {|this_program|
        opnn; e "Now installing the ruby gem called #{slateblue(this_program)}:"
        RBT.install_this_rubygem(this_program)
      }
    end
  end

end

# =========================================================================== #
# === RBT.install_this_rubygem
# =========================================================================== #
def self.install_this_rubygem(i)
  compile_object = RBT::Compile.new(:do_not_run_yet)
  compile_object.determine_the_dataset_for_this_program(i)
  compile_object.set_program_path(compile_object.cookbook_dataset_program_path?)
  compile_object.install_this_ruby_gem(:default, :do_not_exit)
end; self.instance_eval { alias install_this_ruby_gem install_this_rubygem } # === RBT.install_this_ruby_gem

# =========================================================================== #
# === RBT.update_these_rubygems
#
# Usage example:
#   array = %w( thor prawn ); RBT.update_these_rubygems(array)
# =========================================================================== #
def self.update_these_rubygems(
    array_to_use = RBT.array_all_rubygems
  )
  unless array_to_use.empty?
    array_to_use.each {|this_gem|
      RBT.install_this_rubygem(this_gem)
    }
  end
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.update_all_ruby_gems
end # update_all_ruby_gems