#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/sed.rb'
# =========================================================================== #
require 'rbt/misc/sed_wrapper.rb'
require 'rbt/utility_scripts/apply_sed_operations.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === check_for_sed_action                                        (sed tag)
  #
  # Some programs have a sed-entry. We thus need a method to handle these
  # sed entries.
  #
  # As of August 2017, we will delegate towards class RBT::SedWrapper
  # to deal with sed-related editing. However had, since as of February
  # 2018, we will use system-sed (the binary called "sed"), if it is
  # available. The reason as to why is because my sed-wrapper is
  # incomplete and buggy.
  #
  # Perhaps one day class SedWrapper may work reliably, but for the time
  # being, and as new default, we will not use the SedWrapper by default.
  # ========================================================================= #
  def check_for_sed_action
    _ = sed? # Note that this ought to be an Array.
    if _ and !_.empty?
      if use_the_internal_sed_wrapper?
        unless _.empty?
          notify_the_user_that_we_will_use_sed_from_the_pwd
          _.each {|sed_line| # Must be an Array.
            opnn; e 'Next applying the following sed-operation:'
            apply_sed_operation_on(sed_line)
          }
        end
      else
        # ===================================================================== #
        # Else we will delegate towards class ApplySedOperations.
        # ===================================================================== #
        RBT::ApplySedOperations.new(_)
      end
    end
  end

  # ========================================================================= #
  # === verbose_query_whether_we_will_use_the_internal_sed_wrapper
  #
  # Query whether the internal sed-wrapper will be used.
  #
  # Invocation example:
  #   rbt --use-the-internal-sed-wrapper?
  # ========================================================================= #
  def verbose_query_whether_we_will_use_the_internal_sed_wrapper(
      shall_we_exit = :do_not_exit
    )
    if use_the_internal_sed_wrapper?
      opnn; e 'Yes, the internal sed-wrapper will be used ('+
              slateblue('class SedWrapper')+').'
    else
      opnn; e 'No, the internal sed-wrapper will '+crimson('NOT')+
              ' be used (class SedWrapper).'
    end
    case shall_we_exit
    when :and_exit, :and_then_exit
      exit_program
    end
  end

  # ========================================================================= #
  # === use_the_internal_sed_wrapper?
  #
  # Whether we will use the internal sed wrapper, aka class SedWrapper.
  #
  # Since as of February 2018, we will not do so.
  #
  # You can query this via the commandline like so:
  #   rbt use-the-internal-sed-wrapper?
  # ========================================================================= #
  def use_the_internal_sed_wrapper?
    @internal_hash[:use_the_internal_sed_wrapper]
  end

  # ========================================================================= #
  # === disable_internal_sed
  # ========================================================================= #
  def disable_internal_sed
    @internal_hash[:use_the_internal_sed_wrapper] = false
  end

  # ========================================================================= #
  # === sed?
  #
  # Query as to whether we will use sed or not.
  # ========================================================================= #
  def sed?
    _ = cookbook_dataset?.sed?
    if @internal_hash[:sed].nil?
      # ===================================================================== #
      # In this case we will use the sed-variant given by the cookbooks
      # dataset.
      # ===================================================================== #
    else
      _ = @internal_hash[:sed]
    end
    _
  end

  # ========================================================================= #
  # === verbose_disable_sed
  # ========================================================================= #
  def verbose_disable_sed
    opnn; e "We will not use the #{powderblue('internal sed-wrapper')}."
    disable_internal_sed
  end

  # ========================================================================= #
  # === sed_binary_is_available?
  #
  # Determine whether the host-system has the "sed" binary available or
  # whether it has not.
  # ========================================================================= #
  def sed_binary_is_available?
    result = `sed 2>&1`
    result.include? 'Usage:'
  end

  # ========================================================================= #
  # === do_not_use_sed
  #
  # Disable the use of sed through this method.
  # ========================================================================= #
  def do_not_use_sed(
      be_verbose = false
    )
    case be_verbose
    when :be_verbose
      be_verbose = true
    end
    if be_verbose
      opnn; e "We will not be using #{lightgreen('sed')}."
    end
    @internal_hash[:sed] = false
  end

  # ========================================================================= #
  # === notify_the_user_that_we_will_use_sed_from_the_pwd
  # ========================================================================= #
  def notify_the_user_that_we_will_use_sed_from_the_pwd
    opnn; e "Next applying #{teal('sed')} in the "\
            "directory #{sdir_return_pwd}:"
  end

  # ========================================================================= #
  # === apply_sed_operation_on
  #
  # This method will either use class SedWrapper, or it will use the
  # sed-binary.
  # ========================================================================= #
  def apply_sed_operation_on(sed_line)
    if sed_line.is_a? Array
      sed_line.each {|entry|
        apply_sed_operation_on(entry)
      }
    else
      if sed_binary_is_available?
        esystem(sed_line)
      else
        if use_the_internal_sed_wrapper?
          # ================================================================= #
          # In this case, use the SedWrapper as a fallback.
          # ================================================================= #
          SedWrapper.new(sed_line)
        else
          # ================================================================= #
          # Else we will default to the system-sed.
          # ================================================================= #
          opnn; e 'We can not use the sed line '+sfancy(sed_line)+' because '\
                  'sed is not available'
          opnn; e ', and no commandline option has been passed to allow '\
                  'us to use the sed wrapper.'
        end
      end
    end
  end; alias run_this_sed_instruction apply_sed_operation_on # === run_this_sed_instruction

end; end