#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/show_and_report.rb'
# =========================================================================== #
require 'rbt/utility_scripts/show_all_about.rb'
require 'rbt/utility_scripts/blfs.rb'
require 'rbt/utility_scripts/report_the_registered_programs.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === show_all
  #
  # This method will show the full content of a specific .yml file.
  #
  # To invoke this method, do something like the following:
  #
  #   ry htop show_all
  #
  # ========================================================================= #
  def show_all(
      i = internal_program_name?
    )
    ShowAllAbout.new(i)
  end; alias show_all_about show_all # === show_all_about

  # ========================================================================= #
  # === show_all_about_this_dir
  # ========================================================================= #
  def show_all_about_this_dir(
      i = return_pwd
    )
    _ = File.basename(i)
    _ = ProgramInformation.return_short_name(i).delete('-')
    show_all_about(_) # Delegate to that method here.
  end

  # ========================================================================= #
  # === report_where_we_store_then_exit
  # ========================================================================= #
  def report_where_we_store_then_exit
    report_where_we_will_store_logs
    exit_program
  end

  # ========================================================================= #
  # === report_where_we_will_store_logs
  #
  # This will ask where we store our BT-Related files. The Config object
  # defines that.
  #
  # To trigger this, do:
  #
  #   ry --store_where?
  #
  # ========================================================================= #
  def report_where_we_will_store_logs
    opnn; e "Log-files will be stored into the directory `"\
            "#{sdir(log_dir?)}`."
  end; alias store_where? report_where_we_will_store_logs # === store_where?

  # ========================================================================= #
  # === show_compile_chain
  #
  # Show all members of a particular compile-chain.
  #
  # Invocation examples:
  #
  #   ry --show-chain=plasma
  #   ry --show-compile-chain-of=kde5_plasma
  #   ry --compile-chain-of=kde5_plasma
  #   ry --view=kde5_plasma
  #
  # ========================================================================= #
  def show_compile_chain(i)
    ShowCompileChain.of(i)
  end

  # ========================================================================= #
  # === chained_programs?
  #
  # A simpler wrapper to obtain an Array of all chained programs, the
  # keys.
  # ========================================================================= #
  def chained_programs?
    RBT.chained_programs?.keys
  end

  # ========================================================================= #
  # === show_where_the_base_directory_can_be_found
  #
  # To test this method, do:
  #   ry htop base_dir?
  # ========================================================================= #
  def show_where_the_base_directory_can_be_found
    e
    opnn; e 'The base directory is at: '.ljust(36)+
             sfancy(temp_directory?)
    unless extract_to?.to_s.empty?
      opnn; e "We will extract to this directory:  "\
              "#{sfancy(extract_to?.to_s)}"
    end
    e
  end

  # ========================================================================= #
  # === show_temp_directory
  #
  # To invoke this from the commandline, do:
  #   ry --tempdir?
  # ========================================================================= #
  def show_temp_directory
    opnn; e "The temp directory resides at `#{sdir(temp_directory?)}`."
  end

  # ========================================================================= #
  # === show_how_to_compile_kde
  #
  # This method will provide some hints on the commandline as to how
  # to batch-compile the various KDE components.
  #
  # Invocation example:
  #   ry --show-how-to-compile-kde
  # ========================================================================= #
  def show_how_to_compile_kde
    e
    e '  ry --batch-compile=kde5_foundation'
    e '  ry --batch-compile=kde5_plasma'
    e '  ry --batch-compile=kde5_porting_aids'
    e '  ry --batch-compile=kde5_applications'
    e
  end

  # ========================================================================= #
  # === show_rbt_intro
  #
  # Invocation example:
  #
  #   ry --show-rbt-intro
  #   ry --intro
  #
  # ========================================================================= #
  def show_rbt_intro
    e
    e 'Here are some hints to get you started with RBT.'
    e
    e "You can compile a program by issuing #{orange('rbt <name>')}."
    e
    e 'Example:'
    e
    e sfancy('  rbt php')
    e
    e 'You can also compile into a standalone app-dir:'
    e
    e sfancy('  rbt php ntrad')
    e sfancy('  rbt php --app-dir')
    e
    e 'Both variants are synonymous. If in doubt, using the variants'
    e 'that start with '+teal('--')+' is recommended.'
    e
    e 'To speed-up looking for the dataset stored in the various .yml'
    e 'files you can also issue the following command:'
    e
    e sfancy('  rbt --expand')
    e
    e 'This command will expand the dataset into a "sanitized" variant'
    e 'that requires less pre-processing before it can be used for'
    e 'compiling/installing programs from source.'
    e
  end

  # ========================================================================= #
  # === show_n_largest_local_programs
  #
  # Show the largest local programs with this method.
  #
  # Invocation example:
  #   ry htop --show_highest
  # ========================================================================= #
  def show_n_largest_local_programs(
      i = :default
    )
    i = 50 if i == :default
    RBT::Cookbooks::Highest.new(i)
    exit_program
  end

  # ========================================================================= #
  # === show_how_to_batch_compile
  #
  # This method will notify the user in how to batch-compile programs
  # via RBT.
  #
  # Invocation example:
  #   rbt --show_how_to_batch_compile
  # ========================================================================= #
  def show_how_to_batch_compile
    show_available_batch_compile_options
    e
    e 'In order to batch-compile one of these, issue a command such as:'
    e
    e '  rbt --batch-compile=mate'
    e '  rbt --batch-compile=vid_base'
    e '  rbt --batch-compile=terminals'
    e
  end

  # ========================================================================= #
  # === show_available_batch_compile_options
  # ========================================================================= #
  def show_available_batch_compile_options
    this_file = RBT.file_cookbook_chained_programs
    if File.exist? this_file
      dataset = YAML.load_file(this_file)
      keys = dataset.keys
      opnn; e "These #{teal(keys.size)} entries exist:"; e
      keys.each {|this_entry_point|
        e "  #{sfancy(this_entry_point)}"
      }; e
    else
      opnn; no_such_file_exists(this_file)
    end
  end

  # ========================================================================= #
  # === report_the_amount_of_binaries
  #
  # Invocation example:
  #   ry glib --n-binaries?
  # ========================================================================= #
  def report_the_amount_of_binaries(
      i = compile_which_program?
    )
    ensure_proper_dataset_for(i) if cookbook_dataset?.nil?
    _ = binaries?
    opnn; e 'The program '+sfancy(i)+' will '\
            'install '+sfancy(_.size.to_s)+' binaries.'
  end

  # ========================================================================= #
  # === feedback_all_available_formats
  #
  # Invocation example:
  #   ry --feedback_all_available_formats
  # ========================================================================= #  
  def feedback_all_available_formats
    require 'rbt/cookbooks/class/feedback/feedback_all_available_formats.rb'
    RBT::Cookbooks::Cookbook.feedback_all_available_formats
  end

  # ========================================================================= #
  # === show_pc_files_installed_by_this_program
  #
  # Invocation example:
  #   ry samba --pkgconfig-files?
  # ========================================================================= #
  def show_pc_files_installed_by_this_program(
      i = compile_which_program?
    )
    _ = pc_files?
    if _.empty?
      opnn; e 'The program '+simp(i)+' will not install any .pc file.'
    else
      opnn; e "The program #{gold(i)} will install "\
              "the following #{sfancy(_.size.to_s)} .pc file(s):"
      e; _.each {|this_pc_file|
        e "  #{sfancy(this_pc_file)}"
      }; e
    end
  end

  # ========================================================================= #
  # === show_last_updated
  #
  # This method will show the last updated programs.
  #
  # Invocation example:
  #   ry --last_update
  # ========================================================================= #
  def show_last_updated(
      shall_we_exit = true
    )
    require 'rbt/utility_scripts/show_last_updated.rb'
    RBT::Cookbooks::ShowLastUpdated.new # bl $RUBY_COOKBOOKS/lib/cookbooks/show_last_updated.rb
    exit_program if shall_we_exit
  end

  # ========================================================================= #
  # === report_all_blfs_entries
  # ========================================================================= #
  def report_all_blfs_entries
    RBT::Blfs.new
  end

  # ========================================================================= #
  # === show_information_about_blfs
  #
  # We show some information about BLFS webpages.
  #
  # To invoke this method, you can issue a command such as this here:
  #   ry binutils --lfs
  # ========================================================================= #
  def show_information_about_blfs
    if cookbook_dataset?.has_key?('blfs') and
      !cookbook_dataset?.blfs?.empty? # .empty? will assume this to be a String.
      url = cookbook_dataset?.blfs?
      if url.is_a? Array
        url = url.first
      end
      e "Url is: #{sfancy(url)}"
      set_xorg_buffer(url) if is_roebe?
    else
      opnn; e "The program #{royalblue(compile_which_program?)} has no blfs entry."
    end
  end

  # ========================================================================= #
  # === report_whether_a_build_directory_will_be_used_for_this_program
  # ========================================================================= #
  def report_whether_a_build_directory_will_be_used_for_this_program
    opnn; e 'Will a '+teal('build directory')+' be used for the program '+
             sfancy(compile_which_program?)+'? '+
             simp(
               verbose_truth(use_build_directory?).delete('.')
             )+'.'
  end

  # ========================================================================= #
  # === report_where_archives_will_be_extracted_into
  # ========================================================================= #
  def report_where_archives_will_be_extracted_into
    e "Archives will be extracted into this directory: "\
      "#{sdir(extract_to?)}"
  end

  # ========================================================================= #
  # === report_all_programs_that_make_use_of_a_build_directory
  #
  # This method can report all programs that make use of a build
  # directory. Note that this presently depends on the existance
  # of the directory expanded_cookbooks.
  # ========================================================================= #
  def report_all_programs_that_make_use_of_a_build_directory
    if expanded_directory_exists?
      target = directory_expanded_cookbooks?
      all_files = Dir["#{target}*.yml"]
      all_files.select! {|entry|
        dataset = YAML.load_file(entry)
        dataset['use_build_directory']
      }
      all_files.map! {|entry|
        File.basename(entry).sub(/\.yml$/,'')
      }
      if all_files.empty?
        opnn; e 'No cookbook recipe makes use of a build directory.'
      else
        opnn; e "The following #{slateblue(all_files.size.to_s)} programs "\
                "make use of a build directory:"
        all_files.each_with_index {|this_program, index| index += 1
          padded_index = index.to_s.rjust(4).ljust(6)
          colourized_and_padded_index = royalblue(padded_index)
          e colourized_and_padded_index+
            lightblue(this_program)
        }
      end
    else
      # ===================================================================== #
      # else notify the user how to create the relevant directory.
      # ===================================================================== #
      opnn; e 'The expanded directory does not exist. You can (re)create it'
      opnn; e 'by issuing:'
      e
      e '  rbt --create-expanded-directory'
      e
    end
  end

  # ========================================================================= #
  # === show_installation_procedure_file
  #
  # Show the content of the file called installation_procedure.yml
  #
  # To invoke this method, try:
  #   rbt --installation-procedure
  # ========================================================================= #
  def show_installation_procedure_file(
      this_file = BeautifulUrl['installation_procedere']
    )
    begin
      require 'beautiful_url' unless Object.const_defined? :BeautifulUrl
    rescue LoadError; end
    this_file = this_file.first if this_file.is_a? Array
    if File.exist? this_file
      e File.read(this_file)
    else
      opnn; e 'No file called '+sfile(this_file)+' was found.'
    end
  end

  # ========================================================================= #
  # === show_the_recipes_directory
  # ========================================================================= #
  def show_the_recipes_directory
    e sfancy(recipes_dir?) # In $RBT/base/base.rb
  end

  # ========================================================================= #
  # === show_the_last_configure_option_for_this_program
  #
  # This method will make use of the file configure_command_database.yml.
  # Obviously, the file has to exist in order for this to work.
  #
  # Invocation example:
  #
  #   ry attica --show-last-configure-option-for-this-program
  #
  # ========================================================================= #
  def show_the_last_configure_option_for_this_program(
      this_program = compile_which_program?
    )
    _ = RBT.file_configure_database
    if File.exist? _
      dataset = YAML.load_file(_)
      if dataset.has_key? this_program
        result = dataset[this_program]
        opnn; e "The program #{sfancy(this_program)} has "\
                "been compiled with this command:"
        e
        e "  #{sfancy(result)}" # Display it without opnn.
        e
      else
        opnn; e "The program #{sfancy(this_program)} has not "\
                "been registered/compiled yet."
      end
    else
      opnn; e "No file exists at `#{sfile(_)}` so we can not proceed."
      opnn; e 'Nota bene: You should first compile some programs, before'
      opnn; e 'making use of this method.'
    end
  end

  # ========================================================================= #
  # === show_gobolinux_instructions
  #
  # To invoke this method do:
  #   rbt --show-gobo-instructions
  # ========================================================================= #
  def show_gobolinux_instructions
    _ = RUBY_SRC_DIR+'rbt/doc/instructions_for_gobolinux.md'
    if File.exist? _
      e File.read(_)
    else
      opnn; no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === report_n_programs
  #
  # Report how many programs (cookbook files) are currently registered in
  # the RBT project.
  #
  # Commandline-invocation example:
  #
  #   rbt --report-n-programs
  #
  # ========================================================================= #
  def report_n_programs
    RBT.report_the_registered_programs
  end

  # ========================================================================= #
  # === show_success_or_failure_status (success tag, failure tag, status tag)
  #
  # This method will report some things, aka the success or failure status
  # of a program.
  # ========================================================================= #
  def show_success_or_failure_status
    # ======================================================================= #
    # Report the following only if the program has been installed
    # successfully.
    # ======================================================================= #
    if has_the_program_been_installed_successfully?
      opnn; e "The host system architecture was: "\
              "#{slateblue(host_system?)}"
      if is_a_gem?
        # =================================================================== #
        # If it is a .gem file then we will not report as much information.
        # =================================================================== #
        opnn; e teal('The program `')+lightgreen(compile_which_program?)+
                teal('` has been installed successfully.')
      else
        opnn; e teal('The program `')+lightgreen(compile_which_program?)+
                teal('` has been installed successfully, into the prefix')
        opnn; e plum(prefix?.to_s)+teal('). The full configure command was:')
        e
        e '  '+slateblue(return_full_configure_command)
        e
      end
    else
      # ===================================================================== #
      # Else the program has failed during compilation/installation.
      # ===================================================================== #
      opnn; e crimson('The program `')+
              teal(compile_which_program?)+
              crimson('` could not be installed.')
    end
  end; alias report_status                           show_success_or_failure_status # === report_status

  # ========================================================================= #
  # === report_finished_installing_this_program
  #
  # Report that we have finished compiling the program.
  # ========================================================================= #
  def report_finished_installing_this_program(
      i = compile_this_program?
    )
    unless do_not_compile?
      unless problem? || an_error_has_occurred?
        opnn; e "Finished installing the program `#{sfancy(i)}`."
      end
    end
  end


end; end