#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/strip.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === do_strip_the_compiled_binaries
  #
  # This will strip the compiled binaries, but only if we install them
  # in an AppDir-prefix, that is, a path such as:
  #
  #   /Programs/Htop/2.0.2/
  #
  # This stripping-operation will be done by the program called "strip",
  # which is part of the binutils suite.
  #
  # There are three major ways how to use the strip essentially:
  #
  #   (1) --strip-unneeded
  #   (2) --strip-debug
  #   (3) --strip-all
  #
  # "strip --strip-debug" will only remove the debugging symbols.
  #
  # "strip --strip-all" will remove all symbols.
  #
  # "strip --strip-unneeded" will remove all symbols that are
  # not needed for relocation processing.
  #
  # Note that you can also strip the symbol table, from an object
  # file, by using the "-s option" of the strip command.
  #
  # When I tested different strip variants, in particular --strip-debug
  # and --strip-unneeded, the latter one was producing significantly
  # smaller binary files. When I used --strip-all and compared it
  # to --strip-unneeded, the results were the same - so based on
  # this, I am setting "--strip-unneeded" as the default.
  # ========================================================================= #
  def do_strip_the_compiled_binaries(
      # ===================================================================== #
      # Determine which strip-variant to use next:
      #   strip_how = '--strip-debug'
      #   strip_how = '--strip-all'
      # ===================================================================== #
      strip_how = '--strip-unneeded' # <- This is the current default.
    )
    # ======================================================================= #
    # Do not strip the binaries if an error has occurred or if the user
    # has not enabled this option.
    # ======================================================================= #
    return unless can_we_continue?
    return if an_error_has_occurred?
    return if do_not_strip_the_binaries?
    # ======================================================================= #
    # First, we must determine whether we will use the strip command or
    # whether we will not. Presently, we will use the strip command if
    # the user_prefix either includes the programs directory - or if
    # we use a path that is NOT using /usr as part of its prefix.
    #
    # As of January 2019, the above has been changed - we will only use
    # this if the real prefix includes the programs directory. Any other
    # prefixes will be ignored.
    # ======================================================================= #
    if real_prefix?.include?(programs_dir?) # or
       # !real_prefix?.include?('/usr')
      _ = rds(real_prefix?.to_s+'/bin/')
      # ===================================================================== #
      # Next obtain all entries in the bin/ subdirectory there. This may
      # lead to wrong results, though, as it may also include other files
      # if we are not careful.
      # ===================================================================== #
      all_files_in_the_bin_subdirectory = Dir["#{_}*"]
      # ===================================================================== #
      # Since as of 01.02.2019 we will only keep entries that are also
      # registered under "binaries:" in the corresponding .yml file.
      # This was specifically done to prevent running strip on e. g.
      # /usr/bin/* or /root/bin/*.
      # ===================================================================== #
      binaries = binaries?
      all_files_in_the_bin_subdirectory.select! {|entry|
        binaries.include?(File.basename(entry))
      }
      strip_command = "strip #{strip_how.lstrip}"
      unless all_files_in_the_bin_subdirectory.empty?
        how_many = sfancy(all_files_in_the_bin_subdirectory.size.to_s)
        # =================================================================== #
        # Notify the user how many executables will be modified
        # through strip next.
        # =================================================================== #
        text = "Next running \"#{sfancy(strip_command)}\" on".dup
        if all_files_in_the_bin_subdirectory.size > 1
          text << ' all'
        end
        text << ' '+how_many # Add the number here.
        if all_files_in_the_bin_subdirectory.size > 1
          text << ' binaries.'
        else
          text << ' binary.'
        end
        # =================================================================== #
        # Display our findings to the user next:
        # =================================================================== #
        opne text
        all_files_in_the_bin_subdirectory.each {|this_file|
          # ================================================================= #
          # The following command will usually run:
          #   "strip --strip-unneeded *"
          # ================================================================= #
          cmd = "#{strip_command} #{this_file}"
          opne "  #{sfancy(cmd)}"
          system(cmd) 
        }
      end
    end
  end

  # ========================================================================= #
  # === strip_the_binaries_of_this_program
  #
  # Invocation example:
  #
  #   rbt --strip-binaries-of=htop
  #
  # ========================================================================= #
  def strip_the_binaries_of_this_program(i)
    [i].flatten.compact.each {|this_program|
      binaries_of_this_program = RBT.return_binaries_of_this_program(this_program)
      if binaries_of_this_program.empty?
        opnn; e "The program #{sfancy(this_program)} has no associated binaries."
        opnn; e 'Thus, "strip" can not be run on these binaries.'
      else
        opnn; e 'Now running "strip" on '+
                sfancy(binaries_of_this_program.size.to_s)+
                ' programs:'
        binaries_of_this_program.each {|this_binary|
          run_the_strip_command_on_this_input(this_binary)
        }
      end
    }
  end

  # ========================================================================= #
  # === do_not_strip_binaries
  #
  # This is the non-verbose variant to disable stripping of binaries.
  #
  # If you need the verbose variant, use the method called
  # verbose_do_not_strip_the_binaries() instead.
  # ========================================================================= #
  def do_not_strip_binaries
    @internal_hash[:strip_binaries] = false
  end

  # ========================================================================= #
  # === strip_binaries?
  # ========================================================================= #
  def strip_binaries?
    @internal_hash[:strip_binaries]
  end

  # ========================================================================= #
  # === run_the_strip_command_on_this_input
  # ========================================================================= #
  def run_the_strip_command_on_this_input(
      i = :pwd
    )
    case i
    when :pwd
      i = Dir['*']
    end
    [i].flatten.compact.each {|this_file|
      strip_command = 'strip '+this_file.lstrip
      esystem strip_command
    }
  end

  # ========================================================================= #
  # === do_not_strip_binaries?
  # ========================================================================= #
  def do_not_strip_binaries?
    !strip_binaries?
  end; alias do_not_strip_the_binaries? do_not_strip_binaries? # === do_not_strip_the_binaries?

  # ========================================================================= #
  # === verbose_do_not_strip_the_binaries
  #
  # This will notify the user what we will do, and also toggle the flag
  # that keeps track over as to whether we will run "strip" on the
  # compiled binaries or whether we will not.
  # ========================================================================= #
  def verbose_do_not_strip_the_binaries
    opne "We will not strip the binaries (in the "\
         "#{sdir('bin/')} subdirectory)."
    do_not_strip_binaries
  end

end; end