#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/symlink.rb'
# =========================================================================== #
require 'rbt/utility_scripts/symlink_all_upcased_programs_to_downcased_variants.rb'
require 'rbt/utility_scripts/symlink_glib_schema.rb'
require 'rbt/utility_scripts/symlink_pkgconfig_files.rb'
require 'rbt/symlink_this_program/initialize.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === symlink_headers?
  #
  # Use this method if you need to find out whether we shall symlink
  # the .h files into the "/usr/include/" hierarchy or whether we
  # should not.
  # ========================================================================= #
  def symlink_headers?
    _ = cookbook_dataset_symlink_headers?
    unless @internal_hash[:symlink_headers].nil?
      _ = @internal_hash[:symlink_headers]
    end
    _
  end; alias symlink_headers_into_usr_include_hierarchy? symlink_headers? # === symlink_headers_into_usr_include_hierarchy?

  # ========================================================================= #
  # === symlink_glib_schema
  #
  # The default prefix, aka the first input argument, should be fine
  # for this class.
  # ========================================================================= #
  def symlink_glib_schema(
      prefix_where_the_schema_files_reside =
        program_dir?+
        internal_program_name?.capitalize+
        '/Current/share/glib-2.0/schemas/'
    )
    if File.directory? prefix_where_the_schema_files_reside
      RBT::SymlinkGlibSchema.new(prefix_where_the_schema_files_reside) {
        :do_not_report_when_anything_is_missing
      }
    end
  end

  # ========================================================================= #
  # === consider_symlinking_glib_schema_files_into_the_usr_share_hierarchy
  #
  # Invoke this method if we wish to symlink the schema files into the
  # /usr/ hierarchy.
  # ========================================================================= #
  def consider_symlinking_glib_schema_files_into_the_usr_share_hierarchy
    if uses_appdir_prefix? and
       use_glib_schema? and
      !an_error_has_occurred?
      # ===================================================================== #
      # Delegate towards class SymlinGlibSchema eventually:
      # ===================================================================== #
      symlink_glib_schema
    end
  end

  # ========================================================================= #
  # === prepare_symlink_program_instance
  # ========================================================================= #
  def prepare_symlink_program_instance
    symlink_program = symlink_program?
    # ======================================================================= #
    # Assign the prefix to SymlinkProgram next:
    # ======================================================================= #
    symlink_program?.set_use_this_program(
      program_name?
    )
    symlink_program?.set_version(program_version?)
    symlink_program.disable_colours unless use_colours?
    symlink_program.do_not_symlink = do_not_symlink? # Honour configure option here.
    symlink_program.use_opn = RBT::Compile.show_the_classname?
    # ======================================================================= #
    # Next, prevent auto-registering into the local yaml database.
    # This part is already handled via register_program() in
    # the file postinstall.rb
    # ======================================================================= #
    symlink_program.do_not_register_into_the_local_yaml_database
  end

  # ========================================================================= #
  # === do_not_symlink?
  # ========================================================================= #
  def do_not_symlink?
    @internal_hash[:do_not_symlink]
  end

  # ========================================================================= #
  # === cookbook_dataset_symlink_headers?
  # ========================================================================= #
  def cookbook_dataset_symlink_headers?
    cookbook_dataset?.symlink_headers
  end

  # ========================================================================= #
  # === symlink_pkgconfig_files?
  # ========================================================================= #
  def symlink_pkgconfig_files?
    cookbook_dataset?.symlink_pkgconfig_files?
  end

  # ========================================================================= #
  # === remove_all_symlinks
  #
  # Delegate to RBT::RemoveAllSymlinks for this action.
  # ========================================================================= #
  def remove_all_symlinks
    RBT::RemoveAllSymlinks.new
  end

  # ========================================================================= #
  # === do_not_symlink_pkgconfig_files
  # ========================================================================= #
  def do_not_symlink_pkgconfig_files
    set_pkgconfig_symlink(false)
  end

  # ========================================================================= #
  # === replace_symlinks
  #
  # Delegate towards RBT::ReplaceSymlinks here. We will work on the
  # current working directory.
  # ========================================================================= #
  def replace_symlinks(
      from_where = return_pwd
    )
    RBT::ReplaceSymlinks.new(from_where) # bl $RBT/replace_symlinks.rb
  end

  # ========================================================================= #
  # === symlink_all_programs
  #
  # Use this method to symlink all programs in the $PROGRAMS directory.
  #
  # Usage example to trigger this method:
  #
  #   ry htop symlink_all_programs
  #
  # ========================================================================= #
  def symlink_all_programs(
      may_we_symlink = !do_not_symlink?
    )
    case may_we_symlink
    when :do_it
      may_we_symlink = true
    end
    # ======================================================================= #
    # @do_not_symlink = dataset?.do_not_symlink? if dataset?.do_not_symlink?
    # ======================================================================= #
    if may_we_symlink
      _ = RBT::SymlinkAllPrograms.new # bl $RBT/utility_scripts/symlink_all_programs.rb
      opne "We have finished symlinking all programs in "\
           "#{sdir(_.programs_directory?)}."
    end
  end

  # ========================================================================= #
  # === do_symlink_all_programs_then_exit
  # ========================================================================= #
  def do_symlink_all_programs_then_exit
    symlink_all_programs :do_it
    exit_program
  end

  # ========================================================================= #
  # === symlink_all_upcased_programs_to_downcased_variants
  # ========================================================================= #
  def symlink_all_upcased_programs_to_downcased_variants
    RBT::SymlinkAllUpcasedProgramsToDowncasedVariants.new
  end

  # ========================================================================= #
  # === verbose_enable_pkgconfig_symlink
  # ========================================================================= #
  def verbose_enable_pkgconfig_symlink
    show_enable_pkgconfig_symlinking_message
    set_pkgconfig_symlink 'yes'
  end

  # ========================================================================= #
  # === show_enable_pkgconfig_symlinking_message
  # ========================================================================= #
  def show_enable_pkgconfig_symlinking_message
    opne "Enabling pkgconfig-symlinking of .pc files into `"\
         "#{sdir('/usr/lib/pkgconfig/')}`."
  end

  # ========================================================================= #
  # === consider_symlinking_the_pkgconfig_files
  #
  # If the cookbook has symlinking_pkgconfig_files set to true, and
  # if the program is compiled, as AppDir, then we will symlink .pc
  # files into /usr/lib/pkgconfig/, if the latter one exists.
  # ========================================================================= #
  def consider_symlinking_the_pkgconfig_files
    target_dir = '/usr/lib/pkgconfig/'
    if symlink_pkgconfig_files? and
       is_appdir_compilation_style? and
       File.directory?(target_dir) and
       !do_not_symlink?
      appdir_target_with_pc_files = "#{return_appdir_prefix}lib/pkgconfig/"
      # ===================================================================== #
      # Delegate towards class SymlinkPkgconfigFiles next.
      # ===================================================================== #
      SymlinkPkgconfigFiles.new(appdir_target_with_pc_files)
    end
  end; alias consider_symlinking_pkgconfig_files consider_symlinking_the_pkgconfig_files # === consider_symlinking_pkgconfig_files

  # ========================================================================= #
  # === do_symlink_appdir_lib_into_usr_lib
  #
  # This method will enable the symlinkinkg part from an AppDir path
  # to a /usr/lib/ path.
  # ========================================================================= #
  def do_symlink_appdir_lib_into_usr_lib
    @internal_hash[:symlink_into_usr_lib_dir] = true
  end

  # ========================================================================= #
  # === set_pkgconfig_symlink
  # ========================================================================= #
  def set_pkgconfig_symlink(
      i = true
    )
    case i # Sanitize yes/no values.
    when 'yes','y'
      i = true
    when 'no','n'
      i = false
    end
    if block_given?
      case yielded
      when :be_verbose
        if i == true
          show_enable_pkgconfig_symlinking_message
        else
          opne "Disabling pkgconfig-symlinking of .pc files into `"\
               "#{sdir('/usr/lib/pkgconfig/')}`."
        end
      end
    end
    if cookbook_dataset?
      cookbook_dataset?.symlink_pkgconfig_files = i
    else
      e 'No cookbook-dataset object could be found.'
    end
  end

  # ========================================================================= #
  # === verbose_do_symlink_headers_into_usr_include_hierarchy
  #
  # This method can be called from the commandline. As a consequence,
  # we must modify the @internal_hash rather than the dataset stored
  # in the cookbooks.
  # ========================================================================= #
  def verbose_do_symlink_headers_into_usr_include_hierarchy
    opne 'We will try to symlink '+orange('.h')+' files into '\
         'the '+sdir('/usr/include/')+' hierarchy.'
    @internal_hash[:symlink_headers] = true
  end

  # ========================================================================= #
  # === do_not_symlink_in_general_and_also_do_not_symlink_header_files
  # ========================================================================= #
  def do_not_symlink_in_general_and_also_do_not_symlink_header_files
    set_do_not_symlink
    do_not_symlink_header_files
  end

  # ========================================================================= #
  # === do_not_symlink_header_files
  #
  # This will disable symlinking the header files. By default, we will not
  # be verbose and thus do not output anything to the user.
  # ========================================================================= #
  def do_not_symlink_header_files
    @internal_hash[:symlink_headers] = false
  end; alias do_not_symlink_headers do_not_symlink_header_files # === do_not_symlink_headers

  # ========================================================================= #
  # === set_do_not_symlink
  #
  # Do not symlink after "make install" stage. This prevents the
  # symlinking-step from e. g. /Programs/Htop/1.2.0/ into the
  # /Systemd/Index/ hierarchy.
  # ========================================================================= #
  def set_do_not_symlink(
      be_verbose = true
    )
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        be_verbose = true
      end
    end
    case be_verbose
    when :be_verbose
      be_verbose = true
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      opne 'We will not symlink after compilation has finished.'
    end
    @internal_hash[:do_not_symlink] = true
  end; alias do_not_symlink set_do_not_symlink # === do_not_symlink

  # ========================================================================= #
  # === set_symlink_program
  # ========================================================================= #
  def set_symlink_program(
      compile_this_program     = compile_which_program?,
      use_this_program_version = program_version?
    )
    # ======================================================================= #
    # We will delegate towards class RBT::SymlinkThisProgram next.
    # ======================================================================= #
    @internal_hash[:symlink_program] = RBT::SymlinkThisProgram.new(
      [ compile_this_program, use_this_program_version ],
      :do_not_run_yet
    ) # bl symlink_program
  end

end; end