#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Time-related code for class RBT::Compile will be kept in this file.
# =========================================================================== #
# require 'rbt/compile/time.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === show_compile_time_statistics
  #
  # This method will show compile-time statistics for all the programs
  # that the user has compiled through RBT.
  #
  # Invocation example:
  #   ry --show-compile-time-statistics
  # ========================================================================= #
  def show_compile_time_statistics
    _ = RBT.file_storing_the_time_it_took_to_compile_programs
    if File.exist? _
      dataset = YAML.load_file(_)
      if dataset.empty?
        opne 'The dataset is empty. Have there any programs been '\
             'compiled yet, through '+teal('rbt')+'?'
      else
        opne "Showing the compile-time statistics for the "\
             "following #{sfancy(dataset.keys.size.to_s)} programs."
        e
        # =================================================================== #
        # Need to sort this dataset by size still.
        # =================================================================== #
        sorted_longest_compilation_comes_first = dataset.sort_by {|name_of_the_program, n_seconds|
          n_seconds
        }.reverse
        # =================================================================== #
        # Show a little header explaining what this is all about.
        # =================================================================== #
        e gray('-' * 80)
        e sfancy('Name of the program'.ljust(28))+
          simp('Compile time in seconds ')+
          orange('Compile time in minutes')+
          mediumslateblue(' File size of the archive')
        e gray('-' * 80)
        sorted_longest_compilation_comes_first.each {|name_of_the_program, n_seconds|
          left_padded  = ("#{name_of_the_program.strip}:").ljust(26)
          right_padded_showing_n_seconds = (
            '%.2f' % n_seconds.to_s.strip.to_f
          ).rjust(12)
          right_padded_showing_n_seconds << ' seconds'
          n_minutes = (n_seconds.to_f / 60.0).round(2) # Rounding to 2 is better.
          n_minutes = n_minutes.to_s.ljust(4,'0')
          # ================================================================= #
          # Next count the amount of numbers before the '.' point. If there
          # are at the least two numbers, we will .ljust(5,'0') rather than
          # .ljust(4,'0').
          # ================================================================= #
          if n_minutes.include? '.'
            first_splitted = n_minutes.split('.').first
            if first_splitted.size > 1
              n_minutes = n_minutes.to_s.ljust(5,'0')
            end
          end
          # ================================================================= #
          # The program name may be upcased; since we store only downcased
          # variants for the name of the .yml file at hand, we will also
          # .downcase() on the program name in the following code.
          # ================================================================= #
          archive_size = fast_return_file_size_of_this_program(
            name_of_the_program.downcase.delete('-')
          )
          colourized_file_size = mediumslateblue(
            "#{archive_size.to_s.rjust(10)} Kb"
          )
          result = '  '+sfancy(left_padded)+' '+
            simp(right_padded_showing_n_seconds)+' '+
            orange(
              ('('+n_minutes).rjust(6)+' minutes)'
            )
          result << ' '+colourized_file_size
          e result
        }
        e
      end
    else
      opne "No file exists at `#{sfile(_)}`."
      opne 'We thus can not show any statistics.'
    end
  end

  # ========================================================================= #
  # === consider_reporting_the_time_it_took_to_compile_this_program
  # ========================================================================= #
  def consider_reporting_the_time_it_took_to_compile_this_program
    if time_the_compilation? and no_error_has_occurred?
      program_name = compile_which_program?.to_s # Keep a reference.
      _ = @internal_hash[:array_time_compilation]
      # ===================================================================== #
      # We need the start-time and the end-time, in order to calculate
      # the difference.
      # ===================================================================== #
      start_time = @internal_hash[:array_time_compilation][-2]
      end_time   = @internal_hash[:array_time_compilation][-1]
      # ===================================================================== #
      # We must next check that both end time and start time exist, as
      # otherwise we would run into an error.
      # ===================================================================== #
      if start_time and end_time
        duration   = (end_time - start_time)
        rounded_duration = duration.round(3)
        in_minutes = (rounded_duration.to_f / 60.0).round(2)
        if has_to_be_compiled?
          compiling_or_installing = 'Compiling'
        else
          compiling_or_installing = 'Installing'
        end
        # ===================================================================== #
        # Notify the user how long it took to compile/install the given
        # program at hand.
        # ===================================================================== #
        opne "#{compiling_or_installing} the program `"\
             "#{olivedrab(program_name)}"\
             "` took #{sfancy(rounded_duration.to_s)} seconds "\
             "(aka #{sfancy(in_minutes)} minutes)."
        # ===================================================================== #
        # Also store this now into a file.
        # ===================================================================== #
        _ = RBT.file_storing_the_time_it_took_to_compile_programs
        if File.exist? _
          dataset = YAML.load_file(_)
        else
          dataset = {} # Use a Hash format.
        end
        dataset[program_name] = rounded_duration
        opne 'Also storing this information into the file'
        opne "  #{sfile(_)}"
        write_what_into(YAML.dump(dataset), _)
      else
        opne 'Can not store the time as the variable tracking the time'
        opne 'is incorrect. It will be displayed next, to aid in debugging:'
        e
        pp @internal_hash[:array_time_compilation][-1]
        e
      end
    end
  end

  # ========================================================================= #
  # === consider_storing_a_snapshot
  #
  # This method may store the compilation time into an Array.
  # ========================================================================= #
  def consider_storing_a_snapshot
    if @internal_hash[:time_compilation]
      @internal_hash[:array_time_compilation] << Time.now
    end
  end; alias snapshot            consider_storing_a_snapshot # === snapshot
       alias time_snapshot       consider_storing_a_snapshot # === time_snapshot
       alias store_time_snapshot consider_storing_a_snapshot # === store_time_snapshot

  # ========================================================================= #
  # === time_the_compilation?
  # ========================================================================= #
  def time_the_compilation?
    @internal_hash[:time_compilation]
  end

  # ========================================================================= #
  # === return_compile_time_statistics
  # ========================================================================= #
  def return_compile_time_statistics
    _ = RBT.file_storing_the_time_it_took_to_compile_programs
    if File.exist? _
      dataset = YAML.load_file(_)
      return dataset
    end
  end

end

# =========================================================================== #
# === RBT.show_compile_time_statistics
#
# Easier top-level API to show the compile-time statistics on the
# commandline.
# =========================================================================== #
def self.show_compile_time_statistics
  RBT::Compile.new(
    nil, :dont_run_yet
  ).show_compile_time_statistics
end

end