#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Activities related to uploads and downloads will be stored in this
# file here, as far as RBT::Compile is concerned.
# =========================================================================== #
# require 'rbt/compile/upload_and_download.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/time.rb'

module RBT

class Compile < RBT::CompileBase

  # ========================================================================= #
  # === download_this_remote_url_into_that_local_directory
  #
  # The first argument should be the remote URL to the program at
  # hand.
  #
  # The second argument should be a local directory.
  # ========================================================================= #
  def download_this_remote_url_into_that_local_directory(
      url             = url1?,
      local_directory = '/Depot/Temp/downloads/'
    )
    local_directory = local_directory.dup if local_directory.frozen?
    local_directory << '/' unless local_directory.end_with? '/'
    unless File.directory? local_directory
      create_directory(local_directory) { :be_verbose }
    end
    cd local_directory # change directory into that target
    download_program(
      url,
      :we_may_download,
      local_directory,
      :do_not_update_the_dataset
    )
  end

  # ========================================================================= #
  # === download_program           (download tag, download archive, wget tag)
  #
  # The method `download_program` will attempt to download the given 
  # program, which is the first argument passed to this method. An
  # URL is expected as argument here, such as
  # 'http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz'.
  # 
  # For this download-related activity, we will either use a plain
  # system() call to "wget" or we may use a ruby wget-wrapper. On
  # Linux or BSD-based systems 'wget' may be simpler. On windows we
  # may have to use a ruby-variant of 'wget' instead.
  #
  # Do note that the method download_program() will always assume
  # that the first argument given is an Array - that is, an Array of
  # multiple urls. It will download all passed remote programs in
  # a batch-like operation. This allows you to e. g. download the
  # whole KDE stack, in principle, by simply passing the Array of
  # remote entries.
  #
  # Please keep in mind that we will change_directory() in this method
  # too, so you don't have to call that on your own.
  #
  # As of 05 September 2014 we will also honour the "do_not_download"
  # setting, if it was defined. This can be overruled in the arguments
  # to this method.
  # ========================================================================= #
  def download_program(
      url_for_program             = url1?,
      may_we_download             = may_we_download?,
      change_to_this_directory    = base_dir?,
      shall_we_update_the_dataset = true
    )
    url_for_program = [url_for_program].flatten.compact
    # ======================================================================= #
    # We will handle some special Symbols next:
    # ======================================================================= #
    case shall_we_update_the_dataset
    when :do_not_update_the_dataset
      shall_we_update_the_dataset = false
    end
    case may_we_download
    when :do_download,
         :we_may_download
      may_we_download = true
    end
    case change_to_this_directory
    when :default
      change_to_this_directory = base_dir?
    end
    url_for_program.each {|use_this_remote_url|
      if may_we_download
        url_for_program = use_this_remote_url
        url_for_program = url1? if url_for_program == :def
        # ======================================================================= #
        # Since Jul 2011 we use pure wget for this. Something is odd with
        # RubyWget and until we have fixed this, plain wget has to be used.
        # ======================================================================= #
        use_svn = dataset?.svn_url?
        # ======================================================================= #
        # Check whether the base directory exists.
        # ======================================================================= #
        if !File.exist?(change_to_this_directory.to_s)
          opnn; e simp('Creating the directory `')+
                  sdir(change_to_this_directory)+
                  simp('` next.')
          create_directory(change_to_this_directory)
        end
        # ======================================================================= #
        # Change into the base directory first, before continuing to try
        # to download the archive in question.
        # ======================================================================= #
        change_directory(change_to_this_directory) # cd into the archive.
        # ======================================================================= #
        # Change into the archive. Be verbose about it.
        # ======================================================================= #
        cliner {
          opnn; e "#{simp('Downloading ')}#{sfile(url_for_program)}"
          opnn; e simp('into the directory ')+sdir(change_to_this_directory)+
                  simp(' now.')
        }
        if url_for_program.to_s.include? 'tar.gz'
          # overrule here for .tar.gz files.
          _ = cookbook_dataset? #.data
          _.set_archive_type('.tar.gz')
          _.set_program_full_name
          _.assemble_program_path
          @data = cookbook_dataset?.return_dataset
        end
        if use_svn.empty? # Assume no SVN was given. bl rubywget
          do_download_this_program(url_for_program)
          # RubyWget.new(url_for_program)
        else # Else, we should use svn perhaps? Hmm or not.
          if _
            do_download_this_program(_)
          else
            do_download_this_program(url_for_program)
          end
          # RubyWget.new(_)
        end
        if shall_we_update_the_dataset
          # ================================================================= #
          # We will update the dataset as well.
          # ================================================================= #
          yaml_dataset = load_yaml_dataset_for_this_program(
            program_information_return_name(url_for_program), :return_flat_datastructure
          )
          assign_to_the_main_dataset(
            yaml_dataset
          )
        end
      end
    }
  end; alias download download_program # === download

  # ========================================================================= #
  # === do_upload_binaries
  # ========================================================================= #
  def do_upload_binaries
    @internal_hash[:upload_binaries] = true
  end

  # ========================================================================= #
  # === upload_binaries?
  # ========================================================================= #
  def upload_binaries?
    @internal_hash[:upload_binaries]
  end

  # ========================================================================= #
  # === consider_uploading_binaries
  #
  # This method can be used to upload some binaries.
  #
  # Invocation example:
  #   ry htop uploadbinaries
  # ========================================================================= #
  def consider_uploading_binaries(
      upload_binaries = upload_binaries?
    )
    if upload_binaries
      opnn; e 'We will next upload any files found in the bin/ subdirectory.'
      _ = "#{dataset?.prefix}bin/"
      files = Dir["#{_}*"]
      if files.empty?
        opnn; e "No files were found in #{sdir(_)}."
      else
        upload_to(
          files, :default, 'bin'
        )
      end
    end
  end

  # ========================================================================= #
  # === download_using_svn
  #
  # Use this method here to download/checkout from svn. In order for this
  # to properly work, you need to define a svn_url in the yaml file for
  # the respective program at hand.
  #
  # To try out this method, do:
  #
  #   ry dosbox svn
  #
  # ========================================================================= #
  def download_using_svn(
      i = program?
    )
    _ = "svn co #{dataset?.svn_url?}"
    working_directory = sdir(return_current_pwd)
    opnn; e "Now trying to use svn to download the source directly "\
            "(working directory is: #{working_directory}."
    opnn; print '  '
    e _
    _result = `#{_}` # We may want to use this return-value at a later time.
    # ======================================================================= #
    # === Rename the trunk directory next
    # ======================================================================= #
    if File.directory? 'trunk'
      _ = "#{i}-#{today?}"
      opnn; e 'Now renaming directory `'+sdir('trunk')+'` to `'+simp(_)+'`.'
      rename('trunk', _)
      unless Object.const_defined? :ToTarXz
        begin # Try to pull in gem to_tar_xz here.
          require 'to_tar_xz'
        rescue LoadError; end
      end
      if Object.const_defined? :ToTarXz
        opnn; e 'Now trying to package the directory `'+sdir(_)+'`.'
        ToTarXz.new(_)
        package_name = Dir[_+'.tar.xz']
        unless package_name.empty?
          e 'All should be done now. You can move this .tar.xz '\
            'package ('+simp(package_name.first)+').'
        end
      end
    end
  end

  # ========================================================================= #
  # === default_download_then_exit
  # ========================================================================= #
  def default_download_then_exit(i = :default)
    download_program(:def, true, i) # Download here. Overrule settings.
    exit_program # Then exit.
  end

  # ========================================================================= #
  # === may_we_download?
  # ========================================================================= #
  def may_we_download?
    result = ihash?[:may_we_download]
    return result
  end

  # ========================================================================= #
  # === do_download_this_program                               (download tag)
  #
  # This is simply a plain wget wrapper.
  # ========================================================================= #
  def do_download_this_program(i)
    esystem 'wget '+i
  end; alias do_download                  do_download_this_program # === do_download
       alias download_this_remote_program do_download_this_program # === download_this_remote_program
       alias wget                         do_download_this_program # === wget

  # ========================================================================= #
  # === batch_download_all_source_archives
  #
  # For this method we have to delegate towards class
  # Cookbooks::DownloadAllSourceArchives.
  # ========================================================================= #
  def batch_download_all_source_archives
    require 'rbt/toplevel_methods/download_all_source_archives.rb'
    RBT.download_all_source_archives
  end

  # ========================================================================= #
  # === do_not_download
  #
  # Setter-method to disable downloading the remote source package.
  # ========================================================================= #
  def do_not_download
    ihash?[:may_we_download] = false
  end

end; end