#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/version.rb'
# =========================================================================== #
module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === use_a_specific_program_version
  #
  # Use this method to set to a specific program version.
  #
  # Currently we will act on --version= and on --use-version=.
  #
  # To check the regex, look here:
  #
  #   http://rubular.com/r/Wnq91wy8Nx
  #
  # To test this, try any of the following:
  #
  #   ry gcc --version=4.5.3.tar.xz
  #   ry gcc --use-version=4.5.3.tar.xz
  #   ry nano --v=2.4.3
  #   ry mesa --use-this-version=mesa-18.2.6
  # 
  # ========================================================================= #
  def use_a_specific_program_version(i)
    i = i.dup
    # ======================================================================= #
    # We don't want file-extensions here.
    # ======================================================================= #
    i = remove_file_extension(i)
    if i.include?('=')
      splitted = i.split('=')
      i = splitted.last
    end
    # ======================================================================= #
    # Now we have e. g. input such as "2.4.3" or "mesa-18.2.6".
    # ======================================================================= #
    if i.include?('-') or i.include?('_')
      set_program_name(
        ProgramInformation.return_program_name(i)
      )
      set_program_short_name(
        ProgramInformation.return_program_name(i)
      )
      i = ProgramInformation.return_version(i).dup
    else
      # Else we will just use it directly as version.
    end
    # ======================================================================= #
    # In the past we used a regex, like shown here:
    #
    #   https://rubular.com/r/Z6LP2FCENzPj
    #
    # But we probably don't have to do so since January 2019.
    # ======================================================================= #
    set_program_version(i)
    opne "Setting to the specific version `#{royalblue(i)}` "\
         "now (for the program #{orange(compile_which_program?)})."
    # ======================================================================= #
    # Set it in the internal Hash - it will be synced lateron, see the
    # file sync.rb for this.
    # ======================================================================= #
    new_target_prefix = rds(
      "#{programs_directory?}/#{compile_which_program?.capitalize}/#{i}/"
    )
    # ======================================================================= #
    # We will also modify the original program_path.
    # ======================================================================= #
    set_user_prefix(
      new_target_prefix
    )
    unless archive_type?.empty?
      empty_archive_type
    end
    update_program_name
    set_program_full_name(
      cookbooks_program_name?
    )
    update_program_path
  end

  # ========================================================================= #
  # === set_program_version
  #
  # Use this method to assign to a new program version at hand.
  # ========================================================================= #
  def set_program_version(i)
    # ======================================================================= #
    # The program version is presently (December 2018) not allowed
    # to include a '/' character - possibly never allowed to have
    # this character. Thus, if we encounter such a character we
    # will apply File.basename() on this input.
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i).delete('/')
    end
    i.delete!('/') if i.include? '/'
    @internal_hash[:use_this_specific_program_version] = i
    # ======================================================================= #
    # The following code is semi-experimental - we will also sync onto
    # the cookbook dataset object if it exists. We do this primarily
    # so that class Infobox can show the proper prefix in use.
    # ======================================================================= #
    if cookbook_dataset?
      cookbook_dataset?.set_program_version(i)
      if use_appdir_prefix?
        cookbook_dataset?.update_appdir_prefix
        # =================================================================== #
        # We know that we wish to use a special appdir prefix, so use it.
        # =================================================================== #
        set_user_prefix(return_appdir_prefix)
      end
    end
  end

  # ========================================================================= #
  # === do_compile_all_available_local_version_of_this_program
  #
  # Invocation example:
  #   ry libsigc++ --compile-all-available-local-versions
  # ========================================================================= #
  def do_compile_all_available_local_version_of_this_program(
      i = compile_which_program?
    )
    opne 'Checking for the locally available '\
         'versions of the program '+sfancy(i)+' next.'
    target_directory = source_dir?+i.downcase+'/'
    e
    opne "We will check at the directory `#{sdir(target_directory)}`."
    e
    matches = Dir[target_directory+'*']
    if matches.empty?
      opne 'No result was found at the directory `'+sdir(target_directory)+'`.'
      opne 'This usually indicates that the '\
           'directory does not exist - or is empty.'
    else
      matches.each {|entry|
        opne 'Working on the entry `'+sfancy(entry)+'` next.'
        entry = remove_archive_at_the_end(entry)
        basename = File.basename(entry)
        program_name    = ProgramInformation.return_program_name(basename)
        program_version = ProgramInformation.return_version(basename)
        reset
        set_compile_this_program(program_name)
        initialize_dataset_for_this_program(program_name)
        set_program_version(program_version)
        do_compile_this_program(program_name) { :do_not_initialize_the_dataset }
      }
    end
  end; alias do_compile_all_available_local_version do_compile_all_available_local_version_of_this_program # === do_compile_all_available_local_version

  # ========================================================================= #
  # === check_for_available_versions
  #
  # Use this method if you wish to check for available program versions.
  #
  # Usage examples:
  #   ry bluefish available_versions
  #   ry gcc available_versions
  # ========================================================================= #
  def check_for_available_versions(
      for_this_program = program?
    )
    require 'rbt/utility_scripts/check_for_available_versions.rb'
    cfav = RBT::Cookbooks::CheckForAvailableVersions.new(
      for_this_program, :do_not_run_yet
    )
    cfav.be_verbose
    cfav.run
  end

  # ========================================================================= #
  # === use_this_specific_program_version?
  # ========================================================================= #
  def use_this_specific_program_version?
    @internal_hash[:use_this_specific_program_version]
  end

  # ========================================================================= #
  # === use_this_compile_version
  #
  # To try this, do:
  #
  #   ry glib --compile-version=2.16.33_ROEBE
  #
  # ========================================================================= #
  def use_this_compile_version(i)
    compile_as_appdir(nil, :be_silent)
    set_program_version(i)
    set_use_this_prefix(static_appdir_prefix?)
  end

  # ========================================================================= #
  # === report_rbt_version
  #
  # This method will report the version in use by the RBT project.
  #
  # Invocation examples:
  #   ry --version
  #   rbt version?
  # ========================================================================= #
  def report_rbt_version(
      optional_shall_we_exit = false
    )
    case optional_shall_we_exit
    when :also_exit,
         :then_exit
      optional_shall_we_exit = true
    end
    RBT.report_rbt_version
    exit if optional_shall_we_exit
  end

  # ========================================================================= #
  # === do_use_random_version
  #
  # Use a random version of a program.
  #
  # Usage example:
  #   ry gcc --use-random-version
  # ========================================================================= #
  def do_use_random_version(
      this_program = compile_which_program?
    )
    cfav = RBT::Cookbooks::CheckForAvailableVersions.new(this_program)
    available_programs = cfav.programs?.map {|entry| File.basename(entry) }
    _ = available_programs.sample
    opne "We will use another version (for program #{simp(_)})"
    dataset?.set_complex_program_version(_) # This here sets the new version.
  end

  # ========================================================================= #
  # === program_version?
  #
  # This method will return the program version in use. The default will
  # be to simply use the version stored in the cookbook dataset, but
  # we also allow for the user to supply commandline-flags to use any
  # other program version that he/she desires to use.
  # ========================================================================= #
  def program_version?
    _ = cookbook_dataset_program_version? # <- Default value.
    if @internal_hash[:use_this_specific_program_version]
      _ = @internal_hash[:use_this_specific_program_version]
    end
    _.delete('/') if _ # A bit hackish, since it is a workaround, but necessary for now (Jan 2019).
  end; alias version?        program_version? # === version?
       alias program_version program_version? # === program_version

end; end