#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/configuration/initialize_configuration.rb'
# =========================================================================== #
require 'rbt/configuration/configuration_instance_variable.rb'
require 'rbt/toplevel_methods/configuration_directory.rb' # <- This has the method RBT.configuration_directory?.
require 'rbt/toplevel_methods/system_directory.rb'
require 'rbt/toplevel_methods/source_base_directory.rb'
require 'rbt/toplevel_methods/programs_directory.rb'
require 'rbt/toplevel_methods/temp_directory.rb'
require 'rbt/toplevel_methods/depot_directory.rb'
require 'rbt/toplevel_methods/packages_directory.rb'
require 'rbt/toplevel_methods/pkgconfig_directory.rb'
require 'rbt/toplevel_methods/sysbin_directory.rb'
require 'rbt/toplevel_methods/syslib_directory.rb'
require 'rbt/toplevel_methods/sysinclude_directory.rb'
require 'rbt/toplevel_methods/system_settings_directory.rb'
require 'rbt/toplevel_methods/system_share_directory.rb'
require 'rbt/toplevel_methods/rds.rb'
require 'rbt/toplevel_methods/log_directory.rb'

module RBT

  # ========================================================================= #
  # === RBT.initialize_configuration
  #
  # Use a method to initialize the configuration for the RBT project.
  # ========================================================================= #
  def self.initialize_configuration
    # ======================================================================= #
    # We will set the @configuration variable to a specific value here.
    # ======================================================================= #
    RBT.set_configuration(
      # ===================================================================== #
      # Load up the data from the directory. This requires the file called
      # 'configuration_directory.rb'.
      # ===================================================================== #
      RBT.configuration_directory? # <- This is a String such as "/Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/configuration/"
    )
    # ======================================================================= #
    # === enable_gobolinux_config
    #
    # Use a specific "template" when we are on a GoboLinux distribution.
    #
    # Do also note that in the more recent GoboLinux versions, say past
    # the year 2017, the directories have changed a bit. The following
    # code will ONLY honour the current versions, not the older legacy
    # GoboLinux versions.
    # ======================================================================= #
    if @configuration.respond_to?(:enable_gobolinux_config) and
       @configuration.enable_gobolinux_config
      RBT.set_system_directory('/System/') # === SYSTEM_DIR
      @system_index_directory    = "#{SYSTEM_DIR}Index/"
      RBT.set_source_directory('/Files/Compile/Source/') # /Files/Compile/Archives/
      RBT.set_programs_directory('/Programs/') # Hardcoded.
      RBT.set_temp_directory(SYSTEM_DIR+'Variable/tmp/') # CONFIG.temp_dir
      RBT.set_depot_directory('/Depot/')
      @system_links_directory    = "#{SYSTEM_DIR}Links/"
      RBT.set_packages_directory("#{DEPOT_DIR}Packages/") # This is not a Gobolinux Directory, though.
      @pkgconfig_directory       = "#{SYSTEM_LINKS_DIR}Libraries/pkgconfig/"
      RBT.set_sysbin_directory("#{SYSTEM_INDEX}bin/")
      RBT.set_syslib_directory("#{SYSTEM_INDEX}lib/")
      RBT.set_sysinclude_directory("#{SYSTEM_INDEX}include/")
      RBT.set_system_settings_directory("#{SYSTEM_DIR}Settings/")
      RBT.set_system_share_directory("#{SYSTEM_LINKS_DIR}Shared/")
      @system_libexec_directory      = "#{SYSTEM_INDEX}libexec/"
      @system_environment_directory  = "#{SYSTEM_LINKS_DIR}Environment/"
      @system_tasks_directory        = "#{SYSTEM_LINKS_DIR}Tasks/"
      @system_logs_directory         = "#{SYSTEM_DIR}Logs/"
      @system_kernel_directory       = "#{SYSTEM_DIR}Kernel/"
    else # ================================================================== #
         # Else, we will set the default variables here via the CONFIG
         # object. Keep a trailing / here.
      # ===================================================================== #
      # Keep a reference to the TEMP Directory used by the RBT Scripts here.
      # We will extract archives into this directory (the temp directory).
      # Before we will assign to the constant, we first have to do some
      # sanitizing.
      # ===================================================================== #
      _ = rds(
        @configuration.temp_directory
      )
      if _.include? '#' # Chop off in this case.
        _ = _[0, _.index('#')].strip
        _ << '/' unless _.end_with? '/'
      end
      # ===================================================================== #
      # _ may be "/Depot/Temp/rbt/".
      # ===================================================================== #
      RBT.set_temp_directory(_)
      # ===================================================================== #
      # Keep a reference to the Programs dir.
      # ===================================================================== #
      RBT.set_programs_directory(@configuration.programs_directory)
      # ===================================================================== #
      # === The source directory
      # ===================================================================== #
      RBT.set_source_directory(@configuration.source_directory) # Where to store all Source files.
      # ===================================================================== #
      # === The /System/ directory
      # ===================================================================== #
      RBT.set_system_directory(@configuration.system_directory) # '/System/'
      RBT.set_sysbin_directory(@configuration.sysbin_directory) # Standard sysbin directory.
      RBT.set_syslib_directory(@configuration.syslib_directory) # Standard syslib directory.
      RBT.set_sysinclude_directory(@configuration.sysinclude_dir) # Standard include directory.
      # SYSLIB_DIR+'/pkgconfig/' # including pkgconfig
      RBT.set_depot_directory(@configuration.depot_dir)
      @system_logs_directory = rds(@configuration.system_logs+'/') # SYSTEM_DIR+'Logs' # Where to keep our syslogs.
      RBT.set_log_directory # Default it to some value.
      # ===================================================================== #
      # === SYSTEM_SETTINGS_DIR
      #
      # Where to keep the /etc equivalent.
      # ===================================================================== #
      RBT.set_system_settings_directory(@configuration.system_settings_dir)
      # ===================================================================== #
      # === The Packages Dir
      #
      # On my system it is at /Depot/Packages.
      # ===================================================================== #
      RBT.set_packages_directory(@configuration.packages_directory)
      # ===================================================================== #
      # === MY_SYSSHARE
      #
      # Next, handle the SYSSHARE_DIR directory.
      # ===================================================================== #
      if ENV['MY_SYSSHARE']
        RBT.set_system_share_directory(ENV['MY_SYSSHARE'].dup)
      else
        RBT.set_system_share_directory(@configuration.sysshare_directory)
      end
      # ===================================================================== #
      # === PKGCONFIG_DIR
      #
      # PKGCONFIG_DIR is hardcoded for now.
      # ===================================================================== #
      @pkgconfig_directory = rds(@configuration.pkgconfig_dir+'/')
    end
  end

  # ========================================================================= #
  # Initialize the configuration dataset next, unless this has
  # already been done so.
  # ========================================================================= #
  RBT.initialize_configuration if @configuration.nil? # And call it at once.

end