#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
require 'rbt/toplevel_methods/remove_file_extension.rb'

module RBT

module Cookbooks

class Cookbook < RBT::Base

  # ========================================================================= #
  # === guess_short_name
  #
  # This method can be used to guess the short name of a program.
  # ========================================================================= #
  def guess_short_name(
      i = program_name?
    )
    _ = ::ProgramInformation.return_short_name(i)
    set_short_name(_)
  end

  # ========================================================================= #
  # === guess_real_short_name
  # ========================================================================= #
  def guess_real_short_name(
      i = short_name?
    )
    set_real_short_name(i)
  end

  # ========================================================================= #
  # === try_to_guess_the_archive_type
  #
  # This method will try to guess the archive type, e. g. .tar.xz or
  # .tar.gz of a given program at hand, based on which files may
  # exist locally already.
  # ========================================================================= #
  def try_to_guess_the_archive_type
    target = base_dir?+program_name?.downcase+'*'
    files = Dir[target]
    unless files.empty?
      first = files.first
      set_archive(return_archive_type(first))
    end # else we won't set it
  end

  # ========================================================================= #
  # === try_to_guess_the_program_version
  # ========================================================================= #
  def try_to_guess_the_program_version(
      i = program_name?
    )
    _program_name, program_version = ProgramInformation.new(i).return_name_and_version
    set_program_version(program_version)
  end

  # ========================================================================= #
  # === guess_program_version
  #
  # We can use this method to guess the program version. This requires
  # that the entry for program_name exists.
  #
  # We will delegate towards the method set_program_version() then. 
  # ========================================================================= #
  def guess_program_version(i = program_name?)
    _ = ::ProgramInformation.return_version(i)
    set_program_version(_)
  end

  # ========================================================================= #
  # === guess_entries_based_on_the_first_url
  #
  # This method is used to call some entries on the dataset.
  #
  # If the program_name is already set, though, then we will NOT guess
  # it at all. But we will split it up, and set the cascading variants
  # on it.
  # ========================================================================= #
  def guess_entries_based_on_the_first_url
    _ = url1?.dup
    if _.include? '-release.tar'
      _ = chop_off_archive(_)
      _.sub!(/-release/,'')
    end
    if _.include? '-trunk'
      _ = RBT.remove_file_extension(_)
    end
    # ======================================================================= #
    # Next, check if it is a github url and if it may not have a
    # "-" and if it consists only of numbers; in this case we know
    # we have to determine the program name in a different manner.
    # ======================================================================= #
    if is_github_url?(_) and
       !_.include?('-')  and
       (_ =~ /\d{0,2}\.\d{0,2}\.?\d{0,2}/)
       # ===================================================================== #
       # In this case we will re-write this URL; and also modify the
       # variable called _.
       # The current content of _ may be a String such as:
       #
       #   'https://github.com/opencv/opencv/archive/3.4.5.tar.gz'
       #
       # ===================================================================== #
       use_this_regex = /\/(\d{0,2}\.\d{0,2}\.\d{0,2}\.)/ # See: https://rubular.com/r/7AwQrKGdqDoVlm
       # ===================================================================== #
       # We must use the unmodified input-name here because that name may
       # still include '-' characters, which may be important for some
       # programs.
       # ===================================================================== #
       _.sub!(use_this_regex, '/'+original_unmodified_name?.dup+'-\1')
    end
    basename = File.basename(_)
    if basename.include? '%2B' # This means '+'.
      basename.gsub!(/%2B/,'+')
    end
    # ======================================================================= #
    # Some yaml-entries may have program_name set. We have to check for this
    # here.
    # ======================================================================= #
    unless program_name?.to_s.empty?
      basename = program_name?
    end
    # ======================================================================= #
    # At this point, sanitize program_name.
    # ======================================================================= #
    basename = return_sanitized_program_name(basename)
    basename = remove_archive_from_the_end(basename)
    basename.chop! if basename.end_with? '.'
    # ======================================================================= #
    # Obtain information from ProgramInformation.
    # ======================================================================= #
    program_information = ::ProgramInformation.new(basename)
    if program_information.program_version?
      set_program_version(program_information.program_version?)
    end
    if program_information.real_short_name?
      set_real_short_name(program_information.real_short_name?)
    end
    if program_information.short_name?
      set_short_name(program_information.short_name?)
    end
    if File.basename(url?).include? '_'
      unless File.basename(url?).include? '-'
        # ===================================================================== #
        # Specifically change this for programs such as 'artemis_v16.0.11.jar'.
        # ===================================================================== #
        set_use_this_connective_token '_'
      end
    end
    assemble_program_name # This is short_name?+'-'+program_version? usually.
    set_archive_type(_)
    set_base_dir(
      source_dir?+
      real_short_name?.downcase.delete('.')+ # <- Is not allowed to have any '.' tokens.
      '/'
    )
    if !program_name?.empty?
      program_information = ProgramInformation.new(program_name)
      set_short_name(program_information.name?)
      set_program_version(program_information.version?)
    end
    assemble_program_path
    # use_default_connective_token_again <- This should be re-enabled one day.
  end

end; end; end