#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CreateCookbookYamlFile
#
# This class, written from scratch in May 2019, is able to generate a
# new cookbook file. It will make use of a template file and then
# fill in relevant information into that template. The primary reason as
# to why a template file is used is because it allows us to easily
# re-arrange it at any later moment in time. (There was an older class
# which had hardcoded entries in the .rb file directly).
#
# You can pass an URL to this class here and the class will figure out
# the name of the yaml file in question, based on that URL. The URL
# should, in this case, point towards the remote location of a tarball.
#
# Take note that the class allows has an interactive component - see
# "--help" for more information about it. The interactive component
# can be started via "--interactive".
#
# Usage example:
#
#   RBT::Cookbooks::CreateCookbookYamlFile.new
#
# =========================================================================== #
# require 'rbt/cookbooks/create_cookbook_yaml_file/create_cookbook_yaml_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/create_cookbook_yaml_file/constants.rb'
require 'rbt/cookbooks/create_cookbook_yaml_file/misc.rb'
require 'rbt/cookbooks/create_cookbook_yaml_file/menu.rb'
require 'rbt/cookbooks/create_cookbook_yaml_file/reset.rb'
require 'rbt/cookbooks/create_cookbook_yaml_file/run.rb'

module RBT

module Cookbooks

class CreateCookbookYamlFile < ::RBT::Base # === RBT::Cookbooks::CreateCookbookYamlFile

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this class can be the full URL to the target
  # program at hand.
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    @commandline_arguments.first
  end

  # ========================================================================= #
  # === name_of_the_program?
  # ========================================================================= #
  def name_of_the_program?
    File.basename(
      first_argument?
    )
  end; alias program_name? name_of_the_program? # === program_name?
       alias name?         name_of_the_program? # === name?

  # ========================================================================= #
  # === sanitize_first_argument
  # ========================================================================= #
  def sanitize_first_argument
    _ = first_argument?
    if _.include? '-' and (_ =~ /\d+/)
      # ===================================================================== #
      # In this case assume a remote URL:
      # ===================================================================== #
      program_information = ProgramInformation.new(File.basename(_))
      @hash_information_about_the_program[:program_name]    = program_information.program_name?
      @hash_information_about_the_program[:program_version] = program_information.program_version?
      set_url1 _ 
    end
  end

  # ========================================================================= #
  # === set_program_name
  #
  # This sets the short name of the program in question.
  # ========================================================================= #
  def set_program_name(i = '')
    # ======================================================================= #
    # First, do some sanitization.
    # ======================================================================= #
    if i.include? '.' # Assume that a file extension was given. Chop it away thus.
      i = i.to_s.dup
      i.sub!(/#{File.extname(i)}/,'')
    end
    i.delete!('_') if i.include? '_'
    i.downcase!
    @hash_information_about_the_program[:program_name] = i # This variable can be inferred from the url.
  end; alias set_short_name set_program_name # === set_short_name

  # ========================================================================= #
  # === automatically_determine_the_program_name_based_on_the_url1_entry
  # ========================================================================= #
  def automatically_determine_the_program_name_based_on_the_url1_entry(
      i = @hash_information_about_the_program[:url1]
    )
    program_information = ProgramInformation.new(File.basename(i))
    @hash_information_about_the_program[:program_name] = program_information.program_name?
  end

  # ========================================================================= #
  # === set_description
  # ========================================================================= #
  def set_description(
      i = 'Default description of the program.'
    )
    i = i.to_s.dup
    @hash_information_about_the_program[:description] = i
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    _ = ''.dup
    if @hash_information_about_the_program.has_key? :url1
      uri = URI(@hash_information_about_the_program[:url1])
      if uri
        _ = "#{uri.scheme}://#{uri.host}"
      end
    end
    _
  end

  # ========================================================================= #
  # === short_program_name?
  # ========================================================================= #
  def short_program_name?
    _ = program_name?
    if @hash_information_about_the_program.has_key? :program_name
      _ = @hash_information_about_the_program[:program_name]
    end
    _
  end

  # ========================================================================= #
  # === sanitize_dataset                                       (sanitize tag)
  #
  # This method will, in particular, replace some of the generic MACROS
  # in the file with corresponding "sane" values.
  # ========================================================================= #
  def sanitize_dataset
    @dataset.map! {|line|
      case line
      # ===================================================================== #
      # === NAME_OF_THE_PROGRAM
      # ===================================================================== #
      when /NAME_OF_THE_PROGRAM/
        line = program_name?.dup
        if @hash_information_about_the_program.has_key? :url1
          unless @hash_information_about_the_program.has_key? :program_name
            automatically_determine_the_program_name_based_on_the_url1_entry
          end
          line = @hash_information_about_the_program[:program_name].dup
        end
        line << ':' unless line.end_with? ':'
      # ===================================================================== #
      # === homepage:
      # ===================================================================== #
      when / homepage:/
        line = ' homepage: '+homepage?
      # ===================================================================== #
      # === required_deps_on:
      # ===================================================================== #
      when / required_deps_on:/
        line = ' required_deps_on: '+add_hint_for_dependencies
      # ===================================================================== #
      # === last_update
      # ===================================================================== #
      when /last_update: DD MMM YYYY/
        line = ' last_update: '+dd_mmm_yyy
      # ===================================================================== #
      # === description:
      # ===================================================================== #
      when /description: /
        if @hash_information_about_the_program.has_key? :description
          line = ' description: |'+N+
                 '  '+description?
        end
      # ===================================================================== #
      # === url1
      # ===================================================================== #
      when /url1:/
        if @hash_information_about_the_program.has_key? :url1
          line = ' url1: '+@hash_information_about_the_program[:url1]
        end
      end
      line
    }
  end; alias update sanitize_dataset # === update

  # ========================================================================= #
  # === store_dataset_into_file                                  (store tag)
  #
  # This method will store the dataset describing out .yml file into a
  # (local) .yml file finally.
  # ========================================================================= #
  def store_dataset_into_file
    what = @dataset.join(N)
    into = short_program_name?+'.yml'
    if File.exist? into
      if File.directory? into
        opnn; e sdir(into)+swarn(' seems to be a directory. Will not overwrite it.')
      else
        opnn; ewarn 'But there already is a file at '+sfile(into)+swarn('.')
        opnn; ewarn 'Shall we really overwrite this file?'
        case $stdin.gets.chomp
        when 'y','yes','j','ja'
          opnn; ewarn 'Overwriting `'+sfile(into)+swarn('` now.')
          write_what_into(what, into)
        else
          opnn; ewarn 'Ok - not overwriting.'
        end
      end
    else
      opnn; e 'Saving into the file `'+sfile(into)+'` next.'
      write_what_into(what, into)
    end
  end; alias store                                 store_dataset_into_file # === store
       alias save                                  store_dataset_into_file # === save
       alias consider_creating_a_new_cookbook_file store_dataset_into_file # === consider_creating_a_new_cookbook_file

  # ========================================================================= #
  # === verbose_make_use_of_the_template_file_and_then_sanitize_the_dataset
  # ========================================================================= #
  def verbose_make_use_of_the_template_file_and_then_sanitize_the_dataset
    _ = @template_file_to_use
    if File.exist? _
      opnn; e "Making use of a template file (#{sfile(File.basename(_))})."
      @dataset = File.readlines(_).map(&:chomp)
      sanitize_dataset
    end
  end

  # ========================================================================= #
  # === set_url1
  # ========================================================================= #
  def set_url1(
      i = ''
    )
    @hash_information_about_the_program[:url1] = i
    unless i.empty?
      i = return_program_information(
            remove_archive_stuff_at_the_end(
              File.basename(i)
            )
          )
      set_filename(i)
      set_short_name(i)
    end
  end; alias set_url set_url1 # === set_url

  # ========================================================================= #
  # === RBT::Cookbooks::CreateCookbookYamlFile[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

# =========================================================================== #
# === RBT.create_cookbook_yaml_file
# =========================================================================== #
def self.create_cookbook_yaml_file(i = 'php')
  RBT::Cookbooks::CreateCookbookYamlFile.new(i)
end; self.instance_eval { alias create_new_cookbook create_cookbook_yaml_file } # === RBT.create_new_cookbook

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CreateCookbookYamlFile.new(ARGV)
end # ccook php
    # ccook https://download.gimp.org/pub/gegl/0.4/gegl-0.4.16.tar.bz2
    # ccook http://ftp.gnome.org/pub/GNOME/sources/gnome-online-miners/3.9/gnome-online-miners-3.9.90.tar.xz