#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateProgram
#
# This class can create a new program skeleton. By default it will
# create the necessary subdirectories under the /Programs/ hierarchy,
# IF the program at hand does not yet exist there. (Note that this
# is not hardcoded per se; the user can modify /Programs/ to any
# other directory as well).
#
# The class needs three informations for this:
#
# - The name of the program at hand. This will become the name of
#   the respective directory, too.
# - The program version to be used (defaulting to 1.0.0 if omitted).
# - The base directory where the program  
#
# Note that only one entry is mandatory, which is the program name.
# The rest can be guessed or inferred.
#
# The method run_simulation= allows you this program to be run in
# simulation mode, meaning that no changes will be made.
#
# For commandline usage, have a look at the bottom of this .rb file.
#
# Usage example in pure ruby:
#   RBT::CreateProgram.new
# =========================================================================== #
# require 'rbt/create_program/create_program.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/create_program/menu.rb'

begin
  require 'program_information'
rescue LoadError; end

module RBT

class CreateProgram < Base # === RBT::CreateProgram.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_TEST_INPUT
  # ========================================================================= #
  DEFAULT_TEST_INPUT =
    '/home/kumar/Robert/Programs/Htop/1.0.2'

  # ========================================================================= #
  # === DEFAULT_PROGRAM_VERSION
  #
  # This constant specifies which program-version to use if it is
  # otherwise omitted.
  # ========================================================================= #
  DEFAULT_PROGRAM_VERSION = '1.0.0'

  # ========================================================================= #
  # === ARRAY_KEEP_SYMLINKS_FOR_THESE_PROGRAMS
  #
  # we must keep the symlinks for the following programs, as otherwise
  # compilation may fail.
  # ========================================================================= #
  ARRAY_KEEP_SYMLINKS_FOR_THESE_PROGRAMS = %w(
    gcc binutils
  )

  # ========================================================================= #
  # === ARRAY_CREATE_THESE_SUBDIRECTORIES
  # ========================================================================= #
  ARRAY_CREATE_THESE_SUBDIRECTORIES = %w(
    bin
    include
    lib
    man
    sbin
    Shared
    Resources
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = DEFAULT_TEST_INPUT,
      run_already = true
    )
    reset
    set_input(i)
    case run_already
    when :dont_run_yet,
         :do_not_run_yet
      run_already = false
    end
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        if yielded.has_key? :use_colours
          @hash_for_opn.update(use_colours: false) if yielded.delete(:use_colours) == false
        end
        if yielded.has_key? :use_opn
          @hash_for_opn.update(use_opn: false) if yielded.delete(:use_opn) == false
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @hash_for_opn = return_hash_for_opn
    @base_directory_of_the_program = programs_dir?.dup.freeze
    @array_create_these_subdirectories = ARRAY_CREATE_THESE_SUBDIRECTORIES
    set_program_name
    set_program_version
    set_create_current_symlink
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === opnn                                                       (opnn tag)
  # ========================================================================= #
  def opnn
    super(@hash_for_opn)
  end

  # ========================================================================= #
  # === set_input
  #
  # This method will always remove any '_' that it finds, for the
  # time being (November 2018). However had, this is actually not so great
  # since we may also copy/paste this input, such as "LibreOffice_6.2.2.2".
  # Hence, since as of April 2019, we will actually first replace the
  # '_' found if it is only one.
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    case i
    when :default
      i = DEFAULT_TEST_INPUT
    when 'PWD'
      i = File.basename(get_pwd)
    end
    i = i.to_s.dup.downcase
    if i.include? '_'
      if i.count('_') == 1
        i = i.tr('_','-')
      else
        i.delete!('_')
      end 
    end
    @input = i
  end

  # ========================================================================= #
  # === target_directory_and_program_name?
  #
  # This method will return a String that includes the target directory,
  # and the program name at hand. It also MUST return a trailing '/'
  # character.
  # ========================================================================= #
  def target_directory_and_program_name?
    "#{@base_directory_of_the_program}#{program_name?}/"
  end
  
  # ========================================================================= #
  # === set_create_current_symlink
  # ========================================================================= #
  def set_create_current_symlink(i = true)
    @create_current_symlink = i
  end; alias create_current_symlink= set_create_current_symlink # === create_current_symlink=

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end; alias name         program_name? # === name
       alias name?        program_name? # === name
       alias program_name program_name? # === program_name

  # ========================================================================= #
  # === set_program_version
  #
  # This method sets the main program version of our program at hand.
  #
  # 1.0.0 is the default program version to use.
  # ========================================================================= #
  def set_program_version(
      i = DEFAULT_PROGRAM_VERSION
    )
    i = DEFAULT_PROGRAM_VERSION if i.nil?
    i.delete!('/') if i.include? '/'
    # ======================================================================= #
    # Get rid of any possible file extensions next.
    # ======================================================================= #
    i = remove_file_extension(i)
    @program_version = i
  end; alias set_version set_program_version # === set_version

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @program_version
  end; alias version? program_version? # === version?
       alias version  program_version? # === version

  # ========================================================================= #
  # === parse_input_and_assign_to_the_proper_values
  # ========================================================================= #
  def parse_input_and_assign_to_the_proper_values
    _ = input?
    # ======================================================================= #
    # First, check if we have a /Programs entry.
    # ======================================================================= #
    if _.include?('/') and (programs_dir?.include?(_.split('/').first))
      splitted = _.split('/')
      set_program_version(splitted.pop)
      set_program_name(splitted.pop.delete('/').capitalize)
      assume_this_as_the_base_directory = capitalize_first_alphabetical_character(
        splitted.join('/')
      )
      set_base_directory_of_the_program(assume_this_as_the_base_directory)
    # ======================================================================= #
    # Second, check for '/' characters. If these were given then we
    # will assume that the user did input something like:
    #   /home/kumar/Robert/Programs/Htop/1.0.2
    # ======================================================================= #
    elsif _.include?('/') and
      !((_.count('/') == 1) or (_.end_with?('/'))) # <- This check is for input such as "htop-0.9.7/"
      splitted = _.split('/')
      set_program_version(splitted.pop)
      set_program_name(splitted.pop)
      set_base_directory_of_the_program(splitted.join('/'))
    # ======================================================================= #
    # Else the input did not contain any '/' characters, so we have to
    # take another approach.
    # ======================================================================= #
    else
      _ = remove_archive_at_the_end(_)
      # ===================================================================== #
      # Next, delegate towards ProgramInformation.
      # ===================================================================== #
      program_information = ProgramInformation.new(_)
      set_program_version(program_information.program_version?)
      set_program_name(program_information.short_name?)
    end
  end

  # ========================================================================= #
  # === notify_the_user_about_the_target_directory
  # ========================================================================= #
  def notify_the_user_about_the_target_directory
    opnn; e "The target directory will be at `#{sdir(full_target?)}`."
  end

  # ========================================================================= #
  # === create_main_directory_and_the_required_subdirectories
  #
  # This method will create the main directory, and all the required
  # subdirectories - unless these already exist directories.
  #
  # The directory called `Resources/` will list various files such as
  # the file `Dependencies`, which lists all dependencies for the
  # given program at hand, on a GoboLinux system.
  # ========================================================================= #
  def create_main_directory_and_the_required_subdirectories(
      i = full_target?
    )
    unless File.directory? i
      opnn; e "Creating the directory #{sdir(i)} next."
      mkdir(i, verbosity: :be_quiet)
      create_settings_directory
    end
    # ======================================================================= #
    # Next, create the subdirectories.
    # ======================================================================= #
    opnn; e 'Several subdirectories will now be created in that '\
            'directory, namely:'
    opnn; e '  '+sdir(@array_create_these_subdirectories.join('/, ')+'/')
    @array_create_these_subdirectories.each {|this_directory|
      new_target = rds("#{i}#{this_directory}")
      if run_simulation?
        opnn; e 'We run in simulation mode, thus we will not'
        opnn; e "create the directory `#{sdir(new_target)}`."
      else
        unless File.directory? new_target
          mkdir(new_target, { verbosity: :be_quiet, permissions: 0755 } )
        end
      end
    }
    opnn; e 'Done!'
  end

  # ========================================================================= #
  # === full_target?
  #
  # This method assembles back the program base directory, the program
  # name and the program version.
  # ========================================================================= #
  def full_target?
    "#{program_base_dir?}#{program_name?}/#{program_version?}/"
  end; alias target? full_target? # === target?

  # ========================================================================= #
  # === determine_program_name_and_program_version_from_this_directory
  # ========================================================================= #
  def determine_program_name_and_program_version_from_this_directory(
      i = return_pwd
    )
    i = File.basename(i)
    set_input(i)
    # ======================================================================= #
    # This ^^^ alone suffices, because the rest of the program will handle
    # the logic properly.
    # ======================================================================= #
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    opnn; e "This class will create programs under the "\
            "#{sdir(programs_dir?)} hierarchy."
    e
    opnn; e 'You can test this class by doing:'
    e
    opnn; efancy "  rcp #{programs_dir?}Htop/1.0.1"
    opnn; efancy '  rcp htop-1.0.1'
    e
  end

  # ========================================================================= #
  # === return_hash_for_opn
  # ========================================================================= #
  def return_hash_for_opn
   {
     namespace:   NAMESPACE,
     use_colours: use_colours?
   }
  end

  # ========================================================================= #
  # === create_settings_directory
  # ========================================================================= #
  def create_settings_directory
    this_directory = target_directory_and_program_name?+'Settings'
    if in_simulation?
      opnn; e "Normally `#{sdir(this_directory)}` would be created but"
      opnn; e 'as we run in simulation-mode, no change will be made.'
    else
      create_directory(this_directory, :be_quiet) # Also create a Settings/ subdir:
    end
  end

  # ========================================================================= #
  # === set_program_name
  # ========================================================================= #
  def set_program_name(i = '')
    i = i.dup
    if i
      i.downcase! # <- This is done always. On GoboLinux, we may have to change this lateron.
      if CAPITALIZE_PROGRAM_NAME # Now it is Capitalized. Defined in rbt/configuration/configuration.rb
        i.capitalize! unless i.empty?
      end
      if is_on_gobolinux?
        i = return_program_name_for_gobolinux_systems(i)
      end
      i.delete!('-') if i.include? '-'
    end
    @program_name = i
  end

  # ========================================================================= #
  # === create_current_symlink
  #
  # Invoke this part only if you really want to create the symlink called
  # `Current`
  # ========================================================================= #
  def create_current_symlink
    # ======================================================================= #
    # The following instance variable can be used to disable the creation
    # of the Current symlink.
    # ======================================================================= #
    create_current_symlink = @create_current_symlink
    if create_current_symlink
      _ = target_directory_and_program_name?+'Current' # _ will be something like "/Programs/Samba/Current"
      # ===================================================================== #
      # Intercept some programs here.
      # ===================================================================== #
      return if ARRAY_KEEP_SYMLINKS_FOR_THESE_PROGRAMS.include?(@program_name.downcase)
      if File.symlink? _ # remove symlink if it exists.
        if @debug # Give info only when in debug mode.
          opnn; ewarn '=> Removing symlink: '+sfancy(_)+swarn(' next.')
        end
        remove_file(_)
      end
      # ===================================================================== #
      # === Let the method symlink() handle the symlinking action
      # ===================================================================== #
      symlink( # Delegate towards rbt/symlink.rb
        full_target?, _, :default, use_colours?
      )
    end
  end

  # ========================================================================= #
  # === return_program_name_and_program_version
  # ========================================================================= #
  def return_program_name_and_program_version
    "#{@program_name}-#{@program_version}"
  end

  # ========================================================================= #
  # === tell_the_user_what_will_be_done
  #
  # Feedback information back to the user. In Jun 2007 I lessened the 
  # amount it feedbacks. Currently this method is invoked via 
  # create_directories
  # ========================================================================= #
  def tell_the_user_what_will_be_done
    opnn; ee sfancy(return_program_name_and_program_version.downcase)
    if be_verbose?
      ee ' ('+simp('Program name')+': '+sfancy(program_name?)+' and '+
         simp('Program Version')+': '+sfancy(program_version?)+')'
    end; e
  end

  # ========================================================================= #
  # === program_base_dir?
  # ========================================================================= #
  def program_base_dir?
    @base_directory_of_the_program
  end

  # ========================================================================= #
  # === set_base_directory_of_the_program
  #
  # Keep in mind that this method will ensure that there is a trailing
  # '/' token to the instance variable @base_directory_of_the_program.
  # ========================================================================= #
  def set_base_directory_of_the_program(
      i = programs_dir?
    )
    i = i.first if i.is_a? Array
    i = programs_dir? if i.nil?
    i << '/' unless i.end_with? '/'
    i = rds(i)
    @base_directory_of_the_program = i.dup
  end; alias set_specific_program_directory set_base_directory_of_the_program # === set_specific_program_directory
       alias set_base_directory             set_base_directory_of_the_program # === set_base_directory
       alias set_base_directory=            set_base_directory_of_the_program # === set_base_directory=

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    parse_input_and_assign_to_the_proper_values
    tell_the_user_what_will_be_done # Give information to the user.
    notify_the_user_about_the_target_directory
    create_main_directory_and_the_required_subdirectories
    # ======================================================================= #
    # We will create a symlink called 'Current' by default, but this
    # can be disabled.
    # ======================================================================= #
    create_current_symlink
  end

  # ========================================================================= #
  # === RBT::CreateProgram[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.create_program
# =========================================================================== #
def self.create_program(
    use_this_path = :default
  )
  RBT::CreateProgram.new(use_this_path)
end

end

if __FILE__ == $PROGRAM_NAME
  _ = ARGV
  # ========================================================================= #
  # Join the arguments if more than one is given. However had, we will treat
  # at the least the option --tree differently. Since it is superfluous, we
  # will chop it away.
  # ========================================================================= #
  if _.size > 1
    _[0] = nil if _[0] == '--tree'
    _.compact!
    _ = _.join('-') # The '-' indicates a program-version string.
  end
  create_program = RBT::CreateProgram.new(_, :do_not_run_yet)
  # create_program.disable_colours # If you wish to disable the colours, you can use this line here.
  # create_program.use_colours?
  create_program.run 
end # create_program 
    # rcp wesnoth-1.0.2
    # rcp glib-2.35.3.tar.xz
    # rcp /Programs/Htop/1.0.153636
    # RCP GTK+ 666
    # rcp htop 0.9.7
    # rcp /home/kumar/Robert/Programs/Htop/1.0.2
    # rcp /Programs/Ffmpeg/3.4.1/
    # rcp --tree Foo 1.0