#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::Gtk::RecipeMaker

# Purpose of this file:
#
# It shall provide a GTK2 Interface to create Cookbook-Recipes.
#
# After you hit the Update Button, the information which is stored in the
# entries on the right is applied to the field to the left. However, once
# this is done (once the area is filled), we no longer will automagically
# update the content, so you can overrule it if you typed something in
# the field. 
# =========================================================================== #
# Reasoning for this class:
#
# I wanted to load a yaml file, then add the new entries to it, then save 
# it back to a file, but this approach would omit my nice, helpful comments.
# Thus I decided to just append some lines of a recipe info to that file.
# This approach seems at least easier.
# =========================================================================== #
# $RUBY_SRC/rbt/lib/rbt/gui/gtk/recipe_maker.rb
# =========================================================================== #
require 'wget'
require 'cliner/module'
require 'rbt'
require 'gtk_module/base/base_module.rb'

module RBT

module GUI

module Gtk

class RecipeMaker < ::Gtk::VBox # RBT::GUI::Gtk::RecipeMaker.new

  include ::Gtk::BaseModule
  include ::Gtk::Colours
  include Cliner

  # ========================================================================= #
  # === GUESS_NAME
  # ========================================================================= #
  GUESS_NAME = true              # if true, we attempt to guess the name

  # ========================================================================= #
  # === DEBUG
  # ========================================================================= #
  DEBUG = false                  # set this to false to recieve no debug info

  # ========================================================================= #
  # === VERSION_RECIPE_MAKER
  # ========================================================================= #
  VERSION_RECIPE_MAKER = '1.0.5' # which version do we have?

  # ========================================================================= #
  # === USE_OVERRIDE
  # ========================================================================= #
  USE_OVERRIDE = false           # if this is true, we set the override var

  # ========================================================================= #
  # === AUTHOR_NAME
  # ========================================================================= #
  AUTHOR_NAME = RBT.author_name? # your name

  # ========================================================================= #
  # === DEFAULT_FONT_SIZE
  # ========================================================================= #
  DEFAULT_FONT_SIZE = '15'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH  = 820

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = 660

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE  = 'Recipe Maker'

  # ========================================================================= #
  # === ARRAY_WITH_PROGRAM_DIRS
  # ========================================================================= #
  ARRAY_WITH_PROGRAM_DIRS = [
    '','RB', 'PHP', 'PYTHON','AUD','VID','EDITORS',
    'GCC','GNOME','IMG','GAMES','KDE'
  ]

  # ========================================================================= #
  # Must rescue here because GTK Colours may not yet work.
  # ========================================================================= #
  begin
    X1 = CORNFLOWERBLUE # colour shortcut  
    X2 = LIGHTBLUE      # colour shortcut
  rescue
    ewarn ' => GTK Colour probably not yet defined, continuing '+
          'anyway.'
  end

  # ========================================================================= #
  # === initialize
  #
  # Initialize a new instance.
  #
  # First set our flags to false.
  # ========================================================================= #
  def initialize(
      optional_program_name = nil
    )
    reset
    super()
    @program_name = optional_program_name
    set_font_size
    # have passed a special argv to our program. Right now we 
    # have four different modes:
    #   :set_std_base_dir, :set_a_base_dir, 
    #   :tar_package,      :remote_package
  end

  # ========================================================================= #
  # === use_gtk?
  # ========================================================================= #
  def use_gtk?
    @use_gtk
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @create_new_cookbook = ::Cookbooks::CreateNewCookbook.new(nil, :do_not_run_yet)
    @use_gtk = true # Unsure why this flag exists.
    @flag_argv_mode = nil # if this is other than nil, then we
    @flag_will_extract_to_programs_dir = false # default: false
    @array_buttons = []
    @target_base_dir = '' # this var keeps track of the base dir
    # The @yaml_file_location should point at your 
    # yaml file.
    @yaml_file_location = '' #YAML_COOKBOOK_FILE # '/Depot/source_files.yaml' for 
    # non master systems, LINUX/YAML for master systems.
    @buffer_new_program = '' # holds the text for the buffer to the left.
    # The structure of the array is as follows:
    # [0] contains the name 
    # [1] contains the - name of program  (Example:  - archive_name: toppler-1.1.2a) 
    # [2] contains the - description (Example:- description: "A simple game.")
    @hash_yaml_file = {} # will hold data of our yaml file
  end

  # ========================================================================= #
  # === buffer_new_program?
  # ========================================================================= #
  def buffer_new_program?
    @buffer_new_program
  end; alias buffer_new_program buffer_new_program? # === buffer_new_program

  # ========================================================================= #
  # === set_font_size
  # ========================================================================= #
  def set_font_size(
      new_size = DEFAULT_FONT_SIZE
    )
    ::Gtk::Settings.default.gtk_font_name = 'Sans '+new_size
  end

  # ========================================================================= #
  # === toggle_gtk
  #
  # Toggles the gtk mode.
  # ========================================================================= #
  def toggle_gtk
    @use_gtk = @use_gtk ? false : true
  end

  # ========================================================================= #
  # === run_it_all                                             (run tag)
  #
  # Chains our methods together, then runs them. We dont run GTK specific
  # stuff if we did set the var @do_not etc.. If you do NOT want to use
  # any GTK application, for example if you run this in batch mode, then
  # use check_whether_we_shall_install_a_package instead. 
  # ========================================================================= #
  def run_it_all(
      input_optional_package = @program_name
    )
    if @use_gtk
      init_main_window 
      create_gtk_entries # before startup...
      startup_gtk2_tasks(input_optional_package)
      create_gtk_top_menu
      create_gtk_statusbar
      create_gtk_buffer
      create_gtk_buttons
      create_gtk_accesskeys
      create_gtk_tables
      check_whether_we_shall_install_a_package(input_optional_package)
      connect_skeleton
    else
      check_whether_we_shall_install_a_package(input_optional_package)
    end
  end

  # ========================================================================= #
  # === check_whether_we_shall_install_a_package
  #
  # Checks whether we shall install a package.
  #
  # Only applicable when Dir.pwd is PKG_DIR directory, ie in
  # "/Depot/pkg"
  # ========================================================================= #
  def check_whether_we_shall_install_a_package(
      input = '', shall_we_exit = true
    )
    # We are in the correct dir, thus we dont 
    # have to cd to any other dir. We can now proceed to 
    # install our package by invoking install_package. 
    # Normally, install_package will ALWAYS exit the whole script
    install_package(
      input, shall_we_exit, gather_package_information(input)
    ) if Dir.pwd == ENV['PKG_DIR'].to_s
  end

  # ========================================================================= #
  # === init_main_window
  # ========================================================================= #
  def init_main_window
    begin
      self.show
      self.signal_connect(:destroy) { Gtk.main_quit }
    rescue LoadError => error # rescue to not break
      e 'LoadError again in init_main_window '+sfancy(error.to_s)
    end
  end

  # ========================================================================= #
  # === guess_program_name
  #
  # This will guess the name of a program if "update entry" is clicked. It 
  # uses the @entry_url.text string, which is slightly problematic if this 
  # entry is missing.
  # ========================================================================= #
  def guess_program_name
    # if GUESS_NAME
      # Ok, user entered some input, so we must take his input.
      # if @entry_shortcut_name.text.size > 0
      #   guessed_name = @entry_shortcut_name.text
      # else
      #   guessed_name = @entry_url.text.basename.dup
      # end
    #  guessed_name = @entry_url.text.basename.dup
    #  guessed_name.gsub!(/.tar.gz$/, '') # Remove trailing .tar.gz and .tar.bz2
    #  guessed_name.gsub!(/.tar.bz2$/, '')
    #  # tmp_name, tmp_version = ProgramInformation.return_name_and_version(guessed_name)
    #  # tmp_version = '1.0.0' if tmp_version.nil? # error handling for nil
    #  @entry_program_name.text = tmp_name.dup 
    #  @entry_shortcut_name.text = guessed_name # simplification  
    #  e tmp_name, CF; e tmp_version, CF
    #end
  end

  # ========================================================================= #
  # === startup_gtk2_tasks
  #
  # To start the gtk stuff, and also fills up 
  # our buffer and makes our yaml file
  # ========================================================================= #
  def startup_gtk2_tasks(input = '')
    @v_box = gtk_vbox
    fill_up_buffer(input)
  end

  # ========================================================================= #
  # === create_gtk_accesskeys
  #
  # This will be responsible for making the accesskeys.
  # Use <Alt>+1
  # ========================================================================= #
  def create_gtk_accesskeys
    @accel_group_for_entry_program_name    = gtk_accel_group
    @accel_group_for_entry_url             = gtk_accel_group
    @accel_group_for_entry_shortcut_name   = gtk_accel_group
    @accel_group_for_entry_description     = gtk_accel_group
    @accel_group_for_entry_dependencies_on = gtk_accel_group
    make_shortcut_key_for_gtk_entry @accel_group_for_entry_shortcut_name,'1', @entry_shortcut_name
    make_shortcut_key_for_gtk_entry @accel_group_for_entry_program_name,'2', @entry_program_name
    make_shortcut_key_for_gtk_entry @accel_group_for_entry_url,'3', @entry_url
    make_shortcut_key_for_gtk_entry @accel_group_for_entry_description,'4', @entry_description
    make_shortcut_key_for_gtk_entry @accel_group_for_entry_dependencies_on,'5', @entry_dependencies_on
  end

  # ========================================================================= #
  # === create_gtk_about_dialog
  #
  # This creates our about dialog.
  # ========================================================================= #
  def create_gtk_about_dialog
    @a = ::Gtk::AboutDialog.new
    @a.artists = [ AUTHOR_NAME ]
    @a.authors = [ AUTHOR_NAME ]
    @a.comments = 'This Script creates recipe information ' +
    'which will be stored in sources.yaml'
    @a.copyright = 'Copyright (C) 2006 Robert A. Kunz'
    @a.documenters = ['Robert A. Kunz ']
    @a.license = 'This program is licenced under the same license '+
    'as Ruby. There is one additional license part - which simply '+
    'encourages you to send bug reports to '+
    'me so that I can improve the program. ;-)'
    @a.name = 'Ruby::Gtk::ReciperMaker'
    @a.translator_credits = "Robert A. Kunz\n"
    @a.version = VERSION_RECIPE_MAKER
    @a.website = 'http://www.aptenix.org'
    @a.website_label = 'HiveOS Homepage'
    @a.logo = Gdk::Pixbuf.new( ENV['IMG']+'/STD/UHR.png' )
  end

  # ========================================================================= #
  # === make_shortcut_key_for_gtk_entry
  #
  # makes a shortcut key.
  # ========================================================================= #
  def make_shortcut_key_for_gtk_entry(accel_group, second = 'b', entry = '')
    # connect(
    #   accel_key, accel_mods, accel_flags) {...}
    accel_group.connect(
      Gdk::Keyval.from_name(second.to_s),  
      Gdk::Window::MOD1_MASK, ::Gtk::ACCEL_VISIBLE) {
        entry.set_focus(true)
        entry.select_region(0, -1)
    }
  end

  # ========================================================================= #
  # === create_gtk_buffer
  #
  # makes the textedit stuff, also invokes update_buffer
  # ========================================================================= #
  def create_gtk_buffer
    @text_buffer = ::Gtk::TextBuffer.new
    @text_buffer = GTK2_TextBufferTags.create_my_tags(@text_buffer)
    update_buffer
    @text_view_left = ::Gtk::TextView.new(@text_buffer)
    @text_view_left.set_editable(false)
    @scrolled_window = ::Gtk::ScrolledWindow.new
    @scrolled_window.add_with_viewport(@text_view_left)
    @scrolled_window.set_policy(::Gtk::POLICY_NEVER, ::Gtk::POLICY_ALWAYS)
    @v_box.add(@scrolled_window)
  end

  # ========================================================================= #
  # === fill_up_array_that_holds_our_buffer
  # ========================================================================= #
  def fill_up_array_that_holds_our_buffer(this_name = '')
    if this_name.empty?
      @create_new_cookbook.set_real_name ''
    else
      @create_new_cookbook.set_real_name this_name
    end
    @create_new_cookbook.set_base_dir :default
    @create_new_cookbook.set_description
    @create_new_cookbook.set_url
    @create_new_cookbook.set_compile_mode 't'
    @create_new_cookbook.set_required_deps_on
    @create_new_cookbook.set_override 't' if USE_OVERRIDE.true?
  end

  # ========================================================================= #
  # === assign_buffer_to_array
  #
  # This method builds our buffer by joining our array.
  # ========================================================================= #
  def assign_buffer_to_array
    @buffer_new_program = @create_new_cookbook.array_buffer.join(N)
  end

  # ========================================================================= #
  # === update_buffer
  #
  # Sets our main buffer by using @buffer_new_program
  # ========================================================================= #
  def update_buffer
    assign_buffer_to_array
    @text_buffer.set_text('')
    iter = @text_buffer.start_iter
    e sfancy(@buffer_new_program)
    @buffer_new_program.split(N).each { |line|
      case line
      when /name:$/
        @text_buffer.insert(iter, line+N,'bold','darkred')
      when /program_name/
        @text_buffer.insert(iter, line+N,'bold','darkblue')
      when /description/
        @text_buffer.insert(iter, line+N,'bold','darkgreen')
      when /required_debs/
        @text_buffer.insert(iter, line+N,'bold','slateblue')
      when /true/, / t$/
        line.gsub!(/ true/,'')
        line.gsub!(/ t$/,'') if line =~ / t$/
        @text_buffer.insert(iter, line)
        @text_buffer.insert(iter, ' t'+N,'bold','red')
      when /false/, / f$/
        line.gsub!(/ false/,'')
        line.gsub!(/ f$/,'') if line =~ / f$/
        @text_buffer.insert(iter, line)
        @text_buffer.insert(iter, ' f'+N,'bold','red')
      else
        @text_buffer.insert(iter, line+N)
      end
    }
  end
  
  # ========================================================================= #
  # === fill_up_buffer
  #
  # Fills up our buffer.
  # ========================================================================= #
  def fill_up_buffer(input = '')
    fill_up_array_that_holds_our_buffer(@entry_shortcut_name.text)
    modify_base_dir(input)
    check_for_flags
    assign_buffer_to_array
  end

  # ========================================================================= #
  # === check_for_flags
  #
  # Checks if we have a special flag:
  # ========================================================================= #
  def check_for_flags(i = ARGV.first)
    case @flag_argv_mode
    when nil # pass through
    when :set_std_base_dir
    when :set_a_base_dir
    when :tar_package
      @entry_shortcut_name.set_text(i)
    when :remote_package
      @entry_url.set_text(i)
    end
  end

  # ========================================================================= #
  # === modify_base_dir
  #
  # Used to modify the base dir, for example to assign a new value to 
  # @target_base_dir. Since JAN 2007 it also sets a flag. If you pass an 
  # argument, we will use that uppercased value, as its new value. Or, if we 
  # can find a match in the below menu, we use a special strings 
  # (like the current dir) instead. 
  # ========================================================================= #
  def modify_base_dir(input = '')
    if ARGV.first
      copy = ARGV.first.dup
      # If you pass an argv like pwd, we will use the 
      # current base as dir target.
      case copy
      when 'pwd','pw','p'
        copy = Dir.pwd.basename
        @flag_argv_mode = :set_std_base_dir
      else
        # we must get rid of / if its the first char,
        # because our array below will get / as its
        # first appended char already.
        if copy.start_with? '/'
          @flag_argv_mode = :set_a_base_dir
          copy[0,1] = ''
        end
        if copy.include? 'tar' # we assume a package
          @flag_argv_mode = :tar_package
          if copy.include? 'http'  # we assume a remote url
            @flag_argv_mode = :remote_package # default to tar
          end
        end
        if copy.include? 'tgz'
          @flag_argv_mode = :tgz_package
          # change maybe:
          @flag_argv_mode = :remote_package if copy.include? 'http' # we assume a remote url
        end
      end
      case @flag_argv_mode
      when :set_std_base_dir, :set_a_base_dir
        @target_base_dir = copy
        # also, manually fix our entry:
        @entry_base_dir.set_text(copy)
      when :remote_package
      when :tar_package
      else # 
      end
    end
    # Now add to base_dir
    case @flag_argv_mode
    when :remote_package # pass through
    when :tar_package    # pass through
    else
      @create_new_cookbook.set_base_dir @target_base_dir
    end
  end
  
  # ========================================================================= #
  # === download_program
  #
  # This will download a program by using our Download module.
  # ========================================================================= #
  def download_program(url = @entry_url.text)
    e "Downloading #{url} next:"
    e 'In '+Dir.pwd+':' 
    cliner {
      Wget.download(url)
    }
  end

  # ========================================================================= #
  # === create_gtk_top_menu
  #
  # Handles the gtk top menu.
  # ========================================================================= #
  def create_gtk_top_menu
    @menu_ui_design = %Q[
      <ui>
        <menubar name='MenuBar'>
        <menu action='FileMenu'>
          <menuitem action='New' />
          <separator />
          <menuitem action='Open' />
          <separator />
          <menuitem action='Quit' />
          <separator />
          <menuitem action='About' />
          <separator />
        </menu>
        </menubar>
      </ui>
    ]
    # ======================================================================= #
    # The callbacks to use.
    # ======================================================================= # 
    @callback = Proc.new { |actiongroup, action| e "`#{action.name}' was clicked. " }
    @callback_quit = Proc.new {
      e 'Quit was called. Thus, exitting now.'; Gtk.main_quit
    }
    @callback_open = Proc.new { open_a_file }
    @callback_about_dialog = Proc.new { @a.run }    
    # ======================================================================= #
    # Our allowed actions
    # ======================================================================= #
    @menu_actions = [
      ['FileMenu', nil, '_File'],
      ['New',   ::Gtk::Stock::NEW,   '_New',  '<control>N', 'Create a new file', @callback],
      ['Quit',  ::Gtk::Stock::QUIT,  '_Quit', '<control>Q', 'Quit', @callback_quit],
      ['Open',  ::Gtk::Stock::OPEN,  '_Open', '<control>O', 'Open a new file', @callback_open],
      ['About', ::Gtk::Stock::ABOUT, '_About','<control>A', 'About this Application', @callback_about_dialog]
    ]
    @action_group = ::Gtk::ActionGroup.new('Actions')
    @action_group.add_actions(@menu_actions)
    @uimanager = ::Gtk::UIManager.new
    @uimanager.insert_action_group( @action_group, 0)
    @uimanager.add_ui( @menu_ui_design )
    @v_box.pack_start(@uimanager['/MenuBar'], false, false)
  end

  # ========================================================================= #
  # === create_gtk_buttons
  #
  # makes our buttons (butt tag but tag button tag)
  # ========================================================================= #
  def create_gtk_buttons
    # UPDATE BUTTON update tag upd tag
    @button_update = ::Gtk::Button.new('_Update Entry')
    # wenn der update button gedrckt wird,
    # wird der buffer geupdated
    @button_update.signal_connect(:clicked) { 
      sanitize_data
      guess_program_name if GUESS_NAME
      @create_new_cookbook.set_real_name        @entry_shortcut_name.text
      @create_new_cookbook.set_program_name     @entry_program_name.text 
      @create_new_cookbook.set_description      @entry_description.text.gsub("\\n", "\n")
      @create_new_cookbook.set_url              @entry_url.text # zum abfragen des inhalts
      @create_new_cookbook.set_required_deps_on @entry_dependencies_on.text
      update_buffer
    }
    # QUIT BUTTON
    @button_quit = ::Gtk::Button.new('_Quit')
    @button_quit.signal_connect(:clicked) { ::Gtk.main_quit }
    # TRADITIONAL BUTTON
    @button_traditional = ::Gtk::CheckButton.new('_Traditional')
    @button_traditional.signal_connect(:toggled) { |widget|
      if widget.active?
        @create_new_cookbook.set_compile_mode 't'
      else
        @create_new_cookbook.set_compile_mode 'f'
      end
      update_buffer
    }
    # USE_OVERRIDE BUTTON
    @button_override = ::Gtk::CheckButton.new('_Override' )
    @button_override.set_active(USE_OVERRIDE) # startup state
    @button_override.signal_connect(:toggled) { |widget|
      if widget.active?
        @create_new_cookbook.set_override 't'
      else
        @create_new_cookbook.set_override 'f'
      end
      update_buffer
    }
    # FORMER DEBUG BUTTON. debug tag Not always needed...
    # Now it became a Download button
    # e @hash_yaml_file.size
    @button_download = ::Gtk::Button.new('_Download' )
    @button_download.signal_connect(:clicked) {
      download_program @entry_url.text
    }
    # ======================================================================= #
    # MODIFY YAML BUTTON. THIS APPENDS TO YOUR SOURCE FILES YAML.
    # ======================================================================= #
    @button_modify_yaml = ::Gtk::Button.new('_Modify Yaml' )
    @button_modify_yaml.signal_connect(:clicked) { save_buffer_to_yaml }

    @array_buttons << @button_quit
    @array_buttons << @button_traditional
    @array_buttons << @button_override
    @array_buttons << @button_download
    @array_buttons << @button_modify_yaml
    @array_buttons << @button_update
    modify_bg_of_buttons
  end

  # ========================================================================= #
  # === sanitize_data
  #
  # This method will gsub @entry_shortcut_name and do various other
  # sanitize related tasks.
  # ========================================================================= #
  def sanitize_data
    @entry_shortcut_name.set_text( # remove .tar.gz and .tar.bz2
      @entry_shortcut_name.text.gsub(/\.tar\.gz/,'').gsub(/\.tar\.bz2/,'')
    )
    @entry_program_name.set_text(@entry_program_name.text.downcase)
    _ = @entry_dependencies_on.text
    @entry_dependencies_on.text = _.gsub(/ /,',') if _.include? ' '
  end

  # ========================================================================= #
  # === create_gtk_entries
  #
  # Creates all the important entries. Must be called before
  # create_gtk_tables is called
  # ========================================================================= #
  def create_gtk_entries
    @entry_program_name = ::Gtk::Entry.new
    @entry_program_name.set_max_length( 80 )
    @entry_program_name.set_text('name')
    
    @entry_url = ::Gtk::Entry.new
    @entry_url.set_max_length(120)
    @entry_url.set_text('')

    # Why such a high max length? Because i want to be able to copy-paste stuff into it.
    @entry_shortcut_name = ::Gtk::Entry.new
    @entry_shortcut_name.set_max_length(100)
    @entry_shortcut_name.set_text('')
    @entry_shortcut_name.signal_connect(:event) { |widget, event|
      case event.event_type.name
      when 'GDK_LEAVE_NOTIFY'
        update_buffer
        true
      end
    }
    @entry_description = ::Gtk::Entry.new
    @entry_description.set_max_length(800)
    @entry_description.set_text('')
    @entry_dependencies_on = ::Gtk::Entry.new
    @entry_dependencies_on.set_max_length(100)
  end
  
  # ========================================================================= #
  # === create_gtk_tables
  #
  # also connects all parts of the table. table tag.
  # make table. mk table. tab tag.
  # ========================================================================= #
  def create_gtk_tables
    @main_table = ::Gtk::Table.new(10, 2, true)
    # Die vier main buttons
    @main_table.attach_defaults @button_update,0,1, 0, 1
    @main_table.attach_defaults @button_modify_yaml,1,2, 0, 1
    @main_table.attach_defaults @button_quit,0,1, 1, 2
    @main_table.attach_defaults @button_download,1,2, 1, 2
    @main_table.attach_defaults @button_traditional,0,1,2,3
    @main_table.attach_defaults @button_override,1,2,2,3
    @main_table.attach_defaults modify_label('Shortcut Name: ','darkblue'),0,1, 3, 4
    @main_table.attach_defaults @entry_shortcut_name,1,2, 3, 4
    @main_table.attach_defaults modify_bold_label('Program Name: ','cadetblue'),0,1, 4, 5
    @main_table.attach_defaults @entry_program_name,1,2, 4, 5
    @main_table.attach_defaults modify_label('URL: ','darkblue'),0,1, 5, 6
    @main_table.attach_defaults @entry_url,1,2, 5, 6
    # Die scroll-through box. Dropdown.
    @combo_box_entry = ::Gtk::ComboBoxEntry.new(true)
    ARRAY_WITH_PROGRAM_DIRS.each { |val| @combo_box_entry.append_text(val) }
    @combo_box_entry.active = 0
    @combo_box_entry.set_size_request(50, 25)
    @combo_box_entry.wrap_width = 1
    @combo_box_entry.signal_connect(:changed) {
      # on changed event, also change whats inside the base dir 
      if @combo_box_entry.active_iter
        _ = @combo_box_entry.active_text
        case _
        when 'RB' # if a ruby file, we add sth hmmm:
          # @array_that_holds_our_buffer <<
          # " - program_make_options: RINSTALL1\n"
        end
        # finally, update buffer to apply changes.
        update_buffer
      end
    }
    # This contains the combo box and the entry base dir.
    mini_hbox = ::Gtk::HBox.new
    mini_hbox.pack_start(@combo_box_entry)    
    @main_table.attach_defaults(
      modify_bold_label('BASE DIR: ','darkorchid'),0,1, 6, 7)
    @main_table.attach( mini_hbox, 1, 2, 6, 7, 
      ::Gtk::AttachOptions::SHRINK,
      ::Gtk::AttachOptions::SHRINK # xopt, yopt
    )
    @main_table.attach_defaults(modify_bold_label('Description: ','darkgreen'),0,1, 7, 8)
    @main_table.attach_defaults(@entry_description,1,2, 7, 8)
    
    @main_table.attach_defaults(modify_bold_label('Dependencies: ','slateblue'),0,1, 8, 9)
    @main_table.attach_defaults(@entry_dependencies_on,1,2, 8, 9)
    # @main_table.attach_defaults(@v_box_entries.show_all,0,2, 9, 10)
    make_gtk_horizontal_stuff
  end
  
  # ========================================================================= #
  # === make_gtk_horizontal_stuff
  #
  # Makes the horizontal box. Inlaid in it are the @v_box and the 
  # @main_table.
  # ========================================================================= #
  def make_gtk_horizontal_stuff
    @h_box = ::Gtk::HBox.new(true, 4)
    @h_box.add(@v_box)
    @h_box.add(@main_table)
  end

  # ========================================================================= #
  # === create_gtk_statusbar
  #
  # Makes the statusbar
  # ========================================================================= #    
  def create_gtk_statusbar
    @status_bar = ::Gtk::Statusbar.new
    @status_bar.has_resize_grip = false
    @status_bar_context_id = @status_bar.get_context_id('RecipeMaker')
    @status_bar.push(@status_bar_context_id, 'RecipeMaker')
  end

  # ========================================================================= #
  # === connect_skeleton
  #  
  # runs the main part
  # ========================================================================= #
  def connect_skeleton
    create_gtk_about_dialog
    self.add(@h_box)
    self.show_all
  end

  # ========================================================================= #
  # === save_buffer_to_yaml
  #
  # This does the actual saving part. It will modify cookbook.yml
  # ========================================================================= #
  def save_buffer_to_yaml
    @create_new_cookbook.save
  end
  
  # ========================================================================= #
  # === modify_bg_of_buttons
  #
  # Modify the background of our buttons.
  # ========================================================================= #
  def modify_bg_of_buttons
    @array_buttons.each { |b|
      b.modify_bg( ::Gtk::StateType::ACTIVE,   Gdk::Color.parse('coral') )
      b.modify_bg( ::Gtk::StateType::NORMAL,   Gdk::Color.parse('grey')  )
      b.modify_bg( ::Gtk::StateType::PRELIGHT, Gdk::Color.parse('lightblue')  )
    }
  end

  # ========================================================================= #
  # === open_a_file
  # ========================================================================= #
  def open_a_file
    e 'Trying to open a file.'
    filebutton = ::Gtk::FileChooserButton.new('Gtk::FileChooserButton', 
      ::Gtk::FileChooser::ACTION_OPEN)
    filebutton.signal_connect(:current_folder_changed) { |w, e|
      _ = filebutton.filename
      if filebutton.filename
        e 'The filename is: '+_
        _
      end
    }
    self.add filebutton
  end

  # ========================================================================= #
  # === Gtk::RecipeMaker.height?
  # ========================================================================= #
  def self.height?
    HEIGHT
  end

  # ========================================================================= #
  # === Gtk::RecipeMaker.width?
  # ========================================================================= #
  def self.width?
    WIDTH
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  recipe_maker = RBT::GUI::Gtk::RecipeMaker.new(ARGV.first)
  recipe_maker.run_it_all
  r = GtkRunner.new(
    RBT::GUI::Gtk::RecipeMaker.width?,
    RBT::GUI::Gtk::RecipeMaker.height?,
    '/Users/x/DATA/IMG/PC/TABBLE/TABBLE.png',
    RBT::GUI::Gtk::RecipeMaker::TITLE, 10
  )
  r.add recipe_maker
  r.resizable = true # die line wegen einem mglichen .maximize
  # r.maximize
  r.top_left
  r.show_all
  r.modify_bg(Gtk::STATE_NORMAL, Gtk::Colours::GREY)
  r.run
end # rrecipe