#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Infobox
#
# This class will display information about compiling a program in a neat,
# packed ASCII box. 
#
# Usage example:
#   RBT::Infobox.new
# =========================================================================== #
# require 'rbt/infobox/infobox.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/infobox/constants.rb'
require 'rbt/infobox/display.rb'
require 'rbt/infobox/misc.rb'
require 'rbt/infobox/run.rb'

module RBT

class Infobox < RBT::Base # === RBT::Infobox.new

  begin
    require 'roebe/toplevel_methods/unicode/popular_unicode_symbols.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === COLOURS
  # ========================================================================= #
  COLOURS = Colours::AllColourMethods

  # ========================================================================= #
  # === initialize
  #
  # Ideally you should pass the cookbook-dataset to this method as the
  # first parameter.
  # ========================================================================= #
  def initialize(
      i           = nil, # Use the cookbook-dataset here.
      run_already = true
    )
    reset
    set_compile_this_program(i)
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        if yielded.has_key? :use_ccache
          @use_ccache = yielded[:use_ccache]
        end
        if yielded.has_key? :show_required_dependencies_on
          @show_required_dependencies = yielded[:show_required_dependencies_on]
        end
        if yielded.has_key? :use_this_dataset
          set_use_this_as_dataset(
            yielded[:use_this_dataset]
          )
        end
        if yielded.has_key? :do_what
          set_do_what(yielded[:do_what])
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @do_this_action = 'Compiling'
    @ljust_to_the_left   = 32
    @n_just_to_the_right = 44
    @use_ccache = false
    # ======================================================================= #
    # If the next variable is true then dependencies will be shown,
    # unless they are empty/nil. This value should be false by default.
    # ======================================================================= #
    @show_required_dependencies = false
    @vertical_bar = VERTICAL_BAR.dup
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_compile_this_program
  #
  # This will assign to the program to be used; it should be an instance
  # of the main data-loader class.
  # ========================================================================= #
  def set_compile_this_program(i = '')
    i = i.first if i.is_a? Array
    i = i.dup
    if i.is_a? String
      # ===================================================================== #
      # Turn strings into the corresponding dataset.
      # ===================================================================== #
      require 'rbt/cookbooks/class/cookbooks.rb'
      i = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu }
    end
    @compile_this_program = i
  end; alias set_use_this_as_dataset set_compile_this_program # === set_use_this_as_dataset

  # ========================================================================= #
  # === compile_this_program?
  # ========================================================================= #
  def compile_this_program?
    @compile_this_program
  end; alias input? compile_this_program? # === input?

  # ========================================================================= #
  # === enable_ccache
  # ========================================================================= #
  def enable_ccache
    @use_ccache = true
  end

  # ========================================================================= #
  # === add_this_number_to_right_just
  # ========================================================================= #
  def add_this_number_to_right_just(i)
    @n_just_to_the_right += i
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    Colours.rev
  end

  # ========================================================================= #
  # === default_colour
  # ========================================================================= #
  def default_colour(i = '')
    slateblue(i)
  end

  # ========================================================================= #
  # === extract_to?
  # ========================================================================= #
  def extract_to?
    @compile_this_program.extracted_towards?
  end

  # ========================================================================= #
  # === compiler_to_use?
  # ========================================================================= #
  def compiler_to_use?
    @compile_this_program.compiler_to_use?.to_s
  end

  # ========================================================================= #
  # === keep_extracted?
  # ========================================================================= #
  def keep_extracted?
    @compile_this_program.keep_extracted?.to_s # Must be a String.
  end

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    @compile_this_program.postinstall?
  end

  # ========================================================================= #
  # === set_use_ccache
  # ========================================================================= #
  def set_use_ccache(i)
    @use_ccache = i
  end

  # ========================================================================= #
  # === blank_line
  # ========================================================================= #
  def blank_line
    e ' '+return_vertical_bar_while_honouring_unicode+' '.ljust(@ljust_to_the_left - 2)+
      ' '+return_vertical_bar_while_honouring_unicode+' '.rjust(@n_just_to_the_right + 1)+
      ' '+return_vertical_bar_while_honouring_unicode
  end

  # ========================================================================= #
  # === determine_padding_size_to_use
  # ========================================================================= #
  def determine_padding_size_to_use(i)
    if i.size > @n_just_to_the_right
      number = i.size - @n_just_to_the_right
      add_this_number_to_right_just(number)
      add_this_number_to_hliner(number)
    end
  end

  # ========================================================================= #
  # === return_horizontal_interface_token
  #
  # This method will either return '=' or it will return a unicode
  # symbol for this.
  # ========================================================================= #
  def return_horizontal_interface_token
    if really_use_unicode?
      Roebe.box_drawings_heavy_horizontal
    else
      '='
    end
  end

  # ========================================================================= #
  # === padded_colourize_of_this_input
  # ========================================================================= #
  def padded_colourize_of_this_input(
      i, use_this_colour = :seagreen
    )
    _ = i.to_s.ljust(@n_just_to_the_right)
    return COLOURS.send(use_this_colour, _)
  end

  # ========================================================================= #
  # === colourized_verbose_truth
  # ========================================================================= #
  def colourized_verbose_truth(i)
    _ = verbose_truth(i).ljust(@n_just_to_the_right)
    case _
    when /^No./
      return mediumorchid(_)
    when /^Yes./
      return royalblue(_)
    end
  end

  # ========================================================================= #
  # === do_what=
  #
  # This method can be used to determine which action is to be done.
  # ========================================================================= #
  def do_what=(i = 'Compiling')
    @do_this_action = i
  end; alias set_do_what do_what= # === set_do_what

  # ========================================================================= #
  # === extracting_compiling_or_installing
  #
  # This method will return one of three possible, different strings.
  # ========================================================================= #
  def extracting_compiling_or_installing
    @do_this_action
  end

  # ========================================================================= #
  # === parameters_to_make?
  # ========================================================================= #
  def parameters_to_make?
    yaml_dataset?['parameters_to_make']
  end

  # ========================================================================= #
  # === yaml_dataset?
  # ========================================================================= #
  def yaml_dataset?
    @compile_this_program.internal_hash?[:yaml_dataset]
  end

  # ========================================================================= #
  # === add_this_number_to_hliner
  # ========================================================================= #
  def add_this_number_to_hliner(i)
    @vertical_bar += (return_horizontal_interface_token * i)
  end

  # ========================================================================= #
  # === return_vertical_bar_while_respecting_unicode
  # ========================================================================= #
  def return_vertical_bar_while_respecting_unicode
    _ = @vertical_bar
    if really_use_unicode?
      unicode_token = return_horizontal_interface_token
      _.gsub!(/=/, unicode_token)
    end
    _
  end

  # ========================================================================= #
  # === complete_left_padding_with_this_text
  # ========================================================================= #
  def complete_left_padding_with_this_text(i)
    "#{consider_replacing_this_input_with_vertial_unicode_bar(LEFT_PADDED_VERTICAL_BAR)}#{i}:".ljust(@ljust_to_the_left)+' '+
    return_vertical_bar_while_honouring_unicode+' '
  end

  # ========================================================================= #
  # === consider_replacing_this_input_with_vertial_unicode_bar
  # ========================================================================= #
  def consider_replacing_this_input_with_vertial_unicode_bar(i)
    if really_use_unicode? and i.include?('|')
      i = i.dup if i.frozen?
      i.gsub!(/\|/, Roebe.box_drawings_heavy_vertical)
    end
    i
  end

  # ========================================================================= #
  # === return_vertical_bar_while_honouring_unicode
  # ========================================================================= #
  def return_vertical_bar_while_honouring_unicode
    if really_use_unicode?
      Roebe.box_drawings_heavy_vertical
    else
      '|'
    end
  end

  # ========================================================================= #
  # === really_use_unicode?
  # ========================================================================= #
  def really_use_unicode?
    use_unicode? and Object.const_defined?(:Roebe)
  end

  # ========================================================================= #
  # === return_box_drawings_heavy_up_and_horizontal
  # ========================================================================= #
  def return_box_drawings_heavy_up_and_horizontal
    Roebe.box_drawings_heavy_up_and_horizontal
  end

  # ========================================================================= #
  # === ascii_hliner
  #
  # Do a vertical bar through this method.
  # ========================================================================= #
  def ascii_hliner(
      i               = :default,
      extra_arguments = nil
    )
    case i
    when :default
      i = " #{return_vertical_bar_while_respecting_unicode}"
    end
    i = i.dup if i.frozen?
    if extra_arguments
      if extra_arguments.is_a? Symbol
        case extra_arguments
        when :box_drawings_heavy_up_and_left_and_right
          if really_use_unicode?
            i[1,1] = box_drawings_heavy_up_and_right
            i[-1,1] = box_drawings_heavy_up_and_left
          end
        when :box_drawings_heavy_up_and_right
          if really_use_unicode?
            i[1,1] = box_drawings_heavy_up_and_right
          end
        when :box_drawings_heavy_up_and_left
          if really_use_unicode?
            i[-1,1] = box_drawings_heavy_up_and_left
          end
        end
      end
    end
    if block_given?
      yielded = yield
      case yielded
      when :heavy_up_and_horizontal
        if really_use_unicode?
          i[33,1] = return_box_drawings_heavy_up_and_horizontal
        end
      end
    end
    e i
  end

  # ========================================================================= #
  # === box_drawings_heavy_up_and_right
  # ========================================================================= #
  def box_drawings_heavy_up_and_right
    Roebe.box_drawings_heavy_up_and_right
  end

  # ========================================================================= #
  # === box_drawings_heavy_up_and_left
  # ========================================================================= #
  def box_drawings_heavy_up_and_left
    Roebe.box_drawings_heavy_up_and_left
  end

  # ========================================================================= #
  # === box_drawings_heavy_down_and_left
  # ========================================================================= #
  def box_drawings_heavy_down_and_left
    Roebe.box_drawings_heavy_down_and_left
  end

  # ========================================================================= #
  # === box_drawings_heavy_down_and_right
  # ========================================================================= #
  def box_drawings_heavy_down_and_right
    Roebe.box_drawings_heavy_down_and_right
  end

  # ========================================================================= #
  # === box_drawings_heavy_down_and_horizontal
  # ========================================================================= #
  def box_drawings_heavy_down_and_horizontal
    Roebe.box_drawings_heavy_down_and_horizontal
  end

  # ========================================================================= #
  # === ascii_hliner_with_this_text
  # ========================================================================= #
  def ascii_hliner_with_this_text(
      this_text = ' Infobox ', &block
    )
    yielded = nil
    yielded = block.call if block
    _ = " #{return_vertical_bar_while_respecting_unicode}".dup
    # ======================================================================= #
    # Next, add the message, in powderblue() colour:
    # ======================================================================= #
    _[2, this_text.size] = powderblue(this_text)
    if really_use_unicode?
      _[59,1] = box_drawings_heavy_down_and_horizontal
      _[1,1]  = box_drawings_heavy_down_and_right
      case yielded
      when :box_drawings_heavy_down_and_left
        _ [-1,1] = box_drawings_heavy_down_and_left
      end
    end
    e _
  end

  # ========================================================================= #
  # === RBT::Infobox[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  require 'rbt'
  if ARGV.empty?
    _ = RBT::Cookbooks.new('virtualenv') { :bypass_menu_check }
  else
    _ = RBT::Cookbooks.new(ARGV.first) { :bypass_menu_check }
  end
  RBT::Infobox.new(_)
end # infobox
