#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Gobolinux::CreateDependenciesFile
#
# This class will create a Dependencies file, compatible to Gobolinux.
#
# The format of a typical Dependencies file in GoboLinux goes like this:
#
#   GCC 4.6.1
#   Glibc 2.14
#   KDE-Libs 4.8.1
#   Phonon 4.5.1
#   Qt 4.8.0
#   Soprano 2.7.5
#
# Have a look at these examples:
#
#   http://recipes.gobolinux.org/r/?list=Expat&ver=2.2.1-r2&file=Resources/Dependencies
#   http://recipes.gobolinux.org/r/?list=DBus&ver=1.10.8-r3&file=Resources/Dependencies
#
# =========================================================================== #
# require 'rbt/linux/gobolinux/create_dependencies_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Linux

module Gobolinux

class CreateDependenciesFile < Base # === RBT::Linux::Gobolinux::CreateDependenciesFile

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === CreateDependenciesFile::MAY_WE_CREATE_RESOURCES
  #
  # This should only be true on Gobolinux.
  #
  # The method allow_creation_of_resources() allows us to overrule this
  # setting.
  # ========================================================================= #
  MAY_WE_CREATE_RESOURCES = false

  # ========================================================================= #
  # === initialize
  #
  # The first argument should hold where we will create the Dependencies
  # file.
  # ========================================================================= #
  def initialize(
      create_where = return_pwd,
      dependencies = nil,
      run_already  = true
    )
    reset
    set_create_where(
      create_where
    ) # Where we create it.
    set_dependencies(
      dependencies
    )
    case run_already
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @may_we_create_resources = MAY_WE_CREATE_RESOURCES
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === allow_creation_of_resources
  # ========================================================================= #
  def allow_creation_of_resources
    @may_we_create_resources = true
  end

  # ========================================================================= #
  # === set_create_where
  # ========================================================================= #
  def set_create_where(i)
    i.delete!('-') if i.include? programs_directory?
    @create_where = rds(i)
  end

  # ========================================================================= #
  # === add_program_version_to_the_dependencies
  #
  # This method will append the proper program version.
  # ========================================================================= #
  def add_program_version_to_the_dependencies
    dataset?.map! {|program|
      dataset = RBT::Cookbooks::Cookbook.new(program) { :bypass_menu_check }
      program_version = dataset.program_version?
      program << " >= #{program_version}" unless program_version.empty?
      program
    }
  end

  # ========================================================================= #
  # === set_dependencies
  # ========================================================================= #
  def set_dependencies(i)
    if i
      if i.is_a? Array
        i.map! {|entry| rds(entry) }
      else
        i = rds(i)
      end
    end
    @dependencies = i
  end

  # ========================================================================= #
  # === dependencies?
  # ========================================================================= #
  def dependencies?
    @dependencies
  end; alias dataset? dependencies? # === dataset?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if @may_we_create_resources
      file_location = rds(
        "#{@create_where}/Resources/Dependencies"
      )
      remove_file(file_location) if File.exist? file_location # Remove it if it already exists
      target_dir = rds(@create_where+'/Resources')
      unless File.directory? target_dir
        opnn; mkdir(target_dir, :be_verbose)
      end
      if @dependencies
        unless @dependencies.empty?
          opnn; e "Appending now into `#{sfile(file_location)}`."
          add_program_version_to_the_dependencies
          @dependencies.each { |dep|
            append_what_into(dep.capitalize+N,
              file_location) # ProgramInformation.return_version(_['program_name'])
          }
        end
      end
    end
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::Gobolinux::CreateDependenciesFile.new
end # create_dependencies_files htop