#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::Slackware::CreateSlackwarePackage
#
# This class can be used to create a slackware package.
#
# It is currently limited in the sense that it will generate only
# the version that is kept in the yaml file itself, so you can not
# manually overrule this for now.
#
# Perhaps at a later time I may change this behaviour, but for now,
# "keep it simple".
#
# The default is to also automatically install the package after
# having finishing compiling it. If you do not want this to happen
# then pass the argument "--do-not-install" to this class, without
# the quotes.
#
# Logic of the class:
#   - It will simply do some linear step-by-step installation
#     procedure.
# =========================================================================== #
# require 'rbt/linux/slackware/create_slackware_package.rb'
# =========================================================================== #
begin
  require 'program_information'
rescue LoadError; end
require 'rbt/base/base.rb'
require 'rbt/make/make.rb'
require 'rbt/toplevel_methods/run_makepkg_command.rb'
require 'rbt/linux/slackware/generate_slack_desc_file.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Linux

module Slackware

class CreateSlackwarePackage < Base # === RBT::Linux::Slackware::CreateSlackwarePackage

  # ========================================================================= #
  # === ALSO_INSTALL_THE_PACKAGE
  # ========================================================================= #
  ALSO_INSTALL_THE_PACKAGE = true

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === BASIC_CONFIGURE_COMMAND
  # ========================================================================= #
  BASIC_CONFIGURE_COMMAND =
    'configure --prefix=/usr --localstatedir=/var --sysconfdir=/etc'

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the name of the program.
  # ========================================================================= #
  def initialize(
      name_of_the_program = Dir.pwd, # By default we will obtain information from the current working directory.
      run_already         = true
    )
    reset
    set_name_of_the_program(
      name_of_the_program
    )
    if block_given?
      yielded = yielded
      case yielded
      when :do_not_autoinstall
        do_not_autoinstall
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @base_directory = nil
    @also_install_the_package = ALSO_INSTALL_THE_PACKAGE
    @program_name = nil
    @program_version = nil
    @use_cookbook_dataset = true
    @use_directory = false
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Show the help options for this class.
  #
  # Invocation example:
  #
  #   islackware --help
  #
  # ========================================================================= #
  def show_help
    e
    e '  --do-not-install # Do not run the auto-installation part.'
    e '  --use-directory  # Use the current working directory as '\
      'base for input'
    e
  end

  # ========================================================================= #
  # === name_of_the_program?
  # ========================================================================= #
  def name_of_the_program?
    @name_of_the_program
  end; alias input?        name_of_the_program? # === input?
       alias program_name? name_of_the_program? # === program_name?

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @cookbook_dataset
  end

  # ========================================================================= #
  # === run_make_command
  # ========================================================================= #
  def run_make_command
    RBT.colourized_make # Run "make" here.
  end

  # ========================================================================= #
  # === program_name_and_program_version
  # ========================================================================= #
  def program_name_and_program_version
    program_name?+'-'+program_version?
  end

  # ========================================================================= #
  # === enter_base_directory
  # ========================================================================= #
  def enter_base_directory
    cd @base_directory
  end

  # ========================================================================= #
  # === generate_slack_desc_file_into_the_current_directory
  # ========================================================================= #
  def generate_slack_desc_file_into_the_current_directory(i = input?)
    RBT::GenerateSlackDescFile.new(i) { :use_short_name }
  end

  # ========================================================================= #
  # === cd_into_install_subdirectory
  # ========================================================================= #
  def cd_into_install_subdirectory
    cd base_dir?+'install/'
  end

  # ========================================================================= #
  # === create_install_subdirectory
  # ========================================================================= #
  def create_install_subdirectory
    _ = base_dir?+'install/'
    mkdir _
  end

  # ========================================================================= #
  # === run_make_install_destdir_command
  #
  # This will run a command such as:
  #   make install DESTDIR=$MY_TEMP/build/grep-3.0
  # ========================================================================= #
  def run_make_install_destdir_command
    _ = "make install DESTDIR=#{base_dir?}"
    esystem_gold _
  end

  # ========================================================================= #
  # === create_build_directory
  # ========================================================================= #
  def create_build_directory
    mkdir base_dir?
  end

  # ========================================================================= #
  # === base_directory?
  # ========================================================================= #
  def base_directory?
    @base_directory
  end; alias base_dir? base_directory? # === base_dir?

  # ========================================================================= #
  # === configure_options?
  # ========================================================================= #
  def configure_options?
    @cookbook_dataset.configure_options?
  end

  # ========================================================================= #
  # === determine_base_directory
  #
  # The main base directory is at $MY_TEMP/build.
  # ========================================================================= #
  def determine_base_directory
    opnn; e 'Now determining the '+orange('@base_directory')+' variable.'
    @base_directory = convert_global_env(
      '$MY_TEMP/build/'+program_name_and_program_version+'/'
    ) # Should have a trailing '/' character.
  end

  # ========================================================================= #
  # === strip_binaries
  # ========================================================================= #
  def strip_binaries
    _ = 'strip -s '+base_dir?+'usr/lib/* '+base_dir?+'usr/bin/*'
    esystem_gold _
  end

  # ========================================================================= #
  # === notify_the_user_how_to_install_this_package
  #
  # Purpose of this method is to notify the user where the package
  # can be found (and then installed via installpkg).
  # ========================================================================= #
  def notify_the_user_how_to_install_this_package
    opnn; e 'If you wish to install this package, you can use this command:'
    e
    e simp('  installpkg '+@location_of_the_package)
    e
  end

  # ========================================================================= #
  # === also_install_the_package?
  # ========================================================================= #
  def also_install_the_package?
    @also_install_the_package
  end

  # ========================================================================= #
  # === notify_the_user_what_we_will_install_next
  # ========================================================================= #
  def notify_the_user_what_we_will_install_next
    opnn; e 'We will install the program `'+slateblue(input?)+'`.'
  end

  # ========================================================================= #
  # === do_not_autoinstall
  # ========================================================================= #
  def do_not_autoinstall
    @also_install_the_package = false
  end

  # ========================================================================= #
  # === do_install_the_package
  # ========================================================================= #
  def do_install_the_package
    _ = @location_of_the_package
    if File.exist? _
      cd base_dir?
      opnn; e 'We will next install the package, from the path'
      opnn; e '  '+sfile(_)
      cmd = 'installpkg '+_
      esystem_gold(cmd)
    end
  end

  # ========================================================================= #
  # === set_program_version
  # ========================================================================= #
  def set_program_version(i)
    @program_version = i
  end

  # ========================================================================= #
  # === set_program_name
  # ========================================================================= #
  def set_program_name(i)
    @program_name = i.to_s.capitalize
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @program_version
  end

  # ========================================================================= #
  # === set_name_of_the_program
  # ========================================================================= #
  def set_name_of_the_program(i)
    if i.is_a? Array
      if i.empty? # Handle empty Arrays first.
        i = File.basename(Dir.pwd)
      else
        i = i.join(' ').strip.squeeze(' ')
      end
    end
    i = Dir.pwd if i.nil? # Assign a safeguard here.
    i = i.to_s.dup
    # ========================================================================= #
    # Chop away on '--' parts.
    # ========================================================================= #
    if i.include? '--'
      # ======================================================================= #
      # === cpackage --do-not-install
      # ======================================================================= #
      i = check_against_menu_and_chop_away_double_hyphens(i)
    end
    if i.include? '/'
      i = File.basename(i)
    end
    if i.include? '-' # Then grab all but the last part, through ProgramInformation.
      # i = i.split('-').first
      i = ProgramInformation[i].first
    end
    @name_of_the_program = i
    if @use_cookbook_dataset
      @cookbook_dataset = ::RBT.cookbook(@name_of_the_program)
      assign_default_dataset
    end
  end

  # ========================================================================= #
  # === assign_default_dataset
  # ========================================================================= #
  def assign_default_dataset(
      i = input?
    )
    if @use_cookbook_dataset
      @cookbook_dataset = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
      set_program_version(@cookbook_dataset.program_version?)
    end
  end

  # ========================================================================= #
  # === check_against_menu_and_chop_away_double_hyphens            (menu tag)
  #
  # This is like a menu() call, but it also chops away some
  # commandline instructions that include '--'.
  # ========================================================================= #
  def check_against_menu_and_chop_away_double_hyphens(i)
    case i # case tag
    # ======================================================================= #
    # === islackware --use-directory
    # ======================================================================= #
    when /^-?-?use(-|_)?directory$/i
      i = File.basename(Dir.pwd)
      @use_cookbook_dataset = false
      @use_directory = true
      set_program_name(ProgramInformation.return_name(i))
      use_this_version = ProgramInformation.return_version(i)
      set_program_version(use_this_version)
    # ======================================================================= #
    # === islackware --help
    # ======================================================================= #
    when /^-?-?help/
      show_help
      exit
    # ======================================================================= #
    # === islackware --do-not-install
    # ======================================================================= #
    when /(-?-?do-?not-?install)/,/(-?-?not?-?install)/
      _ = $1.to_s.dup
      i.gsub!(/#{_}/,'')
      do_not_autoinstall
      i = Dir.pwd # Must also overrule it in this case.
    end
    return i
  end

  # ========================================================================= #
  # === run_configure_command_into_the_inferred_prefix
  # ========================================================================= #
  def run_configure_command_into_the_inferred_prefix
    if @use_directory or @use_cookbook_dataset
      _ = './configure --prefix='+prefix?
    else
      _ = './'+BASIC_CONFIGURE_COMMAND
    end
    _ = _.dup
    _.squeeze!(' ')
    if @use_cookbook_dataset
      _ << ' '+configure_options?
    end
    esystem_gold _
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    _ = @cookbook_dataset.prefix?.to_s if @use_cookbook_dataset
    # ======================================================================= #
    # The following condition is only true when we pass a certain
    # commandline flag to the class here.
    # ======================================================================= #
    if program_name?
      _ = programs_dir?+program_name?+'/'+program_version?
    end
    _
  end

  # ========================================================================= #
  # === run_makepkg_command
  # ========================================================================= #
  def run_makepkg_command
    # ======================================================================= #
    # The following code will essentially run slackware's "makepkg"
    # command.
    # ======================================================================= #
    @location_of_the_package = ::RBT.run_makepkg_command(
      program_name?, program_version?
    )
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    notify_the_user_what_we_will_install_next
    run_configure_command_into_the_inferred_prefix
    run_make_command # Run "make".
    determine_base_directory
    create_build_directory
    run_make_install_destdir_command
    # ======================================================================= #
    # Next we could run strip, but for now, we skip that.
    # ======================================================================= #
    strip_binaries
    create_install_subdirectory
    cd_into_install_subdirectory
    # ======================================================================= #
    # Next, we can generate the proper slack-desc file directly
    # into the current directory.
    # ======================================================================= #
    generate_slack_desc_file_into_the_current_directory
    enter_base_directory
    run_makepkg_command
    if also_install_the_package?
      do_install_the_package
    else
      notify_the_user_how_to_install_this_package
    end
  end

  # ========================================================================= #
  # === RBT::Linux::Slackware::CreateSlackwarePackage[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end; end

CreateSlackwarePackage = RBT::Linux::Slackware::CreateSlackwarePackage

end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::Slackware::CreateSlackwarePackage.new(ARGV)
end # create_slackage_package
    # create_slackage_package --use-directory