#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::SlackwareGenerateSlackDescFile
#
# The class RBT::GenerateSlackDescFile will generate a slack-desc file.
#
# A description of this can be seen here:
#
#   http://www.slackwiki.com/Slack-desc
#   https://slackbuilds.org/templates/slack-desc
#
# Interestingly enough, the slack-desc file must have exactly 11 lines.
#
# The maximum length per line shall not exceed 71 characters.
#
# Lines without any other text should *not* have a space after the ':'.
# =========================================================================== #
# require 'rbt/linux/slackware/generate_slack_desc_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Linux

module Slackware

class GenerateSlackDescFile < Base # === RBT::Linux::Slackware::GenerateSlackDescFile

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === USE_THIS_FILE_SUFFIX
  #
  # A proper slack-desc file ends with a .slack-desc suffix.
  # ========================================================================= #
  USE_THIS_FILE_SUFFIX = '.slack-desc'

  # ========================================================================= #
  # === CHARACTERS_PER_LINE_MAXIMUM
  # ========================================================================= #
  CHARACTERS_PER_LINE_MAXIMUM = 68

  # ========================================================================= #
  # === HANDY_RULER
  # ========================================================================= #
  HANDY_RULER = 
    '       |-----handy-ruler------------------------------------------------------|'

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be the name of the file in question.
  # ========================================================================= #
  def initialize(
      this_program = ARGV,
      run_already  = true
    )
    reset
    if block_given?
      yielded = yield
      case yielded
      when :use_short_name
        @use_long_name_or_short_name = :short_name
      end
    end
    set_this_program(this_program)
    run if run_already
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i)
    i = i.first if i.is_a? Array
    @this_program = i
  end

  # ========================================================================= #
  # === obtain_main_dataset
  # ========================================================================= #
  def obtain_main_dataset
    @dataset = RBT::Cookbooks::Cookbook.new(program?) { :bypass_menu_check }
  end

  # ========================================================================= #
  # === return_handy_ruler
  # ========================================================================= #
  def return_handy_ruler
    HANDY_RULER
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end; alias name?            this_program? # === name?
       alias program?         this_program? # === program?
       alias name_of_program? this_program? # === name_of_program?
       alias program_name?    this_program? # === program_name?

  # ========================================================================= #
  # === length_of_program_name?
  # ========================================================================= #
  def length_of_program_name?
    @this_program.size
  end

  # ========================================================================= #
  # === short_description?
  # ========================================================================= #    
  def short_description?
    @dataset.short_description?
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    _ = @dataset.description?.dup
    _.tr!("\n",' ')
    return _
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @_ = [] # Should be an Array.
    @use_long_name_or_short_name = :long_name
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === chunk
  #
  # Split at 80 characters.
  # ========================================================================= #
  def chunk(
      string, size = CHARACTERS_PER_LINE_MAXIMUM
    )
    size -= length_of_program_name?
    string.gsub(/(.{1,#{size}})(\s+|$)/, "\\1\n")
  end

  # ========================================================================= #
  # === generate_main_string
  #
  # This will generate the main String, into the variable @_.
  # ========================================================================= #
  def generate_main_string
    result = ''.dup
    _ = name_of_program?
    _ = _.dup
    if short_description?
      _ << " (#{short_description?})" unless short_description?.empty?
    else
      e swarn('No short description for:')
      e swarn(_)
    end
    _ << N
    result << _
    result << N 
    result << chunk(description?)
    if result.is_a? String
      result = result.split(N)
    end
    @_ << result
    @_.flatten!
    # ======================================================================= #
    # Next, fill in the missing slots:
    # ======================================================================= #
    (11 - @_.size).times {
      @_ << ''
    }
    # ======================================================================= #
    # And then, prepend the program name to it.
    # ======================================================================= #
    @_.map {|line|
      line = line.dup if line.frozen?
      if line.empty?
        line.prepend("#{program_name?}:")
      else
        line.prepend("#{program_name?}: ")
      end
    }
    # ======================================================================= #
    # Next, we could add a handy ruler - but I personally find this
    # useless, so the following line is commented away.
    # ======================================================================= #
    # @_[0,0] = return_handy_ruler+N
  end

  # ========================================================================= #
  # === return_name_of_slack_desc_file
  # ========================================================================= #
  def return_name_of_slack_desc_file
    program_name?+USE_THIS_FILE_SUFFIX
  end

  # ========================================================================= #
  # === return_short_name
  # ========================================================================= #
  def return_short_name
    USE_THIS_FILE_SUFFIX.delete('.')
  end; alias return_short_name_of_slack_desc_file return_short_name # === return_short_name_of_slack_desc_file

  # ========================================================================= #
  # === create_slack_desk_file
  # ========================================================================= #
  def create_slack_desk_file
    into = path?
    opnn; e 'Now creating a slack-desc file ('+sfile(into)+').'
    what = @_.join(N)+N
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === path?
  # ========================================================================= #
  def path?
    path = rds(Dir.pwd+'/').dup
    case @use_long_name_or_short_name # case tag
    when :long_name,
         :default # This is the default.
      path << return_name_of_slack_desc_file
    when :short_name
      path << return_short_name_of_slack_desc_file
    end
    path
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_main_dataset
    generate_main_string
    create_slack_desk_file
  end

  # ========================================================================= #
  # === RBT::GenerateSlackDescFile[]
  # ========================================================================= #
  def self.[](i = nil)
    self.new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.generate_slackware_description_file
# =========================================================================== #
def self.generate_slackware_description_file(i)
  RBT::Linux::Slackware::GenerateSlackDescFile[i].path?
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.generate_slackware_description_file(ARGV)
end # generate_slack_desc_file htop