#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Make
#
# This class essentially wraps over 'make' and invokes it, with specific
# options when applicable. Additionally the output can be parsed via
# colours, and also checked for errors which can be 'fed back# to a
# toplevel method on RBT.
#
# While this class will default to 'make', it can also make use of
# 'ninja' or any other build-system rather than make.
# =========================================================================== #
# require 'rbt/make/make.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
require 'rbt/misc/colourize_parser.rb'
require 'rbt/errors/map_line_to_a_specific_error.rb'
require 'rbt/make/menu.rb'

module RBT

class Make < RBT::CompileBase # === RBT::Make

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_RUN_ALREADY
  # ========================================================================= #
  DEFAULT_RUN_ALREADY = false

  # ========================================================================= #
  # === initialize
  #
  # By default run_already will be false, as we wish to let the user more
  # easily decide at which point to .run() on this class.
  #
  # Do note that this class can also be invoked via the following way:
  #
  #   Make.new { :run }
  #
  # ========================================================================= #
  def initialize(
      optional_commandline_arguments = ARGV,
      run_already                    = DEFAULT_RUN_ALREADY
    )
    register_sigint
    reset
    @run_already = run_already
    set_commandline_arguments(
      optional_commandline_arguments
    )
    if block_given?
      yielded = yield
      handle_this_block(yielded)
    end
    run if @run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @run_make         = true # Whether to really invoke "make" or not.
    @run_make_install = false
    reset_internal_dataset
  end

  # ========================================================================= #
  # === set_run_make_install
  # ========================================================================= #
  def set_run_make_install(i)
    @run_make_install = i
  end

  # ========================================================================= #
  # === reset_internal_dataset
  # ========================================================================= #
  def reset_internal_dataset
    # ======================================================================= #
    # The next Array will keep track as to which commands were run.
    # ======================================================================= #
    @array_history_of_commands_run = []
    @make_options = []
    @use_this_make_command = 'make'
    # ======================================================================= #
    # The following Array can be modified by the user.
    # ======================================================================= #
    @array_run_these_commands = []
    # ======================================================================= #
    # We will delegate towards class RBT::ColourizeParser, in order
    # to colourize the output. This could, in principle, be disabled,
    # but I like colours, so the default is to make use of it.
    # ======================================================================= #
    @colourize_parser = RBT::ColourizeParser.new
    # ======================================================================= #
    # The next variable determines whether we will make use of porg
    # during the "make install" step. This is optional, so it will
    # be disabled by default.
    # ======================================================================= #
    @use_porg = false
    # ======================================================================= #
    # The next variable can be used specifically for when we will make use
    # of porg. When set then it should be a String such as "htop-2.2.1".
    # ======================================================================= #
    @program_name_and_program_version = nil
  end

  # ========================================================================= #
  # === set_use_porg
  # ========================================================================= #
  def set_use_porg(
      i = false
    )
    @use_porg = i
  end

  # ========================================================================= #
  # === determine_which_commands_to_run
  # ========================================================================= #
  def determine_which_commands_to_run
    @array_run_these_commands << @use_this_make_command
  end

  # ========================================================================= #
  # === do_use_porg
  # ========================================================================= #
  def do_use_porg
    @use_porg = true
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    help_string = <<-EOF

Documented options include:

   #{gold('--fast')} # #{slateblue('make use of more than one CPU core')}

EOF
    e help_string
  end

  # ========================================================================= #
  # === colourize_parser?
  # ========================================================================= #
  def colourize_parser?
    @colourize_parser
  end; alias cparser? colourize_parser? # === cparser?

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input? commandline_arguments? # === commandline_arguments?

  # ========================================================================= #
  # === use_which_make_command?
  # ========================================================================= #
  def use_which_make_command?
    @use_this_make_command
  end; alias use_which_command? use_which_make_command? # === use_which_command?
       alias make_command?      use_which_make_command? # === make_command?

  # ========================================================================= #
  # === joined_make_options?
  #
  # This method will return a String variant of the content found in
  # @make_options.
  #
  # We have to apply only .rstrip here because there may be valid padding
  # on the left hand side, such as ' install', which may become
  # 'make install' subsequently.
  # ========================================================================= #
  def joined_make_options?
    @make_options.join(' ').rstrip
  end

  # ========================================================================= #
  # === append_make_options
  # ========================================================================= #
  def append_make_options(i)
    # ======================================================================= #
    # Handle String input such as 'foo,bar' or 'foo;bar' next:
    # ======================================================================= #
    if i.is_a? String
      if i.include?(',')
        i = i.split(',')
      elsif i.include?(';')
        i = i.split(';')
      end
    end
    @make_options << i
    @make_options.flatten!
  end; alias parameters_to_make         append_make_options # === parameters_to_make
       alias append_these_options       append_make_options # === append_these_options
       alias append_this_option_to_make append_make_options # === append_this_option_to_make
       alias append_these_make_options  append_make_options # === append_these_make_options

  # ========================================================================= #
  # === try_to_append_this
  # ========================================================================= #
  def try_to_append_this(
      i = ' install'
    )
    unless use_which_command?.end_with? i
      append_these_make_options(i)
    end
  end; alias try_to_append_this_option_to_make try_to_append_this # === try_to_append_this_option_to_make

  # ========================================================================= #
  # === append_these_options?
  # ========================================================================= #
  def append_these_options?
    @make_options
  end; alias make_options? append_these_options? # === make_options?

  # ========================================================================= #
  # === do_run_the_make_install_step_as_well
  # ========================================================================= #
  def do_run_the_make_install_step_as_well
    determine_whether_to_run_make_install(:yup)
  end

  # ========================================================================= #
  # === history_of_commands_run?
  # ========================================================================= #
  def history_of_commands_run?
    @array_history_of_commands_run
  end

  # ========================================================================= #
  # === make
  #
  # This method will essentially just run "make", with any options given
  # to make on top of that.
  # ========================================================================= #
  def make
    cmd = ('make '+joined_make_options?).strip
    esystem cmd
  end

  # ========================================================================= #
  # === do_run_the_commands
  #
  # This is the main powerhorse method of this class, aka the most
  # important one.
  # ========================================================================= #
  def do_run_the_commands(
      i = @array_run_these_commands
    )
    if @run_make
      i.each {|entry|
        unless @make_options.empty?
          # ===================================================================== #
          # Append the make-options to "make" next:
          # ===================================================================== #
          entry = entry.dup if entry.frozen?
          entry << ' '+joined_make_options?
        end
        entry = entry.dup if entry.frozen?
        entry.squeeze!(' ')
        entry << SILENT_REDIRECTION if silent_redirection?
        e "Now running `#{mediumturquoise(entry)}` from #{sdir_return_pwd}."
        run_this_command_via_io_popen(entry)
      }
    end
    # ======================================================================= #
    # Next, run "make install" or the equivalent if the user decided
    # that this should be run. 
    # ======================================================================= #
    if run_make_install?
      if @use_porg
        entry = "porg -lp #{@program_name_and_program_version} \"#{make_command?} install\""
        e "Now running `#{mediumturquoise(entry)}` from #{sdir_return_pwd}."
        run_this_command_via_io_popen(entry)
      else
        esystem "#{make_command?} install"
      end
    end
  end

  # ========================================================================= #
  # === set_program_name_and_program_version
  # ========================================================================= #
  def set_program_name_and_program_version(i)
    @program_name_and_program_version = i
  end

  # ========================================================================= #
  # === run_this_command_via_io_popen
  # ========================================================================= #
  def run_this_command_via_io_popen(entry)
    cparser = cparser?
    io_object = IO.popen(entry, :err => [:child, :out]).each { |line|
      cparser.grab_this_line(line)
      # ===================================================================== #
      # Delegate towards module RBT::Errors::MapLineToASpecificError to
      # register any possible error.
      # ===================================================================== #
      RBT::Errors::MapLineToASpecificError.parse_this_line(line)
      line = cparser.line?
      # ===================================================================== #
      # Store the line in our history:
      # ===================================================================== #
      @array_history_of_commands_run << line
      e line # Output here.
    }
    io_object.close # Close it up again.
  end; alias esystem              run_this_command_via_io_popen # === esystem
       alias run_this_sys_command run_this_command_via_io_popen # === run_this_sys_command

  # ========================================================================= #
  # === run_make_install?
  # ========================================================================= #
  def run_make_install?
    @run_make_install
  end

  # ========================================================================= #
  # === do_run_make_install
  # ========================================================================= #
  def do_run_make_install
    @run_make_install = true
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(
      i = nil
    )
    if i.is_a? Hash
      # ===================================================================== #
      # First pass into a special method:
      # ===================================================================== #
      handle_hash_or_symbol(i)
      # ===================================================================== #
      # === :use_this_make_command
      # ===================================================================== #
      if i.has_key? :use_this_make_command
        set_use_this_make_command(
          i.delete(:use_this_make_command)
        )
      end
      # ===================================================================== #
      # === :run_make_install
      # ===================================================================== #
      if i.has_key? :run_make_install
        if i.delete(:run_make_install) # Then this must be true.
          try_to_append_this_option_to_make(' install')
        end
      end
    end
    @commandline_arguments = [i].flatten.compact
  end

  # ========================================================================= #
  # === handle_hash_or_symbol
  # ========================================================================= #
  def handle_hash_or_symbol(i)
    if i.is_a? Hash
      handle_this_hash(i)
    elsif i.is_a? Symbol
      handle_this_symbol(i)
    end
  end

  # ========================================================================= #
  # === handle_this_hash
  # ========================================================================= #
  def handle_this_hash(yielded)
    if yielded.is_a? Hash
      # ===================================================================== #
      # === append_these_options
      # ===================================================================== #
      if yielded.has_key? :append_these_options
        _ = yielded[:append_these_options]
        _ = _.join(' ').strip if _.is_a? Array
        append_these_options(_.to_s)
      # ===================================================================== #
      # === use_stow
      # ===================================================================== #
      elsif yielded.has_key?(:append)
        append_these_options(yielded[:append].to_s)
      end
      # ===================================================================== #
      # === run_make
      # ===================================================================== #
      if yielded.has_key? :run_make
        @run_make = yielded.delete(:run_make)
      end
      # ===================================================================== #
      # === :use_this_make_command
      # ===================================================================== #
      if yielded.has_key? :use_this_make_command
        use_this_command = yielded.delete(:use_this_make_command)
        set_use_this_make_command(
          use_this_command
        )
      end
      # ===================================================================== #
      # === :use_porg
      # ===================================================================== #
      if yielded.has_key? :use_porg
        use_porg = yielded.delete(:use_porg)
        set_use_porg(
          use_porg
        )
      end
      # ===================================================================== #
      # === :program_name_and_program_version
      # ===================================================================== #
      if yielded.has_key? :program_name_and_program_version
        set_program_name_and_program_version(
          yielded.delete(:program_name_and_program_version)
        )
      end
      # ===================================================================== #
      # === :run_make_install
      # ===================================================================== #
      if yielded.has_key? :run_make_install
        set_run_make_install(
          yielded.delete(:run_make_install)
        )
      end
    elsif yielded.is_a? Symbol
      handle_symbols(yielded)
    end
  end; alias handle_this_block handle_this_hash # === handle_this_hash

  # ========================================================================= #
  # === handle_symbols
  # ========================================================================= #
  def handle_symbols(i = nil)
    return unless i
    case i
    when :run
      @run_already = true
    when :and_run_make_install_as_well,
         :yup
      @run_make_install = true
    end
  end; alias determine_whether_to_run_make_install handle_symbols # === determine_whether_to_run_make_install
       alias handle_this_symbol                    handle_symbols # === handle_this_symbol

  # ========================================================================= #
  # === set_use_this_make_command
  #
  # We need this setter-method so that we can toggle between, e. g. "make"
  # or "ninja" or any other similar build system.
  # ========================================================================= #
  def set_use_this_make_command(
      i = :default
    )
    case i
    when :make_install
      # ===================================================================== #
      # This one is a bit different - we will only run "make install"
      # here, not both "make" and "make install" combined.
      # ===================================================================== #
      i = 'make' # Handle "make install" instruction here.
      append_this_option_to_make('install')
    when :ninja
      i = 'ninja'
    when :rake
      i = 'rake'
    when :default, nil, :make
      i = 'make' # <- We need to use a sane default value, aka "make".
    end
    @use_this_make_command = i
  end; alias set_use_this_command set_use_this_make_command # === set_use_this_command

  # ========================================================================= #
  # === do_use_ninja
  # ========================================================================= #
  def do_use_ninja
    set_use_this_command :ninja
  end; alias use_ninja do_use_ninja # === use_ninja

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    if block_given?
      determine_whether_to_run_make_install(yield)
    end
    determine_which_commands_to_run
    do_run_the_commands
  end

  # ========================================================================= #
  # === RBT::Make[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i).run
  end; self.instance_eval { alias run [] } # === RBT::Make.run

end

ColourMake = Make

# =========================================================================== #
# === RBT.colour_make
# =========================================================================== #
def self.colour_make(&block)
  RBT::Make.new(&block).run
end; self.instance_eval { alias colourized_make colour_make } # === RBT.colourized_make

end

if __FILE__ == $PROGRAM_NAME
  RBT::Make.new(ARGV).run { :and_run_make_install_as_well }
end # rbt_make