#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreateAndPopulatePkgconfigDirectory
#
# This class' primary purpose is to clone an existing share/pkgconfig/
# directory into a lib/pkgconfig/ directory. This will only work on
# AppDir like directories - and only if at the least one .pc file
# can be found.
#
# The file has two possible "modus operandi" to work:
#   - symlink the .pc files into that new directory
# or
#   - copy the .pc files into that new directory
#
#
# The default is to copy the files, so :copy.
#
# The input argument to this method should be a String such
# as "/Programs/Sharedmimeinfo/Current/".
#
# Usage example:
#   RBT::CreateAndPopulatePkgconfigDirectory.new
# =========================================================================== #
# require 'rbt/misc/create_and_populate_pkgconfig_directory.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreateAndPopulatePkgconfigDirectory < Base # === RBT::CreateAndPopulatePkgconfigDirectory.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_check_on_this_directory(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @mode = :copy # <- can be :symlink or :copy
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_check_on_this_directory
  # ========================================================================= #
  def set_check_on_this_directory(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @check_on_this_directory = i
  end

  # ========================================================================= #
  # === check_on_this_directory?
  # ========================================================================= #
  def check_on_this_directory?
    @check_on_this_directory
  end; alias target? check_on_this_directory? # === target?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = target? # This may be e. g. "/Programs/Sharedmimeinfo/Current/".
    if File.directory? _
      # ===================================================================== #
      # Ok, it is a directory. Now we have to check for two things:
      #   (1) does the subdirectory "share/pkgconfig/" exist.
      # ===================================================================== #
      new_target = _+'share/pkgconfig/'
      if File.directory?(new_target) and !(Dir.empty?(new_target))
        # =================================================================== #
        # If we enter here, we know that e. g.
        # "/Programs/Sharedmimeinfo/Current/share/pkgconfig/".
        # must exist. So now we have to check if lib/pkgconfig/ does
        # NOT exist - or if it does, whether it is empty.
        # =================================================================== #
        new_pkgconfig_directory = _+'lib/pkgconfig/'
        unless File.directory?(new_pkgconfig_directory) and
               !Dir.empty?(new_pkgconfig_directory)
          # ================================================================= #
          # In this case first create this directory:
          # ================================================================= #
          mkdir(new_pkgconfig_directory, :be_silent) unless File.directory?(new_pkgconfig_directory)
          if File.directory?(new_pkgconfig_directory) and
             Dir.empty?(new_pkgconfig_directory)
            case @mode
            when :copy
              Dir[new_target+'*'].each {|this_entry|
                opnn; e 'Copying '+sfile(this_entry)+' towards '+
                         sdir(new_pkgconfig_directory)+'.'
                copy(this_entry, new_pkgconfig_directory)
              }
            when :symlink
              Dir[new_target+'*'].each {|this_entry|
                opnn; e 'Symlinking '+sfile(this_entry)+' towards '+
                         sdir(new_pkgconfig_directory)+'.'
                symlink(this_entry, new_pkgconfig_directory)
              }
            end
          end
        end
      end
    else
      opnn; e "#{teal(_)} is not a directory. This class requires a directory."
    end
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CreateAndPopulatePkgconfigDirectory.new(ARGV)
end # create_and_populate_pkgconfig_directory /Programs/Sharedmimeinfo/Current/