#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ExclusiveLogic
#
# This class can detect logic problems found in Configure scripts.
# 
# The variable @problems indicates whether we have found any problems or not.
#
# A "logic problem" exists if we pass a combination of variables such as 
# "--enable-something" and also "--disable-something" at the same time, 
# which obviously would be a conflict.
#
# How to use this class?
#   Simply pass the (possibly conflicting) string into it.
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ExclusiveLogic < Base # === RBT::ExclusiveLogic

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      string_to_check = '',
      run_already     = true
    )
    reset
    assign_string(string_to_check)
    scan_string if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @string = ''.dup
    @problems = false # Can be true or false.
    @duplicates = [] # An Array.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @string
  end; alias string string? # === string

  # ========================================================================= #
  # === no_problems
  #
  # Set no problems here. This is used just for debugging.
  # ========================================================================= #
  def no_problems
    @problems = false
  end

  # ========================================================================= #
  # === problems?
  # ========================================================================= #
  def problems?
    @problems
  end; alias problems problems? # === problems

  # ========================================================================= #
  # === assign_string
  #
  # Perform sanitizing stuff, then assign to @string.
  # ========================================================================= #
  def assign_string(s)
    @string = s.delete(N).gsub(/  /,' ').chomp.squeeze(' ')
  end

  # ========================================================================= #
  # === has_duplicate?
  # ========================================================================= #
  def has_duplicate?(array)
    _ = false # This will become the return value.
    # ======================================================================= #
    # Get rid of all "enable" and "disable" parts next.
    # ======================================================================= #
    copy = array.map {|entry| entry.gsub(/enable/,'').gsub(/disable/,'') }
    _ = true if copy.uniq
    _
  end

  # ========================================================================= #
  # === split_up_string_properly
  #
  # The .map is used to omit the leading ./configure word.
  # We use .squeeze to get rid of multiple ' '
  # ========================================================================= #
  def split_up_string_properly
    splitted = sanitize_string(@string) # we work on @string.
    all_words = splitted.map {|_| 
      _.split('-').join('').squeeze(' ').gsub(/\.\/configure/,'')
     }.select {|_| ! _.empty?}
    return all_words
  end

  # ========================================================================= #
  # === sanitize_string
  #
  # This method replaces '  ' with ' ' and then goes to split at ' '
  # before it returns an array.
  # ========================================================================= #
  def sanitize_string(i)
    i.gsub(/  /,' ').split(' ') if i
  end

  # ========================================================================= #
  # === force_add
  #
  # This method will force-add a specific configure option.
  #
  # Additionally, it will replace any contradictory configuration options,
  # and thus guarantees that our resulting string will be sane.
  # ========================================================================= #
  def force_add(i)
    i = i.flatten.join(' ') if i.is_a? Array
    i = sanitize_string(i).join
    @string.gsub!(/#{i}/,'') # remove it if it already is part of our string.
    if i.include? 'disable'
      @string.gsub!(/#{i.gsub(/disable/, 'enable')}/,'')
    elsif i.include? 'enable'
      @string.gsub!(/#{i.gsub(/enable/, 'disable')}/,'')
    end
    @string << ' '+i
    scan_string
  end

  # ========================================================================= #
  # === problems?
  # ========================================================================= #
  def problems?
    @problems
  end

  # ========================================================================= #
  # === debug
  # ========================================================================= #
  def debug
    opnn; e "Have we found any Problems in the logic given? "\
            "#{simp(vt(@problems.to_s).delete('.'))}."
    pp @duplicates unless @duplicates.empty?
    pp @string
  end

  # ========================================================================= #
  # === duplicates?
  # ========================================================================= #
  def duplicates?
    @duplicates
  end; alias duplicates duplicates? # === duplicates

  # ========================================================================= #
  # === scan_string
  #
  # This is the actual powerhorse. It also sets the @problems variable
  # in case it encountered a problem. 
  # ========================================================================= #
  def scan_string
    all_words = split_up_string_properly # call this method, which is defined in this file here.
    if has_duplicate? all_words # if true then we must have duplicates.
      @problems = true
      all_duplicates = return_duplicates(all_words)
      unless all_duplicates.empty?
        all_duplicates.each {|duplicate|
          @duplicates << splitted.select {|_| _ =~ /#{duplicate}/ }
        }
        @duplicates.flatten!
        @duplicates.map! {|_| _ = '--'+_ }
      end
    end
  end

  # ========================================================================= #
  # === return_duplicates
  #
  # This method will return keys which are duplicates (or in other words,
  # all entries which occur more than once). '' is ignored though as I 
  # can't see it being useful for anything much really.
  #
  # The file "rbt/lib/rbt/exclusive_logic.rb" needs this method.
  #
  # Example:
  #    ['foo', 'bar', 'ble', 'foo', 'ble', 'go','yo','',''].return_duplicates # => ["ble", "foo"]
  # ========================================================================= #
  def return_duplicates(i)
    return_value = []
    hash = Hash.new(0)
    i.each { |word| hash[word] += 1 }
    hash.select {|key, value|
      return_value << key if value > 1 and ! key.empty?
    }
    return_value = [] if return_value.empty?
    return return_value
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    scan_string
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::ExclusiveLogic.new('
   ./configure 
   --enable-lame 
   --enable-libdv 
   --enable-gtk 
   --enable-netstream 
   --enable-theora 
   --enable-background-image 
   --enable-fading 
   --enable-sigsegv 

   --disable-fading 
   --disable-sigsegv 

   --disable-theora 
  '
  ) # ARGV.join ' '
  _.force_add(ARGV) unless ARGV.empty?
  _.debug
end # elogic --disable-gtk