#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SedWrapper
#
# This class acts as a simplified sed-wrapper. It shall be able to
# correctly interprete given input such as:
#
#   sed -i '/gegl/s/2/3/' configure
#
# Usage example:
#   RBT::SedWrapper.new
# =========================================================================== #
# require 'rbt/misc/sed_wrapper.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SedWrapper < Base # === RBT::SedWrapper

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_SED_COMMAND
  # ========================================================================= #
  DEFAULT_SED_COMMAND =
    # "sed -i '/gegl/s/2/3/' configure"
    "sed -i 's/unsigned short/char16_t/' Source/JavaScriptCore/API/JSStringRef.h Source/WebKit2/Shared/API/c/WKString.h"

  # ========================================================================= #
  # === SED_COMMAND2
  # ========================================================================= #
  SED_COMMAND2 = 
    "sed -i 's/png_\(sizeof\)/\1/g' examples/png2theora.c"

  # ========================================================================= #
  # === SED_COMMAND3
  # ========================================================================= #
  SED_COMMAND3 =
    "sed -i '/-fforce-mem/d' configure"

  # ========================================================================= #
  # === REPORT_INPUT_COMMAND
  # ========================================================================= #
  REPORT_INPUT_COMMAND = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = DEFAULT_SED_COMMAND,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(
      i = DEFAULT_SED_COMMAND
    )
    i = i.first if i.is_a? Array
    i = DEFAULT_SED_COMMAND if i.nil?
    i = i.to_s.dup
    case i # case tag
    when 'sed2','s2','2'
      i = SED_COMMAND2
    when 'sed3','s3','3'
      i = SED_COMMAND3
    end
    @input = i
    if REPORT_INPUT_COMMAND
      opnn; e "The sed command used will be: #{sfancy(i)}"
    end
    sanitize_input
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @work_on_these_files = []
    @dataset = nil
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === sanitize_input
  # ========================================================================= #
  def sanitize_input
    if @input.start_with? 'sed -i'
      @input.sub!(/^sed -i /,'')
    end
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === determine_on_which_files_to_work
  # ========================================================================= #
  def determine_on_which_files_to_work
    _ = @input # Grab a copy of the remaining input first.
    if _.include? "'"
      rindex = _.rindex "'"
      string_for_files = _[(rindex+1)..-1]
      set_work_on_these_files string_for_files.strip.split(' ').map(&:chomp)
      @input = _[0..rindex]
    end
  end

  # ========================================================================= #
  # === set_work_on_these_files
  #
  # It is better to store the absolute path towards the file at hand.
  # ========================================================================= #
  def set_work_on_these_files(i)
    unless i.is_a? Array
      i = [i]
    end
    if i.is_a? Array
      i.map! {|entry|
        File.absolute_path(entry)
      }
    end
    @work_on_these_files = i
  end

  # ========================================================================= #
  # === work_on_these_files?
  # ========================================================================= #
  def work_on_these_files?
    @work_on_these_files
  end; alias target_file?        work_on_these_files? # === target_file?
       alias work_on_these_files work_on_these_files? # === work_on_these_files
       alias these_files?        work_on_these_files? # === these_files?

  # ========================================================================= #
  # === try_to_read_in_this_file
  # ========================================================================= #
  def try_to_read_in_this_file(i = target_file?)
    if File.exist? i
      @dataset = File.readlines(i)
    end
  end

  # ========================================================================= #
  # === batch_modify_the_files
  # ========================================================================= #
  def batch_modify_the_files
    what = ''.dup # String to store.
    @input.delete!("'")
    if @input.start_with?('s/')
      @input[0,1] = ''
    end
    splitted     = @input.split('/').reject(&:empty?) # @input must obviously have a '/' character.
    replace_this = splitted.first
    with_that    = splitted[1]
    case with_that
    when 'd'
      # ===================================================================== #
      # This will be assumed to mean 'delete' but only in certain cases.
      # ===================================================================== #
      if @input.count('/') < 3
        with_that = '' # Chop it off in this case.
      end
    end
    work_on_these_files.each {|this_file|
      opnn; e 'Working on the file `'+sfile(this_file)+'`.'
      try_to_read_in_this_file(this_file) # <- This sets @dataset again.
      if @dataset
        @dataset.each {|line|
          if line.include? replace_this
            line.gsub!(/#{replace_this}/, with_that)
          end
          # ================================================================= #
          # Do the substitute-action here.
          # ================================================================= #
          what << line
        }
        opnn; e 'Storing into the file `'+sfile(this_file)+'` now. '\
                '(Substitution on: '+sfancy(replace_this)+'; '\
                'Replacement on: '+sfancy(with_that)+')'
        save_what_into(what, this_file)
        what = '' # Reset again.
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_on_which_files_to_work
    batch_modify_the_files
  end

  # ========================================================================= #
  # === RBT::SedWrapper[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SedWrapper.new(ARGV)
end # sed_wrapper sed3