#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Prefix
#
# This class simply stores the default prefix.
#
# It will be used for EVERY occurance of a prefix in the RBT scripts.
#
# It will only include the target at where it will point to.
#
# Basic Usage example to query the prefix to use goes like this:
#
#   @prefix = RBT::Prefix.new
#
# As of October 2015, this class is no longer important. It is kept
# here for the time being, though, until we have decided whether
# we really do not need a specialized prefix class anymore.
# =========================================================================== #
# To require this class in your project, do:
#   require 'rbt/prefix/prefix.rb'
# =========================================================================== #
require 'program_information'
require 'rbt/base/base.rb'

module RBT

class Prefix < Base # === RBT::Prefix.new

  # ========================================================================= #
  # === USR_PREFIX
  # ========================================================================= #
  USR_PREFIX = '/usr/' # This is the default prefix.
    DEFAULT_PREFIX = USR_PREFIX # === DEFAULT_PREFIX

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_use_this_program_name = nil,
      run_already                    = true
    )
    reset # Must always come first.
    if optional_use_this_program_name
      set_original_input(optional_use_this_program_name)
      set_program(optional_use_this_program_name)
    end
    if block_given?
      yielded = yield
      case yielded
      when :ntrad
        set_prefix(:ntrad)
      else
        run if run_already
      end
    else
      run if run_already
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @prefix = nil # nil initially. This is our prefix variable.
    set_program
    @be_verbose = false
  end

  # ========================================================================= #
  # === sub!
  # ========================================================================= #
  def sub!(old, new)
    @prefix.sub!(old, new)
  end

  # ========================================================================= #
  # === empty?
  # ========================================================================= #
  def empty?
    @prefix.empty?
  end

  # ========================================================================= #
  # === set_original_input
  # ========================================================================= #
  def set_original_input(i)
    @original_input = i.to_s.dup
  end

  # ========================================================================= #
  # === end_with?
  #
  # Determine whether our prefix ends with a specific string.
  # ========================================================================= #
  def end_with?(i)
    @prefix.end_with? i
  end

  # ========================================================================= #
  # === include?
  #
  # Use this to query whether we include a string.
  # ========================================================================= #
  def include?(i)
    @prefix.include? i
  end

  # ========================================================================= #
  # === prefix?
  #
  # This query-method will output the prefix to use.
  # ========================================================================= #
  def prefix?
    @prefix
  end; alias query prefix? # === query
       alias to_s  prefix? # === to_s
       alias _     prefix? # === _

  # ========================================================================= #
  # === set_program
  #
  # Use this method to set the instance variable @program. It shall be
  # kept like that. The input can contain a '-'.
  # ========================================================================= #
  def set_program(i = nil)
    case i
    when :trad
      set_prefix(i)
    end
    @program = i # Can contain something like htop.
  end; alias assign set_program # === assign

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end

  # ========================================================================= #
  # === return_non_traditional_prefix
  #
  # This method will return a proper non-traditional prefix.
  # ========================================================================= #
  def return_non_traditional_prefix(i)
    program_information = ::ProgramInformation::ProgramInformation.new(i,
      true, @be_verbose
    ) # bl $RUBY_COOKBOOKS/lib/cookbooks/program_information.rb
    name    = program_information.name.to_s.capitalize.delete('-_').capitalize
    version = program_information.version
    i = "#{programs_dir?}#{name}/#{version}/".dup # This has the version name
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # As of May 2014 we get rid of all '-'.
    # ======================================================================= #
    i.delete!('-') if i.include? '-'
    return i # This here should usually return a string like "/Programs/Htop/1.1.0"
  end

  # ========================================================================= #
  # === eliminate_hyphens
  # ========================================================================= #
  def eliminate_hyphens
    program?.delete!('-')
    use_non_traditional_prefix # Also sync towards @prefix here.
  end

  # ========================================================================= #
  # === set_prefix
  #
  # This methods assigns to the instance variable @prefix which keeps the
  # prefix in use. The prefix instance variable is stored in the
  # variable @prefix.
  #
  # Input should include the version, i.e. "htop-1.0" would be a proper
  # input. 
  # ========================================================================= #
  def set_prefix(
      i = program?
    )
    i = USR_PREFIX if i == 'def'
    case i.to_s # case tag
    when 'gobolinux','gobolinux_prefix','ntrad','false',
         'default','app_dir_like',
         /^app(_|-)?dir/
      i = return_non_traditional_prefix(program?) # defined in this file here.
      # ===================================================================== #
      # As of May 2014 we get rid of all '-'
      # ===================================================================== #
      i.delete!('-') if i.include? '-'
    when 'true','t','trad','traditional','tra' # True means "usr" prefix.
      i = USR_PREFIX
    else # Else default to USR_PREFIX.
      # i = USR_PREFIX
    end
    assign_prefix(i) # Now, simply assign it here.
  end; alias prefix=        set_prefix # === prefix=
       alias rebuild_prefix set_prefix # === rebuild_prefix
       alias update         set_prefix # === update

  # ========================================================================= #
  # === use_non_traditional_prefix
  #
  # This method will assign to a non-traditional (gobolinux) prefix.
  # ========================================================================= #
  def use_non_traditional_prefix
    set_prefix(:gobolinux)
  end; alias gobolinux_prefix use_non_traditional_prefix # === gobolinux_prefix

  # ========================================================================= #
  # === assign_prefix
  #
  # Simply assign the prefix, without any further checking.
  # ========================================================================= #
  def assign_prefix(i)
    case i
    when :default, nil
      i = DEFAULT_PREFIX
    end
    i = i.dup if i.frozen?
    i.strip!
    i << '/' unless i.end_with? '/'
    @prefix = i
  end; alias assign_prefix=  assign_prefix # === assign_prefix=
       alias use_this_prefix assign_prefix # === use_this_prefix

  # ========================================================================= #
  # === strip!
  # ========================================================================= #
  def strip!
    @prefix.strip!
  end

  # ========================================================================= #
  # === <<
  # ========================================================================= #
  def <<(i)
    @prefix << i
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    set_prefix
  end

  # ========================================================================= #
  # === RBT::Prefix.prefix?
  # ========================================================================= #
  def self.prefix?(i = nil)
    _ = RBT::Prefix.new(i)
    _.prefix? # Return value.
  end

end

# =========================================================================== #
# === @prefix
# =========================================================================== #
@prefix = nil

# =========================================================================== #
# === RBT.prefix?
# =========================================================================== #
def self.prefix?
  @prefix
end; self.instance_eval { alias last_prefix?      prefix? } # === RBT.last_prefix?
     self.instance_eval { alias last_prefix_used? prefix? } # === RBT.last_prefix_used?

# =========================================================================== #
# === RBT.set_prefix
# =========================================================================== #
def self.set_prefix(i)
  RBT::Prefix.new(i)
end; self.instance_eval { alias set_last_prefix_used set_prefix } # === RBT.set_last_prefix_used

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  require 'cliner/autoinclude'
  input = 'htop-1.2.4'
  prefix = RBT::Prefix.new(input)
  pp prefix
  prefix = RBT::Prefix.new
  prefix.set_prefix('def')
  e 'Now checking for: `'+sfancy(input)+'`.'
  prefix.set_program(input)
  cliner
  e 'Now setting to '+sfancy('Gobolinux-like prefix.')
  prefix.set_prefix(:gobolinux)
  pp prefix._
  pp prefix
  cliner
  prefix.set_prefix(true)
  pp prefix._
  e 'Now assigning to '+sfancy('htop-2.3.2')
  prefix.set_program 'htop-2.3.2'
  prefix.gobolinux_prefix
  pp prefix._
  cliner
  e 'Now assigning to ICKI'
  prefix.assign_prefix 'ICKI'
  pp prefix._
  cliner
  e 'Next invoking '+Colours.orange('RBT.last_prefix_used?')
  e
  e RBT.last_prefix_used?
  e
end # prefix