#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Compile::Shell
#
# This shell allows us to compile programs. The reason why this shell
# exists is because we can bundle together information and results
# from the compilation.
#
# Usage example:
#   RBT::Compile::Shell.new
# =========================================================================== #
# require 'rbt/shell/shell.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/compile/compile.rb'
# =========================================================================== #
# Load up some rbt/shell-specific .rb files.
# =========================================================================== #
require 'rbt/shell/constants.rb'
require 'rbt/shell/help.rb'
require 'rbt/shell/menu.rb' # bl $RBT/shell/menu.rb
require 'rbt/utility_scripts/show_last_updated.rb'

module RBT

class Compile

class Shell < Base # === RBT::Compile::Shell.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    case i
    when :dont_run_yet
      run_already = false
      @run_the_main_loop = false
      i = nil
    end
    set_input(i)
    case run
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    begin
      require 'readline'
    rescue LoadError; end
    @namespace = NAMESPACE
    @can_we_use_readline = Object.const_defined? :Readline
    # ======================================================================= #
    # If the following variable is set to true, then the Shell will
    # ask for user-input. This is the default.
    # ======================================================================= #
    @run_the_main_loop = true
    @use_this_prefix = nil
  end

  # ========================================================================= #
  # === set_use_this_prefix
  # ========================================================================= #
  def set_use_this_prefix(i)
    i = i.to_s.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @use_this_prefix = i
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === notify_the_user_that_input_can_be_passed_in_now
  # ========================================================================= #
  def notify_the_user_that_input_can_be_passed_in_now
    opnn; e 'Type your given input next (or pass "help" for help options):'
  end

  # ========================================================================= #
  # === obtain_user_input
  # ========================================================================= #
  def obtain_user_input
    if @can_we_use_readline
      @user_input = Readline.readline('', true)
      # ===================================================================== #
      # Next ignore empty input.
      # ===================================================================== #
      Readline::HISTORY.pop if @user_input.strip.empty?
    else # else use $stdin
      @user_input = $stdin.gets.chomp
    end
  end

  # ========================================================================= #
  # === run_main_loop
  # ========================================================================= #
  def run_main_loop
    notify_the_user_that_input_can_be_passed_in_now
    loop {
      # ===================================================================== #
      # First, we have to obtain the user input.
      # ===================================================================== #
      obtain_user_input
      i = @user_input
      if i.include? ' '
        splitted = i.split(' ')
        @all_arguments = splitted[1 .. -1].join(' ').strip
        i = splitted.first
      else
        @all_arguments = nil
      end
      a = @all_arguments
      menu(i) { a }
    }
  end

  # ========================================================================= #
  # === notify_the_user_that_this_program_will_be_compiled_next
  #
  # This is just a notification - for the actual compile-action
  # you have to add the code after an invocation to this method
  # here.
  # ========================================================================= #
  def notify_the_user_that_this_program_will_be_compiled_next(i)
    opnn; e "Next compiling `#{sfancy(i.strip)}`."
  end

  # ========================================================================= #
  # === do_compile_this_program
  # ========================================================================= #
  def do_compile_this_program(i)
    i.strip!
    if i.include? ' '
      # ===================================================================== #
      # In this case we need to determine what the user wants.
      # ===================================================================== #
      splitted = i.split(' ')
      first = splitted.first
      last  = splitted.last
      case last
      # ===================================================================== #
      # === trad
      # ===================================================================== #
      when 'trad'
        notify_the_user_that_this_program_will_be_compiled_next(first)
        RBT::Compile.new(first) {{ prefix: 'trad' }}
        return
      end
    end
    notify_the_user_that_this_program_will_be_compiled_next(i)
    if @use_this_prefix
      RBT::Compile.new(i) {{ prefix: @use_this_prefix }}
    else
      RBT::Compile.new(i)
    end
  end

  # ========================================================================= #
  # === query_the_prefix_in_use
  # ========================================================================= #
  def query_the_prefix_in_use
    prefix = '/usr/'
    prefix = @use_this_prefix if @use_this_prefix
    prefix
  end

  # ========================================================================= #
  # === show_available_programs
  # ========================================================================= #
  def show_available_programs
    available_programs?.each_with_index {|this_program, index| index += 1
      e sfancy(index)+' '+simp(this_program)
    }
  end

  # ========================================================================= #
  # === show_last_update
  # ========================================================================= #
  def show_last_update(
      n_last_programs = DEFAULT_SHOW_N_PROGRAMS_THAT_WERE_LAST_UPDATED
    )
    if n_last_programs.is_a? Array
      n_last_programs = n_last_programs.first
    end
    if n_last_programs.nil?
      n_last_programs = DEFAULT_SHOW_N_PROGRAMS_THAT_WERE_LAST_UPDATED # Default value.
    end
    n_last_programs = n_last_programs.to_i if n_last_programs.is_a? String
    array = Cookbooks::ShowLastUpdated.dataset?
    result = array[0 .. (n_last_programs - 1)] # -1 because Array start at 0.
    result.each_with_index {|inner_array, index| index += 1 # ["alock", "01 Jul 2011", 55]
      name_of_the_program = inner_array.first
      last_updated_when   = inner_array[1]
      padded_index = ("#{index})").rjust(4,' ')
      e simp(padded_index)+' '+sfancy((name_of_the_program+':').ljust(30))+
        'Last updated: '+orange(last_updated_when)
    }
  end

  # ========================================================================= #
  # === do_autocompile
  # ========================================================================= #
  def do_autocompile
    require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
    e
    e 'The method '+orange('do_autocompile()')+' will attempt to '\
      'compile several programs.'
    e
    e 'The target directory is currently hardcoded towards `'+sdir('/Programs/')+'`.'
    e 'This will change at a later time, to allow any directory as the main'
    e '--prefix= in use.'
    e 
    e 'This method will first check the locally available latest version of'
    e 'the programs that are to be compiled.'
    e
    e 'Compilation will only happen if the program in the Cookbooks directory'
    e 'is more recent than the locally available program.'
    e
    array = %i( sed awk )
    array.each {|this_program|
      version_in_the_cookbooks_file = RBT.return_version_of(this_program)
e 'TODO'
e 'The local version of '+this_program.to_s+' is: '+
   sfancy(version_in_the_cookbooks_file)
e 'We still have to compare this to the local version available on'
e 'this computer.'
    }
  end

  # ========================================================================= #
  # === remove_this_entry_from_the_database
  # ========================================================================= #
  def remove_this_entry_from_the_database(i)
    database_location = rbt_logs?+'database_storing_compile_times.yml'
    if File.exist? database_location
      old_dataset = YAML.load_file(database_location)
      old_dataset.reject! {|key, value|
        key == i
      }
      e 'Storing the dataset into the file '+sfile(database_location)+'.'
      write_what_into(
        YAML.dump(old_dataset), database_location
      )
    else
      opnn; e "No file exists at `#{sfile(database_location)}`."
    end
  end

  # ========================================================================= #
  # === display_colourized_comment
  #
  # Let class Colours handle the colour there, via eparse().
  # ========================================================================= #
  def display_colourized_comment(i)
    eparse(i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_main_loop if @run_the_main_loop
  end

  # ========================================================================= #
  # === return_compile_time
  # ========================================================================= #
  def return_compile_time
    RBT::Compile.new(
      nil, :dont_run_yet
    ).return_compile_time_statistics
  end

  # ========================================================================= #
  # === edit_this_file                                             (edit tag)
  #
  # This method can be used to quickly edit a .yml file from within the
  # compile-shell.
  # ========================================================================= #
  def edit_this_file(
      i = @all_arguments
    )
    if i.is_a? Array
      i.each {|entry| edit_this_file(entry) }
    else
      i = i.dup if i.frozen?
      i << '.yml' unless i.end_with? '.yml'
      unless i.include? '/'
        i.prepend RBT.individual_cookbooks_directory?
      end
      i.prepend('nano ')
      esystem i
    end
  end

  # ========================================================================= #
  # === show_compile_time
  #
  # To invoke this method from the commandline, do:
  #    RBT::Compile::Shell.new
  # ========================================================================= #
  def show_compile_time
    RBT::Compile.new(
      nil, :dont_run_yet
    ).show_compile_time_statistics
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Compile::Shell.new(ARGV)
end # compile_shell