#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileStrategies
#
# This class will simply report some statistics about the compile-strategies
# used. In order for this to work, we require the directory at
# RBT.directory_expanded_cookbooks? to be populated with the
# expanded cookbook dataset to all registered programs.
#
# Usage example:
#   RBT::CompileStrategies.new
# =========================================================================== #
# require 'rbt/statistics/compile_strategies.rb'
# RBT::CompileStrategies.new
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/n_programs_available.rb'

module RBT

class CompileStrategies < ::RBT::Base # === RBT::CompileStrategies.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(
      i
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @hash = {}
    @n_programs_are_assumed_to_be_run_via_makefiles = 0
    # ======================================================================= #
    # Remember if you need to know the amount of programs, within this
    # class the variable @n_programs_are_registered keeps track of this.
    # ======================================================================= #
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === show_introduction
  # ========================================================================= #
  def show_introduction
    opnn; e 'Next gathering some compile-statistics about the'
    opnn; e 'registered programs.'
  end

  # ========================================================================= #
  # === report_how_many_programs_are_registered_in_total
  # ========================================================================= #
  def report_how_many_programs_are_registered_in_total
    e "There are a total of #{springgreen(@n_programs_are_registered)} "\
      "programs registered."
  end

  # ========================================================================= #
  # === reset_the_main_hash
  # ========================================================================= #
  def reset_the_main_hash
    @hash.clear
  end

  # ========================================================================= #
  # === load_dataset_from_the_expanded_data
  # ========================================================================= #
  def load_dataset_from_the_expanded_data
    reset_the_main_hash
    _ = RBT.directory_expanded_cookbooks?
    all_files = Dir["#{_}*.yml"]
    all_files.each {|this_file|
      dataset_from_the_yaml_file = YAML.load_file(this_file)
      name_of_the_program = File.basename(this_file).sub(/\.yml$/,'')
      hash = {}
      hash[name_of_the_program] = dataset_from_the_yaml_file
      @hash.update(hash)
    }
    determine_how_many_programs_are_registered
  end

  # ========================================================================= #
  # === determine_how_many_programs_are_registered
  # ========================================================================= #
  def determine_how_many_programs_are_registered
    @n_programs_are_registered = @hash.keys.size
  end

  # ========================================================================= #
  # === n_programs_are_registered?
  # ========================================================================= #
  def n_programs_are_registered?
    @n_programs_are_registered
  end; alias n_total_programs? n_programs_are_registered? # === n_total_programs?

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    _ = @hash
    # ======================================================================= #
    # Seed the variable @n_programs_are_assumed_to_be_run_via_makefiles
    # with an initial value, which will lateron be decremented accordingly.
    # ======================================================================= #
    @n_programs_are_assumed_to_be_run_via_makefiles = @n_programs_are_registered
    cmake_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('cmake')
    }
    python_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('python')
    }
    meson_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('meson')
    }
    ruby_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('ruby')
    }
    perl_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('perl')
    }
    scons_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('scons')
    }
    waf_deps = _.select {|key, inner_value|
      array_required_deps_on = inner_value['required_deps_on']
      array_required_deps_on.include?('waf')
    }
    e
    report_how_many_programs_are_registered_in_total
    e
    # ======================================================================= #
    # === cmake
    # ======================================================================= #
    n_programs_depend_on('cmake', cmake_deps)
    # ======================================================================= #
    # === ruby
    # ======================================================================= #
    n_programs_depend_on('ruby', ruby_deps)
    # ======================================================================= #
    # === python
    # ======================================================================= #
    n_programs_depend_on('python', python_deps)
    # ======================================================================= #
    # === meson
    # ======================================================================= #
    n_programs_depend_on('meson', meson_deps)
    # ======================================================================= #
    # === perl
    # ======================================================================= #
    n_programs_depend_on('perl', perl_deps)
    # ======================================================================= #
    # === scons
    # ======================================================================= #
    n_programs_depend_on('scons', scons_deps)
    # ======================================================================= #
    # === waf
    # ======================================================================= #
    n_programs_depend_on('waf', waf_deps)
    e slateblue('---')
    e colour_on_the_left_hand_side(
      @n_programs_are_registered - @n_programs_are_assumed_to_be_run_via_makefiles
    )+' programs make use of any of the above ^^^ compile-time strategies.'
    e
    e "#{sfancy(@n_programs_are_assumed_to_be_run_via_makefiles.to_s)} programs"\
      " are assumed to be compiled through #{orange('make')} (and possibly"
    e '     '+lightsalmon('GNU autoconfigure')+'). Note that meson depends on python,'
    e '     so strictly speaking, the meson-dependencies should have'
    e '     to be added to the python-subsection.'
    e
  end

  # ========================================================================= #
  # === n_programs_depend_
  # ========================================================================= # 
  def n_programs_depend_on(
      a = 'cmake',
      b = cmake_deps
    )
    @n_programs_are_assumed_to_be_run_via_makefiles -= b.keys.size
    n_percent = (
      b.keys.size * 100.0 / @n_programs_are_registered
    ).round(1).to_s.rjust(4)+'%'
    # Next append the number:
    n_percent << ' ('+b.keys.size.to_s.rjust(4)+' / '+n_total_programs?.to_s+')'
    colourized_n_percent = mediumseagreen(n_percent)
    e "#{colour_on_the_left_hand_side(b.keys.size.to_s.rjust(3))} programs "\
      "depend on #{royalblue((a+'.').ljust(7))} #{colourized_n_percent}"
  end

  # ========================================================================= #
  # === colour_on_the_left_hand_side
  # ========================================================================= #
  def colour_on_the_left_hand_side(i)
    sfancy(i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    show_introduction
    load_dataset_from_the_expanded_data
    report
  end

  # ========================================================================= #
  # === RBT::CompileStrategies[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.compile_strategies?
# =========================================================================== #
def self.compile_strategies?
  RBT::CompileStrategies.new
end; self.instance_eval { alias report_the_compile_strategies compile_strategies? } # === RBT.report_the_compile_strategies

end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileStrategies.new(ARGV)
end # compile_strategies