#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/download.rb'
# =========================================================================== #
require 'rbt/colours/colours.rb'
require 'rbt/toplevel_methods/e.rb'
require 'rbt/toplevel_methods/esystem.rb'
require 'rbt/toplevel_methods/find_url_for.rb'
require 'rbt/toplevel_methods/misc.rb'

module RBT

  # ========================================================================= #
  # === RBT.download
  #
  # This method can be used to download a remote source archive, if the
  # argument is a symbol. Additionally direct URLs can also be given.
  #
  # The second argument is optional. If given then the method will attempt
  # to download only a specific version from the remote location. In
  # order for this to work, the method may first have to "probe" whether
  # the remote file exists. For this functionality, wget it used.
  #
  # In general the method depends on "wget", largely due to reasons of
  # simplicity.
  #
  # Invocation examples:
  #
  #   RBT.download :htop
  #   RBT.download :php
  #   RBT.download :libx11
  #
  # ========================================================================= #
  def self.download(
      i                         = ARGV,
      optional_use_this_version = nil
    )
    # ======================================================================= #
    # Treat all input as an Array.
    # ======================================================================= #
    i = [i].flatten.compact
    i.each {|remote_url|
      # ===================================================================== #
      # === Handle symbols next
      #
      # Symbols will be treated differently, as we can assume that a user
      # normally would not pass in a Symbol to the method. So if we see
      # a Symbol, then we can also handle this situation in a special
      # manner.
      # ===================================================================== #
      if remote_url.is_a? Symbol
        case remote_url
        when :pwd # This is special - we will take the dirname.
          remote_url = File.basename(Dir.pwd).to_s
          if Object.const_defined?(ProgramInformation) and
            (remote_url =~ /\d+/) # <- And if it has at the least one number.
            remote_url = ProgramInformation.return_short_name(remote_url)
          end
          remote_url = remote_url.delete('-').to_sym
        end
        remote_url = RBT.location_of?(remote_url) # <- This should return an Array.
        if remote_url.is_a? Array
          remote_url = remote_url.first
        end
      end
      # ===================================================================== #
      # Next, honour the situation where the user wants to specifically
      # download a specific version of a program.
      # ===================================================================== #
      if optional_use_this_version
        if optional_use_this_version.is_a? Hash
          if optional_use_this_version.has_key? :version
            optional_use_this_version = optional_use_this_version[:version]
          end
        end
        _ = optional_use_this_version
        # =================================================================== #
        # else it does not exist, so we try other archive formats. This
        # will be handled by the following Array.
        # =================================================================== #
        array_formats_to_check_for = %w(
          .tar.xz
          .tar.gz
          .tar.bz2
        )
        # =================================================================== #
        # The old URL will already be available in the variable remote_url.
        # =================================================================== #
        remote_url = RBT.return_remote_url_from_this_input(remote_url, optional_use_this_version)
        require 'wget'
        if Wget.check_if_remote_file_exists(remote_url)
          array_formats_to_check_for = [] # Clear it here.
        else
          # ================================================================= #
          # We reject the ones that we already have checked before:
          # ================================================================= #
          array_formats_to_check_for.reject! {|entry|
            remote_url.end_with? entry
          }
          unless array_formats_to_check_for.empty?
            array_formats_to_check_for.each {|inner_entry|
              if Wget.check_if_remote_file_exists(inner_entry)
                # =========================================================== #
                # The next check is not ideal, but it may suffice for now.
                # After all we only need at the least one remote URl that
                # can be used.
                # =========================================================== #
                remote_url = inner_entry
              end
            }
          end
        end
      end
      if remote_url.empty?
        e "The given #{crimson('URL')} was not found - thus, "\
          "no download can occur."
      else
        e "Next downloading from the remote URL `#{sfancy(remote_url)}`."
        use_this_command = "wget #{remote_url}"
        e use_this_command
        result = `#{use_this_command} 2>&1`
        if result.include? 'ERROR 404: Not Found'
          e "No archive/program was found at the remote url #{remote_url}."
        end
        if File.exist? File.basename(remote_url)
          e "Success! The file can be found at `"\
            "#{sfile(
              File.absolute_path(File.basename(remote_url))
            )}`."
        end
      end
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  # RBT.download(ARGV)
  # pp RBT.download :htop
  pp RBT.download(:glibc, '2.28')
end # rbt_download htop