#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/infer_build_system.rb'
# =========================================================================== #
require 'rbt/constants/file_and_directory_constants.rb'
require 'rbt/toplevel_methods/does_this_file_exist.rb'
require 'rbt/toplevel_methods/e.rb'

module RBT

  # ========================================================================= #
  # === RBT.infer_build_system
  #
  # This method will return a String, indicating which build system is
  # most likely to be used for a given program at hand.
  # ========================================================================= #
  def self.infer_build_system(
      i             = :infer_automatically,
      program_path  = nil,
      depends_on    = nil, # <- This optional argument can be an Array.
      optional_hash = {}
    )
    case i
    when nil, :default
      i = :infer_automatically
    end
    use_a_build_directory = false
    use_this_build_system = 'configure' # <- Default value.
    unless optional_hash.empty?
      if optional_hash.has_key? :use_a_build_directory
       if optional_hash[:use_a_build_directory] == true
         use_a_build_directory = true
       end
      end
    end
    case i
    # ======================================================================= #
    # Must query whether we will try to infer the build system automatically.
    # ======================================================================= #
    when :infer_automatically
      # ===================================================================== #
      # We will start with a check for configure first. Do note that
      # ideally the order here should reflect the order given in the
      # file called 'build_systems_priorities.yml'.
      # ===================================================================== #
      if does_this_file_exist?('configure', use_a_build_directory)
        use_this_build_system = 'configure'
      # ===================================================================== #
      # Check for a rubygem file:
      # ===================================================================== #
      elsif program_path and program_path.end_with?('.gem')
        use_this_build_system = 'rubygem'
      # ===================================================================== #
      # Check for ninja:
      # ===================================================================== #
      elsif depends_on and depends_on_ninja?(depends_on)
        use_this_build_system = 'ninja'
      # ===================================================================== #
      # === Handle cmake installation types next
      #
      # This ought to come quite early.
      # ===================================================================== #
      elsif does_this_file_exist?(FILE_CMAKE_LISTS, use_a_build_directory)
        if depends_on and depends_on_cmake?
          use_this_build_system = 'cmake'
        else
          use_this_build_system = 'cmake'
        end
      # ===================================================================== #
      # === Handle 'waf' installations next
      # ===================================================================== #
      elsif does_this_file_exist?(FILE_WAF, use_a_build_directory) and
           !meson_build_file_exists?
        if depends_on and depends_on_waf?
          use_this_build_system = 'waf'
        else
          use_this_build_system = 'waf'
        end
      # ===================================================================== #
      # === Handle programs based on scons next
      # ===================================================================== #
      elsif does_this_file_exist?(FILE_SCONS) and
           !meson_build_file_exists?
        if depends_on and depends_on_python?
          use_this_build_system = 'scons'
        else
          use_this_build_system = 'scons'
        end
      # ===================================================================== #
      # === Handle "setup.py" files next
      # ===================================================================== #
      elsif does_this_file_exist?(FILE_SETUP_PY) and
           !meson_build_file_exists?
        if depends_on and depends_on_python?
          use_this_build_system = 'python'
        else
          use_this_build_system = 'python'
        end
      # ===================================================================== #
      # === Handle meson build next                             (meson tag)
      #
      # However had, some programs may have both a configure-script
      # and meson files. In this case, we have to determine which
      # one is to be run, which is done within the following "elsif"
      # clause.
      # ===================================================================== #
      elsif meson_build_file_exists?(:default, use_a_build_directory) # In this case, handle meson.
        use_this_build_system = 'meson'
        if does_this_file_exist?('configure') and
           configure_has_a_higher_priority_than_meson?
          use_this_build_system = 'configure'
        end
      end
    else
      if does_this_file_exist?(FILE_SETUP_PY)
        use_this_build_system = 'python'
      elsif meson_build_file_exists?(:default, use_a_build_directory)
        use_this_build_system = 'meson'
      else # This is our default then.
        if i.is_a? Symbol
          e 'Unregistered symbol: '+i.to_s
        else
          use_this_build_system = i
        end
      end
    end
    use_this_build_system # Return our findings here.
  end; self.instance_eval { alias infer infer_build_system } # === RBT.infer

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.infer_build_system
end # infer_build_system