#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/is_included.rb'
# =========================================================================== #
require 'rbt/aliases/aliases.rb'

module RBT

  # ========================================================================= #
  # === RBT.is_included?
  #
  # This method can be used to query whether a given input is included
  # or whether it is not - that is, if the input may be a program that
  # is registered in the RBT project.
  #
  # By default we will also include aliases, but this can be disabled,
  # such as by using the Symbol :do_not_check_for_aliases.
  #
  # Remember that all aliases in use are stored in a yaml file. If you
  # want to add new aliases or change the existing ones, you may have
  # to modify that yaml file.
  #
  # In principle, we could also restrict this method here to only-aliases
  # but I found that the API reads differently than this intent.
  #
  # Usage examples:
  #
  #    RBT.is_included? 'abiword' # => true
  #    RBT.is_included? 'aclocal'
  # 
  # ========================================================================= #
  def self.is_included?(
      i,
      honour_aliases = true
    )
    if i
      i = i.join(' ').strip if i.is_a? Array
      i.delete!(':') if i.include? ':' # We ignore ':' tokens.
      keys = available_programs?
      case honour_aliases
      when :default # Restore the default state in this case.
        honour_aliases = true
      when :do_not_honour_aliases,
           :do_not_check_for_aliases,
           :do_not_use_aliases,
           :ignore_aliases,
           :user_disabled_this_setting_via_the_commandline
        honour_aliases = false
      end
      if honour_aliases
        aliases = RBT::Cookbooks::Aliases.new(i)
        all_aliases = aliases.dataset
        # =================================================================== #
        # Merge the keys together next, starting with the default programs.
        # ==================================================================== #
        keys << all_aliases.keys
      end
      keys.flatten!
      return keys.include?(i)
    else
      false
    end
  end; self.instance_eval { alias really_does_include? is_included? } # === Cookbooks.really_does_include?
       self.instance_eval { alias program_is_included? is_included? } # === Cookbooks.program_is_included?
       self.instance_eval { alias does_include?        is_included? } # === Cookbooks.does_include?

  # ========================================================================= #
  # === RBT.find_cookbook_alias_for
  #
  # Usage example:
  #   RBT.find_cookbook_alias_for 'perlx'
  # ========================================================================= #
  def self.find_cookbook_alias_for(i)
    RBT::Cookbooks::Aliases.new(i).value?
  end; self.instance_eval { alias alias?         find_cookbook_alias_for } # === RBT.find_cookbook_alias_for
       self.instance_eval { alias find_alias_for find_cookbook_alias_for } # === RBT.find_alias_for

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  require 'verbose_truth/autoinclude'
  e 'Is the program given by '+ARGV.first+' included? '+
    sfancy(VerboseTruth[RBT.program_is_included?(ARGV)])
end # program_is_included?
    # program_is_included? gtk