#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/source_base_directory.rb'
# =========================================================================== #
require 'rbt/toplevel_methods/configuration_directory.rb'
require 'rbt/toplevel_methods/convert_global_env.rb'
require 'rbt/toplevel_methods/save_file.rb'
require 'rbt/toplevel_methods/create_directory.rb'

module RBT

  # ========================================================================= #
  # === @source_base_directory
  #
  # This variable keeps track as to where all source archives are usually
  # kept.
  # ========================================================================= #
  @source_base_directory = '/Users/x/SRC/' # <- Hardcoded default value.

  # ========================================================================= #
  # === RBT.file_source_directory
  # ========================================================================= #
  def self.file_source_directory
    "#{RBT.configuration_directory?}source_directory.yml"
  end

  # ========================================================================= #
  # === RBT.source_base_directory?
  #
  # This is a simple query-method for the source directory in use (called
  # "source base directory").
  #
  # The default value to it is "/Users/x/SRC/" on my system - which may be
  # different on other computers, which is why this has to be settable
  # and modifiable by the user via the setter method called
  # RBT.set_source_base_directory().
  # ========================================================================= #
  def self.source_base_directory?
    @source_base_directory
  end; self.instance_eval { alias source_directory? source_base_directory? } # === RBT.source_directory?
       self.instance_eval { alias source_dir?       source_base_directory? } # === RBT.source_dir?
       self.instance_eval { alias src_dir?          source_base_directory? } # === RBT.src_dir?

  # ========================================================================= #
  # === RBT.set_source_base_directory
  #
  # Note that the environment variable called RBT_SOURCE_DIRECTORY can
  # be used to use another default value.
  # ========================================================================= #
  def self.set_source_base_directory(
      i = :default_but_consider_environment_variables # <- Preferred default. 
    )
    case i
    # ======================================================================= #
    # === :default_but_consider_environment_variables
    # ======================================================================= #
    when :default_but_consider_environment_variables
      # ===================================================================== #
      # Handle the environment-variable RBT_SOURCE_DIRECTORY first, as
      # it has a higher priority in this case.
      # ===================================================================== #
      if ENV.has_key? 'RBT_SOURCE_DIRECTORY'
        i = ENV['RBT_SOURCE_DIRECTORY'].to_s.dup
      else
        # =================================================================== #
        # else the user does not have this ENV-variable, so we use the
        # "regular" default.
        # =================================================================== #
        i = @source_base_directory
      end
    # ======================================================================= #
    # This is for the "regular" default. This one will always assign.
    # ======================================================================= #
    when :default, nil, ''
      i = @source_base_directory
    end
    i = i.dup if i.frozen?
    i = RBT.convert_global_env(i) if i.include? '$'
    i = @source_base_directory if i.include? '$' # <- Overrule missing environment variables.
    i << '/' unless i.end_with? '/'
    @source_base_directory = i
  end; self.instance_eval { alias set_source_directory set_source_base_directory } # === RBT.set_source_directory
       self.instance_eval { alias set_source_dir       set_source_base_directory } # === RBT.set_source_dir
       self.instance_eval { alias source_directory=    set_source_base_directory } # === RBT.source_directory=

  # ========================================================================= #
  # And set it at once to a default, for my home directory. This will
  # default to the path /Users/x/SRC/.
  # ========================================================================= #
  set_source_base_directory(:default)

  # ========================================================================= #
  # === RBT.permanently_set_source_directory
  #
  # Use this method if you wish to permanently modify the source directory,
  # that is, the directory where our archives will be stored.
  #
  # Invocation example from the commandline:
  #   rbt --set_source_dir=/opt
  #   rbt --permanently-set-base-directory-to=/opt
  # ========================================================================= #
  def self.permanently_set_source_directory(i)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    unless i.start_with? '/'
      i.prepend('/') # We require leading '/'.
    end
    # ======================================================================= #
    # Modify the yaml file next:
    # ======================================================================= #
    what = i
    into = RBT.file_source_directory
    RBT.write_what_into(what, into)
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        e "We will permanently set to a new source "\
          "directory next (#{sdir(i)})"
      end
    end
    opn; e "Setting the new source directory to point towards `#{sdir(what)}`."
    opn; e "Storing into the file `#{sfile(into)}` next."
    # ======================================================================= #
    # And also modify it on my home system, if IS_ROEBE is true. This will
    # go right into the configuration/ subirectory.
    # ======================================================================= #
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_RBT}yaml/configuration/"\
             "#{File.basename(into)}"
      opn; e "Will also store into the file `#{sfile(into)}`."
      RBT.write_what_into(what, into)
    end
    RBT.set_source_base_directory(i) # And also sync it back.
    unless File.directory? i
      opnn; e 'Since that directory does not exist, it will be created now.'
      mkdir(i) { :be_quiet }
    end
  end; self.instance_eval { alias permanently_set_path_to_this_source_directory permanently_set_source_directory } # === RBT.permanently_set_path_to_this_source_directory
       self.instance_eval { alias permanently_set_source_dir                    permanently_set_source_directory } # === RBT.permanently_set_source_dir
       self.instance_eval { alias permanently_set_source_dir=                   permanently_set_source_directory } # === RBT.permanently_set_source_dir=
       self.instance_eval { alias permanently_set_source_directory=             permanently_set_source_directory } # === RBT.permanently_set_source_directory=

end

if __FILE__ == $PROGRAM_NAME
  puts RBT.source_dir?
end