#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::AllUrls
#
# This class will simply give us back the main URL of every program
# that is registered in the Cookbooks project.
#
# You can also trigger this via the commandline, by issuing:
#
#   cookbooks --all-urls
#
# Usage examples:
#   RBT::Cookbooks::AllUrls.new
# =========================================================================== #
# require 'rbt/utility_scripts/all_urls.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/find_url_for.rb'
begin
  require 'xorg_buffer'
rescue LoadError; end

module RBT

module Cookbooks

class AllUrls < Base # === Cookbooks::AllUrls.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize  
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    if commandline_arguments
      set_commandline_arguments(commandline_arguments)
    end
    run if run_already
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = ARGV)
    i = i.join(' ').strip if i.is_a? Array
    @commandline_arguments = i # Will be kept as String.
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @xorg_buffer = nil
    @array = []
    @use_opn = true # Whether to call opnn() or whether we don't.
    @namespace = NAMESPACE
    @also_show_url2 = true # <- Whether to try to show url2, too.
    set_commandline_arguments(nil) # Initialize @commandline_arguments here.
  end

  # ========================================================================= #
  # === show_url2?
  # ========================================================================= #
  def show_url2?
    @also_show_url2
  end

  # ========================================================================= #
  # === use_opn?
  # ========================================================================= #
  def use_opn?
    @use_opn
  end

  # ========================================================================= #
  # === commandline?
  # ========================================================================= #
  def commandline?
    @commandline_arguments
  end

  # ========================================================================= #
  # === display_start_message_to_the_user
  # ========================================================================= #
  def display_start_message_to_the_user
    opnn; e 'We will gather all URLs next - this may take a '\
            'short while. Please remain patient.'
  end

  # ========================================================================= #
  # === check_for_commandline_arguments
  #
  # We will work on a String - no need to care for Symbols here.
  # ========================================================================= #
  def check_for_commandline_arguments
    case commandline?.to_s # case tag
    # ======================================================================= #
    # === all_urls --no-opn
    # ======================================================================= #
    when /-?-?no(_|-)?opn/,'disable_opn'
      no_opn
    # ======================================================================= #
    # === all_urls --help
    # ======================================================================= #
    when 'HELP',/^-?-?help/
      show_help
    # when 'MISSING','--missing','missing'
    #   @show_only_missing_programs = true
    end
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Use '+simp('--missing')+' if you wish to display only the programs'
    e 'without an URL entry.'
    e
  end

  # ========================================================================= #
  # === data?
  # ========================================================================= #
  def data?
    @array
  end; alias result? data? # === result?

  # ========================================================================= #
  # === colourize_this_link
  # ========================================================================= #
  def colourize_this_link(i)
    palevioletred(i)
  end

  # ========================================================================= #
  # === save_result
  # ========================================================================= #
  def save_result
    if @xorg_buffer
      what = @xorg_buffer
      into = "#{temp_dir?}All_Urls.md"
      opnn; e "We will store this result in the file `#{sfile(into)}`."
      write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === report_result
  #
  # Here we can report the result, which is usually what we stored in
  # our main array called @array.
  # ========================================================================= #
  def report_result(
      i = data?
    )
    index_counter = 0
    buffer_string = ''.dup # This will become the Xorg buffer-string.
    pad_right = ' ' * 41
    # ======================================================================= #
    # Next we iterate over our result.
    # ======================================================================= #
    i.each {|name_of_the_program, remote_url1, remote_url2|
      string_to_display = ''.dup
      name_of_the_program = name_of_the_program.strip
      increase_index_counter = true
      remote_url1 = remote_url1.to_s.strip
      index_counter += 1 if increase_index_counter
      colourized_counter = index_counter.to_s.rjust(7)
      colourized_counter = slateblue(index_counter)
      string_to_display << ('('+colourized_counter+') ').rjust(15)
      # ===================================================================== #
      # Build the return String next.
      # ===================================================================== #
      remote_url1 = colourize_this_link(remote_url1)
      string_to_display = ''.dup
      string_to_display << name_of_the_program.ljust(30)+' -> '
      string_to_display << remote_url1.strip.ljust(40)+N
      buffer_string << string_to_display
      if show_url2?
        remote_url2 = pad_right+remote_url2.strip
        buffer_string << string_to_display
        string_to_display << colourize_this_link(
          remote_url2
        )+N
        buffer_string << N
      end
      buffer_string << string_to_display
      e string_to_display
    }
    # ======================================================================= #
    # Also assign this to the xorg buffer.
    # ======================================================================= #
    set_the_xorg_buffer_next(buffer_string)
  end

  # ========================================================================= #
  # === set_the_xorg_buffer_next
  # ========================================================================= #
  def set_the_xorg_buffer_next(i = '')
    if i.empty?
      opnn; e 'We can not set the xorg buffer as the input is empty.'
    else
      e; opnn; e 'We will set the xorgbuffer next with the above '\
                'programs as our result.'
      @xorg_buffer = i
      set_xorg_buffer(@xorg_buffer)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_for_commandline_arguments
    display_start_message_to_the_user
    all_programs = available_programs?
    all_programs.each {|program|
      result = RBT.find_url_for(program) # bl $COOKBOOKS/module_methods/module_methods.rb
      url1 = result.first.to_s.strip
      url2 = result.last.to_s.strip
      @array << [program, url1, url2]
    }
    report_result
    save_result
  end

end; end

  # ========================================================================= #
  # === RBT.all_urls
  #
  # This is simply a convenience method.
  # ========================================================================= #
  def self.all_urls(optional_commands = nil)
    RBT::Cookbooks::AllUrls.new(optional_commands)
  end
  
end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::AllUrls.new(ARGV)
end # all_urls