#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BuildDetector
#
# This class can help us detect the build type or configure type in use.
#
# The most important variable is @build_type. It can have these values:
#
#   :configure # default
#   :cmake
#   :meson
#   :waf
#   :scons
#
# This file will store the detected build type in the variable @build_type
# and also report it, via the report() method - see the end of the file
# here.
#
# Usage example:
#   RBT::BuildDetector.new
# =========================================================================== #
# require 'rbt/utility_scripts/build_detector.rb'
# =========================================================================== #
require 'rbt/base/prototype.rb'

module RBT

class BuildDetector < Base # === RBT::BuildDetector.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @build_type = :configure
    set_use_this_base_directory
  end

  # ========================================================================= #
  # === determine_whether_the_input_is_an_existing_directory
  #
  # By default, this class will use the current directory as the
  # input directory that is to be checked.
  # ========================================================================= #
  def determine_whether_the_input_is_an_existing_directory
    if File.exist? @input and File.directory? @input
      # ===================================================================== #
      # In this case, use that input as directory.
      # ===================================================================== #
      set_main_dir(@input)
    end
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = return_pwd if i.nil?
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === set_use_this_base_directory
  # ========================================================================= #
  def set_use_this_base_directory(
      i = return_pwd
    )
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    @use_this_base_directory = i
  end; alias set_main_dir set_use_this_base_directory # === set_main_dir

  # ========================================================================= #
  # === use_this_base_directory?
  # ========================================================================= #
  def use_this_base_directory?
    @use_this_base_directory
  end; alias base_dir? use_this_base_directory? # === base_dir?

  # ========================================================================= #
  # === is_cmake_build_type
  # ========================================================================= #
  def is_cmake_build_type
    set_build_type :cmake
  end

  # ========================================================================= #
  # === is_meson_build_type
  # ========================================================================= #
  def is_meson_build_type
    set_build_type :meson
  end

  # ========================================================================= #
  # === is_waf_build_type
  # ========================================================================= #
  def is_waf_build_type
    set_build_type :waf
  end

  # ========================================================================= #
  # === is_scons_build_type
  # ========================================================================= #
  def is_scons_build_type
    set_build_type :scons
  end

  # ========================================================================= #
  # === determine_build_type
  # ========================================================================= #
  def determine_build_type
    files_from_pwd = Dir[base_dir?+'*']
    files_from_pwd_but_only_their_filenames = files_from_pwd.map { |entry|
      File.basename(entry)
    }
    # ======================================================================= #
    # Check for cmake build types first
    # ======================================================================= #
    if files_from_pwd_but_only_their_filenames.include? 'CMakeLists.txt'
      is_cmake_build_type
    # ======================================================================= #
    # Next, check for meson build types
    # ======================================================================= #
    elsif files_from_pwd_but_only_their_filenames.include? 'meson.build'
      is_meson_build_type
    # ======================================================================= #
    # Next, check for waf-based build types
    # ======================================================================= #  
    elsif files_from_pwd_but_only_their_filenames.include? 'waf'
      is_waf_build_type
    # ======================================================================= #
    # Next, scheck for "scons"-based build types
    # ======================================================================= #
    elsif files_from_pwd_but_only_their_filenames.include?('SConscript') and
          files_from_pwd_but_only_their_filenames.include?('SConstruct')
      is_scons_build_type
    # ======================================================================= #
    # Next, GNU configure-based scripts
    # ======================================================================= #
    else # This is the default value.
      set_build_type
    end
  end

  # ========================================================================= #
  # === build_type?
  # ========================================================================= #
  def build_type?
    @build_type
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report
    # ======================================================================= #
    # The build type is stored in the variable called @build_type.
    # ======================================================================= #
    e "The guessed build type is `#{sfancy(@build_type)}`."
  end

  # ========================================================================= #
  # === set_build_type
  #
  # The @build_type variable will always be a Symbol.
  # ========================================================================= #
  def set_build_type(i = :configure)
    @build_type = i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_whether_the_input_is_an_existing_directory
    determine_build_type
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

  # ========================================================================= #
  # === RBT::BuildDetector.do_guess_build_type
  #
  # This method can be used to guess the build type of a target archive.
  # Prefer to use the method RBT.build_type? instead, though, since
  # it is more convenient.
  # ========================================================================= #
  def self.do_guess_build_type
    self.new.build_type?
  end; self.instance_eval { alias build_type? do_guess_build_type } # RBT::BuildDetector.build_type?

end

# =========================================================================== #
# === RBT.build_type?
#
# This should default to the current directory.
#
# The method will return a Symbol, such as :configure or :cmake.
#
# Usage examples:
#
#   RBT.build_type? # => :configure
#   RBT.build_type? # => :cmake
#
# =========================================================================== #
def self.build_type?
  RBT::BuildDetector.build_type?
end; self.instance_eval { alias guess_build_type        build_type? } # === RBT.guess_build_type
     self.instance_eval { alias do_guess_build_type     build_type? } # === RBT.do_guess_build_type
     self.instance_eval { alias do_guess_the_build_type build_type? } # === RBT.do_guess_the_build_type

end

if __FILE__ == $PROGRAM_NAME
  RBT::BuildDetector.new(ARGV).report
end # build_detector