#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::BuildDirectory
#
# This class handles everything related to a "build directory".
#
# What is a build directory?
#
# A build directory is a directory that may hold compilation data, object
# files, and so forth. Typically we may use such a directory in order
# to compile a given program at hand, from source.
#
# This class here is required to adhere to certain characteristics:
#
#   - It must be able to store the absolute path to the build directory
#     at hand at all times. The method .path? has to return a String
#     that shows us the full path to this directory.
#
# Usage example:
#   RBT::BuildDirectory.new
# =========================================================================== #
# require 'rbt/utility_scripts/build_directory.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class BuildDirectory < RBT::Base # === RBT::BuildDirectory.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_use_this_path(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @use_a_build_directory = true
    @be_verbose = false
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === use_a_build_directory?
  #
  # This method is required because the RBT::Compile class may not always
  # want to use a build directory.
  # ========================================================================= #
  def use_a_build_directory?
    @use_a_build_directory
  end; alias do_we_use_a_build_directory? use_a_build_directory? # === do_we_use_a_build_directory?

  # ========================================================================= #
  # === set_use_this_path                                           (set tag)
  #
  # Keep in mind that the build direcory can be nil too.
  #
  # The path is the full path towards the build directory.
  # ========================================================================= #
  def set_use_this_path(
      i = Dir.pwd
    )
    if i
      # ======================================================================= #
      # === Handle the case when the input is an Array
      # ======================================================================= #
      if i.is_a? Array
        i = i.join(' ').strip
      # ======================================================================= #
      # === Handle the case when the input is a Hash
      # ======================================================================= #
      elsif i.is_a? Hash
        if i.has_key? :path_to_build_directory
          i = i[:path_to_build_directory]
        end
      end
      unless i.include? '/'
        i = (Dir.pwd+'/'+i).squeeze('/')
      end
      unless i.end_with? '/'
        i = i.dup if i.frozen?
        i << '/' unless i.empty?
      end
    end # nil values are also valid here.
    @use_this_path = i
  end; alias use_this_build_directory= set_use_this_path # === use_this_build_directory?
       alias use_this_path=            set_use_this_path # === use_this_path=

  # ========================================================================= #
  # === do_change_directory                                          (cd tag)
  #
  # Use this method if you need to change directory.
  #
  # The change-directory action will be silent.
  # ========================================================================= #
  def do_change_directory(
      i          = @use_this_path,
      be_verbose = @be_verbose
    )
    case i
    when :be_verbose
      i = @use_this_path
      be_verbose = true
    end
    if block_given?
      yielded = yield
      case yielded
      when :ensure_that_the_directory_exists
        if i # But only for non-nil values.
          mkdir(i) unless File.directory? i
        end
      end
    end
    if i and File.directory?(i)
      if be_verbose
        e "Now entering the directory `#{sdir(i)}`."
      end
      chdir(i)
    else
      opnn; e "Can not change directory to non-existing "\
              "path at `#{sfile(i.to_s)}`."
    end
  end; alias change_directory do_change_directory # === change_directory
       alias cd               do_change_directory # === cd

  # ========================================================================= #
  # === name?
  #
  # This method will just return the name of the path.
  #
  # For example, if the path is "/abc/def/build_dir/" then this
  # method will return the string:
  #
  #   "build_dir"
  #
  # ========================================================================= #
  def name?
    File.basename(@use_this_path)
  end

  # ========================================================================= #
  # === use_this_path?
  #
  # Determine the path to the build directory at hand.
  # ========================================================================= #
  def use_this_path?
    @use_this_path
  end; alias path?      use_this_path? # === path
       alias full_path? use_this_path? # === full_path?

  # ========================================================================= #
  # === silently_make_sure_that_the_build_directory_exists
  # ========================================================================= #
  def silently_make_sure_that_the_build_directory_exists
    unless File.directory? @use_this_path
      mkdir(@use_this_path, :be_silent)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::BuildDirectory.new(ARGV)
end # build_directory