#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ChangePrefix
#
# This class can change the prefix of a specific file. The primary use
# case is to modify entries such as:
#
#    "PREFIX = /usr/local"
#
# that can be found in a Makefile.
#
# Do note that the class has been designed to be a bit more flexible than
# that task alone, though. While the primary use case is still to modify
# the PREFIX variable in a Makefile, ultimately the class could be used
# to modify just about any line in any text file. It is recommended to
# keep its primary use case confined to PREFIX-like variables in general,
# so that we can use this for the RBT project.
#
# The primary use case was to be able to modify such hardcoded PREFIX
# entries, in particular for projects that only ship with a Makefile
# and no other script.
#
# Usage example:
#   RBT::ChangePrefix.new
# =========================================================================== #
# require 'rbt/utility_scripts/change_prefix.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ChangePrefix < Base # === RBT::ChangePrefix.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    # ======================================================================= #
    # Designate which file is to be modified next:
    # ======================================================================= #
    set_modify_this_file(:default_file)
    # ======================================================================= #
    # Designate which pattern we are trying to find in that file.
    #
    # Unfortunately there does not seem to be a general consensus in
    # what is allowed and what is not. For example, the program called
    # 'mxk-1.10' makes use of exactly this line "prefix  = /usr/local",
    # whereas the program called 'libcli-1.9.8-4' makes use of exactly
    # this line "PREFIX = /usr/local". Thus, we will have to check for
    # these variants, which is why we will ask rubular for help here.
    #
    # See:
    #
    #   https://rubular.com/r/iAYqSQTaSllK5f
    #
    # ======================================================================= #
    @search_for_this_pattern = /^(PREFIX\s*=) ?(.+)/i
    # ======================================================================= #
    # Infer the name and program version from Dir.pwd, via ProgramInformation.
    # ======================================================================= #
    program_information = ProgramInformation.new(Dir.pwd)
    version = program_information.version?
    name    = program_information.program_name?
    @use_this_as_the_new_prefix = 
      rds(
        programs_dir?+
        name.capitalize+'/'+
        version+'/'
      )
  end

  # ========================================================================= #
  # === set_use_this_as_the_new_prefix
  #
  # This method can be used to designate another prefix that is to be
  # used - the one that will be written into the "Makefile".
  # ========================================================================= #
  def set_use_this_as_the_new_prefix(i)
    @use_this_as_the_new_prefix = i
  end

  # ========================================================================= #
  # === modify_this_file?
  # ========================================================================= #
  def modify_this_file?
    @modify_this_file
  end; alias modify_which_file? modify_this_file? # === modify_which_file?
       alias of_this_makefile   modify_this_file? # === of_this_makefile
       alias main_file?         modify_this_file? # === main_file?

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e
    e 'Presently this class has no help option documented.'
    e
  end

  # ========================================================================= #
  # === perform_the_modification
  #
  # This is the method that will be used to modify 
  # ========================================================================= #
  def perform_the_modification
    _ = main_file?
    if File.exist? _
      old_dataset = File.readlines(_)
      pattern = @search_for_this_pattern
      joined_dataset = old_dataset.join("\n")
      if joined_dataset =~ pattern # Must be a Regex.
        opnn; e 'A match was found within the file '+sfile(_)+'.'
        new_file_content = ''.dup
        old_dataset.each {|line|
          if line =~ pattern
            # =============================================================== #
            # Here we have to check for two cases:
            #
            #   1) prefix is downcased
            #   2) PREFIX is upcased
            # =============================================================== #
            if line.include? 'prefix '
              # ============================================================= #
              # Then use the downcased variant.
              # ============================================================= #
              line = 'prefix = '+@use_this_as_the_new_prefix
            elsif line.include? 'PREFIX='
              line = 'PREFIX = '+@use_this_as_the_new_prefix
            else
              # ============================================================= #
              # Else use the upcased variant.
              # ============================================================= #
              line = 'PREFIX = '+@use_this_as_the_new_prefix
            end
          end
          new_file_content << "#{line.rstrip}\n"
        }
        opnn; e "Storing into the file `#{sfile(_)}`."
        write_what_into(new_file_content, _)
      else
        opnn; e 'The pattern '+sfancy(pattern)+' could '\
                'not be found in the file '+sfile(_)+'.'
      end
    else
      opnn; no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === change_prefix --use-the-working-directory-as-input
      # ===================================================================== #
      when /^-?-?use(_|-)?the(_|-)?working(_|-)?directory(_|-)?as(_|-)?input$/i
        _ = File.basename(Dir.pwd)
        program_name    = ProgramInformation.return_program_name(_)
        program_version = ProgramInformation.return_program_version(_)
        new_prefix = programs_dir?+
                     program_name.capitalize+'/'+
                     program_version+'/'
        set_use_this_as_the_new_prefix(new_prefix)
      # ===================================================================== #
      # === change_prefix --help
      # ===================================================================== #
      when /^-?-?help$/i,
           /^-?-?show(_|-)?help$/i
        show_help
      else
        if i.is_a?(String) and File.exist?(i)
          # ================================================================= #
          # Use this existing file in that case.
          # ================================================================= #
          set_main_file(i)
        else
          # ================================================================= #
          # Try to find a Makefile in the current location.
          # ================================================================= #
          if File.exist?('Makefile') and !File.exist?(@modify_this_file)
            set_main_file(
              File.absolute_path(Dir.pwd)+'/Makefile'
            )
          end
        end
      end
    end
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    if i.is_a? Hash
      # ===================================================================== #
      # If a Hash is given then we will modify this Hash a bit before
      # continuing here.
      # ===================================================================== #
      if i.has_key? :use_this_makefile
        set_modify_this_file(
          i.delete(:use_this_makefile)
        )
      end
      # ===================================================================== #
      # Query for another prefix that is to be used next:
      # ===================================================================== #
      if i.has_key? :use_this_as_the_new_prefix
        new_value = i.delete(:use_this_as_the_new_prefix)
        set_use_this_as_the_new_prefix(new_value)
      end
    end
    i = [i].flatten.compact
    if i.empty?
      # ===================================================================== #
      # In that case we will attempt to use a Makefile in the current
      # directory.
      # ===================================================================== #
      i << 'Makefile'
    end
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === set_modify_this_file
  #
  # This method can be used to designate where the target "Makefile"
  # is exactly.
  #
  # Do note that set_main_file() is an alias to this method here.
  # ========================================================================= #
  def set_modify_this_file(i = :default)
    case i
    when :default,
         :default_file
      i = '/Depot/jjj/libcli-1.9.8-4/Makefile' 
    end
    i = i.dup if i.frozen?
    unless i.include? '/'
      i = File.absolute_path(i)
    end
    i = rds(i)
    @modify_this_file = i
  end; alias set_main_file set_modify_this_file # === set_main_file

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    perform_the_modification
  end

  # ========================================================================= #
  # === RBT::ChangePrefix[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ChangePrefix.new(ARGV)
end # change_prefix /Depot/jjj/libcli-1.9.8-4/Makefile
    # change_prefix /Depot/Temp/rbt/bzip2-1.0.7/Makefile --use-the-working-directory-as-input
    # change_prefix /Depot/Temp/rbt/bzip2-1.0.7/Makefile --use-this-prefix=/Programs/Bzip2/1.0.7