#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CheckForDuplicateBinaries
#
# This class can check for duplicate binaries in the given target
# directory at hand. By default, this will be the directory /usr/bin/,
# but you can also use --pwd or PWD to denote the current working
# directory as the base directory (see the --help) options.
#
# Every file in that directory will be checked, whether it is part
# of a registered program. If found a match, the class here will
# report that match, including as to which program it does match.
#
# Usage example:
#   RBT::CheckForDuplicateBinaries.new
# =========================================================================== #
# require 'rbt/utility_scripts/check_for_duplicate_binaries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/registered_binaries.rb'

module RBT

class CheckForDuplicateBinaries < RBT::Base # === RBT::CheckForDuplicateBinaries.new

  # ========================================================================= #
  # === DEFAULT_TARGET_DIRECTORY
  # ========================================================================= #
  DEFAULT_TARGET_DIRECTORY = '/usr/bin/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(commandline_arguments)
    menu
    run if run_already
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i)
    @commandline_arguments = [i].flatten.compact
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    use_default_target_directory # Set the default here.
  end

  # ========================================================================= #
  # === set_target_directory
  # ========================================================================= #
  def set_target_directory(
      i = DEFAULT_TARGET_DIRECTORY
    )
    i = i.join(' ').strip if i.is_a? Array
    case i
    when nil, ''
      i = DEFAULT_TARGET_DIRECTORY
    end
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @target_directory = i
  end; alias set_directory set_target_directory # === set_directory
       alias set_dir       set_target_directory # === set_dir

  # ========================================================================= #
  # === use_default_target_directory
  # ========================================================================= #
  def use_default_target_directory
    set_target_directory(DEFAULT_TARGET_DIRECTORY)
  end

  # ========================================================================= #
  # === set_pwd_as_directory
  # ========================================================================= #
  def set_pwd_as_directory
    set_target_directory(Dir.pwd)
  end

  # ========================================================================= #
  # === target_directory?
  # ========================================================================= #
  def target_directory?
    @target_directory
  end; alias target_dir? target_directory? # === target_dir?

  # ========================================================================= #
  # === check_on_the_files_of_the_target_directory
  # ========================================================================= #
  def check_on_the_files_of_the_target_directory
    all_binaries = RBT.all_binaries?
    array_all_binaries = RBT.registered_binaries?
    entries = Dir[target_dir?+'*']
    entries.each {|this_file|
      result = sfile(this_file.ljust(32))
      if array_all_binaries.include?(File.basename(this_file))
        program_name = all_binaries[File.basename(this_file)]
        e result+' is part of the program `'+
          saddlebrown(program_name)+'`.'
        # =================================================================== #
        # Next, we will also check whether there is a duplicate.
        # =================================================================== #
        target_for_binary_duplicate = programs_directory?+
                                      program_name.capitalize+'/'+
                                      'Current/bin/'+program_name
        if File.exist?(target_for_binary_duplicate)
          e (' ' * 31)+
            crimson('This appears to be a duplicate entry, for the file at')
          e (' ' * 31)+
            crimson('`')+sfancy(target_for_binary_duplicate)+crimson('` exists.')
        end
      end
    }
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i = i.join(' ').strip
    end
    if i.empty?
      use_default_target_directory
    else
      case i
      # ===================================================================== #
      # === check_for_duplicate_binaries --dir=/usr/local/bin/
      # ===================================================================== #
      when /--dir=(.+)/
        set_directory($1.to_s.dup)
      # ===================================================================== #
      # === check_for_duplicate_binaries --pwd
      # ===================================================================== #
      when /--pwd/,'PWD'
        set_pwd_as_directory
      else
        if File.directory? i
          set_dir(i)
        end
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_on_the_files_of_the_target_directory
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CheckForDuplicateBinaries.new(ARGV)
end # check_for_duplicate_binaries
    # check_for_duplicate_binaries --dir=/usr/local/bin/
    # check_for_duplicate_binaries --dir=/usr/bin/