#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForInclusion
#
# This script scans over all entries in $SRC and checks
# whether they are included in the cookbook or not.
# =========================================================================== #
# require 'rbt/utility_scripts/check_for_inclusion.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class CheckForInclusion < Base # === RBT::Cookbooks::CheckForInclusion

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === ARRAY_WITH_EXCEPTIONS
  #
  # The following Array has a list of exceptions.
  # ========================================================================= #
  ARRAY_WITH_EXCEPTIONS = %w(
    /Users/x/SRC/firmware/
    /Users/x/SRC/hardware/
    /Users/x/SRC/nvidia/
    /Users/x/SRC/gobolinux/
    /Users/x/SRC/windows/
    /Users/x/SRC/wlan/
    /Users/x/SRC/flash/
    /Users/x/SRC/fonts/
    /Users/x/SRC/ruby/gems/
  )

  # ========================================================================= #
  # === DEFAULT_FILE
  #
  # Where to store our results. A .md markdown file may seem appropriate.
  # ========================================================================= #
  DEFAULT_FILE = "#{RBT.depot_directory?}Information/UnidentifiedFiles.md"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @base_dir = source_directory?
    @array_files_which_were_not_found = []
    @all_files = []
    @be_verbose = true
    @query = RBT::Cookbooks::Cookbook.new { :bypass_menu_check }
  end

  # ========================================================================= #
  # === scan_for_all_files
  #
  # Scan for all files and fill up the array @all_files
  # ========================================================================= #
  def scan_for_all_files
    _ = @base_dir+'*/**'
    opnn; e 'Scanning for all files in directory `'+sdir(_)+'` now.'
    opnn; e 'Be patient, this may take a while.'
    @all_files = Dir[_]
  end

  # ========================================================================= #
  # === check_each_file_whether_it_is_included
  #
  # Here we report only if @be_verbose is true. (3)
  # ========================================================================= #
  def check_each_file_whether_it_is_included
    @all_files.each { |file_long_name, file_short_name|
      unless ARRAY_WITH_EXCEPTIONS.empty?
        dirname = File.dirname(file_long_name)+'/'
        next if ARRAY_WITH_EXCEPTIONS.include?(dirname)
      end
      if file_short_name.nil?
        file_short_name = ProgramInformation.return_real_name(
          File.basename(file_long_name)
        ).downcase
        opnn; e 'Now scanning for the program `'+sfile(file_short_name)+'`.'
      end
      # ===================================================================== #
      # If it is included then everything is fine.
      # ===================================================================== #
      if is_included?(file_short_name)
        # f '`'+file_long_name+'` is included.' if @be_verbose # Be silent here. 
      else # Else it is not included.
        if be_verbose?
          opnn; ewarn '`'+simp(file_long_name)+swarn('` is not included.')
        end
        @array_files_which_were_not_found << file_long_name
      end
    }
  end

  # ========================================================================= #
  # === is_included?
  #
  # Query whether the program is included or not.
  # ========================================================================= #
  def is_included?(i)
    i.delete!('_') if i.include? '_'
    is_included = @query.is_included?(i)
    return true if is_included
    false # default return value
  end

  # ========================================================================= #
  # === report_to_user_which_files_are_not_included
  # ========================================================================= #
  def report_to_user_which_files_are_not_included
    e sfancy('These files were not found:')
    cliner {
      @array_files_which_were_not_found.each {|entry|
        e '  '+entry
      }
    }
  end

  # ========================================================================= #
  # === store_results_to_file
  # ========================================================================= #
  def store_results_to_file(
      what     = @array_files_which_were_not_found,
      where_to = DEFAULT_FILE
    )
    remove_file(where_to) # Delete the old file first, before appending into a new one.
    @array_files_which_were_not_found.each { |f|
      append_what_into(f+N, where_to)
    }
    e 'Stored into:'
    e sfancy("  #{where_to}")
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    reset                                       # (1)
    scan_for_all_files                          # (2)
    check_each_file_whether_it_is_included      # (3)
    report_to_user_which_files_are_not_included # (4)
    store_results_to_file                       # (5)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # Run tag
    run_everything
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForInclusion.new
end # cinc