#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CheckForInvalidCurrentSymlinks
#
# This class will check for invalid Symlink entries and report these
# faulty entries.
#
# Usage example:
#   RBT::CheckForInvalidCurrentSymlinks.new
# =========================================================================== #
# require 'rbt/utility_scripts/check_for_invalid_current_symlinks.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CheckForInvalidCurrentSymlinks < Base # === RBT::CheckForInvalidCurrentSymlinks.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # We collect all non-symlinks into the following Array.
    # ======================================================================= #
    @array_these_entries_are_not_symlinks = []
    discover_which_entries_are_not_symlinks # Fill up our main Array first, always.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
    menu(i)
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === inform_the_user_what_can_be_done_about_incorrect_entries
  # ========================================================================= #
  def inform_the_user_what_can_be_done_about_incorrect_entries
    opnn; e 'You can correct this by passing '+
             orange('--autocorrect')+' to this class.'
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === check_for_invalid_current_symlinks --help
      # ===================================================================== #
      when /-?-?help/
        opnn; e 'Right now only --autocorrect exists.'
        opnn; e 'Alternatively, do not pass any argument to this class.'
        exit
      # ===================================================================== #
      # === check_for_invalid_current_symlinks --autocorrect
      # ===================================================================== #
      when /autocorrect/
        autocorrect_erroneous_entries
        exit
      end
    end
  end

  # ========================================================================= #
  # === main_array?
  # ========================================================================= #
  def main_array?
    @array_these_entries_are_not_symlinks
  end

  # ========================================================================= #
  # === autocorrect_erroneous_entries
  # ========================================================================= #
  def autocorrect_erroneous_entries
    _ = main_array?
    unless _.empty?
      _.each {|entry|
        opnn; e "Next autocorrecting `#{sfancy(entry)}`."
        # =================================================================== #
        # First, determine the possible targets. One of these targets will
        # become our new main symlink.
        # =================================================================== #
        dirname = File.dirname(entry)
        possible_targets = Dir[dirname+'/*'].select {|line|
          File.basename(line) =~ /^\d+/
        }.sort_by {|line|
          File.basename(line)
        }.reverse
        this_target = possible_targets.first
        # =================================================================== #
        # Then, remove that directory.
        # =================================================================== #
        if File.directory? entry
          remove_directory(entry)
        end
        # =================================================================== #
        # Finally, set up the new symlink there. The old_target is the
        # current program version of the program in question, such
        # as "/Programs/Htop/2.0.2".
        # =================================================================== #
        old_target = this_target
        new_target = entry # This is the entry, yup.
        if File.exist? old_target
          symlink(old_target, new_target)
        else
          opnn; no_target_exists_at(old_target)
        end
      }
    end
  end

  # ========================================================================= #
  # === show_which_entries_are_not_symlinks
  # ========================================================================= #
  def show_which_entries_are_not_symlinks
    unless main_array?.empty?
      main_array?.each {|entry|
        e 'Note that '+sfancy(entry.ljust(35))+' is '+sfancy('NOT')+
          ' a symlink.'
      }; e
    end
  end; alias report show_which_entries_are_not_symlinks # === report (report tag)

  # ========================================================================= #
  # === discover_which_entries_are_not_symlinks
  #
  # This will fill up the Array @array_these_entries_are_not_symlinks.
  #
  # Note that this method will NOT report anything - look at the report
  # method instead if you require output from this class.
  # ========================================================================= #
  def discover_which_entries_are_not_symlinks
    _ = programs_directory?
    array_all_programs = Dir["#{_}*"].sort
    array_all_programs.each {|entry|
      _ = rds(entry+'/Current')
      if File.exist? _
        is_it_a_symlink = File.symlink?(_)
        unless is_it_a_symlink
          @array_these_entries_are_not_symlinks << _
        end
      end
    }
  end; alias discover_stuff discover_which_entries_are_not_symlinks # === discover_stuff

  # ========================================================================= #
  # === notify_the_user_that_everything_appears_to_be_fine
  # ========================================================================= #
  def notify_the_user_that_everything_appears_to_be_fine
    opnn; e 'Everything appears to be fine. The checked '
    opnn; e 'directory was `'+sdir(programs_dir?)+'`'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    show_which_entries_are_not_symlinks
    if main_array?.empty?
      notify_the_user_that_everything_appears_to_be_fine
    else
      inform_the_user_what_can_be_done_about_incorrect_entries
    end
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CheckForInvalidCurrentSymlinks.new(ARGV)
end # check_for_invalid_current_symlinks --help