#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CleanupSystemTags
#
# This class can cleanup the /System/Tags/ hierarchy. Presently all it does
# is to remove stray (non-existing) symlinks.
#
# Usage example:
#   RBT::CleanupSystemTags.new
# =========================================================================== #
# require 'rbt/utility_scripts/cleanup_system_tags.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CleanupSystemTags < Base # === RBT::CleanupSystemTags.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === BASE_DIR
  # ========================================================================= #
  BASE_DIR = RBT.system_tags?

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    if block_given?
      yielded = yield
      case yielded
      when :be_silent,
           :be_quiet
        @be_verbose = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @all_entries = [] # <- Must be empty on startup.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === select_non_existing_symlinks_from_the_entries
  # ========================================================================= #
  def select_non_existing_symlinks_from_the_entries
    # ======================================================================= #
    # We must check for symlinks that exist, which we will do in the
    # following code:
    # ======================================================================= #
    @all_entries.select! {|entry|
      File.symlink?(entry) and !File.exist?(entry)
    }
  end

  # ========================================================================= #
  # === obtain_all_entries_from_the_system_tags_directory
  # ========================================================================= #
  def obtain_all_entries_from_the_system_tags_directory
    _ = BASE_DIR
    if File.directory? _
      @all_entries = Dir[_+'**/**']
    else
      if @be_verbose
        opnn; e 'No directory exists at '+sdir(_)+'.'
      end
    end
  end

  # ========================================================================= #
  # === consider_puring_and_reporting_non_existing_symlinks_from_the_system
  # ========================================================================= #
  def consider_puring_and_reporting_non_existing_symlinks_from_the_system
    if @all_entries.empty?
      if @be_verbose
        opnn; e "No #{royalblue('stray symlinks')} could be found. This "\
                "is good! #{gold('\\o/')}"
      end
    else
      if @be_verbose
        opnn; e 'The following stray (non-existing) symlinks will '\
                'be removed next:'
      end
      @all_entries.each {|this_non_existing_symlink|
        # =================================================================== #
        # The next line of code has to be explained. We will here
        # ALWAYS report when a stray symlink has been found. This
        # allows us to embed this class into RBT::Compile, and
        # notify the user when a stray symlink file is deleted.
        # =================================================================== #
        if File.symlink?(this_non_existing_symlink) # <- This check is not necessary but I like it.
          opnn; e 'Now removing the symlink `'+
                  sfancy(this_non_existing_symlink)+'`.'
          remove_this_symlink(this_non_existing_symlink)
        end
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_all_entries_from_the_system_tags_directory
    select_non_existing_symlinks_from_the_entries
    consider_puring_and_reporting_non_existing_symlinks_from_the_system
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::CleanupSystemTags.new(ARGV)
end # cleanup_system_tags