#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileBaseSystem
#
# Ths class will batch-compile a default linux base system. It can also
# report which programs would be used for this.
#
# Usage example:
#   RBT::CompileBaseSystem.new
# =========================================================================== #
# require 'rbt/utility_scripts/compile_base_system.rb'
# =========================================================================== #
require 'rbt/base/prototype.rb'
require 'rbt/compile/compile.rb'

module RBT

class CompileBaseSystem < Base # === RBT::CompileBaseSystem.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    case i
    when :dont_run_yet, :do_not_run_yet
      i = nil
      run_already = false
    end
    case run_already
    when :dont_run_yet, :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    # ======================================================================= #
    # Next comes the Array for the programs that will be compiled.
    # Ideally, the programs that are more important than the others,
    # should come first. All of these programs must be compiled
    # via an AppDir prefix.
    # ======================================================================= #
    @array_compile_these_programs = %i(
      make
      sed
      awk
      coreutils
      utillinux
      gawk
      m4
    )
    @compile = RBT::Compile.new(:do_not_run_yet)
  end

  # ========================================================================= #
  # === array_compile_these_programs?
  # ========================================================================= #
  def array_compile_these_programs?
    @array_compile_these_programs
  end; alias programs? array_compile_these_programs? # === programs?

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === do_compile_the_programs
  # ========================================================================= #
  def do_compile_the_programs
    @array_compile_these_programs.each {|this_program|
      opnn; e 'Now compiling the program `'+sfancy(this_program)+
              '` (via an AppDir prefix).'
      @compile.reset # Reset internally before continuing.
      @compile.do_compile_this_program(this_program) { :use_appdir_prefix }
    }
  end

  # ========================================================================= #
  # === notify_the_user_what_this_program_will_be_doing
  # ========================================================================= #
  def notify_the_user_what_this_program_will_be_doing
    opnn; e "This class will compile "\
            "#{sfancy(@array_compile_these_programs.size.to_s)}"\
            " programs."
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    notify_the_user_what_this_program_will_be_doing
    do_compile_the_programs
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

  # ========================================================================= #
  # === RBT::CompileBaseSystem.report_which_programs_will_be_compiled
  # ========================================================================= #
  def self.report_which_programs_will_be_compiled
    array = RBT::CompileBaseSystem.new(:do_not_run_yet).programs?
    _ = "#{array.size} programs"
    if RBT.use_colours?
      _ = Colours.sfancy(_)
    end
    RBT.e "The following #{_}, for a basic linux system, will be compiled:"
    RBT.e
    print '  '; RBT.e array.join('; ')
    RBT.e
  end; self.instance_eval { alias report_which_programs_may_be_compiled report_which_programs_will_be_compiled } # === report_which_programs_may_be_compiled

end; end

if __FILE__ == $PROGRAM_NAME
  # RBT::CompileBaseSystem.new(ARGV)
  RBT::CompileBaseSystem.report_which_programs_will_be_compiled
end # compile_base_system