#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CompileViaEnvironmentVariableAsPrefix
#
# This class can be used to compile a component via a specific environment
# variable as prefix. That way you can omit specifying any specific prefix
# on the commandline while still being able to group specific programs
# together, e. g. all components of XFCE to reside under
# /Programs/Xfce/4.12.0/.
#
# As I needed to have this specifically due to XFCE, but also others,
# next comes a list of XFCE-specific components:
# 
#   env_prefix libxfce4util             # xfce1
#   env_prefix xfconf                   # xfce2
#   env_prefix exo                      # xfce3
#   env_prefix garcon                   # xfce4
#   env_prefix libxfce4ui               # xfce5
#   env_prefix xfce4panel               # xfce6
#   env_prefix thunar                   # xfce7
#   env_prefix xfce4appfinder           # xfce8
#   env_prefix xfce4session             # xfce9
#   env_prefix xfdesktop                # xfce10
#   env_prefix xfceengine               # xfce11
#   env_prefix xfwm4                    # xfce12
#   env_prefix xfmedia                  # xfce13
#   env_prefix xfce4devtools            # xfce14
#   env_prefix xfce4settings            # xfce15
#   env_prefix xfce4cpufreqplugin       # xfce16
#   env_prefix xfce4panelprofiles       # xfce17
#   env_prefix xfce4screensaver         # xfce18
#   env_prefix xfce4notifyd             # xfce19
#   env_prefix xfce4screenshooter       # xfce20
#   env_prefix xfce4systemloadplugin    # xfce21
#   env_prefix xfce4whiskermenuplugin   # xfce22
#
# Usage example from within ruby code:
#
#   RBT::CompileViaEnvironmentVariableAsPrefix.new
#
# =========================================================================== #
# require 'rbt/utility_scripts/compile_via_environment_variable_as_prefix.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/compile/compile.rb'

module RBT

class CompileViaEnvironmentVariableAsPrefix < ::RBT::Base # === RBT::CompileViaEnvironmentVariableAsPrefix.new

  # ========================================================================= #
  # === initialize
  #
  # The second argument specifies which environment variable is
  # to be used.
  # ========================================================================= #
  def initialize(
      compile_these_programs           = nil,
      use_this_as_environment_variable = :default,
      run_already                      = true
    )
    reset
    set_compile_these_programs(
      compile_these_programs
    )
    set_use_this_as_environment_variable(
      use_this_as_environment_variable
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @this_environment_variable = nil
  end

  # ========================================================================= #
  # === set_use_this_as_environment_variable
  #
  # This method also honours a few symbols as shortcuts.
  # ========================================================================= #
  def set_use_this_as_environment_variable(
      i = :default
    )
    case i # case tag
    when :default,
         nil
      i = 'XFCE_PREFIX'
    when :mate,
         'MATE_DESKTOP',
         'MATE'
      i = 'MATE_DESKTOP_PREFIX'
    end
    @this_environment_variable = i
  end

  # ========================================================================= #
  # === set_compile_these_programs
  # ========================================================================= #
  def set_compile_these_programs(i = '')
    i = [i].flatten.compact
    @compile_these_programs = i
  end

  # ========================================================================= #
  # === compile_these_programs?
  # ========================================================================= #
  def compile_these_programs?
    @compile_these_programs
  end

  # ========================================================================= #
  # === setup_compile_object
  # ========================================================================= #
  def setup_compile_object
    @rbt_object = RBT::Compile.new(:do_not_run_yet)
    @rbt_object.use_this_environment_variable(
      @this_environment_variable
    )
  end

  # ========================================================================= #
  # === do_compile_these_programs
  # ========================================================================= #
  def do_compile_these_programs(
      i = @compile_these_programs
    )
    i.each {|this_program|
      @rbt_object.compile_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    setup_compile_object
    do_compile_these_programs(@compile_these_programs)
  end

  # ========================================================================= #
  # === RBT::CompileViaEnvironmentVariableAsPrefix[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.compile_via_environment_variable_as_prefix
# =========================================================================== #
def self.compile_via_environment_variable_as_prefix(
    compile_these_programs        = ARGV,
    use_this_environment_variable = :default
  )
  RBT::CompileViaEnvironmentVariableAsPrefix.new(
    compile_these_programs, use_this_environment_variable
  )
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::CompileViaEnvironmentVariableAsPrefix.new(ARGV[0], ARGV[1])
end # compile_via_environment_variable_as_prefix exo XFCE_PREFIX
    # compile_via_environment_variable_as_prefix xfconf
    # env_compile_via_prefix 