#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ConfigLogParser
#
# This class will scan through 'config.log' files. It will then report
# some of the problems, and may also be used to autofix some of these
# problems as well.
#
# Additionally, the class will also store useful information to the user,
# which can then be used to query, but also to feed into other files.
#
# The reason why this class was deemed necessary was because sometimes
# the error reporting by GNU autoconfigure is non-obvious and tedious
# to follow manually. So a class should help here programmatically.
#
# Usage example:
#   RBT::ConfigLogParser.new
# =========================================================================== #
# require 'rbt/utility_scripts/config_log_parser.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ConfigLogParser < Base # === RBT::ConfigLogParser

  # ========================================================================= #
  # === THIS_FILE
  # ========================================================================= #
  THIS_FILE = 'config.log'

  # ========================================================================= #
  # === CHROOT_TARGET
  # ========================================================================= #
  CHROOT_TARGET = '/Depot/Chroot/'

  # ========================================================================= #
  # === USR_INCLUDE
  # ========================================================================= #
  USR_INCLUDE = '/usr/include/'

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TRY_TO_AUTOCORRECT_HEADER_FILES
  #
  # If the following constant is set to true, then we will attempt to
  # copy missing .h files into the /Depot/Chroot/usr/include/ hierarchy.
  # ========================================================================= #
  TRY_TO_AUTOCORRECT_HEADER_FILES = true

  # ========================================================================= #
  # === RBT::ConfigLogParser::REGEX_FOR_MISSING_HEADER_FILES
  # ========================================================================= #
  REGEX_FOR_MISSING_HEADER_FILES =
    /error: (.+.h): No such file or directory/ # URL: http://rubular.com/r/5mn62sab1v

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_file   = ARGV,
      run_already = true
    )
    reset
    set_this_file(this_file)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @dataset = nil
    @missing_header_files = [] # Keep track of missing .h files.
    @missing_shared_libraries = []
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_this_file
  # ========================================================================= #
  def set_this_file(
      i = THIS_FILE
    )
    i = i.first if i.is_a? Array
    i = THIS_FILE if i.nil?
    case i
    when :do_not_use_any_input_file
      @this_file = nil
    else
      i = i.to_s.dup
    end
    @this_file = i
  end

  # ========================================================================= #
  # === this_file?
  # ========================================================================= #
  def this_file?
    @this_file
  end

  # ========================================================================= #
  # === scan_for_missing_header_files
  #
  # This method will scan for missing .h files.
  # ========================================================================= #
  def scan_for_missing_header_files
    set_missing_header_files @dataset.select {|line|
      # ===================================================================== #
      # The next line is valid for entries such as:
      #   include/stdio.h:164:10: fatal error: bits/stdio_lim.h: No such file or directory
      # ===================================================================== #
      line.include?('.h') and
      line.include?('fatal error:') and
      line.include?('No such file or directory')
    }
  end

  # ========================================================================= #
  # === scan_for_missing_shared_libraries
  # ========================================================================= #
  def scan_for_missing_shared_libraries
    _ = @dataset.select {|line|
      line.include?('error while loading shared libraries:')
    }
    @missing_shared_libraries = _
  end

  # ========================================================================= #
  # === set_missing_header_files
  # ========================================================================= #
  def set_missing_header_files(i)
    @missing_header_files = i
  end

  # ========================================================================= #
  # === missing_header_files?
  # ========================================================================= #
  def missing_header_files?
    @missing_header_files
  end

  # ========================================================================= #
  # === consider_copying_this_header_file
  # ========================================================================= #
  def consider_copying_this_header_file(
      i = missing_header_files?
    )
    if i.is_a? Array
      i.each {|entry| consider_copying_this_header_file(entry) }
    else
      if TRY_TO_AUTOCORRECT_HEADER_FILES
        regex = REGEX_FOR_MISSING_HEADER_FILES
        i =~ regex
        target = $1.to_s.dup
        from = "#{USR_INCLUDE}#{target}"
        to   = (CHROOT_TARGET+USR_INCLUDE+target).squeeze('/')
        unless File.exist? to
          if File.exist? from
            opnn; e "Trying to autocorrect this error next. "\
                    "(From: #{sfile(from)} | To: #{sfile(to)})"
            opnn; copy_file(from, to, :be_verbose)
          else
            opnn; e 'No file exists at `'+sfile(from)+'`.'
          end
        end
      end
    end
  end

  # ========================================================================= #
  # === report_results
  # ========================================================================= #
  def report_results
    # ======================================================================= #
    # First, check whether there are any missing header files.
    # ======================================================================= #
    unless @missing_header_files.empty?
      e 'These header files are missing:'; e
      @missing_header_files.each {|file|
        e "  #{sfile(file)}"
        consider_copying_this_header_file(file)
      }; e
    end
    unless @missing_shared_libraries.empty?
      e 'These shared libraries appear to be missing:'
      e
      @missing_shared_libraries.each {|file|
        e '  '+sfile(file)
        # consider_copying_this_header_file(file)
      }; e
    end
  end; alias report report_results # === report (report tag)

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = @this_file
    if _.nil?
      # ===================================================================== #
      # This, for now, can only happen if you input the instruction
      # :do_not_use_any_input_file is used. For now we do absolutely
      # nothing in this case.
      # ===================================================================== #
    else
      if File.exist? _ # The file is 'config.log'
        @dataset = File.readlines(_)
        # =================================================================== #
        # Ok, now we have the full dataset.
        # First, we scan for missing header files.
        # =================================================================== #
        scan_for_missing_header_files
        # =================================================================== #
        # Then we scan for missing shared libraries.
        # =================================================================== #
        scan_for_missing_shared_libraries
        scan_for_erroneous_assembler_messages
      else
        opnn; no_file_exists_at(_)
      end
      report_results
    end
  end

  # ========================================================================= #
  # === scan_for_erroneous_assembler_messages
  # ========================================================================= #
  def scan_for_erroneous_assembler_messages
    if @dataset.any? {|line| line.include? '.s: Assembler messages:' }
      opnn; e 'Some problem with Assembler-messages (binutils)'
    end
  end

  # ========================================================================= #
  # === RBT::ConfigLogParser[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ConfigLogParser.new(ARGV)
end # config_log_parser