#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreatePkgconfigFile
#
# This class can be used to generate a .pc file.
#
# Usage examples:
#
#   RBT::CreatePkgconfigFile.new
#
#   pc_file = RBT::CreatePkgconfigFile.new(
#     pkgconfig_filename: '/usr/lib/pkgconfig/mad.pc',
#     prefix: '/usr',
#     version: program_version?,
#     description: short_desc?
#   )
#
# =========================================================================== #
# require 'rbt/utility_scripts/create_pkgconfig_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreatePkgconfigFile < Base # === RBT::CreatePkgconfigFile.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input_hash(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @store_this_string  = ''.dup
    @prefix             = '/usr' # This is the default prefix.
    @program_version    = '1.0.0'
    @pkgconfig_filename = 'mad'
    @description        = ''
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === store_this_string?
  # ========================================================================= #
  def store_this_string?
    @store_this_string
  end; alias main_string? store_this_string? # === main_string?

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    @description
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    @prefix
  end

  # ========================================================================= #
  # === set_input_hash
  # ========================================================================= #
  def set_input_hash(i = {})
    i = i.first if i.is_a? Array
    @input_hash = i
  end

  # ========================================================================= #
  # === input_hash?
  # ========================================================================= #
  def input_hash?
    @input_hash
  end

  # ========================================================================= #
  # === pkgconfig_filename?
  # ========================================================================= #
  def pkgconfig_filename?
    @pkgconfig_filename
  end

  # ========================================================================= #
  # === program_name?
  #
  # This method is guaranteed to return the name without the .pc name.
  # ========================================================================= #
  def program_name?
    File.basename(pkgconfig_filename?).sub(/\.pc$/,'').dup
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i = '')
    @store_this_string << i+N
  end

  # ========================================================================= #
  # === set_pkgconfig_filename
  # ========================================================================= #
  def set_pkgconfig_filename(i)
    @pkgconfig_filename = rds(i.dup)
  end

  # ========================================================================= #
  # === parse_input_hash
  # ========================================================================= #
  def parse_input_hash
    _ = input_hash?
    if _.has_key? :pkgconfig_filename
      set_pkgconfig_filename(_.delete(:pkgconfig_filename))
    end
    if _.has_key? :prefix
      @prefix = _.delete(:prefix)
    end
    if _.has_key? :program_version
      set_program_version _.delete(:program_version)
    elsif _.has_key? :version
      set_program_version _.delete(:version)
    end
    if _.has_key? :description
      @description = _.delete(:description)
    end
  end

  # ========================================================================= #
  # === set_program_version
  # ========================================================================= #
  def set_program_version(i)
    @program_version = i
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @program_version
  end

  # ========================================================================= #
  # === build_up_the_main_string
  # ========================================================================= #
  def build_up_the_main_string
    add 'prefix='+prefix?.to_s # Need .to_s here since we can also pass class Prefix.
    add 'exec_prefix=${prefix}'
    add 'libdir=${exec_prefix}/lib'
    add 'includedir=${prefix}/include'
    add
    add 'Name: '+program_name?.dup
    add "Description: #{description?}"
    add 'Requires:'
    add 'Version: '+program_version?
    add 'Libs: -L${libdir} -l'+program_name?
    add 'Cflags: -I${includedir}'
  end

  # ========================================================================= #
  # === store_pc_file
  #
  # We must ensure that the pkgconfig/ directory also exists.
  # ========================================================================= #
  def store_pc_file
    what = main_string?
    into = pkgconfig_filename?.dup
    into << '.pc' unless into.end_with? '.pc'
    ensure_that_the_pkgconfig_directory_exists_for(into)
    opnn; e "Now storing into the file `#{sfile(into)}`."
    store_what_into(what, into)
  end

  # ========================================================================= #
  # === ensure_that_the_pkgconfig_directory_exists_for
  # ========================================================================= #
  def ensure_that_the_pkgconfig_directory_exists_for(i)
    i = File.dirname(i)
    mkdir(i, :be_quiet) unless File.directory? i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    parse_input_hash # Must parse the Hash prior to String build-up.
    build_up_the_main_string
    store_pc_file
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty? # For testing purposes.
   prefix_to_use = '/usr/'
   RBT::CreatePkgconfigFile.new(
     prefix: prefix_to_use,
     pkgconfig_filename: prefix_to_use+'/lib/pkgconfig/mad.pc',
     version: '1.0.3',
     description: 'bla bla bla'
   )
  else
    RBT::CreatePkgconfigFile.new(ARGV)
  end
end # create_pkgconfig_file