#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FeedbackInformation
#
# This class will create a web page. Before it will do so, it will
# report to the user.
#
# Usage example:
#   RBT::FeedbackInformation.new
# =========================================================================== #
# require 'rbt/feedback_information.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/toplevel_methods/find_url_for.rb'

module RBT

class FeedbackInformation < Base # === RBT::FeedbackInformation.new

  begin
    require 'cyberweb'
  rescue LoadError; end

  begin
    require 'html_tags'
  rescue LoadError; end

  # ========================================================================= #
  # === HTML_FILE
  # ========================================================================= #
  HTML_FILE = "#{RBT.temp_dir?}Information/URLs.html"

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #  
  DEFAULT_PROGRAM = 'php'

  # ========================================================================= #
  # === BR
  # ========================================================================= #
  if Object.const_defined?(:HtmlTags) and HtmlTags.respond_to? :br
    BR = HtmlTags.br
  else
    BR = '<br>'
  end

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(
      i = 'all'
    )
    # For now, if the input is an Array, we will just grab the first entry.
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    # ======================================================================= #
    # Convert to Array if first argument is of String type.
    # We assume they will be delineated by a ',' in this case though.
    # ======================================================================= #
    i = i.split(',') if i.is_a? String
    if i.is_a? Array
      i.map! {|entry|
        case entry
        when 'all','everything' # Shortcuts to assign ALL program names.
          entry = available_programs?
        end
        entry
      }
      i.flatten!
    end
    case i.to_s # And also for Strings.
    when 'all','everything' # Shortcuts to assign ALL program names.
      i = available_programs?
    end
    i = i.sort
    @input = i
  end

  # ========================================================================= #
  # === run_on_every_program
  #
  # This will walk over each program given and report the URL.
  #
  # Obviously, this may take a long time.
  # ========================================================================= #
  def run_on_every_program(
      this_program = DEFAULT_PROGRAM
    )
    this_program = DEFAULT_PROGRAM if this_program.nil?
    opnn; e 'Feedback information about these programs next: '
    e @input.join(',  ')
    hash_result = {}
    @input.each { |program|
      program = program.strip # remove whitespace first
      program.downcase!
      padded_program = ('`'+sfancy(program)+'`').ljust(38)
      opnn; e 'Fetching URL(s) for '+padded_program+' and '\
              'adding it to the main hash.'
      _ = feedback_url(program)
      hash_result[program] = _
    }
    create_html_page(hash_result) # We pass our hash to it.
  end

  # ========================================================================= #
  # === feedback_url
  #
  # Only report a match if we include it.
  # ========================================================================= #
  def feedback_url(i)
    RBT.find_url_for(i)
  end

  # ========================================================================= #
  # === string_s
  # ========================================================================= #
  def string_s(
      i = '', hash = {}
    )
    HtmlTags.span(i, hash)
  end

  # ========================================================================= #
  # === ensure_that_target_directory_exists
  # ========================================================================= #
  def ensure_that_target_directory_exists
    _ = File.dirname(HTML_FILE)
    mkdir(_) unless Dir.exist? _
  end

  # ========================================================================= #
  # === create_html_page                                           (html tag)
  #
  # This method will create a new HTML info page.
  #
  # It expects a hash as input.
  #
  # Pass a hash with arrays to create a HTML Info page.
  # 
  # Valid input i.e. the hash_result would be something like this:
  #
  #   {
  #     "zebra"=>"http://www.zebra.org/",
  #     "zee"=>"http://sourceforge.net/projects/zee/"
  #   }
  # ========================================================================= #  
  def create_html_page(hash_result)
    # ======================================================================= #
    # Start to create the HTML page next.
    # ======================================================================= #
    result = ''.dup
    result << Cyberweb.return_strict_doctype
    result << Cyberweb.return_html_start
    result << Cyberweb.return_head_start
    hash_result.each_pair { |key, url| # key is i.e. php
      basename = File.basename(key).
                 gsub(/\.gz/,'').gsub(/\.tar/,'').gsub(/\.bz2/,'')
      result << '<b style="font-size:2em;">'+basename+'</b>:<br>'
      result << '<p style="font-size:1.05em; padding-left:2.2em;text-indent:0.22em">'
      # Get description of the program in question next:
      # ===================================================================== #
      #
      # ===================================================================== #
      _ = RBT::Cookbooks::Cookbook.new(key) { :bypass_menu_check }
      description = _.desc?
      result << string_s(
        '<b style="color:darkgreen;text-decoration:underline">Description:</b><br><br>'
      )
      result << '</p>'
      begin
        result << '<p style="margin-top:5px; font-size:1.05em; padding-left:2.2em; text-indent:1.5em">'
        result << description.gsub(/\n/,'<br>').delete('"')
        result << '</p>'
      rescue ArgumentError
        opnn; e 'ArgumentError happened for `'+key+'`: byte-error in wrong encoding'
      end
      # ===================================================================== #
      # Next, we will build-up the URLs.
      # ===================================================================== #
      result << BR+string_s( '<b style="color:darkblue"># URL(s) for '+basename+': </b>' )
      result << BR
      if url.is_a? Array
        url = url.join
      end
      result << string_s('wget <a href="'+url+'">'+url+'</a>',
                  :css_style => 'margin-left:2em'
                )
      result << N << BR
      result << '</p>'
    }
    result << RBT.return_html_fin
    ensure_that_target_directory_exists
    # ======================================================================= #
    # Last but not least, write this into a html page
    # ======================================================================= #
    remove_file(HTML_FILE)
    write_what_into(result, HTML_FILE)
    opnn; e 'A HTML file was created at:'
    opnn; e sfile('  '+HTML_FILE)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_on_every_program
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::FeedbackInformation.new(ARGV)
end # finfo
    # finfo everything
    # finfo "readline, gettext, tcl, perl, xmlparser, python, apache2, libxml2, mysql, sqlite3, postgresql, zlib, libpng, m4, php"