#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FetchAllM4Macros
#
# This class will symlink all .m4 macros into /usr/share/aclocal.
#
# As argument, we should provide the path to the 'aclocal/' directory
# to this class.
#
# Basic usage example:
#
#   f4 /Programs/Lighttpd/Current/
#   f4 /Programs/Automake/1.15/
#   f4 /Programs/Automake/Current/
#   f4 /Programs/Flac/Current/
#   f4 $PROGRAMS/Xorgserver/Current/share/aclocal/
#   f4 /Programs/Xfce/Current
#
# =========================================================================== #
# require 'rbt/utility_scripts/fetch_all_m4_macros.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class FetchAllM4Macros < Base # RBT::FetchAllM4Macros.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_LOCATION
  # ========================================================================= #
  DEFAULT_LOCATION = '/Programs/Automake/1.12.5/'

  # ========================================================================= #
  # === DEFAULT_USR_SHARE_ACLOCAL_DIRECTORY
  # ========================================================================= #
  DEFAULT_USR_SHARE_ACLOCAL_DIRECTORY = '/usr/share/aclocal/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = DEFAULT_LOCATION,
      run_already = true
    )
    reset
    set_target(i)
    if block_given?
      yielded = yield
      case yielded
      when :do_not_report_anything_if_there_were_no_matches
        @report_no_matches = false
      end
    end
    case run_already
    when :dont_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    set_target
    @report_no_matches = true # If true then we will report to the user if no matches were found.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_target
  # ========================================================================= #
  def set_target(
      i = DEFAULT_LOCATION
    ) # as @target must be a directory, append a '/'
    i = i.first if i.is_a? Array
    i = Dir.pwd if i.nil?
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    @target = i # Will always have a trailing /.
    set_aclocal_target(@target)
  end

  # ========================================================================= #
  # === target?
  # ========================================================================= #
  def target?
    @target
  end; alias target target? # === target

  # ========================================================================= #
  # === set_aclocal_target
  # ========================================================================= #
  def set_aclocal_target(i = '')
    if i.is_a? String
      i = i.to_s.dup # Work on a copy.
    end
    # ======================================================================= #
    # We must ensure that we end with 'share/aclocal'. This is, however
    # had, not always wanted. Sometimes the user has given a real
    # directory that has .m4 files. So we must do a check prior to
    # that.
    # ======================================================================= #
    unless i.end_with? 'share/aclocal/'
      if Dir[rds("#{i}/*.m4")].empty?
        i << 'share/aclocal/'
      end 
    end
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # Next, delete all '-' that are part of the input, but only in some
    # cases, not all of them.
    #
    # For example, input such as
    # "/Programs/Xfce/4.12/share/xfce4/dev-tools/m4macros/"
    # must remain valid input.
    # ======================================================================= #
    if i.include?('-') and i.include?(programs_directory?)
      i.delete!('-') unless i.count('/') > 3
    end
    @aclocal_target = i
  end

  # ========================================================================= #
  # === all_macros
  #
  # This will return a list of all .m4 macros found.
  # ========================================================================= #
  def all_macros
    @all_m4_files
  end; alias m4_files? all_macros # === m4_files?

  # ========================================================================= #
  # === set_all_m4_files
  # ========================================================================= #
  def set_all_m4_files(i)
    i = [i] if i.is_a? String
    @all_m4_files = i
  end

  # ========================================================================= #
  # === symlink_all_m4_files_into_usr_share_aclocal
  #
  # Next, we will symlink all found files, into our target at
  # /usr/share/aclocal.
  # ========================================================================= #
  def symlink_all_m4_files_into_usr_share_aclocal
    @all_m4_files.each {|file|
      _ = DEFAULT_USR_SHARE_ACLOCAL_DIRECTORY+File.basename(file)
      opnn; e 'Working on `'+sfile(file)+'`.'
      if File.exist? _
        opnn; e 'Removing the file `'+sfile(_)+'` now '\
                'before we can symlink.'
        remove_file(_)
      end
      symlink(file, _)
    } unless m4_files?.empty?
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    _ = rds("#{@aclocal_target}*")
    matches = Dir[_]
    if matches.empty? and @report_no_matches
      opnn; e 'We found no matches from '+sfancy(_)
    else # Only if we found a match, will we continue here.
      # ===================================================================== #
      # longest_match = matches.max # Assume the longest match is good.
      # But why did we want to symlink only the longest match? I have
      # no idea.
      # ===================================================================== #
      set_all_m4_files(matches)
      symlink_all_m4_files_into_usr_share_aclocal
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::FetchAllM4Macros.new(ARGV, :dont_run_yet)
  _.run
end # fetchm