#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === RBT::FindAlternativeArchive
#
# This class can be used to find a likely alternative to a given
# (locally existing) archive.
#
# Usage example:
#   RBT::FindAlternativeArchive.new
# =========================================================================== #
# require 'rbt/utility_scripts/find_alternative_archive.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class FindAlternativeArchive < Base # === RBT::FindAlternativeArchive

  begin
    require 'determine_archive_type'
  rescue LoadError; end

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_ARGUMENT
  # ========================================================================= #
  DEFAULT_ARGUMENT =
    '/Users/x/SRC/glib/'\
    'glib-2.60.2.tar.xz' # '/Users/x/SRC/glib/glib-2.60.2.tar.xz'

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be the full, "expected" path to our archive
  # in question. We need this full path so that this class can replace
  # it with a proper new path, if necessary.
  # ========================================================================= #
  def initialize(
      commandline_arguments = DEFAULT_ARGUMENT,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    if i.empty?
      i << DEFAULT_ARGUMENT
    end
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === archive_type?
  # ========================================================================= #
  def archive_type? # Return the archive type here.
    return DetermineArchiveType[path?]
  end; alias archive_type archive_type? # === archive_type

  # ========================================================================= #
  # === file_path?
  #
  # This method will return the file path.
  # ========================================================================= #
  def file_path?
    @file_path
  end; alias path?    file_path? # === path?
       alias path     file_path? # === path
       alias results? file_path? # === results?
       alias result?  file_path? # === result?
       alias result   file_path? # === result

  # ========================================================================= #
  # === determine_file_path
  # ========================================================================= #
  def determine_file_path
    set_file_path(
      @commandline_arguments.first
    )
    determine_directory_name
  end

  # ========================================================================= #
  # === determine_directory_name
  # ========================================================================= #
  def determine_directory_name
    @directory_name = rds("#{File.dirname(@file_path)}/")
  end

  # ========================================================================= #
  # === set_file_path
  # ========================================================================= #
  def set_file_path(i)
    @file_path = i
  end

  # ========================================================================= #
  # === filename?
  # ========================================================================= #
  def filename?
    remove_file_suffix(
      @file_path
    )
  end

  # ========================================================================= #
  # === return_default_files
  #
  # Return the default files from the @directory_name entry. Note that we
  # will not traverse into subdirectories.
  # ========================================================================= #
  def return_default_files(from = "#{@directory_name}*")
    Dir[from]
  end

  # ========================================================================= #
  # === return_alternative_archives
  # ========================================================================= #
  def return_alternative_archives
    return_default_files.select {|entry|
      is_an_archive? entry
    }
  end; alias check_for_other_archive_types return_alternative_archives # === check_for_other_archive_types

  # ========================================================================= #
  # === find_other_archive
  #
  # The input to this method should be a version, as a number.
  #
  # The method will then try to find other archives.
  # ========================================================================= #
  def find_other_archive(i)
    i = i.to_f unless i.is_a? Float
    _ = return_alternative_archives.select {|entry| entry =~ /#{i}/ }
    return _.first unless _.empty?
    nil # else we will return nil.
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input? commandline_arguments? # === input?

  # ========================================================================= #
  # === has_alternative?
  # ========================================================================= #
  def has_alternative?
    return true unless return_alternative_archives.empty?
  end; alias more_than_one_entry? has_alternative? # === more_than_one_entry?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_file_path
  end

  # ========================================================================= #
  # === RBT::FindAlternativeArchive[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.find_alternative_archive
# =========================================================================== #
def self.find_alternative_archive(i = ARGV)
  RBT::FindAlternativeArchive.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/autoinclude'
  _ = RBT.find_alternative_archive(ARGV)
  puts '-' * 80
  e
  e "The #{sfancy('alternative archives')} may be:"
  e
  pp _.return_alternative_archives
  e
  e 'The filename may be: '+sfancy(_.filename?)
  e
  e 'Are there alternatives in that directory?'
  e
  e _.has_alternative?
  e
  # ========================================================================= #
  # Next, feedback all results:
  # ========================================================================= #
  puts '-' * 80
  e 'The path to the archive in question is at:'
  e '  '+_.path?.to_s
end # faa /Users/x/SRC/glib/glib-2.58.3.tar.xz
    # faa $MY_SRC/gdkpixbuf/gdk-pixbuf-2.26.5.tar.xz
    # faa $MY_SRC/gdkpixbuf/gdk-pixbuf
    # faa /Users/x/SRC/ruby/ruby-03.07.2017