#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateShellCompletion
#
# This class will generate a shell completion file.
#
# It defaults to bash for now, but it can also work for the zsh shell since
# as of October 2012. The latter is less-well tested, though.
#
# Usage example with my alias:
#
#    rbt_completions
#
# =========================================================================== #
# require 'rbt/utility_scripts/generate_shell_completion.rb'
# =========================================================================== #
begin
  require 'generate_shell_completion' # We depend on this, but we will still rescue it.
rescue LoadError; end
require 'rbt/base/base.rb'

module RBT

class GenerateShellCompletion < Base # === RBT::GenerateShellCompletion

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_COMPLETION
  # ========================================================================= #
  DEFAULT_COMPLETION = :bash

  # ========================================================================= #
  # === ARRAY_USE_THESE_ALIASES
  #
  # This constant specifies which aliases we will use, for completion
  # in a shell such as bash or zsh.
  # ========================================================================= #
  ARRAY_USE_THESE_ALIASES = YAML.load_file(
    ::RBT.project_yaml_directory?+'tab_completion_for_these_programs.yml'
  ).sort

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      generate_shell_completion_for_this_shell = DEFAULT_COMPLETION,
      run_already                              = true
    )
    reset
    set_generate_shell_completion_for_this_shell(
      generate_shell_completion_for_this_shell
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @generate_shell_completion_for_this_shell = nil
    # ======================================================================= #
    # Next, we will store into the directory "shell_completion/".
    # ======================================================================= #
    @use_this_as_base_directory = "#{rbt_log_dir?}shell_completion/"
    unless File.directory? @use_this_as_base_directory
     mkdir(@use_this_as_base_directory) # <- Create this directory too.
    end
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_generate_shell_completion_for_this_shell
  # ========================================================================= #
  def set_generate_shell_completion_for_this_shell(
      i = :all
    )
    case i
    when nil
      i = DEFAULT_COMPLETION
    when :all # <- This means to generate tab-completion for all shells.
      i = [:bash, :zsh, :fish]
    end
    i = [i].flatten.compact # <- Must always be an Array.
    @generate_shell_completion_for_this_shell = i
  end

  # ========================================================================= #
  # === add_all_cookbook_aliases
  # ========================================================================= #
  def add_all_cookbook_aliases
    RBT.aliases?
  end

  # ========================================================================= #
  # === generate_completion_for_bash                               (bash tag)
  #
  # This method will generate the completion file for the bash shell.
  # ========================================================================= #
  def generate_completion_for_bash
    do_generate_a_completion_file_for_this_shell(:bash)
  end

  # ========================================================================= #
  # === generate_completion_for_zsh                                (zsh tag)
  #
  # This method will generate the completion file for the zsh shell.
  # ========================================================================= #
  def generate_completion_for_zsh
    do_generate_a_completion_file_for_this_shell(:zsh)
  end

  # ========================================================================= #
  # === report_store_into_where
  # ========================================================================= #
  def report_store_into_where(i)
    opnn; e "Storing into #{sfile(i)}"
    opnn; e 'To source this file, do issue:'
    e
    e "  . #{sfile(i)}"
    e
  end

  # ========================================================================= #
  # === run_everything
  # ========================================================================= #
  def run_everything
    # ======================================================================= #
    # Bach-process the given input at hand:
    # ======================================================================= #
    @generate_shell_completion_for_this_shell.each {|this_shell|
      do_generate_a_completion_file_for_this_shell(this_shell)
    }
  end

  # ========================================================================= #
  # === do_generate_a_completion_file_for_this_shell
  #
  # This is the main method when we want to create a tab-completion file
  # for a specific shell, such as "bash" or "zsh" or "fish".
  # ========================================================================= #
  def do_generate_a_completion_file_for_this_shell(i)
    array = available_programs? # <- Obtain the available programs.
    # ======================================================================= #
    # Next, add aliases to that list.
    # ======================================================================= #
    array << add_all_cookbook_aliases
    array.flatten!
    _ = ::GenerateShellCompletion::GenerateShellCompletion.new(ARGV, :dont_run_yet)
    _.set_name_of_the_shell_function 'completion_for_ry'
    _.set_dataset(array)
    #_.remove_old_file
    # ======================================================================= #
    # === Determine the aliases
    #
    # Which aliases to use here.
    # ======================================================================= #
    _.set_aliases(ARRAY_USE_THESE_ALIASES)
    case i
    # ======================================================================= #
    # === Bash
    #
    # Generate the file for bash-completion.
    # ======================================================================= #
    when :bash
      _.store_here "#{@use_this_as_base_directory}bash_completion.sh"
    # ======================================================================= #
    # === Zsh                                                       (zsh tag)
    #
    # We can also generate the zsh-completion.
    # ======================================================================= #
    when :zsh
      _.store_here "#{@use_this_as_base_directory}zsh_completion.sh"
      _.prepend_this_string '
autoload bashcompinit
bashcompinit
'
    _.run
      # ===================================================================== #
      # _ = "compdef 'compadd "+all_programs.join(' ')+" ' ry"+N
      # ^^^ this can be used for zsh perhaps.
      # ===================================================================== #
    when :fish
      _.store_here "#{@use_this_as_base_directory}fish_completion.sh"
    end
    # ======================================================================= #
    # Notify the user what we will do.
    # ======================================================================= #
    opnn; e 'Now generating the shell script (completion file) '\
            '`'+lightgreen(i)+'`.'
    report_store_into_where(_.store_where?)
    _.run
    if is_on_roebe?
      # ===================================================================== #
      # On roebe, we will also generate more files. Note that storing into
      # the directory called /Users/x/DATA/PC/OS/LINUX/SHELL/SCRIPTS/ is
      # no longer an active setting.
      # ===================================================================== #
      case i
      when :bash
        # _.store_here "/Users/x/DATA/PC/OS/LINUX/SHELL/SCRIPTS/bash_completion.sh"
        # _.run
        _.store_here "/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/shell/bash_completion.sh"
        _.run
      when :zsh
        # _.store_here "/Users/x/DATA/PC/OS/LINUX/SHELL/SCRIPTS/zsh_completion.sh"
        # _.run
        _.store_here "/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/lib/rbt/shell/zsh_completion.sh"
        _.run
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_everything
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GenerateShellCompletion.new(:all)
end # gshell_completion