#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Homepage
#
# This class can show the homepage of a given program.
#
# It can also report all URLs that do not have a homepage
# entry, such as via:
#
#   homepage --missing-url?
#
# Usage example:
#   RBT::Cookbooks::Homepage.new
# =========================================================================== #
# require 'rbt/utility_scripts/homepage.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class Homepage < Base # === RBT::Cookbooks::Homepage.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS
  #
  # If the following constant is set to true then we will also try
  # to modify the xorg buffer.
  # ========================================================================= #
  MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program              = ARGV.first,
      optional_use_this_dataset = nil,
      run_already               = true
    )
    reset
    set_this_program(this_program)
    determine_the_dataset_to_use(optional_use_this_dataset)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @modify_the_xorg_buffer_if_a_homepage_entry_exists =
      MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS
  end

  # ========================================================================= #
  # === modify_the_xorg_buffer_if_a_homepage_entry_exists
  # ========================================================================= #
  def modify_the_xorg_buffer_if_a_homepage_entry_exists?
    @modify_the_xorg_buffer_if_a_homepage_entry_exists
  end; alias modify_the_xorg_buffer? modify_the_xorg_buffer_if_a_homepage_entry_exists? # === modify_the_xorg_buffer?

  # ========================================================================= #
  # === determine
  # ========================================================================= #
  def determine_the_dataset_to_use(i = nil)
    if i.nil?
      unless RBT::Cookbooks.const_defined? :Cookbook
        require 'rbt/cookbooks/class/class.rb'
      end
      @dataset = RBT::Cookbooks::Cookbook.new(which_program?) { :bypass_menu }
    else
      @dataset = i
    end
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    menu(i)
    @this_program = i
  end

  # ========================================================================= #
  # === which_program?
  # ========================================================================= #
  def which_program?
    @this_program
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === homepage --help
      # ===================================================================== #
      when /^-?-?help$/
        show_help
        exit
      # ===================================================================== #
      # === homepage --missing-url?
      # ===================================================================== #
      when /^-?-?missing(-|_)?url\??$/,
           /^-?-?missing$/
        show_all_programs_that_miss_a_homepage_entry
        exit
      end
    end
  end

  # ========================================================================= #
  # === show_all_programs_that_miss_a_homepage_entry
  #
  # This method will show all programs that miss a homepage entry.
  # ========================================================================= #
  def show_all_programs_that_miss_a_homepage_entry
    require 'rbt/cookbooks/class/class.rb'
    available_programs?.each {|this_program|
      dataset = RBT::Cookbooks::Cookbook.new(this_program) { :bypass_menu_check }
      homepage_entry = dataset.homepage?
      if homepage_entry.nil?
        _ = ("`#{this_program}`").ljust(15)
        opnn; e "The program #{sfancy(_)} has no "\
                "#{sfancy('homepage: ')} entry."
      end
    }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Invoke this entry point via:
  #   homepage --help
  # ========================================================================= #
  def show_help
    opnn; e 'Currently these options are available for `'+
             sfancy('class '+NAMESPACE)+'`:'
    e
    e '  --missing-url?      # show which cookbook entries lack '\
      'a proper '+sfancy('homepage: entry')
    e
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === return_homepage
  # ========================================================================= #
  def return_homepage
    dataset?.homepage?
  end

  # ========================================================================= #
  # === show_homepage_of_this_program
  # ========================================================================= #
  def show_homepage_of_this_program(
      this_program = which_program?
    )
    if this_program.include? '.yml'
      this_program.sub!(/\.yml$/,'')
    end
    _ = return_homepage
    if _
      if _.empty?
        opnn; e "No homepage has been registered for the program "\
                "`#{sfancy(this_program)}`." 
      else
        # =================================================================== #
        # Else we have found a homepage entry and will display
        # our findings to the end-user.
        #
        # To test this subsection, try:
        #
        #   homepage geany
        #
        # =================================================================== #
        opnn; e "The homepage of the program `#{sfancy(this_program)}"\
                "` is at:"
        e
        e "  #{orange(_)}"
        e
        if modify_the_xorg_buffer?
          set_xorg_buffer(_)
        end
      end
    else
      opnn; e 'No homepage entry is registered in `'+
               sfancy(this_program)+'` (The .yml file should be at '
      opnn; e  simp("  #{individual_cookbooks_dir?}#{this_program}.yml")+
               ')'
    end
  end; alias report_the_homepage show_homepage_of_this_program # === report_the_homepage

  # ========================================================================= #
  # === consider_opening_the_url_to_the_homepage_in_the_browser
  # ========================================================================= #
  def consider_opening_the_url_to_the_homepage_in_the_browser
    if is_on_roebe?
      open_in_browser(return_homepage)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_the_homepage
    consider_opening_the_url_to_the_homepage_in_the_browser
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::Homepage.new(ARGV)
end # homepage bluefish