#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::InstallGlibSchema
#
# This small class is just a helper script really. It allows us to install
# glib-schema files into the main directory at /usr/share/glib-2.0/schemas/
#
# This script should only be invoked for programs that are compiled in
# an AppDir like manner, such as with a prefix like /Programs/Htop/2.02.
#
# If a program is compiled with the /usr prefix, then this script is not
# needed at all, as the default locations should already work as-is.
#
# Usage example:
#   RBT::InstallGlibSchema.new
# =========================================================================== #
# require 'rbt/utility_scripts/install_glib_schema.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class InstallGlibSchema < Base # === RBT::InstallGlibSchema.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM_NAME
  # ========================================================================= #
  DEFAULT_PROGRAM_NAME = 'evince'

  # ========================================================================= #
  # === COPY_THE_SCHEMA_FILES_OR_SYMLINK_THE_SCHEMA_FILES
  #
  # This constant may have one of two allowed values:
  #
  #   :symlink
  #   :copy
  #
  # When :symlink is used then we will symlink the .xml files into the
  # main target directory at '/usr/share/glib-2.0/schemas/'.
  #
  # When :copy is used, then we will simply copy the .xml files into that
  # directory instead. The symlink-approach is probably cleaner, so this
  # is why I prefer it, since you instantly recognize which files are
  # just symlinks. If you'd copy it, you'd lose that information.
  # ========================================================================= #
  COPY_THE_SCHEMA_FILES_OR_SYMLINK_THE_SCHEMA_FILES = :symlink

  # ========================================================================= #
  # === RELATIVE_TARGET_TO_SCHEMA_FILES
  #
  # This String here can be used to find the relatve target directory
  # for schema files.
  # ========================================================================= #
  RELATIVE_TARGET_TO_SCHEMA_FILES =
    'share/glib-2.0/schemas/'

  # ========================================================================= #
  # === FHS_TARGET_DIRECTORY_FOR_GLIB
  #
  # This should point to the main directory for where the glib-schema
  # files are normally stored, on a FHS-conforming system.
  #
  # This is usually a directory such as '/usr/share/glib-2.0/schemas/'.
  # ========================================================================= #
  FHS_TARGET_DIRECTORY_FOR_GLIB =
    "/usr/#{RELATIVE_TARGET_TO_SCHEMA_FILES}"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    if block_given?
      yielded = yield
      case yielded
      when :do_symlink
        @copy_the_schema_files_or_symlink_the_schema_files = :symlink
      when :do_copy
        @copy_the_schema_files_or_symlink_the_schema_files = :copy
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @are_there_any_xml_files = false
    @copy_the_schema_files_or_symlink_the_schema_files = COPY_THE_SCHEMA_FILES_OR_SYMLINK_THE_SCHEMA_FILES
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(
      i = DEFAULT_PROGRAM_NAME
    )
    i = i.first if i.is_a? Array
    i = DEFAULT_PROGRAM_NAME if i.nil?
    i = i.to_s.dup
    if i.include? '/'
      i = File.basename(i)
    end
    i.capitalize!
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end; alias upcased_program_name? input? # === upcased_program_name?

  # ========================================================================= #
  # === run_glib_compile_schemas_command
  # ========================================================================= #
  def run_glib_compile_schemas_command
    _ = "glib-compile-schemas #{FHS_TARGET_DIRECTORY_FOR_GLIB}"
    esystem _
  end

  # ========================================================================= #
  # === appdir_schema_directory?
  # ========================================================================= #
  def appdir_schema_directory?
    "#{programs_dir?}#{upcased_program_name?}/Current/share/glib-2.0/schemas/"
  end

  # ========================================================================= #
  # === copy_or_symlink
  # ========================================================================= #
  def copy_or_symlink
    @copy_the_schema_files_or_symlink_the_schema_files.to_s
  end

  # ========================================================================= #
  # === symlink_mode?
  # ========================================================================= #
  def symlink_mode?
    copy_or_symlink == 'symlink'
  end

  # ========================================================================= #
  # === copy_or_symlink_this_file
  # ========================================================================= #
  def copy_or_symlink_this_file(from, to)
    case @copy_the_schema_files_or_symlink_the_schema_files
    when :symlink
      symlink_file(from, to)
    when :copy
      copy_file(from, to)
    end
  end

  # ========================================================================= #
  # === copy_or_symlink_the_schema_xml_files
  #
  # This method will copy the relative .gschema.xml files. We will also
  # symlink enums.xml files though, because that is required e. g. for
  # the program "epiphany".
  #
  # The optional argument is, as the name implies, optional. If it is
  # given and of the Symbol :be_verbose, then we will also notify
  # the user about what we will be doing next.
  # ========================================================================= #
  def copy_or_symlink_the_schema_xml_files(
      optional_argument = nil
    )
    target_dir = appdir_schema_directory?
    all_xml_files = Dir["#{target_dir}*.xml"]
    gscheme_file = target_dir+'gschema.dtd'
    # ======================================================================= #
    # The next line is, sort of, an exception for glib itself.
    # ======================================================================= #
    if File.exist? gscheme_file
      all_xml_files << gscheme_file
    end
    # ======================================================================= #
    # We only copy .xml files if they actually do happen to exist.
    # ======================================================================= #
    unless all_xml_files.empty?
      @are_there_any_xml_files = true
      if optional_argument
        case optional_argument
        when :be_verbose
          opnn; e "We will next #{sfancy(copy_or_symlink)} all .xml "\
                  "files from `#{sdir(target_dir)}`."
          opnn; e 'into `'+sdir(FHS_TARGET_DIRECTORY_FOR_GLIB)+'`.'
        end
      end
      # ===================================================================== #
      # Obtain all .xml files next.
      # ===================================================================== #
      all_xml_files.each {|this_schema_file|
        new_target_file = FHS_TARGET_DIRECTORY_FOR_GLIB+File.basename(this_schema_file)
        # =================================================================== #
        # Delete the target file if we are in "symlink mode".
        # =================================================================== #
        delete(new_target_file) if symlink_mode?
        copy_or_symlink_this_file(
          this_schema_file,
          new_target_file
        )
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    copy_or_symlink_the_schema_xml_files(:be_verbose)
    run_glib_compile_schemas_command if @are_there_any_xml_files
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::InstallGlibSchema.new(ARGV)
end # install_glib_schema