#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::LFS
#
# Purpose of this class:
# 
#   Run a Linux from Scratch, from zero to complete installation.
#
# Initially, we will be 100% compatible with the LFS book.
#
# Once we have successully run the LFS, we can deviate from it.
# =========================================================================== #
# The official URL for this starts here:
#
#   http://www.linuxfromscratch.org/lfs/view/development/chapter02/mounting.html
#   http://www.linuxfromscratch.org/lfs/view/development/chapter05/chapter05.html
#
# =========================================================================== #
# require 'rbt/utility_scripts/lfs/lfs.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'
require 'rbt/utility_scripts/lfs/constants.rb'
require 'rbt/utility_scripts/lfs/reset.rb'
require 'rbt/utility_scripts/lfs/run.rb'

module RBT

class LFS < Base # === RBT::LFS

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === symlink_tools_to_root
  #
  # The following is equivalent to:
  #
  #   ln -sv $LFS/tools /
  #
  # ========================================================================= #
  def symlink_tools_to_root(i = TOOLS_DIR) # ln -sv $MOLLI/tools /
    #symlink(TOOLS_DIR, '/')
    symlink_cmd = 'ln -sv '+i+' /'
    _ = '/'+File.basename(i)
    if File.exist? _
      e 'Can not symlink into '+Colours.col(_)+' as that one already exists.'
      e 'We remove it though.'
      remove(_)
      symlink_tools_to_root
    elsif File.exist?(TOOLS_DIR) and File.symlink?(TOOLS_DIR)
      e 'There already exists a symlink at '+gold(TOOLS_DIR)
    else
      e 'Now symlinking '+sfancy(i)+' to /'
      s symlink_cmd
    end
  end

  # ========================================================================= #
  # === create_tools_directory
  # ========================================================================= #
  def create_tools_directory
    create_directory TOOLS_DIR,'',true
  end

  # ========================================================================= #
  # === show_welcome_message
  # ========================================================================= #
  def show_welcome_message
    e 'Welcome to '+slateblue('Linux from Scratch')+'!'
    e 'This project will make use of the '+
      sfancy('RBT (Ruby Build Tools)')+'project'
    e 'to try and compile a smooth LFS for you from scratch.'
    e
    e 'For this, we will need to know a few things.'
    e
  end

  # ========================================================================= #
  # === enter_lfs_partition
  # ========================================================================= #
  def enter_lfs_partition(this_directory = MOUNT_DIR)
    change_directory this_directory
  end

  # ========================================================================= #
  # === consider_creating_mount_directory
  #
  # Consider creating the directory /Mount/lfs/.
  # ========================================================================= #
  def consider_creating_mount_directory(i = MOUNT_DIR)
    unless File.directory? i
      e "Anyway, let's get started and first "+
         sfancy('create a new directory.')
      e i
      mkdir i
      change_permission i
    else
      e 'Directory '+sdir(i)+' already exists, thus'
      e 'we will not create it.'
    end
  end

  # ========================================================================= #
  # === install_linux_kernel_headers
  #
  # This is used to install the Linux kernel headers.
  #
  # URL resource can be found at:
  #   http://www.linuxfromscratch.org/lfs/view/development/chapter05/linux-headers.html
  # ========================================================================= #
  def install_linux_kernel_headers
    esystem 'make mrproper'
    esystem 'make INSTALL_HDR_PATH=dest headers_install'
    esystem 'cp -rv dest/include/* /tools/include'
  end

  # ========================================================================= #
  # === create_bash_profile
  # ========================================================================= #
  def create_bash_profile # creates ~/.bash_profile
    # what = "exec env -i HOME=$HOME TERM=$TERM PS1='\u:\w\$ ' /bin/bash"
    # to = '~/.bash_profile'
    # append_what_to what, File.expand_path(to)
  end

  # ========================================================================= #
  # === prepare_user_then_login
  # ========================================================================= #
  def prepare_user_then_login(i = @user)
    s 'chown -v '+i+' '+TOOLS_DIR
    s 'chown -v '+i+' '+MOUNT_DIR+'sources'
    s 'chown -v '+i+' '+MOUNT_DIR+'temp'
    s 'su - '+i
  end

  # ========================================================================= #
  # === set_sticky_bit
  # ========================================================================= #
  def set_sticky_bit(i = @sources_dir) # Sets the sticky bit.
    begin
      File.chmod(01666, i)
    rescue Exception => error
      pp error
    end
  end

  # ========================================================================= #
  # === mount_lfs_partition
  # ========================================================================= #
  def mount_lfs_partition # mount tag.
    s 'export LFS='+MOUNT_DIR
    s 'mount -v '+LOCATION_OF_PARTITION+' '+MOUNT_DIR
  end

  # ========================================================================= #
  # === compile
  #
  # This will compile. If name contains a number, we split it up.
  # ========================================================================= #
  def compile(
      name, optional_version = nil
    )
    if name =~ /-\d+/
      name = name.split '-'
      optional_version = name[1]
      name = name[0]
    end # from here on everything is fine again.
    installer = RBT::Compile.new(name)
    installer.set_version optional_version if optional_version
    installer.run
  end

  # ========================================================================= #
  # === s
  #
  # Easier wrapper towards system. (system tag)
  # ========================================================================= #
  def s(i)
    esystem(i)
  end

  # ========================================================================= #
  # === create_all_directories
  # ========================================================================= #
  def create_all_directories
    e 'Next creating the '+sdir('source/')+' and '+sdir('tools/')+' directory.'
    create_source_directory
    create_temp_directory
    create_tools_directory # mkdir -v $MOLLI/tools
  end

  # ========================================================================= #
  # === create_source_directory
  # ========================================================================= #
  def create_source_directory
    create_directory @mounted_sources_directory,'',true
  end
  
  # ========================================================================= #
  # === create_temp_directory
  # ========================================================================= #
  def create_temp_directory
    create_directory @mounted_temp_directory,'',true
  end

  # ========================================================================= #
  # === add_user
  #
  # This method will add a new user, in this context the lfs-user.
  #
  # This is equivalent to this step:
  #
  #   http://www.linuxfromscratch.org/lfs/view/development/chapter04/addinguser.html
  #
  # ========================================================================= #
  def add_user(i = DEFAULT_USER)
    @user = i.to_s
    e 'Next adding a new '+slateblue('user')+' via groupadd/useradd:'
    s 'groupadd '+i.to_s
    s 'useradd -s /bin/bash -g '+i.to_s+' -m -k /dev/null '+i.to_s
  end

  # ========================================================================= #
  # === RBT::LFS.esystem
  # ========================================================================= #
  def self.esystem(i)
    ::RBT.esystem(i)
  end

  # ========================================================================= #
  # === RBT::LFS.mkdir
  # ========================================================================= #
  def self.mkdir(i)
    ::RBT.mkdir(i)
  end

  # ========================================================================= #
  # === RBT::LFS.populate_dev_device
  #
  # This method can be used to populate the linux /dev entries, as specified
  # in the LFS project.
  #
  # See the following link for an in-depth explanation:
  #
  #   http://www.linuxfromscratch.org/lfs/view/6.1/chapter06/devices.html
  #
  # ========================================================================= #
  def self.populate_dev_device
    esystem 'mknod -m 600 /dev/console c 5 1'
    esystem 'mknod -m 666 /dev/null c 1 3'
    esystem 'mount -n -t tmpfs none /dev'
    esystem 'mknod -m 622 /dev/console c 5 1'
    esystem 'mknod -m 666 /dev/null c 1 3'
    esystem 'mknod -m 666 /dev/zero c 1 5'
    esystem 'mknod -m 666 /dev/ptmx c 5 2'
    esystem 'mknod -m 666 /dev/tty c 5 0'
    esystem 'mknod -m 444 /dev/random c 1 8'
    esystem 'mknod -m 444 /dev/urandom c 1 9'
    esystem 'chown 0:tty /dev/{console,ptmx,tty}'
    esystem 'ln -s /proc/self/fd /dev/fd'
    esystem 'ln -s /proc/self/fd/0 /dev/stdin'
    esystem 'ln -s /proc/self/fd/1 /dev/stdout'
    esystem 'ln -s /proc/self/fd/2 /dev/stderr'
    esystem 'ln -s /proc/kcore /dev/core'
    mkdir '/dev/pts'
    mkdir '/dev/shm'
    esystem 'mount -t devpts -o gid=4,mode=620 none /dev/pts'
    esystem 'mount -t tmpfs none /dev/shm'
  end

end; end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty? # This is the default entry point.
    RBT::LFS.new
  else # lfsrb --populate-dev-devices
    RBT::LFS.populate_dev_device
  end
end # lfsrb