#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::NotFoundLibraries
#
# This class will scan through all .so files on the given machine
# and keep track of which programs are missing. Then it will
# report these on the commandline.
#
# This class will also store which binaries have msising .so files
# into a .yml file.
#
# Usage example:
#   RBT::NotFoundLibraries.new
# =========================================================================== #
# require 'rbt/utility_scripts/not_found_libraries.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class NotFoundLibraries < RBT::Base # === RBT::NotFoundLibraries.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === NOT_FOUND_STRING
  # ========================================================================= #
  NOT_FOUND_STRING = ' => not found'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    # ======================================================================= #
    # The next Hash is our main data-structure for this class.
    # ======================================================================= #
    @hash_these_entries_are_missing = {}
    @hash_these_entries_are_missing.default = []
    @search_through_these_directories = %w(
      /bin/
      /sbin/
      /usr/bin/
      /usr/local/bin/
      /System/Index/bin/
    )
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === determine_all_binaries_on_this_system
  #
  # Determine which .sp files exist on this system.
  # ========================================================================= #
  def determine_all_binaries_on_this_system
    @all_shared_object_libaries = []
    @search_through_these_directories.each {|this_directory|
      @all_shared_object_libaries << Dir["#{this_directory}*"]
    }
    @all_shared_object_libaries.flatten!
    @all_shared_object_libaries.uniq!
    @all_shared_object_libaries.select! {|entry| File.exist? entry }
  end

  # ========================================================================= #
  # === run_ldd_on_each_discovered_binary
  # ========================================================================= #
  def run_ldd_on_each_discovered_binary
    @all_shared_object_libaries.each {|this_file|
      result = `ldd #{this_file} 2>&1`
      if result.include? NOT_FOUND_STRING
        matches = result.split("\n").select {|inner_entry|
          inner_entry.include? NOT_FOUND_STRING
        }
        these_matches_are_available = matches.map {|most_inner_entry|
          most_inner_entry.sub(/\t/,'')
        }
        these_matches_are_available.each {|match|
          # ================================================================= #
          # Must eliminate a part of this:
          # ================================================================= #
          match.sub!(/ => not found/,'')
          unless @hash_these_entries_are_missing.has_key? match
            @hash_these_entries_are_missing[match] = []
          end
          @hash_these_entries_are_missing[match] << this_file
          @hash_these_entries_are_missing[match].uniq!
        } 
      end
    }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e '  --dirs? # show which directories were searched'
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === missing_libs --dirs?
      # ===================================================================== #
      when /^-?-?dirs\??$/
        show_the_directories_that_will_be_searched
        exit
      # ===================================================================== #
      # === missing_libs --help
      # ===================================================================== #
      when /^-?-?help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === show_the_directories_that_will_be_searched
  # ========================================================================= #
  def show_the_directories_that_will_be_searched
    e
    @search_through_these_directories.each {|this_directory|
      e sdir("  #{this_directory}")
    }
    e
  end

  # ========================================================================= #
  # === notify_the_user_what_will_be_done_next
  # ========================================================================= #
  def notify_the_user_what_will_be_done_next
    opnn; e 'This class will scan for all binaries on this system and'
    opnn; e 'determine whether any shared-libraries are missing.'
  end

  # ========================================================================= #
  # === hash_these_entries_are_missing?
  # ========================================================================= #
  def hash_these_entries_are_missing?
    @hash_these_entries_are_missing
  end; alias dataset? hash_these_entries_are_missing? # === dataset?

  # ========================================================================= #
  # === report_whether_entries_are_missing_and_which_ones_are_missing
  # ========================================================================= #
  def report_whether_entries_are_missing_and_which_ones_are_missing
    _ = @hash_these_entries_are_missing
    if _.empty?
      opnn; e 'No shared-object library appears to be missing. '\
              'This is good. '+gold('\o/')
      e
      opnn; e 'The following directories were searched:'
      show_the_directories_that_will_be_searched
    else
      opnn; e "These entries appear to be #{orange('missing')}:"
      _.each {|this_library_is_missing, array_these_binaries_have_problems|
        e 'The following binaries refer to this missing .so library: '+orange(
            File.basename(this_library_is_missing)
          )
        array_these_binaries_have_problems.each {|this_binary_has_a_problem|
          e royalblue('    '+this_binary_has_a_problem)
        }
      }
      save_the_hash_into_a_yaml_file
    end
  end; alias report report_whether_entries_are_missing_and_which_ones_are_missing # === report

  # ========================================================================= #
  # === save_the_hash_into_a_yaml_file
  # ========================================================================= #
  def save_the_hash_into_a_yaml_file
    what = YAML.dump(@hash_these_entries_are_missing)
    into = rbt_log_dir?+'missing_libraries.yml'
    opnn; e 'Also keeping a log of these files into `'+sfile(into)+'`.'
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    notify_the_user_what_will_be_done_next
    determine_all_binaries_on_this_system
    run_ldd_on_each_discovered_binary
    report_whether_entries_are_missing_and_which_ones_are_missing
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::NotFoundLibraries.new(ARGV)
end # not_found_libraries