#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PostInstall
#
# This class bundles together PostInstall actions.
# =========================================================================== #
# If you wish to require it in your project, use this require line:
#
#   require 'rbt/utility_scripts/post_install.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
# =========================================================================== #
# This class depends on class FixPkgconfigFile and class
# CreatePkgconfigFile, so we have to pull these two files in.
# =========================================================================== #
require 'rbt/utility_scripts/fix_pkgconfig_file.rb'
require 'rbt/utility_scripts/create_pkgconfig_file.rb'

module RBT

class PostInstall < Base # === RBT::PostInstall

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  #
  # The first argument must be our dataset, which shall be of class
  # Cookbooks::Cookbook.
  # ========================================================================= #
  def initialize(
      use_this_dataset = nil,
      run_already      = true
    )
    reset
    set_dataset(
      use_this_dataset
    )
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        if yielded.has_key? :use_this_compile_prefix
          set_compile_prefix(
            yielded.delete(:use_this_compile_prefix)
          )
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @compile_prefix = nil # This prefix is initially nil.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_compile_prefix
  #
  # The compile_prefix is the one that is used by RBT::Compile, e. g.
  # /usr/ prefix usually or an AppDir prefix. The reason why this class
  # here may need to know about it, is because some actions depend on
  # this - such as postinstall-action involving symlinks.
  # ========================================================================= #
  def set_compile_prefix(i)
    @compile_prefix = i
  end

  # ========================================================================= #
  # === compile_prefix?
  # ========================================================================= #
  def compile_prefix?
    @compile_prefix.to_s
  end

  # ========================================================================= #
  # === set_dataset
  # ========================================================================= #
  def set_dataset(
      i = nil
    )
    i = i.first if i.is_a? Array
    # ======================================================================= #
    # Next handle nils.
    # ======================================================================= #
    if i.nil?
      if Dir.pwd.include? programs_dir?
        # =================================================================== #
        # Ok, input can be such as "/Programs/Gcc/5.2.0". Let's work on that.
        # =================================================================== #
        i = Dir.pwd.split('/').reject(&:empty?)[1].downcase
      end
    end
    if i.is_a? String # postinstall gcc
      i = RBT::Cookbooks::Cookbook.new(i) { :bypass_menu_check }
    end
    @dataset = i
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === short_name?
  # ========================================================================= #
  def short_name?
    dataset?.short_name?
  end; alias program?            short_name? # === program?
       alias short_program_name? short_name? # === short_program_name?

  # ========================================================================= #
  # === short_desc?
  # ========================================================================= #
  def short_desc?
    dataset?.short_desc?
  end

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    dataset?.postinstall?
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    dataset?.program_version?
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    dataset?.program_name?
  end
 
  # ========================================================================= #
  # === create_pkgconfig_file
  #
  # This method will delegate towards creating a new .pc file.
  # ========================================================================= #
  def create_pkgconfig_file
    RBT::CreatePkgconfigFile.new(
      pkgconfig_filename: pkgconfig_filename?,
      prefix: prefix?,
      version: program_version?,
      description: short_desc?
    )
  end

  # ========================================================================= #
  # === pkgconfig_filename?
  # ========================================================================= #
  def pkgconfig_filename?
    program_name = short_program_name?
    if program_name.start_with? 'lib' # .pc files do not usually have the lib part.
      program_name.sub!(/^lib/,'')
    end
    _ = rds("#{prefix?}/lib/pkgconfig/#{program_name}")
    _ << '.pc' unless _.end_with? '.pc'
    _ # <- Return that result here.
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    dataset?.prefix?
  end

  # ========================================================================= #
  # === use_appdir_prefix?
  # ========================================================================= #
  def use_appdir_prefix?
    prefix?.include? programs_dir?
  end; alias appdir_prefix? use_appdir_prefix? # === use_appdir_prefix?
       alias is_appdir?     use_appdir_prefix? # === is_appdir?

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    show_header
    # ======================================================================= #
    # We will split on ',' entries.
    # ======================================================================= #
    dataset = postinstall?
    dataset = dataset.split(',') if dataset.is_a? String
    dataset.each { |post_install|
      # ===================================================================== #
      # Keep in mind that the variable post_install is now a String, ideally.
      # ===================================================================== #
      next if post_install.to_s.empty? # Skip empty entries.
      post_install = rds(post_install).strip
      opnn; e sfancy('Contemplating postinstall-action in `')+
              sdir_return_pwd+
              sfancy('` with')
      # ===================================================================== #
      # Use another colour next.
      # ===================================================================== #
      opnn; e lightblue("  #{post_install}")
      opnn; e sfancy('next.')
      # ===================================================================== #
      # Next, we have to split up the line based on ' ', so that we can
      # determine which action ought to be applied.
      # ===================================================================== #
      splitted = post_install.split(' ').map(&:strip)
      work_on_this_action = splitted.first
      first_argument      = splitted[1]
      second_argument     = splitted[2]
      third_argument      = splitted[3]
      fourth_argument     = splitted[4]
      splitted = convert_env_variable(splitted) # Get rid of all $.
      # ===================================================================== #
      # Next, we work on the commands given in the postinstall entry.
      # ===================================================================== #
      case work_on_this_action # case tag
      # ===================================================================== #
      # === symlink_binary
      # ===================================================================== #
      when /^symlink(_|-)?binary$/,
           /^symlink(_|-)?this(_|-)?binary$/
        _ = "#{prefix?}/bin/"
        symlink(_+splitted[1],_+splitted[2])
      # ===================================================================== #
      # === ln
      # ===================================================================== #
      when 'ln',
           'symlink' # run symlink here, but handle it internally.
        splitted.delete_at(1) if splitted[1] == '-s'
        splitted.delete_at(1) if splitted[1] == '-v'
        first_argument  = splitted[-1]
        second_argument = splitted[-2]
        if File.exist?(second_argument)
          if first_argument.include?(programs_directory?) and
            !appdir_prefix?
            opnn; e crimson('Can not symlink into ')+sfancy(first_argument)+crimson(' as this program')
            opnn; e crimson('is not compiled via an AppDir prefix.')
          else
            if @compile_prefix and !@compile_prefix.include?(programs_directory?)
              opnn; e crimson('Can not symlink into an AppDir path, as the prefix in use is')
              opnn; e crimson('`')+sfancy(@compile_prefix)+crimson('`.')
            else
              symlink(second_argument, first_argument)
            end
          end
        else
          opnn; e "Can not symlink towards #{sfancy(second_argument)}"\
                  " as it does not exist."
        end
      # ===================================================================== #
      # === copy_files
      # ===================================================================== #
      when 'copy_files'
        # =================================================================== #
        # This entry point can involve some more complicated instructions.
        # =================================================================== #
        if (first_argument == '{') and # In this case we assume the user wants to batch-copy some files.
           (third_argument == '}')
          target_dir = fourth_argument
          second_argument_splitted = second_argument.split(',')
          second_argument_splitted.each {|this_file|
            new_target = ("#{target_dir}/#{this_file}").squeeze('/')
            ftype = File.ftype(this_file)
            case ftype
            when 'directory'
              copy_directory(this_file, new_target, :be_verbose)
            when 'file'
              copy_file(this_file, new_target, :be_verbose)
            end
          }
        end
      # ===================================================================== #
      # === copy_file
      # ===================================================================== #
      when 'copy_file'
        copy_file(splitted[1], splitted[2])
      # ===================================================================== #
      # === create_directory
      # ===================================================================== #
      when 'create_directory'
        create_directory(splitted[1])
      # ===================================================================== #
      # === create_pkgconfig_file
      # ===================================================================== #
      when /^create(_|-)?pkgconfig(_|-)?file$/
        create_pkgconfig_file
      # ===================================================================== #
      # === patch_pkgconfig_file
      # ===================================================================== #
      when 'patch_pkgconfig_file:',/patch_?pkgconfig_?file/
        opnn; e 'Now patching the version in the pkg-config file '\
                'for that program.'
        new_version = splitted.last
        # =================================================================== #
        # Currently, the path is hardcoded.
        # =================================================================== #
        use_this_as_pkgconfig_file = Dir[
          '/usr/lib/pkgconfig/*'+program?+'*.pc'
        ].first
        # =================================================================== #
        # Now we can delegate to class RBT::FixPkgconfigFile.
        # =================================================================== #
        RBT::FixPkgconfigFile.new(
          {
            use_this_pkgconfig_file:     use_this_as_pkgconfig_file,
            use_this_as_the_new_version: new_version
          }
        )
      # ===================================================================== #
      # === add_user
      # ===================================================================== #
      when /^add_?user$/
        AddUser.new(splitted[1]) if Object.const_defined? :AddUser
      # when 'install' # Install actions are handled by the else clause below.
      else # else, simply run through esystem().
        cliner {
          esystem(post_install)
        }
      end
    }
    cliner
  end

  # ========================================================================= #
  # === show_header
  # ========================================================================= #
  def show_header
    cliner
    opnn; efancy 'Starting postinstall instructions next.'
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::PostInstall.new(ARGV)
end