#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::PostinstallationStep
#
# This class has bundled together the necessary postinstallation-steps. It
# is a standalone class so that the user can also invoke it from the
# commandline directly.
#
# Usage example:
#   RBT::PostinstallationStep.new
# =========================================================================== #
# require 'rbt/utility_scripts/postinstallation_step/postinstallation_step.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'

module RBT

class PostinstallationStep < CompileBase # === RBT::PostinstallationStep.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      hash        = {},
      run_already = true
    )
    reset
    set_internal_hash(
      hash
    )
    if block_given?
      yielded = yield
      case yielded
      when :do_not_run_yet
        run_already = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @internal_hash = {} # This Hash stores the internal dataset.
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    _ = @internal_hash
    if _.has_key? :dataset_to_use
      _ = _[:dataset_to_use]
    end
    _ = _.internal_hash?
    _[:yaml_dataset]['program_version']
  end

  # ========================================================================= #
  # === set_internal_hash
  # ========================================================================= #
  def set_internal_hash(i = '')
    if i.is_a? String
      i = { compile_which_program: i }
    end
    @internal_hash = i
    ensure_that_useful_internal_values_exist
  end

  # ========================================================================= #
  # === dataset_to_use?
  #
  # Which dataset is to be used - this reader method returns that
  # result.
  # ========================================================================= #
  def dataset_to_use?
    @internal_hash[:dataset_to_use]
  end; alias cookbook_dataset? dataset_to_use? # === cookbook_dataset?
       alias dataset?          dataset_to_use? # === dataset?

  # ========================================================================= #
  # === full_configure_command?
  #
  # The @full_configure_command contains the full configure command
  # that we will use. The advantage in using this is that we can
  # easily copy/paste this at a later time, should we need to do so.
  # ========================================================================= #
  def full_configure_command?
    @internal_hash[:full_configure_command]
  end; alias return_full_configure_command full_configure_command? # === return_full_configure_command

  # ========================================================================= #
  # === shall_we_invoke_symlink_program?
  # ========================================================================= #
  def shall_we_invoke_symlink_program?
    @internal_hash[:shall_we_invoke_symlink_program]
  end

  # ========================================================================= #
  # === compile_which_program?
  # ========================================================================= #
  def compile_which_program?
    @internal_hash[:compile_which_program]
  end

  # ========================================================================= #
  # === do_register_the_program?
  # ========================================================================= #
  def do_register_the_program?
    @internal_hash[:do_register_the_program]
  end

  # ========================================================================= #
  # === use_tags?
  # ========================================================================= #
  def use_tags?
    @internal_hash[:use_tags]
  end

  # ========================================================================= #
  # === delete_libtool_files?
  # ========================================================================= #
  def delete_libtool_files?
    @internal_hash[:delete_libtool_files]
  end

  # ========================================================================= #
  # === do_run_fetch_all_m4_macros?
  # ========================================================================= #
  def do_run_fetch_all_m4_macros?
    @internal_hash[:do_run_fetch_all_m4_macros]
  end

  # ========================================================================= #
  # === internal_hash?
  # ========================================================================= #
  def internal_hash?
    @internal_hash
  end

  # ========================================================================= #
  # === opne
  # ========================================================================= #
  def opne(i = '')
    super(i)
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    real_prefix?.to_s
  end

  # ========================================================================= #
  # === use_this_prefix?
  # ========================================================================= #
  def use_this_prefix?
    @internal_hash[:use_this_prefix]
  end; alias real_prefix?          use_this_prefix? # === real_prefix?
       alias return_proper_prefix? use_this_prefix? # === return_proper_prefix?
       alias return_proper_prefix  use_this_prefix? # === return_proper_prefix

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    prefix = real_prefix?
    # ======================================================================= #
    # Consider removing the libtool .la files next
    # ======================================================================= #
    if use_appdir_prefix?
      @internal_hash[:symlink_program].set_prefix(prefix?)
      # ===================================================================== #
      # First we must create a new 'Current' symlink:
      # ===================================================================== #
      create_a_new_current_symlink_here(real_prefix?.to_s)
      consider_removing_libtool_la_files # <- This will check whether we will remove .la files or not.
      consider_symlinking_cmake_modules
      if symlink_into_usr_lib_dir?
        RBT::SymlinkIntoUsrLibDirectory.new(
          rds(real_prefix?.to_s+'/lib/')
        )
      end
      run_post_install_class_unless_postinstall_is_empty
    end
    # ======================================================================= #
    # Eliminate stray symlinks from the /System/Tags/ directory, should
    # this directory exist.
    # ======================================================================= #
    cleanup_system_tag_directory
    # ======================================================================= #
    # Next, consider symlinking the .h files into the "/usr/include/"
    # hierarchy. This is, by default, NOT done; only some programs
    # may do so, and it will only happen for programs that are
    # installed via an AppDir way.
    # ======================================================================= #
    consider_symlinking_header_files_into_the_usr_include_hierarchy
    # ======================================================================= #
    # === Check as to whether we will instally only programs from the
    #     bin/ subdirectory. If so, we will delete the other files.
    # ======================================================================= #
    check_whether_we_will_install_only_programs_from_the_bin_subdirectory
    # ======================================================================= #
    # === The AppDir approach comes next
    #
    # If you need to tie actions related to the "/Programs/" directory
    # based installations, aka AppDirs, then this is the way to go.
    # ======================================================================= #
    if prefix.include? programs_directory?
      # ===================================================================== #
      # === Create the Architecture file on GoboLinux systems next
      # ===================================================================== #
      create_architecture_file
      create_symlinks(prefix)
      symlink_m4_macros(prefix)
      consider_symlinking_lib64_into_lib
      consider_symlinking_appdir_libraries_into_the_usr_lib_hierarchy
      register_program(prefix) # Register it into the AppDir-database.
      # ===================================================================== #
      # We next store into the "Resources/" directory, but only for
      # programs compiled via an AppDir prefix. This has to happen
      # after register_program() was called because we don't want to
      # keep track of these files in the database.
      # ===================================================================== #
      store_full_configure_command_into_resources_directory
    else
      # ===================================================================== #
      # Else we still may check for a few things, if we are on roebe that
      # is. For example, we will run the postinstallation step for bash
      # symlinking bin/bash towards bin/sh.
      # ===================================================================== #
      if is_on_roebe? and prefix.start_with?(home_dir?)
        run_post_install_class_unless_postinstall_is_empty
      end
    end
    # ======================================================================= #
    # Next, we symlink into /System/Tags.
    # ======================================================================= #
    symlink_into_tags(
      compile_which_program?,
      real_prefix?.to_s,
      cookbook_dataset?,
      use_tags?
    )
    # ======================================================================= #
    # Cleanup: We must get rid of special files.
    # ======================================================================= #
    remove_compiling_is_in_progress
    # ======================================================================= #
    # We may play a song, irrespective over as to whether we have used
    # an AppDir installation or not.
    # ======================================================================= #
    play_song if play_song?
  end

  # ========================================================================= #
  # === postinstall?
  # ========================================================================= #
  def postinstall?
    @internal_hash[:postinstall]
  end

  # ========================================================================= #
  # === run_post_install_class_unless_postinstall_is_empty
  # ========================================================================= #
  def run_post_install_class_unless_postinstall_is_empty
    unless postinstall?.empty?
      # ===================================================================== #
      # Delegate to class RBT::PostInstall next, but only if the
      # postinstall setting in the cookbook .yml file is NOT empty.
      # ===================================================================== #
      RBT::PostInstall.new(dataset?) {{
        use_this_compile_prefix: real_prefix?.to_s
      }}
    end
  end

  # ========================================================================= #
  # === create_symlinks
  #
  # This is only needed when the prefix is not traditional (i.e. Gobolinux
  # prefix). It will be called via the method do_postinstall_actions().
  # ========================================================================= #
  def create_symlinks(
      i                               = compile_which_program?,
      shall_we_invoke_symlink_program = shall_we_invoke_symlink_program?,
      use_this_as_program_version     = program_version?
    )
    if shall_we_invoke_symlink_program
      opne "Now running #{sfancy('RBT::SymlinkThisProgram')} to "\
           "symlink the content of `#{sdir(i)}`."
      do_invoke_symlink_program(i, use_this_as_program_version)
    end
  end

  # ========================================================================= #
  # === do_not_symlink?
  # ========================================================================= #
  def do_not_symlink?
    if dataset_to_use?
      dataset_to_use?.do_not_symlink == true
    else
      false
    end
  end

  # ========================================================================= #
  # === ensure_that_useful_internal_values_exist       (default tag, def tag)
  #
  # This method will ensure that some default internal-values exist.
  # ========================================================================= #
  def ensure_that_useful_internal_values_exist
    unless @internal_hash.has_key? :use_tags
      @internal_hash[:use_tags] = false
    end
    unless @internal_hash.has_key? :do_register_the_program
      @internal_hash[:do_register_the_program] = true # This is the default.
    end
    unless @internal_hash.has_key? :do_run_fetch_all_m4_macros
      @internal_hash[:do_run_fetch_all_m4_macros] = true # This is the default.
    end
    @internal_hash[:symlink_program] = RBT::SymlinkThisProgram.new { :do_not_run_yet }
  end

  # ========================================================================= #
  # === consider_symlinking_appdir_libraries_into_the_usr_lib_hierarchy
  # ========================================================================= #
  def consider_symlinking_appdir_libraries_into_the_usr_lib_hierarchy
    unless do_not_symlink?
      if is_on_roebe? and
        uses_appdir_approach? and
        no_error_has_occurred?
        target = prefix?+'lib/'
        RBT.symlink_appdir_libraries_into_usr_lib_directory(target)
      end
    end
  end

  # ========================================================================= #
  # === RBT::PostinstallationStep[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  # ========================================================================= #
  # This class may require an instance of RBT::Compile.
  # ========================================================================= #
  require 'rbt/compile/compile.rb'
  this_program = ARGV.first
  compile_object = RBT::Compile.new(this_program) { :do_not_run_yet }
  hash = compile_object.return_hash_for_the_postinstallation_step
  _ = RBT::PostinstallationStep.new(hash) { :do_not_run_yet }
  _.run
end # postinstallation_step htop