#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ProfileInstall
#
# This class can take up information given via a yaml file, and then
# compile programs - based from the info stored in that yaml file.
#
# This allows us to quickly install some standard programs.
#
# By default, the class will prefer to take information from a file
# called: installation_procedere.yml
# 
# Specific subsections can be used from that yaml file.
#
# Usage example:
#   RBT::ProfileInstall.new
# =========================================================================== #
# require 'rbt/utility_scripts/profile_install.rb'
# =========================================================================== #
begin
  require 'beautiful_url'
rescue LoadError; end
require 'rbt/base/base.rb'
require 'rbt/compile/class_methods.rb'

module RBT

class ProfileInstall < Base # === ProfileInstall.new

  # =========================================================================== #
  # === FILE_INSTALLATION_PROCEDURE
  # =========================================================================== #
  if Object.const_defined? :BeautifulUrl
    FILE_INSTALLATION_PROCEDURE =
      BeautifulUrl['installation_procedere'].first # .first because it is an Array.
  else
    FILE_INSTALLATION_PROCEDURE = nil
  end

  # =========================================================================== #
  # === WHICH_SUBSECTION_TO_USE
  # =========================================================================== #
  WHICH_SUBSECTION_TO_USE = 'default' # 'absolute' #'pclinuxos-may2007' #

  # ========================================================================= #
  # === initialize
  #
  # Invocation examples:
  #   rpinstall testing
  # ========================================================================= #
  def initialize(
      which_subsection_to_use = WHICH_SUBSECTION_TO_USE,
      run_already             = true
    )
    reset
    set_which_subsection_to_use(which_subsection_to_use)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @yaml_file_location = FILE_INSTALLATION_PROCEDURE
    if File.exist? @yaml_file_location
      @yaml_file_dataset  = YAML.load_file(@yaml_file_location) # Base this info on the line before.
    end
  end

  # ========================================================================= #
  # === set_which_subsection_to_use
  # ========================================================================= #
  def set_which_subsection_to_use(i = WHICH_SUBSECTION_TO_USE)
    i = WHICH_SUBSECTION_TO_USE if i.nil?  
    @which_subsection_to_use = i.to_s # Needs be a String.
  end

  # ========================================================================= #
  # === which_subsection_to_use?
  # ========================================================================= #
  def which_subsection_to_use?
    @which_subsection_to_use
  end

  # =========================================================================== #
  # === sanitize_subsection
  #
  # Sanitizes the variable @which_subsection_to_use, so that we 
  # can pass short-hand notations.
  # =========================================================================== #  
  def sanitize_subsection
    unless @which_subsection_to_use.include? 'gobolinux'
      @which_subsection_to_use.gsub!(/gobo/,'gobolinux')
    end
  end

  # =========================================================================== #
  # === process_data
  # =========================================================================== #
  def process_data
    if @yaml_file_dataset.has_key? @which_subsection_to_use
      @yaml_file_dataset[@which_subsection_to_use].each { |entry|
        efancy entry
        if entry.include? ';'
          tmp = entry.split ';'
          tmp.each { |entry| process_after_split( entry.split(' ') ) }
        else
          splitted = entry.split ' '
          process_after_split splitted 
        end
      }
    end
  end
  
  # =========================================================================== #
  # === process_after_split
  #
  # After splitting, we enter this method and process further.
  #
  # splitted must be an array.
  # =========================================================================== #
  def process_after_split(splitted)
    _ = splitted.first
    case _ # case tag
    when 'rm'
      ewarn ' => Removing a file now:'
      e splitted[1]
      remove_file splitted[1]
    when 'ry'
      ewarn ' => Compiling a program next:'
      e splitted[1]
      compile_programs splitted[1]
    else
      compile_program(_)
    end
  end

  # =========================================================================== #
  # === compile_programs
  #
  # Use this method to compile all things in the section.
  # =========================================================================== #
  def compile_programs(i)
    RBT.do_compile(i)
  end; alias compile_program compile_programs # === compile_program

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    sanitize_subsection 
    process_data
  end

  # ========================================================================= #
  # === ProfileInstall[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ProfileInstall.new(ARGV.first)
end # profile_install