#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::QueryHeaderToPackage
#
# This class will tell us to which program a specific header file belongs
# to.
#
# By default, this will only work for entries that were registered in the
# various yaml files (the cookbook files). A flag also exists to denote
# (and show) which entries are not registered.
#
# The class here will assume that the input argument given is a directory
# that will have .h files.
#
# Usage example from within Ruby code:
#
#   RBT::QueryHeaderToPackage.new
#
# Usage example from the commandline:
#
#   qheader /usr/include
#
# =========================================================================== #
# require 'rbt/utility_scripts/query_header_to_package.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class QueryHeaderToPackage < Base # === RBT::QueryHeaderToPackage.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === HASH_ALL_REGISTERED_HEADERS
  #
  # All registered headers are stored here in this Hash.
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_HEADERS
    begin
      HASH_ALL_REGISTERED_HEADERS = YAML.load_file(
        FILE_REGISTERED_HEADERS
      )
    rescue Psych::SyntaxError => error
      puts "Problem with the file `#{FILE_REGISTERED_HEADERS}`" 
      pp error
    end
  else
    if is_on_roebe?
      puts 'No file found called '+FILE_REGISTERED_HEADERS
    end
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @commandline_arguments = [] # <- Iniitialize it first.
    @show_all_header_files = false
    @filter_for = nil # Whether to filter for an individual program only.
    set_target_directory # Must initialize it at the least once.
  end

  # ========================================================================= #
  # === set_commandline_arguments
  #
  # Keep track of the commandline arguments used.
  # ========================================================================= #
  def set_commandline_arguments(i = ARGV)
    i = [i].flatten.compact
    i << Dir.pwd if i.empty?
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    if @commandline_arguments
      @commandline_arguments.first
    else
      nil
    end
  end

  # ========================================================================= #
  # === set_target_directory
  # ========================================================================= #
  def set_target_directory(
      i = Dir.pwd
    )
    i = i.first if i.is_a? Array
    i = Dir.pwd if i.nil?
    i = i.to_s.dup
    unless i.end_with? '/'
      i << '/'
    end
    if first_argument?.is_a? Symbol
      case first_argument?
      when :default
        i = '/usr/include/'
      end
    end if first_argument?
    @target_directory = i
  end; alias set_use_this_directory set_target_directory # === set_use_this_directory

  # ========================================================================= #
  # === target_directory?
  # ========================================================================= #
  def target_directory?
    @target_directory
  end; alias target? target_directory? # === target?

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === query_header_to_package /usr/include --filter_for=xextproto
      # ===================================================================== #
      when /^-?-?filter_for=(.+)/,
           /^-?-?filter=(.+)/
        @filter_for = $1.to_s.dup
      # ===================================================================== #
      # === query_header_to_package --unassigned
      # ===================================================================== #
      when /^-?-?unassigned$/
        @show_all_header_files = true
      # ===================================================================== #
      # === query_header_to_package --help
      # ===================================================================== #
      when /help/
        show_help
        exit
      else
        if i.is_a? Symbol
          set_use_this_directory(i)
        elsif File.exist?(i) and
              File.directory?(i)
         # ================================================================== #
         # Assign to this directory, if a valid path has been given to us:
         # ================================================================== #
         set_use_this_directory(i)
        end
      end
    end
  end

  # ========================================================================= #
  # === try_to_process_the_available_headers
  #
  # We work through the discovered .h files.
  # ========================================================================= #
  def try_to_process_the_available_headers
    ljust_value = 55
    if @filter_for
    end
    if @array_headers.empty?
      opnn; e 'No headers were found.'
    else
      @array_headers.each {|this_header|
        _ = this_header.sub(/\/usr\/include\//,'')
        # =================================================================== #
        # Query the big Hash next that includes all our .h files.
        # =================================================================== #
        if HASH_ALL_REGISTERED_HEADERS.has_key? _
          this_program = HASH_ALL_REGISTERED_HEADERS[_]
          if @filter_for and
             (@filter_for != this_program)
            # Filter for a specific program in this case.
          else
            e 'The header '+sfancy(this_header.ljust(ljust_value))+' belongs to '\
              'the program called `'+orange(this_program)+'`.'
          end
        else
          if @show_all_header_files
            e 'The header '+sfancy(this_header.ljust(ljust_value))+' is currently '\
              'unassigned. (An orphan header).'
          end
        end
      }
    end
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    opnn; e 'The following options are available:'
    e
    e '  --unassigned           # Show entries without an attribute as well.'
    e '  --filter_for=xextproto # Show only .h files belonging to '\
      'this program (e. g. xextproto in this example)'
    e
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input? commandline_arguments? # === input?

  # ========================================================================= #
  # === obtain_all_headers
  #
  # This method will obtain all .h files from the target directoy.
  # ========================================================================= #
  def obtain_all_headers
    _ = target_directory?
    @array_headers = Dir["#{_}**/**.h"]
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu(commandline_arguments?) # Must come before obtain_all_headers is invoked.
    obtain_all_headers # <- Obtain all .h files.
    try_to_process_the_available_headers
  end

  # ========================================================================= #
  # === RBT::QueryHeaderToPackage[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === RBT.query_header_to_package
#
# The input argument should be the path to the directory that you wish
# to test. By default we will use '/usr/include/' as a target here.
# =========================================================================== #
def self.query_header_to_package(i = '/usr/include/')
  RBT::QueryHeaderToPackage.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::QueryHeaderToPackage.new(ARGV)
end # query_header_to_package /usr/include