#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RegisterProgramFilesIntoYamlDatabase
#
# This class will register all files residing in a specific individual
# "/Programs/AppDir/Current" directory.
#
# For example, let's assume that the path will be a String such as
# "/Programs/Htop/Current". The class here will then register all
# files into the .yml file called
# "/Programs/Htop/Current/yaml/registered_files.yml".
# =========================================================================== #
# require 'rbt/utility_scripts/register_program_files_into_yaml_database.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

# =========================================================================== #
# === @register_program_files_into_yaml_database
#
# We need to be able to toggle whether we will make use of
# class RBT::RegisterProgramFilesIntoYamlDatabase or whether
# we should not.
# =========================================================================== #
@register_program_files_into_yaml_database = true

# =========================================================================== #
# === RBT.register_program_files_into_yaml_database?
# =========================================================================== #
def self.register_program_files_into_yaml_database?
  @register_program_files_into_yaml_database
end

# =========================================================================== #
# === RBT.set_register_program_files_into_yaml_database
# =========================================================================== #
def self.set_register_program_files_into_yaml_database(i)
  @register_program_files_into_yaml_database = i
end

# =========================================================================== #
# === RBT.do_not_register_into_the_yaml_database
# =========================================================================== #
def self.do_not_register_into_the_yaml_database
  @register_program_files_into_yaml_database = false
end

class RegisterProgramFilesIntoYamlDatabase < Base # === RBT::RegisterProgramFilesIntoYamlDatabase

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_INPUT_DIRECTORY
  # ========================================================================= #
  DEFAULT_INPUT_DIRECTORY =
    '/Programs/Xload/Current'

  # ========================================================================= #
  # === initialize
  #
  # The first argument tells us where we should register.
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_register_where(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @entries = []
    @run_simulation = false
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_register_where
  # ========================================================================= #
  def set_register_where(
      i = DEFAULT_INPUT_DIRECTORY
    )
    i = DEFAULT_INPUT_DIRECTORY if i.nil?
    i = i.to_s.dup # Get a copy here. Must be a String.
    case i # case tag
    # ========================================================================= #
    # === register_program_files --batch
    # ========================================================================= #
    when /^-?-?batch$/
      self.class.batch
    # ========================================================================= #
    # === register_program_files --pwd
    #
    # This entry point can, for example, be invokved in a directory such
    # as /Programs/Htop/Current.
    # ========================================================================= #
    when /^-?-?pwd$/,'pwd','PWD','Dir.pwd','cwd','HERE' # Handle pwd
      i = Dir.pwd
    end
    i = rds(i)
    i << '/' unless i.end_with? '/'
    # ========================================================================= #
    # Append 'Current/'. A proper target must end with 'Current/', since
    # that is the symlink on GoboLinux.
    # ========================================================================= #
    unless i.count('/') > 2
      i << 'Current/' unless i.include? 'Current/'
    end
    # ========================================================================= #
    # i.delete!('-') if i.include? '-' # Added Aug 2014.
    # ^^^ Disabled as of Sep 2017. If we want to use this, then we may have
    # to use a configuration setting for the whole RBT project. If this is
    # done, then we can re-enable the above, but for now, it stays disabled.
    # ========================================================================= #
    @register_where = i
  end

  # ========================================================================= #
  # === entries?
  # ========================================================================= #
  def entries?
    @entries
  end; alias files? entries? # === files?

  # ========================================================================= #
  # === set_versioned_directory
  #
  # We need to keep track here so that we can get rid of the version
  # string of the directory when we save into the yaml file.
  # ========================================================================= #
  def set_versioned_directory(i)
    i = i.to_s.dup
    i << '/' unless i.end_with? '/'
    @versioned_directory = i
  end

  # ========================================================================= #
  # === fetch_entries
  # ========================================================================= #
  def fetch_entries
    opnn; e "Obtaining all files from `#{sdir(where?)}`."
    path = "#{@register_where}**/**"
    _ = Dir[path]
    # ======================================================================= #
    # Select only files:
    # ======================================================================= #
    _.select! {|entry| File.file? entry }
    @entries = _
  end

  # ========================================================================= #
  # === register_where?
  # ========================================================================= #
  def register_where?
    @register_where
  end; alias where? register_where? # === where?

  # ========================================================================= #
  # === run_simulation?
  # ========================================================================= #
  def run_simulation?
    @run_simulation
  end; alias run_simulation run_simulation? # === run_simulation

  # ========================================================================= #
  # === run_simulation=
  # ========================================================================= #
  def run_simulation=(i)
    @run_simulation = i
  end

  # ========================================================================= #
  # === register_into_yaml_file
  #
  # This method will use the Current symlink.
  # ========================================================================= #
  def register_into_yaml_file # This will register @array_register_these_files finally
    _ = register_where?
    set_versioned_directory(_)
    if (_.count('/') < 3) and !_.end_with?('Current/')
      _ << 'Current/'
      unless File.symlink?(_) and return_pwd.include?(programs_dir?) # Try to re-create the Symlink here if we are in the /Programs directory.
        target = @register_where
        target = File.dirname(target) if target.end_with? 'Current/'
        if File.directory? target
          Dir.chdir(target)
          target = get_directories_from(target).sort.first
          set_versioned_directory(target)
          if target
            remove_directory('Current') if File.directory?('Current') # Must be a Symlink.
            symlink(target.chop, _.chop) # .chop to get rid of '/'.
          end
        else
          opnn; e sdir(target)+swarn(' is not a directory.')
        end
      end
    end
    _ << 'yaml/'
    mkdir _ unless File.directory? _ # The directory should exist.
    # ======================================================================= #
    # We will store in a yaml/ subdirectory, in a file called
    # registered_files.yml
    # ======================================================================= #
    registered_files = rds("#{_}registered_files.yml")
    if run_simulation?
      e "We would now remove #{simp(registered_files.to_s)} if we"
      e 'would not be in simulation mode.'
    else
      # ===================================================================== #
      # Get rid of the old .yml file, if it exists.
      # ===================================================================== #
      remove(registered_files) if File.exist? registered_files
    end
    unless run_simulation?
      if @entries.empty?
        opne 'We will not store the registered files as'
        opne 'the dataset is empty.'
      else
        opne 'Storing dataset ('+sfancy(registered_files.size.to_s)+
             ' files) into:'
        opne '  `'+sfile(registered_files)+'`.'
        _ = @entries.map {|entry|
          entry.sub(/#{@register_where}/,'')
        }
        _.reject! {|entry|
          entry.include?  'registered_files.yml'
        } # Reject self-name.
        _.reject! {|entry|
          entry.end_with? 'Resources/full_configure_command' or
          entry.end_with? 'Resources/full_configure_command.md' or
          entry.end_with? 'Resources/Architecture'
        }
        _.map!    {|entry| entry.gsub(/#{Regexp.quote(@versioned_directory)}/,'')   }
        unless _.empty?
          save_what_into( YAML.dump(_), registered_files )
        end
      end
    end
  end

  # ========================================================================= #
  # === report_files
  # ========================================================================= #
  def report_files
    opnn; @entries.each {|entry|
      e entry
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    fetch_entries
    register_into_yaml_file
  end

  # ========================================================================= #
  # === RBT::RegisterProgramFilesIntoYamlDatabase.batch
  #
  # This method will run through all entries at /Programs, and register it.
  # ========================================================================= #
  def self.batch
    files = Dir["#{RBT.programs_directory?}*"].map {|entry| entry << '/Current/'}
    files.sort.each {|entry|
      e "Working on #{sfancy(entry)} next."
      e
      RBT::RegisterProgramFilesIntoYamlDatabase.new(entry)
      e
    }
  end

end; end

if __FILE__ == $PROGRAM_NAME
  case ARGV.first
  when 'batch','ALL','every_program'
    RBT::RegisterProgramFilesIntoYamlDatabase.batch
  else
    if ARGV.first
      RBT::RegisterProgramFilesIntoYamlDatabase.new(ARGV.first)
    else
      RBT::RegisterProgramFilesIntoYamlDatabase.new(Dir.pwd)
    end
  end
  # _.report_files
end # register /Programs/Yasm
    # register /Programs/Htop/Current/
    # register /Programs/Xrandr/Current/
    # register batch