#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredBinaries
#
# The task of this class is simple:
#
#   Iterate over all binaries and collect them, storing them into a
#   yaml file.
#
#   If however had an argument is passed, then we will try to find
#   out to which program this binary belongs to.
# =========================================================================== #
# require 'rbt/utility_scripts/registered_binaries.rb'
# =========================================================================== #
require 'rbt/utility_scripts/registered.rb'
require 'rbt/cookbooks/class/class.rb'
require 'rbt/toplevel_methods/all_binaries.rb'
require 'rbt/toplevel_methods/available_programs.rb'
require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'

module RBT

module Cookbooks

class RegisteredBinaries < Registered # === RBT::Cookbooks::RegisteredBinaries

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @hash_to_be_stored = {} # This is the hash that will be stored.
    @cookbook_dataset = nil
    @this_program = nil
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opnn; e 'Obtaining Information about available binaries next.'
    opnn; e 'This may take a while.'
    opnn; e orange('Please be patient.')
  end

  # ========================================================================= #
  # === RBT::Cookbooks::RegisteredBinaries.location?
  # ========================================================================= #
  def self.location?
    opn; e FILE_REGISTERED_BINARIES
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # binaries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    into = FILE_REGISTERED_BINARIES
    what = YAML.dump(hash?)
    e sfancy('Storing yaml dataset in ')
    e '  '+sfile(into)
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_AT_HOME}rbt/lib/rbt/yaml/#{File.basename(into)}"
      e "Also storing into `#{sfile(into)}`."
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === Cookbooks::RegisteredBinaries.search_for_this_program
  # ========================================================================= #
  def self.search_for_this_program(i)
    _ = Cookbooks::RegisteredBinaries.new(false)
    _.set_this_program(i)
    _.search_for_this_program(i)
  end

  # ========================================================================= #
  # === RegisteredBinaries.opnn
  # ========================================================================= #
  def self.opnn
    Opn.opn(namespace: NAMESPACE)
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    RegisteredBinaries.opnn
  end

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    i = i.to_s
    if i.include? '/'
      i = File.basename(i)
      opnn; e "The input includes a #{simp('/')} token. We will "\
              "truncate this to the"
      opnn; e 'last part only, as we only keep the names registered, '\
              'not the paths.'
      opnn; e "Thus, our input will be #{sfancy(i)}."
    end
    opnn; e "Searching for `#{sfancy(i)}` now:"
    # ======================================================================= #
    # Query whether it is already included or not.
    # ======================================================================= #
    if RBT.does_include? i
      opnn; e "Yes, this appears to be already registered with the "\
              "same name #{simp(i)}."
      program_version = RBT.swift_return_version_of_this_program(i)
      opnn; e "Its program version is: #{lightblue(program_version)}"
    else
      hash = load_yaml(FILE_REGISTERED_BINARIES)
      if hash.keys.include? i
        opnn; e "The binary called `#{sfancy(i)}` was found."
        opnn; e "It is part of the package -> `#{simp(hash[i])}`."
      else
        opnn; e sfancy(i)+' is '+swarn('NOT')+' included. We could not '\
                'find any program'
        opnn; e 'including this program.'
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    available_programs?.each { |program_name|
      begin
        _ = RBT::Cookbooks::Cookbook.new(program_name) { :bypass_menu }
        binaries = _.binaries? # This is an Array.
        unless binaries.empty?
          e program_name if SHOW_PROGRAM_NAME
          binaries.each {|entry|
            @hash_to_be_stored[entry] = program_name
          }
        end
      rescue Exception => error
        opnn; e 'An error happened for the program '+
                sfile(program_name)+
                '. (Error: '+error.class.to_s+')'
        pp error
      end
    }
    save_yaml_file
  end

end; end

# =========================================================================== #
# === RBT.registered_binaries?
#
# Simply return an Array with all the binaries.
#
# Full usage example:
#   require 'rbt/utility_scripts/registered_binaries.rb'
#   RBT.registered_binaries?
# =========================================================================== #
def self.registered_binaries?
  ALL_BINARIES.keys.uniq.sort
end; self.instance_eval { alias registered_binaries registered_binaries? } # === RBT.registered_binaries

# =========================================================================== #
# === RBT.hash_registered_binaries?
#
# This method is similar to the method RBT.registered_binaries? - but
# rather than an Array we will return the original Hash.
# =========================================================================== #
def self.hash_registered_binaries?
  ALL_BINARIES
end

# =========================================================================== #
# === RBT.show_registered_binaries
# =========================================================================== #
def self.show_registered_binaries
  _ = ::RBT.registered_binaries?
  RegisteredBinaries.opnn; e 'These '+_.size.to_s+' binaries are registered:'
  e _.reject {|entry| entry.to_s.empty? }.join(', ').strip.chop
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredBinaries.new(false)
  if ARGV.first
    argument = ARGV.first
    if argument.include? 'show'
      RBT.show_registered_binaries # regbin --show
    else # Pass the argument into the method search_for_this_program().
      _.search_for_this_program(argument)
    end
  else
    _.run
  end
end # regbin