#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredHeader
#
# The task of this class is simple:
#
#   Iterate over all headers and collect them, storing them into a
#   yaml file.
#
#   If however had an argument is passed, then we will try to find
#   out to which program this header file belongs to. (This is
#   already done by FindHeaders, so we delegate to it.)
# =========================================================================== #
# require 'rbt/utility_scripts/registered_headers.rb'
# =========================================================================== #
require 'rbt/project/project_base_and_yaml_directory.rb'
require 'rbt/utility_scripts/registered.rb'
require 'rbt/utility_scripts/find_headers.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

  # ========================================================================= #
  # === LOCATION_REGISTERED_HEADERS
  #
  # cat $COOKBOOKS/yaml/registered_headers.yml
  # ========================================================================= #
  LOCATION_REGISTERED_HEADERS = FILE_REGISTERED_HEADERS

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_HEADERS
  # ========================================================================= #
  if File.exist? LOCATION_REGISTERED_HEADERS
    begin
      YAML_FILE_REGISTERED_HEADERS = YAML.load_file(LOCATION_REGISTERED_HEADERS)
    rescue Psych::SyntaxError => error
      YAML_FILE_REGISTERED_HEADERS = {}
      puts 'Problem for the file '+LOCATION_REGISTERED_HEADERS
      pp error
    end
  else
    YAML_FILE_REGISTERED_HEADERS = {} # Otherwise, initialize as empty Hash.
  end

  # ========================================================================= #
  # === ALL_HEADERS
  # ========================================================================= #
  ALL_HEADERS = YAML_FILE_REGISTERED_HEADERS

  # ========================================================================= #
  # === RBT::Cookbooks.all_headers?
  # ========================================================================= #
  def self.all_headers?
    ALL_HEADERS
  end

  # ========================================================================= #
  # === all_headers?
  # ========================================================================= #
  def all_headers?
    Cookbooks.all_headers?
  end

class RegisteredHeaders < RBT::Cookbooks::Registered

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  #
  # Whether to show the program name as well, as we iterate over our
  # dataset.
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @hash_to_be_stored = {} # This is the hash that will be stored.
    @query = RBT::Cookbooks::Cookbook.new(:do_not_run_yet) { :bypass_menu }
  end

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(p)
    p = p.to_s
    opnn; e 'Searching for '+simp(p)+' now:'
    hash = load_yaml(LOCATION_REGISTERED_HEADERS)
    if hash.keys.include? p
      opnn; e 'The header called `'+sfancy(p)+'` was found.'
      opnn; e 'It is part of the package `'+simp(hash[p])+'`.'
    else
      opnn; e p+' is '+swarn('NOT')+' included. We could not find any program'
      opnn; e 'including this header.'
    end
  end

  # ========================================================================= #
  # === RegisteredBinaries.location?
  # ========================================================================= #
  def self.location?
    opn; e LOCATION_REGISTERED_HEADERS
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    available_programs?.each { |program_name|
      @query.update_with_this_program(program_name)
      headers = @query.headers? # This is guaranteed to be an Array.
      unless headers.empty?
        if SHOW_PROGRAM_NAME
          opnn; e "The program called #{sfancy(program_name)} has "\
                  "some header files."
        end
        headers.each {|entry|
          if entry.include? ','
            entry.split(',').each {|inner_entry|
              @hash_to_be_stored[inner_entry] = program_name
            }
          else
            @hash_to_be_stored[entry] = program_name
          end
        }
      end
    }
    save_yaml_file
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opnn; e 'Obtaining Information about available headers next.'
    opnn; e 'This may take a while.'
    opnn; e 'Please be patient.'
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # binaries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    what = YAML.dump(@hash_to_be_stored)
    into = LOCATION_REGISTERED_HEADERS
    e sfancy('Storing yaml dataset in')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if on_roebe? # <- Store on my home dir as well.
      into = ruby_src_dir?+'rbt/lib/rbt/yaml/'+
             File.basename(into)
      e sfancy('Storing the yaml dataset also in')
      e '  '+sfile(into)
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

end; end

# =========================================================================== #
# === RBT.all_headers?
# =========================================================================== #
def self.all_headers?
  RBT::Cookbooks.all_headers?
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredHeaders.new(false)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # regheaders