#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredTags
#
# This script will scan through each program, get the tags, and register
# these tags.
#
# It will report when it finds an entry without any tag, so that you
# can add a tag to it if you want to.
# =========================================================================== #
# require 'cookbooks/utility_scripts/registered_tags.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

  # ========================================================================= #
  # === LOCATION_REGISTERED_TAGS
  #
  # cat $COOKBOOKS/yaml/registered_tags.yml
  # ========================================================================= #
  LOCATION_REGISTERED_TAGS = "#{RBT.yaml_directory?}registered_tags.yml"

  # ========================================================================= #
  # === RBT.file_registered_tags?
  # ========================================================================= #
  def self.file_registered_tags?
    "#{RBT.yaml_directory?}registered_tags.yml"
  end; self.instance_eval { alias file_registered_tags file_registered_tags? } # === RBT.file_registered_tags

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_TAGS
  # ========================================================================= #
  if File.exist? LOCATION_REGISTERED_TAGS
    YAML_FILE_REGISTERED_TAGS = YAML.load_file(LOCATION_REGISTERED_TAGS)
  else
    YAML_FILE_REGISTERED_TAGS = {} # Otherwise, initialize as empty hash.
  end; ALL_TAGS = YAML_FILE_REGISTERED_TAGS # === ALL_TAGS

  # ========================================================================= #
  # === RBT.all_tags?
  # ========================================================================= #
  def self.all_tags?
    ALL_TAGS
  end

  # ========================================================================= #
  # === all_tags?
  # ========================================================================= #
  def all_tags?
    RBT.all_tags?
  end

class RegisteredTags < RBT::Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_commandline_arguments = ARGV,
      run_already                    = true
    )
    super()
    reset
    menu(optional_commandline_arguments) if optional_commandline_arguments
    run if run_already
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === registered_tags --help
      # ===================================================================== #
      when /help/i
        show_help(:then_exit)
      end
    end
  end

  # ========================================================================= #
  # === show_help
  #
  # To invoke this method from the commandline, do:
  #   registered_tags --help
  # ========================================================================= #
  def show_help(
      shall_we_exit_afterwards = false
    )
    case shall_we_exit_afterwards
    when :then_exit
      shall_we_exit_afterwards = true
    end
    opnn; e 'This class will simply register all available tags into the file'
    opnn; e "  #{sfile(store_where?)}"
    exit if shall_we_exit_afterwards
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @hash_available_tags = {}
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_hash
  #
  # Use this method to set to the main hash.
  # ========================================================================= #
  def set_hash(i)
    raise 'Must be a hash.' unless i.is_a? Hash
    @hash_available_tags = i
  end

  # ========================================================================= #
  # === available_cookbooks?
  # ========================================================================= #
  def available_cookbooks?
    RBT.available_programs?
  end

  # ========================================================================= #
  # === store_where?
  # ========================================================================= #
  def store_where?
    LOCATION_REGISTERED_TAGS
  end

  # ========================================================================= #
  # === store_main_hash
  #
  # Write the hash into a file.
  # ========================================================================= #
  def store_main_hash
    _ = store_where?
    e sfancy('Storing yaml dataset in ')
    e '  '+sfile(_)
    # ======================================================================= #
    # Purge the old file first before adding new data to it.
    # ======================================================================= #
    remove_file(_) if File.exist? _
    File.open(_,'w+') { |file|
      YAML.dump(tags?, file)
    }
    # ======================================================================= #
    # Also store in the right file on my home system.
    #   cat $RSRC/cookbooks/lib/cookbooks/yaml/registered_tags.yml
    # ======================================================================= #
    if is_on_roebe?
      _ = RUBY_SRC_DIR_AT_HOME+'rbt/lib/rbt/yaml/registered_tags.yml'
      File.delete(_) if File.exist? _
      e sfancy('Also storing into ')
      e '  '+sfile(_)
      File.open(_,'w+') { |file|
        YAML.dump(tags?, file)
      }
    end
  end

  # ========================================================================= #
  # === tags?
  # ========================================================================= #
  def tags?
    @hash_available_tags
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    available_cookbooks?.each {|program|
      # opn; e 'Now working on `'+sfancy(program)+'`.'
      # ^^^ Too spammy as output.
      dataset = RBT::Cookbooks::Cookbook.new(program) { :bypass_menu }
      tags = dataset.tags?
      if tags
        if tags.empty?
          opnn; e "#{crimson('Notice')}: The program `#{sfancy(program)}` has "\
                  "no registered tags."
        else # else add it to the Hash.
          @hash_available_tags[program] = tags
        end
      else
        opnn; e 'No tags for `'+sfancy(program.to_s)+'`.'
      end
    }
    # ======================================================================= #
    # Note that at this point, the hash is not correct yet.
    # It will have entries such as:
    #   "thor"=>["ruby"]
    # but it should be the other way around. 
    # ======================================================================= #
    inverted_hash = {}
    tags?.each_pair {|name_of_the_program, value|
      unless value.is_a? Array
        e 'Must be an Array, but is a `'+sfancy(value.class.to_s)+'`.'
        e 'This is not allowed - correct the entry please.'
        e 'Exiting now as a consequence.'
        exit
      end
      value.each {|program_type|
        unless inverted_hash.has_key? program_type
          inverted_hash[program_type] = []
        end
        inverted_hash[program_type] << name_of_the_program 
      }
    }
    set_hash(inverted_hash)
    store_main_hash
  end

  # ========================================================================= #
  # === RBT::RegisteredTags[]
  # ========================================================================= #
  def self.[]
    self.new
  end

end

# =========================================================================== #
# === RBT.create_registered_tags
# =========================================================================== #
def self.create_registered_tags
  RegisteredTags.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.create_registered_tags
end # regtag