#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemoveProgram
#
# Purpose of this class:
#
#   Remove a program from the /Programs/ hierarchy. By default it
#   will check under the $PROGRAMS hierarchy, which is usually the
#   same as /Programs/, but depending on the @install_type variable,
#   the class may also check in the /usr/ prefix.
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/remove_symlinks.rb' # This must work.

module RBT

class RemoveProgram < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'xset'

  # ========================================================================= #
  # === initialize
  #
  # @install_type may be of these two values:
  #
  #     :fhs
  #     :app_dir
  #
  # The default is :app_dir. This means to check under the /Programs/
  # hierarchy.
  # ========================================================================= #
  def initialize(
      this_program = DEFAULT_PROGRAM,
      run_already  = true
    )
    reset
    set_this_program(this_program)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @install_type = :app_dir
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_this_program
  #
  # This will simply set the variable @remove_this_program.
  # ========================================================================= #
  def set_this_program(i = DEFAULT_PROGRAM)
    i = i.first if i.is_a? Array
    i = DEFAULT_PROGRAM if i.nil?
    i = i.capitalize
    @remove_this_program = i
  end

  # ========================================================================= #
  # === set_program_path
  # ========================================================================= #
  def set_program_path(
      i = rds(programs_dir?)
    )
    @program_path = i
  end

  # ========================================================================= #
  # === try_to_remove_stray_symlinks_from_the_usr_include_hierarchy
  # ========================================================================= #
  def try_to_remove_stray_symlinks_from_the_usr_include_hierarchy(
      i = @this_dir+'include/'
    )
    possible_targets = Dir[i+'*']
    possible_targets.each {|entry|
      # ===================================================================== #
      # Specifcy the /usr/include/ target next.
      # ===================================================================== #
      target = "/usr/include/#{File.basename(entry)}"
      # ===================================================================== #
      # Ok, we must check whether the target exists, and whether it is
      # a symlink, and whether it points to @this_dir.
      # ===================================================================== #
      if File.exist?(target) and
         File.symlink?(target) and
         File.readlink(target).include?(@this_dir) # This line is just an additional safeguard.
        opnn; e 'Now removing the symlink at `'+sfile(target)+'` as well.'
        delete(target)
      end
    }
  end

  # ========================================================================= #
  # === remove_program
  #
  # This is the actual powerhorse of this script.
  # ========================================================================= #
  def remove_program(
      this_program = @remove_this_program
    )
    _ = @install_type
    case _ # case tag
    # ======================================================================= #
    # === remove_program evince
    # ======================================================================= #
    when :app_dir, :gobo, :gobolinux_mode
      set_program_path(rds(programs_dir?))
      base_dir = @program_path+this_program
      @this_dir = "#{base_dir}/Current/"
      opnn; e 'Removing program `'+sdir(@this_dir)+'` now' # Tell the user what to do.
      # ===================================================================== #
      # First, clean-up any stray symlinks into the /usr/include
      # hierarchy.
      # ===================================================================== #
      try_to_remove_stray_symlinks_from_the_usr_include_hierarchy(
        @this_dir+'include/'
      )
      remove_directory(@this_dir)
      # ===================================================================== #
      # Next, we get rid of the symlinks and the current link.
      # ===================================================================== #
      remove_broken_symlinks # Remove all broken Symlinks.
      remove_current_link    # And remove the 'Current' symlink as well.
    when :fhs
      # ===================================================================== #
      # Currently not implemented.
      # ===================================================================== #
    else
      e 'Unregistered Type ('+sfancy(_)+').'
    end
    run_ldconfig
  end

  # ========================================================================= #
  # === remove_current_link
  #
  # This method attempts to remove the 'Current' symlink.
  # ========================================================================= #
  def remove_current_link
    _ = @this_dir[0..-2]
    if File.exist? _
      result = 'Now removing'.dup
      result << ' the symlink' if File.symlink? _
      result << " `#{sdir(_)}`."
      opnn; e result
      remove(_)
    end
  end

  # ========================================================================= #
  # === remove_broken_symlinks
  #
  # Delegate towards class RBT::RemoveSymlinks here.
  # ========================================================================= #
  def remove_broken_symlinks
    opnn; e 'Removing broken symlinks next.'
    RBT::RemoveSymlinks.new
  end

  # ========================================================================= #
  # === run_ldconfig
  # ========================================================================= #
  def run_ldconfig
    system 'ldconfig' if RBT.configuration?.run_ldconfig
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    remove_program
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::RemoveProgram.new(ARGV)
end # rprog xwud