#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RemoveSymlinks
#
# Purpose of this class:
#
#   If the given (assumed) target of a local symlink does not exist, then
#   this symlink will be removed/deleted.
#
# The class will check on all symlinks, that is, all symlinks that reside
# under the /Programs/ hierarchy (or respectively where the user has
# designated his /Programs/ hierarchy to be at).
#
# We can use File.readlink() for this functionality.
# =========================================================================== #
# require 'rbt/utility_scripts/remove_symlinks.rb'
# RBT::RemoveSymlinks.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/utility_scripts/remove_empty_directories.rb'

module RBT

class RemoveSymlinks < Base # === RBT::RemoveSymlinks

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === DEFAULT_MODE
  # ========================================================================= #
  DEFAULT_MODE = :gobolinux_mode

  # ========================================================================= #
  # === SHALL_WE_CREATE_MISSING_DIRECTORIES
  #
  # This may create missing directories.
  # ========================================================================= #
  SHALL_WE_CREATE_MISSING_DIRECTORIES = false

  # ========================================================================= #
  # === ARRAY_DIRECTORIES_TO_CHECK
  # ========================================================================= #
  ARRAY_DIRECTORIES_TO_CHECK = [
    RBT.syslib_directory?,
    RBT.sysbin_directory?,
    "#{RBT.syslib_directory?}pkgconfig/",
    RBT.sysinclude_directory?,
    RBT.sysshare_directory?
  ]

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      mode        = :gobolinux_mode,
      run_already = true
    )
    reset
    set_mode(mode)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super() # Need to invoke the parent method here.
    @array_directories_to_check = ARRAY_DIRECTORIES_TO_CHECK
    @array_remove_these_broken_symlinks = []
    @be_verbose = false
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_base_dir
  # ========================================================================= #
  def set_base_dir(i)
    i = rds(i)
    i << '/' unless i.end_with? '/' # We require a trailing '/' here.
    @base_dir = i
  end

  # ========================================================================= #
  # === register_broken_symlink_for_removal
  # ========================================================================= #
  def register_broken_symlink_for_removal(i)
    unless @array_remove_these_broken_symlinks.include? i # Do not include duplicates.
      @array_remove_these_broken_symlinks << i
    end
  end

  # ========================================================================= #
  # === process_each_directory
  # ========================================================================= #
  def process_each_directory(
      i = @array_directories_to_check
    )
    unless i.is_a? Array
      i = [i]
    end 
    i.each {|entry|
      unless File.exist? entry
        if SHALL_WE_CREATE_MISSING_DIRECTORIES
          opnn; ewarn 'Directory `'+sdir(entry)+swarn('` does not exist.')
          opnn; ewarn 'Thus, we will create it now.'
          create_directory(entry)
        end
      end
      # ===================================================================== #
      # Next, obtain all symlinks from the target.
      # ===================================================================== #
      _ = get_all_symlinks_from(entry)
      _.each {|inner_entry|
        if File.exist? inner_entry # File exist, this is good, passthrough.
        else # The file does not exist, so register it for removal.
          # e 'FILE DOES NOT EXIST '+sfile(inner_entry)
          opnn; e sfile(inner_entry)+' does not exist.'
          opnn; e '-> Thus we delete `'+sfile(inner_entry)+'`.'
          register_broken_symlink_for_removal(inner_entry) # First register, then delete.
        end
      }
    }
  end

  # ========================================================================= #
  # === get_all_symlinks_from
  #
  # This method will fetch and then return all symlinks from a given
  # directory.
  # ========================================================================= #
  def get_all_symlinks_from(i = Dir.pwd, be_verbose = true)
    set_base_dir(i)
    _ = Dir[@base_dir+'*'].reject {|file| ! File.symlink? file}
    if be_verbose
      opnn; e "There are #{sfancy(_.size.to_s.rjust(3))} symlinks "+
              "listed in "+sdir(i)+'.'
    end
    return _
  end

  # ========================================================================= #
  # === notify_the_user_how_many_symlinks_were_removed
  # ========================================================================= #
  def notify_the_user_how_many_symlinks_were_removed
    _ = @array_remove_these_broken_symlinks
    unless _.empty?
      opnn; e 'We were able to remove '+
              orange(_.size.to_s)+
              ' non-existing symlinks.'
    end
  end

  # ========================================================================= #
  # === remove_empty_directories
  # ========================================================================= #
  def remove_empty_directories
    # ======================================================================= #
    # Delegate towards class RBT::RemoveEmptyDirectories next:
    # ======================================================================= #
    RemoveEmptyDirectories.new
  end

  # ========================================================================= #
  # === consider_using_other_directories
  # ========================================================================= #
  def consider_using_other_directories
    case @mode
    when 'HERE','PWD','pwd' # This allows us to work on the current directory instead.
      @array_directories_to_check[0,0] = Dir.pwd
    end unless File.exist? @mode
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # To invoke this method, try:
  #
  #   remove_symlinks --help
  #
  # ========================================================================= #
  def show_help
    opnn { :no_trailing_character }
    e
    e
    e '  --remove-all-symlinks # Remove all symlinks in '\
      'the current directory.'
    e '  --pwd                 # ^^^ an alias to the above'
    e
  end

  # ========================================================================= #
  # === remove_broken_symlinks
  # ========================================================================= #
  def remove_broken_symlinks
    if @array_remove_these_broken_symlinks.empty?
      opnn; e 'We did not find any broken symlinks on the system. '\
              'This is good!'
    else
      @array_remove_these_broken_symlinks.each {|this_symlink|
        # =================================================================== #
        # The reason we handle it in this class is so that we
        # can use opnn() on each removed symlink.
        # =================================================================== #
        opnn; delete_symlink(this_symlink, :be_verbose)
      }
    end
  end

  # ========================================================================= #
  # === set_mode
  #
  # Not yet sure if we need this.
  # ========================================================================= #
  def set_mode(
      i = DEFAULT_MODE
    )
    if i.is_a? Array
      i = i.first
    end
    i = DEFAULT_MODE if i.nil?
    case i # case tag
    # ======================================================================= #
    # === remove_symlinks --pwd
    #
    # This entry point will use the current working directory as our
    # input-base.
    # ======================================================================= #
    when /^-?-?pwd$/,
         /^-?-?remove(-|_)?all(-|_)?symlinks$/
      remove_these_symlinks(Dir['*'], :be_verbose)
      exit
    # ======================================================================= #
    # === remove_symlinks --help
    # ======================================================================= #
    when /^-?-?help$/
      show_help
      exit
    end
    @mode = i.to_s # Will be guaranteed to be a String.
  end

  # ========================================================================= #
  # === remove_all_symlinks
  #
  # This method will remove all symlinks in the current directory.
  #
  # Invocation examples:
  #
  #   remove_symlinks --remove-all-symlinks
  #   remove_symlinks --pwd
  #
  # ========================================================================= #
  def remove_all_symlinks(
      these_entries = Dir['*'],
      be_verbose    = :be_quiet
    )
    _ = these_entries.select {|entry| File.symlink? entry }
    delete_symlink(_, be_verbose)
  end; alias remove_these_symlinks remove_all_symlinks # === remove_these_symlinks

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    consider_using_other_directories
    remove_empty_directories
    process_each_directory
    remove_broken_symlinks
    notify_the_user_how_many_symlinks_were_removed
  end; alias start   run # === start
       alias default run # === default

  # ========================================================================= #
  # === RBT::RemoveSymlinks[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME 
  RBT::RemoveSymlinks.new(ARGV)
end # removesymlinks