#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ShowCompileChain
#
# This small class allows us to show the compile chain of a certain
# "meta"-package. That is, of programs that are registered in the
# yaml file called 'chained_programs.yml'.
#
# I created this class in November 2018 to make available functionality
# that already existed from within class RBT::Compile.
#
# Usage example:
#   RBT::ShowCompileChain.new
# =========================================================================== #
# require 'rbt/utility_scripts/show_compile_chain.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ShowCompileChain < Base # === RBT::ShowCompileChain.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @dataset = chained_programs?
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === chained_programs?
  # ========================================================================= #
  def chained_programs?
    RBT.return_chained_programs
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
    sanitize_commandline_arguments
  end

  # ========================================================================= #
  # === sanitize_commandline_arguments
  # ========================================================================= #
  def sanitize_commandline_arguments
    @commandline_arguments.map! {|entry|
      case entry # case tag
      # ===================================================================== #
      # === kde_apps
      # ===================================================================== #
      when /^kde(_|-)?apps$/
        entry = 'kde5_applications'
      # ===================================================================== #
      # === mate
      # ===================================================================== #
      when 'mate' # <- ShowCompileChainOf mate
        entry = 'all_of_the_mate_desktop'
      # ===================================================================== #
      # === portingaids
      # ===================================================================== #
      when 'portingaids' # <- ShowCompileChainOf portingaids
        entry = 'kde5_porting_aids'
      end
      entry
    }
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    @commandline_arguments.first
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    i = first_argument?
    # ======================================================================= #
    # The next variable will have a format such as:
    #   {"kde5_plasma"=> ["bluedevil"
    # ======================================================================= #
    selection = @dataset.select {|key, value|
      key.downcase.include? i.downcase
    }
    if selection.empty?
      # ===================================================================== #
      # Enter this clause when no match was found.
      # ===================================================================== #
      opnn; e "No match was found for `#{slateblue(i)}`."
      opnn; e 'The available chained programs are:'
      e
      available_programs = '  '+chained_programs?.keys.join(' ').strip
      available_programs = wrap_at(available_programs, 54)
      array_available_programs = available_programs.split(N).map(&:strip)
      array_available_programs.each_with_index {|these_programs, index| index += 1
        padded_index = index.to_s.ljust(4)
        e '  '+padded_index+
          royalblue(
          these_programs.tr(' ',', ')
        )
      }
      e
    else
      # ===================================================================== #
      # Else we have at the least one entry in the Array called "selection".
      # ===================================================================== #
      first_entry = selection.keys.first
      array_available_programs = selection[first_entry]
      if array_available_programs.is_a?(String) and
         array_available_programs.include?(',')
        array_available_programs = array_available_programs.split(',').map(&:strip)
      end
      if array_available_programs.is_a?(Array) and
        !array_available_programs.empty?
        opnn; e "The following entries belong to the group "\
                "`#{sfancy(first_entry)}`:"
        e
        array_available_programs.each_with_index {|this_entry, index| index += 1
          padded_index = (index.to_s.rjust(3)).ljust(6)
          e "#{lightgreen(padded_index)}#{royalblue(this_entry)}"
        }
        e
      end
    end
  end

  # ========================================================================= #
  # === RBT::ShowCompileChain.of
  # ========================================================================= #
  def self.of(i = ARGV)
    [i].flatten.compact.each {|this_program|
      RBT::ShowCompileChain.new(this_program)
    }
  end; self.instance_eval { alias [] of } # === ShowCompileChain[]

end

# =========================================================================== #
# === RBT.show_compile_chain
# =========================================================================== #
def self.show_compile_chain(of = ARGV)
  [of].flatten.compact.each {|this_program|
    RBT::ShowCompileChain.new(this_program)
  }
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::ShowCompileChain.new(ARGV)
end # show_compile_chain_of kde5_plasma