#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkAllUpcasedProgramsToDowncasedVariants
#
# This class can symlink all upcased program names to downcased
# variants. So, "/Programs/GLib" would become "/Programs/Glib".
# The first character is ignored though. 
#
# Usage example:
#   RBT::SymlinkAllUpcasedProgramsToDowncasedVariants.new
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_all_upcased_programs_to_downcased_variants.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkAllUpcasedProgramsToDowncasedVariants < Base # === RBT::Symlinkallupcasedprogramstodowncasedvariants.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
    @n_programs_were_modified = 0
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    all_program_entries = Dir["#{programs_dir?}*"].select {|entry|
      File.directory? entry
    }
    check_these_entries_for_upcased_program_names(all_program_entries)
  end

  # ========================================================================= #
  # === check_these_entries_for_upcased_program_names
  # ========================================================================= #
  def check_these_entries_for_upcased_program_names(i)
    if i.empty?
      do_notify_the_user_that_no_program_was_symlinked
    else
      i.each {|entry|
        dirname = File.basename(entry)
        dirname[0,1] = dirname[0,1].upcase
        downcased_variant = dirname.downcase
        downcased_variant[0,1] = downcased_variant[0,1].upcase
        if dirname == downcased_variant # Then all is fine.
        else
          @n_programs_were_modified += 1
          opnn; e 'Found an upcased program name: '+sfancy(entry)
          properly_downcase_this_program(entry)
        end
      }
    end
    if @n_programs_were_modified and !i.empty?
      do_notify_the_user_that_no_program_was_symlinked
    end
  end

  # ========================================================================= #
  # === properly_downcase_this_program
  #
  # This is the action that will do the correct symlink-operation.
  # ========================================================================= #
  def properly_downcase_this_program(real_existing_name)
    dirname  = File.dirname(real_existing_name)
    basename = File.basename(real_existing_name).downcase
    basename[0,1] = basename[0,1].upcase
    new_name = File.join(dirname, basename)
    new_name.delete!('-') if new_name.include? '-'
    opnn; e "Symlinking `#{sfancy(real_existing_name)}"\
            "` towards `#{sfancy(new_name)}`."
    symlink(real_existing_name, new_name) { :be_quiet }
  end

  # ========================================================================= #
  # === do_notify_the_user_that_no_program_was_symlinked
  # ========================================================================= #
  def do_notify_the_user_that_no_program_was_symlinked
    opnn; e 'No program was symlinked. This either means that you have had'
    opnn; e "no entry under #{sfancy(programs_dir?)}, that you have "\
            "had no program that could"
    opnn; e 'be symlinked to its non-upcased variant, or that all programs '\
            'that'
    opnn; e 'had an upcased variant were already symlinked.' 
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkAllUpcasedProgramsToDowncasedVariants.new(ARGV)
end # symlink_all_upcased_programs_to_downcased_variants