#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkGlibSchema
#
# This small class will symlink glib-schema files into the 
# directory /usr/share/glib-2.0/schemas/.
#
# The input to this class should be the directory that contains
# the schema-files that you wish to symlink.
#
# Usage example:
#   RBT::SymlinkGlibSchema.new
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_glib_schema.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkGlibSchema < Base # === RBT::SymlinkGlibSchema.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === GLIB_SCHEMA_DIRECTORY
  # ========================================================================= #
  GLIB_SCHEMA_DIRECTORY =
    '/usr/share/glib-2.0/schemas/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    if block_given?
      yielded = yield
      case yielded # case tag
      when :do_not_report_when_anything_is_missing
        @report_when_anything_is_missing = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @xml_files = []
    @report_when_anything_is_missing = true
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === obtain_all_xml_files
  # ========================================================================= #
  def obtain_all_xml_files
    @xml_files = Dir["#{input?}*.xml"]
  end

  # ========================================================================= #
  # === glib_schema_directory?
  # ========================================================================= #
  def glib_schema_directory?
    GLIB_SCHEMA_DIRECTORY
  end; alias target? glib_schema_directory? # === target?

  # ========================================================================= #
  # === consider_symlinking_all_found_xml_files
  # ========================================================================= #
  def consider_symlinking_all_found_xml_files
    _ = @xml_files
    if _.empty?
      if @report_when_anything_is_missing
        opnn; e "No #{royalblue('.xml')} file found under #{sfancy(input?)}."
      end
    else
      # ===================================================================== #
      # Ok, we found at the least one .xml file, so symlink these.
      # ===================================================================== #
      opnn; e 'At the least one .xml file was found, which will next be'
      opnn; e "symlinked into the #{sdir(target?)} hierarchy."
      _.each {|this_xml_file|
        new_target = "#{target?}#{File.basename(this_xml_file)}"
        if File.exist?(new_target) and File.file?(new_target)
          delete_file(new_target)
        end
        symlink(this_xml_file, new_target)
      }
      glib_compile_schemas
    end
  end

  # ========================================================================= #
  # === glib_compile_schemas
  # ========================================================================= #
  def glib_compile_schemas
    this_command = "glib-compile-schemas #{glib_schema_directory?}"
    system this_command
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_all_xml_files
    consider_symlinking_all_found_xml_files
  end

  # ========================================================================= #
  # === RBT::SymlinkGlibSchema[]
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkGlibSchema.new(ARGV)
end # symlink_glib_schema
    # symlink_glib_schema /Programs/Matesystemtools/Current/share/glib-2.0/schemas/
    # symlink_glib_schema /Programs/Caja/Current/share/glib-2.0/schemas/
    # symlink_schema      /Programs/Gedit/Current/share/glib-2.0/schemas/