#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ToCurrent
#
# This class is able to create a new 'Current' symlink in the /Programs/
# hierarchy (or any other hierarchy).
#
# You can use a short input such as "python" but you can also pass
# a full path to this class, such as "/Programs/Python/3.6.2". The
# latter means that this version will become the new target of the
# Current symling. I personally recommend to provide the full path
# since there will be less ambiguity that way.
#
# Usage example:
#   RBT::ToCurrent.new
# =========================================================================== #
# require 'rbt/utility_scripts/to_current.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/symlink_this_program/initialize.rb'

module RBT

class ToCurrent < Base # === RBT::ToCurrent.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  #
  # The input to this method should be a String such as
  # "/Programs/Python/3.6.2/".
  # ========================================================================= #
  def initialize(
      program_version = nil,
      run_already     = true
    )
    reset
    set_symlink_this_program_version(program_version)
    if block_given?
      yielded = yield
      case yielded
      when :do_not_register
        @also_do_register_into_the_yaml_database = false
      else
        if yielded.is_a? Hash
          if yielded.has_key? :use_this_symlink_program_instance
            set_symlink_program(yielded.delete(:use_this_symlink_program_instance))
          end
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    set_program_name # Must initialize it at the least once.
    @also_do_register_into_the_yaml_database = true
    @symlink_program = RBT::SymlinkThisProgram.new(nil, :do_not_run_yet)
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_symlink_program
  #
  # This method can be used to assign towards the symlink-program that
  # will be used. For example, class RBT::Compile can pass its own
  # modified variant into this method here, thus being able to control
  # what class RBT::ToCurrent is doing at the symlink-progam step.
  # ========================================================================= #
  def set_symlink_program(i)
    @symlink_program = i
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @symlink_this_program_version
  end

  # ========================================================================= #
  # === set_program_name
  #
  # The setter of the program name.
  # ========================================================================= #
  def set_program_name(i = nil)
    i = i.first if i.is_a? Array
    if i.nil?
      # ===================================================================== #
      # In this case, we will try to obtain the program name from
      # the target directory.
      # ===================================================================== #
      _ = Dir.pwd
      if _.include? programs_dir? and
         (_.count('/') > 1)
         i = _
      end
    end
    i = i.to_s.dup
    i = File.basename(i) if i.include? '/'
    i.capitalize!
    @program_name = i
  end

  # ========================================================================= #
  # === set_symlink_this_program_version
  #
  # This setter keeps track of the particular program version that
  # we will use. For example, if we have an input such as 
  # "/Programs/Python/3.6.2", then the program_version will be
  # 3.6.2.
  # ========================================================================= #
  def set_symlink_this_program_version(i)
    i = i.first if i.is_a? Array
    # ======================================================================= #
    # === Safeguard against nil-entries
    # ======================================================================= #
    if i.nil? # fetch random entry
      entries = Dir['*'] # Fetch all entries first.
      if entries.size > 1 # Try to find a number
        i = entries.select {|line| line =~ /\d+/ }.first
      else
        i = entries.first
      end
    end
    # ======================================================================= #
    # === Get rid of trailing '/' tokens.
    # ======================================================================= #
    if i.end_with? '/'
      i = i.dup if i.frozen?
      i.chop!
    end
    if i.include? programs_dir? and
       (i =~ /\d+/) and
       (i.count('/') > 1) # Has more than one '/' token.
      # ===================================================================== #
      # In this case, we assume that the user has input something
      # such as "/Programs/Python/3.6.2".
      # ===================================================================== #
      splitted = i.split('/').reject(&:empty?)
      set_program_name(splitted[1])
      i = splitted[2]
    end
    @symlink_this_program_version = i
  end; alias set_program_version set_symlink_this_program_version # === set_program_version

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end

  # ========================================================================= #
  # === current_target?
  #
  # This method will assemble the target residing under the /Programs
  # hierarchy.
  # ========================================================================= #
  def current_target?
    programs_dir?+program_name?+'/Current'
  end

  # ========================================================================= #
  # === full_program_version_target?
  #
  # This method is assumed to point to a directory.
  # ========================================================================= #
  def full_program_version_target?
    "#{programs_dir?}#{program_name?}/#{program_version?}/"
  end

  # ========================================================================= #
  # === considering_deleting_the_current_symlink_if_it_exists
  # ========================================================================= #
  def considering_deleting_the_current_symlink_if_it_exists
    _ = current_target? # Get the symlink target.
    if File.symlink? _
      opnn; e "There already is a symlink at `#{sfile(_)}`. "\
              "It will be removed next."
      delete(_)
    end
  end

  # ========================================================================= #
  # === do_create_the_new_symlink
  # ========================================================================= #
  def do_create_the_new_symlink
    # ======================================================================= #
    # Next, create a new symlink.
    # ======================================================================= #
    _ = current_target? # Get the symlink target.
    opnn; e "Creating a new symlink at `#{sfile(_)}`."
    new_target = full_program_version_target?
    opnn; e "This symlink will point towards `#{sfile(new_target)}`."
    if File.directory? new_target
      symlink(program_version?, _)
      # ===================================================================== #
      # Consider invoking rnsymc as well. 
      # ===================================================================== #
      rnsymc(new_target)
    end
  end

  # ========================================================================= #
  # === rnsymc
  #
  # This method delegates towards class RBT::SymlinkThisProgram.
  # ========================================================================= #
  def rnsymc(
      i = full_program_version_target?
    )
    @symlink_program.set_use_this_program(i)
    if @also_do_register_into_the_yaml_database
      # pass through in this case
    else
      @symlink_program.do_not_register
    end
    @symlink_program.run
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    considering_deleting_the_current_symlink_if_it_exists
    do_create_the_new_symlink
  end

  # ========================================================================= #
  # === RBT::ToCurrent[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ToCurrent.new(ARGV)
end # to_current python
    # to_current /Programs/Python/3.6.2