#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Url
#
# Just a small helper-class to output the URLs of a program in question.
# =========================================================================== #
# require 'rbt/utility_scripts/url.rb'
# RBT.return_url1_of_this_program
# =========================================================================== #
require 'xorg_buffer/module'
require 'rbt/aliases/aliases.rb'
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class Url < Base # === RBT::Cookbooks::Url

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_find_this_program(i)
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet
        @be_verbose = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @be_verbose = true # Explicitely set it here.
    @dataset = nil
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_find_this_program
  #
  # Use this method to designate which program we wish to find the URL for.
  # ========================================================================= #
  def set_find_this_program(i)
    i = i.to_s.strip
    i.delete!('-') if i.include? '-' # Since 05.11.2011
    i.delete!('/') if i.include? '/'
    i.sub!(/\.yml$/,'') if i.end_with? '.yml'
    case i # case tag
    when 'PWD','pwd'
      i = File.basename(Dir.pwd+'/')
    end unless File.exist? i
    i = i.downcase # Since 24.09.2014.
    # ======================================================================= #
    # Check for a Cookbook-alias next:
    # ======================================================================= #
    if RBT.is_an_alias?(i)
      i = RBT.find_cookbook_alias_for(i)
    end unless RBT.has_this_program?(i) # This check is necessary so that "ruby" won't become rubysqlite.
    # ======================================================================= #
    # If the input was not found, then we also check for 'lib' as prefix.
    #
    # This will correct commands such as "url memcached" into
    # "url libmemcached".
    # ======================================================================= #
    unless RBT.does_include? i
      # ===================================================================== #
      # Check a +lib variant here.
      # ===================================================================== #
      if RBT.does_include? 'lib'+i
        i.prepend('lib')
      # ===================================================================== #
      # Check a -lib variant here.
      # ===================================================================== #
      elsif RBT.does_include? i.sub(/^lib/,'')
        i.sub!(/^lib/,'')
      end
    end
    if i.empty? and
       is_on_roebe? and
       return_pwd.include?(source_directory?)
      i = File.basename(return_pwd)
    end
    @find_this_program = i
  end; alias search_for_url set_find_this_program # === search_for_url

  # ========================================================================= #
  # === tell_the_user_what_we_seek
  # ========================================================================= #
  def tell_the_user_what_we_seek(
      be_verbose = be_verbose?
    )
    if be_verbose
      opnn; e "Seeking URLs to `#{sfancy(search_for?)}` next."
    end
  end

  # ========================================================================= #
  # === search_for?
  # ========================================================================= #
  def search_for?
    @find_this_program
  end; alias find? search_for? # === find?

  # ========================================================================= #
  # === determine_dataset
  # ========================================================================= #
  def determine_dataset(
      search_for = search_for?
    )
    if RBT.does_include?(search_for) # In this case, we can search for it.
      @dataset = RBT::Cookbooks::Cookbook.new(search_for) { :bypass_menu_check }
    else # Else it was not included.
      opnn; e "The search_term `#{sfancy(search_for)}` is not included."
      exit
    end
  end

  # ========================================================================= #
  # === array_urls?
  #
  # This method will return the URLs in Array format.
  # ========================================================================= #
  def array_urls?
    [url1, url2]
  end; alias result array_urls? # === result

  # ========================================================================= #
  # === url1?
  # ========================================================================= #
  def url1?
    @dataset.url1?
  end; alias return_main_url url1? # === return_main_url
       alias url1            url1? # === url1

  # ========================================================================= #
  # === url2?
  # ========================================================================= #
  def url2?
    @dataset.url2?
  end; alias return_second_url url2? # === return_second_url
       alias url               url2? # === url
       alias url?              url2? # === url?
       alias url2              url2? # === url2

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    @dataset.homepage?
  end; alias homepage homepage? # === homepage

  # ========================================================================= #
  # === consider_reporting_url2
  #
  # We will try to report the second URL, aka url2.
  #
  # As of December 2017, if url2 does not exist BUT a homepage
  # url exists, we will use that one instead.
  # ========================================================================= #
  def consider_reporting_url2
    if url2? # <- Prevent nil entries here.
      # ===================================================================== #
      # Check whether there is a homepage-entry, which is not empty - but
      # only use it if url2? is empty. This allows us to use the homepage
      # entry when the cookbook file in question does not have a url2
      # entry set.
      # ===================================================================== #
      if homepage? and !homepage.empty? and url2?.empty?
        e "homepage: #{homepage?.to_s}"
        use_this_as_xorg_buffer(homepage?)
      elsif !url2?.empty?
        result = ''.dup
        result << 'url2: ' if be_verbose?
        result << url2?.to_s
        e result
        use_this_as_xorg_buffer(url2?)
      end
    end
  end

  # ========================================================================= #
  # === use_this_as_xorg_buffer
  # ========================================================================= #
  def use_this_as_xorg_buffer(i)
    XorgBuffer[i]
  end

  # ========================================================================= #
  # === feedback_urls
  # ========================================================================= #
  def feedback_urls
    if url1?.empty?
      e "url.rb: We did not find `"+search_for?+"`." if be_verbose?
    else 
      use_this_as_xorg_buffer(url1?)
      result = ''.dup
      result << 'url1: ' if be_verbose?
      result << url1?.to_s
      if be_verbose?
        e result
        consider_reporting_url2 # Delegate to this method.
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    tell_the_user_what_we_seek
    determine_dataset
    feedback_urls
  end

end; end

# =========================================================================== #
# === RBT.return_url1_of_this_program
#
# This method will return the url1: entry of the given program at hand.
# =========================================================================== #
def self.return_url1_of_this_program(i = 'ruby')
  RBT::Cookbooks::Url.new(i) { :be_quiet }.url1?
end

end

if __FILE__ == $PROGRAM_NAME
  ARGV.each {|entry| RBT::Cookbooks::Url.new(entry) }
  # puts RBT.return_url1_of_this_program('ruby')
end # url ruby