#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::UseFlags
#
# This class will provide UseFlags data.
#
# You pass data into the ".is_included?" method and this method will then
# store information in the main @string (which is a getter method as well)
# 
# Intended and documented Behaviour of this class:
#   - If a USE flag is not registered, we must warn the user, but
#     we will continue nonetheless.
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class UseFlags < Base

  # ========================================================================= #
  # === REGISTERED_COMPILE_FLAGS
  #
  # bl $RBT/YAML/registered_compile_flags.yml
  # Right now it is hardcoded.
  # ========================================================================= #
  REGISTERED_COMPILE_FLAGS = YAML.load_file(
    "#{RBT.yaml_directory?}registered_compile_flags.yml"
  )

  # ========================================================================= #
  # === initialize
  #
  # Note: the constant USE_FLAGS can be found in the shared code part, in
  # a yaml file. REGISTERED_COMPILE_FLAGS can be found in RBT_Base.rb
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @environment_use_flags = nil
    dataset = YAML.load_file(FILE_USE_FLAGS)
    @use_flags = dataset['USE'].strip # {"USE"=>"-arts +mp3 +ogg"}
    @registered_compile_flags = REGISTERED_COMPILE_FLAGS
  end

  # ========================================================================= #
  # === sanitize_registered_compile_flags
  #
  # This is needed to make proper default values for the '+' use flags.
  #
  # It is run once on startup. Afterwards it does not need to be run again.
  # ========================================================================= #
  def sanitize_registered_compile_flags
    @registered_compile_flags.dup.each_pair { |k, v|
      @registered_compile_flags[ k[1..-1] ] = v if k.include? '+' # k[1..-1] includes all but leading +
    }
  end

  # ========================================================================= #
  # === check_for_environment
  #
  # Here, we must check if the user did set to use a specific USE 
  # environment flag.
  # ========================================================================= #
  def check_for_environment
    @environment_use_flags = ENV['USE'].dup if ENV['USE']
  end

  # ========================================================================= #
  # === use_flags?
  #
  # This will simply return all flags that are available. (Rarely used.)
  # ========================================================================= #
  def use_flags?
    @use_flags
  end

  # ========================================================================= #
  # === environment_use_flags?
  #
  # Stores ENV stuff.
  # ========================================================================= #
  def environment_use_flags?
    @environment_use_flags
  end; alias environment_use_flags environment_use_flags? # === environment_use_flags

  # ========================================================================= #
  # === is_included?
  #
  # Find out if the use-flag we want to use is already registered.
  # ========================================================================= #
  def is_included?(
      this_use_flag = use_flags?
    )
    @string = ''.dup # Empty it again here.
    if @registered_compile_flags.keys.include? this_use_flag
      @string << ' '+@registered_compile_flags[this_use_flag]
    elsif this_use_flag.include? ' '
      this_use_flag.split(' ').each { |splitted|
        if @registered_compile_flags.keys.include? splitted
          @string << ' '+@registered_compile_flags[splitted]
        end
      }
    else # Else it may be enable or disable
         # We keep that consistent.
      if    this_use_flag.include? 'enable'
        is_included? value.gsub(/enable/,'with')
      elsif this_use_flag.include? 'disable'
        is_included? value.gsub(/disable/,'without')
      end
    end
    @string.strip!
  end

  # ========================================================================= #
  # === string?
  # ========================================================================= #
  def string?
    @string # Use this string as the return value.
  end; alias string string?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    sanitize_registered_compile_flags    
    check_for_environment
    is_included?
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::UseFlags.new
  pp _.use_flags?
  puts; puts 'Feedback: - Our translated USEflags are these:'
  puts '       '+_.string
end # useflags