#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForCorrectDependencies
#
# This class will check whether the entries in the cookbook files do all
# happen to have valid, existing dependencies. If the file is not valid,
# the class will output this information on the commandline. That way
# the problem(s) can be fixed.
#
# Usage example:
#   RBT::Cookbooks::CheckForCorrectDependencies.new
# =========================================================================== #
# require 'rbt/validation/check_for_correct_dependencies.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/cookbooks/class/class.rb'

module RBT

module Cookbooks

class CheckForCorrectDependencies < RBT::Base # === RBT::Cookbooks::CheckForCorrectDependencies.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = nil)
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end; alias input? commandline_arguments? # === input?

  # ========================================================================= #
  # === format_program
  # ========================================================================= #
  def format_program(i)
    _ = "#{i}:"
    _ = _.ljust(22)
    splitted = _.split(':')
    splitted[0] = sfancy(splitted.first)
    _ = splitted.join(':')
    _
  end

  # ========================================================================= #
  # === check_for_every_entry
  #
  # This method will check for every available (registered) program.
  # ========================================================================= #
  def check_for_every_entry
    opnn; e 'Now checking every program. This may take a while.'
    available_programs?.each {|program|
      # ===================================================================== #
      # The next line is very verbose, so it may be best to enable it only
      # for debugging purposes really.
      # ===================================================================== #
      # opnn; e 'Working on the program `'+sfancy(program)+'` next.'
      # ===================================================================== #
      # We have two ways for the next line - we will either instantiate a
      # new Cookbooks object - or we will make use of an already existing,
      # expanded .yml file that holds the dataset for the program at hand.
      # ===================================================================== #
      if expanded_directory_exists? and
         File.exist?(
           path_to_this_expanded_cookbooks_dataset(program)
         )
        dataset = YAML.load_file(
          path_to_this_expanded_cookbooks_dataset(program)
        )
        deps = dataset['required_deps_on']
      else
        dataset = RBT::Cookbooks::Cookbook.new(program) { :bypass_menu }
        deps    = dataset.deps?
      end
      result  = deps.all? {|entry| RBT.really_does_include?(entry) }
      if result # Ok, in this case all entries were found.
      else # else it was not found, so work on that.
        deps.each {|entry|
          if entry.include? ' '
            entry = entry.split(' ').first.strip
          end
          unless RBT.really_does_include? entry
            opnn; e "#{format_program(program)}No "\
                    "entry for `#{simp(entry)}` was found."
          end
        }
      end
    }
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e 'Currently this class has no special commandline flags.'
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === correct_deps? --help
      # ===================================================================== #
      when /^help$/
        show_help
      else
        opnn; e 'Unregistered commandline argument: '+sfancy(i)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    unless commandline_arguments?.empty?
      menu
    else # else check for every entry
      check_for_every_entry
    end
  end

  # ========================================================================= #
  # === RBT::Cookbooks::CheckForCorrectDependencies[]
  # ========================================================================= #
  def self.[](i = '')
    self.new
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForCorrectDependencies.new(ARGV)
end # correct_dependencies