#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForInvalidEntriesInThisCookbook
#
# This class will check whether a given cookbook file has any
# invalid entries.
#
# If an entry is not valid then this will be report to the user.
# =========================================================================== #
# require 'rbt/validation/check_for_invalid_entries_in_this_cookbook.rb'
# =========================================================================== #
require 'rbt/base/base.rb'
require 'rbt/aliases/cookbook_aliases.rb'

module RBT

module Cookbooks

class CheckForInvalidEntriesInThisCookbook < RBT::Base # === RBT::Cookbooks::CheckForInvalidEntriesInThisCookbook

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      check_these_entries = ARGV,
      run_already         = true
    )
    reset
    set_check_these_entries(
      check_these_entries
    )
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @namespace = NAMESPACE
  end

  # ========================================================================= #
  # === set_check_these_entries
  # ========================================================================= #
  def set_check_these_entries(i)
    @check_these_entries = [i].flatten.compact
    sanitize_the_main_entries
  end

  # ========================================================================= #
  # === sanitize_the_main_entries
  # ========================================================================= #
  def sanitize_the_main_entries
    case @check_these_entries.first # case tag
    when /^-?-?everything$/i
      @check_these_entries = available_programs?.map {|line| "#{line}.yml" }
    end
    @check_these_entries.map! {|entry|
      if entry.end_with? '.yml'
        unless File.exist? entry
          entry = return_location_to_this_programs_yaml_file(entry)
        end
      end
      entry
    }
  end

  # ========================================================================= #
  # === batch_process_the_entries
  # ========================================================================= #
  def batch_process_the_entries
    @check_these_entries.each {|this_cookbook_entry|
      @dataset = File.read(this_cookbook_entry)
      @first_line = @dataset.split(N).first.strip.delete(':')
      # ===================================================================== #
      # We have to check for registered entries.
      # ===================================================================== #
      check_if_the_first_entry_is_the_name_of_the_file(this_cookbook_entry)
      check_whether_all_left_entries_are_valid
    }
  end

  # ========================================================================= #
  # === check_whether_all_left_entries_are_valid
  #
  # This method will check whether the left entries of the file are
  # all valid.
  # ========================================================================= #
  def check_whether_all_left_entries_are_valid
    valid_entries = array_registered_and_valid_cookbook_entries?
    splitted = @dataset.split(N)
    these_entries_were_found = splitted.select {|entry|
      entry.start_with?(' ') and
     !entry.start_with?(' #') and
     !entry.start_with?(' - ') and
     !entry.start_with?('  ')
    }.map {|line|
      line.strip!
      if line.include? ':'
        line = line.split(':').first
      end
      line.strip!
      line
    }
    # ======================================================================= #
    # Next, check each of these entries:
    # ======================================================================= #
    these_entries_were_found.each {|check_this_entry|
      if valid_entries.include? check_this_entry
      else
        opnn; e crimson('Not registered entry called ')+
                sfancy(check_this_entry)+
                crimson('.')
      end
    }
  end

  # ========================================================================= #
  # === array_registered_and_valid_cookbook_entries?
  # ========================================================================= #
  def array_registered_and_valid_cookbook_entries?
    YAML.load_file(
      file_specification_of_registered_cookbook_entries
    ).keys
  end

  # ========================================================================= #
  # === check_if_the_first_entry_is_the_name_of_the_file
  # ========================================================================= #
  def check_if_the_first_entry_is_the_name_of_the_file(i)
    i = File.basename(i)
    name_of_the_file_without_extension = i.sub(/#{File.extname(i)}$/,'')
    if @first_line == name_of_the_file_without_extension
    else
      opnn { :no_colon}
      e
      e crimson('The name of the file is not the same as the filename.')
      e crimson('This should be corrected.')
      e
      e slateblue(@first_line)+
        lightgreen(' != ')+
        royalblue(name_of_the_file_without_extension)
      e
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    batch_process_the_entries
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(ARGV)
end # CheckForInvalidEntriesInThisCookbook
    # CheckForInvalidEntriesInThisCookbook libebml.yml
    # CheckForInvalidEntriesInThisCookbook --everything