#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/compile/menu.rb'
# =========================================================================== #
require 'rbt/compile/base/compile_base.rb'
require 'rbt/compile/statistics.rb'
require 'rbt/validation/scan_for_missing_last_update_entries.rb'
require 'rbt/validation/validate_cookbook_aliases.rb'
require 'rbt/linux/slackware/create_slackware_package.rb'
require 'rbt/toplevel_methods/create_log_directory.rb'
require 'rbt/toplevel_methods/extract.rb'
require 'rbt/toplevel_methods/misc.rb'
require 'rbt/utility_scripts/binary_name_belongs_to_which_program.rb'
require 'rbt/utility_scripts/create_program_version_url_file.rb'
require 'rbt/utility_scripts/find_all_archive_types.rb'
require 'rbt/utility_scripts/suggest_cookbook_for.rb'
require 'rbt/utility_scripts/remove_symlinks.rb'
require 'rbt/check_for_updates/update_kde_porting_aids.rb'

module RBT

class Compile < RBT::CompileBase # === RBT::Compile

  # ========================================================================= #
  # === menu                                                       (menu tag)
  #
  # This menu may not have an else-clause, so we have to handle any other
  # cases elsewhere.
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    elsif i.is_a? Hash
      # ===================================================================== #
      # Handle Hashes next. Such a Hash may look like this:
      #   { :prefix=>"/usr/" }
      # ===================================================================== #
      if i.has_key? :prefix
        set_user_prefix(
          i.delete(:prefix)
        )
      end
    else
      case i # case tag, cas tag
      # ===================================================================== #
      # === Prioritize on the traditional /usr/bin/ $PATH
      #
      # Invocation example:
      #
      #   ry fluxbox --traditional-path
      #
      # ===================================================================== #
      when /^-?-?traditional(_|-)?path$/
        use_traditional_env_path
      # ===================================================================== #
      # === Create the programs URL file
      #
      # This entry point allows us to create the local programs-url file.
      #
      # Invocation examples:
      #
      #   rbt --create-programs-url-file
      #   rbt --create-cookbooks-url-file
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?programs(-|_| )?url(-|_| )?file$/,
           /^-?-?create(-|_| )?programs(-|_| )?url$/,
           /^-?-?create(-|_| )?programs(-|_| )?file$/,
           /^-?-?programs(-|_| )?url(-|_| )?file$/,
           /^-?-?programs(-|_| )?url$/,
           /^-?-?generate(-|_| )?programs(-|_| )?url$/,
           /^-?-?create(-|_| )?cookbooks(-|_| )?url(-|_| )?file$/
        RBT.create_programs_url_file
        exit_program
      # ===================================================================== #
      # === Use a profile/decorator for the configure options
      #
      # This entry point allows us to use pre-defined configure options
      # for some programs.
      #
      # Invocation examples:
      #
      #   ry gcc --profile=gcc_lfs_pass1
      #   ry qt --use-profile=qt trad
      #
      # ===================================================================== #
      when /^-?-?profile=(.+)$/i,
           /^-?-?use(-|_)?profile=(.+)$/i
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        try_to_use_this_profile(_)
      # ===================================================================== #
      # === Query to which program a particular binary belongs to
      #
      # This entry point allows us to find out to which particular
      # program a specific binary belongs to.
      #
      # Invocation examples:
      #
      #   ry --this-binary?=shar
      #   ry --binary=env
      #
      # ===================================================================== #
      when /^-?-?this(-|_| )?binary\?=(.+)$/i, # For $2
           /^-?-?binary=(.+)$/i                # For $1
        _ = $1.to_s.dup # Default.
        _ = $2.to_s.dup if $2
        RBT::BinaryNameBelongsToWhichProgram.new(_)
        exit_program
      # ===================================================================== #
      # === Query to which program a binary belongs to
      #
      # This entry point allows us to query to which program a given
      # locally existing binary belongs to. That way you can e. g.
      # query the /usr/bin/ hierarchy for programs, and the class here
      # will report to which program it belongs to.
      #
      # Invocation examples:
      #
      #   rbt --query-programs
      #   rbt --query-programs?
      #   rbt --attribute?
      # 
      # ===================================================================== #
      when /^-?-?query(-|_| )?programs$/i,
           /^-?-?query(-|_| )?programs\??$/i,
           /^-?-?attribute\??$/i
        require 'rbt/utility_scripts/query_binary_to_package.rb'
        RBT.query_binary_to_package('/usr/bin/')
      # ===================================================================== #
      # === Query the existance of a binary name
      #
      # Query whether a binary program exists or not of the given name.
      #
      # Invocation examples:
      #
      #   ry file binary?
      #   ry esetroot binary?
      #   ry xgc binary?
      #   ry xgc --binary?
      #
      # ===================================================================== #
      when *ARRAY_HELP_QUERY_THE_EXISTANCE_OF_A_BINARY_NAME
        binary?(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Update binaries and libraries of a given program automatically
      #
      # Invocation example:
      #
      #   rbt --autoupdate=htop
      #
      # ===================================================================== #
      when /^-?-?autoupdate=(.+)$/i
        this_program = $1.to_s.dup # Default.
        update_binaries_of_this_program(this_program)
        update_libraries_of_this_program(this_program)
      # ===================================================================== #
      # === Feedback all registered binaries
      #
      # This entry point allows us to feedback all registered binaries.
      #
      # Invocation example:
      #   rbt --all-binaries?
      # ===================================================================== #
      when /^-?-?all(-|_| )?binaries\??$/,
           /^-?-?registered(-|_| )?binaries\??$/
        feedback_all_registered_binaries_of_all_programs
        exit_program
      # ===================================================================== #
      # === Check for binary duplicates in the current working directory
      #
      # Usage examples:
      #
      #   rbt --check-for-binary-duplicates
      #   rbt --check-for-duplicates
      #   rbt --duplicates-in-pwd?
      #
      # ===================================================================== #
      when /^-?-?check(-|_)?for(-|_)?binary(-|_)?duplicates$/i,
           /^-?-?check(-|_)?for(-|_)?duplicates$/i,
           /^-?-?duplicates(-|_)?in(-|_)?pwd\??$/i
        check_for_binary_duplicates_in_this_directory(Dir.pwd)
      # ===================================================================== #
      # === Show binary duplicates
      #
      # Invocation example:
      #   rbt gettext --show-binary-duplicates
      #   rbt gtk+ --show-binary-duplicates
      # ===================================================================== #
      when /^-?-?show(-|_)?binary(-|_)?duplicates$/,
           /^-?-?duplicates$/ # <- This variant is experimental; may be changed one day.
        try_to_show_all_binary_duplicates_of_this_program(
          compile_which_program?
        )
        exit_program
      # ===================================================================== #
      # === Check for binary duplicates under /usr/bin/
      #
      # Via the following invocation you can check for binary duplicates
      # in the /usr/bin/ hierarchy.
      #
      # Invocation examples:
      #
      #   rbt --binary-duplicates?
      #   rbt --duplicates?
      #
      # ===================================================================== #
      when *ARRAY_CHECK_FOR_BINARY_DUPLICATES_OPTIONS
        do_check_for_binary_duplicates
        exit_program
      # ===================================================================== #
      # === Update binaries of a given program
      #
      # This entry point allows us to automatically update the
      # binaries of a compiled, AppDir-installed program.
      #
      # Invocation examples:
      #
      #   rbt --update-binaries-of=poppler
      #   rbt --update-binaries-for=poppler
      #   rbt --autoupdate=opencv
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?binaries(-|_| )?of=(.+)$/i,
           /^-?-?update(-|_| )?binaries(-|_| )?for=(.+)$/i
        this_program = $1.to_s.dup # Default.
        this_program = $3.to_s.dup if $3 # Only for $3.
        update_binaries_of_this_program(this_program)
      # ===================================================================== #
      # === Update libraries of a given program
      #
      # This entry point allows us to automatically update the
      # libraries of a compiled, AppDir-installed program.
      #
      # Invocation examples:
      #
      #   rbt --update-libraries-of=poppler
      #   rbt --update-libraries-for=poppler
      #   rbt --autoupdate=opencv
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?libraries(-|_| )?of=(.+)$/i,
           /^-?-?update(-|_| )?libraries(-|_| )?for=(.+)$/i
        this_program = $1.to_s.dup # Default.
        this_program = $3.to_s.dup if $3 # Only for $3.
        update_libraries_of_this_program(this_program)
      # ===================================================================== #
      # === Feedback whether we will strip the produced binaries or not
      #
      # This is a simple query-method, to determine whether RBT will
      # run the "strip" command on the resulting binaries or whether
      # it will not.
      #
      # Invocation examples:
      #
      #   rbt --use-strip?
      #   rbt --will-we-strip?
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?strip\??$/i,
           /^-?-?strip\??$/i,
           /^-?-?will(-|_)?we(-|_)?use(-|_)?strip\??$/i,
           /^-?-?will(-|_)?we(-|_)?strip\??$/i
        feedback_whether_we_will_strip_or_whether_we_will_not
        exit_program
      # ===================================================================== #
      # === Show the available profiles
      #
      # This entry point allows us to show the available profiles.
      #
      # Invocation example:
      #
      #   rbt --available-profiles?
      #
      # ===================================================================== #
      when /^-?-?available(-|_)?profiles\??$/i,
           /^-?-?show(-|_)?available(-|_)?profiles$/i
        show_available_profiles
        exit_program
      # ===================================================================== #
      # === Ad-hoc install palemoon
      #
      # This entry point is mostly for my home system, so that I can
      # quickly install palemoon.
      #
      # Invocation example:
      #
      #   rbt --install-palemoon
      #
      # ===================================================================== #
      when /^-?-?install(-|_)?palemoon$/i
        install_palemoon
        exit_program
      # ===================================================================== #
      # === Easily compile the KDE5 foundation module
      #
      # This entry point allows you to compile all of the KDE5 foundation
      # programs.
      #
      # Invocation examples:
      #
      #   rbt --compile-kde-framework
      #   rbt --compile-kde-frameworks
      #   rbt --compile-kde-foundation
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?kde(-|_| )?frameworks?$/i,
           /^-?-?compile(-|_| )?kde(-|_| )?foundation$/i,
           /^-?-?kde(-|_| )?frameworks$/i,# <- This alias may change at a later time.
           /^-?-?kde(-|_| )?foundation$/i,
           /^-?-?kde(-|_| )?base$/i
        opnn; e 'All KDE5 foundation programs will be compiled next.'
        set_compile_these_programs(return_kde5_foundation)
      # ===================================================================== #
      # === Query whether this program can be compiled statically
      #
      # This entry point allows us to query whether the given program at
      # hand can be compiled statically or whether it can not.
      #
      # Invocation example:
      #
      #   rbt tar --can-be-compiled-statically?
      #
      # ===================================================================== #
      when /^-?-?can(-|_| )?be(-|_| )?compiled(-|_| )?statically\??$/i
        verbose_query_whether_this_program_can_be_compiled_statically
        exit_program
      # ===================================================================== #
      # === Compile all KDE5 porting aids components
      #
      # This entry point allows us to update the KDE porting aids from
      # within RBT itself.
      #
      # Invocation example:
      #
      #   rbt --update_porting_aids
      #
      # ===================================================================== #
      when /^-?-?update(-|_| )?porting(-|_| )?aids$/i
        RBT::Cookbooks::UpdateKdePortingAids.new
        exit_program
      # ===================================================================== #
      # === Compile all KDE5 porting aids components
      #
      # The KDE5 porting aids include components such as "kdelibs4support",
      # "khtml", "kjs", "kjsembed", "kmediaplayer" or "kross".
      #
      # Invocation examples:
      #
      #   rbt --porting-aids
      #   rbt --compile-kde-porting-aids
      #
      # ===================================================================== #
      when /^-?-?porting(-|_| )?aids$/,
           /^-?-?kde(-|_| )?porting(-|_| )?aids$/,
           /^-?-?compile(-|_| )?kde(-|_| )?porting(-|_| )?aids$/
        set_compile_these_programs(return_porting_aids)
      # ===================================================================== #
      # === Compile a KDE5 porting aids component
      #
      # Invocation examples:
      #
      #   ry portingaids1
      #   ry portingaids2
      #   ry portingaids3
      #   ry portingaids4
      #   ry portingaids5
      #   ry portingaids6
      #
      # ===================================================================== #
      when /^-?-?portingaids(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_portingaids_component($1.to_s.dup)
      # ===================================================================== #
      # === Permanently set programs dir
      #
      # This compiles gcc with "language rich" configure options.
      #
      # Invocation example:
      #
      #   ry gcc --ntrad --language-rich
      #
      # ===================================================================== #
      when /^-?-?language(_|-)?rich$/i
        try_to_use_this_profile(i.to_s.delete('-_'))
      # ===================================================================== #
      # === Permanently set programs dir
      #
      # This entry point can be used to permanently set a new PROGRAMS
      # directory.
      #
      # Invocation example:
      #
      #   rbt --permanently_set_programs_dir=/pkg
      #   rbt --use-this-as-programs-dir=/pkg
      #   
      # ===================================================================== #
      when /^-?-?permanently(_|-)?set(_|-)?programs(_|-)?dir=(.+)$/i, # $4
           /^-?-?use(_|-)?this(_|-)?as(_|-)?programs(_|-)?dir=(.+)$/i # $5
        _ = $4.to_s.dup
        _ = $5.to_s.dup if $5
        permanently_set_programs_dir(_)
      # ===================================================================== #
      # === Build a Linux from Scratch system
      #
      # This entry point will delegate towards the class that will do a
      # LFS build for us.
      #
      # Invocation example:
      #
      #   ry --lfs
      #
      # ===================================================================== #
      when /^-?-?lfs$/i,
           /^linux(_|-)?from(_|-)?scratch$/i,'tolfs'
        linux_from_scratch
        exit_program
      # ===================================================================== #
      # === Compile minimal GCC but with lots of languages enabled
      #
      # Invocation example:
      #
      #   ry --gcc_minimal_but_lots_of_languages_enabled
      #
      # ===================================================================== #
      when /^-?-?gcc(-|_)?minimal(-|_)?but(-|_)?lots(-|_)?of(-|_)?languages(-|_)?enabled$/
        try_to_use_this_profile(i)
      # ===================================================================== #
      # === Download the remote BLFS page and display it
      #
      # This entry point allows you to grab the remote content of
      # the BLFS page (about a given program) and display its
      # content on the commandline.
      #
      # Invocation example:
      #
      #   ry libx11 --paste-blfs-page
      #
      # ===================================================================== #
      when /-?-?paste(-|_)?blfs(-|_)?page$/i,
           /-?-?blfs(-|_)?page$/i
        try_to_show_the_BLFS_content_of_this_program
        exit_program
      # ===================================================================== #
      # === Show BLFS Information
      #
      # This will show BLFS information.
      #
      # Invocation example:
      #   ry binutils --lfs
      #   ry binutils lfs
      # ===================================================================== #
      when /^-?-?lfs$/,
           'blfs','blfs?','show_information_about_blfs'
        show_information_about_blfs
        exit_program
      # ===================================================================== #
      # === Generate a LFS-like homepage
      #
      # This will generate a LFS-like homepage, where the individual programs
      # are written down into standalone .html pages.
      #
      # In other words, this is then a web-interface.
      #
      # The Cookbooks submodule solves this problem.
      #
      # Usage examples:
      #
      #   ry htop generate_homepage
      #   ry --generate-homepage
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_HOMEPAGE_OPTIONS
        generate_homepage
        exit_program
      # ===================================================================== #
      # === Report BLFS entries
      #
      # This entry point can be used to report all entries that have a
      # BLFS tag.
      #
      # Invocation example:
      #
      #   ry --blfs2
      #
      # ===================================================================== #
      when /^report(-|_)?all(-|_)?blfs(-|_)?entries/,
           /^-?-?blfs2$/i
        report_all_blfs_entries
        exit_program
      # ===================================================================== #
      # === Show all entries that have a BLFS entry
      #
      # Use this entry point if you wish to show all cookbook entries that
      # have a BLFS entry.
      #
      # Invocation examples:
      #
      #   ry --show-programs-with-a-blfs-entry
      #   ry --all-blfs
      #
      # ===================================================================== #
      when /^-?-?show(_|-)?programs(_|-)?with(_|-)?a(_|-)?blfs(_|-)?entry$/,
           /^-?-?show(_|-)?all(_|-)?blfs(_|-)?entries$/,
           /^-?-?all(_|-)?blfs$/
        show_programs_with_a_blfs_entry
        exit_program
      # ===================================================================== #
      # === Compile all of mate
      #
      # This entry point allows us to compile all components of the
      # mate-desktop.
      #
      # Invocation examples:
      #
      #   rbt --compile-all-of-mate
      #   rbt --all-of-mate
      #   rbt --compile-mate-desktop
      #
      # ===================================================================== #
      when /^-?-?compile(-|_)?all(-|_)?of(-|_)?mate$/i,
           /^-?-?all(-|_)?of(-|_)?mate$/i,
           /^-?-?compile(-|_)?mate(-|_)?desktop$/i,
           /^-?-?compile(-|_)?mate$/i
        do_compile_all_of_mate
      # ===================================================================== #
      # === Use /root as prefix
      #
      # This entry point allows us to use /root as the prefix value.
      #
      # Invocation example:
      #
      #   ry bison --root
      #
      # ===================================================================== #
      when /^-?-?root$/i
        set_user_prefix('/root/')
      # ===================================================================== #
      # === Use default chroot prefix + /usr/, at /Depot/Chroot/usr/
      #
      # This entry point is a modification of --chroot.
      #
      # Invocation examples:
      #
      #   ry bison --uchroot
      #   ry flex --uchroot
      #   ry nasm --uchroot
      #   ry sed --uchroot
      #
      # ===================================================================== #
      when /^-?-?uchroot$/i
        set_user_prefix('/Depot/Chroot/usr/')
      # ===================================================================== #
      # === Compile into your home directory
      #
      # This option allows you to compile into your user's home directory.
      # This does NOT use an AppDir layout.
      #
      # Invocation examples:
      #
      #   ry htop --into-home-dir
      #   rbt xwd --homedir
      #   rbt pango --compile-into-home-dir
      #
      # ===================================================================== #
      when 'HOME_DIR',
           /^-?-?tohomedir$/,
           /^-?-?home$/,
           /^-?-?into(_|-)?home(_|-)?dir$/,
           /^-?-?compile(_|-)?into(_|-)?the(_|-)?home(_|-)?dir$/,
           /^-?-?use(-|_)?home$/,
           /^-?-?home(-|_)?dir$/,
           /^-?-?compile(-|_)?into(-|_)?home(-|_)?dir$/,
           /^-?-?home(-|_)?directory$/,
           /^-?-?homedir$/
        compile_into_the_home_directory
      # ===================================================================== #
      # === Static compilation
      #
      # If you wish to compile something statically, use this entry point
      # here.
      #
      # Invocation examples:
      #
      #   ry m4 static
      #   ry make static
      #   ry make static ntrad
      #
      # ===================================================================== #
      when *ARRAY_HELP_ENABLE_STATIC
        verbose_enable_static
      # ===================================================================== #
      # === Enable shared and no static compilation
      #
      # This entry point combines --enable-shared and --disable-static
      # into one option.
      #
      # Invocation example:
      #   rbt libxmu --shared-no-static ntrad
      # ===================================================================== #
      when /^-?-?shared(-|_| )?no(-|_| )?static$/i
        enable_shared_disable_static
      # ===================================================================== #
      # === Permanently remove a global flag
      #
      # Invocation example:
      #
      #   rbt --remove-global-flag=--disable-static
      #
      # ===================================================================== #
      when /-?-?remove(-|_)?global(-|_)?flag=(-?-?.+)$/
        permanently_remove_this_global_flag($3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Output All URLS
      #
      # This entry point will output all URLs.
      #
      # Invocation examples:
      #
      #   ry --all_urls
      #   ry all_urls
      #   ry all_urls?
      #  
      # ===================================================================== #
      when *ARRAY_HELP_OUTPUT_ALL_URLS_OPTIONS
        output_all_urls_then_exit
      # ===================================================================== #
      # === Permanently enable ccache
      #
      # Use this entry point if you wish to make use of ccache permanently.
      #
      # Invocation example:
      #
      #   ry --permanently-enable-ccache
      #
      # ===================================================================== #
      when /^-?-?permanently(_|-)?enable(_|-)?ccache$/,
           /^-?-?do(_|-)?permanently(_|-)?enable(_|-)?ccache$/
        do_permanently_enable_ccache
        exit_program
      # ===================================================================== #
      # === Permanently disable ccache
      #
      # Use this entry point if you do not want to use ccache.
      #
      # Invocation example:
      #
      #   ry --permanently-disable-ccache
      #
      # ===================================================================== #
      when /^-?-?permanently(_|-)?disable(_|-)?ccache$/,
           /^-?-?do(_|-)?permanently(_|-)?disable(_|-)?ccache$/
        do_permanently_disable_ccache
        exit_program
      # ===================================================================== #
      # === Disable ccache
      #
      # This entry point will disable ccache, for the current compilation
      # run.
      #
      # Invocation examples:
      #
      #   rbt ruby --ntrad --disable-ccache
      #   rbt --disable-ccache
      #   rbt --do-not-use-ccache
      #   rbt --no-ccache
      #   rbt --ccache-
      #
      # ===================================================================== #
      when /^-?-?disable(_|-)?ccache$/i,
           /^-?-?do(_|-)?not(_|-)?use(_|-)?ccache$/i,
           /^-?-?no(_|-)?ccache$/i,
           /^-?-?ccache-$/i,
           /^-?-?dont(_|-)?use(_|-)?ccache$/i
        verbose_disable_ccache
      # ===================================================================== #
      # === Disable porg
      #
      # Through this entry point you can disable porg. You can also do so
      # via the configuration files, but an ad-hoc commandline way like
      # this one here also exists. 
      # ===================================================================== #
      when /^-?-?disable(-|_)?porg$/i,
           /^-?-?no(-|_)?porg$/i
        disable_porg
      # ===================================================================== #
      # === Permanently disable porg
      #
      # This is as disable_porg(), but we will permanently disable porg.
      #
      # Invocation example:
      #   rbt --permanently-disable-porg
      # ===================================================================== #
      when /-?-?permanently(-|_)?disable(-|_)?porg$/i
        permanently_disable_porg
        exit_program
      # ===================================================================== #
      # === Whether we use porg or whether we dont
      #
      # Invocation example:
      #   rbt --use-porg?
      # ===================================================================== #
      when /^-?-?use(-|_)?porg\??$/
        report_whether_we_will_use_porg
        exit_program
      # ===================================================================== #
      # === Querying whether porg is available on the given computer system
      #
      # Invocation example:
      #
      #   rbt --is_porg_available?
      #
      # ===================================================================== #
      when /^-?-?is(_|-)?porg(_|-)?available\??$/i
        report_whether_porg_is_available
      # ===================================================================== #
      # === Quickly compile a GCC version based on other configure options
      #
      # This entry point allows us to quickly compile another gcc version.
      #
      # Invocation example:
      #
      #   ry --compile-gcc=8.1.0
      #
      # ===================================================================== #
      when /^-?-?compile(_|-)?gcc=(.+)$/i # === $2
        version_to_use = $2.to_s.dup
        gcc = RBT::Cookbooks::Cookbook.new(:gcc) { :bypass_menu_check }
        gcc.use_this_version = version_to_use
        gcc.rebuild_path
        if File.exist? gcc.path?
          opnn; e 'Now compiling version '+sfancy(version_to_use)+
                  ' of the program called '+royalblue('gcc')+'.'
          compile_object = RBT::Compile.new(:gcc) { :do_not_run_yet }
          compile_object.use_this_as_cookbook = gcc
          compile_object.ntrad
          compile_object.do_not_initialize_a_new_cookbook_dataset
          compile_object.run
        end
        exit_program
      # ===================================================================== #
      # === Show the infobox only
      #
      # Invocation example:
      #
      #   rbt php --show-infobox-only
      #
      # ===================================================================== #
      when /^-?-?show(_|-)?infobox(_|-)?only/
        dataset_from_the_programs_that_are_to_be_compiled_next = compile_these_programs?.map {|program_name|
          create_a_new_cookbook_instance(program_name)
        }
        show_infobox(dataset_from_the_programs_that_are_to_be_compiled_next)
        exit
      # ===================================================================== #
      # === Set extract to
      #
      # Specify our target directory for extracting an archive. Special
      # shortcuts, such as pwd for "current working directry", also
      # exist.
      #
      # Usage examples:
      #
      #   ry htop --extract_to=pwd
      #   ry htop --base-dir=/tmp/
      #   ry htop --base-dir=/opt/
      #   ry htop ntrad --use-this-temp-dir=/opt/foobar_create_it
      #   ry htop --extract-to=/opt/test
      #
      # ===================================================================== #
      when /^-?-?extract(_|-)?to=(.*)$/,                        # === $2
           /^-?-?temp(_|-)?dir=(.*)$/,                          # === $2
           /^-?-?use(_|-)?temp(_|-)?dir=(.*)$/,                 # === $3
           /^-?-?base(_|-)?dir=(.+)$/,                          # === $2
           /^-?-?use(_|-)?this(_|-)?temp(_|-)?directory=(.*)$/, # === $4
           /^-?e=(.*)$/                                         # === $1
        _ = $1.to_s.dup
        _ = $2.to_s.dup if $2
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        set_extract_to_this_directory_as_specified_by_the_user(
          _
        )
      # ===================================================================== #
      # === Query where we will extract to
      #
      # This entry point can be used to determine where to we will extract
      # archives.
      #
      # Invocation example:
      #
      #   rbt --extract-to?
      #
      # ===================================================================== #
      when /^-?-?extract(-|_)?to\??$/i
        report_where_archives_will_be_extracted_into
        exit_program
      # ===================================================================== #
      # === Permanently extract into the home directory
      #
      # This entry point allows us to extract into the home directory. That
      # way we should be able to easily compile programs in a restricted
      # environment. 
      #
      # Invocation examples:
      #
      #   ry --permanently-extract-to-home
      #   ry --permanently-extract-to-home-dir
      #   ry --extract-to-home
      #
      # ===================================================================== #
      when /^-?-?permanently(-|_)?extract(-|_)?to(-|_)?home$/i,
           /^-?-?permanently(-|_)?extract(-|_)?to(-|_)?home(-|_)?dir$/i,
           /^-?-?extract(-|_)?to(-|_)?home$/i
        set_permanently_extract_to(:home_directory)
        exit_program
      # ===================================================================== #
      # === Set permanently extract to
      #
      # This entry point allows us to permanently redefine the extract_to
      # target.
      #
      # In other words, we will permanently determine where to we will
      # extract a given archive to.
      #
      # Usage example:
      #   rbt --permanently_extract_to=pwd
      #   rbt --permanently_extract_to=/Depot
      #   rbt --permanently_extract_to=/tmp
      #   rbt --permanently-set-temp-dir=/user/bioinf4/Temp/
      #   rbt --permanently-set-temp-dir-to=/user/bioinf4/Temp/
      # ===================================================================== #
      when /^-?-?permanently(_|-)?extract(_|-)?to=(.+)$/,               # This is for $3
           /^-?-?permanently(_|-)?set(_|-)?temp(_|-)?dir=(.+)$/,        # This is for $4
           /^-?-?permanently(_|-)?set(_|-)?temp(_|-)?dir(_|-)?to=(.+)$/ # This is for $5
        _ = $3.to_s.dup
        _ = $4.to_s.dup if $4
        _ = $5.to_s.dup if $5
        # =================================================================== #
        # We also silently ensure that this directory exists, if it does
        # not yet exist that is.
        # =================================================================== #
        mkdir(_) { :be_quiet } unless File.directory?(_)
        mkdir(_+'rbt/') { :be_quiet } unless File.directory?(_+'rbt/')
        set_permanently_extract_to(_)
        exit_program
      # ===================================================================== #
      # === Compile all LXQT-desktop components
      #
      # This entry point allows us to compile all LXQT-components.
      #
      # Invocation example:
      #
      #   ry --compile-all-lxqt-components
      #   ry --batch-compile-lxqt
      #
      # ===================================================================== #
      when /^-?-?compile(_|-)?all(_|-)?lxqt(_|-)?components$/i,
           /^-?-?compile(_|-)?lxqt$/i,
           /^-?-?batch(_|-)?compile(_|-)?lxqt$/i
        compile_all_lxqt_components
        exit_program
      # ===================================================================== #
      # === Mouse and keyboard and similar input devices for Xorg
      #
      # This entry point is quite similar to --post-xorg-server but
      # simpler. We'll mostly just compile some input-devices, through
      # a hardcoded Array as entry.
      #
      # Invocation example:
      #
      #   ry --compile-mouse-and-keyboard
      #
      # ===================================================================== #
      when /^-?-?mouse(-|_| )?and(-|_| )?keyboard$/i,
           /^-?-?mouse(-|_| )?keyboard$/i,
           /^-?-?meyboard$/i,
           /^-?-?compile(-|_| )?mouse(-|_| )?and(-|_| )?keyboard$/i,
           /^-?-?compile(-|_| )?mouse(-|_| )?and(-|_| )?keyboard(-|_| )?xorg(-|_| )?server(-|_| )?components$/i,
           /^-?-?input(-|_| )?drivers$/i,
           /^-?-?input(-|_| )?devices$/i,
           /^-?-?xorg(-|_| )?input(-|_| )?devices$/i,
           /^-?-?xorg(-|_| )?input$/i
        compile_mouse_and_keyboard_xorg_server_components
      # ===================================================================== #
      # === Deliberately use meson
      #
      # This entry point allows you to use meson.
      #
      # Invocation examples:
      #
      #   rbt htop --use-meson
      #   rbt glib --use-meson
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?meson$/
        do_use_meson_and_ninja
      # ===================================================================== #
      # === Specify to use configure as the build-system
      #
      # This entry point allows us to overrule the build-system that is to
      # be used - and specifically use configure.
      #
      # Invocation example:
      #
      #   ry htop --use-configure
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?configure$/i,
           /^-?-?use(_|-)?configure(_|-)?system$/i
        opne "Using #{royalblue('configure')} as build system."
        use_configure_as_build_system
      # ===================================================================== #
      # === Enable the use of the clang compiler
      #
      # This option will enable the (LLVM) clang compiler, in a verbose
      # manner.
      #
      # Invocation example:
      #
      #   ry htop --use-clang
      #
      # ===================================================================== #
      when /-?-?use(-|_)?clang/,
           /-?-?do(-|_)?use(-|_)?clang/,
           /-?-?clang/
        do_use_the_clang_compiler :and_notify_the_user
      # ===================================================================== #
      # === Post xorg-server components
      #
      # This entry point can be used to install components of the
      # xorg-server after the xorg-server has been compiled/installed.
      #
      # Invocation example:
      #
      #   ry --post-xorg-server
      #
      # ===================================================================== #
      when /^-?-?post(-|_| )?xorg(-|_| )?server$/i,
           /^-?-?post(-|_| )?xorg$/i,
           /^-?-?compile(-|_| )?post(-|_| )?xorg(-|_| )?server$/i
        compile_post_xorg_server_components
      # ===================================================================== #
      # === Compile a specific XORG component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular xorg component. So for example, "xorg1" will compile
      # the first xorg component, "xorg2" will refer to the second
      # xorg component and so forth.
      #
      # Invocation examples:
      #
      #   ry xorg1
      #   ry --xorg2
      #   ry xorg1..20
      #
      # ===================================================================== #
      when /^-?-?xorg(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_xorg_component($1.to_s.dup)
      # ===================================================================== #
      # == Search for a specific tag
      #
      # Invocation example:
      #
      #   rbt --tags=xorg
      #
      # ===================================================================== #
      when /^-?-?tags=(.+)$/i
        search_for_tag($1.to_s.dup)
      # ===================================================================== #
      # === Compile a specific GNOME component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular gnome component. So for example, "gnome1" will compile
      # the first gnome component, "gnome2" will refer to the second
      # gnome component and so forth.
      #
      # Invocation examples:
      #   ry gnome1
      #   ry --gnome2
      #   ry gnome1..20
      # ===================================================================== #
      when /^-?-?gnome(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_gnome_component($1.to_s.dup)
      # ===================================================================== #
      # === Install all registered rubygems
      #
      # If you wish to install all registered rubygems, then this
      # entry point may be useful to you.
      #
      # In order for this to work, the RBT project has to tap into the
      # cookbooks project, so make sure that the latter is also installed.
      #
      # Invocation examples:
      #
      #   rbt --install-all-rubygems
      #   rbt --install-all-gems
      #
      # ===================================================================== #
      when /^-?-?install(-|_)?all(-|_)?rubygems$/i,
           /^-?-?install(-|_)?all(-|_)?gems$/i,             
           /^-?-?batch(-|_)?install(-|_)?all(-|_)?gems$/i,
           /^-?-?install(-|_)?all(-|_)?registered(-|_)?gems$/i,
           /^-?-?install(-|_)?rubygems$/i
        update_these_rubygems
      # ===================================================================== #
      # === Query where the source directory is
      #
      # This entry point allows us to query where the source directory
      # on the local computer system resides at.
      #
      # Invocation examples:
      #
      #   rbt --source-dir?
      #   rbt --source-archive?
      #
      # ===================================================================== #
      when /^-?-?source(-|_| )?dir\??$/,
           /^-?-?source(-|_| )?archive\??$/,
           /^-?-?src(-|_| )?dir\??$/
        e sdir(source_directory?)
      # ===================================================================== #
      # === Use a specific file for compilation
      #
      # The following entry point allows us to make use of a locally
      # existing file and compile that.
      #
      # Invocation example:
      #
      #   ry --this_file=/Users/x/SRC/php/php-7.3.4.tar.xz
      #
      # ===================================================================== #
      when /^-?-?this(_|-)?file=(.+)$/i
        _ = $2.to_s.dup
        set_compile_these_programs(_)
      # ===================================================================== #
      # === Batch-install the registered python addons
      #
      # This entry point allows us to batch-install the registered python
      # addons. It exists mostly due to convenience.
      #
      # Invocation examples:
      #
      #   rbt --enable-python-addons
      #   rbt --python-addons
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?python(-|_| )?addons$/i,
           /^-?-?python(-|_| )?addons$/i,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?python$/i,
           /^-?-?compile(-|_)?python(-|_)?addons$/
        array = ::RBT.chained_programs?['python_addons'].reject {|entry| entry == 'python' }
        opnn; e 'Installing various registered python-addons ('+array.size.to_s+') next.'
        set_compile_these_programs(array)
      # ===================================================================== #
      # === Compile all registered perl-addons
      #
      # This entry point allows us to batch-install the registered perl
      # components.
      #
      # Invocation example:
      #
      #   rbt --compile-perl-addons
      #
      # ===================================================================== #
      when /^-?-?compile(-|_)?perl(-|_)?addons$/,
           /^-?-?perl(-|_)?addons$/,
           /^-?-?enable(-|_| )?perl(-|_| )?addons$/i,
           /^-?-?compile(-|_| )?all(-|_| )?of(-|_| )?perl$/i
        array = ::RBT.chained_programs?['perl_addons'].reject {|entry| entry == 'perl' }
        opnn; e 'Installing various registered perl-addons ('+array.size.to_s+') next.'
        set_compile_these_programs(array)
      # ===================================================================== #
      # === Check for updates of all registered gems
      #
      # This entry point allows us to batch-update all registered gems in
      # the RBT project.
      #
      # A working internet connection is required for this functionality,
      # obviously.
      #
      # Invocation examples:
      #
      #   rbt --update-all-gems
      #   rbt --update-ruby-gems
      #
      # ===================================================================== #
      when /^-?-?update(-|_)?all(-|_)?gems$/i,
           /^-?-?update(-|_)?gems$/,
           /^-?-?update(-|_)?ruby(-|_)?gems$/,
           /^-?-?update(-|_)?all(-|_)?ruby(-|_)?gems$/i,
           /^-?-?upgrade(-|_)?gems$/i
        update_all_gems
      # ===================================================================== #
      # === Show all available (registered) gems
      #
      # This entry point allows us to display, onto the commandline, which
      # gems are registered in the RBT project.
      #
      # Invocation example:
      #
      #   rbt --registered-gems
      #
      # ===================================================================== #
      when /^-?-?registered(-|_)?gems$/i,
           /^-?-?show(-|_)?registered(-|_)?gems$/i,
           /^-?-?rubygems$/i
        show_registered_gems
        exit_program
      # ===================================================================== #
      # == Non-Traditional compilation (ntrad tag)
      #
      # This entry point allows the user to compile the given program in
      # an AppDir variant. This would be equivalent to a --prefix value
      # of, for example, /Programs/Ruby/2.6.2/.
      #
      # Invocation example:
      #
      #   ry awk gstyle
      #   ry ruby --app_dir
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_COMPILE_IN_A_NON_TRADITIONAL_WAY_OPTIONS
        do_compile_in_a_non_traditional_way
      # ===================================================================== #
      # === Show last update
      #
      # Show the last updated programs in a table format, on the commandline.
      #
      # RBT will, by default, show all available programs.
      #
      # Invocation example:
      #
      #   ry htop last_update?
      #   ry --last_update?
      #
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_LAST_UPDATE_OPTIONS
        show_last_updated
      # ===================================================================== #
      # === Create a specific shellscript
      #
      # Create a specific (single) shellscript via this entry point here.
      #
      # Invocation examples:
      #
      #   ry wxwidget   shellscript
      #   ry gnunetgtk  create_shellscript
      #   ry php        cshell
      #   ry htop       generate_shellscript
      #   ry sed       --generate_shellscript
      #
      # ===================================================================== #
      when *ARRAY_HELP_CREATE_SHELLSCRIPT_OPTIONS
        unless yaml_dataset?
          this_program = compile_these_programs?.first
          assign_to_the_main_dataset(
            initialize_dataset_for(this_program)
          )
        end
        create_shellscript(
          dataset?,
          :save_into_file,
          :display_result
        )
        exit_program
      # ===================================================================== #
      # === Show every available programs
      #
      # Feedback all available programs:
      #
      #   ry all_programs
      #   ry --all_available_programs?
      #
      # ===================================================================== #
      when *ARRAY_HELP_ALL_AVAILABLE_PROGRAMS
        all_available_programs?
        exit_program
      # ===================================================================== #
      # === Show whether RBT uses ccache or whether it does not
      #
      # This entry point just queries whether RBT will be making use of
      # ccache or whether it will not. The trailing '?' is mandatory.
      #
      # Invocation example:
      #
      #   ry --use-ccache?
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?ccache\?$/
        show_whether_we_are_using_ccache_or_whether_we_are_not
      # ===================================================================== #
      # === Use a specific value for "make -jN" where N is a number
      #
      # To use this, try something like:
      #
      #   rbt lame -j5
      #
      # ===================================================================== #
      when /^-j\d{1,2}$/
        set_parameters_to_make(i)
      # ===================================================================== #
      # === Feedback the make options
      #
      # Use this entry point to feedback all possible make options.
      #
      # Invocation example:
      #
      #   ry gcc make_opts
      #
      # ===================================================================== #
      when *ARRAY_FEEDBACK_MAKE_OPTIONS
        feedback_make_options
        exit_program
      # ===================================================================== #
      # === Stop before the 'make' stage/step
      #
      # This entry point allows us to stop at, or rather right before, the
      # 'make' stage.
      #
      # Invocation example:
      #
      #   rbt htop --stop-at-make-stage
      #
      # ===================================================================== #
      when /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make$/i,
           /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?step$/i
        stop_at_make_stage
      # ===================================================================== #
      # === Use slackware makepkg
      #
      # The RBT.run_makepkg_command can be used to install a slackware
      # package.
      #
      # Invocation example:
      #
      #   ry --makepkg
      #
      # ===================================================================== #
      when /^-?-?makepkg$/
        RBT.run_makepkg_command(:from_the_current_working_directory) # bl $RBT/toplevel_methods/run_makepkg_command.rb
        exit_program
      # ===================================================================== #
      # === Permanently enable GoboLinux layout and behaviour from the
      #     commandline
      #
      # Use this method if you wish to let RBT assume that we are
      # running on gobolinux, without changing any ENV variable.
      #
      # Invocation example:
      #
      #   ry --permanently-enable-is-on-gobolinux
      #
      # ===================================================================== #
      when /^-?-?permanently(_|-)?enable(_|-)?is(_|-)?on(_|-)?gobolinux$/i
        permanently_enable_is_on_gobolinux(:be_verbose)
        exit_program
      # ===================================================================== #
      # === Query whether we are on a GoboLinux system
      #
      # This entry point can be used to determine whether the RBT scripts
      # are running on GoboLinux or whether they are not.
      #
      # Invocation examples:
      #
      #   ry --is-on-gobolinux?
      #   ry --are-we-on-gobolinux?
      #
      # ===================================================================== #
      when /^-?-?is(_|-)?on(_|-)?gobolinux\??$/i,
           /^-?-?are(_|-)?we(_|-)?on(_|-)?gobolinux\??$/i
        report_whether_we_are_on_gobolinux
        exit_program
      # ===================================================================== #
      # === Enable the GoboLinux layout and behaviour from the commandline
      #
      # Note that this entry point enables the GoboLinux layout only
      # for the CURRENT invocation. If you want to enable it permanently,
      # then either change an environment variable, or use the entry
      # point --permanently-enable-is-on-gobolinux.
      #
      # Invocation example:
      #   rbt --enable-is-on-gobolinux
      #   rbt htop ntrad --enable-is-on-gobolinux
      # ===================================================================== #
      when /^-?-?is(_|-)?on(_|-)?gobolinux$/,
           /^-?-?enable(_|-)?is(_|-)?on(_|-)?gobolinux$/,
           /^-?-?do(_|-)?we(_|-)?create(_|-)?the(_|-)?architecture(_|-)?file$/
        enable_is_on_gobolinux
      # ===================================================================== #
      # === Use a GoboLinux Recipe
      #
      # Invocation example:
      #
      #   ry --load_data_from_this_gobolinux_recipe=Dir.pwd
      #
      # ===================================================================== #
      when /^-?-?load(_|-)?data(_|-)?from(_|-)?this(_|-)?gobolinux(_|-)?recipe=(.+)/
        load_data_from_this_gobolinux_recipe($6)
      # ===================================================================== #
      # === GoboLinux Dependency File
      #
      # This will create a GoboLinux dependency file.
      #
      # Invocation example:
      #   ry sox  --create_dependencies_file
      #   ry htop create_dependencies_file
      #   ry php --create_dependencies_file
      # ===================================================================== #
      when /-?-?create(_|-)?dependencies(_|-)?file$/,
           /-?-?dependency(_|-)?file$/
        create_dependencies_file
        exit_program
      # ===================================================================== #
      # === Generate a gobolinux recipe
      #
      # Use this entry point to generate a GoboLinux recipe.
      #
      # You can find all the GoboLinux recipes here:
      #
      #   http://recipes.gobolinux.org/r/
      #
      # Invocation examples:
      #
      #   ry zenity  --gobolinux-recipe
      #   ry LibDRM  --gobolinux-recipe
      #   ry zziplib --gobolinux-recipe
      #   ry ruby    --gobolinux-recipe
      #   ry php     --gobolinux-recipe
      #
      # ===================================================================== #
      when /^-?-?gobolinux(_|-)?recipe$/,
           /^-?-?create(_|-)?gobo(_|-)?recipe$/, # See: http://rubular.com/r/8vWvfd2izB
           /^-?-?grecipe$/i,
           /^-?-?gobolinux(_|-)?package$/i,
           /^-?-?generate(_|-)?gobolinux(_|-)?description(_|-)?file$/i
        generate_gobolinux_recipe
        exit_program
      # ===================================================================== #
      # === Convert a Gobolinux-like Package into the traditional structure:
      #
      # This entry point allows us to take a program structure such as
      # /Programs/Ruby/2.5.1/ and "convert" it into the "/usr/"
      # hierarchy.
      #
      # Usage examples:
      #
      #   ry php convert
      #   ry htop convert
      #   ry ruby --convert-to-traditional
      #
      # ===================================================================== #
      when /^-?-?convert(_|-)?to(_|-)?traditional$/i,
           'convert','conv','convert_package',
           'cpackage','to_usr','dumpp','tousr','converttotrad','totrad',
           'cpa','ctt','to_traditional','ctotrad'
        convert_to_traditional
        exit_program
      # ===================================================================== #
      # === Generate all GoboLinux recipes
      #
      # This entry point will generate all GoboLinux recipes.
      #
      # Invocation example:
      #
      #   ry --generate-all-gobolinux-recipes
      #
      # ===================================================================== #
      when /^-?-?generate(_|-)?all(_|-)?gobolinux(_|-)?recipes$/,
           /^-?-?generate(_|-)?gobolinux(_|-)?recipes$/,
           /^-?-?create(_|-)?all(_|-)?gobolinux(_|-)?recipes$/,
           /^-?-?create(_|-)?gobolinux(_|-)?recipes$/
        generate_all_gobolinux_recipes
        exit_program
      # ===================================================================== #
      # === Use GoboLinux SymlinkProgram
      #
      # The RBT scripts provide a class called RBT::SymlinkProgram, which
      # will symlink a program that was installed under the /Programs
      # hierarchy. However had, this class is not 100% the same as the
      # GoboLinux SymlinkProgram script. If you want to invoke the
      # GoboLinux SymlinkProgram rather than the RBT::SymlinkProgram,
      # then this entry point here is allowing you to do precisely that -
      # to invoke the SymlinkProgram script from GoboLinux rather than
      # the integrated class called RBT::SymlinkProgram.
      # ===================================================================== #
      when /^-?-?use(_|-)?gobolinux(_|-)?symlink(_|-)?program$/i,
           /^-?-?do(_|-)?use(_|-)?gobolinux(_|-)?symlink(_|-)?program$/i
        do_use_gobolinux_symlinkprogram
      # ===================================================================== #
      # === Show the GoboLinux instructions on the commandline
      #
      # This entry point allows us to show the GoboLinux instructions,
      # as far as RBT is concerned.
      #
      # Invocation example:
      #   rbt --show-gobo-instructions
      # ===================================================================== #
      when /^-?-?show(_|-)?gobo(_|-)?instructions$/
        show_gobolinux_instructions
      # ===================================================================== #
      # === Show some instructions for GoboLinux
      #
      # Invocation examples:
      #
      #   rbt --gobolinux-instructions
      #   rbt --show-instructions-for=gobolinux
      #
      # ===================================================================== #
      when /^-?-?gobolinux(-|_| )?instructions$/i,
           /^-?-?show(-|_| )?instructions(-|_| )?for=(.+)$/i
        show_instructions_for_gobolinux
        exit_program
      # ===================================================================== #
      # === Query who the main maintainers are for RBT right now
      #
      # Invocation example:
      #
      #   rbt --maintainer?
      #
      # ===================================================================== #
      when /^-?-?maintainers?\??$/i
        e 'You may be able to contact the current lead maintainer(s) via:'
        e
        e "  #{sfancy(RBT.configuration?.email)}"
        e
      # ===================================================================== #
      # === Create an AppDir program
      #
      # This entry point allows you to create an AppDir program in
      # the respective $PROGRAMS hierarchy - which is usually at
      # /Programs/.
      #
      # Invocation example:
      #
      #   rbt --create-program=foobar-1.0
      #
      # ===================================================================== #
      when /^-?-?create(-|_)?program=(.+)$/i
        RBT.create_program($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Make use of CreateProgram with the current directory as input
      #
      # This entry point allows us to make use of the current working
      # directory as the input-argument.
      #
      # Invocation examples:
      #
      #   rbt --create-program
      #   rbt --rcp
      #
      # ===================================================================== #
      when /^-?-?create(-|_)?program$/i,
           /^-?-?rcp$/i
        _ = File.basename(return_pwd)
        RBT.create_program(_)
        exit_program
      # ===================================================================== #
      # === Permanently set a specific source directory
      #
      # This entry point allows us to permanently specify any source
      # directory on the local filesystem.
      #
      # Invocation examples:
      #
      #   rbt --permanently-set-source-archive=/Mount/USB1/last_backup/SRC/
      #   rbt --permanently-set-this-source-directory=/opt/rbt/source_archives
      #   rbt --set-source-dir=/opt/rbt/source_archives
      #
      # ===================================================================== #
      when /^-?-?permanently(_|-)?set(_|-)?base(_|-)?directory(_|-)?to=(.+)$/i,     # for $5
           /^-?-?permanently(_|-)?set(_|-)?source(_|-)?archive=(.+)$/i,             # for $4
           /^-?-?permanently(_|-)?set(_|-)?source(_|-)?dir(_|-)?to=(.+)$/i,         # for $5
           /^-?-?permanently(_|-)?set(_|-)?source(_|-)?dir=(.+)$/i,                 # for $4
           /^-?-?permanently(_|-)?set(_|-)?this(_|-)?source(_|-)?directory=(.+)$/i, # for $5
           /^-?-?set(_|-)?source(_|-)?dir=(.+)$/i,                                  # for $3
           /^-?-?set(_|-)?src(_|-)?dir=(.+)$/i,                                     # for $3
           /^-?-?source(_|-)?dir=(.+)$/i                                            # for $2
        _ = $2.to_s.dup
        _ = $3.to_s.dup if $3
        _ = $4.to_s.dup if $4
        _ = $5.to_s.dup if $5 # This is for the variant on the first line, see: http://rubular.com/r/6E6v45rB7u
        opnn; RBT.permanently_set_source_directory(_) { :be_verbose }
      # ===================================================================== #
      # === Symlink a single program.
      #
      # Symlink a specific (single) program from within this file.
      #
      # This requires an AppDir like setup, such as within the /Programs/
      # hierarchy.
      #
      # Usage example:
      #   ry lua --symlink
      #   ry xload --symlink
      #   ry htop --symlink
      # ===================================================================== #
      when /^-?-?symlink$/,
           /^-?-?advanced(_|-)?symlink$/,
           'asymlink'
        do_invoke_symlink_program(
          compile_which_program?,
          program_version? # bl $RBT/utility_scripts/symlink_program/symlink_program.rb
        )
        exit_program
      # ===================================================================== #
      # === Remove all broken symlinks via class RBT::RemoveSymlinks
      #
      # Use this entry point if you simply wish to invoke class
      # RBT::RemoveSymlinks.
      #
      # Invocation example:
      #
      #   ry --remove_symlinks
      #
      # ===================================================================== #
      when /^-?-?remove(-|_)?symlinks$/,
           /^-?-?remove(-|_)?broken$/i
        RBT::RemoveSymlinks.new
        exit_program
      # ===================================================================== #
      # === Remove all Symlinks in the /Programs/ hierarchy
      #
      # Use this entry point if you wish to eliminate all symlinks for
      # the /Programs/ hierarchy.
      #
      # Invocation examples:
      #
      #   ry --remove_symlinks
      #   ry remove_symlinks
      #   ry CLEAR_SYMLINKS
      #
      # ===================================================================== #
      when *ARRAY_HELP_REMOVE_ALL_SYMLINKS_OPTIONS
        remove_all_symlinks
        exit_program
      # ======================================================================= #
      # === Permanently disable the colours
      #
      # If you wish to permanently disable the colours for the RBT project,
      # use this entry point here.
      #
      # Invocation example:
      #
      #   ry --permanently-disable-colours
      #
      # ======================================================================= #
      when /^-?-?permanentl?y?(-|_)?disable(-|_)?colou?rs$/,
           /^-?-?permanentl?y?(-|_)?no(-|_)?colours$/
        do_permanently_disable_the_colours
        exit_program
      # ======================================================================= #
      # === Permanently enable the colours
      #
      # If you wish to permanently enable the colours for the RBT project,
      # use this entry point here. This is specifically enabled to allow
      # us to reset after having done the entry point above, e. g. at
      # rbt --permanently-disable-colours.
      #
      # Invocation example:
      #
      #   ry --permanently-enable-colours
      #
      # ======================================================================= #
      when /^-?-?permanentl?y?(-|_)?enable(-|_)?colou?rs$/,
           /^-?-?permanentl?y?(-|_)?colours$/
        do_permanently_enable_the_colours
        exit_program
      # ===================================================================== #
      # === Permanently add a global flag
      #
      # Invocation example:
      #
      #   rbt --add-global-flag=--disable-static
      #
      # ===================================================================== #
      when /^-?-?add(-|_)?global(-|_)?flag=(-?-?.+)/
        permanently_add_this_global_flag($3.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Automatically determining a few things
      #
      # Invocation example:
      #
      #   rbt --auto
      #
      # ===================================================================== #
      when /^-?-?auto$/i
        try_to_automatically_setup_the_rbt_project
        exit_program
      # ===================================================================== #
      # === Compile both traditional and non-traditional
      #
      # Use this to compile both traditional and non-traditional.
      #
      # Usage example:
      #   ry htop trad_ntrad
      # ===================================================================== #
      when /^-?-?trad(_|-)?ntrad$/i,
           'trad_and_nontrad',
           /trad_?and_?ntrad/
        do_compile_in_both_traditional_and_non_traditional_manner
      # ===================================================================== #
      # === rbt php --ntrad
      #
      # Handle prefix-situations for "non-traditional", aka AppDir-specific
      # prefixes.
      # ===================================================================== #
      when /^-?-?ntrad$/i
        set_user_prefix(:use_ntrad_prefix_at_a_later_time)
      # ===================================================================== #
      # === Disabling colours
      #
      # Use this method if you wish to disable colours on a per-project
      # (per compile-run) basis. This means that the complete RBT suite
      # will not make use of any colours.
      #
      # Colours can be disabled like in this way:
      #
      #   ry nocolours
      #   ry --nocolours
      #   ry htop --nocolours
      #
      # ===================================================================== #
      when *OPTIONS_NO_COLOURS # This constant is defined in bl $RBT/compile/compile.rb
        disable_colours
      # ===================================================================== #
      # === Show downloadable programs
      #
      # This entry point will display all programs that are downloadable,
      # in a condensed manner.
      #
      # Invocation example:
      #
      #   rbt --show-downloadable-programs
      #
      # ===================================================================== #
      when /^-?-?show(-|_)?downloadable(-|_)?programs$/i,
           /^-?-?downloadable(-|_)?programs\??$/i
        show_downloadable_programs
      # ===================================================================== #
      # === Batch-download all source archives
      #
      # This entry point allows us to batch-download all source archives
      # onto the local filesystem.
      #
      # Invocation example:
      #
      #   rbt --batch-download-all-source-archives
      #   rbt --download-all-source-archives
      #
      # ===================================================================== #
      when /^-?-?batch(_|-)?download(_|-)?all(_|-)?source(_|-)?archives$/i,
           /^-?-?download(_|-)?all(_|-)?source(_|-)?archives$/i
        batch_download_all_source_archives
        exit_program
      # ===================================================================== #
      # === Download a remote program into the default directory
      #
      # This entry point allows us to quickly download a remote program
      # into a local directory.
      #
      # Invocation example:
      #
      #   rbt --download-into-proper-directory htop
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?into(-|_| )?proper(-|_| )?directory$/i
        download_this_remote_url_into_that_local_directory(url?, base_dir?)
      # ===================================================================== #
      # === Download a remote program into a given directory at hand
      #
      # This entry point allows us to download a remote program into
      # a local directory.
      #
      # Invocation example:
      #
      #   rbt htop --download-into-this-directory=/opt/bla2/
      #
      # ===================================================================== #
      when /^-?-?download(-|_| )?into(-|_| )?this(-|_| )?directory=(.+)$/
        _ = $4.to_s.dup
        url = url1?
        download_this_remote_url_into_that_local_directory(url, _)
      # ===================================================================== #
      # === Download via svn
      #
      # If you wish to download via svn, then you can use this entry point
      # here.
      #
      # Invocation examples:
      #
      #   ry dosbox --svn
      #   ry dosbox svn
      #
      # ===================================================================== #
      when /^-?-?svn$/,
           'use_svn',
           'svn_download',
           'svndownload',
           'svndown',
           'download_via_svn'
        download_using_svn
        exit_program
      # ===================================================================== #
      # === Compile all KDE5 plasma components
      #
      # This entry point is a shortcut to batch-compile all KDE5
      # plasma components.
      #
      # Invocation examples:
      #
      #   rbt --compile-plasma
      #   rbt --kde-plasma
      #   rbt --compile-plasma-desktop
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?plasma$/,
           /^-?-?compile(-|_| )?all(-|_| )?kde5-plasma(-|_| )?components$/,
           /^-?-?kde(-|_| )?plasma$/,
           /^-?-?compile(-|_| )?kde(-|_| )?plasma$/,
           /^-?-?compile(-|_| )?plasma(-|_| )?desktop$/
        compile_all_kde5_plasma_components
      # ===================================================================== #
      # === Compile the KDE Applications
      #
      # This entry point allows us to compile KDE applications.
      #
      # Usage example:
      #   rbt --compile-kde-apps
      # ===================================================================== #
      when /^-?-?compile(-|_| )?kde(-|_| )?apps$/,
           /^-?-?compile(-|_| )?kde(-|_| )?applications$/
        compile_kde_apps
      # ===================================================================== #
      # === Batch compile the given input
      #
      # If you need to batch-compile several programs, then this is
      # one way to do so. This used to be called "chain compiling"
      # but is now called batch-compiling.
      #
      # Invocation examples:
      #
      #   rbt --batch-compile=mate
      #   rbt --batch-compile=python,php,perl,ruby
      #   rbt --chained=xfce
      #   rbt --chained=mate
      #   rbt --batch-compile=kde5_porting_aids
      #   rbt --batch-compile=kde5_foundation
      #
      # ===================================================================== #
      when /^-?-?batch-?compile=(.+)$/i, # URL: http://rubular.com/r/uItIs454yH
           /^-?-?chained=(.+)$/i,
           /^-?-?batch=(.+)$/i
        match = $1.to_s.dup
        if match.include? ','
          these_programs = match.split(',')
        else
          these_programs = try_to_batch_compile_this_stack(match)
        end
        if these_programs
          opne "We will batch-compile these "\
               "#{simp(these_programs.size.to_s)} programs next."
          e; these_programs.each {|install_this_program|
            e sfancy("  #{install_this_program}")
          }; e
          set_compile_these_programs(these_programs)
        else
          opne "No result found for #{orange(match)}."
          exit_program
        end
      # ===================================================================== #
      # === Specify to use another build-system
      #
      # This entry point allows us to overrule the build-system that is
      # to be used.
      #
      # Invocation example:
      #
      #   ry htop --use-this-build-system=configure
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?this(_|-)?build(_|-)?system=(.+)$/i
        this = $4.to_s.dup
        opne "Using #{royalblue(this)} as build system."
        set_use_this_build_system(this)
      # ===================================================================== #
      # === Tell people where they can find the .sh files
      #
      # Invokation example:
      #
      #   rbt --shell-completion?
      #
      # ===================================================================== #
      when /^-?-?shell(-|_| )?completion\??$/i
        e 'The .sh file for bash completion can be found here:'
        e
        e sfile("  #{project_base_dir?}shell/bash_completion.sh")
        e
        exit_program
      # ===================================================================== #
      # === Generate shell completion
      #
      # Generate the general tab completions here. Right now this
      # will only work for bash-completions.
      #
      # Invocation example:
      #
      #   ry --generate-shell-completion
      #   ry --gsc
      #
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_SHELL_COMPLETION_OPTIONS
        generate_shell_completion(:all)
        exit_program
      # ===================================================================== #
      # === Compile all registered games
      #
      # This entry point allows us to compile all registered games.
      #
      # It is recommend to get all dependencies compiled first, e. g.
      # allegro, SDL, and so forth, as the code is very "dumb" - it
      # will not check much at all.
      #
      # Invocation example:
      #
      #   rbt --compile-all-games
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?games$/i
        compile_all_games
      # ===================================================================== #
      # === Compile a specific game
      #
      # This entry point allows us to use numbers in order to compile
      # a particular mate-desktop component. So for example, "game1"
      # will compile the first game, "game2" will refer to the second
      # game and so forth.
      #
      # Invocation examples:
      #
      #   ry game1
      #   ry --game2
      #   ry game1..20
      #
      # ===================================================================== #
      when /^-?-?game(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_game_component($1.to_s.dup)
      # ===================================================================== #
      # === Use instructions to GNU configure from a local file
      #
      # This entry point allows us to use configure-options stored in
      # a local file. This is useful because other parts of the RBT
      # suite can generate this file, which we can then easily use.
      #
      # For example, if the xorg-server does not work then copy/pasting
      # through the mouse may be harder. That is one possible use
      # case for this option.
      #
      # Invocation example:
      #
      #   ry gcc --use-instructions-from-this-file=/Depot/foobar.md
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?instructions(-|_| )?from(-|_| )?this(-|_| )?file=(.+)$/i # $5
        _ = $5.to_s.dup
        consider_using_this_file_for_the_configure_options_that_are_to_be_used(_)
      # ===================================================================== #
      # === Compile all of XFCE
      #
      # This entry point allows us to compile all XFCE components in
      # one go.
      #
      # Invocation example:
      #   ry --compile-xfce
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xfce$/i
        compile_all_of_xfce
        exit_program
      # ===================================================================== #
      # === Compile a specific XFCE component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular xfce component. So for example, "xfce1" will compile
      # the first xfce component, "xfce2" will refer to the second
      # xfce component and so forth.
      #
      # Invocation examples:
      #
      #   ry xfce1
      #   ry --xfce2
      #   ry xfce1..20
      #
      # ===================================================================== #
      when /^-?-?xfce(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_xfce_component($1.to_s.dup)
      # ===================================================================== #
      # === Compile all of XFCE into a standalone directory
      #
      # This entry point allows us to compile all of XFCE into a
      # standalone directory, such as /Programs/Xfce/4.12.0.
      #
      # I considered this sufficiently useful to add this functionality.
      # That way we could easily and quickly relocate XFCE between
      # different computers.
      #
      # Invocation examples:
      #   rbt --compile-xfce-components-into-one-standalone-directory
      #   rbt --compile-xfce-components-into-a-standalone-directory
      #   rbt --compile-xfce-components-into-standalone-directory
      # ===================================================================== #
      when /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?one(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?a-standalone(-|_| )?directory$/i,
           /^-?-?compile(-|_| )?xfce(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?components(-|_| )?standalone(-|_| )?directory$/i,
           /^-?-?xfce(-|_| )?into(-|_| )?standalone(-|_| )?dir$/i,
           /^-?-?xfce(-|_| )?standalone$/i
        compile_xfce_components_into_one_standalone_directory
        exit_program
      # ===================================================================== #
      # === Do not symlink
      #
      # Use this option if you do not wish to automatically symlink
      # the compiled/installed program. This will also disable the
      # instruction "symlink_header".
      #
      # Invocation example:
      #
      #   ry htop dontsymlink
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_NOT_SYMLINK_OPTIONS
        do_not_symlink_in_general_and_also_do_not_symlink_header_files
      # ===================================================================== #
      # === Compile in a traditional way
      #
      # We use traditional compilation here by changing the compile mode
      # (traditional or non-gobolinux way) here.
      #
      # Usage example:
      #
      #   ry make trad
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_COMPILE_IN_A_TRADITIONAL_WAY_OPTIONS
        do_compile_in_a_traditional_way
      # ===================================================================== #
      # === rbt php --trad
      # ===================================================================== #
      when /^-?-?trad$/i
        _ = Prefix.new(:trad)
        set_user_prefix(_)
      # ===================================================================== #
      # === Designate another source location that is to be used
      #
      # This entry point allows us to use another source location,
      # e. g. if we have an USB stick and data on this stick then
      # this entry point may be very helpful.
      #
      # Invocation example:
      #
      #   ry gdb ntrad --use-this-source-location=/Depot/j/gdb-8.2.1.tar.xz
      #
      # ===================================================================== #
      when /^-?-?use(-|_| )?this(-|_| )?source(-|_| )?location=(.+)$/i # $4
        _ = $4.to_s.dup
        if File.exist? _
          if cookbook_dataset?
            cookbook_dataset?.set_program_path(_)
            disallow_changing_the_cookbook_dataset
          end
          opnn; e "Making use of the target location at `#{slateblue(_)}`."
          set_program_path(_)
        else
          opnn; e swarn('No target exists at ')+sfile(_)+swarn('. Exiting now.')
          exit_program
        end
      # ===================================================================== #
      # === Use an environment variable
      #
      # This method will use a specific environment variable as the
      # new prefix to be used.
      #
      # Do note that class RBT::CompileViaEnvironmentVariableAsPrefix can
      # also be used as an alternative.
      #
      # In order to use this entry point, try:
      #
      #   ry htop --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4screensaver --use-this-environment-variable=XFCE_PREFIX
      #   ry libxfce4util --use-this-environment-variable=XFCE_PREFIX
      #   ry xfconf --use-this-environment-variable=XFCE_PREFIX
      #   ry exo --use-this-environment-variable=XFCE_PREFIX
      #   ry garcon --use-this-environment-variable=XFCE_PREFIX
      #   ry libxfce4ui --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4panel --use-this-environment-variable=XFCE_PREFIX
      #   ry thunar --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4appfinder --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4session --use-this-environment-variable=XFCE_PREFIX
      #   ry xfdesktop --use-this-environment-variable=XFCE_PREFIX
      #   ry xfceengine --use-this-environment-variable=XFCE_PREFIX
      #   ry xfwm4 --use-this-environment-variable=XFCE_PREFIX
      #   ry xfmedia --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4devtools --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4settings --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4cpufreqplugin --use-this-environment-variable=XFCE_PREFIX
      #   ry xfce4panelprofiles --use-this-environment-variable=XFCE_PREFIX
      #   ry matedesktop  --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmatemixer --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmateweather --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matemenus --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matepanel --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matenetbook --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matemedia --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry marco --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry libmatekbd --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matesensorsapplet --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matesettingsdaemon --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry caja --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry cajaextensions --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry engrampa --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry eom --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry mateutils --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry pluma --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry mateterminal --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #   ry matecontrolcenter --use-this-environment-variable=MATE_DESKTOP_PREFIX
      #
      # Or:
      #
      #   env_compile_via_prefix libxfce4util
      #   env_compile_via_prefix xfconf
      #   env_compile_via_prefix exo
      #   env_compile_via_prefix garcon
      #   env_compile_via_prefix libxfce4ui
      #   env_compile_via_prefix xfce4panel
      #   env_compile_via_prefix thunar
      #   env_compile_via_prefix xfce4appfinder
      #   env_compile_via_prefix xfce4session
      #   env_compile_via_prefix xfdesktop
      #   env_compile_via_prefix xfceengine
      #   env_compile_via_prefix xfwm4
      #   env_compile_via_prefix xfmedia
      #   env_compile_via_prefix xfce4devtools
      #   env_compile_via_prefix xfce4settings
      #   env_compile_via_prefix xfce4cpufreqplugin
      #   env_compile_via_prefix xfce4panelprofiles
      #   env_compile_via_prefix xfce4screensaver
      #   env_compile_via_prefix xfce4notifyd
      #   env_compile_via_prefix xfce4screenshooter
      #   env_compile_via_prefix xfce4systemloadplugin
      #   env_compile_via_prefix xfce4whiskermenuplugin
      #   env_compile_via_prefix xfce4taskmanager
      #
      # or
      #
      #   env_compile_via_prefix matedesktop MATE
      #   env_compile_via_prefix libmatemixer MATE
      #   env_compile_via_prefix libmateweather MATE
      #   env_compile_via_prefix matemenus MATE
      #   env_compile_via_prefix matepanel MATE
      #   env_compile_via_prefix matenetbook MATE
      #   env_compile_via_prefix matemedia MATE
      #   env_compile_via_prefix marco MATE
      #   env_compile_via_prefix libmatekbd MATE
      #   env_compile_via_prefix matesensorsapplet MATE
      #   env_compile_via_prefix matesettingsdaemon MATE
      #   env_compile_via_prefix matecontrolcenter MATE
      #   env_compile_via_prefix caja MATE
      #   env_compile_via_prefix cajaextensions MATE
      #   env_compile_via_prefix engrampa MATE
      #   env_compile_via_prefix eom MATE
      #   env_compile_via_prefix mateutils MATE
      #   env_compile_via_prefix pluma MATE
      #   env_compile_via_prefix mateterminal MATE
      #
      # ===================================================================== #
      when /^-?-?use(-|_)?this(-|_)?environment(-|_)?variable=/,
           /^-?-?use(-|_)?environment(-|_)?variable=/
        use_this_environment_variable(i)
      # ===================================================================== #
      # === Generate a slack-desc file
      #
      # This entry point allows us to use RBT to create a slack-desc file
      # for a slackware package.
      #
      # Invocation example:
      #
      #   rbt --create-slackdesc-file-for=htop
      #
      # ===================================================================== #
      when /^-?-?create(_|-)?slackdesc(_|-)?file(_|-)?for=(.+)$/i # $4
        this_program = $4.to_s.dup
        create_a_slackdesc_file_for_this_program(this_program)
        exit_program
      # ===================================================================== #
      # === Generate all slackware files
      #
      # This will create a bunch of .desc files into the target
      # directory at rbt_log_directory?+"slackware_desc_files/".
      #
      # Invocation examples:
      #   ry --all-slack-files
      #   ry all-slack-files
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_ALL_SLACK_DESC_FILES_OPTIONS
        generate_slack_desc_files_for_every_program
        exit_program
      # ===================================================================== #
      # === Create a Slackware Package
      #
      # You can create a slackware package from the current directory
      # through this entry point.
      #
      # Invoke this by issuing:
      #
      #   rbt --create-slackware-package
      #
      # ===================================================================== #
      when /^-?-?create(_|-)?slackware(_|-)?package$/i
        this_program = File.basename(Dir.pwd)
        if this_program.include? '-'
          this_program = ProgramInformation.return_name(this_program)
        end
        RBT::CreateSlackwarePackage.new(this_program) { :do_not_autoinstall }
        exit_program
      # ===================================================================== #
      # === Use old configure options
      #
      # This entry point allows you to re-use configure options used by
      # an existing program. This works only for a few programs though
      # such as gcc and perhaps nano.
      #
      # Invocation examples:
      #
      #   ry gcc --use-this-version=7.1.0 --use-old-configure-options
      #   ry gcc --copy-old-version
      #   ry gcc --reuse-old-version
      #   ry gcc --copy-old-version --ntrad
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?old(_|-)?configure(_|-)?options$/i,
           /^-?-?copy(_|-)?old(_|-)?version$/i,
           /^-?-?reuse(_|-)?old(_|-)?configure(_|-)?option$/i,
           /^-?-?reuse(_|-)?configure(_|-)?options$/i,
           /^-?-?use(_|-)?old(_|-)?config(_|-)?version$/i,
           /^-?-?use(_|-)?old(_|-)?configure(_|-)?version$/i,
           /^-?-?re(_|-)?use(_|-)?gcc(_|-)?configure$/i,
           /^-?-?re(_|-)?use(_|-)?old(_|-)?gcc(_|-)?version$/i,
           /^-?-?re(_|-)?use(_|-)?old(_|-)?configure(_|-)?options$/i,
           /^-?-?re(_|-)?use(_|-)?old$/i,
           /^-?-?reuse(_|-)?options$/i,
           /^-?-?old$/i,
           /^-?-?reuse(_|-)?old(_|-)?version$/i
        reuse_old_configure_options
      # ===================================================================== #
      # === Show which build-system will be prioritized
      #
      # This entry point allows us to query which build-systems are
      # our priorities, e. g. whether we will invoke meson before
      # configure or vice versa, in the event that the given
      # package has more than one build-systems in use.
      #
      # Invocation examples:
      #
      #   rbt --prioritize?
      #   rbt --priorities?
      #   rbt --build-priorities?
      #
      # ===================================================================== #
      when /^-?-?prioritize\??$/i,
           /^-?-?priorities\??$/i,
           /^-?-?build-priorities\??$/i
        show_build_systems_priorities
        exit_program
      # ===================================================================== #
      # === Use a specific prefix
      #
      # This entry point allows us to quickly use a specific prefix.
      #
      # Invocation example:
      #
      #   ry coreutils --use-this-prefix=/Depot/Chroot/usr/
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?this(_|-)?prefix=(.+)$/i # $3
        set_user_prefix($3.to_s.dup)
      # ===================================================================== #
      # === Use default chroot prefix being /Depot/Chroot
      #
      # This is a convenience method mostly, which allows me to save
      # typing a manual prefix for a chrooted environment.
      #
      # Invocation examples:
      #
      #   ry htop --chroot
      #   ry coreutils --chroot
      #   ry glibc 2.23 --do-not-run-ldconfig --do-not-symlink --chroot
      #
      # ===================================================================== #
      when /-?-?chroot$/,
           /-?-?chroot(-|_)?prefix/,
           'CHROOT'
        use_default_chroot_as_prefix
      # ===================================================================== #
      # === Compile a specific LXQT-desktop component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular lx-qt-desktop component. So for example, "lxqt1"
      # will be a shortcut for "libqtxdg" whereas "lxqt2" will
      # be a shortcut for "lxqtbuildtools", and so forth.
      #
      # Invocation examples:
      #   ry lxqt1
      #   ry lxqt2
      #   ry lxqt3
      #   ry --lxqt1
      #   ry --lxqt2
      #   ry --lxqt3
      #   ry lxqt11
      #   ry lxqt..100
      # ===================================================================== #
      when /^-?-?lxqt(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_lxqt_component($1.to_s.dup)
      # ===================================================================== #
      # === Do not use ldconfig
      #
      # We will not use ldconfig, but we will continue regardless.
      #
      # Usage example:
      #   ry htop --do-not-use-ldconfig
      #   ry htop noldconfig
      #   ry htop do_not_use_ldconfig
      # ===================================================================== #
      when /^-?-?no(_|-)?ldconfig/,
           'nold',
           /^-?-?do(_|-)?not(_|-)?use(_|-)?ldconfig$/i,
           /^-?-?dont(_|-)?run(_|-)?ldconfig$/,
           /^-?-?do(_|-)?not(_|-)?run(_|-)?ldconfig$/
        do_not_use_ldconfig
      # ===================================================================== #
      # === Use /opt/ as prefix
      #
      # This entry point allows us to use /opt/ as the prefix. It is mostly
      # a convenience entry point.
      #
      # Invocation examples:
      #
      #   ry llvm --use-opt-as-prefix
      #   ry zlib --opt
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?opt(_|-)?as(_|-)?prefix$/,
           /^-?-?opt(_|-)?prefix$/,
           /^-?-?opt$/
        use_opt_as_prefix
      # ===================================================================== #
      # === Set another prefix
      #
      # ry htop --prefix=$J
      # ===================================================================== #
      when /^--prefix/,
           'set_prefix'
        set_user_prefix(i)
      # ===================================================================== #
      # === Set another prefix
      #
      # This variant will make use of a regex capture group.
      #
      # Invocation example:
      #
      #   ry xfce4panel --prefix=:xfce
      #   ry xfce4panel :xfce
      #   ry htop --use-this-appdir=/Programs/Bla/1.2.3
      #
      # ===================================================================== #
      when /^-?-?prefix=(.+)/, # for $1
           /^(:.+)/,           # for $1
           /^-?-?use(-|_)?this(-|_)?appdir=(.+)$/i # for $3
        _ = $1.to_s.dup
        _ = $3.to_s.dup if $3
        set_user_prefix(_)
      # ===================================================================== #
      # === Report the RBT version in use
      #
      # This part will tell us which specific version of RBT we
      # are currently using.
      #
      # Invocation examples:
      #   ry --version
      #   ry VERSION
      # ===================================================================== #
      when *ARRAY_HELP_VERSION_OPTIONS,
           /^VERSION/
        report_rbt_version :then_exit
      # ===================================================================== #
      # === Feedback various versions
      #
      # This entry point will tell us which version is in use for the
      # RBT project, the Cookbooks::Cookbook and the RBT project.
      #
      # Invocation example:
      #   ry feedback_version
      #   ry --feedback-version
      #   ry --versions-in-use?
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_VERSIONS_OPTIONS
        feedback_various_versions :do_not_use_opn
        exit_program
      # ===================================================================== #
      # === Use a specific version based on numbers-input only
      #
      # This entry point allows us to respond to input such as "1.5.2".
      #
      # This is assumed to be a shortcut for --use-this-version=1.5.2.
      #
      # Usage examples:
      #   ry caja 1.21.4
      #   ry caja 1.21.4 --trad
      # ===================================================================== #
      when /^(\d{0,2}\.\d{0,2}\.?\d{0,2}?)$/
        use_a_specific_program_version($1.to_s.dup)
      # ===================================================================== #
      # === Use a specific version                              (version tag)
      #
      # Note that we will ignore trailing archive information such as
      # ".tar.xz" and similar.
      #
      # Usage examples:
      #
      #   ry htop --version=2.6.30.1
      #   ry gcc  --version=4.9.3.tar.xz
      #   ry bluefish --version=1.0.7
      #   ry bluefish --use-this-version=1.0.7
      #   ry networkmanager ntrad --use-this-version=NetworkManager-1.12.6.tar.xz
      #   ry mesa trad --use-this-version=mesa-18.2.6
      # 
      # ===================================================================== #
      when /^-?-?version=/,
           /^-?-?use(-|_| )?version=/,
           /^-?-?version(-|_| )?number=/,
           /^-?-?version(-|_| )?number/,
           /^-?-?version(-|_| )?number /,
           /^-?e=/,
           /^-?-?v=/,
           /^-?-?use(-|_| )?version=/,
           /^-?-?use(-|_| )?this(_|-)?version=/
        use_a_specific_program_version(i) # <- This method will handle the case.
      # ===================================================================== #
      # === /usr/local/ prefix
      #
      # This is a convenient feature mostly, in that we can quickly
      # compile into the /usr/local/ prefix.
      #
      # Example:
      #   ry gcc ULOCAL
      #   ry htop ULOCAL
      # ===================================================================== #
      when /^ULOCAL$/,
           /^-?-?opt(_|-)?prefix$/
        set_user_prefix(i)
      # ===================================================================== #
      # === Feedback the prefix in use
      #
      # If you wish to determine the --prefix that will be used for
      # the given program at hand, use this entry point here.
      #
      # Invocation example:
      #
      #   rbt htop --prefix?
      #
      # ===================================================================== #
      when *ARRAY_HELP_PREFIX_IN_USE
        feedback_prefix_in_use
      # ===================================================================== #
      # === Compile a base system for Linux
      #
      # This entry point taps into class RBT::CompileBaseSystem,
      # which will compile the programs for a typical Linux base
      # system, into an AppDir prefix.
      #
      # Invocation examples:
      #   rbt --compile-base-system
      #   rbt --compile-basic-system
      # ===================================================================== #
      when /^-?-?compile(_|-)?base(_|-)?system$/,
           /^-?-?compile(_|-)?basic(_|-)?system$/,
           /^-?-?compile(_|-)?basic(_|-)?subsystem$/,
           /^-?-?compile(_|-)?basic(_|-)?linux(_|-)?core$/,
           /^-?-?compile(_|-)?basic(_|-)?linux$/
        require 'rbt/utility_scripts/compile_base_system.rb'
        RBT::CompileBaseSystem.report_which_programs_may_be_compiled
        RBT::CompileBaseSystem.new
        exit_program
      # ===================================================================== #
      # === Feedback multiple versions
      #
      # Invocation example:
      #
      #   ry --multiple-versions
      #
      # ===================================================================== #
      when /^-?-?multiple-?versions$/,
           /^-?-?find-?duplicates$/,
           'mversions'
        RBT.find_multiple_versions
        exit_program
      # ===================================================================== #
      # === Run the post-installation step at once
      #
      # This entry point allows us to specifically run the various
      # postinstallation steps.
      #
      # Invocation example:
      #
      #   rbt htop --do-postinstall-step
      #
      # ===================================================================== #
      when /^-?-?do(_|-)?postinstall(_|-)?step$/i
        consider_performing_postinstallation_actions
        exit_program
      # ===================================================================== #
      # === Show all about a specific program
      #
      # This entry point will show all about a particular program.
      #
      # Invocation example:
      #
      #   rbt --show-all-about=htop
      #
      # ===================================================================== #
      when /^-?-?show(_|-)?all(_|-)?about=(.+)/i,
           /^-?-?show(_|-)?all=(.+)/i,
           /^-?-?show(_|-)?file=(.+)/i
        show_this_program = $2.to_s.dup # This is valid for two entry points ^^^.
        show_this_program = $3.to_s.dup if $3
        RBT::ShowAllAbout.new(show_this_program)
      # ===================================================================== #
      # === Strip the binaries of the given program at hand
      #
      # This option is similar to the "Strip the binaries as-is" variant,
      # but the difference is that we will run "strip" only on the
      # selected binaries of the passed program. Note that this also
      # requires that your working directory is where these files
      # are as well.
      #
      # Invocation example:
      #
      #   rbt --strip-binaries-of=htop
      #
      # ===================================================================== #
      when /^-?-?strip(-|_)?binaries(-|_)?of=(.+)$/i
        _ = $3.to_s.dup
        strip_the_binaries_of_this_program(_)
      # ===================================================================== #
      # === Strip the binaries as-is
      #
      # This option here allows us to call "strip" at once. The entry point
      # in particular will invoke "strip" on the current working directory,
      # which should be a bin/ subdirectory or a bin/-like subdirectory.
      #
      # Invocation example:
      #
      #   rbt --strip-binaries
      #
      # ===================================================================== #
      when /^-?-?strip(-|_)?binaries$/
        run_the_strip_command_on_this_input(:pwd)
      # ===================================================================== #
      # === Do not strip the binaries
      #
      # Invocation examples:
      #
      #   ry htop ntrad donotstripbinaries
      #   ry awk ntrad donotstripbinaries
      #
      # ===================================================================== #
      when /^-?-?do(-|_)?not(-|_)?strip(-|_)?the(-|_)?binaries$/,
           /^-?-?do(-|_)?not(-|_)?strip(-|_)?binaries$/,
           /^-?-?do(-|_)?not(-|_)?strip$/
        verbose_do_not_strip_the_binaries
      # ===================================================================== #
      # === Show the last configure options stored
      #
      # This entry point allows us to query the last configure options
      # stored (which will be stored in a .md file).
      #
      # Invocation example:
      #   rbt --last-configure-options?
      # ===================================================================== #
      when /^-?-?last(_|-)?configure(_|-)?options\??$/
        _ = RBT.file_last_configure_options
        if File.exist? _
          e orange(File.read(_).strip)
        else
          opnn; no_file_exists_at(_)
        end
      # ===================================================================== #
      # === Show the last configure options
      #
      # Note that this entry point depends on show_last_options(). Since
      # as of January 2018, another way exists, based on a .yml file
      # that keeps track of the last configure option in use.
      #
      # Usage example:
      #   ry ffmpeg last_options?
      #   ry htop last_options?
      # ===================================================================== #
      when *ARRAY_HELP_LAST_CONFIGURE_OPTIONS
        show_last_configure_options_then_exit
      # ===================================================================== #
      # === Show the last configure option for a given program
      #
      # Invocation example:
      #   ry attica --show-last-configure-option
      # ===================================================================== #
      when /^-?-?show(_|-)?last(_|-)?configure(_|-)?option$/i,
           /^-?-?show(_|-)?last(_|-)?configure(_|-)?option(_|-)?for(_|-)?this(_|-)?program$/i,
           /^-?-?was(_|-)?compiled(_|-)?via\??$/
        show_the_last_configure_option_for_this_program(compile_which_program?)
        exit
      # ===================================================================== #
      # === Delegate to RBT::Cookbooks::Cookbook
      #
      # Invocation example:
      #   rbt --scookie=php
      # ===================================================================== #
      when /^-?-?scookie=(.+)$/i
        instantiate_a_new_cookbook_for($1.to_s.dup).report
        exit_program
      # ===================================================================== #
      # === Delegate to RBT::Cookbooks::Cookbook
      #
      # Invocation example:
      #   ry calligra --scookie
      # ===================================================================== #
      when /^-?-?scookie$/,
           /^-?-?sanitize(-|_)?cookbook(-|_)?dataset$/
        instantiate_a_new_cookbook_for(program?).report
        exit_program
      # ===================================================================== #
      # === Repackage archives
      #
      # This entry point can be used to repackage an archive file.
      #
      # Usage example:
      #
      #   rbt --repackage=poppler-0.73.0.tar.gz
      #
      # ===================================================================== #
      when /^-?-?repackage=(.+)$/
        _ = $1.to_s.dup
        if File.exist? _
          repackage_this(_)
        else
          opnn; no_file_exists_at(_)
        end
      # ===================================================================== #
      # === Show the temp directory
      #
      # This entry point allows us to show the temp-directory on the
      # commandline.
      #
      # The resulting output is equivalent to:
      #
      #   RBT.config_directory?+'temp_dir.yml'
      #
      # Invocation example:
      #
      #   rbt --temp_dir?
      #
      # ===================================================================== #
      when *ARRAY_HELP_TEMP_DIR
        show_temp_directory
        exit_program
      # ===================================================================== #
      # === Show everything about a program based on the current working
      #     directory as input
      #
      # Invocation example:
      #
      #   rbt --show-all-about-this-dir
      #
      # ===================================================================== #
      when /^-?-?show(_|-)?all(_|-)?about(_|-)?this(_|-)?dir$/i,
           /^-?-?download(_|-)?as(_|-)?name$/ # === rbt --download-as-name
        show_all_about_this_dir(Dir.pwd)
      # ===================================================================== #
      # === Extract a local archive
      #
      # This entry point allows us to quickly extract a source archive
      # that exists locally. As argument to this invocation, specify
      # the location to the archive at hand.
      #
      # Invocation example:
      #
      #   rbt --extract-this=bison-3.2.4.tar.xz
      #
      # ===================================================================== #
      when /^-?-?extract(-|_)?this=(.+)$/i
        RBT.extract($2.to_s.dup)
      # ===================================================================== #
      # === Check whether a specific program is registered
      #
      # This is similar to the other inclusion check, but uses a slightly
      # different syntax.
      #
      # Invocation examples:
      #
      #   rbt --registered?=awk
      #   rbt --registered?=gawk
      #
      # ===================================================================== #
      when /^-?-?registered\??=(.+)$/i
        i = $1.to_s.dup
        opnn
        case RBT.is_this_program_included?(i, :be_quiet)
        when true
          e 'Yes, the program '+sfancy(i)+' is available and registered in the RBT project.'
        when false
          e 'No, the program '+sfancy(i)+' is NOT registered in the RBT project.'
        end 
        exit_program
      # ===================================================================== #
      # === Inclusion check
      #
      # Invocation examples:
      #   ry php  --included?
      #   ry php2 --included?
      #   ry ruby --included?
      # ===================================================================== #
      when *ARRAY_HELP_INCLUDED_OPTIONS
        report_whether_this_program_is_included_or_whether_it_is_not(program?)
        exit_program
      # ===================================================================== #
      # === Check for inclusion
      #
      # Check whether our entries in $SRC are included in our cookbooks too.
      #
      # Hence, you can use a dummy name like "x" for this:
      #
      #   ry x inclusions?
      #   ry htop check_for_inclusion
      #
      # ===================================================================== #
      when /^-?-?check(_|-)?for(_|-)?inclusion$/i,
           'inclusions','inclusions?',
           'cinc','inc?','cure','not_included?',
           'cinc?','all_not_included?'
        check_for_inclusion
        exit_program
      # ===================================================================== #
      # === Show everything about a registered program
      #
      # This entry point will show, on the commandline, the content
      # of the queried program. This allows you to quickly show
      # the .yml file that is associated with a given program at
      # hand.
      #
      # Usage example:
      #   rbt --show-all-about=ruby
      # ===================================================================== #
      when /^-?-?show(-|_)?all(-|_)?about=(.+)$/
        show_all_about($3.to_s.dup)
      # ===================================================================== #
      # === Show all locally available programs
      #
      # Invocation example:
      #   rbt --locally-available-programs?
      # ===================================================================== #
      when /^-?-?locally(-|_)?available(-|_)?programs\??$/
        show_all_locally_available_programs
        exit_program
      # ===================================================================== #
      # === Compile into your home directory in an AppDir variant
      #
      # This option allows you to compile into your user's home directory,
      # in an AppDir variant.
      #
      # Invocation example:
      #   ry htop --appdir-into-home-dir
      # ===================================================================== #
      when /^-?-?appdir(_|-)?into(_|-)?home(_|-)?dir$/,
           /^-?-?appdir(_|-)?at(_|-)?home$/
        compile_into_the_home_directory_as_appdir
      # ===================================================================== #
      # === Show important status information
      #
      # Invocation examples:
      #
      #   rbt --report-status
      #   rbt --report
      #
      # ===================================================================== #
      when /^-?-?report(-|_| )?status$/i,
           /^-?-?report$/i
        report_status_information
      # ===================================================================== #
      # === Where to store log files
      #
      # Use this entry point to report where RBT will store logfiles.
      #
      # Invocation examples:
      #
      #   rbt log_dir?
      #   rbt --log_dir?
      #
      # ===================================================================== #
      when *ARRAY_HELP_LOG_DIR_OPTIONS
        report_where_we_store_then_exit
      # ===================================================================== #
      # === Show libraries that are not available on this system
      #
      # This entry point allows us to display all library files that
      # may be missing on this host.
      #
      # Invocation example:
      #   ry --not-found
      # ===================================================================== #
      when /^-?-?not(-|_| )?found$/i
        report_which_libraries_are_missing
        exit_program
      # ===================================================================== #
      # === Download the Source of a given program at hand
      #
      # Do not build, just fetch the source. This is sort of equivalent to
      # applying "wget".
      #
      # Usage example:
      #   ry htop --dload
      #   ry ruby dload
      # ===================================================================== #
      when *ARRAY_HELP_DOWNLOAD_THE_SOURCE_OPTIONS
        # =================================================================== #
        # First check whether we may actually download the remote archive
        # or not.
        # =================================================================== #
        if may_we_download?
          _ = base_dir?
          mkdir _ unless File.exist? _
          default_download_then_exit
        else
          opne "We are not allowed to download "\
               "#{teal(compile_which_program?.to_s)}."
          opne 'The method '+royalblue('may_we_download?')+
               ' determined this, via a configuration setting.'
          exit_program
        end
      # ===================================================================== #
      # === Feedback the URL of a specific program
      #
      # This entry point allows us to make use of bin/rbt in order to
      # find out where the (remote) URL of a registered program may
      # be.
      #
      # Invocation example:
      #
      #   rbt --show-url=readline
      #
      # ===================================================================== #
      when /^-?-?show(-|_)?url=(.+)$/
        feedback_url($2.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Feedback the Program URL
      #
      # Feedback the url of a given program
      #
      # Usage examples:
      #   ry libxrandr url
      #   ry ruby url?
      #   ry python --url?
      # ===================================================================== #
      when /^-?-?url\??$/,'ulr','ur','u','url?','urla',
           /feedback_?url/
        feedback_url
        exit_program
      # ===================================================================== #
      # === Feedback program URL
      #
      # Invocation example:
      #   ry libxrandr url2
      # ===================================================================== #
      when 'url2'
        RBT::Cookbooks::Url.new(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Combining base? and url? query.
      #
      # Invocation example:
      #   ry ruby --baseurl?
      #   ry pygtk baseurl?
      # ===================================================================== #
      when /urlbase\??/,'base?url?','url?base?',/baseurl\??/,
           'basurl?','burl'
        feedback_url
        feedback_local_location
        exit_program
      # ===================================================================== #
      # === Use program URL
      #
      # Use url. Overrules the program_name of the cookbook yaml file.
      #
      #   ry ffmpeg use_url
      #
      # ===================================================================== #
      when 'use_url','useurl','uurl'
        use_url
      # ===================================================================== #
      # === Feedback local location
      #
      # This entry point will show the local location of the target
      # program (archive) at hand.
      #
      # Usage examples:
      #   ry gcc    --local_url
      #   ry aalib  local_url
      #   ry libx11 lurl
      # ===================================================================== #
      when /^-?-?local(-|_)?url$/i,
           'local','lurl','localurl?','lurl?',
           'location?','location','fulllocation?','fulllocation',
           'where?','feedback_local_location'
        feedback_local_location
        exit_program
      # ===================================================================== #
      # === Ensure that the required directories exist
      #
      # Invocation example:
      #
      #   rbt --ensure-that-the-required-directories-exist
      #
      # ===================================================================== #
      when /^-?-?ensure(-|_| )?that(-|_| )?the(-|_| )?required(-|_| )?directories(-|_| )?exist$/i
        RBT.ensure_that_the_required_directories_exist { :be_verbose }
        exit_program
      # ===================================================================== #
      # === Search for tags
      #
      # Do a search for specific tags via the entry point here.
      #
      # Invocation examples:
      #   ry --tags pdf
      #   ry tags? database
      # ===================================================================== #
      when /^-?-?tags$/,
           'tag','tags?'
        search_for_tag(compile_which_program?)
      # ===================================================================== #
      # === Register the Cookbook tags
      #
      # This method will create all registered tags (associated with the
      # cookbook entries).
      #
      # Invocation example:
      #   ry --register-tags
      # ===================================================================== #
      when /register(_|-)?tags/
        RBT.create_registered_tags
        exit_program
      # ===================================================================== #
      # === Compile all programs that have a specific tag
      #
      # This entry point allows you to compile all programs that have
      # a specific tag, such as "pdf".
      #
      # Invocation examples:
      #   rbt --compile_all_of=science
      #   rbt --compile_these_tags=science
      #   rbt --tag_compile=science
      # ===================================================================== #
      when /^-?-?compile(_|-)?all(_|-)?of=(.+)/,
           /^-?-?compile(_|-)?these(_|-)?tags=(.+)/,
           /^-?-?tag(_|-)?compile=(.+)/
        _ = $3.to_s.dup
        _ = $2.to_s.dup if _.empty?
        compile_these_tags(_)
      # ===================================================================== #
      # === Check all tags
      #
      # This entry point will check all tags under the /Programs/
      # hierarchy.
      #
      # Invocation example:
      #   ry --setup-tags
      # ===================================================================== #
      when *ARRAY_HELP_CHECK_ALL_TAGS_OPTIONS
        check_all_tags_under_programs_directory
        exit_program
      # ===================================================================== #
      # === Query the valid /System/Tags in use
      #
      # Here you can query as to which /System/Tags are valid. And
      # which ones are not.
      #
      # Invocation example:
      #   ry --valid-sys-tags?
      #   ry --available-tags?
      # ===================================================================== #
      when /^-?-?valid-?sys-?tags\??$/,
           /^-?-?allowed(-|_)?tags$/,
           /^-?-?available(-|_)?tags\??$/,
           /^-?-?allowed(-|_)?tags\??$/
        show_valid_system_tags
        exit
      # ===================================================================== #
      # === Show the project's base directory here
      #
      # Invocation example:
      #   rbt --project-base-directory?
      # ===================================================================== #
      when /^-?-?project(-|_| )?base(-|_| )?directory\??$/
        e PROJECT_BASE_DIRECTORY
      # ===================================================================== #
      # === Create the log-directory
      #
      # This entry point will create the log-directory, that is, the
      # directory that will be used by RBT::Compile to store some
      # files.
      #
      # Invocation examples:
      #
      #   rbt --create-log-dir
      #   rbt --create-log-directory
      #
      # ===================================================================== #
      when /^-?-?create(-|_| )?log(-|_| )?dir$/i,
           /^-?-?create(-|_| )?log(-|_| )?directory$/i
        ::RBT.create_log_directory
      # ===================================================================== #
      # === Skip the extracting stage
      #
      # Skip extracting the source archive, via a configuration flag.
      #
      # Invocation examples for this entry point:
      #   ry php skip_extracting
      #   ry gtk2 --skipextract
      # ===================================================================== #
      when *ARRAY_HELP_SKIP_EXTRACTING_OPTIONS
        do_not_extract
      # ===================================================================== #
      # === Query the host system
      #
      # This entry point allows us to query the host-system of the computer.
      #
      # Invocation example:
      #
      #   rbt --host-system
      #
      # ===================================================================== #
      when /^-?-?host(_|-)?system$/i
        e "The host system (CPU) in use is: "\
          "#{sfancy(RBT.host_cpu?)}"
      # ===================================================================== #
      # === Suggest the content to a .yml file
      #
      # This entry point allows us to show the content of an assumed
      # .yml file via the commandline. That way we should be more
      # rapidly able to create new .yml files or update existing
      # ones with new content. 
      #
      # Invocation example:
      #
      #   rbt --suggest-cookbook-for=glib
      #
      # ===================================================================== #
      when /^-?-?suggest(_|-)?cookbook(_|-)?for=(.+)$/i
        RBT.suggest_cookbook_for($3.to_s.dup)
      # ===================================================================== #
      # === Show which compiler we will use
      #
      # This entry point will simply display, on the commandline, which
      # compiler is used.
      #
      # Invocation examples:
      #
      #   ry --use-which-compiler?
      #   ry --compiler?
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?which(_|-)?compiler\??$/i,
           /^-?-?compiler\??$/i
        show_which_compiler_will_be_used
        exit_program
      # ===================================================================== #
      # === Run preinstall action from the commandline
      #
      # Sometimes you may wish to run a pre-install action from the
      # commandline, rather than have to rely on the .yml file (which
      # may not include this pre-install action).
      #
      # This entry point allows you to do so, and supply your own
      # pre-install actions. Individual entries can be separated via
      # a ',' character.
      #
      # Invocation examples:
      #   rbt libcaca --preinstall="sh bootstrap"
      #   rbt libcaca --prerun="sh bootstrap"
      # ===================================================================== #
      when /^-?-?preinstall="?(.+)"?$/,
           /^-?-?prerun="?(.+)"?$/
        set_preinstall_action($1.to_s.dup)
      # ===================================================================== #
      # === Create the program_version_url file
      #
      # Invocation example:
      #   rbt --create-programs-version-file
      #   rbt --programs-version-file
      # ===================================================================== #
      when /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?file$/i,
           /^-?-?programs(-|_| )?version$/i,
           /^-?-?programs(-|_| )?version(-|_| )?file$/,
           /^-?-?create(-|_| )?program(-|_| )?version$/,
           /^-?-?create(-|_| )?programs(-|_| )?version(-|_| )?html(-|_| )?file$/,
           /^-?-?create(-|_| )?programs(-|_| )?version$/
        require 'rbt/utility_scripts/create_program_version_url_file.rb'
        CreateProgramVersionUrlFile.new
      # ===================================================================== #
      # === Set a specific source directory
      #
      # This entry point allows us to use a specific source archive
      # directory.
      #
      # Invocation example:
      #
      #   rbt --set-source-archive=/Mount/USB1/last_backup/SRC/
      #
      # ===================================================================== #
      when /^-?-?set(-|_| )?source(-|_| )?archive=(.+)$/i
        opnn; set_source_dir($3.to_s.dup)
      # ===================================================================== #
      # === Stop before the 'make install' stage
      #
      # This entry point allows us to stop at, or rather right before,
      # the 'make install' stage.
      #
      # Invocation example:
      #   rbt htop --stop-at-make-install-stage
      # ===================================================================== #
      when /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?install(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?install(-|_| )?stage$/i,
           /^-?-?stop(-|_| )?before(-|_| )?make(-|_| )?install$/i,
           /^-?-?stop(-|_| )?at(-|_| )?make(-|_| )?install(-|_| )?step$/i
        stop_at_make_install_stage
      # ===================================================================== #
      # === Set source directory
      #
      # Set the source dir here.
      #
      # Usage example:
      #   ry htop --source_dir=/Depot/j
      # ===================================================================== #
      when /^-?-?source(-|_| )?dir$/,
           'source_dir',
           'set_source_dir'
        opnn; set_source_dir(first_arg)
      # ===================================================================== #
      # === Check if source directories exist
      #
      # This entry point, a "clean-up step", can be used to find out
      # whether all source directories exist or whether they do not.
      #
      # Invocation example:
      #   ry --check-if-source-dirs-exist
      # ===================================================================== #
      when /-?-?check(-|_| )?if(-|_| )?source(-|_| )?dirs(-|_| )?exist$/
        RBT.check_if_source_directories_exist
        exit_program
      # ===================================================================== #
      # === Do not use any configure options
      #
      # This entry point allows you to completely ignore all configure
      # options specified in the particular .yml file.
      #
      # Invocation examples:
      #
      #   ry htop --ntad --dont-use-configure-options
      #   ry glib --user-home-prefix --use-meson --no-configure-options
      #   ry ruby --do-not-use-any-configure-options
      #
      # ===================================================================== #
      when /^-?-?dont(_|-)?use(_|-)?configure(_|-)?options$/i,
           /^-?-?do(_|-)?not(_|-)?use(_|-)?configure(_|-)?options$/i,
           /^-?-?no(_|-)?configure(_|-)?options$/i,
           /^-?-?barebones$/i,
           /^-?-?do(_|-)?not(_|-)?use(_|-)?any(_|-)?configure(_|-)?options$/i,
           /^-?-?dont(_|-)?use(_|-)?any(_|-)?configure(_|-)?options$/i
        do_not_use_any_configure_option { :be_verbose }
      # ===================================================================== #
      # === Enable shared compilation
      #
      # This entry point allows a user to enable shared compilation.
      #
      # Invocation example:
      #   ry htop ntrad --enable-shared
      # ===================================================================== #
      when /^-?-?enable(-|_| )?shared$/i
        append_enable_shared_to_the_configure_options
      # ===================================================================== #
      # === Store all abbreviations in a file
      #
      # Invocation example:
      #
      #   rbt --store-abbreviations
      #
      # ===================================================================== #
      when /^-?-?store(-|_| )?abbreviations$/,
           /^-?-?store$/i,
           /^-?-?generate(-|_| )?abbreviations$/i,
           /^-?-?generate(-|_| )?aliases$/i,
           /^-?-?abbreviations(-|_| )?to(-|_| )?the(-|_| )?programs$/i,
           /^-?-?create(-|_| )?abbreviations$/i
        RBT.store_abbreviations
      # ===================================================================== #
      # === Compile a specific KDE5 application
      #
      # Invocation examples:
      #   ry --kde_apps1
      #   ry --kde_apps4
      # ===================================================================== #
      when /^-?-?kde(-|_| )?apps(\d+)$/i
        index = $2.to_s.to_i - 1
        this_kde5_application = return_kde5_applications[index]
        set_compile_these_programs(this_kde5_application)
      # ===================================================================== #
      # === Query the PATH in use
      #
      # Invocation example:
      #   ry --path?
      # ===================================================================== #
      when /^-?-?path\?$/i
        e slateblue(ENV['PATH'])
      # ===================================================================== #
      # === Query the path to the windows cookbook file
      #
      # Invocation example:
      #   rbt --windows-cookbook?
      # ===================================================================== #
      when /^-?-?windows(-|_| )?cookbook\??$/i
        e 'The windows_cookbook.yml file resides at:'
        e
        e "  #{sfile(RBT::FILE_WINDOWS_COOKBOOK)}"
        e
      # ===================================================================== #
      # === Disable static compilation
      #
      # This entry point exists so that users can disable static
      # compilation, e. g. when the corresponding .yml file has
      # a "build_static: t" entry by default.
      #
      # Invocation example:
      #   ry wget --disable-static-compilation
      # ===================================================================== #
      when /^-?-?disable(-|_| )?static(-|_| )?compilation$/i,
           /^-?-?dont(-|_| )?compile(-|_| )?statically$/i
        disable_static_compilation(:be_verbose)
      # ===================================================================== #
      # === Specify to use cmake as build-system
      #
      # Invocation example:
      #
      #   ry htop --use-cmake
      #
      # ===================================================================== #
      when /^-?-?use(_|-)?cmake$/i
        set_use_this_build_system(:cmake)
      # ===================================================================== #
      # === Feedback compile strategies used
      #
      # Invocation example:
      #   rbt --compile_strategies?
      #   rbt --build-statistics
      # ===================================================================== #
      when /^-?-?compile(_|-)?strategies\??$/i,
           /^-?-?build(_|-)?statistics\??$/i
        report_the_compile_strategies
      # ===================================================================== #
      # === Generate a MacHomebrew Formula
      #
      # This should allow you to generate a Mac homebrew formula.
      #
      # Usage example:
      #   ry m4 machomebrew
      # ===================================================================== #
      when /^-?-?generate_?machomebrew_?formula$/,
           /^-?-?mac-?home-?brew$/
        generate_machomebrew_formula
        exit_program
      # ===================================================================== #
      # === Generate a mac homebrew script
      #
      # This entry point may be merged with the entry point above in the
      # long run. For now (November 2018) it will be kept separate though.
      #
      # Usage example:
      #   ry php ghs
      # ===================================================================== #
      when 'generate_homebrew_script',
           'generatehomebrewscript',
           'ghs'
        generate_homebrew_script(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Compile a specific kde component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular KDE component. So for example, "kde1" will be
      # a shortcut for "extracmakemodules".
      #
      # Invocation examples:
      #   ry kde1
      #   ry kde2
      #   ry kde3
      #   ry --kde1
      #   ry --kde2
      #   ry --kde3
      #   ry --kde4
      #   ry --kde5
      #   ry --kde10
      #   ry kde11
      #   ry kde15
      #   ry kde1..100
      #   ry --kde1..kde250
      # ===================================================================== #
      when /^-?-?kde(\d{1,3}\.{0,2}[a-z]{0,3}\d{0,3})/ # <- See: http://rubular.com/r/PzADEmm1Jc
        try_to_compile_this_kde_component($1.to_s.dup)
      # ===================================================================== #
      # === Compile a specific KDE5 plasma component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular KDE5 Plasma component.
      #
      # Invocation examples:
      #   ry plasma1
      # ===================================================================== #
      when /^-?-?plasma(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_plasma_component($1.to_s.dup)
      # ===================================================================== #
      # === Copy header (.h) files to the current working directory
      #
      # This entry point copies .h files that are registered for
      # a given program at hand, into the current working directory.
      #
      # This entry point is a bit different from the variant that is
      # used via --copy-headers-from=, in that we will ONLY copy
      # registered .h files that are available under an AppDir-like
      # prefix. The other method can also use .h files that reside
      # under the /usr/include/ hierarchy.
      #
      # Invocation examples:
      #   ry glibc --copy-headers
      #   ry glibc --copy-headers-into-pwd
      #   ry glibc --copy-headers-to-pwd
      # ===================================================================== #
      when /^-?-?copy(-|_| )?headers$/i,
           /^-?-?copy(-|_| )?headers(-|_| )?into(-|_| )?pwd$/i,
           /^-?-?copy(-|_| )?headers(-|_| )?to(-|_| )?pwd$/
        copy_headers_from(appdir_prefix?+'include/')
        exit_program
      # ===================================================================== #
      # === Copy header (.h) files from a certain path
      #
      # This entry point allows us to e. g. copy the .h header files from,
      # say, glibc into the current working directory.
      #
      # Invocation example:
      #   ry glibc --copy-headers-from=/usr/include
      # ===================================================================== #
      when /^-?-?copy(-|_| )?headers(-|_| )?from=(.+)$/
        this_directory = $3.to_s.dup
        copy_headers_from(this_directory)
        exit_program
      # ===================================================================== #
      # === Do not use any special environment flag at all
      #
      # Sometimes you really do not want to use ENV, that is, the
      # environment. The entry point here allows you to not only
      # empty ENV, but to also not set any new ENV flags.
      #
      # Invocation examples:
      #
      #   ry htop --do-not-use-special-flags
      #   ry nspr --clear-envs ntrad
      #   ry nss  --clear-envs ntrad
      #
      # ===================================================================== #
      when /^-?-?do(_|-)?not(_|-)?use(_|-)?special(_|-)?flags$/,
           /^-?-?clear(_|-)?envs$/
        verbose_do_not_use_special_flags
      # ===================================================================== #
      # === Clear the ENV (environment) variable
      #
      # This entry point allows us to clear the environment that ruby
      # uses, aka ENV.
      #
      # Invocation example:
      #   ry gtk --clear-env
      # ===================================================================== #
      when /^-?-?clear(_|-)?env$/
        clear_env_variables_but_ensure_the_path_variable_works
      # ===================================================================== #
      # === Query whether the cookbook will modify env variables
      #
      # This option exists to give users means to query, from the
      # commandline, whether the target cookbook-yaml file will make
      # use of special ENV variables.
      #
      # For example, the program called "mesa" may require the
      # manipulation of the ENV variable called "GLL_DRV" being
      # set, before compilation should start.
      #
      # Invocation example:
      #   ry mesa --modify_env_variables?
      # ===================================================================== #
      when /^-?-?modify(_|-)?env(_|-)?variables\??$/
        feedback_whether_the_cookbook_will_modify_env_variables
        exit_program
      # ===================================================================== #
      # === Compile all locally available versions of a source archive
      #
      # This entry point allows us to compile all locally available
      # versions of a program, if we have stored it as an archive,
      # such as a .tar.xz file or something like that.
      #
      # The advantage, or "big win" here, is that we can batch-compile
      # all these programs into the corresponding /Programs/NAME/VERSION/
      # directory.
      #
      # Invocation examples:
      #   ry libsigc++ --compile-all-available-local-versions
      #   ry libsigc++ --all-available-local-versions
      # ===================================================================== #
      when /^-?-?compile(-|_| )?all(-|_| )?available(-|_| )?local(-|_| )?versions$/i,
           /^-?-?all(-|_| )?available(-|_| )?local(-|_| )?versions$/i
        do_compile_all_available_local_version_of_this_program(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Shortcut for a specific program version
      #
      # This entry point allows us to pick a specific version more easily.
      #
      # See the following usage example.
      #
      # Usage example:
      #   ry python --2
      # ===================================================================== #
      when /^--(\d+)$/
        this_program = compile_these_programs?.first
        directory = source_directory?+this_program+'/'
        available_programs = Dir[directory+'*']
        this_version = $1.to_s.dup
        selection = available_programs.select {|entry|
          entry =~ /-#{this_version}/
        }
        if selection.empty?
          opne "Not found any local program version for "\
               "the input `#{sfancy(this_version)}`."
        else
          _ = selection.first
          this_version = ProgramInformation.return_version_of(_)
          set_program_version(this_version)
        end
      # ===================================================================== #
      # === Check for available program versions
      #
      # This entry point will check for the available program versions
      # of the given program at hand.
      #
      # Usage examples:
      #   ry libsigc++ --available_versions
      #   ry bluefish --available_versions
      #   ry gcc ?
      # ===================================================================== #
      when *ARRAY_HELP_AVAILABLE_PROGRAM_VERSIONS
        check_for_available_versions
        exit_program
      # ===================================================================== #
      # === Enable bootstrap for compilation
      #
      # Invocation example:
      #
      #   ntrad binutils --enable-bootstrap
      #
      # ===================================================================== #
      when /^-?-?enable(-|_| )?bootstrap$/i
        append_this_to_the_configure_options(:enable_bootstrap)
      # ===================================================================== #
      # === Symlink the lib/ subdirectory into /usr/lib/
      #
      # This entry point allows us to symlink the compiled library-files,
      # in particular the .so files, into the /usr/lib/ hierarchy.
      #
      # This will only be honoured when the program is compiled in an
      # AppDir-like fashion, e. g. under /Programs/Foobar/1.1.0/.
      #
      # Invocation examples:
      #
      #   ry gtk+ --symlink-into-usr-lib-dir
      #   ry gtk+ --symlink-into-lib-dir
      #   ry gtk+ --symlink-lib-dir
      #   ry gtk+ --symlink-libs
      #
      # ===================================================================== #
      when /^-?-?symlink(-|_| )?into(-|_| )?usr(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?into(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?lib(-|_| )?dir$/i,
           /^-?-?symlink(-|_| )?libs$/i,
           /^-?-?symlink(-|_| )?into(-|_| )?usr(-|_| )?lib$/i
        do_symlink_appdir_lib_into_usr_lib
      # ===================================================================== #
      # === Use different configure options
      #
      # We must allow the user to use a different set of configure options.
      # This entry point ensures that.
      #
      # The user will be able to pass in the configure options specified
      # in a commandline-flag.
      #
      # Invocation examples:
      #   ry gcc --use-these-configure-options="--enable-shared --enable-bootstrap --enable-languages=c,c++,lto,objc,fortran --enable-threads=posix --enable-checking=release --enable-objc-gc --with-system-zlib --enable-libstdcxx-dual-abi --with-default-libstdcxx-abi=gcc4-compatible --disable-libunwind-exceptions --enable-__cxa_atexit --enable-libssp --enable-lto --disable-install-libiberty --with-gnu-ld --verbose --with-arch-directory=amd64 --disable-gtktest --disable-multilib --target=x86_64-slackware-linux --build=x86_64-slackware-linux --host=x86_64-slackware-linux"
      #   ry poppler --use-these-configure-options="--enable-xpdf-headers"
      # ===================================================================== #
      when /^-?-?use(-|_| )?these(-|_| )?configure(-|_| )?options=(.+)$/
        @internal_hash[:configure_options] = $4.to_s.dup
      # ===================================================================== #
      # === Enable pkgconfig-symlinks
      #
      # This option exists in order to allow, on the commandline, to
      # enable the symlink-action of .pc files into /usr/lib/pkgconfig.
      #
      # This is only useful if you use an AppDir compilation step of
      # course.
      #
      # Note that the regex below is anchored, so to avoid conflicts
      # with instructions such as --symlink-pkgconfig-files.
      #
      # Invocation example:
      #   ry htop --pkgconfig-symlink
      #   ry libxi --symlink-pkgconfig-files
      # ===================================================================== #
      when /^-?-?pkgconfig(_|-)?symlink$/,
           /^-?-?symlink(_|-)?pkgconfig(_|-)?files$/
        verbose_enable_pkgconfig_symlink
      # ===================================================================== #
      # === Show the .pc files installed by the program at hand
      #
      # If you want to find out which .pc files are installed by the
      # program at hand, this entry point allows you to do so.
      #
      # Issue something similar to the following usage examples:
      #
      #   ry glib --pkgconfig-files?
      #   ry samba --pkgconfig-files?
      #   ry poppler pkgconfig_files?
      #
      # ===================================================================== #
      when /^-?-?pkgconfig(_|-)?files\??/,
           /^-?-?show(_|-)?pkgconfig(_|-)?files/
        show_pc_files_installed_by_this_program(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Symlink all upcased programs to downcased variants
      #
      # This entry point will add a symlink for an existing, upcased
      # program directory. For example, /Programs/ZLib will receive
      # a symlink called /Programs/Zlib.
      #
      # Invocation example:
      #   rbt --symlink-upcased-programs
      # ===================================================================== #
      when /^-?-?symlink(_|-)?all(_|-)?upcased(_|-)?programs(_|-)?to(_|-)?downcased(_|-)?variants$/i,
           /^-?-?symlink(_|-)?upcased(_|-)?programs$/i,
           /^-?-?downcase(_|-)?programs$/i,
           /^-?-?symlink(_|-)?all(_|-)?programs$/
        symlink_all_upcased_programs_to_downcased_variants
      # ===================================================================== #
      # === Resymlink all programs
      #
      # This entry point will resymlink all programs under /Programs again,
      # or respectively, wherever you have set your $PROGRAMS value at.
      #
      # Invocation examples:
      #   rbt --resymlink
      #   rbt --resymlink-all-programs
      # ===================================================================== #
      when /^-?-?resymlink$/i,
           /^-?-?do(_|-)?symlink(_|-)?all(_|-)?programs$/i,
           /^-?-?resymlink(-|_| )?all(-|_| )?programs$/
        do_symlink_all_programs_then_exit
      # ===================================================================== #
      # === Purge all binaries of the given program at hand
      #
      # This entry point will remove all binaries of the given input
      # program at hand under /usr/bin/.
      #
      # Invocation examples:
      #
      #   rbt poppler --purge-its-binaries
      #   rbt vim --purge-binaries
      #
      # ===================================================================== #
      when /^-?-?purge(-|_| )?its(-|_| )?binaries$/,
           /^-?-?purge(-|_| )?binaries$/
        purge_these_binaries(binaries?)
        exit_program
      # ===================================================================== #
      # === Purge FHS binaries of a specific program
      #
      # This will delete all binaries under /usr/bin/ hierarchy of the
      # given program at hand.
      #
      # It currently depends on all registered programs in the yaml file.
      # In principle we could also work on existing bin/ files in the
      # AppDir hierarchy, but for now, let's stick to the yaml file instead.
      #
      # Invocation examples:
      #
      #   ry littleutils --purge-fhs-binaries
      #   ry --purge-traditional-binaries-from=xscreensaver
      #   ry --purge-traditional-binaries-of=gnupg
      #   ry --purge-traditional-binaries-of=flac
      #
      # ===================================================================== #
      when /^-?-?purge-?fhs-?binaries$/i,
           /^-?-?purge-?traditional-?binaries-?from=(.+)/i,
           /^-?-?purge-?fhs-?binaries-?from=(.+)/i,
           /^-?-?purge-?fhs-?binaries=(.+)/i,
           /^-?-?fhs-?purge=(.+)/i,
           /^-?-?purge-?binaries-?of=(.+)/i,
           /^-?-?purge(-|_)?traditional(-|_)?binaries(-|_)?of=(.+)$/i
        _ = nil
        _ = $1 if $1
        _ = $4 if $4 # This is for the last entry.
        _ = program_name? if _.nil?
        verbose_purge_fhs_binaries_of_this_program(_.to_s.dup)
        exit
      # ===================================================================== #
      # === No man pages
      #
      # Use this subsection to get rid of all manpages in the /Programs/
      # hierarchy. Invoke this only when you are sure to not need these
      # man-pages.
      #
      # Usage example:
      #   rbt --no-man-pages
      # ===================================================================== #
      when *ARRAY_HELP_PURGE_ALL_MAN_ENTRIES
        purge_all_man_files_in_the_programs_hierarchy
        exit_program
      # ===================================================================== #
      # === Show a specific compile chain
      #
      # This entry point allows you to e. g. display which members
      # belong to the compile-chain plasma.
      #
      # Invocation examples:
      #
      #   ry --view=kde5_plasma
      #   ry --show-chain=plasma
      #   ry --compile-chain-of=kde5_plasma
      #   ry --show-compile-chain-of=kde5_plasma
      #   ry --show-compile-chain-of=mate
      #
      # ===================================================================== #
      when /^-?-?view=(.+)$/i,                                # <- Aim for $1.
           /^-?-?show(-|_)?chain=(.+)$/i,                     # <- Aim for $2.
           /^-?-?compile(-|_)?chain(-|_)?of=(.+)$/i,          # <- Aim for $3.
           /^-?-?show(-|_)?compile(-|_)?chain(-|_)?of=(.+)$/i # <- Aim for $4.
        if $4
          of_this_program = $4.to_s.dup
        elsif $3
          of_this_program = $3.to_s.dup
        elsif $2
          of_this_program = $2.to_s.dup
        elsif $1
          of_this_program = $1.to_s.dup
        end
        show_compile_chain(of_this_program)
        exit_program
      # ===================================================================== #
      # === Feedback everything
      #
      # Feedback EVERY available information.
      #
      # This is the most important option for feedback. It chains the
      # existing feedback options together.
      #
      # Usage example:
      #   ry ruby --feedback_all
      # ===================================================================== #  
      when 'ud','fall','uv','vu','info','lal','aall','al','aa','info?',
           '?',/^-?-?feedback(-|_)?all$/,'allinfo','everything',
           'feedback_everything','feedbackeverything'
        feedback_all # Wrapper.
        exit_program
      # ===================================================================== #
      # === Show the configuration options for a given program
      #
      # Invocation example:
      #   rbt gcc --show-configuration-options
      # ===================================================================== #
      when /^-?-?show(-|_| )?configuration(-|_| )?options$/i
        RBT::Cookbooks::ShowConfigurationOptions.new(compile_which_program?)
        exit_program
      # ===================================================================== #
      # === Guessing the build type
      #
      # This entry point allows us to guess the build type of a target
      # program at hand.
      #
      # Invocation examples:
      #   rbt --guess-build-type
      #   rbt --build-type
      # ===================================================================== #
      when /^-?-?build(-|_| )?type$/i,
           /^-?-?guess(-|_| )?build(-|_| )?type$/i
        verbose_guess_the_build_type
        exit_program
      # ===================================================================== #
      # === Copy archives to the current working directory
      #
      # This entry point allows us to copy e. g. a directory, such as the archive
      # of ruby (at a local location like /Users/x/SRC/ruby/, into the current
      # working directory. The idea here is to be able to quickly copy a select
      # subset of source archives, which we can then transfer via FTP or a local
      # USB device.
      #
      # Invocation examples:
      #
      #   rbt --copy-these-archives=ruby,php,python
      #   rbt --copy-archives=kde5_plasma
      #   rbt --copy-archives=:kde5_plasma
      #
      # ===================================================================== #
      when /^-?-?copy(-|_| )?these(-|_| )?archives=(.+)$/i,
           /^-?-?copy(-|_| )?archives=(.+)$/i
        these_programs = $2.to_s.dup
        these_programs = $3.to_s.dup if $3 # <- Valid for the first regex-entry above.
        copy_these_archives(these_programs)
        exit
      # ===================================================================== #
      # === Compile all of Mate-Desktop into a standalone directory
      #
      # This entry point allows us to compile all of MATE into a
      # standalone directory, such as /Programs/Mate/1.20.
      #
      # Invocation examples:
      #
      #   rbt --compile-mate-desktop-components-into-one-standalone-directory
      #   rbt --compile-mate-desktop-components-into-a-standalone-directory
      #   rbt --compile-mate-desktop-components-into-standalone-directory
      #
      # ===================================================================== #
      when /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?one(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?a-standalone(-|_| )?directory$/,
           /^-?-?compile(-|_| )?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?components(-|_| )?into(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?components(-|_| )?standalone(-|_| )?directory$/,
           /^-?-?mate(-|_| )?desktop(-|_| )?into(-|_| )?standalone(-|_| )?dir$/,
           /^-?-?mate(-|_| )?prefix$/,
           /^-?-?mate(-|_| )?standalone$/
        compile_mate_desktop_components_into_one_standalone_directory
        exit_program
      # ===================================================================== #
      # === Show n last compiled programs
      #
      # This entry point allows us to show the last n compiled programs.
      #
      # Invocation example:
      #
      #   ry --last-compiled=20
      #
      # ===================================================================== #
      when /^-?-?last(-|_| )?compiled=(\d+)$/
        n_programs = $2.to_s.to_i
        feedback_last_compiled(n_programs)
        exit_program
      # ===================================================================== #
      # === Show the last compiled program
      #
      # Use this method if you wish to show the last compiled program.
      #
      # Invocation example:
      #   ry htop last_compile?
      # ===================================================================== #
      when *ARRAY_HELP_LAST_COMPILED_OPTIONS
        show_last_compiled_program :then_exit
      # ===================================================================== #
      # === Compile a specific mate-desktop component
      #
      # This entry point allows us to use numbers in order to compile
      # a particular mate-desktop component. So for example, "mate1"
      # will be a shortcut for "matedesktop" whereas "mate2" will
      # be a shortcut for "libmatemixer", and so forth.
      #
      # Invocation examples:
      #
      #   ry mate1
      #   ry mate2
      #   ry mate3
      #   ry --mate1
      #   ry --mate2
      #   ry --mate3
      #   ry mate11
      #   ry mate1..100
      #
      # ===================================================================== #
      when /^-?-?mate(\d{1,3}\.{0,2}\d{0,3})$/ # <- See: http://rubular.com/r/n2r8DwcTnU
        try_to_compile_this_mate_component($1.to_s.dup)
      # ===================================================================== #
      # === Enable python3
      #
      # Invocation example:
      #   rbt --enable_python3
      # ===================================================================== #
      when /^-?-?enable(-|_| )?python3$/
        enable_python3
      # ===================================================================== #
      # === Compile all the protos next
      #
      # Invocation example:
      #   ry --compile-all-protos
      #   ry protos
      #   ry --protos
      # ===================================================================== #
      when /^-?-?compile(_|-)?all(_|-)?protos$/i, # ry --compile-all-protos
           /^-?-?protos$/,
           /^-?-?all(_|-)?protos$/,
           /^-?-?compile(_|-)?protos$/,
           'proto'
        register_all_protos_for_compilation
      # ===================================================================== #
      # === Feedback all binaries
      #
      # Query for all binaries of any given program and show them when it
      # was found.
      #
      # This obviously requires of you to provide the name of a program.
      #
      # See the following use cases.
      #
      # Usage examples:
      #   ry htop     --all_binaries?
      #   ry automake all_binaries?
      #   ry coreutils binaries?
      #   ry xz feedback_all_binaries
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_ALL_BINARIES_OPTIONS
        show_all_binaries_of_this_program
        exit_program
      # ===================================================================== #
      # === Upload binaries
      #
      # Invocation example:
      #   rbt --upload-binaries
      # ===================================================================== #
      when /^-?-?upload(-|_)?binaries$/
        opne 'We will upload all files found in the respective '\
             'bin/ directory'
        opne 'of this program ('+simp(program?)+').'
        do_upload_binaries
      # ===================================================================== #
      # === Feedback how many binaries are registered
      #
      # This entry point tells us how many binaries are registered in
      # total. Binaries in this context refers to programs that reside
      # under the corresponding bin/ hierarchy.
      #
      # However had, if a program has been assigned to the main dataset
      # already then the behaviour of the method will be different -
      # in this case the method will report how many binaries are
      # part of that program at hand.
      #
      # Invocation examples:
      #   rbt --n-binaries?
      #   rbt glib --n-binaries?
      # ===================================================================== #
      when /^-?-?n(_|-)?binaries\??$/
        either_report_the_total_amount_of_binaries_or_how_many_binaries_this_program_will_install
        exit_program
      # ===================================================================== #
      # === Upload a ruby binary
      #
      # Use this entry point to upload a ruby binary to a remote FTP site.
      #
      # Usage example:
      #   rbt --upload-ruby-binary
      # ===================================================================== #
      when /(-|_)?(-|_)?upload(-|_)?ruby(-|_)?binary/
        do_upload_ruby_binary
        exit_program
      # ===================================================================== #
      # === Compile all programs that belong to a certain tag
      #
      # This entry point allows us to compile all programs that belong
      # to a certain tag, such as "gnome" or "kde5".
      #
      # Invocation examples:
      #  rbt --compile-this-tag=gnome
      #  rbt --compile-this-tag=kde
      # ===================================================================== #
      when /^-?-?compile(-|_| )?this(-|_| )?tag=(.+)$/
        _ = $3.to_s.dup
        compile_all_programs_that_belong_to_this_tag(_)
      # ===================================================================== #
      # === Do not remove empty directories
      #
      # This entry point exists so that we can keep empty directories
      # in the respective /Programs/PROGRAM_NAME/PROGRAM_VERSION/
      # subdirectory.
      #
      # Invocation examples:
      #
      #   ry feh --ntrad --do-not-remove-empty-directories
      #   ry htop --ntrad --do-not-remove-empty-directories
      #
      # ===================================================================== #
      when /^-?-?do(-|_| )?not(-|_| )?remove(-|_| )?empty(-|_| )?directories$/
        opne slateblue('class RBT::SymlinkProgram')+' will '+
             crimson('not')+' remove empty directories.'
        symlink_program?.do_keep_empty_directories
      # ===================================================================== #
      # === Get compile-instructions from a locally existing file
      #
      # This entry point can be used to use the content of a file for
      # determining which programs should be compiled. That way you
      # may have a bit more flexibility, such as when another ruby
      # script generates this file, which can then be used by RBT.
      #
      # In fact, this is the original use case why this method has been
      # created in the first place.
      #
      # The file can handle .yml files too, where we will use YAML.load_file()
      # to load the content, rather than File.read() or File.readlines().
      #
      # Invocation examples:
      #   rbt --upgrade-from-this-file=/Depot/jjj/these_programs_can_be_upgraded.yml
      #   rbt --upgrade-from-this-file=:default
      #   rbt --upgrade-from-this-file=:def
      # ===================================================================== #
      when /^-?-?upgrade(-|_| )?from(-|_| )?this(-|_| )?file=(.+)$/i # See: http://rubular.com/r/HjEJRwWc4f
        this_file = $4.to_s.dup
        do_upgrade_from_this_file(this_file)
      # ===================================================================== #
      # === Do not keep the archive
      #
      # After extracting and compiling, we will get rid of the archive
      # again. We will get rid of the archive independent over whether
      # the installation process was a success or whether it was not,
      # mind you. 
      #
      # Usage examples:
      #   ry htop --remove-archive
      #   ry htop dontkeeparchive
      # ===================================================================== #
      when *ARRAY_HELP_DONT_KEEP_EXTRACTED_OPTIONS
        set_dont_keep_archive # Remove archive(s) after a successful build.
      # ===================================================================== #
      # === Query the headers of the system
      #
      # This entry point is just a wrapper towards class
      # RBT::QueryHeaderToPackage.
      #
      # Invocation examples:
      #   rbt --query-headers
      #   rbt --query-headers=/usr/local/include/
      # ===================================================================== #
      when /^-?-?query(-|_)?headers$/i,
           /^-?-?query(-|_)?headers=(.+)$/i
        _ = $2.to_s.dup
        _.delete!('-')
        if _.empty?
          _ = :default
        end
        require 'rbt/utility_scripts/query_header_to_package.rb'
        RBT::QueryHeaderToPackage.new(_)
        exit_program
      # ===================================================================== #
      # === Show postinstall
      #
      # This entry point allows you to simply show the postinstall
      # dataset for the given program at hand.
      #
      # Usage examples:
      #   ry docbookxml postinstall?
      #   ry perl postinstall?
      #   ry libraw1394 postinstall?
      # ===================================================================== #
      when /^-?-?postinstall?/,
           /^-?-?show(_|-)?postinstall/i,
           'pinstall?',
           'p?',
           /^-?-?display(_|-)?postinstall$/,
           /^-?-?postinstallation(_|-)?steps\??$/i
        display_postinstall_dataset
        exit_program
      # ===================================================================== #
      # === Obtain a random program and compile it
      #
      # This may be useful if you just want to check whether compiling
      # any of the registered programs works, without caring too much
      # which entry it is. RBT will randomly pick a program and
      # use that for compilation.
      #
      # Invocation example:
      #   ry --random
      #   ry --random-program
      # ===================================================================== #
      when 'RANDOM',
           /^-?-?random$/,
           /^-?-?return(_|-)?random(_|-)?program$/,
           /^-?-?random(_|-)?program$/i
        _ = return_random_program
        opne "Compiling the program `#{sfancy(_)}` next."
        set_compile_these_programs(_) # <- This will also invoke set_compile_this_program(_)
      # ===================================================================== #
      # === Validate Cookbook Aliases
      #
      # Invocation example:
      #   ry htop vca
      #   ry htop --vca
      # ===================================================================== #
      when /^validate_?cookbook_?aliases$/,
           /^-?-?vca/,/-?-?cookbook(_|-)?aliases$/,
           'calias'
        RBT::Cookbooks::ValidateCookbookAliases.new # bl $RUBY_COOKBOOKS/lib/cookbooks/validate_cookbook_aliases.rb
        exit_program
      # ===================================================================== #
      # === Use the working directory as input for everything
      #
      # This entry point allows us to simply use information gathered by
      # the current working directory as input.
      #
      # An example for such a directory would be:
      #
      #   /Depot/j/libxkbcommon-0.7.2/
      #
      # Invocation example:
      #   ry --use-current-working-directory-as-input
      # ===================================================================== #
      when /^-?-?use(-|_)?current(-|_)?working(-|_)?directory(-|_)?as(-|_)?input$/
        _ = File.basename(Dir.pwd)
        do_skip_extracting
        do_not_download
        set_extract_to File.dirname(Dir.pwd)
        set_configure_base_directory Dir.pwd
        program_information = ProgramInformation.new(_)
        @internal_hash[:program_version] = program_information.version?
        @internal_hash[:short_name]      = program_information.short_name?
        @internal_hash[:real_short_name] = program_information.real_short_name?
        @internal_hash[:base_dir]        = rds(Dir.pwd+'/')
        set_compile_this_program(@internal_hash[:real_short_name])
        set_compile_these_programs(@internal_hash[:real_short_name])
      # ===================================================================== #
      # === Feedback Programs Directory
      #
      # This entry point shows us where the /Programs/ directory is at.
      #
      # Invocation example:
      #   ry --programs_dir?
      # ===================================================================== #
      when /^-?-?programs(_|-)?dir\??$/,'pdir?'
        feedback_programs_dir
        exit_program
      # ===================================================================== #
      # === Compile with the listed dependencies as well
      #
      # This entry point allows us to compile all the specified
      # dependencies in a .yml file, prior to the main program
      # at hand.
      #
      # Note that this will always compile everything listed there,
      # even if it is already installed. We may have to change this
      # behaviour in the future, which will then be mentioned here,
      # as well - but for the time being, this is the documented
      # behaviour.
      #
      # Invocation examples:
      #   ry kbreakout --compile-dependencies-as-well
      #   ry python --with-deps   
      # ===================================================================== #
      when /^-?-?compile(_|-)?dependencies(_|-)?as(_|-)?well$/,
           /^-?-?with(_|-)?deps$/
        compile_the_main_program_including_the_dependencies
      # ===================================================================== #
      # === Create a gentoo Ebuild Recipe
      #
      # This entry point allows us to build a gentoo-specific ebuild
      # file. This is an experimental feature as of the year 2018.
      #
      # For the format specification you can look at this remote wiki:
      #
      #   https://wiki.gentoo.org/wiki/Ebuild
      #
      # Invoke it in such a way:
      #   ry ruby --create_ebuild_recipe
      #   ry htop create_ebuild_recipe
      #   ry python ebuild
      # ===================================================================== #
      when /^-?-?create(_|-)?ebuild(_|-)?recipe$/,
           'make_ebuild',
           'ebuild','gentoo','gent'
        create_ebuild_recipe
        exit_program
      # ===================================================================== #
      # === rbt --repeat_last_configure_option
      # ===================================================================== #
      when /^-?-?repeat(_|-)?last(_|-)?configure(_|-)?option$/i,
           /^-?-?repeat$/
        esystem(return_last_configure_command)
        exit_program
      # ===================================================================== #
      # === Skip the postinstall action
      #
      # This allows you to ignore the postinstall instructions.
      #
      # Usage example:
      #   ry htop --no-postinstall
      # ===================================================================== #
      when /^-?-?skip(_|-)?postinstall(_|-)?actions$/i,
           '-P',
           /^-?-?no(_|-)?postinstall$/i
        opne 'We will skip any potential postinstallation actions.'
        skip_postinstall_actions
      # ===================================================================== #
      # === Backup Program
      #
      # This can be used to backup a single (individual) program.
      #
      # Invocation example:
      #   ry htop --backup-program
      # ===================================================================== #
      when /^-?-?backup(_|-)?programs?$/,
           'backup-programs',
           'programs_backup',
           'packup',
           'backup'
        backup_program
        exit_program
      # ===================================================================== #
      # === Do not use any compile time flags
      #
      # Sometimes you may not wish to use any FLAG variables such as
      # CFLAGS or LDFLAGS. This option here allows you to ignore these
      # FLAGS globally. 
      #
      # Invocation example:
      #   rbt htop --no-compile-time-flags
      #   rbt strace --no-compile-time-flags ntrad
      # ===================================================================== #
      when /^-?-?no(_|-)?compile(_|-)?time(_|-)?flags$/,
           /^-?-?plain$/,
           /^-?-?buildtype=plain$/
        do_not_use_any_compile_time_flags
      # ===================================================================== #
      # === Obtain available Packages
      #
      # Get all available packages as a filtered result.
      #   ry htop --packages?
      # ===================================================================== #
      when /^-?-?packages\??$/i,
          'package',
          'available?',
          'available',
          'pgk?',
          'pkg?'
        get_packages
        exit_program
      # ===================================================================== #
      # === Validate the cookbook entries
      #
      # Invocation example:
      #   rbt --validate-entries
      # ===================================================================== #
      when /^-?-?validate(_|-)?entries$/
        require 'rbt/validation/batch_validate_the_cookbook_recipes.rb'
        RBT.validate_entries
      # ===================================================================== #
      # === set_pkgconfig_symlink
      #
      # Invocation example:
      #   ry htop --pkgconfig-symlink=yes
      # ===================================================================== #
      when /^-?-?pkgconfig-?symlink=(.+)$/
        set_pkgconfig_symlink($1.to_s.dup) { :be_verbose }
      # ===================================================================== #
      # === Disable the use of md5sum calculation
      #
      # Invocation example:
      #   ry --no-md5sum
      # ===================================================================== #
      when /^-?-?no-?md5sum$/,
           'no-md5sum','--nomd5sum',
           'nomd5sum',
           /^-?-?disable_?md5sum_?calculation$/
        disable_md5sum_calculation
      # ===================================================================== #
      # === Speed up compilation via make -jNUMBER
      #
      # This will automatically use the "-j" option that equals the
      # number of processors that the current computer has.
      #
      # Invocation example:
      #   ry flac ntrad --speed-up-the-compilation
      # ===================================================================== #
      when /^-?-?speed(_|-)?up(_|-)?the(_|-)?compilation$/i,
           /^-?-?speed(_|-)?up(_|-)?compilation$/
        speed_up_the_compilation
      # ===================================================================== #
      # === Show the last compiled programs
      #
      # Usage example:
      #   rbt --last-5-compiled?
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_LAST_COMPILED_OPTIONS,
           /^lastcompiled\?=$/,
           /^-?-?last-compiled\?=$/,
           /^-?-?last(-\d+-)compiled\?$/
        feedback_last_compiled(i)
        exit_program
      # ===================================================================== #
      # === Reset the status.yml file next
      #
      # Invocation example:
      #   ry --reset-the-status-file
      # ===================================================================== #
      when /^-?-?reset(_|-)?the(_|-)?status(_|-)?file$/
        opne 'Reseting the file '+sfile(RBT.file_status)+' next.'
        reset_the_status_file
        exit_program
      # ===================================================================== #
      # === Generate the pdf tutorial, then upload it
      #
      # Invocation example:
      #   rbt --generate_pdf_tutorial_then_upload_it
      # ===================================================================== #
      when /^-?-?generate(_|-)?pdf(_|-)?tutorial(_|-)?then(_|-)?upload(_|-)?it$/i
        ::RBT.generate_pdf_tutorial_then_upload_it
        exit_program
      # ===================================================================== #
      # === Check for gnome updates
      # ===================================================================== #
      when /^-?-?gnome$/,
           /^-?-?check(_|-)?for(_|-)?gnome(_|-)?updates$/
        RBT::Cookbooks::CheckForGnomeUpdates.new
        exit_program
      # ===================================================================== #
      # === rbt PWD
      # ===================================================================== #
      when /^PWD$/i
        _ = File.basename(Dir.pwd)
        set_compile_this_program(_)
        set_compile_these_programs(_)
      # ===================================================================== #
      # === Feedback extended description about a program
      #
      # Feedback extended description to the user.
      #
      # ry htop desc2
      # ===================================================================== #
      when /^-?-?description2$/,
           'desc2'
        feedback_extra_description
        feedback_description
        exit_program
      # ===================================================================== #
      # === Update library names
      #
      # Invocation example:
      #   ry --uln
      # ===================================================================== #
      when /^-?-?update(_|-)?library(_|-)?names/i,
           /^-?-?uln/i,
           'registered_libraries'
        update_library_names
        exit_program
      # ===================================================================== #
      # === Generate new cookbook
      #
      # Invocation example:
      #   ry --generate_new_cookbook
      # ===================================================================== #
      when /^-?-?generate(_|-)?new(_|-)?cookbook$/i
        generate_new_cookbook
      # ===================================================================== #
      # === Symlink into the /usr/include/ hierarchy
      #
      # Invocation example:
      #   rbt mpfr ntrad --symlink-headers
      # ===================================================================== #
      when /^-?-?symlink(_|-)?headers$/,
           /^-?-?symlink(_|-)?h$/,
           'sheaders'
        verbose_do_symlink_headers_into_usr_include_hierarchy
      # ===================================================================== #
      # === Feedback every case option of the main menu here
      #
      # Invocation example:
      #   ry htop case_options?
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_EVERY_CASE_OPTION_OPTIONS
        feedback_all_case_options_of_the_main_menu
        exit_program
      # ===================================================================== #
      # === Do not use maximum speed
      #
      # Entry point:
      #   rbt lame --do-not-use-maximum-speed
      # ===================================================================== #
      when /^-?-?do(_|-)?not(_|-)?use(_|-)?maximum(_|-)?speed$/i,
           /^-?-?no(_|-)?auto(_|-)?speedup$/i
        verbose_disable_maximum_speed_for_compilation
      # ===================================================================== #
      # === Use a specific build directory
      #
      # Sometimes you may want to use another build directory.
      #
      # This options here ensures that you can do so.
      #
      # Usage example:
      #
      #   ry htop --use-this-build-dir=/Depot/j
      #   ry htop --build-dir=/Depot/j
      #   ry python --build-dir=/Depot/jjj/
      # ===================================================================== #
      when /^-?-?use-this-build-dir=(.+)/,
           /^-?-?use-this-build-directory=(.+)/,
           /^-?-?build-dir=(.+)/
        do_use_a_build_directory :be_verbose
        this_build_directory = $1.to_s.dup
        use_this_build_directory(this_build_directory, :report_back)
      # ===================================================================== #
      # === Feedback all Libaries
      #
      # This will feedback all libraries that are installed by the given
      # program at hand. This of course requires of you to actually pass
      # in the name of the program.
      #
      # Invocation example:
      #   ry htop --all-libraries?
      # ===================================================================== #
      when /^-?-?all(-|_)?libraries\??$/,
           'libraries?'
        feedback_all_libraries
        exit_program
      # ===================================================================== #
      # === Register all programs
      #
      # Usage example:
      #   ry --register_all
      # ===================================================================== #
      when /-?-?register(_|-)?all$/,
           'regall'
        register_all_programs
        exit_program
      # ===================================================================== #
      # === Do not make install
      #
      # Invocation example:
      #   ry gcc dontmakeinstall
      # ===================================================================== #
      when /^-?-?dont(-|_)?make(-|_)?install$/i
        dont_make_install
      # ===================================================================== #
      # === Clear the global flags
      #
      # Invocation example:
      #   rbt --clear-global-flags
      # ===================================================================== #
      when /^-?-?clear(_|-)?global(_|-)?flags$/i
        clear_global_flags
        exit_program
      # ===================================================================== #
      # === Show the Recipes directory
      #
      # ry htop recipes_dir?
      # ===================================================================== #
      when /^-?-?recipes(_|-)?dir\??/,'recipesdir'
        show_the_recipes_directory
        exit_program
      # ===================================================================== #
      # == Set to verbose output
      #
      # Invocation example:
      #   ry htop --verbose
      # ===================================================================== #
      when '--verbose','-verbose','verbose','verb','set_verbose','be_verbose',
           'beverbose','setverbose'
        set_be_verbose :report_to_the_user # Second true so we report to the user.
      # ===================================================================== #
      # === Verify the md5sum
      #
      # Examples for md5:
      #   ry gcc3   md5
      #   ry python md
      #   ry ruby   --md
      # ===================================================================== #
      when 'md5',/^-?-?md$/,'mdcheck'
        verbose_md5_checking
        exit_program
      # ===================================================================== #
      # === Query the debug value
      #
      # Query the debug value:
      #   ry --debug?
      # ===================================================================== #
      when /^-?-?query(_|-)?debug(_|-)?value$/,/^-?-?debug\??$/
        query_debug_value
        exit_program
      # ===================================================================== #
      # === Feedback the program description
      #
      # Feedback the program description here:
      #
      #   ry gcc --desc
      #   ry htop desc?
      #
      # ===================================================================== #
      when /^-?-?desc$/,
           'de','des','descr','descri','descrip','descript',
           'descripti','descriptio','description','beschreibung',
           'desc?','description?','d?','feedback_description',
           'desc1'
        feedback_program_description
        exit_program
      # ===================================================================== #
      # === Create a standalone .yml file
      #
      # Invocation example:
      #   ry ruby --create_standalone_yml
      #   ry ruby syml
      # ===================================================================== #
      when /^-?-?create(_|-)?standalone(_|-)?yaml(_|-)?file$/i,
           /^-?-?create(_|-)?standalone(_|-)?yml$/i,
           'syml','generate_standalone',
           /^-?-?csyf/ # 'yaml' <-- can not be used.
        create_standalone_yaml_file
        exit_program
      # ===================================================================== #
      # === Use a specific compile-version
      #
      # This allows us to use another version at compile-time.
      #
      # This only works if we compile in an app-dir like variant.
      # ===================================================================== #
      when /^-?-?compile-?version=(.+)/,
           /^-?-?use_?this_?compile_?version=(.+)/
        use_this_compile_version($1.to_s.dup)
      # ===================================================================== #
      # === Scan the Source Archive
      #
      # Scan source archive:
      #
      #   ry --scan_source_archive
      #
      # ===================================================================== #
      when *ARRAY_HELP_SCAN_SOURCE_ARCHIVE_OPTIONS
        scan_source_archive
        exit_program
      # ===================================================================== #
      # === Add a packager-user
      #
      # Add a packager-user this way:
      #   ry php add_package_user
      # ===================================================================== #
      when /^-?-?add(_|-)?package(_|-)?user$/i,
           'adduser','packageuser','useradd'
        add_package_user
        exit_program
      # ===================================================================== #
      # === Generate global yaml database
      #
      # To generate the global yaml database, do this:
      #   ry --gdatabase
      # ===================================================================== #
      when /^-?-?generate(_|-)?global(_|-)?database$/,
           /^-?-?database$/,
           /^-?-?gdatabase$/
        generate_global_yaml_database
        exit_program
      # ===================================================================== #
      # === Do not run a GNU configure script
      #
      # Skip running a configure script. This may be necessary for
      # entries that solely rely on "make" and "make install".
      #
      # Usage example:
      #
      #   ry htop --skip_configure
      #
      # ===================================================================== #
      when /^-?-?skip(_|-)?configure$/i,
           'skipconfigure','sconfigure','skip','skipconf',
           'dontconfigure','skipc','sconfig','sc','dconf',
           'dcon','dconfigure',
           /^-?-?do(-|_)?not(-|_)?run(-|_)?configure$/i,
           /^-?-?do(-|_)?not(-|_)?run(-|_)?configure(-|_)?script$/i,
           /^-?-?skip(-|_)?configure$/i
        do_not_run_configure
      # ===================================================================== #
      # === Show all registered keys
      #
      # Show which keys are registered.
      #
      # Invocation examples: 
      #   ry --allregistered?
      #   ry --show_registered_keys
      #   ry allregistered?
      #   ry  show_registered_keys
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_REGISTERED_KEYS
        show_registered_keys
        exit_program
      # ===================================================================== #
      # === Install only headers
      #
      # Use this to install only the headers of a particular program.
      #
      # Usage example:
      #   ry gmp --only-headers
      # ===================================================================== #
      when /^-?-?install(_|-)?only(_|-)?headers$/,
           /^-?-?only(_|-)?headers$/,
           /^-?-?do(_|-)?install(_|-)?only(_|-)?the(_|-)?headers$/,
           'headers',
           'header'
        do_install_only_the_headers
      # ===================================================================== #
      # === Install only the bin/ components
      #
      # This entry point can be used to install only the bin/ components
      # of a given program.
      #
      # The reasoning for this option is that we sometimes may only wish
      # to install all entries in the bin/ subdirectory, for one reason
      # or the other.
      #
      # Usage example:
      #   ry htop do_install_only_binaries
      #   ry xpr  --onlybinaries
      # ===================================================================== #
      when /^-?-?only(_|-)?binaries$/,
           '--only-bin',
           /^-?-?do(_|-)?install(_|-)?only(_|-)?binaries/
        do_install_only_binaries
      # ===================================================================== #
      # === Query which headers are part of this package
      #
      # Use this if you wish to give back all .h files of a specific
      # program at hand. For this to work, of course the program in
      # question needs to install any .h files in the first place; and
      # we will also have had these .h files registered in the first
      # place.
      #
      # Invocation example:
      #   ry glibc --headers?
      # ===================================================================== #
      when /^-?-?headers\??$/,
           /^-?-?show(_|-)?available(_|-)?headers(_|-)?of(_|-)?this(_|-)?program$/
        show_available_headers_of_this_program
        exit
      # ===================================================================== #
      # === Upload to a remote site
      #
      # ry htop upload_to
      # ===================================================================== #
      when 'upload_to','uploadto','upload','uto'
        upload_to(i)
        exit_program
      # ===================================================================== #
      # === Do not show the classname
      #
      # If we don't want to display the classname, use this.
      #
      # Trying this:
      #   ry php noclass fall
      # ===================================================================== #
      when 'noclassname','noclass'
        @internal_hash[:do_show_the_classname] = :be_silent
      # ===================================================================== #
      # === Run in simulation mode
      #
      # Use this to run a simulation.
      #   ry htop simulation
      # ===================================================================== #
      when *ARRAY_HELP_SIMULATION_MODE_OPTIONS
        enable_simulation('Running in simulation mode. No real '\
                          'action will be performed.')
      # ===================================================================== #
      # === Symlink all the programs
      #
      # If you ever want to symlink all entries under /Programs, this
      # is the way to go.
      #
      # Symlink all programs:
      #   ry htop symlinkall
      #   ry x traverse
      #   ry symlink_every_program 
      # ===================================================================== #
      when *ARRAY_SYMLINK_ALL_PROGRAMS_OPTIONS
        do_symlink_all_programs_then_exit
      # ===================================================================== #
      # === Feedback the main configuration
      #
      # This entry point can be used to show the main configuration of
      # the RBT project.
      #
      # Invocation example:
      #   ry --main-config?
      # ===================================================================== #
      when *ARRAY_HELP_CONFIG_OPTIONS
        feedback_main_config
        exit_program
      # ===================================================================== #
      # === Toggle the debug value
      #
      # This method will toggle the debug value, then query it.
      #
      # Invocation example:
      #   ry htop debug
      # ===================================================================== #
      when *ARRAY_HELP_TOGGLE_DEBUG_VALUE_OPTIONS
        toggle_the_debug_value_then_query_the_debug_value
        exit_program
      # ===================================================================== #
      # === Use a white background
      #
      # Invocation example:
      #   ry htop --whitebg
      # ===================================================================== #
      when /^-?-?background-?white$/,'--white-background',
           /^-?-?whitebg$/,
           /^-?-?do_?use_?a_?white_?background/
        do_use_a_white_background
      # ===================================================================== #
      # === Last compiled program
      #
      # Invocation example:
      #   ry LAST
      # ===================================================================== #
      when *ARRAY_HELP_LAST_COMPILED_PROGRAM_OPTIONS
        set_compile_these_programs(last_compiled_program?)
      # ===================================================================== #
      # === Feedback program version
      #
      # Use this to feedback the specific program version, that is,
      # the version that is specified in the specific yaml file
      # belonging to a program.
      #
      # Invocation example:
      #
      #   ry gcc version?
      #
      # =>   5.3.0
      # ===================================================================== #
      when *ARRAY_HELP_FEEDBACK_PROGRAM_VERSION_OPTIONS
        feedback_program_version
        exit_program
      # ===================================================================== #
      # === Show help options
      #
      # Show the help options here, then exit.
      #   ry --help
      # ===================================================================== #
      when *ARRAY_HELP_OPTIONS
        display_standard_help_options
        exit_program
      # ===================================================================== #
      # === Apply only the sed operation next
      #
      # Invocation examples:
      #   rbt htop --only-sed
      #   rbt krb5 --only-sed
      # ===================================================================== #
      when /^-?-?only(-|_)?sed$/i
        determine_the_configure_base_directory_to_use
        extract_program
        cd_to_the_configure_base_directory_to_be_used
        check_for_sed_action
        exit_program
      # ===================================================================== #
      # === Do not make use of cookbook aliases
      #
      # This entry point here exists to prevent the use of cookbook
      # aliases. Normally, the aliases are useful, since they allow
      # us to use abbreviations, and easier-to-type names for commonly
      # used programs. However had, sometimes we distinctly do NOT want
      # to use a specific alias, such as when we wish to compile "sdlttf"
      # and the cookbook alias automatically picks "sdlttff" for us
      # instead.
      #
      # Invocation examples:
      #   ry sdlttf --do-not-use-cookbook-aliases
      #   ry sdlttf --do-not-use-cookbook-alias
      # ===================================================================== #
      when /^-?-?do(_|-)?not(_|-)?use(_|-)?cookbook(_|-)?aliases$/i,
           /^-?-?do(_|-)?not(_|-)?use(_|-)?cookbook(_|-)?alias$/i,
           /^-?-?do(_|-)?not(_|-)?make(_|-)?use(_|-)?of(_|-)?cookbook(_|-)?aliases$/i,
           /^-?-?no(_|-)?alias$/
        do_not_use_cookbook_aliases(:be_verbose)
      # ===================================================================== #
      # === Use another directory as the new /Programs/ dir
      # ===================================================================== #
      when /^-?-?use-?this-?programs-?dir=(.+)/
        use_this_as_programs_dir($1.to_s.dup) # See: http://rubular.com/r/HR72sazzMi
      # ===================================================================== #
      # === Convert to non-traditional
      #
      # Convert a FHS-conforming program back into a non-traditional
      # hierarchy:
      #
      #   ry htop to_programs
      #
      # ===================================================================== #
      when 'to_programs','toprograms','convert_to_non_traditional','tobin',
           'to_fhs','toubin'
        convert_to_non_traditional
        exit_program
      # ===================================================================== #
      # === rbt LAST_DOWNLOADED
      #
      # The input is normally downcased - but I will keep the upcased
      # variant as main entry point; it seems easer to remember.
      # ===================================================================== #
      when *ARRAY_HELP_LAST_DOWNLOADED_OPTIONS
        # =================================================================== #
        # The format of the following file may be like this here:
        #   /Users/x/SRC/redmine/redmine-3.2.0.tar.gz # 2015-12-31 23:48:48 +0100
        # =================================================================== #
        file = LAST_DOWNLOADED_FILE
        if File.exist? file
          # ================================================================= #
          # We grab the name of the program found in the above file.
          # ================================================================= #
          i = ::ProgramInformation.return_program_name(
            File.basename(File.readlines(file).first)
          )
          i = [i] # We expect it to be an Array lateron.
          clear_commandline_options
        else
          ewarn 'No file at `'+sfile(file)+'` exists. Exiting now.'
          exit_program
        end
      # ===================================================================== #
      # === Uninstall a specific AppDir                       (uninstall tag)
      #
      # This option allows you to uninstall a specific AppDir again.
      #
      # All clean-up steps will be run.
      #
      # Invocation example:
      #   rbt evince --uninstall-appdir
      # ===================================================================== #
      when /^-?-?uninstall(_|-)?appdir$/i,
           /^-?-?uninstall$/i
        uninstall_this_appdir
        exit_program
      # ===================================================================== #
      # === Compile every program
      #
      # Here we will try to compile every program that is part of our
      # database.
      #
      # Invocation example:
      #   ry --compile_all
      # ===================================================================== # 
      when /^-?-?compile(_|-)?all$/,
           'all',
           'compile_all_programs',
           'rall'
        compile_all_programs(:do_exit_afterwards) # Argument true for "shall we exit".
      # ===================================================================== #
      # === Extract a program only
      #
      # Extract a given program only, without continuing past this stage.
      #
      # Invocation examples:
      #   ry php eonly
      #   ry htop --extract-only
      # ===================================================================== #
      when 'extract_only','extractonly','extract','eonly',
           'extract-only',/^-?-?extract-?only$/,
           'eo'
        verbose_do_extract_only
      # ===================================================================== #
      # === Publish a list of all programs versions
      #
      # Invocation example:
      #
      #   rbt --publish-list-of-all-programs-version
      #
      # ===================================================================== #
      when /^-?-?publish(_|-)?list(_|-)?of(_|-)?/i # This is copied 1:1 from the Cookbooks project.
        RBT.publish_list_of_all_programs_version
        exit_program
      # ===================================================================== #
      # === Do not compile
      #
      # The following instruction will run configure, but not make.
      #
      # Invocation examples:
      #   ry gcc dont_compile
      #   ry htop dc
      # ===================================================================== #
      when *ARRAY_HELP_DO_NOT_COMPILE_OPTIONS
        do_not_compile
        do_not_run_configure
        skip_postinstall_actions
      # ===================================================================== #
      # === Feedback all programs that do use a build directory
      #
      # This entry point simply reports only those programs that make
      # use of a build directory.
      #
      # Invocation examples:
      #
      #   ry --show-all-programs-that-make-use-of-a-build-directory
      #   ry --all-build-directory
      #   ry --all-build-directories
      #   ry --build-directories?
      #   ry --build-directories
      #
      # ===================================================================== #
      when /^-?-?show(_|-)?all(_|-)?programs(_|-)?that(_|-)?make(_|-)?use(_|-)?of(_|-)?a(_|-)?build(_|-)?directory$/i,
           /^-?-?all(_|-)?build(_|-)?directory$/i,
           /^-?-?all(_|-)?build(_|-)?directories$/i,
           /^-?-?build(_|-)?directories\??$/i
        report_all_programs_that_make_use_of_a_build_directory
        exit_program
      # ===================================================================== #
      # === Be completely quiet
      #
      # This directive will tell us to be completely silent.
      #
      # Invocation example:
      #   ry htop --be_so_quiet
      # ===================================================================== #
      when /^-?-?be(_|-)?completely(_|-)?quiet$/,
           '--be-very-quiet','be_so_quiet',
           /^-?-?be(_|-)?so(_|-)?quiet$/
        be_completely_quiet
      # ===================================================================== #
      # === Generate a slack-desc file
      #
      # Usage examples:
      #   ry htop --gsc
      #   ry ruby slack-desc-file
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_SLACK_DESC_FILE
        generate_slack_desc_file_for_this_program(program?)
        exit_program
      # ===================================================================== #
      # === Find all archives
      #
      # Invocation example:
      #   ry --archives?
      # ===================================================================== #
      when *ARRAY_HELP_FIND_ALL_ARCHIVES
        find_all_archive_types
        exit_program
      # ===================================================================== #
      # === Query whether we will use colours
      #
      # This entry point allows you to query whether colours will be
      # used in class RBT::Compile for the current compilation run.
      #
      # Invocation example:
      #   ry --use-colours?
      # ===================================================================== #
      when /^-?-?use(-|_)?colours\??$/
        report_whether_colours_will_be_used
        exit_program
      # ===================================================================== #
      # === Show the Basename
      #
      # This entry point will show where a particular program can be
      # found locally.
      #
      # Invocation example:
      #   ry gcc basename?
      # ===================================================================== #
      when 'basedir2?','basedirr','bdir','base?','base','localbase?','bdir?',
           'b?','whereis?','whereis',
           /^-?-?basename\??$/i
        feedback_local_base
        exit_program
      # ===================================================================== #
      # === Show the dependencies of a given program
      # 
      # Tell which dependencies a given program has:
      #
      #   ry sed --dependencies
      #   ry gcc --dependencies
      #   ry xsm fdep
      #
      # ===================================================================== #
      when 'abhängigkeiten',/^-?-?dependencies$/,
           'depe','deps','dep','fdep',
           'd','deps?','dep?','dependencies?',
           /^-?-?feedback_?dependencies_?of_?this_?program/
        feedback_dependencies_of_this_program
        exit_program
      # ===================================================================== #
      # === Show the installation_procedure.yml file
      #
      # rbt --installation-procedure
      # ===================================================================== #
      when /-?-?installation(-|_)?procedure$/
        show_installation_procedure_file
        exit_program
      # ===================================================================== #
      # === Show the intro for RBT.
      #
      # Invocation example:
      #
      #   rbt --intro
      # 
      # ===================================================================== #
      when /-?-?intro$/,
           /show(_|-)?intro$/,
           /^show(_|-)?rbt(_|-)?intro$/
        show_rbt_intro
        exit_program
      # ===================================================================== #
      # === Use build directory
      #
      # Instruct RBT to use a build directory.
      #
      # Usage examples:
      #   ry htop build
      #   ry automake usebuilddir
      #   ry automake use_build_dir ntrad
      #   ry modules  use_build_dir
      #   ry yakuake --use-build-dir
      #   ry yakuake --trad --use-build-dir
      # ===================================================================== #
      when *ARRAY_HELP_USE_BUILD_DIRECTORY_OPTIONS
        do_use_a_build_directory
      # ===================================================================== #
      # === Finding all Archives
      #
      # This entry point can be used to find all local archives.
      #
      # Finding all Archives:
      #   ry --archives?
      # ===================================================================== #
      when /-?-?find_?all_?archive_?types/,'farchives',
           /-?-?archives\??/
        _ = RBT::Cookbooks::FindAllArchiveTypes.new(:do_not_run_yet)
        _.be_silent
        _.run_and_report
        exit_program
      # ===================================================================== #
      # === Expand the cookbooks
      #
      # Invocation examples:
      #   rbt --expand
      #   rbt --create-expanded-directory
      # ===================================================================== #
      when /^-?-?expand$/,
           /^-?-?expand(-|_)?cookbooks$/,
           /^-?-?create(-|_)?expanded(-|_)?directory$/
        do_expand_the_cookbooks
        exit_program
      # ===================================================================== #
      # === Installed files
      #
      # This will tell us which files are to be installed. For this
      # to work, we will load the .yml file.
      #
      # Usage example:
      #   ry htop --show_which_files_will_be_installed
      # ===================================================================== #
      when '--installed-files?',
           /^-?-?show(_|-)?which(_|-)?files(_|-)?will(_|-)?be(_|-)?installed$/,
           'ifiles','--installed-files','--installedfiles'
        show_which_files_will_be_installed
        exit_program
      # ===================================================================== #
      # === Package RBT Scripts
      #
      # Package all the RBT Scripts together, into one tarball/archive.
      #
      #   ry --package_rbt_scripts
      #   ry --prs
      #   ry --make_release
      # ===================================================================== # 
      when *ARRAY_HELP_PACKAGE_RBT_OPTIONS
        package_rbt_scripts :then_exit
      # ===================================================================== #
      # === Replace Symlinks
      #
      # Use this to replace the symlinks, using RBT::ReplaceSymlinks.
      # Then we will exit.
      #
      # Usage example:
      #   ry --replace_symlinks
      # ===================================================================== #
      when *ARRAY_HELP_REPLACE_SYMLINKS_OPTIONS
        replace_symlinks
        exit_program
      # ===================================================================== #
      # === Check for the latest program
      #
      # Check for the latest program:
      #   ry --latest
      #   ry --check-for-latest
      # ===================================================================== #
      when /^-?-?latest$/,
           'latest?','not_latest?',
           'not_latest','sanitize',
           'clean',
           /^-?-?check(_|-)?for(_|-)?latest$/
        check_for_latest
        exit_program
      # ===================================================================== #
      # === Query whether to use a specific build directory
      #
      # This entry point allows us to determine whether a given program
      # will use a build directory.
      #
      # Usage example:
      #   ry htop --usebuilddir?
      #   ry libsysstat usebuilddir?
      # ===================================================================== #
      when /^-?-?usebuilddir\??$/i
        report_whether_a_build_directory_will_be_used_for_this_program
        exit_program
      # ===================================================================== #
      # === Update registered binaries
      #
      # Update our list of Registered Binaries (the same as running regbin)
      #
      # Invocation example:
      #   ry --update-registered-binaries
      # ===================================================================== #
      when *ARRAY_HELP_UPDATE_BINARY_NAMES_OPTIONS
        update_binary_names :then_exit
      # ===================================================================== #
      # === Do use autoconf
      #
      # This allows us to use/invoke autoconf.
      #
      # Usage example:
      #   ry htop --run-autoconf
      # ===================================================================== #
      when /^-?-?run-?autoconf$/,
           /do(-|_)?use(-|_)?autoconf$/,
           'autoconf+'
        do_use_autoconf
      # ===================================================================== #
      # === Do not use autoconf
      # ===================================================================== #
      when /^-?-?dontautoconf$/,
           /do(-|_)?not(-|_)?use(-|_)?autoconf$/
        do_not_use_autoconf
      # ===================================================================== #
      # === Show the base directory
      #
      # Show the base directory of a given program.
      #
      # Invocation example:
      #   ry htop base_dir?
      # ===================================================================== #
      when /^-?-?base(_|-)?dir\??$/,
           /^basedir\??/
        show_where_the_base_directory_can_be_found
        exit_program
      # ===================================================================== #
      # === Generate PDF tutorial
      #
      # Invocation example:
      #   ry --pdf-generation
      # ===================================================================== #
      when *ARRAY_HELP_GENERATE_PDF_TUTORIAL_OPTIONS
        verbose_generate_the_pdf_tutorial
        exit_program
      # ===================================================================== #
      # === Show the last 20 compiled programs
      #
      # Invocation example:
      #   rbt --last-compiled
      # ===================================================================== #
      when /^-?-?last(-|_)?compiled$/,
           /^-?-?last(-|_)?programs$/
        feedback_last_compiled(20)
        exit_program # ^^^ This variant uses hardcoded 20 programs.
      # ===================================================================== #
      # === Feedback whether we are currently compiling anything
      #
      # Invocation example:
      #   rbt --currently_compiling?
      # ===================================================================== #
      when /-?-?currently(_|-)?compiling\??/,
           /-?-?what-?are-?we-?currently-?compiling\??/,
           /-?-?currently(_|-)?compiled\??/
        feedback_what_we_are_currently_compiling?
        exit
      # ===================================================================== #
      # === Feedback configure options
      #
      # This part will show the configure options for a given program.
      #
      # Feedback the configure option for a specific program at hand:
      #   ry htop configure?
      #   ry gcc  configure?
      # ===================================================================== #
      when *ARRAY_FEEDBACK_CONFIGURE_OPTIONS,
           /configure_?option\??/,
           'showconfig?'
        feedback_configure_options
        exit_program
      # ===================================================================== #
      # === Enable cmake
      #
      # We deliberately enable cmake here.
      #
      # Usage example:
      #   ry htop cmake
      # ===================================================================== #
      when 'cmake','--cmake','-cmake',
           /^-?-?use(_|-)?ccmake$/
        do_use_cmake
      # ===================================================================== #
      # === Show which colours we will use
      #
      # Invocation example:
      #   ry --show-which-colours-will-be-used
      #   ry htop colours?
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_WHICH_COLOURS_WILL_BE_USED
        show_which_colours_will_be_used
        exit_program
      # ===================================================================== #
      # === Report the registered cookbook entries/programs
      #
      # Show how many programs are registered.
      #
      # Usage examples:
      #   ry --nprograms
      #   ry --reportregisteredprograms
      # ===================================================================== #
      when *ARRAY_HELP_REPORT_N_PROGRAMS_OPTIONS
        report_n_programs
        exit_program
      # ===================================================================== #
      # === Show Last Updated
      #
      # Show when some programs were last updated.
      #
      # Invocation example:
      #   ry --slu
      # ===================================================================== #
      when *ARRAY_SHOW_LAST_UPDATED_OPTIONS
        RBT::Cookbooks::ShowLastUpdated.new(display_n_programs: 875)
        exit_program
      # ===================================================================== #
      # === Designate which programs to compile
      #
      # This entry point can be used to assign which programs are to be
      # compiled. This is most useful if you wish to compile more than
      # one program.
      #
      # Commandline invocation example:
      #
      #   rbt --compile-these-programs=make,sed,awk,coreutils,utillinux,gawk,
      #
      # ===================================================================== #
      when /^-?-?compile(-|_)?these(-|_)?programs=(.+)/
        compile_these_programs = $3.to_s.dup
        compile_these_programs.chop! if compile_these_programs.end_with? ','
        if compile_these_programs.include? ','
          compile_these_programs = compile_these_programs.split(',')
        end
        set_compile_these_programs(compile_these_programs)
      # ===================================================================== #
      # === Show how to compile KDE
      #
      # This entry point will give some help as to how to compile KDE.
      #
      # Invocation example:
      #   rbt --how-to-compile-kde?
      #   rbt --show-how-to-compile-kde
      # ===================================================================== #
      when /^-?-?how(_|-)?to(_|-)?compile(_|-)?kde\??$/,
           /^-?-?show(_|-)?how(_|-)?to(_|-)?compile(_|-)?kde\??$/
        show_how_to_compile_kde
        exit_program
      # ===================================================================== #
      # === Show help options                                      (help tag)
      #
      # Invocation example:
      #   ry --help
      # ===================================================================== #
      when /^-?-?help$/
        show_help
        exit
      # ===================================================================== #
      # === Enable stow
      #
      # Enable stow via this option.
      #
      # Invocation example:
      #   ry --enable-stow
      # ===================================================================== #
      when /-?-?enable-?stow/
        enable_stow
        exit_program
      # ===================================================================== #
      # === Show the highest programs
      #
      # Show the highest programs, that is, the biggest in size, available
      # locally.
      #
      # Invocation example: 
      #   ry htop --show_highest
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_HIGHEST_PROGRAMS_OPTIONS
        show_n_largest_local_programs(150)
      # ===================================================================== #
      # === Sleep before continuing
      #
      # Delay also works as alias to it (since Oct 2011).
      #
      # Usage example:
      #   ry htop sleep5
      #   ry htop --sleep5 --ntrad
      # ===================================================================== #
      when /^-?-?sleep(\d+)$/,
           /^-?-?delay(\d+)$/,
           /-sl(\d+)$/
        n_seconds = $1
        opne 'Sleeping for `'+teal(n_seconds.to_s)+'` seconds before continuing.'
        set_delay_before_continuing(n_seconds)
      # ===================================================================== #
      # === Whether we will use stow
      #
      # Invoke this entry point like so:
      #
      #   ry --stow?
      #
      # ===================================================================== #
      when *ARRAY_HELP_DO_USE_STOW_OPTIONS
        feedback_whether_we_will_use_stow
        exit_program
      # ===================================================================== #
      # === Show how to use batch-compile
      #
      # Invocation example:
      #   ry --batch-compile?
      # ===================================================================== #
      when /-?-?batch-?compile\??$/,
           /-?-?show(_|-)?how(_|-)?to(_|-)?batch(_|-)?compile/,
           /--batch-?help/
        show_how_to_batch_compile
        exit_program
      # ===================================================================== #
      # === Use glib schemata
      #
      # This option allows us to install an AppDir program with the
      # additional glib-schemata step, thus overruling any other
      # setting from the cookbook-yaml file at hand.
      #
      # Invocation example:
      #   rbt anjuta ntrad --use-glib-schemata
      # ===================================================================== #
      when /^-?-?use-?glib-?scheme$/i,
           /^-?-?use-?glib-?schemata$/i,
           /^-?-?glib-?scheme$/i,
           /^-?schemata$/i
        verbose_enable_glib_schema
      # ===================================================================== #
      # === Show all configure options
      #
      # This will show a beautiful summary of the configure options.
      #
      # Invocation example:
      #   ry htop allconfigureoptions?
      #   ry htop --so-pretty
      # ===================================================================== #
      when /-?-?all(_|-)?configure(_|-)?options\??/,
           '--beautify','--configure_help','do_beautify',
           'dobeautify',/-?-?pretty/,'--beauty','--so-pretty'
        all_configure_options?
      # ===================================================================== #
      # === Remove all temporary directories
      #
      # This entry point will get rid of all directories found at the
      # /Depot/Temp/ hierarchy - or whever else you have defined your
      # temp hierarchy to reside.
      #
      # Why is this entry point useful? Well, sometimes this directory
      # may be a bit messy, so this entry point allows you a simple
      # way to "start over". We only remove directories because these
      # may include stray object files or cmake-cache files and so on.
      #
      # Invocation example:
      #   ry --remove-all-temp-directories
      # ===================================================================== #
      when /^-?-?remove(_|-)?all(_|-)?temp-directories$/,
           /^-?-?purge(_|-)?old(_|-)?directories$/,
           /^-?-?purge(_|-)?old(_|-)?dirs$/,
           /^-?-?purge(_|-)?base(_|-)?directory$/,
           /^-?-?purge(_|-)?base(_|-)?dir$/,
           /^-?-?clear(_|-)?temp(_|-)?dir$/,
           /^-?-?purge(_|-)?directories$/,
           /^-?-?purge(_|-)?dirs$/,
           /^-?-?purge(_|-)?dir$/,
           /^-?-?purge(_|-)?base(_|-)?dirs$/,
           /^-?-?purge(_|-)?temp(_|-)?dir$/,
           /^-?-?purge$/
        remove_all_temp_directories
        exit_program
      # ===================================================================== #
      # === Feedback all available licenses
      #
      # Use this to feedback all available licenses.
      #
      # Usage examples:
      #   ry --all_licenses
      #   ry gcc license lgpl
      # ===================================================================== #
      when *ARRAY_HELP_VIEW_AVAILABLE_LICENSES_OPTIONS
        feedback_licenses
        exit_program
      # ===================================================================== #
      # === Output how many processors the local computer system has
      #
      # Use this entry point if you wish to find out how many
      # processors/core the target machine has.
      #
      # Invocation examples:
      #   rbt --n-processors?
      #   rbt --n-cores?
      # ===================================================================== #
      when /^-?-?n(_|-)?processors\??$/i,
           /^-?-?n(_|-)?cores\??$/i
        try_to_output_how_many_processors_this_computer_has
      # ===================================================================== #
      # === Use a random build directory
      #
      # This entry point allows us to use a build directory that has
      # a random name. The idea is that this random name will not
      # interfere with any existing directory.
      #
      # Invocation example:
      #   ry htop --use-random-build-directory
      #   ry conky --use-random-build-directory
      #   ry conky --use-random-build-directory --ntrad
      # ===================================================================== #
      when /^-?-?use(_|-)?random(_|-)?build(_|-)?directory$/
        do_use_random_build_directory
      # ===================================================================== #
      # === Show which programs will be compiled for a base system on Linux
      #
      # Invocation examples:
      #   rbt --compile-base-system?
      #   rbt --compile-basic-system?
      # ===================================================================== #
      when /^-?-?compile(_|-)?base(_|-)?system\?$/,
           /^-?-?compile(_|-)?basic(_|-)?system\?$/,
           /^-?-?compile(_|-)?basic(_|-)?subsystem\?$/,
           /^-?-?compile(_|-)?basic(_|-)?linux(_|-)?core\?$/,
           /^-?-?compile(_|-)?basic(_|-)?linux\?$/
        require 'rbt/utility_scripts/compile_base_system.rb'
        RBT::CompileBaseSystem.report_which_programs_will_be_compiled
        exit
      # ===================================================================== #
      # === Keep the archive extracted
      #
      # Use this entry point if you do not want to delete the extracted
      # archive again.
      #
      # Keep our source extracted:
      #
      #   ry htop ke
      #
      # ===================================================================== #
      when *ARRAY_HELP_KEEP_EXTRACTED_OPTIONS
        do_set_keep_extracted
      # ===================================================================== #
      # === Create an expanded cookbook dataset for a program
      #
      # This entry point allows us to use RBT to create a .yml file
      # holding all relevant information to describe a program that
      # is to be compiled/installed.
      #
      # Invocation examples:
      #   rbt --create-yaml-file-for=subversion
      #   rbt --create-yaml-file-for=ruby
      # ===================================================================== #
      when /^-?-?create(_|-)?yaml(_|-)?file(_|-)?for=(.+)/
        create_yaml_file_for_this_program($4.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Show statistics about the whole RBT project
      #
      # Invocation examples:
      #   rbt --stats?
      #   rbt --stats
      # ===================================================================== #
      when /^-?-?stats\??$/i
        show_statistics_about_the_rbt_project
      # ===================================================================== #
      # === rbt --return_compile_time_statistics
      # ===================================================================== #
      when /^-?-?return(_|-)?compile(_|-)?time(_|-)?statistics$/
        e 'Compile time statistics:'
        pp return_compile_time_statistics
      # ===================================================================== #
      # === Show compile time statistics
      #
      # This entry point will display on the commandline some
      # compile-statistics, that is how long it took to compile
      # this or that particular program at hand.
      #
      # Usage examples:
      #   rbt --show-compile-times
      #   rbt --show-compile-time-statistics
      # ===================================================================== #
      when /^-?-?show(_|-)?compile(_|-)?times$/,
           /^-?-?show(_|-)?compile(_|-)?time(_|-)?statistics/,
           /^-?-?compile(_|-)?times\??/,
           /^-?-?compile(_|-)?stats$/
        show_compile_time_statistics
        exit_program
      # ===================================================================== #
      # === Do not display the Infobox
      #
      # Invocation example:
      #   rbt sed --no-infobox --ntrad
      # ===================================================================== #
      when /-?-?do(_|-)?not(_|-)?display(_|-)?the(_|-)?infobox$/,
           /-?-?no(_|-)?infobox$/
        opne 'No infobox will be shown.'
        do_not_display_the_infobox
      # ===================================================================== #
      # === Feedback the default values of the internal hash
      #
      # If you ever need a way to query the values of the internal
      # hash, you can do so through this entry point here.
      #
      # Invocation example:
      #   ry --feedback-the-default-values-of-the-internal-hash
      # ===================================================================== #
      when /^-?-?feedback(-|_)?default(-|_)?values(-|_)?of(-|_)?the(-|_)?internal(-|_)?hash$/i,
           /^-?-?feedback(-|_)?the(-|_)?default(-|_)?values(-|_)?of(-|_)?the(-|_)?internal(-|_)?hash$/i
        feedback_default_values_of_the_internal_hash
        exit_program
      # ===================================================================== #
      # === Purge FHS libraries of a specific program
      #
      # This entry point will get rid of all registered libraries of a
      # target program. Use with care!
      #
      # It will remove all the libraries belonging to a particular
      # program at the /usr/lib/ hierarchy.
      #
      # Invocation example:
      #   ry --fhs-purge-libraries=libgc
      # ===================================================================== #
      when /^-?-?purge-?fhs-?libraries$/i,
           /^-?-?purge-?traditional-?libraries-?from=(.+)/,
           /^-?-?purge-?fhs-?libraries-?from=(.+)/,
           /^-?-?purge-?fhs-?libraries=(.+)/,
           /^-?-?fhs-?library-?purge=(.+)/,
           /^-?-?fhs-?purge-libraries=(.+)/,
           /^-?-?purge(-|_)?traditional(-|_)?libraries(-|_)?of=(.+)$/
        _ = nil
        _ = $1 if $1
        _ = $4 if $4
        _ = program_name? if _.nil?
        verbose_purge_fhs_libraries_of_this_program(_.to_s.dup)
        exit
      # ===================================================================== #
      # === Install the dependencies of the given program at hand
      #
      # This entry point allows us to install the dependencies of the
      # given program at hand. This will NOT check for any other
      # dependencies, so it will not be a transverse-installation.
      #
      # It will only install the dependencies that are directly
      # noted down in the "required_deps_on" part.
      #
      # Invocation example:
      #   rbt rails --install-its-dependencies
      # ===================================================================== #
      when /^-?-?install(-|_)?its(-|_)?dependencies$/i
        install_the_dependencies_of_this_program
      # ===================================================================== #
      # === What will be installed
      #
      # Usage example:
      #   ry htop --installs_what?
      # ===================================================================== #
      when /^-?-?installs(_|-)?what\??$/i,
           'iwhat?'
        feedback_all_libraries_and_all_binaries
        exit_program
      # ===================================================================== #
      # === Fix all Shebangs
      #
      # Try to fix the shebangs of the RBT project:
      #
      #   ry x shebangfixer
      # ===================================================================== #
      when 'shebang','shebangfixer',/try_?to_?fix_?all_?shebangs/
        try_to_fix_all_shebangs
        exit_program
      # ===================================================================== #
      # == Do not rename the konsole tab.
      #
      # If you do not want to rename the KDE konsole tab, use this entry
      # point here.
      #
      # Invocation example:
      #   ry pango trad dont-rename-tab
      # ===================================================================== #
      when /^-?-?dont-?rename-?tab/,
           /^-?-?do-?not-?rename-?konsole/,
           /^-?-?dont-?change-?tab-?title/
        do_not_rename_the_kde_konsole_tab
      # ===================================================================== #
      # === Remove .la files
      #
      # This entry point allows you to get rid of the .la files associated
      # with the given program. Currently it will only be done for programs
      # that are installed in an AppDir-like manner, but this behaviour
      # may change at a later time.
      #
      # Invocation example:
      #   ry pcre2 ntrad --remove-la-files
      # ===================================================================== #
      when /^-?-?remove(_|-)?la(_|-)?files$/
        if is_installed_in_an_appdir_like_fashion?
          opne "Any .la files found to be part of the program "\
               "#{sfancy(program_name?)} will be deleted."
          do_remove_la_files_after_compilation
        end
      # ===================================================================== #
      # === Open a specific yaml file via the editor
      #
      # This entry point allows you to open a specific yaml file.
      #
      # Invocation examples:
      #   rbt --edit-file=nano
      #   rbt --edit=nano
      # ===================================================================== #
      when /-?-?edit-?file=(.+)/,
           /-?-?edit=(.+)/
        edit_this_yaml_file_via_the_main_editor($1.to_s.dup)
        exit_program
      # ===================================================================== #
      # === Edit/Open the yaml file in your editor
      #
      # Invocation example:
      #   ry --edit
      # ===================================================================== #
      when /^-?-?edit$/,/-?-?open/,'EDIT','OPEN','modify'
        edit_the_yaml_file_via_the_main_editor
        exit_program
      # ===================================================================== #
      # === Feedback the available formats
      #
      # This entry point will simply show the available (registered)
      # formats.
      #
      # Invocation examples:
      #   rbt --formats?
      #   rbt --feedback_all_available_formats
      # ===================================================================== #
      when /-?-?format\??/,
           /^-?-?feedback(-|_)?all(-|_)?available(-|_)?formats$/,
            'formats?'
        feedback_all_available_formats
        exit_program
      # ===================================================================== #
      # === Enable porg
      #
      # Use this specifically to enable porg for the specific program
      # at hand.
      # ===================================================================== #
      when /-?-?enable(-|_)?porg/,
           /-?-?use(-|_)?porg/,
           /-?-?do(-|_)?use(-|_)?porg/,
           /-?-?enable(-|_)?porg/,
           /-?-?use(-|_)?paco/,
           /--paco$/,
           /--porg$/
        verbose_enable_porg
      # ===================================================================== #
      # === Show the content of the yaml file
      #
      # This entry point can be used to simply display the full yaml
      # file here. That is, the content of a program's recipe.
      #
      # Invocation examples:
      #   ry htop show_all
      #   ry db show_all
      #   ry libmad show-content
      # ===================================================================== #
      when *ARRAY_HELP_SHOW_ALL_OPTIONS
        show_all
        exit_program
      # ===================================================================== #
      # === Do not use Opn
      #
      # This means that we will not use Opn to show the names of the
      # class in use.
      #
      # Trigger it like so:
      #
      #   ry php --no_opn
      #   ry htop dont_show_names
      # ===================================================================== #
      when /^-?-?do(_|-)?not(_|-)?use(_|-)?opn$/i,
           /-?-?no(-|_)?opn/,
           /-?-?disable(-|_)?opn/,
           'noopn',
           'do_not_output_the_name',
           /no_?names/,
           /do_?not_?show_?names/,
           /dont_?show_?names/,
           'short'
        disable_opn # Defined in the file "opn.rb".
      # ===================================================================== #
      # === Report how many programs are registered
      #
      # Invocation example:
      #   rbt --report-n-programs
      # ===================================================================== #
      when /^-?-?report(-|_)?n(-|_)?programs$/i,
           /^-?-?n(-|_)?programs\??$/i
        report_n_programs
        exit_program
      # ===================================================================== #
      # === Querying the Configuration Options
      #
      # Feedback the url of a given program:
      #   ry glibc qco
      # ===================================================================== #
      when /^query(-|_)?configuration(-|_)?options$/,
           'qco'
        query_configuration_options
      # ===================================================================== #
      # === Use random version
      #
      # The user can pick to use a random version. This means that we will
      # obtain all matches found in the specific directory, and randomly
      # select one of them.
      #
      # Usage example:
      #   ry gcc --use-random-version
      # ===================================================================== #
      when /-?-?use(-|_)?random(-|_)?version/,'--userandomversion'
        do_use_random_version
      # ===================================================================== #
      # === Query whether the internal sed wrapper will be used
      #
      # Invocation example:
      #   rbt --use-the-internal-sed-wrapper?
      # ===================================================================== #
      when /^-?-?use(-|_)?the(-|_)?internal(-|_)?sed(-|_)?wrapper\??$/,
           'usetheinternalsedwrapper?'
        verbose_query_whether_we_will_use_the_internal_sed_wrapper(:and_then_exit)
      # ===================================================================== #
      # === Do not use sed at all
      #
      # This entry point will disable sed.
      #
      # Usage examples:
      #   rbt htop --disable-sed
      #   rbt htop --disable-sed-modifications
      #   rbt extracmakemodules --disable-sed-modifications trad
      # ===================================================================== #
      when /^-?-?disable(-|_)?sed$/,
           /^-?-?disable(-|_)?sed(-|_)?modifications$/
        do_not_use_sed(:be_verbose)
      # ===================================================================== #
      # === Do not use the sed wrapper                              (sed tag)
      #
      # If you do not want to apply the internal sed-wrapper, then
      # this is the proper entry point.
      #
      # Invocation example:
      #   ry libmad --disable-sed
      #   ry cogl --use-simple-sed
      # ===================================================================== #
      when /^-?-?disable(-|_)?internal(-|_)?sed$/,
           /^-?-?use(-|_)?simple(-|_)?sed$/,
           /^-?-?use(-|_)?oldschool(-|_)?sed$/,
           /^-?-?use(-|_)?traditional(-|_)?sed$/,
           /^-?-?oldschool(-|_)?sed$/,
           /^-?-?disable(-|_)?sed(-|_)?wrapper$/
        verbose_disable_sed
      # ===================================================================== #
      # === Keep .la files
      #
      # This allows us to keep .la files, even if the configuration setting
      # specifies otherwises.
      #
      # Invocation example:
      #   ry pcre ntrad --keep-la-files
      # ===================================================================== #
      when /^-?-?keep(-|_)?la(-|_)?files$/i,
           /^-?-?do(-|_)?keep(-|_)?la(-|_)?files$/i
        do_keep_la_files
      # ===================================================================== #
      # === Fix the libtool file
      #
      # This entry point allows us to fix a faulty .la file, based
      # from reading this information from a file.
      #
      # Invocation examples:
      #   rbt --fix-libtool-from-file
      #   rbt --fix-it
      #   rbt --fixit
      #   rbt --patch-it
      # ===================================================================== #
      when /^-?-?fix(-|_)?libtool(-|_)?from(-|_)?file$/i,
           /^-?-?fix(-|_)?it$/i,
           /^-?-?patch(-|_)?it$/i
        fix_libtool_la_file_from_saved_file
      # ===================================================================== #
      # === Query whether libtool files will be removed
      #
      # This entry point allows us to query whether we will delete or
      # keep libtool .la files.
      #
      # Invocation example:
      #
      #   ry --delete_libtool_files?
      #
      # ===================================================================== #
      when /^-?-?delete(-|_)?libtool(-|_)?files\??$/i
        opne 'Will we remove libtool files: '+
              teal(
                verbose_truth(delete_libtool_files?)
              )
      # ===================================================================== #
      # === Do not use autogen
      #
      # This entry point here can be used to disable the use of autogen.
      #
      # This may sometimes be necessary if you do not want to run the
      # autogen.sh script for example, despite the yaml file associated
      # with the program defaulting to "do use autogen".
      #
      # Note that by default, RBT will not run autogen for most programs -
      # the default for "use autogen" is simply set to false. It is only
      # a few programs that may need a regeneration through autogen
      # anyway.
      #
      # Invocation examples:
      #   ry gtk+ --disable-autogen
      #   ry gtk+ disable-autogen
      #   ry matesystemmonitor disable-autogen
      # ===================================================================== #
      when *ARRAY_HELP_DISABLE_AUTOGEN
        disable_autogen
      # ===================================================================== #
      # === Use autogen
      #
      # Use this entry point to enable autogen.
      #
      # Invocation example:
      #   ry gtk+ use-autogen
      # ===================================================================== #
      when *ARRAY_HELP_USE_AUTOGEN
        enable_autogen
      # ===================================================================== #
      # === Query whether we will use autogen
      #
      # Use this entry point if you wish to find out whether we will use
      # autogen or whether we will not.
      #
      # Invocation example:
      #   ry matesystemmonitor --use-autogen?
      # ===================================================================== #
      when /^-?-?use(_|-)?autogen\??/
        verbose_report_will_we_use_autogen
        exit_program
      # ===================================================================== #
      # === Allowed cookbook entries
      #
      # Inform the user which cookbook entries are allowed.
      #
      # Invocation example:
      #   rbt --allowed-cookbook-entries
      # ===================================================================== #
      when /-?-?allowed-?cookbook-?entries/,'ace','allowed?'
        require 'rbt/utility_scripts/allowed_cookbook_entries.rb'
        RBT::Cookbooks::AllowedCookbookEntries.new
        exit_program
      # ===================================================================== #
      # === Feedback extra description of a program
      #
      # Feedback extra description for a given program here, by making use
      # of class SanitizedCookbookDataset.
      #
      # Usage example:
      #   ry php edesc
      # ===================================================================== #
      when /^extra(-|_)?description\??$/i,
           'extra_description',
           'extradescription',
           'edesc'
        feedback_extra_description
        exit_program
      # ===================================================================== #
      # === Scan for missing last update entries in the cookbooks entries
      #
      # This entry point allows us to scan for missing "last_update"
      # entries in the various cookbooks .yml files.
      #
      # Invocation example:
      #   ry --scan
      # ===================================================================== #
      when /-?-?scan/
        RBT.scan_for_missing_last_update_entries
        exit_program
      # ===================================================================== #
      # === Generate all shell scripts
      #
      # This will create a file which bash can use, containing all
      # available shellscripts.
      #
      # Invoke the following as in:
      #   ry --bash
      # ===================================================================== #
      when 'to_bash','2bash','tobash','generate_all','generateall',
           'gall','generate_bash','shell_stuff','allshell',
           'generate_shellscripts','cbs','generate_shell_script',
           'allshellscripts','allsh','toshellscripts',/create_?shell_?scripts/,
           'generateshellscripts','shellscripts',
           'def',/--bash/ # def is special; 'bash' can not be used.
        create_shell_scripts
        exit_program
      # ===================================================================== #
      # === rbt --help
      # ===================================================================== #
      when /help/
        show_help
      else
        # =================================================================== #
        # else we could not find the given argument at hand
        # =================================================================== #
        warning_argument_was_not_found(i) unless i.nil?
      end
    end
  end; alias check_against_menu menu # === check_against_menu

end; end
